#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include <piembedded/pie_message.h>
#include <piw/piw_output.h>
#include <piw/piw_canchor.h>
#include <piw/piw_anchor.h>
#include <piw/piw_data.h>
#include <piw/piw_server.h>
#include <piw/piw_client.h>
#include <piw/piw_gate.h>
#include <piw/piw_index.h>
#include <piw/piw_table.h>
#include <piw/piw_midi_from_belcanto.h>
#include <piw/piw_midi_decoder.h>
#include <piw/piw_bundle.h>
#include <piw/piw_aggregator.h>
#include <piw/piw_clone.h>
#include <piw/piw_sclone.h>
#include <piw/piw_slowchange.h>
#include <piw/piw_multiplexer.h>
#include <piw/piw_function1.h>
#include <piw/piw_function2.h>
#include <piw/piw_selector.h>
#include <piw/piw_pitchbender.h>
#include <piw/piw_polyctl.h>
#include <piw/piw_cycler.h>
#include <piw/piw_stringer.h>
#include <piw/piw_thing.h>
#include <piw/piw_address.h>
#include <piw/piw_fastdata.h>
#include <piw/piw_clock.h>
#include <piw/piw_normalize.h>
#include <piw/piw_policy.h>
#include <piw/piw_gain.h>
#include <piw/piw_panner.h>
#include <piw/piw_velocitydetect.h>
#include <piw/piw_scroller.h>
#include <piw/piw_scroller2.h>
#include <piw/piw_scheduler.h>
#include <piw/piw_controller.h>
#include <piw/piw_backend.h>
#include <piw/piw_sample.h>
#include <piw/piw_sampler.h>
#include <piw/piw_rpc.h>
#include <piw/piw_scaler.h>
#include <piw/piw_kgroup.h>
#include <piw/piw_correlator.h>
#include <piw/piw_connector.h>
#include <piw/piw_ranger.h>
#include <piw/piw_lights.h>
#include <piw/piw_dumper.h>
#include <piw/piw_mixer.h>
#include <piw/piw_evtdump.h>
#include <piw/piw_capture.h>
#include <piw/piw_delay.h>
#include <piw/piw_wavrecorder.h>
#include <piw/piw_termparse.h>
#include <piw/piw_state.h>
#include <piw/piw_window.h>
#include <piw/piw_status.h>
#include <piw/piw_controllerdict.h>
#include <piw/piw_midi_converter.h>
#include <piw/piw_gui_mapper.h>
#include <piw/piw_control_mapping.h>
#include <piw/piw_strummer.h>

static PyObject *piw_make_d2b_functor(PyObject *args);

static PyObject *piw_make_d2b_nb_functor(PyObject *args);

static PyObject *piw_make_d2d_functor(PyObject *args);

static PyObject *piw_make_d2d_nb_functor(PyObject *args);

static PyObject *piw_make_dd2d_functor(PyObject *args);

static PyObject *piw_make_dd2d_nb_functor(PyObject *args);

static PyObject *piw_make_change_functor(PyObject *args);

static PyObject *piw_make_change_nb_functor(PyObject *args);


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef piw::event_t event_type_; 


  


struct event_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(event_type_ *, bool owned);
    PyObject *(*wrapcopy)(event_type_ *);
    event_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< event_type_ *, PyObject * > &);
    void (*erase)(event_type_ *);
    PyObject *(*find)(event_type_ *);
};

struct event_dispatch_ *event_dispatcher_;



typedef piw::scheduler_t scheduler_type_; 


 
static std::map<scheduler_type_ *, PyObject *> scheduler_wrapped_;
 


struct scheduler_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(scheduler_type_ *, bool owned);
    PyObject *(*wrapcopy)(scheduler_type_ *);
    scheduler_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< scheduler_type_ *, PyObject * > &);
    void (*erase)(scheduler_type_ *);
    PyObject *(*find)(scheduler_type_ *);
};

struct scheduler_dispatch_ *scheduler_dispatcher_;



typedef piw::rpctoken_t rpctoken_type_; 


 
static std::map<rpctoken_type_ *, PyObject *> rpctoken_wrapped_;
 


struct rpctoken_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(rpctoken_type_ *, bool owned);
    PyObject *(*wrapcopy)(rpctoken_type_ *);
    rpctoken_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< rpctoken_type_ *, PyObject * > &);
    void (*erase)(rpctoken_type_ *);
    PyObject *(*find)(rpctoken_type_ *);
};

struct rpctoken_dispatch_ *rpctoken_dispatcher_;



typedef piw::rpcserver_t rpcserver_type_; 


 
static std::map<rpcserver_type_ *, PyObject *> rpcserver_wrapped_;
 


struct rpcserver_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(rpcserver_type_ *, bool owned);
    PyObject *(*wrapcopy)(rpcserver_type_ *);
    rpcserver_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< rpcserver_type_ *, PyObject * > &);
    void (*erase)(rpcserver_type_ *);
    PyObject *(*find)(rpcserver_type_ *);
};

struct rpcserver_dispatch_ *rpcserver_dispatcher_;



typedef piw::rpcclient_t rpcclient_type_; 


 
static std::map<rpcclient_type_ *, PyObject *> rpcclient_wrapped_;
 


struct rpcclient_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(rpcclient_type_ *, bool owned);
    PyObject *(*wrapcopy)(rpcclient_type_ *);
    rpcclient_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< rpcclient_type_ *, PyObject * > &);
    void (*erase)(rpcclient_type_ *);
    PyObject *(*find)(rpcclient_type_ *);
};

struct rpcclient_dispatch_ *rpcclient_dispatcher_;



typedef piw::samplearrayref_t samplearray_type_; 


 
static std::map<samplearray_type_ *, PyObject *> samplearray_wrapped_;
 


struct samplearray_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(samplearray_type_ *, bool owned);
    PyObject *(*wrapcopy)(samplearray_type_ *);
    samplearray_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< samplearray_type_ *, PyObject * > &);
    void (*erase)(samplearray_type_ *);
    PyObject *(*find)(samplearray_type_ *);
};

struct samplearray_dispatch_ *samplearray_dispatcher_;



typedef piw::sampleref_t sample_type_; 


 
static std::map<sample_type_ *, PyObject *> sample_wrapped_;
 


struct sample_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(sample_type_ *, bool owned);
    PyObject *(*wrapcopy)(sample_type_ *);
    sample_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< sample_type_ *, PyObject * > &);
    void (*erase)(sample_type_ *);
    PyObject *(*find)(sample_type_ *);
};

struct sample_dispatch_ *sample_dispatcher_;



typedef piw::zoneref_t zone_type_; 


 
static std::map<zone_type_ *, PyObject *> zone_wrapped_;
 


struct zone_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(zone_type_ *, bool owned);
    PyObject *(*wrapcopy)(zone_type_ *);
    zone_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< zone_type_ *, PyObject * > &);
    void (*erase)(zone_type_ *);
    PyObject *(*find)(zone_type_ *);
};

struct zone_dispatch_ *zone_dispatcher_;



typedef piw::presetref_t preset_type_; 


 
static std::map<preset_type_ *, PyObject *> preset_wrapped_;
 


struct preset_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(preset_type_ *, bool owned);
    PyObject *(*wrapcopy)(preset_type_ *);
    preset_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< preset_type_ *, PyObject * > &);
    void (*erase)(preset_type_ *);
    PyObject *(*find)(preset_type_ *);
};

struct preset_dispatch_ *preset_dispatcher_;



typedef bct_clocksink_t clockref_type_; 


 
static std::map<clockref_type_ *, PyObject *> clockref_wrapped_;
 


struct clockref_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockref_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockref_type_ *);
    clockref_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockref_type_ *, PyObject * > &);
    void (*erase)(clockref_type_ *);
    PyObject *(*find)(clockref_type_ *);
};

struct clockref_dispatch_ *clockref_dispatcher_;



typedef piw::strummer_t strummer_type_; 


 
static std::map<strummer_type_ *, PyObject *> strummer_wrapped_;
 


struct strummer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(strummer_type_ *, bool owned);
    PyObject *(*wrapcopy)(strummer_type_ *);
    strummer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< strummer_type_ *, PyObject * > &);
    void (*erase)(strummer_type_ *);
    PyObject *(*find)(strummer_type_ *);
};

struct strummer_dispatch_ *strummer_dispatcher_;



typedef piw::stringer_t stringer_type_; 


 
static std::map<stringer_type_ *, PyObject *> stringer_wrapped_;
 


struct stringer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(stringer_type_ *, bool owned);
    PyObject *(*wrapcopy)(stringer_type_ *);
    stringer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< stringer_type_ *, PyObject * > &);
    void (*erase)(stringer_type_ *);
    PyObject *(*find)(stringer_type_ *);
};

struct stringer_dispatch_ *stringer_dispatcher_;



typedef piw::cycler_t cycler_type_; 


 
static std::map<cycler_type_ *, PyObject *> cycler_wrapped_;
 


struct cycler_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cycler_type_ *, bool owned);
    PyObject *(*wrapcopy)(cycler_type_ *);
    cycler_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cycler_type_ *, PyObject * > &);
    void (*erase)(cycler_type_ *);
    PyObject *(*find)(cycler_type_ *);
};

struct cycler_dispatch_ *cycler_dispatcher_;



typedef piw::polyctl_t polyctl_type_; 


 
static std::map<polyctl_type_ *, PyObject *> polyctl_wrapped_;
 


struct polyctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(polyctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(polyctl_type_ *);
    polyctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< polyctl_type_ *, PyObject * > &);
    void (*erase)(polyctl_type_ *);
    PyObject *(*find)(polyctl_type_ *);
};

struct polyctl_dispatch_ *polyctl_dispatcher_;



typedef piw::stereomixer_t stereomixer_type_; 


 
static std::map<stereomixer_type_ *, PyObject *> stereomixer_wrapped_;
 


struct stereomixer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(stereomixer_type_ *, bool owned);
    PyObject *(*wrapcopy)(stereomixer_type_ *);
    stereomixer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< stereomixer_type_ *, PyObject * > &);
    void (*erase)(stereomixer_type_ *);
    PyObject *(*find)(stereomixer_type_ *);
};

struct stereomixer_dispatch_ *stereomixer_dispatcher_;



typedef piw::stereosummer_t stereosummer_type_; 


 
static std::map<stereosummer_type_ *, PyObject *> stereosummer_wrapped_;
 


struct stereosummer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(stereosummer_type_ *, bool owned);
    PyObject *(*wrapcopy)(stereosummer_type_ *);
    stereosummer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< stereosummer_type_ *, PyObject * > &);
    void (*erase)(stereosummer_type_ *);
    PyObject *(*find)(stereosummer_type_ *);
};

struct stereosummer_dispatch_ *stereosummer_dispatcher_;



typedef piw::monomixer_t monomixer_type_; 


 
static std::map<monomixer_type_ *, PyObject *> monomixer_wrapped_;
 


struct monomixer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(monomixer_type_ *, bool owned);
    PyObject *(*wrapcopy)(monomixer_type_ *);
    monomixer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< monomixer_type_ *, PyObject * > &);
    void (*erase)(monomixer_type_ *);
    PyObject *(*find)(monomixer_type_ *);
};

struct monomixer_dispatch_ *monomixer_dispatcher_;



typedef piw::consolemixer_t consolemixer_type_; 


 
static std::map<consolemixer_type_ *, PyObject *> consolemixer_wrapped_;
 


struct consolemixer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(consolemixer_type_ *, bool owned);
    PyObject *(*wrapcopy)(consolemixer_type_ *);
    consolemixer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< consolemixer_type_ *, PyObject * > &);
    void (*erase)(consolemixer_type_ *);
    PyObject *(*find)(consolemixer_type_ *);
};

struct consolemixer_dispatch_ *consolemixer_dispatcher_;



typedef piw::gate_t gate_type_; 


 
static std::map<gate_type_ *, PyObject *> gate_wrapped_;
 


struct gate_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(gate_type_ *, bool owned);
    PyObject *(*wrapcopy)(gate_type_ *);
    gate_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< gate_type_ *, PyObject * > &);
    void (*erase)(gate_type_ *);
    PyObject *(*find)(gate_type_ *);
};

struct gate_dispatch_ *gate_dispatcher_;



typedef piw::correlator_t correlator_type_; 


 
static std::map<correlator_type_ *, PyObject *> correlator_wrapped_;
 


struct correlator_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(correlator_type_ *, bool owned);
    PyObject *(*wrapcopy)(correlator_type_ *);
    correlator_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< correlator_type_ *, PyObject * > &);
    void (*erase)(correlator_type_ *);
    PyObject *(*find)(correlator_type_ *);
};

struct correlator_dispatch_ *correlator_dispatcher_;



typedef piw::connector_t connector_type_; 


  


struct connector_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(connector_type_ *, bool owned);
    PyObject *(*wrapcopy)(connector_type_ *);
    connector_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< connector_type_ *, PyObject * > &);
    void (*erase)(connector_type_ *);
    PyObject *(*find)(connector_type_ *);
};

struct connector_dispatch_ *connector_dispatcher_;



typedef piw::converter_ref_t converter_ref_type_; 


 
static std::map<converter_ref_type_ *, PyObject *> converter_ref_wrapped_;
 


struct converter_ref_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(converter_ref_type_ *, bool owned);
    PyObject *(*wrapcopy)(converter_ref_type_ *);
    converter_ref_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< converter_ref_type_ *, PyObject * > &);
    void (*erase)(converter_ref_type_ *);
    PyObject *(*find)(converter_ref_type_ *);
};

struct converter_ref_dispatch_ *converter_ref_dispatcher_;



typedef piw::capture_backend_t capture_backend_type_; 


 
static std::map<capture_backend_type_ *, PyObject *> capture_backend_wrapped_;
 


struct capture_backend_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(capture_backend_type_ *, bool owned);
    PyObject *(*wrapcopy)(capture_backend_type_ *);
    capture_backend_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< capture_backend_type_ *, PyObject * > &);
    void (*erase)(capture_backend_type_ *);
    PyObject *(*find)(capture_backend_type_ *);
};

struct capture_backend_dispatch_ *capture_backend_dispatcher_;



typedef piw::functor_backend_t functor_backend_type_; 


 
static std::map<functor_backend_type_ *, PyObject *> functor_backend_wrapped_;
 


struct functor_backend_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(functor_backend_type_ *, bool owned);
    PyObject *(*wrapcopy)(functor_backend_type_ *);
    functor_backend_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< functor_backend_type_ *, PyObject * > &);
    void (*erase)(functor_backend_type_ *);
    PyObject *(*find)(functor_backend_type_ *);
};

struct functor_backend_dispatch_ *functor_backend_dispatcher_;



typedef piw::controller_t controller_type_; 


 
static std::map<controller_type_ *, PyObject *> controller_wrapped_;
 


struct controller_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(controller_type_ *, bool owned);
    PyObject *(*wrapcopy)(controller_type_ *);
    controller_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< controller_type_ *, PyObject * > &);
    void (*erase)(controller_type_ *);
    PyObject *(*find)(controller_type_ *);
};

struct controller_dispatch_ *controller_dispatcher_;



typedef piw::xxcontrolled_t xxcontrolled_type_; 


 
static std::map<xxcontrolled_type_ *, PyObject *> xxcontrolled_wrapped_;
 


struct xxcontrolled_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(xxcontrolled_type_ *, bool owned);
    PyObject *(*wrapcopy)(xxcontrolled_type_ *);
    xxcontrolled_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< xxcontrolled_type_ *, PyObject * > &);
    void (*erase)(xxcontrolled_type_ *);
    PyObject *(*find)(xxcontrolled_type_ *);
};

struct xxcontrolled_dispatch_ *xxcontrolled_dispatcher_;



typedef piw::controlled_t controlled_type_; 


  


struct controlled_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(controlled_type_ *, bool owned);
    PyObject *(*wrapcopy)(controlled_type_ *);
    controlled_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< controlled_type_ *, PyObject * > &);
    void (*erase)(controlled_type_ *);
    PyObject *(*find)(controlled_type_ *);
};

struct controlled_dispatch_ *controlled_dispatcher_;



typedef piw::fasttrigger_t fasttrigger_type_; 


  


struct fasttrigger_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(fasttrigger_type_ *, bool owned);
    PyObject *(*wrapcopy)(fasttrigger_type_ *);
    fasttrigger_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< fasttrigger_type_ *, PyObject * > &);
    void (*erase)(fasttrigger_type_ *);
    PyObject *(*find)(fasttrigger_type_ *);
};

struct fasttrigger_dispatch_ *fasttrigger_dispatcher_;



typedef piw::cookie_t cookie_type_; 


 
static std::map<cookie_type_ *, PyObject *> cookie_wrapped_;
 


struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::decode_ctl_t decode_ctl_type_; 


 
static std::map<decode_ctl_type_ *, PyObject *> decode_ctl_wrapped_;
 


struct decode_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(decode_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(decode_ctl_type_ *);
    decode_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< decode_ctl_type_ *, PyObject * > &);
    void (*erase)(decode_ctl_type_ *);
    PyObject *(*find)(decode_ctl_type_ *);
};

struct decode_ctl_dispatch_ *decode_ctl_dispatcher_;



typedef piw::decoder_t decoder_type_; 


 
static std::map<decoder_type_ *, PyObject *> decoder_wrapped_;
 


struct decoder_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(decoder_type_ *, bool owned);
    PyObject *(*wrapcopy)(decoder_type_ *);
    decoder_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< decoder_type_ *, PyObject * > &);
    void (*erase)(decoder_type_ *);
    PyObject *(*find)(decoder_type_ *);
};

struct decoder_dispatch_ *decoder_dispatcher_;



typedef piw::xevent_data_buffer_t event_data_buffer_type_; 


 
static std::map<event_data_buffer_type_ *, PyObject *> event_data_buffer_wrapped_;
 


struct event_data_buffer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(event_data_buffer_type_ *, bool owned);
    PyObject *(*wrapcopy)(event_data_buffer_type_ *);
    event_data_buffer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< event_data_buffer_type_ *, PyObject * > &);
    void (*erase)(event_data_buffer_type_ *);
    PyObject *(*find)(event_data_buffer_type_ *);
};

struct event_data_buffer_dispatch_ *event_data_buffer_dispatcher_;



typedef piw::event_data_source_t event_data_source_type_; 


 
static std::map<event_data_source_type_ *, PyObject *> event_data_source_wrapped_;
 


struct event_data_source_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(event_data_source_type_ *, bool owned);
    PyObject *(*wrapcopy)(event_data_source_type_ *);
    event_data_source_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< event_data_source_type_ *, PyObject * > &);
    void (*erase)(event_data_source_type_ *);
    PyObject *(*find)(event_data_source_type_ *);
};

struct event_data_source_dispatch_ *event_data_source_dispatcher_;



typedef piw::event_data_source_real_t event_data_source_real_type_; 


 
static std::map<event_data_source_real_type_ *, PyObject *> event_data_source_real_wrapped_;
 


struct event_data_source_real_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(event_data_source_real_type_ *, bool owned);
    PyObject *(*wrapcopy)(event_data_source_real_type_ *);
    event_data_source_real_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< event_data_source_real_type_ *, PyObject * > &);
    void (*erase)(event_data_source_real_type_ *);
    PyObject *(*find)(event_data_source_real_type_ *);
};

struct event_data_source_real_dispatch_ *event_data_source_real_dispatcher_;



typedef piw::wire_ctl_t wire_ctl_type_; 


 
static std::map<wire_ctl_type_ *, PyObject *> wire_ctl_wrapped_;
 


struct wire_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(wire_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(wire_ctl_type_ *);
    wire_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< wire_ctl_type_ *, PyObject * > &);
    void (*erase)(wire_ctl_type_ *);
    PyObject *(*find)(wire_ctl_type_ *);
};

struct wire_ctl_dispatch_ *wire_ctl_dispatcher_;



typedef piw::root_ctl_t root_ctl_type_; 


 
static std::map<root_ctl_type_ *, PyObject *> root_ctl_wrapped_;
 


struct root_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(root_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(root_ctl_type_ *);
    root_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< root_ctl_type_ *, PyObject * > &);
    void (*erase)(root_ctl_type_ *);
    PyObject *(*find)(root_ctl_type_ *);
};

struct root_ctl_dispatch_ *root_ctl_dispatcher_;



typedef piw::wire_t wire_type_; 


 
static std::map<wire_type_ *, PyObject *> wire_wrapped_;
 


struct wire_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(wire_type_ *, bool owned);
    PyObject *(*wrapcopy)(wire_type_ *);
    wire_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< wire_type_ *, PyObject * > &);
    void (*erase)(wire_type_ *);
    PyObject *(*find)(wire_type_ *);
};

struct wire_dispatch_ *wire_dispatcher_;



typedef piw::root_t root_type_; 


 
static std::map<root_type_ *, PyObject *> root_wrapped_;
 


struct root_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(root_type_ *, bool owned);
    PyObject *(*wrapcopy)(root_type_ *);
    root_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< root_type_ *, PyObject * > &);
    void (*erase)(root_type_ *);
    PyObject *(*find)(root_type_ *);
};

struct root_dispatch_ *root_dispatcher_;



typedef piw::multiplexer_t multiplexer_type_; 


 
static std::map<multiplexer_type_ *, PyObject *> multiplexer_wrapped_;
 


struct multiplexer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(multiplexer_type_ *, bool owned);
    PyObject *(*wrapcopy)(multiplexer_type_ *);
    multiplexer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< multiplexer_type_ *, PyObject * > &);
    void (*erase)(multiplexer_type_ *);
    PyObject *(*find)(multiplexer_type_ *);
};

struct multiplexer_dispatch_ *multiplexer_dispatcher_;



typedef piw::ranger_t ranger_type_; 


 
static std::map<ranger_type_ *, PyObject *> ranger_wrapped_;
 


struct ranger_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(ranger_type_ *, bool owned);
    PyObject *(*wrapcopy)(ranger_type_ *);
    ranger_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< ranger_type_ *, PyObject * > &);
    void (*erase)(ranger_type_ *);
    PyObject *(*find)(ranger_type_ *);
};

struct ranger_dispatch_ *ranger_dispatcher_;



typedef piw::lightconvertor_t lightconvertor_type_; 


 
static std::map<lightconvertor_type_ *, PyObject *> lightconvertor_wrapped_;
 


struct lightconvertor_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(lightconvertor_type_ *, bool owned);
    PyObject *(*wrapcopy)(lightconvertor_type_ *);
    lightconvertor_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< lightconvertor_type_ *, PyObject * > &);
    void (*erase)(lightconvertor_type_ *);
    PyObject *(*find)(lightconvertor_type_ *);
};

struct lightconvertor_dispatch_ *lightconvertor_dispatcher_;



typedef piw::statusbuffer_t statusbuffer_type_; 


 
static std::map<statusbuffer_type_ *, PyObject *> statusbuffer_wrapped_;
 


struct statusbuffer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(statusbuffer_type_ *, bool owned);
    PyObject *(*wrapcopy)(statusbuffer_type_ *);
    statusbuffer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< statusbuffer_type_ *, PyObject * > &);
    void (*erase)(statusbuffer_type_ *);
    PyObject *(*find)(statusbuffer_type_ *);
};

struct statusbuffer_dispatch_ *statusbuffer_dispatcher_;



typedef piw::lightsource_t lightsource_type_; 


 
static std::map<lightsource_type_ *, PyObject *> lightsource_wrapped_;
 


struct lightsource_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(lightsource_type_ *, bool owned);
    PyObject *(*wrapcopy)(lightsource_type_ *);
    lightsource_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< lightsource_type_ *, PyObject * > &);
    void (*erase)(lightsource_type_ *);
    PyObject *(*find)(lightsource_type_ *);
};

struct lightsource_dispatch_ *lightsource_dispatcher_;



typedef piw::statusmixer_t statusmixer_type_; 


 
static std::map<statusmixer_type_ *, PyObject *> statusmixer_wrapped_;
 


struct statusmixer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(statusmixer_type_ *, bool owned);
    PyObject *(*wrapcopy)(statusmixer_type_ *);
    statusmixer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< statusmixer_type_ *, PyObject * > &);
    void (*erase)(statusmixer_type_ *);
    PyObject *(*find)(statusmixer_type_ *);
};

struct statusmixer_dispatch_ *statusmixer_dispatcher_;



typedef piw::controllerdict_t controllerdict_type_; 


 
static std::map<controllerdict_type_ *, PyObject *> controllerdict_wrapped_;
 


struct controllerdict_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(controllerdict_type_ *, bool owned);
    PyObject *(*wrapcopy)(controllerdict_type_ *);
    controllerdict_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< controllerdict_type_ *, PyObject * > &);
    void (*erase)(controllerdict_type_ *);
    PyObject *(*find)(controllerdict_type_ *);
};

struct controllerdict_dispatch_ *controllerdict_dispatcher_;



typedef piw::mapping_info_t mapping_info_type_; 


 
static std::map<mapping_info_type_ *, PyObject *> mapping_info_wrapped_;
 


struct mapping_info_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(mapping_info_type_ *, bool owned);
    PyObject *(*wrapcopy)(mapping_info_type_ *);
    mapping_info_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< mapping_info_type_ *, PyObject * > &);
    void (*erase)(mapping_info_type_ *);
    PyObject *(*find)(mapping_info_type_ *);
};

struct mapping_info_dispatch_ *mapping_info_dispatcher_;



typedef piw::midi_converter_t midi_converter_type_; 


 
static std::map<midi_converter_type_ *, PyObject *> midi_converter_wrapped_;
 


struct midi_converter_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(midi_converter_type_ *, bool owned);
    PyObject *(*wrapcopy)(midi_converter_type_ *);
    midi_converter_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< midi_converter_type_ *, PyObject * > &);
    void (*erase)(midi_converter_type_ *);
    PyObject *(*find)(midi_converter_type_ *);
};

struct midi_converter_dispatch_ *midi_converter_dispatcher_;



typedef piw::mapping_observer_t mapping_observer_type_; 


 
static std::map<mapping_observer_type_ *, PyObject *> mapping_observer_wrapped_;
 


struct mapping_observer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(mapping_observer_type_ *, bool owned);
    PyObject *(*wrapcopy)(mapping_observer_type_ *);
    mapping_observer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< mapping_observer_type_ *, PyObject * > &);
    void (*erase)(mapping_observer_type_ *);
    PyObject *(*find)(mapping_observer_type_ *);
};

struct mapping_observer_dispatch_ *mapping_observer_dispatcher_;



typedef piw::kgroup_mapper_t kgroup_mapper_type_; 


 
static std::map<kgroup_mapper_type_ *, PyObject *> kgroup_mapper_wrapped_;
 


struct kgroup_mapper_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(kgroup_mapper_type_ *, bool owned);
    PyObject *(*wrapcopy)(kgroup_mapper_type_ *);
    kgroup_mapper_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< kgroup_mapper_type_ *, PyObject * > &);
    void (*erase)(kgroup_mapper_type_ *);
    PyObject *(*find)(kgroup_mapper_type_ *);
};

struct kgroup_mapper_dispatch_ *kgroup_mapper_dispatcher_;



typedef piw::sclone_t sclone_type_; 


 
static std::map<sclone_type_ *, PyObject *> sclone_wrapped_;
 


struct sclone_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(sclone_type_ *, bool owned);
    PyObject *(*wrapcopy)(sclone_type_ *);
    sclone_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< sclone_type_ *, PyObject * > &);
    void (*erase)(sclone_type_ *);
    PyObject *(*find)(sclone_type_ *);
};

struct sclone_dispatch_ *sclone_dispatcher_;



typedef piw::clone_t clone_type_; 


 
static std::map<clone_type_ *, PyObject *> clone_wrapped_;
 


struct clone_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clone_type_ *, bool owned);
    PyObject *(*wrapcopy)(clone_type_ *);
    clone_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clone_type_ *, PyObject * > &);
    void (*erase)(clone_type_ *);
    PyObject *(*find)(clone_type_ *);
};

struct clone_dispatch_ *clone_dispatcher_;



typedef piw::aggregator_t aggregator_type_; 


 
static std::map<aggregator_type_ *, PyObject *> aggregator_wrapped_;
 


struct aggregator_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(aggregator_type_ *, bool owned);
    PyObject *(*wrapcopy)(aggregator_type_ *);
    aggregator_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< aggregator_type_ *, PyObject * > &);
    void (*erase)(aggregator_type_ *);
    PyObject *(*find)(aggregator_type_ *);
};

struct aggregator_dispatch_ *aggregator_dispatcher_;



typedef piw::scrolldelegate_t scrolldelegate_type_; 


 
static std::map<scrolldelegate_type_ *, PyObject *> scrolldelegate_wrapped_;
 


struct scrolldelegate_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(scrolldelegate_type_ *, bool owned);
    PyObject *(*wrapcopy)(scrolldelegate_type_ *);
    scrolldelegate_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< scrolldelegate_type_ *, PyObject * > &);
    void (*erase)(scrolldelegate_type_ *);
    PyObject *(*find)(scrolldelegate_type_ *);
};

struct scrolldelegate_dispatch_ *scrolldelegate_dispatcher_;



typedef piw::scroller_t scroller_type_; 


 
static std::map<scroller_type_ *, PyObject *> scroller_wrapped_;
 


struct scroller_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(scroller_type_ *, bool owned);
    PyObject *(*wrapcopy)(scroller_type_ *);
    scroller_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< scroller_type_ *, PyObject * > &);
    void (*erase)(scroller_type_ *);
    PyObject *(*find)(scroller_type_ *);
};

struct scroller_dispatch_ *scroller_dispatcher_;



typedef piw::scroller2_t scroller2_type_; 


 
static std::map<scroller2_type_ *, PyObject *> scroller2_wrapped_;
 


struct scroller2_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(scroller2_type_ *, bool owned);
    PyObject *(*wrapcopy)(scroller2_type_ *);
    scroller2_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< scroller2_type_ *, PyObject * > &);
    void (*erase)(scroller2_type_ *);
    PyObject *(*find)(scroller2_type_ *);
};

struct scroller2_dispatch_ *scroller2_dispatcher_;



typedef piw::splitter_node_t splitter_node_type_; 


  


struct splitter_node_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(splitter_node_type_ *, bool owned);
    PyObject *(*wrapcopy)(splitter_node_type_ *);
    splitter_node_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< splitter_node_type_ *, PyObject * > &);
    void (*erase)(splitter_node_type_ *);
    PyObject *(*find)(splitter_node_type_ *);
};

struct splitter_node_dispatch_ *splitter_node_dispatcher_;



typedef piw::splitter_t splitter_type_; 


 
static std::map<splitter_type_ *, PyObject *> splitter_wrapped_;
 


struct splitter_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(splitter_type_ *, bool owned);
    PyObject *(*wrapcopy)(splitter_type_ *);
    splitter_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< splitter_type_ *, PyObject * > &);
    void (*erase)(splitter_type_ *);
    PyObject *(*find)(splitter_type_ *);
};

struct splitter_dispatch_ *splitter_dispatcher_;



typedef piw::d2b_t d2b_type_; 


 
static std::map<d2b_type_ *, PyObject *> d2b_wrapped_;
 


struct d2b_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(d2b_type_ *, bool owned);
    PyObject *(*wrapcopy)(d2b_type_ *);
    d2b_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< d2b_type_ *, PyObject * > &);
    void (*erase)(d2b_type_ *);
    PyObject *(*find)(d2b_type_ *);
};

struct d2b_dispatch_ *d2b_dispatcher_;



typedef piw::d2b_nb_t d2b_nb_type_; 


 
static std::map<d2b_nb_type_ *, PyObject *> d2b_nb_wrapped_;
 


struct d2b_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(d2b_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(d2b_nb_type_ *);
    d2b_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< d2b_nb_type_ *, PyObject * > &);
    void (*erase)(d2b_nb_type_ *);
    PyObject *(*find)(d2b_nb_type_ *);
};

struct d2b_nb_dispatch_ *d2b_nb_dispatcher_;



typedef piw::d2d_t d2d_type_; 


 
static std::map<d2d_type_ *, PyObject *> d2d_wrapped_;
 


struct d2d_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(d2d_type_ *, bool owned);
    PyObject *(*wrapcopy)(d2d_type_ *);
    d2d_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< d2d_type_ *, PyObject * > &);
    void (*erase)(d2d_type_ *);
    PyObject *(*find)(d2d_type_ *);
};

struct d2d_dispatch_ *d2d_dispatcher_;



typedef piw::d2d_nb_t d2d_nb_type_; 


 
static std::map<d2d_nb_type_ *, PyObject *> d2d_nb_wrapped_;
 


struct d2d_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(d2d_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(d2d_nb_type_ *);
    d2d_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< d2d_nb_type_ *, PyObject * > &);
    void (*erase)(d2d_nb_type_ *);
    PyObject *(*find)(d2d_nb_type_ *);
};

struct d2d_nb_dispatch_ *d2d_nb_dispatcher_;



typedef piw::dd2d_t dd2d_type_; 


 
static std::map<dd2d_type_ *, PyObject *> dd2d_wrapped_;
 


struct dd2d_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(dd2d_type_ *, bool owned);
    PyObject *(*wrapcopy)(dd2d_type_ *);
    dd2d_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< dd2d_type_ *, PyObject * > &);
    void (*erase)(dd2d_type_ *);
    PyObject *(*find)(dd2d_type_ *);
};

struct dd2d_dispatch_ *dd2d_dispatcher_;



typedef piw::dd2d_nb_t dd2d_nb_type_; 


 
static std::map<dd2d_nb_type_ *, PyObject *> dd2d_nb_wrapped_;
 


struct dd2d_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(dd2d_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(dd2d_nb_type_ *);
    dd2d_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< dd2d_nb_type_ *, PyObject * > &);
    void (*erase)(dd2d_nb_type_ *);
    PyObject *(*find)(dd2d_nb_type_ *);
};

struct dd2d_nb_dispatch_ *dd2d_nb_dispatcher_;



typedef piw::change_t change_type_; 


 
static std::map<change_type_ *, PyObject *> change_wrapped_;
 


struct change_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_type_ *);
    change_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_type_ *, PyObject * > &);
    void (*erase)(change_type_ *);
    PyObject *(*find)(change_type_ *);
};

struct change_dispatch_ *change_dispatcher_;



typedef piw::change_nb_t change_nb_type_; 


 
static std::map<change_nb_type_ *, PyObject *> change_nb_wrapped_;
 


struct change_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_nb_type_ *);
    change_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_nb_type_ *, PyObject * > &);
    void (*erase)(change_nb_type_ *);
    PyObject *(*find)(change_nb_type_ *);
};

struct change_nb_dispatch_ *change_nb_dispatcher_;



typedef piw::function1_t function1_type_; 


 
static std::map<function1_type_ *, PyObject *> function1_wrapped_;
 


struct function1_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(function1_type_ *, bool owned);
    PyObject *(*wrapcopy)(function1_type_ *);
    function1_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< function1_type_ *, PyObject * > &);
    void (*erase)(function1_type_ *);
    PyObject *(*find)(function1_type_ *);
};

struct function1_dispatch_ *function1_dispatcher_;



typedef piw::function2_t function2_type_; 


 
static std::map<function2_type_ *, PyObject *> function2_wrapped_;
 


struct function2_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(function2_type_ *, bool owned);
    PyObject *(*wrapcopy)(function2_type_ *);
    function2_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< function2_type_ *, PyObject * > &);
    void (*erase)(function2_type_ *);
    PyObject *(*find)(function2_type_ *);
};

struct function2_dispatch_ *function2_dispatcher_;



typedef piw::window_t window_type_; 


 
static std::map<window_type_ *, PyObject *> window_wrapped_;
 


struct window_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(window_type_ *, bool owned);
    PyObject *(*wrapcopy)(window_type_ *);
    window_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< window_type_ *, PyObject * > &);
    void (*erase)(window_type_ *);
    PyObject *(*find)(window_type_ *);
};

struct window_dispatch_ *window_dispatcher_;



typedef piw::thing_t thing_type_; 


 
static std::map<thing_type_ *, PyObject *> thing_wrapped_;
 


struct thing_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(thing_type_ *, bool owned);
    PyObject *(*wrapcopy)(thing_type_ *);
    thing_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< thing_type_ *, PyObject * > &);
    void (*erase)(thing_type_ *);
    PyObject *(*find)(thing_type_ *);
};

struct thing_dispatch_ *thing_dispatcher_;



typedef piw::index_t index_type_; 


 
static std::map<index_type_ *, PyObject *> index_wrapped_;
 


struct index_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(index_type_ *, bool owned);
    PyObject *(*wrapcopy)(index_type_ *);
    index_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< index_type_ *, PyObject * > &);
    void (*erase)(index_type_ *);
    PyObject *(*find)(index_type_ *);
};

struct index_dispatch_ *index_dispatcher_;



typedef piw::server_t server_type_; 


 
static std::map<server_type_ *, PyObject *> server_wrapped_;
 


struct server_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(server_type_ *, bool owned);
    PyObject *(*wrapcopy)(server_type_ *);
    server_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< server_type_ *, PyObject * > &);
    void (*erase)(server_type_ *);
    PyObject *(*find)(server_type_ *);
};

struct server_dispatch_ *server_dispatcher_;



typedef piw::client_t client_type_; 


 
static std::map<client_type_ *, PyObject *> client_wrapped_;
 


struct client_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(client_type_ *, bool owned);
    PyObject *(*wrapcopy)(client_type_ *);
    client_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< client_type_ *, PyObject * > &);
    void (*erase)(client_type_ *);
    PyObject *(*find)(client_type_ *);
};

struct client_dispatch_ *client_dispatcher_;



typedef piw::subcanchor_t subcanchor_type_; 


  


struct subcanchor_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(subcanchor_type_ *, bool owned);
    PyObject *(*wrapcopy)(subcanchor_type_ *);
    subcanchor_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< subcanchor_type_ *, PyObject * > &);
    void (*erase)(subcanchor_type_ *);
    PyObject *(*find)(subcanchor_type_ *);
};

struct subcanchor_dispatch_ *subcanchor_dispatcher_;



typedef piw::canchor_t canchor_type_; 


 
static std::map<canchor_type_ *, PyObject *> canchor_wrapped_;
 


struct canchor_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(canchor_type_ *, bool owned);
    PyObject *(*wrapcopy)(canchor_type_ *);
    canchor_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< canchor_type_ *, PyObject * > &);
    void (*erase)(canchor_type_ *);
    PyObject *(*find)(canchor_type_ *);
};

struct canchor_dispatch_ *canchor_dispatcher_;



typedef piw::anchor_t anchor_type_; 


 
static std::map<anchor_type_ *, PyObject *> anchor_wrapped_;
 


struct anchor_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(anchor_type_ *, bool owned);
    PyObject *(*wrapcopy)(anchor_type_ *);
    anchor_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< anchor_type_ *, PyObject * > &);
    void (*erase)(anchor_type_ *);
    PyObject *(*find)(anchor_type_ *);
};

struct anchor_dispatch_ *anchor_dispatcher_;



typedef piw::f_event_sender_t f_event_sender_type_; 


 
static std::map<f_event_sender_type_ *, PyObject *> f_event_sender_wrapped_;
 


struct f_event_sender_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(f_event_sender_type_ *, bool owned);
    PyObject *(*wrapcopy)(f_event_sender_type_ *);
    f_event_sender_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< f_event_sender_type_ *, PyObject * > &);
    void (*erase)(f_event_sender_type_ *);
    PyObject *(*find)(f_event_sender_type_ *);
};

struct f_event_sender_dispatch_ *f_event_sender_dispatcher_;



typedef piw::f_event_receiver_t f_event_receiver_type_; 


 
static std::map<f_event_receiver_type_ *, PyObject *> f_event_receiver_wrapped_;
 


struct f_event_receiver_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(f_event_receiver_type_ *, bool owned);
    PyObject *(*wrapcopy)(f_event_receiver_type_ *);
    f_event_receiver_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< f_event_receiver_type_ *, PyObject * > &);
    void (*erase)(f_event_receiver_type_ *);
    PyObject *(*find)(f_event_receiver_type_ *);
};

struct f_event_receiver_dispatch_ *f_event_receiver_dispatcher_;



typedef piw::dataqueue_t dataqueue_type_; 


 
static std::map<dataqueue_type_ *, PyObject *> dataqueue_wrapped_;
 


struct dataqueue_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(dataqueue_type_ *, bool owned);
    PyObject *(*wrapcopy)(dataqueue_type_ *);
    dataqueue_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< dataqueue_type_ *, PyObject * > &);
    void (*erase)(dataqueue_type_ *);
    PyObject *(*find)(dataqueue_type_ *);
};

struct dataqueue_dispatch_ *dataqueue_dispatcher_;



typedef piw::fastdata_t fastdata_type_; 


 
static std::map<fastdata_type_ *, PyObject *> fastdata_wrapped_;
 


struct fastdata_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(fastdata_type_ *, bool owned);
    PyObject *(*wrapcopy)(fastdata_type_ *);
    fastdata_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< fastdata_type_ *, PyObject * > &);
    void (*erase)(fastdata_type_ *);
    PyObject *(*find)(fastdata_type_ *);
};

struct fastdata_dispatch_ *fastdata_dispatcher_;



typedef piw::fastdata_receiver_t fastdata_receiver_type_; 


  


struct fastdata_receiver_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(fastdata_receiver_type_ *, bool owned);
    PyObject *(*wrapcopy)(fastdata_receiver_type_ *);
    fastdata_receiver_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< fastdata_receiver_type_ *, PyObject * > &);
    void (*erase)(fastdata_receiver_type_ *);
    PyObject *(*find)(fastdata_receiver_type_ *);
};

struct fastdata_receiver_dispatch_ *fastdata_receiver_dispatcher_;



typedef piw::term_t term_type_; 


 
static std::map<term_type_ *, PyObject *> term_wrapped_;
 


struct term_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(term_type_ *, bool owned);
    PyObject *(*wrapcopy)(term_type_ *);
    term_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< term_type_ *, PyObject * > &);
    void (*erase)(term_type_ *);
    PyObject *(*find)(term_type_ *);
};

struct term_dispatch_ *term_dispatcher_;



typedef piw::data_base_t data_base_type_; 


 
static std::map<data_base_type_ *, PyObject *> data_base_wrapped_;
 


struct data_base_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(data_base_type_ *, bool owned);
    PyObject *(*wrapcopy)(data_base_type_ *);
    data_base_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< data_base_type_ *, PyObject * > &);
    void (*erase)(data_base_type_ *);
    PyObject *(*find)(data_base_type_ *);
};

struct data_base_dispatch_ *data_base_dispatcher_;



typedef piw::data_t data_type_; 


  


struct data_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(data_type_ *, bool owned);
    PyObject *(*wrapcopy)(data_type_ *);
    data_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< data_type_ *, PyObject * > &);
    void (*erase)(data_type_ *);
    PyObject *(*find)(data_type_ *);
};

struct data_dispatch_ *data_dispatcher_;



typedef piw::data_nb_t data_nb_type_; 


  


struct data_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(data_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(data_nb_type_ *);
    data_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< data_nb_type_ *, PyObject * > &);
    void (*erase)(data_nb_type_ *);
    PyObject *(*find)(data_nb_type_ *);
};

struct data_nb_dispatch_ *data_nb_dispatcher_;



typedef piw::clockdomain_ctl_t clockdomain_ctl_type_; 


 
static std::map<clockdomain_ctl_type_ *, PyObject *> clockdomain_ctl_wrapped_;
 


struct clockdomain_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockdomain_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockdomain_ctl_type_ *);
    clockdomain_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockdomain_ctl_type_ *, PyObject * > &);
    void (*erase)(clockdomain_ctl_type_ *);
    PyObject *(*find)(clockdomain_ctl_type_ *);
};

struct clockdomain_ctl_dispatch_ *clockdomain_ctl_dispatcher_;



typedef piw::clocksink_t clocksink_type_; 


  


struct clocksink_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clocksink_type_ *, bool owned);
    PyObject *(*wrapcopy)(clocksink_type_ *);
    clocksink_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clocksink_type_ *, PyObject * > &);
    void (*erase)(clocksink_type_ *);
    PyObject *(*find)(clocksink_type_ *);
};

struct clocksink_dispatch_ *clocksink_dispatcher_;



typedef piw::clocksource_t clocksource_type_; 


 
static std::map<clocksource_type_ *, PyObject *> clocksource_wrapped_;
 


struct clocksource_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clocksource_type_ *, bool owned);
    PyObject *(*wrapcopy)(clocksource_type_ *);
    clocksource_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clocksource_type_ *, PyObject * > &);
    void (*erase)(clocksource_type_ *);
    PyObject *(*find)(clocksource_type_ *);
};

struct clocksource_dispatch_ *clocksource_dispatcher_;



typedef piw::fast_pitchbender_t fast_pitchbender_type_; 


 
static std::map<fast_pitchbender_type_ *, PyObject *> fast_pitchbender_wrapped_;
 


struct fast_pitchbender_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(fast_pitchbender_type_ *, bool owned);
    PyObject *(*wrapcopy)(fast_pitchbender_type_ *);
    fast_pitchbender_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< fast_pitchbender_type_ *, PyObject * > &);
    void (*erase)(fast_pitchbender_type_ *);
    PyObject *(*find)(fast_pitchbender_type_ *);
};

struct fast_pitchbender_dispatch_ *fast_pitchbender_dispatcher_;



typedef piw::panner_t panner_type_; 


 
static std::map<panner_type_ *, PyObject *> panner_wrapped_;
 


struct panner_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(panner_type_ *, bool owned);
    PyObject *(*wrapcopy)(panner_type_ *);
    panner_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< panner_type_ *, PyObject * > &);
    void (*erase)(panner_type_ *);
    PyObject *(*find)(panner_type_ *);
};

struct panner_dispatch_ *panner_dispatcher_;



typedef piw::gain_t gain_type_; 


 
static std::map<gain_type_ *, PyObject *> gain_wrapped_;
 


struct gain_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(gain_type_ *, bool owned);
    PyObject *(*wrapcopy)(gain_type_ *);
    gain_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< gain_type_ *, PyObject * > &);
    void (*erase)(gain_type_ *);
    PyObject *(*find)(gain_type_ *);
};

struct gain_dispatch_ *gain_dispatcher_;



typedef piw::delay_t delay_type_; 


 
static std::map<delay_type_ *, PyObject *> delay_wrapped_;
 


struct delay_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(delay_type_ *, bool owned);
    PyObject *(*wrapcopy)(delay_type_ *);
    delay_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< delay_type_ *, PyObject * > &);
    void (*erase)(delay_type_ *);
    PyObject *(*find)(delay_type_ *);
};

struct delay_dispatch_ *delay_dispatcher_;



typedef piw::linear_gain_t linear_gain_type_; 


 
static std::map<linear_gain_type_ *, PyObject *> linear_gain_wrapped_;
 


struct linear_gain_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(linear_gain_type_ *, bool owned);
    PyObject *(*wrapcopy)(linear_gain_type_ *);
    linear_gain_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< linear_gain_type_ *, PyObject * > &);
    void (*erase)(linear_gain_type_ *);
    PyObject *(*find)(linear_gain_type_ *);
};

struct linear_gain_dispatch_ *linear_gain_dispatcher_;



typedef piw::selector_t selector_type_; 


 
static std::map<selector_type_ *, PyObject *> selector_wrapped_;
 


struct selector_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(selector_type_ *, bool owned);
    PyObject *(*wrapcopy)(selector_type_ *);
    selector_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< selector_type_ *, PyObject * > &);
    void (*erase)(selector_type_ *);
    PyObject *(*find)(selector_type_ *);
};

struct selector_dispatch_ *selector_dispatcher_;



typedef piw::velocitydetect_t velocitydetect_type_; 


 
static std::map<velocitydetect_type_ *, PyObject *> velocitydetect_wrapped_;
 


struct velocitydetect_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(velocitydetect_type_ *, bool owned);
    PyObject *(*wrapcopy)(velocitydetect_type_ *);
    velocitydetect_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< velocitydetect_type_ *, PyObject * > &);
    void (*erase)(velocitydetect_type_ *);
    PyObject *(*find)(velocitydetect_type_ *);
};

struct velocitydetect_dispatch_ *velocitydetect_dispatcher_;



typedef piw::scaler_controller_t scaler_controller_type_; 


 
static std::map<scaler_controller_type_ *, PyObject *> scaler_controller_wrapped_;
 


struct scaler_controller_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(scaler_controller_type_ *, bool owned);
    PyObject *(*wrapcopy)(scaler_controller_type_ *);
    scaler_controller_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< scaler_controller_type_ *, PyObject * > &);
    void (*erase)(scaler_controller_type_ *);
    PyObject *(*find)(scaler_controller_type_ *);
};

struct scaler_controller_dispatch_ *scaler_controller_dispatcher_;



typedef piw::scaler_t scaler_type_; 


 
static std::map<scaler_type_ *, PyObject *> scaler_wrapped_;
 


struct scaler_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(scaler_type_ *, bool owned);
    PyObject *(*wrapcopy)(scaler_type_ *);
    scaler_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< scaler_type_ *, PyObject * > &);
    void (*erase)(scaler_type_ *);
    PyObject *(*find)(scaler_type_ *);
};

struct scaler_dispatch_ *scaler_dispatcher_;



typedef piw::evtdump_t evtdump_type_; 


 
static std::map<evtdump_type_ *, PyObject *> evtdump_wrapped_;
 


struct evtdump_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(evtdump_type_ *, bool owned);
    PyObject *(*wrapcopy)(evtdump_type_ *);
    evtdump_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< evtdump_type_ *, PyObject * > &);
    void (*erase)(evtdump_type_ *);
    PyObject *(*find)(evtdump_type_ *);
};

struct evtdump_dispatch_ *evtdump_dispatcher_;



typedef piw::wavrecorder_t wavrecorder_type_; 


 
static std::map<wavrecorder_type_ *, PyObject *> wavrecorder_wrapped_;
 


struct wavrecorder_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(wavrecorder_type_ *, bool owned);
    PyObject *(*wrapcopy)(wavrecorder_type_ *);
    wavrecorder_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< wavrecorder_type_ *, PyObject * > &);
    void (*erase)(wavrecorder_type_ *);
    PyObject *(*find)(wavrecorder_type_ *);
};

struct wavrecorder_dispatch_ *wavrecorder_dispatcher_;



typedef piw::python_delegate_t python_delegate_type_; 


 
static std::map<python_delegate_type_ *, PyObject *> python_delegate_wrapped_;
 


struct python_delegate_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(python_delegate_type_ *, bool owned);
    PyObject *(*wrapcopy)(python_delegate_type_ *);
    python_delegate_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< python_delegate_type_ *, PyObject * > &);
    void (*erase)(python_delegate_type_ *);
    PyObject *(*find)(python_delegate_type_ *);
};

struct python_delegate_dispatch_ *python_delegate_dispatcher_;



typedef piw::midi_channel_delegate_t midi_channel_delegate_type_; 


 
static std::map<midi_channel_delegate_type_ *, PyObject *> midi_channel_delegate_wrapped_;
 


struct midi_channel_delegate_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(midi_channel_delegate_type_ *, bool owned);
    PyObject *(*wrapcopy)(midi_channel_delegate_type_ *);
    midi_channel_delegate_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< midi_channel_delegate_type_ *, PyObject * > &);
    void (*erase)(midi_channel_delegate_type_ *);
    PyObject *(*find)(midi_channel_delegate_type_ *);
};

struct midi_channel_delegate_dispatch_ *midi_channel_delegate_dispatcher_;



typedef piw::midi_from_belcanto_t midi_from_belcanto_type_; 


 
static std::map<midi_from_belcanto_type_ *, PyObject *> midi_from_belcanto_wrapped_;
 


struct midi_from_belcanto_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(midi_from_belcanto_type_ *, bool owned);
    PyObject *(*wrapcopy)(midi_from_belcanto_type_ *);
    midi_from_belcanto_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< midi_from_belcanto_type_ *, PyObject * > &);
    void (*erase)(midi_from_belcanto_type_ *);
    PyObject *(*find)(midi_from_belcanto_type_ *);
};

struct midi_from_belcanto_dispatch_ *midi_from_belcanto_dispatcher_;



typedef piw::tsd_snapshot_t tsd_snapshot_type_; 


 
static std::map<tsd_snapshot_type_ *, PyObject *> tsd_snapshot_wrapped_;
 


struct tsd_snapshot_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(tsd_snapshot_type_ *, bool owned);
    PyObject *(*wrapcopy)(tsd_snapshot_type_ *);
    tsd_snapshot_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< tsd_snapshot_type_ *, PyObject * > &);
    void (*erase)(tsd_snapshot_type_ *);
    PyObject *(*find)(tsd_snapshot_type_ *);
};

struct tsd_snapshot_dispatch_ *tsd_snapshot_dispatcher_;



typedef pic::notify_t notify_type_; 





struct notify_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(notify_type_ *, bool owned);
    PyObject *(*wrapcopy)(notify_type_ *);
    notify_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< notify_type_ *, PyObject * > &);
    void (*erase)(notify_type_ *);
    PyObject *(*find)(notify_type_ *);
};

struct notify_dispatch_ *notify_dispatcher_;



typedef pic::status_t status_type_; 





struct status_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(status_type_ *, bool owned);
    PyObject *(*wrapcopy)(status_type_ *);
    status_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< status_type_ *, PyObject * > &);
    void (*erase)(status_type_ *);
    PyObject *(*find)(status_type_ *);
};

struct status_dispatch_ *status_dispatcher_;



typedef pic::f_string_t f_string_type_; 





struct f_string_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(f_string_type_ *, bool owned);
    PyObject *(*wrapcopy)(f_string_type_ *);
    f_string_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< f_string_type_ *, PyObject * > &);
    void (*erase)(f_string_type_ *);
    PyObject *(*find)(f_string_type_ *);
};

struct f_string_dispatch_ *f_string_dispatcher_;



typedef pic::f2f_t f2f_type_; 





struct f2f_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(f2f_type_ *, bool owned);
    PyObject *(*wrapcopy)(f2f_type_ *);
    f2f_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< f2f_type_ *, PyObject * > &);
    void (*erase)(f2f_type_ *);
    PyObject *(*find)(f2f_type_ *);
};

struct f2f_dispatch_ *f2f_dispatcher_;





int fpcvt_event(PyObject *o, void *a)
{
    event_type_ *object;
    if(o == Py_None)
    {
        *((event_type_ **)a) = 0;
        return 1;
    }
    object = event_dispatcher_->unwrap(o);
    *((event_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_event_copy fpcvt_event

PyObject *tpcvt_event(void *a)
{
    event_type_ *o = *((event_type_ **)a);
    return event_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_event_copy(void *a)
{
    event_type_ *o = (event_type_ *)a;
    return event_dispatcher_->wrapcopy(o);
}



int fpcvt_scheduler(PyObject *o, void *a)
{
    scheduler_type_ *object;
    if(o == Py_None)
    {
        *((scheduler_type_ **)a) = 0;
        return 1;
    }
    object = scheduler_dispatcher_->unwrap(o);
    *((scheduler_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_scheduler_copy fpcvt_scheduler

PyObject *tpcvt_scheduler(void *a)
{
    scheduler_type_ *o = *((scheduler_type_ **)a);
    return scheduler_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_scheduler_copy(void *a)
{
    scheduler_type_ *o = (scheduler_type_ *)a;
    return scheduler_dispatcher_->wrapcopy(o);
}



int fpcvt_rpctoken(PyObject *o, void *a)
{
    rpctoken_type_ *object;
    if(o == Py_None)
    {
        *((rpctoken_type_ **)a) = 0;
        return 1;
    }
    object = rpctoken_dispatcher_->unwrap(o);
    *((rpctoken_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_rpctoken_copy fpcvt_rpctoken

PyObject *tpcvt_rpctoken(void *a)
{
    rpctoken_type_ *o = *((rpctoken_type_ **)a);
    return rpctoken_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_rpctoken_copy(void *a)
{
    rpctoken_type_ *o = (rpctoken_type_ *)a;
    return rpctoken_dispatcher_->wrapcopy(o);
}



int fpcvt_rpcserver(PyObject *o, void *a)
{
    rpcserver_type_ *object;
    if(o == Py_None)
    {
        *((rpcserver_type_ **)a) = 0;
        return 1;
    }
    object = rpcserver_dispatcher_->unwrap(o);
    *((rpcserver_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_rpcserver_copy fpcvt_rpcserver

PyObject *tpcvt_rpcserver(void *a)
{
    rpcserver_type_ *o = *((rpcserver_type_ **)a);
    return rpcserver_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_rpcserver_copy(void *a)
{
    rpcserver_type_ *o = (rpcserver_type_ *)a;
    return rpcserver_dispatcher_->wrapcopy(o);
}



int fpcvt_rpcclient(PyObject *o, void *a)
{
    rpcclient_type_ *object;
    if(o == Py_None)
    {
        *((rpcclient_type_ **)a) = 0;
        return 1;
    }
    object = rpcclient_dispatcher_->unwrap(o);
    *((rpcclient_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_rpcclient_copy fpcvt_rpcclient

PyObject *tpcvt_rpcclient(void *a)
{
    rpcclient_type_ *o = *((rpcclient_type_ **)a);
    return rpcclient_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_rpcclient_copy(void *a)
{
    rpcclient_type_ *o = (rpcclient_type_ *)a;
    return rpcclient_dispatcher_->wrapcopy(o);
}



int fpcvt_samplearray(PyObject *o, void *a)
{
    samplearray_type_ *object;
    if(o == Py_None)
    {
        *((samplearray_type_ **)a) = 0;
        return 1;
    }
    object = samplearray_dispatcher_->unwrap(o);
    *((samplearray_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_samplearray_copy fpcvt_samplearray

PyObject *tpcvt_samplearray(void *a)
{
    samplearray_type_ *o = *((samplearray_type_ **)a);
    return samplearray_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_samplearray_copy(void *a)
{
    samplearray_type_ *o = (samplearray_type_ *)a;
    return samplearray_dispatcher_->wrapcopy(o);
}



int fpcvt_sample(PyObject *o, void *a)
{
    sample_type_ *object;
    if(o == Py_None)
    {
        *((sample_type_ **)a) = 0;
        return 1;
    }
    object = sample_dispatcher_->unwrap(o);
    *((sample_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_sample_copy fpcvt_sample

PyObject *tpcvt_sample(void *a)
{
    sample_type_ *o = *((sample_type_ **)a);
    return sample_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_sample_copy(void *a)
{
    sample_type_ *o = (sample_type_ *)a;
    return sample_dispatcher_->wrapcopy(o);
}



int fpcvt_zone(PyObject *o, void *a)
{
    zone_type_ *object;
    if(o == Py_None)
    {
        *((zone_type_ **)a) = 0;
        return 1;
    }
    object = zone_dispatcher_->unwrap(o);
    *((zone_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_zone_copy fpcvt_zone

PyObject *tpcvt_zone(void *a)
{
    zone_type_ *o = *((zone_type_ **)a);
    return zone_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_zone_copy(void *a)
{
    zone_type_ *o = (zone_type_ *)a;
    return zone_dispatcher_->wrapcopy(o);
}



int fpcvt_preset(PyObject *o, void *a)
{
    preset_type_ *object;
    if(o == Py_None)
    {
        *((preset_type_ **)a) = 0;
        return 1;
    }
    object = preset_dispatcher_->unwrap(o);
    *((preset_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_preset_copy fpcvt_preset

PyObject *tpcvt_preset(void *a)
{
    preset_type_ *o = *((preset_type_ **)a);
    return preset_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_preset_copy(void *a)
{
    preset_type_ *o = (preset_type_ *)a;
    return preset_dispatcher_->wrapcopy(o);
}



int fpcvt_clockref(PyObject *o, void *a)
{
    clockref_type_ *object;
    if(o == Py_None)
    {
        *((clockref_type_ **)a) = 0;
        return 1;
    }
    object = clockref_dispatcher_->unwrap(o);
    *((clockref_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockref_copy fpcvt_clockref

PyObject *tpcvt_clockref(void *a)
{
    clockref_type_ *o = *((clockref_type_ **)a);
    return clockref_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockref_copy(void *a)
{
    clockref_type_ *o = (clockref_type_ *)a;
    return clockref_dispatcher_->wrapcopy(o);
}



int fpcvt_strummer(PyObject *o, void *a)
{
    strummer_type_ *object;
    if(o == Py_None)
    {
        *((strummer_type_ **)a) = 0;
        return 1;
    }
    object = strummer_dispatcher_->unwrap(o);
    *((strummer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_strummer_copy fpcvt_strummer

PyObject *tpcvt_strummer(void *a)
{
    strummer_type_ *o = *((strummer_type_ **)a);
    return strummer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_strummer_copy(void *a)
{
    strummer_type_ *o = (strummer_type_ *)a;
    return strummer_dispatcher_->wrapcopy(o);
}



int fpcvt_stringer(PyObject *o, void *a)
{
    stringer_type_ *object;
    if(o == Py_None)
    {
        *((stringer_type_ **)a) = 0;
        return 1;
    }
    object = stringer_dispatcher_->unwrap(o);
    *((stringer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_stringer_copy fpcvt_stringer

PyObject *tpcvt_stringer(void *a)
{
    stringer_type_ *o = *((stringer_type_ **)a);
    return stringer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_stringer_copy(void *a)
{
    stringer_type_ *o = (stringer_type_ *)a;
    return stringer_dispatcher_->wrapcopy(o);
}



int fpcvt_cycler(PyObject *o, void *a)
{
    cycler_type_ *object;
    if(o == Py_None)
    {
        *((cycler_type_ **)a) = 0;
        return 1;
    }
    object = cycler_dispatcher_->unwrap(o);
    *((cycler_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cycler_copy fpcvt_cycler

PyObject *tpcvt_cycler(void *a)
{
    cycler_type_ *o = *((cycler_type_ **)a);
    return cycler_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cycler_copy(void *a)
{
    cycler_type_ *o = (cycler_type_ *)a;
    return cycler_dispatcher_->wrapcopy(o);
}



int fpcvt_polyctl(PyObject *o, void *a)
{
    polyctl_type_ *object;
    if(o == Py_None)
    {
        *((polyctl_type_ **)a) = 0;
        return 1;
    }
    object = polyctl_dispatcher_->unwrap(o);
    *((polyctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_polyctl_copy fpcvt_polyctl

PyObject *tpcvt_polyctl(void *a)
{
    polyctl_type_ *o = *((polyctl_type_ **)a);
    return polyctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_polyctl_copy(void *a)
{
    polyctl_type_ *o = (polyctl_type_ *)a;
    return polyctl_dispatcher_->wrapcopy(o);
}



int fpcvt_stereomixer(PyObject *o, void *a)
{
    stereomixer_type_ *object;
    if(o == Py_None)
    {
        *((stereomixer_type_ **)a) = 0;
        return 1;
    }
    object = stereomixer_dispatcher_->unwrap(o);
    *((stereomixer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_stereomixer_copy fpcvt_stereomixer

PyObject *tpcvt_stereomixer(void *a)
{
    stereomixer_type_ *o = *((stereomixer_type_ **)a);
    return stereomixer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_stereomixer_copy(void *a)
{
    stereomixer_type_ *o = (stereomixer_type_ *)a;
    return stereomixer_dispatcher_->wrapcopy(o);
}



int fpcvt_stereosummer(PyObject *o, void *a)
{
    stereosummer_type_ *object;
    if(o == Py_None)
    {
        *((stereosummer_type_ **)a) = 0;
        return 1;
    }
    object = stereosummer_dispatcher_->unwrap(o);
    *((stereosummer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_stereosummer_copy fpcvt_stereosummer

PyObject *tpcvt_stereosummer(void *a)
{
    stereosummer_type_ *o = *((stereosummer_type_ **)a);
    return stereosummer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_stereosummer_copy(void *a)
{
    stereosummer_type_ *o = (stereosummer_type_ *)a;
    return stereosummer_dispatcher_->wrapcopy(o);
}



int fpcvt_monomixer(PyObject *o, void *a)
{
    monomixer_type_ *object;
    if(o == Py_None)
    {
        *((monomixer_type_ **)a) = 0;
        return 1;
    }
    object = monomixer_dispatcher_->unwrap(o);
    *((monomixer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_monomixer_copy fpcvt_monomixer

PyObject *tpcvt_monomixer(void *a)
{
    monomixer_type_ *o = *((monomixer_type_ **)a);
    return monomixer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_monomixer_copy(void *a)
{
    monomixer_type_ *o = (monomixer_type_ *)a;
    return monomixer_dispatcher_->wrapcopy(o);
}



int fpcvt_consolemixer(PyObject *o, void *a)
{
    consolemixer_type_ *object;
    if(o == Py_None)
    {
        *((consolemixer_type_ **)a) = 0;
        return 1;
    }
    object = consolemixer_dispatcher_->unwrap(o);
    *((consolemixer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_consolemixer_copy fpcvt_consolemixer

PyObject *tpcvt_consolemixer(void *a)
{
    consolemixer_type_ *o = *((consolemixer_type_ **)a);
    return consolemixer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_consolemixer_copy(void *a)
{
    consolemixer_type_ *o = (consolemixer_type_ *)a;
    return consolemixer_dispatcher_->wrapcopy(o);
}



int fpcvt_gate(PyObject *o, void *a)
{
    gate_type_ *object;
    if(o == Py_None)
    {
        *((gate_type_ **)a) = 0;
        return 1;
    }
    object = gate_dispatcher_->unwrap(o);
    *((gate_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_gate_copy fpcvt_gate

PyObject *tpcvt_gate(void *a)
{
    gate_type_ *o = *((gate_type_ **)a);
    return gate_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_gate_copy(void *a)
{
    gate_type_ *o = (gate_type_ *)a;
    return gate_dispatcher_->wrapcopy(o);
}



int fpcvt_correlator(PyObject *o, void *a)
{
    correlator_type_ *object;
    if(o == Py_None)
    {
        *((correlator_type_ **)a) = 0;
        return 1;
    }
    object = correlator_dispatcher_->unwrap(o);
    *((correlator_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_correlator_copy fpcvt_correlator

PyObject *tpcvt_correlator(void *a)
{
    correlator_type_ *o = *((correlator_type_ **)a);
    return correlator_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_correlator_copy(void *a)
{
    correlator_type_ *o = (correlator_type_ *)a;
    return correlator_dispatcher_->wrapcopy(o);
}



int fpcvt_connector(PyObject *o, void *a)
{
    connector_type_ *object;
    if(o == Py_None)
    {
        *((connector_type_ **)a) = 0;
        return 1;
    }
    object = connector_dispatcher_->unwrap(o);
    *((connector_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_connector_copy fpcvt_connector

PyObject *tpcvt_connector(void *a)
{
    connector_type_ *o = *((connector_type_ **)a);
    return connector_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_connector_copy(void *a)
{
    connector_type_ *o = (connector_type_ *)a;
    return connector_dispatcher_->wrapcopy(o);
}



int fpcvt_converter_ref(PyObject *o, void *a)
{
    converter_ref_type_ *object;
    if(o == Py_None)
    {
        *((converter_ref_type_ **)a) = 0;
        return 1;
    }
    object = converter_ref_dispatcher_->unwrap(o);
    *((converter_ref_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_converter_ref_copy fpcvt_converter_ref

PyObject *tpcvt_converter_ref(void *a)
{
    converter_ref_type_ *o = *((converter_ref_type_ **)a);
    return converter_ref_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_converter_ref_copy(void *a)
{
    converter_ref_type_ *o = (converter_ref_type_ *)a;
    return converter_ref_dispatcher_->wrapcopy(o);
}



int fpcvt_capture_backend(PyObject *o, void *a)
{
    capture_backend_type_ *object;
    if(o == Py_None)
    {
        *((capture_backend_type_ **)a) = 0;
        return 1;
    }
    object = capture_backend_dispatcher_->unwrap(o);
    *((capture_backend_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_capture_backend_copy fpcvt_capture_backend

PyObject *tpcvt_capture_backend(void *a)
{
    capture_backend_type_ *o = *((capture_backend_type_ **)a);
    return capture_backend_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_capture_backend_copy(void *a)
{
    capture_backend_type_ *o = (capture_backend_type_ *)a;
    return capture_backend_dispatcher_->wrapcopy(o);
}



int fpcvt_functor_backend(PyObject *o, void *a)
{
    functor_backend_type_ *object;
    if(o == Py_None)
    {
        *((functor_backend_type_ **)a) = 0;
        return 1;
    }
    object = functor_backend_dispatcher_->unwrap(o);
    *((functor_backend_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_functor_backend_copy fpcvt_functor_backend

PyObject *tpcvt_functor_backend(void *a)
{
    functor_backend_type_ *o = *((functor_backend_type_ **)a);
    return functor_backend_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_functor_backend_copy(void *a)
{
    functor_backend_type_ *o = (functor_backend_type_ *)a;
    return functor_backend_dispatcher_->wrapcopy(o);
}



int fpcvt_controller(PyObject *o, void *a)
{
    controller_type_ *object;
    if(o == Py_None)
    {
        *((controller_type_ **)a) = 0;
        return 1;
    }
    object = controller_dispatcher_->unwrap(o);
    *((controller_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_controller_copy fpcvt_controller

PyObject *tpcvt_controller(void *a)
{
    controller_type_ *o = *((controller_type_ **)a);
    return controller_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_controller_copy(void *a)
{
    controller_type_ *o = (controller_type_ *)a;
    return controller_dispatcher_->wrapcopy(o);
}



int fpcvt_xxcontrolled(PyObject *o, void *a)
{
    xxcontrolled_type_ *object;
    if(o == Py_None)
    {
        *((xxcontrolled_type_ **)a) = 0;
        return 1;
    }
    object = xxcontrolled_dispatcher_->unwrap(o);
    *((xxcontrolled_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_xxcontrolled_copy fpcvt_xxcontrolled

PyObject *tpcvt_xxcontrolled(void *a)
{
    xxcontrolled_type_ *o = *((xxcontrolled_type_ **)a);
    return xxcontrolled_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_xxcontrolled_copy(void *a)
{
    xxcontrolled_type_ *o = (xxcontrolled_type_ *)a;
    return xxcontrolled_dispatcher_->wrapcopy(o);
}



int fpcvt_controlled(PyObject *o, void *a)
{
    controlled_type_ *object;
    if(o == Py_None)
    {
        *((controlled_type_ **)a) = 0;
        return 1;
    }
    object = controlled_dispatcher_->unwrap(o);
    *((controlled_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_controlled_copy fpcvt_controlled

PyObject *tpcvt_controlled(void *a)
{
    controlled_type_ *o = *((controlled_type_ **)a);
    return controlled_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_controlled_copy(void *a)
{
    controlled_type_ *o = (controlled_type_ *)a;
    return controlled_dispatcher_->wrapcopy(o);
}



int fpcvt_fasttrigger(PyObject *o, void *a)
{
    fasttrigger_type_ *object;
    if(o == Py_None)
    {
        *((fasttrigger_type_ **)a) = 0;
        return 1;
    }
    object = fasttrigger_dispatcher_->unwrap(o);
    *((fasttrigger_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_fasttrigger_copy fpcvt_fasttrigger

PyObject *tpcvt_fasttrigger(void *a)
{
    fasttrigger_type_ *o = *((fasttrigger_type_ **)a);
    return fasttrigger_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_fasttrigger_copy(void *a)
{
    fasttrigger_type_ *o = (fasttrigger_type_ *)a;
    return fasttrigger_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_decode_ctl(PyObject *o, void *a)
{
    decode_ctl_type_ *object;
    if(o == Py_None)
    {
        *((decode_ctl_type_ **)a) = 0;
        return 1;
    }
    object = decode_ctl_dispatcher_->unwrap(o);
    *((decode_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_decode_ctl_copy fpcvt_decode_ctl

PyObject *tpcvt_decode_ctl(void *a)
{
    decode_ctl_type_ *o = *((decode_ctl_type_ **)a);
    return decode_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_decode_ctl_copy(void *a)
{
    decode_ctl_type_ *o = (decode_ctl_type_ *)a;
    return decode_ctl_dispatcher_->wrapcopy(o);
}



int fpcvt_decoder(PyObject *o, void *a)
{
    decoder_type_ *object;
    if(o == Py_None)
    {
        *((decoder_type_ **)a) = 0;
        return 1;
    }
    object = decoder_dispatcher_->unwrap(o);
    *((decoder_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_decoder_copy fpcvt_decoder

PyObject *tpcvt_decoder(void *a)
{
    decoder_type_ *o = *((decoder_type_ **)a);
    return decoder_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_decoder_copy(void *a)
{
    decoder_type_ *o = (decoder_type_ *)a;
    return decoder_dispatcher_->wrapcopy(o);
}



int fpcvt_event_data_buffer(PyObject *o, void *a)
{
    event_data_buffer_type_ *object;
    if(o == Py_None)
    {
        *((event_data_buffer_type_ **)a) = 0;
        return 1;
    }
    object = event_data_buffer_dispatcher_->unwrap(o);
    *((event_data_buffer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_event_data_buffer_copy fpcvt_event_data_buffer

PyObject *tpcvt_event_data_buffer(void *a)
{
    event_data_buffer_type_ *o = *((event_data_buffer_type_ **)a);
    return event_data_buffer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_event_data_buffer_copy(void *a)
{
    event_data_buffer_type_ *o = (event_data_buffer_type_ *)a;
    return event_data_buffer_dispatcher_->wrapcopy(o);
}



int fpcvt_event_data_source(PyObject *o, void *a)
{
    event_data_source_type_ *object;
    if(o == Py_None)
    {
        *((event_data_source_type_ **)a) = 0;
        return 1;
    }
    object = event_data_source_dispatcher_->unwrap(o);
    *((event_data_source_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_event_data_source_copy fpcvt_event_data_source

PyObject *tpcvt_event_data_source(void *a)
{
    event_data_source_type_ *o = *((event_data_source_type_ **)a);
    return event_data_source_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_event_data_source_copy(void *a)
{
    event_data_source_type_ *o = (event_data_source_type_ *)a;
    return event_data_source_dispatcher_->wrapcopy(o);
}



int fpcvt_event_data_source_real(PyObject *o, void *a)
{
    event_data_source_real_type_ *object;
    if(o == Py_None)
    {
        *((event_data_source_real_type_ **)a) = 0;
        return 1;
    }
    object = event_data_source_real_dispatcher_->unwrap(o);
    *((event_data_source_real_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_event_data_source_real_copy fpcvt_event_data_source_real

PyObject *tpcvt_event_data_source_real(void *a)
{
    event_data_source_real_type_ *o = *((event_data_source_real_type_ **)a);
    return event_data_source_real_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_event_data_source_real_copy(void *a)
{
    event_data_source_real_type_ *o = (event_data_source_real_type_ *)a;
    return event_data_source_real_dispatcher_->wrapcopy(o);
}



int fpcvt_wire_ctl(PyObject *o, void *a)
{
    wire_ctl_type_ *object;
    if(o == Py_None)
    {
        *((wire_ctl_type_ **)a) = 0;
        return 1;
    }
    object = wire_ctl_dispatcher_->unwrap(o);
    *((wire_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_wire_ctl_copy fpcvt_wire_ctl

PyObject *tpcvt_wire_ctl(void *a)
{
    wire_ctl_type_ *o = *((wire_ctl_type_ **)a);
    return wire_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_wire_ctl_copy(void *a)
{
    wire_ctl_type_ *o = (wire_ctl_type_ *)a;
    return wire_ctl_dispatcher_->wrapcopy(o);
}



int fpcvt_root_ctl(PyObject *o, void *a)
{
    root_ctl_type_ *object;
    if(o == Py_None)
    {
        *((root_ctl_type_ **)a) = 0;
        return 1;
    }
    object = root_ctl_dispatcher_->unwrap(o);
    *((root_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_root_ctl_copy fpcvt_root_ctl

PyObject *tpcvt_root_ctl(void *a)
{
    root_ctl_type_ *o = *((root_ctl_type_ **)a);
    return root_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_root_ctl_copy(void *a)
{
    root_ctl_type_ *o = (root_ctl_type_ *)a;
    return root_ctl_dispatcher_->wrapcopy(o);
}



int fpcvt_wire(PyObject *o, void *a)
{
    wire_type_ *object;
    if(o == Py_None)
    {
        *((wire_type_ **)a) = 0;
        return 1;
    }
    object = wire_dispatcher_->unwrap(o);
    *((wire_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_wire_copy fpcvt_wire

PyObject *tpcvt_wire(void *a)
{
    wire_type_ *o = *((wire_type_ **)a);
    return wire_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_wire_copy(void *a)
{
    wire_type_ *o = (wire_type_ *)a;
    return wire_dispatcher_->wrapcopy(o);
}



int fpcvt_root(PyObject *o, void *a)
{
    root_type_ *object;
    if(o == Py_None)
    {
        *((root_type_ **)a) = 0;
        return 1;
    }
    object = root_dispatcher_->unwrap(o);
    *((root_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_root_copy fpcvt_root

PyObject *tpcvt_root(void *a)
{
    root_type_ *o = *((root_type_ **)a);
    return root_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_root_copy(void *a)
{
    root_type_ *o = (root_type_ *)a;
    return root_dispatcher_->wrapcopy(o);
}



int fpcvt_multiplexer(PyObject *o, void *a)
{
    multiplexer_type_ *object;
    if(o == Py_None)
    {
        *((multiplexer_type_ **)a) = 0;
        return 1;
    }
    object = multiplexer_dispatcher_->unwrap(o);
    *((multiplexer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_multiplexer_copy fpcvt_multiplexer

PyObject *tpcvt_multiplexer(void *a)
{
    multiplexer_type_ *o = *((multiplexer_type_ **)a);
    return multiplexer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_multiplexer_copy(void *a)
{
    multiplexer_type_ *o = (multiplexer_type_ *)a;
    return multiplexer_dispatcher_->wrapcopy(o);
}



int fpcvt_ranger(PyObject *o, void *a)
{
    ranger_type_ *object;
    if(o == Py_None)
    {
        *((ranger_type_ **)a) = 0;
        return 1;
    }
    object = ranger_dispatcher_->unwrap(o);
    *((ranger_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_ranger_copy fpcvt_ranger

PyObject *tpcvt_ranger(void *a)
{
    ranger_type_ *o = *((ranger_type_ **)a);
    return ranger_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_ranger_copy(void *a)
{
    ranger_type_ *o = (ranger_type_ *)a;
    return ranger_dispatcher_->wrapcopy(o);
}



int fpcvt_lightconvertor(PyObject *o, void *a)
{
    lightconvertor_type_ *object;
    if(o == Py_None)
    {
        *((lightconvertor_type_ **)a) = 0;
        return 1;
    }
    object = lightconvertor_dispatcher_->unwrap(o);
    *((lightconvertor_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_lightconvertor_copy fpcvt_lightconvertor

PyObject *tpcvt_lightconvertor(void *a)
{
    lightconvertor_type_ *o = *((lightconvertor_type_ **)a);
    return lightconvertor_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_lightconvertor_copy(void *a)
{
    lightconvertor_type_ *o = (lightconvertor_type_ *)a;
    return lightconvertor_dispatcher_->wrapcopy(o);
}



int fpcvt_statusbuffer(PyObject *o, void *a)
{
    statusbuffer_type_ *object;
    if(o == Py_None)
    {
        *((statusbuffer_type_ **)a) = 0;
        return 1;
    }
    object = statusbuffer_dispatcher_->unwrap(o);
    *((statusbuffer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_statusbuffer_copy fpcvt_statusbuffer

PyObject *tpcvt_statusbuffer(void *a)
{
    statusbuffer_type_ *o = *((statusbuffer_type_ **)a);
    return statusbuffer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_statusbuffer_copy(void *a)
{
    statusbuffer_type_ *o = (statusbuffer_type_ *)a;
    return statusbuffer_dispatcher_->wrapcopy(o);
}



int fpcvt_lightsource(PyObject *o, void *a)
{
    lightsource_type_ *object;
    if(o == Py_None)
    {
        *((lightsource_type_ **)a) = 0;
        return 1;
    }
    object = lightsource_dispatcher_->unwrap(o);
    *((lightsource_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_lightsource_copy fpcvt_lightsource

PyObject *tpcvt_lightsource(void *a)
{
    lightsource_type_ *o = *((lightsource_type_ **)a);
    return lightsource_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_lightsource_copy(void *a)
{
    lightsource_type_ *o = (lightsource_type_ *)a;
    return lightsource_dispatcher_->wrapcopy(o);
}



int fpcvt_statusmixer(PyObject *o, void *a)
{
    statusmixer_type_ *object;
    if(o == Py_None)
    {
        *((statusmixer_type_ **)a) = 0;
        return 1;
    }
    object = statusmixer_dispatcher_->unwrap(o);
    *((statusmixer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_statusmixer_copy fpcvt_statusmixer

PyObject *tpcvt_statusmixer(void *a)
{
    statusmixer_type_ *o = *((statusmixer_type_ **)a);
    return statusmixer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_statusmixer_copy(void *a)
{
    statusmixer_type_ *o = (statusmixer_type_ *)a;
    return statusmixer_dispatcher_->wrapcopy(o);
}



int fpcvt_controllerdict(PyObject *o, void *a)
{
    controllerdict_type_ *object;
    if(o == Py_None)
    {
        *((controllerdict_type_ **)a) = 0;
        return 1;
    }
    object = controllerdict_dispatcher_->unwrap(o);
    *((controllerdict_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_controllerdict_copy fpcvt_controllerdict

PyObject *tpcvt_controllerdict(void *a)
{
    controllerdict_type_ *o = *((controllerdict_type_ **)a);
    return controllerdict_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_controllerdict_copy(void *a)
{
    controllerdict_type_ *o = (controllerdict_type_ *)a;
    return controllerdict_dispatcher_->wrapcopy(o);
}



int fpcvt_mapping_info(PyObject *o, void *a)
{
    mapping_info_type_ *object;
    if(o == Py_None)
    {
        *((mapping_info_type_ **)a) = 0;
        return 1;
    }
    object = mapping_info_dispatcher_->unwrap(o);
    *((mapping_info_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_mapping_info_copy fpcvt_mapping_info

PyObject *tpcvt_mapping_info(void *a)
{
    mapping_info_type_ *o = *((mapping_info_type_ **)a);
    return mapping_info_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_mapping_info_copy(void *a)
{
    mapping_info_type_ *o = (mapping_info_type_ *)a;
    return mapping_info_dispatcher_->wrapcopy(o);
}



int fpcvt_midi_converter(PyObject *o, void *a)
{
    midi_converter_type_ *object;
    if(o == Py_None)
    {
        *((midi_converter_type_ **)a) = 0;
        return 1;
    }
    object = midi_converter_dispatcher_->unwrap(o);
    *((midi_converter_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_midi_converter_copy fpcvt_midi_converter

PyObject *tpcvt_midi_converter(void *a)
{
    midi_converter_type_ *o = *((midi_converter_type_ **)a);
    return midi_converter_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_midi_converter_copy(void *a)
{
    midi_converter_type_ *o = (midi_converter_type_ *)a;
    return midi_converter_dispatcher_->wrapcopy(o);
}



int fpcvt_mapping_observer(PyObject *o, void *a)
{
    mapping_observer_type_ *object;
    if(o == Py_None)
    {
        *((mapping_observer_type_ **)a) = 0;
        return 1;
    }
    object = mapping_observer_dispatcher_->unwrap(o);
    *((mapping_observer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_mapping_observer_copy fpcvt_mapping_observer

PyObject *tpcvt_mapping_observer(void *a)
{
    mapping_observer_type_ *o = *((mapping_observer_type_ **)a);
    return mapping_observer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_mapping_observer_copy(void *a)
{
    mapping_observer_type_ *o = (mapping_observer_type_ *)a;
    return mapping_observer_dispatcher_->wrapcopy(o);
}



int fpcvt_kgroup_mapper(PyObject *o, void *a)
{
    kgroup_mapper_type_ *object;
    if(o == Py_None)
    {
        *((kgroup_mapper_type_ **)a) = 0;
        return 1;
    }
    object = kgroup_mapper_dispatcher_->unwrap(o);
    *((kgroup_mapper_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_kgroup_mapper_copy fpcvt_kgroup_mapper

PyObject *tpcvt_kgroup_mapper(void *a)
{
    kgroup_mapper_type_ *o = *((kgroup_mapper_type_ **)a);
    return kgroup_mapper_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_kgroup_mapper_copy(void *a)
{
    kgroup_mapper_type_ *o = (kgroup_mapper_type_ *)a;
    return kgroup_mapper_dispatcher_->wrapcopy(o);
}



int fpcvt_sclone(PyObject *o, void *a)
{
    sclone_type_ *object;
    if(o == Py_None)
    {
        *((sclone_type_ **)a) = 0;
        return 1;
    }
    object = sclone_dispatcher_->unwrap(o);
    *((sclone_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_sclone_copy fpcvt_sclone

PyObject *tpcvt_sclone(void *a)
{
    sclone_type_ *o = *((sclone_type_ **)a);
    return sclone_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_sclone_copy(void *a)
{
    sclone_type_ *o = (sclone_type_ *)a;
    return sclone_dispatcher_->wrapcopy(o);
}



int fpcvt_clone(PyObject *o, void *a)
{
    clone_type_ *object;
    if(o == Py_None)
    {
        *((clone_type_ **)a) = 0;
        return 1;
    }
    object = clone_dispatcher_->unwrap(o);
    *((clone_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clone_copy fpcvt_clone

PyObject *tpcvt_clone(void *a)
{
    clone_type_ *o = *((clone_type_ **)a);
    return clone_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clone_copy(void *a)
{
    clone_type_ *o = (clone_type_ *)a;
    return clone_dispatcher_->wrapcopy(o);
}



int fpcvt_aggregator(PyObject *o, void *a)
{
    aggregator_type_ *object;
    if(o == Py_None)
    {
        *((aggregator_type_ **)a) = 0;
        return 1;
    }
    object = aggregator_dispatcher_->unwrap(o);
    *((aggregator_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_aggregator_copy fpcvt_aggregator

PyObject *tpcvt_aggregator(void *a)
{
    aggregator_type_ *o = *((aggregator_type_ **)a);
    return aggregator_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_aggregator_copy(void *a)
{
    aggregator_type_ *o = (aggregator_type_ *)a;
    return aggregator_dispatcher_->wrapcopy(o);
}



int fpcvt_scrolldelegate(PyObject *o, void *a)
{
    scrolldelegate_type_ *object;
    if(o == Py_None)
    {
        *((scrolldelegate_type_ **)a) = 0;
        return 1;
    }
    object = scrolldelegate_dispatcher_->unwrap(o);
    *((scrolldelegate_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_scrolldelegate_copy fpcvt_scrolldelegate

PyObject *tpcvt_scrolldelegate(void *a)
{
    scrolldelegate_type_ *o = *((scrolldelegate_type_ **)a);
    return scrolldelegate_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_scrolldelegate_copy(void *a)
{
    scrolldelegate_type_ *o = (scrolldelegate_type_ *)a;
    return scrolldelegate_dispatcher_->wrapcopy(o);
}



int fpcvt_scroller(PyObject *o, void *a)
{
    scroller_type_ *object;
    if(o == Py_None)
    {
        *((scroller_type_ **)a) = 0;
        return 1;
    }
    object = scroller_dispatcher_->unwrap(o);
    *((scroller_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_scroller_copy fpcvt_scroller

PyObject *tpcvt_scroller(void *a)
{
    scroller_type_ *o = *((scroller_type_ **)a);
    return scroller_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_scroller_copy(void *a)
{
    scroller_type_ *o = (scroller_type_ *)a;
    return scroller_dispatcher_->wrapcopy(o);
}



int fpcvt_scroller2(PyObject *o, void *a)
{
    scroller2_type_ *object;
    if(o == Py_None)
    {
        *((scroller2_type_ **)a) = 0;
        return 1;
    }
    object = scroller2_dispatcher_->unwrap(o);
    *((scroller2_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_scroller2_copy fpcvt_scroller2

PyObject *tpcvt_scroller2(void *a)
{
    scroller2_type_ *o = *((scroller2_type_ **)a);
    return scroller2_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_scroller2_copy(void *a)
{
    scroller2_type_ *o = (scroller2_type_ *)a;
    return scroller2_dispatcher_->wrapcopy(o);
}



int fpcvt_splitter_node(PyObject *o, void *a)
{
    splitter_node_type_ *object;
    if(o == Py_None)
    {
        *((splitter_node_type_ **)a) = 0;
        return 1;
    }
    object = splitter_node_dispatcher_->unwrap(o);
    *((splitter_node_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_splitter_node_copy fpcvt_splitter_node

PyObject *tpcvt_splitter_node(void *a)
{
    splitter_node_type_ *o = *((splitter_node_type_ **)a);
    return splitter_node_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_splitter_node_copy(void *a)
{
    splitter_node_type_ *o = (splitter_node_type_ *)a;
    return splitter_node_dispatcher_->wrapcopy(o);
}



int fpcvt_splitter(PyObject *o, void *a)
{
    splitter_type_ *object;
    if(o == Py_None)
    {
        *((splitter_type_ **)a) = 0;
        return 1;
    }
    object = splitter_dispatcher_->unwrap(o);
    *((splitter_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_splitter_copy fpcvt_splitter

PyObject *tpcvt_splitter(void *a)
{
    splitter_type_ *o = *((splitter_type_ **)a);
    return splitter_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_splitter_copy(void *a)
{
    splitter_type_ *o = (splitter_type_ *)a;
    return splitter_dispatcher_->wrapcopy(o);
}



int fpcvt_d2b(PyObject *o, void *a)
{
    d2b_type_ *object;
    if(o == Py_None)
    {
        *((d2b_type_ **)a) = 0;
        return 1;
    }
    object = d2b_dispatcher_->unwrap(o);
    *((d2b_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_d2b_copy fpcvt_d2b

PyObject *tpcvt_d2b(void *a)
{
    d2b_type_ *o = *((d2b_type_ **)a);
    return d2b_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_d2b_copy(void *a)
{
    d2b_type_ *o = (d2b_type_ *)a;
    return d2b_dispatcher_->wrapcopy(o);
}



int fpcvt_d2b_nb(PyObject *o, void *a)
{
    d2b_nb_type_ *object;
    if(o == Py_None)
    {
        *((d2b_nb_type_ **)a) = 0;
        return 1;
    }
    object = d2b_nb_dispatcher_->unwrap(o);
    *((d2b_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_d2b_nb_copy fpcvt_d2b_nb

PyObject *tpcvt_d2b_nb(void *a)
{
    d2b_nb_type_ *o = *((d2b_nb_type_ **)a);
    return d2b_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_d2b_nb_copy(void *a)
{
    d2b_nb_type_ *o = (d2b_nb_type_ *)a;
    return d2b_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_d2d(PyObject *o, void *a)
{
    d2d_type_ *object;
    if(o == Py_None)
    {
        *((d2d_type_ **)a) = 0;
        return 1;
    }
    object = d2d_dispatcher_->unwrap(o);
    *((d2d_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_d2d_copy fpcvt_d2d

PyObject *tpcvt_d2d(void *a)
{
    d2d_type_ *o = *((d2d_type_ **)a);
    return d2d_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_d2d_copy(void *a)
{
    d2d_type_ *o = (d2d_type_ *)a;
    return d2d_dispatcher_->wrapcopy(o);
}



int fpcvt_d2d_nb(PyObject *o, void *a)
{
    d2d_nb_type_ *object;
    if(o == Py_None)
    {
        *((d2d_nb_type_ **)a) = 0;
        return 1;
    }
    object = d2d_nb_dispatcher_->unwrap(o);
    *((d2d_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_d2d_nb_copy fpcvt_d2d_nb

PyObject *tpcvt_d2d_nb(void *a)
{
    d2d_nb_type_ *o = *((d2d_nb_type_ **)a);
    return d2d_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_d2d_nb_copy(void *a)
{
    d2d_nb_type_ *o = (d2d_nb_type_ *)a;
    return d2d_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_dd2d(PyObject *o, void *a)
{
    dd2d_type_ *object;
    if(o == Py_None)
    {
        *((dd2d_type_ **)a) = 0;
        return 1;
    }
    object = dd2d_dispatcher_->unwrap(o);
    *((dd2d_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_dd2d_copy fpcvt_dd2d

PyObject *tpcvt_dd2d(void *a)
{
    dd2d_type_ *o = *((dd2d_type_ **)a);
    return dd2d_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_dd2d_copy(void *a)
{
    dd2d_type_ *o = (dd2d_type_ *)a;
    return dd2d_dispatcher_->wrapcopy(o);
}



int fpcvt_dd2d_nb(PyObject *o, void *a)
{
    dd2d_nb_type_ *object;
    if(o == Py_None)
    {
        *((dd2d_nb_type_ **)a) = 0;
        return 1;
    }
    object = dd2d_nb_dispatcher_->unwrap(o);
    *((dd2d_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_dd2d_nb_copy fpcvt_dd2d_nb

PyObject *tpcvt_dd2d_nb(void *a)
{
    dd2d_nb_type_ *o = *((dd2d_nb_type_ **)a);
    return dd2d_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_dd2d_nb_copy(void *a)
{
    dd2d_nb_type_ *o = (dd2d_nb_type_ *)a;
    return dd2d_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_change(PyObject *o, void *a)
{
    change_type_ *object;
    if(o == Py_None)
    {
        *((change_type_ **)a) = 0;
        return 1;
    }
    object = change_dispatcher_->unwrap(o);
    *((change_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_copy fpcvt_change

PyObject *tpcvt_change(void *a)
{
    change_type_ *o = *((change_type_ **)a);
    return change_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_copy(void *a)
{
    change_type_ *o = (change_type_ *)a;
    return change_dispatcher_->wrapcopy(o);
}



int fpcvt_change_nb(PyObject *o, void *a)
{
    change_nb_type_ *object;
    if(o == Py_None)
    {
        *((change_nb_type_ **)a) = 0;
        return 1;
    }
    object = change_nb_dispatcher_->unwrap(o);
    *((change_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_nb_copy fpcvt_change_nb

PyObject *tpcvt_change_nb(void *a)
{
    change_nb_type_ *o = *((change_nb_type_ **)a);
    return change_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_nb_copy(void *a)
{
    change_nb_type_ *o = (change_nb_type_ *)a;
    return change_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_function1(PyObject *o, void *a)
{
    function1_type_ *object;
    if(o == Py_None)
    {
        *((function1_type_ **)a) = 0;
        return 1;
    }
    object = function1_dispatcher_->unwrap(o);
    *((function1_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_function1_copy fpcvt_function1

PyObject *tpcvt_function1(void *a)
{
    function1_type_ *o = *((function1_type_ **)a);
    return function1_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_function1_copy(void *a)
{
    function1_type_ *o = (function1_type_ *)a;
    return function1_dispatcher_->wrapcopy(o);
}



int fpcvt_function2(PyObject *o, void *a)
{
    function2_type_ *object;
    if(o == Py_None)
    {
        *((function2_type_ **)a) = 0;
        return 1;
    }
    object = function2_dispatcher_->unwrap(o);
    *((function2_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_function2_copy fpcvt_function2

PyObject *tpcvt_function2(void *a)
{
    function2_type_ *o = *((function2_type_ **)a);
    return function2_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_function2_copy(void *a)
{
    function2_type_ *o = (function2_type_ *)a;
    return function2_dispatcher_->wrapcopy(o);
}



int fpcvt_window(PyObject *o, void *a)
{
    window_type_ *object;
    if(o == Py_None)
    {
        *((window_type_ **)a) = 0;
        return 1;
    }
    object = window_dispatcher_->unwrap(o);
    *((window_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_window_copy fpcvt_window

PyObject *tpcvt_window(void *a)
{
    window_type_ *o = *((window_type_ **)a);
    return window_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_window_copy(void *a)
{
    window_type_ *o = (window_type_ *)a;
    return window_dispatcher_->wrapcopy(o);
}



int fpcvt_thing(PyObject *o, void *a)
{
    thing_type_ *object;
    if(o == Py_None)
    {
        *((thing_type_ **)a) = 0;
        return 1;
    }
    object = thing_dispatcher_->unwrap(o);
    *((thing_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_thing_copy fpcvt_thing

PyObject *tpcvt_thing(void *a)
{
    thing_type_ *o = *((thing_type_ **)a);
    return thing_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_thing_copy(void *a)
{
    thing_type_ *o = (thing_type_ *)a;
    return thing_dispatcher_->wrapcopy(o);
}



int fpcvt_index(PyObject *o, void *a)
{
    index_type_ *object;
    if(o == Py_None)
    {
        *((index_type_ **)a) = 0;
        return 1;
    }
    object = index_dispatcher_->unwrap(o);
    *((index_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_index_copy fpcvt_index

PyObject *tpcvt_index(void *a)
{
    index_type_ *o = *((index_type_ **)a);
    return index_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_index_copy(void *a)
{
    index_type_ *o = (index_type_ *)a;
    return index_dispatcher_->wrapcopy(o);
}



int fpcvt_server(PyObject *o, void *a)
{
    server_type_ *object;
    if(o == Py_None)
    {
        *((server_type_ **)a) = 0;
        return 1;
    }
    object = server_dispatcher_->unwrap(o);
    *((server_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_server_copy fpcvt_server

PyObject *tpcvt_server(void *a)
{
    server_type_ *o = *((server_type_ **)a);
    return server_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_server_copy(void *a)
{
    server_type_ *o = (server_type_ *)a;
    return server_dispatcher_->wrapcopy(o);
}



int fpcvt_client(PyObject *o, void *a)
{
    client_type_ *object;
    if(o == Py_None)
    {
        *((client_type_ **)a) = 0;
        return 1;
    }
    object = client_dispatcher_->unwrap(o);
    *((client_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_client_copy fpcvt_client

PyObject *tpcvt_client(void *a)
{
    client_type_ *o = *((client_type_ **)a);
    return client_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_client_copy(void *a)
{
    client_type_ *o = (client_type_ *)a;
    return client_dispatcher_->wrapcopy(o);
}



int fpcvt_subcanchor(PyObject *o, void *a)
{
    subcanchor_type_ *object;
    if(o == Py_None)
    {
        *((subcanchor_type_ **)a) = 0;
        return 1;
    }
    object = subcanchor_dispatcher_->unwrap(o);
    *((subcanchor_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_subcanchor_copy fpcvt_subcanchor

PyObject *tpcvt_subcanchor(void *a)
{
    subcanchor_type_ *o = *((subcanchor_type_ **)a);
    return subcanchor_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_subcanchor_copy(void *a)
{
    subcanchor_type_ *o = (subcanchor_type_ *)a;
    return subcanchor_dispatcher_->wrapcopy(o);
}



int fpcvt_canchor(PyObject *o, void *a)
{
    canchor_type_ *object;
    if(o == Py_None)
    {
        *((canchor_type_ **)a) = 0;
        return 1;
    }
    object = canchor_dispatcher_->unwrap(o);
    *((canchor_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_canchor_copy fpcvt_canchor

PyObject *tpcvt_canchor(void *a)
{
    canchor_type_ *o = *((canchor_type_ **)a);
    return canchor_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_canchor_copy(void *a)
{
    canchor_type_ *o = (canchor_type_ *)a;
    return canchor_dispatcher_->wrapcopy(o);
}



int fpcvt_anchor(PyObject *o, void *a)
{
    anchor_type_ *object;
    if(o == Py_None)
    {
        *((anchor_type_ **)a) = 0;
        return 1;
    }
    object = anchor_dispatcher_->unwrap(o);
    *((anchor_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_anchor_copy fpcvt_anchor

PyObject *tpcvt_anchor(void *a)
{
    anchor_type_ *o = *((anchor_type_ **)a);
    return anchor_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_anchor_copy(void *a)
{
    anchor_type_ *o = (anchor_type_ *)a;
    return anchor_dispatcher_->wrapcopy(o);
}



int fpcvt_f_event_sender(PyObject *o, void *a)
{
    f_event_sender_type_ *object;
    if(o == Py_None)
    {
        *((f_event_sender_type_ **)a) = 0;
        return 1;
    }
    object = f_event_sender_dispatcher_->unwrap(o);
    *((f_event_sender_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_f_event_sender_copy fpcvt_f_event_sender

PyObject *tpcvt_f_event_sender(void *a)
{
    f_event_sender_type_ *o = *((f_event_sender_type_ **)a);
    return f_event_sender_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_f_event_sender_copy(void *a)
{
    f_event_sender_type_ *o = (f_event_sender_type_ *)a;
    return f_event_sender_dispatcher_->wrapcopy(o);
}



int fpcvt_f_event_receiver(PyObject *o, void *a)
{
    f_event_receiver_type_ *object;
    if(o == Py_None)
    {
        *((f_event_receiver_type_ **)a) = 0;
        return 1;
    }
    object = f_event_receiver_dispatcher_->unwrap(o);
    *((f_event_receiver_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_f_event_receiver_copy fpcvt_f_event_receiver

PyObject *tpcvt_f_event_receiver(void *a)
{
    f_event_receiver_type_ *o = *((f_event_receiver_type_ **)a);
    return f_event_receiver_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_f_event_receiver_copy(void *a)
{
    f_event_receiver_type_ *o = (f_event_receiver_type_ *)a;
    return f_event_receiver_dispatcher_->wrapcopy(o);
}



int fpcvt_dataqueue(PyObject *o, void *a)
{
    dataqueue_type_ *object;
    if(o == Py_None)
    {
        *((dataqueue_type_ **)a) = 0;
        return 1;
    }
    object = dataqueue_dispatcher_->unwrap(o);
    *((dataqueue_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_dataqueue_copy fpcvt_dataqueue

PyObject *tpcvt_dataqueue(void *a)
{
    dataqueue_type_ *o = *((dataqueue_type_ **)a);
    return dataqueue_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_dataqueue_copy(void *a)
{
    dataqueue_type_ *o = (dataqueue_type_ *)a;
    return dataqueue_dispatcher_->wrapcopy(o);
}



int fpcvt_fastdata(PyObject *o, void *a)
{
    fastdata_type_ *object;
    if(o == Py_None)
    {
        *((fastdata_type_ **)a) = 0;
        return 1;
    }
    object = fastdata_dispatcher_->unwrap(o);
    *((fastdata_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_fastdata_copy fpcvt_fastdata

PyObject *tpcvt_fastdata(void *a)
{
    fastdata_type_ *o = *((fastdata_type_ **)a);
    return fastdata_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_fastdata_copy(void *a)
{
    fastdata_type_ *o = (fastdata_type_ *)a;
    return fastdata_dispatcher_->wrapcopy(o);
}



int fpcvt_fastdata_receiver(PyObject *o, void *a)
{
    fastdata_receiver_type_ *object;
    if(o == Py_None)
    {
        *((fastdata_receiver_type_ **)a) = 0;
        return 1;
    }
    object = fastdata_receiver_dispatcher_->unwrap(o);
    *((fastdata_receiver_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_fastdata_receiver_copy fpcvt_fastdata_receiver

PyObject *tpcvt_fastdata_receiver(void *a)
{
    fastdata_receiver_type_ *o = *((fastdata_receiver_type_ **)a);
    return fastdata_receiver_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_fastdata_receiver_copy(void *a)
{
    fastdata_receiver_type_ *o = (fastdata_receiver_type_ *)a;
    return fastdata_receiver_dispatcher_->wrapcopy(o);
}



int fpcvt_term(PyObject *o, void *a)
{
    term_type_ *object;
    if(o == Py_None)
    {
        *((term_type_ **)a) = 0;
        return 1;
    }
    object = term_dispatcher_->unwrap(o);
    *((term_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_term_copy fpcvt_term

PyObject *tpcvt_term(void *a)
{
    term_type_ *o = *((term_type_ **)a);
    return term_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_term_copy(void *a)
{
    term_type_ *o = (term_type_ *)a;
    return term_dispatcher_->wrapcopy(o);
}



int fpcvt_data_base(PyObject *o, void *a)
{
    data_base_type_ *object;
    if(o == Py_None)
    {
        *((data_base_type_ **)a) = 0;
        return 1;
    }
    object = data_base_dispatcher_->unwrap(o);
    *((data_base_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_data_base_copy fpcvt_data_base

PyObject *tpcvt_data_base(void *a)
{
    data_base_type_ *o = *((data_base_type_ **)a);
    return data_base_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_data_base_copy(void *a)
{
    data_base_type_ *o = (data_base_type_ *)a;
    return data_base_dispatcher_->wrapcopy(o);
}



int fpcvt_data(PyObject *o, void *a)
{
    data_type_ *object;
    if(o == Py_None)
    {
        *((data_type_ **)a) = 0;
        return 1;
    }
    object = data_dispatcher_->unwrap(o);
    *((data_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_data_copy fpcvt_data

PyObject *tpcvt_data(void *a)
{
    data_type_ *o = *((data_type_ **)a);
    return data_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_data_copy(void *a)
{
    data_type_ *o = (data_type_ *)a;
    return data_dispatcher_->wrapcopy(o);
}



int fpcvt_data_nb(PyObject *o, void *a)
{
    data_nb_type_ *object;
    if(o == Py_None)
    {
        *((data_nb_type_ **)a) = 0;
        return 1;
    }
    object = data_nb_dispatcher_->unwrap(o);
    *((data_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_data_nb_copy fpcvt_data_nb

PyObject *tpcvt_data_nb(void *a)
{
    data_nb_type_ *o = *((data_nb_type_ **)a);
    return data_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_data_nb_copy(void *a)
{
    data_nb_type_ *o = (data_nb_type_ *)a;
    return data_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_clockdomain_ctl(PyObject *o, void *a)
{
    clockdomain_ctl_type_ *object;
    if(o == Py_None)
    {
        *((clockdomain_ctl_type_ **)a) = 0;
        return 1;
    }
    object = clockdomain_ctl_dispatcher_->unwrap(o);
    *((clockdomain_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockdomain_ctl_copy fpcvt_clockdomain_ctl

PyObject *tpcvt_clockdomain_ctl(void *a)
{
    clockdomain_ctl_type_ *o = *((clockdomain_ctl_type_ **)a);
    return clockdomain_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockdomain_ctl_copy(void *a)
{
    clockdomain_ctl_type_ *o = (clockdomain_ctl_type_ *)a;
    return clockdomain_ctl_dispatcher_->wrapcopy(o);
}



int fpcvt_clocksink(PyObject *o, void *a)
{
    clocksink_type_ *object;
    if(o == Py_None)
    {
        *((clocksink_type_ **)a) = 0;
        return 1;
    }
    object = clocksink_dispatcher_->unwrap(o);
    *((clocksink_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clocksink_copy fpcvt_clocksink

PyObject *tpcvt_clocksink(void *a)
{
    clocksink_type_ *o = *((clocksink_type_ **)a);
    return clocksink_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clocksink_copy(void *a)
{
    clocksink_type_ *o = (clocksink_type_ *)a;
    return clocksink_dispatcher_->wrapcopy(o);
}



int fpcvt_clocksource(PyObject *o, void *a)
{
    clocksource_type_ *object;
    if(o == Py_None)
    {
        *((clocksource_type_ **)a) = 0;
        return 1;
    }
    object = clocksource_dispatcher_->unwrap(o);
    *((clocksource_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clocksource_copy fpcvt_clocksource

PyObject *tpcvt_clocksource(void *a)
{
    clocksource_type_ *o = *((clocksource_type_ **)a);
    return clocksource_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clocksource_copy(void *a)
{
    clocksource_type_ *o = (clocksource_type_ *)a;
    return clocksource_dispatcher_->wrapcopy(o);
}



int fpcvt_fast_pitchbender(PyObject *o, void *a)
{
    fast_pitchbender_type_ *object;
    if(o == Py_None)
    {
        *((fast_pitchbender_type_ **)a) = 0;
        return 1;
    }
    object = fast_pitchbender_dispatcher_->unwrap(o);
    *((fast_pitchbender_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_fast_pitchbender_copy fpcvt_fast_pitchbender

PyObject *tpcvt_fast_pitchbender(void *a)
{
    fast_pitchbender_type_ *o = *((fast_pitchbender_type_ **)a);
    return fast_pitchbender_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_fast_pitchbender_copy(void *a)
{
    fast_pitchbender_type_ *o = (fast_pitchbender_type_ *)a;
    return fast_pitchbender_dispatcher_->wrapcopy(o);
}



int fpcvt_panner(PyObject *o, void *a)
{
    panner_type_ *object;
    if(o == Py_None)
    {
        *((panner_type_ **)a) = 0;
        return 1;
    }
    object = panner_dispatcher_->unwrap(o);
    *((panner_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_panner_copy fpcvt_panner

PyObject *tpcvt_panner(void *a)
{
    panner_type_ *o = *((panner_type_ **)a);
    return panner_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_panner_copy(void *a)
{
    panner_type_ *o = (panner_type_ *)a;
    return panner_dispatcher_->wrapcopy(o);
}



int fpcvt_gain(PyObject *o, void *a)
{
    gain_type_ *object;
    if(o == Py_None)
    {
        *((gain_type_ **)a) = 0;
        return 1;
    }
    object = gain_dispatcher_->unwrap(o);
    *((gain_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_gain_copy fpcvt_gain

PyObject *tpcvt_gain(void *a)
{
    gain_type_ *o = *((gain_type_ **)a);
    return gain_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_gain_copy(void *a)
{
    gain_type_ *o = (gain_type_ *)a;
    return gain_dispatcher_->wrapcopy(o);
}



int fpcvt_delay(PyObject *o, void *a)
{
    delay_type_ *object;
    if(o == Py_None)
    {
        *((delay_type_ **)a) = 0;
        return 1;
    }
    object = delay_dispatcher_->unwrap(o);
    *((delay_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_delay_copy fpcvt_delay

PyObject *tpcvt_delay(void *a)
{
    delay_type_ *o = *((delay_type_ **)a);
    return delay_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_delay_copy(void *a)
{
    delay_type_ *o = (delay_type_ *)a;
    return delay_dispatcher_->wrapcopy(o);
}



int fpcvt_linear_gain(PyObject *o, void *a)
{
    linear_gain_type_ *object;
    if(o == Py_None)
    {
        *((linear_gain_type_ **)a) = 0;
        return 1;
    }
    object = linear_gain_dispatcher_->unwrap(o);
    *((linear_gain_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_linear_gain_copy fpcvt_linear_gain

PyObject *tpcvt_linear_gain(void *a)
{
    linear_gain_type_ *o = *((linear_gain_type_ **)a);
    return linear_gain_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_linear_gain_copy(void *a)
{
    linear_gain_type_ *o = (linear_gain_type_ *)a;
    return linear_gain_dispatcher_->wrapcopy(o);
}



int fpcvt_selector(PyObject *o, void *a)
{
    selector_type_ *object;
    if(o == Py_None)
    {
        *((selector_type_ **)a) = 0;
        return 1;
    }
    object = selector_dispatcher_->unwrap(o);
    *((selector_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_selector_copy fpcvt_selector

PyObject *tpcvt_selector(void *a)
{
    selector_type_ *o = *((selector_type_ **)a);
    return selector_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_selector_copy(void *a)
{
    selector_type_ *o = (selector_type_ *)a;
    return selector_dispatcher_->wrapcopy(o);
}



int fpcvt_velocitydetect(PyObject *o, void *a)
{
    velocitydetect_type_ *object;
    if(o == Py_None)
    {
        *((velocitydetect_type_ **)a) = 0;
        return 1;
    }
    object = velocitydetect_dispatcher_->unwrap(o);
    *((velocitydetect_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_velocitydetect_copy fpcvt_velocitydetect

PyObject *tpcvt_velocitydetect(void *a)
{
    velocitydetect_type_ *o = *((velocitydetect_type_ **)a);
    return velocitydetect_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_velocitydetect_copy(void *a)
{
    velocitydetect_type_ *o = (velocitydetect_type_ *)a;
    return velocitydetect_dispatcher_->wrapcopy(o);
}



int fpcvt_scaler_controller(PyObject *o, void *a)
{
    scaler_controller_type_ *object;
    if(o == Py_None)
    {
        *((scaler_controller_type_ **)a) = 0;
        return 1;
    }
    object = scaler_controller_dispatcher_->unwrap(o);
    *((scaler_controller_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_scaler_controller_copy fpcvt_scaler_controller

PyObject *tpcvt_scaler_controller(void *a)
{
    scaler_controller_type_ *o = *((scaler_controller_type_ **)a);
    return scaler_controller_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_scaler_controller_copy(void *a)
{
    scaler_controller_type_ *o = (scaler_controller_type_ *)a;
    return scaler_controller_dispatcher_->wrapcopy(o);
}



int fpcvt_scaler(PyObject *o, void *a)
{
    scaler_type_ *object;
    if(o == Py_None)
    {
        *((scaler_type_ **)a) = 0;
        return 1;
    }
    object = scaler_dispatcher_->unwrap(o);
    *((scaler_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_scaler_copy fpcvt_scaler

PyObject *tpcvt_scaler(void *a)
{
    scaler_type_ *o = *((scaler_type_ **)a);
    return scaler_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_scaler_copy(void *a)
{
    scaler_type_ *o = (scaler_type_ *)a;
    return scaler_dispatcher_->wrapcopy(o);
}



int fpcvt_evtdump(PyObject *o, void *a)
{
    evtdump_type_ *object;
    if(o == Py_None)
    {
        *((evtdump_type_ **)a) = 0;
        return 1;
    }
    object = evtdump_dispatcher_->unwrap(o);
    *((evtdump_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_evtdump_copy fpcvt_evtdump

PyObject *tpcvt_evtdump(void *a)
{
    evtdump_type_ *o = *((evtdump_type_ **)a);
    return evtdump_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_evtdump_copy(void *a)
{
    evtdump_type_ *o = (evtdump_type_ *)a;
    return evtdump_dispatcher_->wrapcopy(o);
}



int fpcvt_wavrecorder(PyObject *o, void *a)
{
    wavrecorder_type_ *object;
    if(o == Py_None)
    {
        *((wavrecorder_type_ **)a) = 0;
        return 1;
    }
    object = wavrecorder_dispatcher_->unwrap(o);
    *((wavrecorder_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_wavrecorder_copy fpcvt_wavrecorder

PyObject *tpcvt_wavrecorder(void *a)
{
    wavrecorder_type_ *o = *((wavrecorder_type_ **)a);
    return wavrecorder_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_wavrecorder_copy(void *a)
{
    wavrecorder_type_ *o = (wavrecorder_type_ *)a;
    return wavrecorder_dispatcher_->wrapcopy(o);
}



int fpcvt_python_delegate(PyObject *o, void *a)
{
    python_delegate_type_ *object;
    if(o == Py_None)
    {
        *((python_delegate_type_ **)a) = 0;
        return 1;
    }
    object = python_delegate_dispatcher_->unwrap(o);
    *((python_delegate_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_python_delegate_copy fpcvt_python_delegate

PyObject *tpcvt_python_delegate(void *a)
{
    python_delegate_type_ *o = *((python_delegate_type_ **)a);
    return python_delegate_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_python_delegate_copy(void *a)
{
    python_delegate_type_ *o = (python_delegate_type_ *)a;
    return python_delegate_dispatcher_->wrapcopy(o);
}



int fpcvt_midi_channel_delegate(PyObject *o, void *a)
{
    midi_channel_delegate_type_ *object;
    if(o == Py_None)
    {
        *((midi_channel_delegate_type_ **)a) = 0;
        return 1;
    }
    object = midi_channel_delegate_dispatcher_->unwrap(o);
    *((midi_channel_delegate_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_midi_channel_delegate_copy fpcvt_midi_channel_delegate

PyObject *tpcvt_midi_channel_delegate(void *a)
{
    midi_channel_delegate_type_ *o = *((midi_channel_delegate_type_ **)a);
    return midi_channel_delegate_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_midi_channel_delegate_copy(void *a)
{
    midi_channel_delegate_type_ *o = (midi_channel_delegate_type_ *)a;
    return midi_channel_delegate_dispatcher_->wrapcopy(o);
}



int fpcvt_midi_from_belcanto(PyObject *o, void *a)
{
    midi_from_belcanto_type_ *object;
    if(o == Py_None)
    {
        *((midi_from_belcanto_type_ **)a) = 0;
        return 1;
    }
    object = midi_from_belcanto_dispatcher_->unwrap(o);
    *((midi_from_belcanto_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_midi_from_belcanto_copy fpcvt_midi_from_belcanto

PyObject *tpcvt_midi_from_belcanto(void *a)
{
    midi_from_belcanto_type_ *o = *((midi_from_belcanto_type_ **)a);
    return midi_from_belcanto_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_midi_from_belcanto_copy(void *a)
{
    midi_from_belcanto_type_ *o = (midi_from_belcanto_type_ *)a;
    return midi_from_belcanto_dispatcher_->wrapcopy(o);
}



int fpcvt_tsd_snapshot(PyObject *o, void *a)
{
    tsd_snapshot_type_ *object;
    if(o == Py_None)
    {
        *((tsd_snapshot_type_ **)a) = 0;
        return 1;
    }
    object = tsd_snapshot_dispatcher_->unwrap(o);
    *((tsd_snapshot_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_tsd_snapshot_copy fpcvt_tsd_snapshot

PyObject *tpcvt_tsd_snapshot(void *a)
{
    tsd_snapshot_type_ *o = *((tsd_snapshot_type_ **)a);
    return tsd_snapshot_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_tsd_snapshot_copy(void *a)
{
    tsd_snapshot_type_ *o = (tsd_snapshot_type_ *)a;
    return tsd_snapshot_dispatcher_->wrapcopy(o);
}



int fpcvt_notify(PyObject *o, void *a)
{
    notify_type_ *object;
    if(o == Py_None)
    {
        *((notify_type_ **)a) = 0;
        return 1;
    }
    object = notify_dispatcher_->unwrap(o);
    *((notify_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_notify_copy fpcvt_notify

PyObject *tpcvt_notify(void *a)
{
    notify_type_ *o = *((notify_type_ **)a);
    return notify_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_notify_copy(void *a)
{
    notify_type_ *o = (notify_type_ *)a;
    return notify_dispatcher_->wrapcopy(o);
}



int fpcvt_status(PyObject *o, void *a)
{
    status_type_ *object;
    if(o == Py_None)
    {
        *((status_type_ **)a) = 0;
        return 1;
    }
    object = status_dispatcher_->unwrap(o);
    *((status_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_status_copy fpcvt_status

PyObject *tpcvt_status(void *a)
{
    status_type_ *o = *((status_type_ **)a);
    return status_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_status_copy(void *a)
{
    status_type_ *o = (status_type_ *)a;
    return status_dispatcher_->wrapcopy(o);
}



int fpcvt_f_string(PyObject *o, void *a)
{
    f_string_type_ *object;
    if(o == Py_None)
    {
        *((f_string_type_ **)a) = 0;
        return 1;
    }
    object = f_string_dispatcher_->unwrap(o);
    *((f_string_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_f_string_copy fpcvt_f_string

PyObject *tpcvt_f_string(void *a)
{
    f_string_type_ *o = *((f_string_type_ **)a);
    return f_string_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_f_string_copy(void *a)
{
    f_string_type_ *o = (f_string_type_ *)a;
    return f_string_dispatcher_->wrapcopy(o);
}



int fpcvt_f2f(PyObject *o, void *a)
{
    f2f_type_ *object;
    if(o == Py_None)
    {
        *((f2f_type_ **)a) = 0;
        return 1;
    }
    object = f2f_dispatcher_->unwrap(o);
    *((f2f_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_f2f_copy fpcvt_f2f

PyObject *tpcvt_f2f(void *a)
{
    f2f_type_ *o = *((f2f_type_ **)a);
    return f2f_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_f2f_copy(void *a)
{
    f2f_type_ *o = (f2f_type_ *)a;
    return f2f_dispatcher_->wrapcopy(o);
}



 

struct event_object_ {
    PyObject_HEAD
    xxcontrolled_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct event_wrapper_: public event_type_ 
{

    static void __deleter__(event_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    event_wrapper_( PyObject *o, PyInterpreterState *i  ,scheduler_type_ * a0  ,bool a1  ,const change_type_ & a2  ): event_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~event_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(event_object_ *self, visitproc visit, void *arg)
    {
        event_wrapper_ *o = (event_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(event_object_ *self)
    {
        event_wrapper_ *o = (event_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void modulo_call_(bool owned ,unsigned int a0 ,unsigned int a1 ,float a2 )
    {
        if(owned)
        {
            
                
                
                    return event_type_::modulo( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                modulo
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *modulo_method_(event_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1;  float a2; 

        event_wrapper_ *o = (event_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null event c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1  ,fpcvt_f,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->modulo_call_ (self->owned ,a0,a1,a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in event::modulo");
        }

        return 0;
    }



    void event_enable_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return event_type_::event_enable();
                
            
            
        }

        return
            
                event_enable
            
            
                ();
    }

    static PyObject *event_enable_method_(event_object_ *self, PyObject *args)
    {
        

        event_wrapper_ *o = (event_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null event c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->event_enable_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in event::event_enable");
        }

        return 0;
    }



    void upper_bound_call_(bool owned ,unsigned int a0 ,float a1 )
    {
        if(owned)
        {
            
                
                
                    return event_type_::upper_bound( a0  ,a1 );
                
            
            
        }

        return
            
                upper_bound
            
            
                ( a0  ,a1 );
    }

    static PyObject *upper_bound_method_(event_object_ *self, PyObject *args)
    {
         unsigned int a0;  float a1; 

        event_wrapper_ *o = (event_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null event c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_f,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->upper_bound_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in event::upper_bound");
        }

        return 0;
    }



    void lower_bound_call_(bool owned ,unsigned int a0 ,float a1 )
    {
        if(owned)
        {
            
                
                
                    return event_type_::lower_bound( a0  ,a1 );
                
            
            
        }

        return
            
                lower_bound
            
            
                ( a0  ,a1 );
    }

    static PyObject *lower_bound_method_(event_object_ *self, PyObject *args)
    {
         unsigned int a0;  float a1; 

        event_wrapper_ *o = (event_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null event c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_f,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->lower_bound_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in event::lower_bound");
        }

        return 0;
    }



    void event_clear_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return event_type_::event_clear();
                
            
            
        }

        return
            
                event_clear
            
            
                ();
    }

    static PyObject *event_clear_method_(event_object_ *self, PyObject *args)
    {
        

        event_wrapper_ *o = (event_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null event c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->event_clear_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in event::event_clear");
        }

        return 0;
    }



    void zone_call_(bool owned ,unsigned int a0 ,unsigned int a1 ,float a2 ,float a3 )
    {
        if(owned)
        {
            
                
                
                    return event_type_::zone( a0  ,a1  ,a2  ,a3 );
                
            
            
        }

        return
            
                zone
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *zone_method_(event_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1;  float a2;  float a3; 

        event_wrapper_ *o = (event_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null event c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1  ,fpcvt_f,&a2  ,fpcvt_f,&a3 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->zone_call_ (self->owned ,a0,a1,a2,a3 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in event::zone");
        }

        return 0;
    }



    void event_disable_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return event_type_::event_disable();
                
            
            
        }

        return
            
                event_disable
            
            
                ();
    }

    static PyObject *event_disable_method_(event_object_ *self, PyObject *args)
    {
        

        event_wrapper_ *o = (event_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null event c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->event_disable_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in event::event_disable");
        }

        return 0;
    }





    static PyObject *_method_lock(event_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(event_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void event_delete_(event_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            xxcontrolled_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (event_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        event_wrapper_::__deleter__((event_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void event_dealloc_(event_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    event_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * event_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    event_object_ *self;

    if(!(self = (event_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int event_init_(event_object_ *self, PyObject *args, PyObject *kwds)
{
    event_delete_(self);

    

    {
         scheduler_type_ * a0;  bool a1;  change_type_ * a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_scheduler,&a0,fpcvt_b,&a1,fpcvt_change_copy,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new event_wrapper_((PyObject *)self,interp ,a0,a1,*a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing event");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"event invalid arguments");
    return -1;

}


static const char *event_modulo_docstring_ = "";

static const char *event_event_enable_docstring_ = "";

static const char *event_upper_bound_docstring_ = "";

static const char *event_lower_bound_docstring_ = "";

static const char *event_event_clear_docstring_ = "";

static const char *event_zone_docstring_ = "";

static const char *event_event_disable_docstring_ = "";




static PyMethodDef event_methods_[] = {
    
        { "modulo",(PyCFunction)event_wrapper_::modulo_method_,METH_VARARGS,(char *)event_modulo_docstring_ },
    
        { "event_enable",(PyCFunction)event_wrapper_::event_enable_method_,METH_VARARGS,(char *)event_event_enable_docstring_ },
    
        { "upper_bound",(PyCFunction)event_wrapper_::upper_bound_method_,METH_VARARGS,(char *)event_upper_bound_docstring_ },
    
        { "lower_bound",(PyCFunction)event_wrapper_::lower_bound_method_,METH_VARARGS,(char *)event_lower_bound_docstring_ },
    
        { "event_clear",(PyCFunction)event_wrapper_::event_clear_method_,METH_VARARGS,(char *)event_event_clear_docstring_ },
    
        { "zone",(PyCFunction)event_wrapper_::zone_method_,METH_VARARGS,(char *)event_zone_docstring_ },
    
        { "event_disable",(PyCFunction)event_wrapper_::event_disable_method_,METH_VARARGS,(char *)event_event_disable_docstring_ },
    
    
        { "py_lock",(PyCFunction)event_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)event_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *event_docstring_ = "";

PyTypeObject event_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.event",/*tp_name*/
    sizeof(event_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)event_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)event_docstring_,       /* tp_doc */
    (traverseproc)event_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)event_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    event_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)event_init_,  /* tp_init */
    0,                         /* tp_alloc */
    event_new_,             /* tp_new */
};

PyObject *event_wrapcopy_(event_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *event_wrap_(event_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = xxcontrolled_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    event_object_ *self = 
        
        PyObject_GC_New(event_object_, &event_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

event_type_ *event_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &event_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected event object");
        return 0;
    }

    event_type_ *object = static_cast<event_type_ *>(((event_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null event object in unwrap");
        return 0;
    }

    return object;
}

void event_insert_(const std::pair< event_type_ *, PyObject *> &p)
{


    xxcontrolled_dispatcher_->insert(p);




}

void event_erase_(event_type_ *p)
{


    xxcontrolled_dispatcher_->erase(p);




}

PyObject * event_find_(event_type_ *p)
{


    return xxcontrolled_dispatcher_->find(p);




}

static struct event_dispatch_ event_realdispatcher_ =
{
    &event_typeobject_,
    event_wrap_,
    event_wrapcopy_,
    event_unwrap_,
    event_insert_,
    event_erase_,
    event_find_
};

  

struct scheduler_object_ {
    PyObject_HEAD
    scheduler_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct scheduler_wrapper_: public scheduler_type_ 
{

    static void __deleter__(scheduler_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    scheduler_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned int a0  ): scheduler_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~scheduler_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scheduler_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(scheduler_object_ *self, PyObject *args)
    {
        

        scheduler_wrapper_ *o = (scheduler_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scheduler c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scheduler::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(scheduler_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(scheduler_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void scheduler_delete_(scheduler_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            scheduler_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (scheduler_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        scheduler_wrapper_::__deleter__((scheduler_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void scheduler_dealloc_(scheduler_object_* self)
{
    
    scheduler_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * scheduler_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    scheduler_object_ *self;

    if(!(self = (scheduler_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int scheduler_init_(scheduler_object_ *self, PyObject *args, PyObject *kwds)
{
    scheduler_delete_(self);

    

    {
         unsigned int a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new scheduler_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing scheduler");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            scheduler_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"scheduler invalid arguments");
    return -1;

}


static const char *scheduler_cookie_docstring_ = "";




static PyMethodDef scheduler_methods_[] = {
    
        { "cookie",(PyCFunction)scheduler_wrapper_::cookie_method_,METH_VARARGS,(char *)scheduler_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)scheduler_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)scheduler_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *scheduler_docstring_ = "";

PyTypeObject scheduler_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.scheduler",/*tp_name*/
    sizeof(scheduler_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)scheduler_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)scheduler_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    scheduler_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)scheduler_init_,  /* tp_init */
    0,                         /* tp_alloc */
    scheduler_new_,             /* tp_new */
};

PyObject *scheduler_wrapcopy_(scheduler_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *scheduler_wrap_(scheduler_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = scheduler_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    scheduler_object_ *self = 
        PyObject_NEW(scheduler_object_, &scheduler_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        scheduler_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

scheduler_type_ *scheduler_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &scheduler_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected scheduler object");
        return 0;
    }

    scheduler_type_ *object = static_cast<scheduler_type_ *>(((scheduler_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null scheduler object in unwrap");
        return 0;
    }

    return object;
}

void scheduler_insert_(const std::pair< scheduler_type_ *, PyObject *> &p)
{




    scheduler_wrapped_.insert(p);


}

void scheduler_erase_(scheduler_type_ *p)
{




    scheduler_wrapped_.erase(p);


}

PyObject * scheduler_find_(scheduler_type_ *p)
{




    PyObject *ret = 0;
    std::map< scheduler_type_ *, PyObject *>::iterator where = scheduler_wrapped_.find(p);
    if(where != scheduler_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct scheduler_dispatch_ scheduler_realdispatcher_ =
{
    &scheduler_typeobject_,
    scheduler_wrap_,
    scheduler_wrapcopy_,
    scheduler_unwrap_,
    scheduler_insert_,
    scheduler_erase_,
    scheduler_find_
};

  

struct rpctoken_object_ {
    PyObject_HEAD
    rpctoken_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct rpctoken_wrapper_: public rpctoken_type_ 
{

    static void __deleter__(rpctoken_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    rpctoken_wrapper_( PyObject *o, PyInterpreterState *i  ): rpctoken_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    rpctoken_wrapper_( PyObject *o, PyInterpreterState *i  ,const rpctoken_type_ & a0  ): rpctoken_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~rpctoken_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(rpctoken_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(rpctoken_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void rpctoken_delete_(rpctoken_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            rpctoken_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (rpctoken_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        rpctoken_wrapper_::__deleter__((rpctoken_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void rpctoken_dealloc_(rpctoken_object_* self)
{
    
    rpctoken_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * rpctoken_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    rpctoken_object_ *self;

    if(!(self = (rpctoken_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int rpctoken_init_(rpctoken_object_ *self, PyObject *args, PyObject *kwds)
{
    rpctoken_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new rpctoken_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing rpctoken");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            rpctoken_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         rpctoken_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_rpctoken_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new rpctoken_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing rpctoken");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            rpctoken_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"rpctoken invalid arguments");
    return -1;

}





static PyMethodDef rpctoken_methods_[] = {
    
    
        { "py_lock",(PyCFunction)rpctoken_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)rpctoken_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *rpctoken_docstring_ = "";

PyTypeObject rpctoken_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.rpctoken",/*tp_name*/
    sizeof(rpctoken_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)rpctoken_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)rpctoken_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    rpctoken_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)rpctoken_init_,  /* tp_init */
    0,                         /* tp_alloc */
    rpctoken_new_,             /* tp_new */
};

PyObject *rpctoken_wrapcopy_(rpctoken_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            rpctoken_object_ *obj = 
                PyObject_NEW(rpctoken_object_, rpctoken_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new rpctoken_type_(*o);
                }

                try
                {
                    rpctoken_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying rpctoken");
                return 0;
            }
        

    
}

PyObject *rpctoken_wrap_(rpctoken_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = rpctoken_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    rpctoken_object_ *self = 
        PyObject_NEW(rpctoken_object_, &rpctoken_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        rpctoken_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

rpctoken_type_ *rpctoken_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &rpctoken_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected rpctoken object");
        return 0;
    }

    rpctoken_type_ *object = static_cast<rpctoken_type_ *>(((rpctoken_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null rpctoken object in unwrap");
        return 0;
    }

    return object;
}

void rpctoken_insert_(const std::pair< rpctoken_type_ *, PyObject *> &p)
{




    rpctoken_wrapped_.insert(p);


}

void rpctoken_erase_(rpctoken_type_ *p)
{




    rpctoken_wrapped_.erase(p);


}

PyObject * rpctoken_find_(rpctoken_type_ *p)
{




    PyObject *ret = 0;
    std::map< rpctoken_type_ *, PyObject *>::iterator where = rpctoken_wrapped_.find(p);
    if(where != rpctoken_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct rpctoken_dispatch_ rpctoken_realdispatcher_ =
{
    &rpctoken_typeobject_,
    rpctoken_wrap_,
    rpctoken_wrapcopy_,
    rpctoken_unwrap_,
    rpctoken_insert_,
    rpctoken_erase_,
    rpctoken_find_
};

  

struct rpcserver_object_ {
    PyObject_HEAD
    rpcserver_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct rpcserver_wrapper_: public rpcserver_type_ 
{

    static void __deleter__(rpcserver_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
    }
*/



    rpcserver_wrapper_( PyObject *o, PyInterpreterState *i  ): rpcserver_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~rpcserver_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void rpcserver_invoke(  const rpctoken_type_ & a0  ,const data_type_ & a1  ,const data_type_ & a2  ,const data_type_ & a3  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"rpcserver_invoke");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&O&O&)" ,tpcvt_rpctoken_copy,&a0,tpcvt_data_copy,&a1,tpcvt_data_copy,&a2,tpcvt_data_copy,&a3);
        //printf("rpcserver_invoke before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("rpcserver_invoke after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void rpcserver_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"rpcserver_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("rpcserver_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("rpcserver_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    void completed_call_(bool owned ,const rpctoken_type_ & a0 ,bool a1 ,const data_type_ & a2 )
    {
        if(owned)
        {
            
                
                
                    return rpcserver_type_::completed( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                completed
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *completed_method_(rpcserver_object_ *self, PyObject *args)
    {
         rpctoken_type_ * a0;  bool a1;  data_type_ * a2; 

        rpcserver_wrapper_ *o = (rpcserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null rpcserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_rpctoken_copy,&a0  ,fpcvt_b,&a1  ,fpcvt_data_copy,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->completed_call_ (self->owned ,*a0,a1,*a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in rpcserver::completed");
        }

        return 0;
    }



    bool open_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return rpcserver_type_::open();
                
            
            
        }

        return
            
                open
            
            
                ();
    }

    static PyObject *open_method_(rpcserver_object_ *self, PyObject *args)
    {
        

        rpcserver_wrapper_ *o = (rpcserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null rpcserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in rpcserver::open");
        }

        return 0;
    }



    bool opening_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return rpcserver_type_::opening();
                
            
            
        }

        return
            
                opening
            
            
                ();
    }

    static PyObject *opening_method_(rpcserver_object_ *self, PyObject *args)
    {
        

        rpcserver_wrapper_ *o = (rpcserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null rpcserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->opening_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in rpcserver::opening");
        }

        return 0;
    }



    bool closing_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return rpcserver_type_::closing();
                
            
            
        }

        return
            
                closing
            
            
                ();
    }

    static PyObject *closing_method_(rpcserver_object_ *self, PyObject *args)
    {
        

        rpcserver_wrapper_ *o = (rpcserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null rpcserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->closing_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in rpcserver::closing");
        }

        return 0;
    }



    bool running_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return rpcserver_type_::running();
                
            
            
        }

        return
            
                running
            
            
                ();
    }

    static PyObject *running_method_(rpcserver_object_ *self, PyObject *args)
    {
        

        rpcserver_wrapper_ *o = (rpcserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null rpcserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->running_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in rpcserver::running");
        }

        return 0;
    }



    void close_rpcserver_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return rpcserver_type_::close_rpcserver();
                
            
            
        }

        return
            
                close_rpcserver
            
            
                ();
    }

    static PyObject *close_rpcserver_method_(rpcserver_object_ *self, PyObject *args)
    {
        

        rpcserver_wrapper_ *o = (rpcserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null rpcserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_rpcserver_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in rpcserver::close_rpcserver");
        }

        return 0;
    }



    void rpcserver_invoke_call_(bool owned ,const rpctoken_type_ & a0 ,const data_type_ & a1 ,const data_type_ & a2 ,const data_type_ & a3 )
    {
        if(owned)
        {
            
                
                
                    return rpcserver_type_::rpcserver_invoke( a0  ,a1  ,a2  ,a3 );
                
            
            
        }

        return
            
                rpcserver_invoke
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *rpcserver_invoke_method_(rpcserver_object_ *self, PyObject *args)
    {
         rpctoken_type_ * a0;  data_type_ * a1;  data_type_ * a2;  data_type_ * a3; 

        rpcserver_wrapper_ *o = (rpcserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null rpcserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_rpctoken_copy,&a0  ,fpcvt_data_copy,&a1  ,fpcvt_data_copy,&a2  ,fpcvt_data_copy,&a3 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->rpcserver_invoke_call_ (self->owned ,*a0,*a1,*a2,*a3 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in rpcserver::rpcserver_invoke");
        }

        return 0;
    }



    void rpcserver_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return rpcserver_type_::rpcserver_closed();
                
            
            
        }

        return
            
                rpcserver_closed
            
            
                ();
    }

    static PyObject *rpcserver_closed_method_(rpcserver_object_ *self, PyObject *args)
    {
        

        rpcserver_wrapper_ *o = (rpcserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null rpcserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->rpcserver_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in rpcserver::rpcserver_closed");
        }

        return 0;
    }





    static PyObject *_method_lock(rpcserver_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(rpcserver_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void rpcserver_delete_(rpcserver_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            rpcserver_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (rpcserver_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        rpcserver_wrapper_::__deleter__((rpcserver_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void rpcserver_dealloc_(rpcserver_object_* self)
{
    
    rpcserver_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * rpcserver_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    rpcserver_object_ *self;

    if(!(self = (rpcserver_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int rpcserver_init_(rpcserver_object_ *self, PyObject *args, PyObject *kwds)
{
    rpcserver_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new rpcserver_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing rpcserver");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            rpcserver_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"rpcserver invalid arguments");
    return -1;

}


static const char *rpcserver_completed_docstring_ = "";

static const char *rpcserver_open_docstring_ = "";

static const char *rpcserver_opening_docstring_ = "";

static const char *rpcserver_closing_docstring_ = "";

static const char *rpcserver_running_docstring_ = "";

static const char *rpcserver_close_rpcserver_docstring_ = "";

static const char *rpcserver_rpcserver_invoke_docstring_ = "";

static const char *rpcserver_rpcserver_closed_docstring_ = "";




static PyMethodDef rpcserver_methods_[] = {
    
        { "completed",(PyCFunction)rpcserver_wrapper_::completed_method_,METH_VARARGS,(char *)rpcserver_completed_docstring_ },
    
        { "open",(PyCFunction)rpcserver_wrapper_::open_method_,METH_VARARGS,(char *)rpcserver_open_docstring_ },
    
        { "opening",(PyCFunction)rpcserver_wrapper_::opening_method_,METH_VARARGS,(char *)rpcserver_opening_docstring_ },
    
        { "closing",(PyCFunction)rpcserver_wrapper_::closing_method_,METH_VARARGS,(char *)rpcserver_closing_docstring_ },
    
        { "running",(PyCFunction)rpcserver_wrapper_::running_method_,METH_VARARGS,(char *)rpcserver_running_docstring_ },
    
        { "close_rpcserver",(PyCFunction)rpcserver_wrapper_::close_rpcserver_method_,METH_VARARGS,(char *)rpcserver_close_rpcserver_docstring_ },
    
        { "rpcserver_invoke",(PyCFunction)rpcserver_wrapper_::rpcserver_invoke_method_,METH_VARARGS,(char *)rpcserver_rpcserver_invoke_docstring_ },
    
        { "rpcserver_closed",(PyCFunction)rpcserver_wrapper_::rpcserver_closed_method_,METH_VARARGS,(char *)rpcserver_rpcserver_closed_docstring_ },
    
    
        { "py_lock",(PyCFunction)rpcserver_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)rpcserver_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *rpcserver_docstring_ = "";

PyTypeObject rpcserver_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.rpcserver",/*tp_name*/
    sizeof(rpcserver_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)rpcserver_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)rpcserver_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    rpcserver_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)rpcserver_init_,  /* tp_init */
    0,                         /* tp_alloc */
    rpcserver_new_,             /* tp_new */
};

PyObject *rpcserver_wrapcopy_(rpcserver_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *rpcserver_wrap_(rpcserver_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = rpcserver_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    rpcserver_object_ *self = 
        PyObject_NEW(rpcserver_object_, &rpcserver_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        rpcserver_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

rpcserver_type_ *rpcserver_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &rpcserver_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected rpcserver object");
        return 0;
    }

    rpcserver_type_ *object = static_cast<rpcserver_type_ *>(((rpcserver_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null rpcserver object in unwrap");
        return 0;
    }

    return object;
}

void rpcserver_insert_(const std::pair< rpcserver_type_ *, PyObject *> &p)
{




    rpcserver_wrapped_.insert(p);


}

void rpcserver_erase_(rpcserver_type_ *p)
{




    rpcserver_wrapped_.erase(p);


}

PyObject * rpcserver_find_(rpcserver_type_ *p)
{




    PyObject *ret = 0;
    std::map< rpcserver_type_ *, PyObject *>::iterator where = rpcserver_wrapped_.find(p);
    if(where != rpcserver_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct rpcserver_dispatch_ rpcserver_realdispatcher_ =
{
    &rpcserver_typeobject_,
    rpcserver_wrap_,
    rpcserver_wrapcopy_,
    rpcserver_unwrap_,
    rpcserver_insert_,
    rpcserver_erase_,
    rpcserver_find_
};

  

struct rpcclient_object_ {
    PyObject_HEAD
    rpcclient_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct rpcclient_wrapper_: public rpcclient_type_ 
{

    static void __deleter__(rpcclient_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
    }
*/



    rpcclient_wrapper_( PyObject *o, PyInterpreterState *i  ): rpcclient_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~rpcclient_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void rpcclient_complete(  int a0  ,const data_type_ & a1  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"rpcclient_complete");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&)" ,tpcvt_i,&a0,tpcvt_data_copy,&a1);
        //printf("rpcclient_complete before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("rpcclient_complete after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    bool open_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return rpcclient_type_::open();
                
            
            
        }

        return
            
                open
            
            
                ();
    }

    static PyObject *open_method_(rpcclient_object_ *self, PyObject *args)
    {
        

        rpcclient_wrapper_ *o = (rpcclient_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null rpcclient c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in rpcclient::open");
        }

        return 0;
    }



    void close_rpcclient_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return rpcclient_type_::close_rpcclient();
                
            
            
        }

        return
            
                close_rpcclient
            
            
                ();
    }

    static PyObject *close_rpcclient_method_(rpcclient_object_ *self, PyObject *args)
    {
        

        rpcclient_wrapper_ *o = (rpcclient_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null rpcclient c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_rpcclient_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in rpcclient::close_rpcclient");
        }

        return 0;
    }



    void rpcclient_complete_call_(bool owned ,int a0 ,const data_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return rpcclient_type_::rpcclient_complete( a0  ,a1 );
                
            
            
        }

        return
            
                rpcclient_complete
            
            
                ( a0  ,a1 );
    }

    static PyObject *rpcclient_complete_method_(rpcclient_object_ *self, PyObject *args)
    {
         int a0;  data_type_ * a1; 

        rpcclient_wrapper_ *o = (rpcclient_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null rpcclient c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_i,&a0  ,fpcvt_data_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->rpcclient_complete_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in rpcclient::rpcclient_complete");
        }

        return 0;
    }





    static PyObject *_method_lock(rpcclient_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(rpcclient_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void rpcclient_delete_(rpcclient_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            rpcclient_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (rpcclient_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        rpcclient_wrapper_::__deleter__((rpcclient_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void rpcclient_dealloc_(rpcclient_object_* self)
{
    
    rpcclient_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * rpcclient_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    rpcclient_object_ *self;

    if(!(self = (rpcclient_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int rpcclient_init_(rpcclient_object_ *self, PyObject *args, PyObject *kwds)
{
    rpcclient_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new rpcclient_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing rpcclient");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            rpcclient_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"rpcclient invalid arguments");
    return -1;

}


static const char *rpcclient_open_docstring_ = "";

static const char *rpcclient_close_rpcclient_docstring_ = "";

static const char *rpcclient_rpcclient_complete_docstring_ = "";




static PyMethodDef rpcclient_methods_[] = {
    
        { "open",(PyCFunction)rpcclient_wrapper_::open_method_,METH_VARARGS,(char *)rpcclient_open_docstring_ },
    
        { "close_rpcclient",(PyCFunction)rpcclient_wrapper_::close_rpcclient_method_,METH_VARARGS,(char *)rpcclient_close_rpcclient_docstring_ },
    
        { "rpcclient_complete",(PyCFunction)rpcclient_wrapper_::rpcclient_complete_method_,METH_VARARGS,(char *)rpcclient_rpcclient_complete_docstring_ },
    
    
        { "py_lock",(PyCFunction)rpcclient_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)rpcclient_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *rpcclient_docstring_ = "";

PyTypeObject rpcclient_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.rpcclient",/*tp_name*/
    sizeof(rpcclient_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)rpcclient_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)rpcclient_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    rpcclient_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)rpcclient_init_,  /* tp_init */
    0,                         /* tp_alloc */
    rpcclient_new_,             /* tp_new */
};

PyObject *rpcclient_wrapcopy_(rpcclient_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *rpcclient_wrap_(rpcclient_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = rpcclient_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    rpcclient_object_ *self = 
        PyObject_NEW(rpcclient_object_, &rpcclient_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        rpcclient_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

rpcclient_type_ *rpcclient_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &rpcclient_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected rpcclient object");
        return 0;
    }

    rpcclient_type_ *object = static_cast<rpcclient_type_ *>(((rpcclient_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null rpcclient object in unwrap");
        return 0;
    }

    return object;
}

void rpcclient_insert_(const std::pair< rpcclient_type_ *, PyObject *> &p)
{




    rpcclient_wrapped_.insert(p);


}

void rpcclient_erase_(rpcclient_type_ *p)
{




    rpcclient_wrapped_.erase(p);


}

PyObject * rpcclient_find_(rpcclient_type_ *p)
{




    PyObject *ret = 0;
    std::map< rpcclient_type_ *, PyObject *>::iterator where = rpcclient_wrapped_.find(p);
    if(where != rpcclient_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct rpcclient_dispatch_ rpcclient_realdispatcher_ =
{
    &rpcclient_typeobject_,
    rpcclient_wrap_,
    rpcclient_wrapcopy_,
    rpcclient_unwrap_,
    rpcclient_insert_,
    rpcclient_erase_,
    rpcclient_find_
};

  

struct samplearray_object_ {
    PyObject_HEAD
    samplearray_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct samplearray_wrapper_: public samplearray_type_ 
{

    static void __deleter__(samplearray_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    samplearray_wrapper_( PyObject *o, PyInterpreterState *i  ,const samplearray_type_ & a0  ): samplearray_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~samplearray_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(samplearray_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(samplearray_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void samplearray_delete_(samplearray_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            samplearray_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (samplearray_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        samplearray_wrapper_::__deleter__((samplearray_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void samplearray_dealloc_(samplearray_object_* self)
{
    
    samplearray_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * samplearray_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    samplearray_object_ *self;

    if(!(self = (samplearray_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int samplearray_init_(samplearray_object_ *self, PyObject *args, PyObject *kwds)
{
    samplearray_delete_(self);

    

    {
         samplearray_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_samplearray_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new samplearray_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing samplearray");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            samplearray_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"samplearray invalid arguments");
    return -1;

}





static PyMethodDef samplearray_methods_[] = {
    
    
        { "py_lock",(PyCFunction)samplearray_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)samplearray_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *samplearray_docstring_ = "";

PyTypeObject samplearray_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.samplearray",/*tp_name*/
    sizeof(samplearray_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)samplearray_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)samplearray_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    samplearray_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)samplearray_init_,  /* tp_init */
    0,                         /* tp_alloc */
    samplearray_new_,             /* tp_new */
};

PyObject *samplearray_wrapcopy_(samplearray_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            samplearray_object_ *obj = 
                PyObject_NEW(samplearray_object_, samplearray_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new samplearray_type_(*o);
                }

                try
                {
                    samplearray_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying samplearray");
                return 0;
            }
        

    
}

PyObject *samplearray_wrap_(samplearray_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = samplearray_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    samplearray_object_ *self = 
        PyObject_NEW(samplearray_object_, &samplearray_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        samplearray_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

samplearray_type_ *samplearray_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &samplearray_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected samplearray object");
        return 0;
    }

    samplearray_type_ *object = static_cast<samplearray_type_ *>(((samplearray_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null samplearray object in unwrap");
        return 0;
    }

    return object;
}

void samplearray_insert_(const std::pair< samplearray_type_ *, PyObject *> &p)
{




    samplearray_wrapped_.insert(p);


}

void samplearray_erase_(samplearray_type_ *p)
{




    samplearray_wrapped_.erase(p);


}

PyObject * samplearray_find_(samplearray_type_ *p)
{




    PyObject *ret = 0;
    std::map< samplearray_type_ *, PyObject *>::iterator where = samplearray_wrapped_.find(p);
    if(where != samplearray_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct samplearray_dispatch_ samplearray_realdispatcher_ =
{
    &samplearray_typeobject_,
    samplearray_wrap_,
    samplearray_wrapcopy_,
    samplearray_unwrap_,
    samplearray_insert_,
    samplearray_erase_,
    samplearray_find_
};

  

struct sample_object_ {
    PyObject_HEAD
    sample_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct sample_wrapper_: public sample_type_ 
{

    static void __deleter__(sample_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    sample_wrapper_( PyObject *o, PyInterpreterState *i  ,const sample_type_ & a0  ): sample_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~sample_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(sample_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(sample_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void sample_delete_(sample_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            sample_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (sample_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        sample_wrapper_::__deleter__((sample_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void sample_dealloc_(sample_object_* self)
{
    
    sample_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * sample_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    sample_object_ *self;

    if(!(self = (sample_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int sample_init_(sample_object_ *self, PyObject *args, PyObject *kwds)
{
    sample_delete_(self);

    

    {
         sample_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_sample_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new sample_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing sample");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            sample_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"sample invalid arguments");
    return -1;

}





static PyMethodDef sample_methods_[] = {
    
    
        { "py_lock",(PyCFunction)sample_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)sample_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *sample_docstring_ = "";

PyTypeObject sample_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.sample",/*tp_name*/
    sizeof(sample_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)sample_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)sample_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    sample_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)sample_init_,  /* tp_init */
    0,                         /* tp_alloc */
    sample_new_,             /* tp_new */
};

PyObject *sample_wrapcopy_(sample_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            sample_object_ *obj = 
                PyObject_NEW(sample_object_, sample_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new sample_type_(*o);
                }

                try
                {
                    sample_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying sample");
                return 0;
            }
        

    
}

PyObject *sample_wrap_(sample_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = sample_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    sample_object_ *self = 
        PyObject_NEW(sample_object_, &sample_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        sample_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

sample_type_ *sample_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &sample_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected sample object");
        return 0;
    }

    sample_type_ *object = static_cast<sample_type_ *>(((sample_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null sample object in unwrap");
        return 0;
    }

    return object;
}

void sample_insert_(const std::pair< sample_type_ *, PyObject *> &p)
{




    sample_wrapped_.insert(p);


}

void sample_erase_(sample_type_ *p)
{




    sample_wrapped_.erase(p);


}

PyObject * sample_find_(sample_type_ *p)
{




    PyObject *ret = 0;
    std::map< sample_type_ *, PyObject *>::iterator where = sample_wrapped_.find(p);
    if(where != sample_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct sample_dispatch_ sample_realdispatcher_ =
{
    &sample_typeobject_,
    sample_wrap_,
    sample_wrapcopy_,
    sample_unwrap_,
    sample_insert_,
    sample_erase_,
    sample_find_
};

  

struct zone_object_ {
    PyObject_HEAD
    zone_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct zone_wrapper_: public zone_type_ 
{

    static void __deleter__(zone_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    zone_wrapper_( PyObject *o, PyInterpreterState *i  ,const zone_type_ & a0  ): zone_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~zone_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(zone_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(zone_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void zone_delete_(zone_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            zone_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (zone_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        zone_wrapper_::__deleter__((zone_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void zone_dealloc_(zone_object_* self)
{
    
    zone_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * zone_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    zone_object_ *self;

    if(!(self = (zone_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int zone_init_(zone_object_ *self, PyObject *args, PyObject *kwds)
{
    zone_delete_(self);

    

    {
         zone_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_zone_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new zone_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing zone");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            zone_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"zone invalid arguments");
    return -1;

}





static PyMethodDef zone_methods_[] = {
    
    
        { "py_lock",(PyCFunction)zone_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)zone_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *zone_docstring_ = "";

PyTypeObject zone_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.zone",/*tp_name*/
    sizeof(zone_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)zone_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)zone_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    zone_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)zone_init_,  /* tp_init */
    0,                         /* tp_alloc */
    zone_new_,             /* tp_new */
};

PyObject *zone_wrapcopy_(zone_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            zone_object_ *obj = 
                PyObject_NEW(zone_object_, zone_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new zone_type_(*o);
                }

                try
                {
                    zone_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying zone");
                return 0;
            }
        

    
}

PyObject *zone_wrap_(zone_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = zone_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    zone_object_ *self = 
        PyObject_NEW(zone_object_, &zone_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        zone_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

zone_type_ *zone_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &zone_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected zone object");
        return 0;
    }

    zone_type_ *object = static_cast<zone_type_ *>(((zone_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null zone object in unwrap");
        return 0;
    }

    return object;
}

void zone_insert_(const std::pair< zone_type_ *, PyObject *> &p)
{




    zone_wrapped_.insert(p);


}

void zone_erase_(zone_type_ *p)
{




    zone_wrapped_.erase(p);


}

PyObject * zone_find_(zone_type_ *p)
{




    PyObject *ret = 0;
    std::map< zone_type_ *, PyObject *>::iterator where = zone_wrapped_.find(p);
    if(where != zone_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct zone_dispatch_ zone_realdispatcher_ =
{
    &zone_typeobject_,
    zone_wrap_,
    zone_wrapcopy_,
    zone_unwrap_,
    zone_insert_,
    zone_erase_,
    zone_find_
};

  

struct preset_object_ {
    PyObject_HEAD
    preset_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct preset_wrapper_: public preset_type_ 
{

    static void __deleter__(preset_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    preset_wrapper_( PyObject *o, PyInterpreterState *i  ): preset_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    preset_wrapper_( PyObject *o, PyInterpreterState *i  ,const preset_type_ & a0  ): preset_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~preset_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void add_zone_call_(bool owned ,const zone_type_ & a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->add_zone( a0 );
            
        }

        return
            
            
                (operator->())->add_zone
            
                ( a0 );
    }

    static PyObject *add_zone_method_(preset_object_ *self, PyObject *args)
    {
         zone_type_ * a0; 

        preset_wrapper_ *o = (preset_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null preset c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_zone_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock (true);
                o->add_zone_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in preset::add_zone");
        }

        return 0;
    }





    static PyObject *_method_lock(preset_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(preset_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void preset_delete_(preset_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            preset_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (preset_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        preset_wrapper_::__deleter__((preset_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void preset_dealloc_(preset_object_* self)
{
    
    preset_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * preset_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    preset_object_ *self;

    if(!(self = (preset_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int preset_init_(preset_object_ *self, PyObject *args, PyObject *kwds)
{
    preset_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new preset_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing preset");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            preset_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         preset_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_preset_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new preset_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing preset");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            preset_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"preset invalid arguments");
    return -1;

}


static const char *preset_add_zone_docstring_ = "";




static PyMethodDef preset_methods_[] = {
    
        { "add_zone",(PyCFunction)preset_wrapper_::add_zone_method_,METH_VARARGS,(char *)preset_add_zone_docstring_ },
    
    
        { "py_lock",(PyCFunction)preset_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)preset_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *preset_docstring_ = "";

PyTypeObject preset_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.preset",/*tp_name*/
    sizeof(preset_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)preset_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)preset_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    preset_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)preset_init_,  /* tp_init */
    0,                         /* tp_alloc */
    preset_new_,             /* tp_new */
};

PyObject *preset_wrapcopy_(preset_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            preset_object_ *obj = 
                PyObject_NEW(preset_object_, preset_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new preset_type_(*o);
                }

                try
                {
                    preset_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying preset");
                return 0;
            }
        

    
}

PyObject *preset_wrap_(preset_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = preset_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    preset_object_ *self = 
        PyObject_NEW(preset_object_, &preset_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        preset_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

preset_type_ *preset_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &preset_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected preset object");
        return 0;
    }

    preset_type_ *object = static_cast<preset_type_ *>(((preset_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null preset object in unwrap");
        return 0;
    }

    return object;
}

void preset_insert_(const std::pair< preset_type_ *, PyObject *> &p)
{




    preset_wrapped_.insert(p);


}

void preset_erase_(preset_type_ *p)
{




    preset_wrapped_.erase(p);


}

PyObject * preset_find_(preset_type_ *p)
{




    PyObject *ret = 0;
    std::map< preset_type_ *, PyObject *>::iterator where = preset_wrapped_.find(p);
    if(where != preset_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct preset_dispatch_ preset_realdispatcher_ =
{
    &preset_typeobject_,
    preset_wrap_,
    preset_wrapcopy_,
    preset_unwrap_,
    preset_insert_,
    preset_erase_,
    preset_find_
};

  

struct clockref_object_ {
    PyObject_HEAD
    clockref_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct clockref_wrapper_: public clockref_type_ 
{

    static void __deleter__(clockref_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    void __safe_dtors()
    {
        
    }

    virtual ~clockref_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(clockref_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(clockref_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void clockref_delete_(clockref_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            clockref_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (clockref_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        clockref_wrapper_::__deleter__((clockref_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void clockref_dealloc_(clockref_object_* self)
{
    
    clockref_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * clockref_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    clockref_object_ *self;

    if(!(self = (clockref_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int clockref_init_(clockref_object_ *self, PyObject *args, PyObject *kwds)
{
    clockref_delete_(self);

    

    PyErr_SetString(PyExc_RuntimeError,"clockref invalid arguments");
    return -1;

}





static PyMethodDef clockref_methods_[] = {
    
    
        { "py_lock",(PyCFunction)clockref_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)clockref_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *clockref_docstring_ = "";

PyTypeObject clockref_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.clockref",/*tp_name*/
    sizeof(clockref_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)clockref_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)clockref_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    clockref_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)clockref_init_,  /* tp_init */
    0,                         /* tp_alloc */
    clockref_new_,             /* tp_new */
};

PyObject *clockref_wrapcopy_(clockref_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *clockref_wrap_(clockref_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = clockref_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    clockref_object_ *self = 
        PyObject_NEW(clockref_object_, &clockref_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        clockref_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

clockref_type_ *clockref_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &clockref_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected clockref object");
        return 0;
    }

    clockref_type_ *object = static_cast<clockref_type_ *>(((clockref_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null clockref object in unwrap");
        return 0;
    }

    return object;
}

void clockref_insert_(const std::pair< clockref_type_ *, PyObject *> &p)
{




    clockref_wrapped_.insert(p);


}

void clockref_erase_(clockref_type_ *p)
{




    clockref_wrapped_.erase(p);


}

PyObject * clockref_find_(clockref_type_ *p)
{




    PyObject *ret = 0;
    std::map< clockref_type_ *, PyObject *>::iterator where = clockref_wrapped_.find(p);
    if(where != clockref_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct clockref_dispatch_ clockref_realdispatcher_ =
{
    &clockref_typeobject_,
    clockref_wrap_,
    clockref_wrapcopy_,
    clockref_unwrap_,
    clockref_insert_,
    clockref_erase_,
    clockref_find_
};

  

struct strummer_object_ {
    PyObject_HEAD
    strummer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct strummer_wrapper_: public strummer_type_ 
{

    static void __deleter__(strummer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    strummer_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): strummer_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~strummer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ data_cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return strummer_type_::data_cookie();
                
            
            
        }

        return
            
                data_cookie
            
            
                ();
    }

    static PyObject *data_cookie_method_(strummer_object_ *self, PyObject *args)
    {
        

        strummer_wrapper_ *o = (strummer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null strummer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->data_cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in strummer::data_cookie");
        }

        return 0;
    }



    cookie_type_ ctl_cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return strummer_type_::ctl_cookie();
                
            
            
        }

        return
            
                ctl_cookie
            
            
                ();
    }

    static PyObject *ctl_cookie_method_(strummer_object_ *self, PyObject *args)
    {
        

        strummer_wrapper_ *o = (strummer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null strummer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->ctl_cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in strummer::ctl_cookie");
        }

        return 0;
    }



    void enable_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return strummer_type_::enable( a0 );
                
            
            
        }

        return
            
                enable
            
            
                ( a0 );
    }

    static PyObject *enable_method_(strummer_object_ *self, PyObject *args)
    {
         bool a0; 

        strummer_wrapper_ *o = (strummer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null strummer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enable_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in strummer::enable");
        }

        return 0;
    }



    void set_key_mix_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return strummer_type_::set_key_mix( a0 );
                
            
            
        }

        return
            
                set_key_mix
            
            
                ( a0 );
    }

    static PyObject *set_key_mix_method_(strummer_object_ *self, PyObject *args)
    {
         float a0; 

        strummer_wrapper_ *o = (strummer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null strummer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_key_mix_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in strummer::set_key_mix");
        }

        return 0;
    }



    void set_on_threshold_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return strummer_type_::set_on_threshold( a0 );
                
            
            
        }

        return
            
                set_on_threshold
            
            
                ( a0 );
    }

    static PyObject *set_on_threshold_method_(strummer_object_ *self, PyObject *args)
    {
         float a0; 

        strummer_wrapper_ *o = (strummer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null strummer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_on_threshold_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in strummer::set_on_threshold");
        }

        return 0;
    }



    void set_off_threshold_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return strummer_type_::set_off_threshold( a0 );
                
            
            
        }

        return
            
                set_off_threshold
            
            
                ( a0 );
    }

    static PyObject *set_off_threshold_method_(strummer_object_ *self, PyObject *args)
    {
         float a0; 

        strummer_wrapper_ *o = (strummer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null strummer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_off_threshold_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in strummer::set_off_threshold");
        }

        return 0;
    }



    void set_trigger_window_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return strummer_type_::set_trigger_window( a0 );
                
            
            
        }

        return
            
                set_trigger_window
            
            
                ( a0 );
    }

    static PyObject *set_trigger_window_method_(strummer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        strummer_wrapper_ *o = (strummer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null strummer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_trigger_window_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in strummer::set_trigger_window");
        }

        return 0;
    }





    static PyObject *_method_lock(strummer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(strummer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void strummer_delete_(strummer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            strummer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (strummer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        strummer_wrapper_::__deleter__((strummer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void strummer_dealloc_(strummer_object_* self)
{
    
    strummer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * strummer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    strummer_object_ *self;

    if(!(self = (strummer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int strummer_init_(strummer_object_ *self, PyObject *args, PyObject *kwds)
{
    strummer_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new strummer_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing strummer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            strummer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"strummer invalid arguments");
    return -1;

}


static const char *strummer_data_cookie_docstring_ = "";

static const char *strummer_ctl_cookie_docstring_ = "";

static const char *strummer_enable_docstring_ = "";

static const char *strummer_set_key_mix_docstring_ = "";

static const char *strummer_set_on_threshold_docstring_ = "";

static const char *strummer_set_off_threshold_docstring_ = "";

static const char *strummer_set_trigger_window_docstring_ = "";




static PyMethodDef strummer_methods_[] = {
    
        { "data_cookie",(PyCFunction)strummer_wrapper_::data_cookie_method_,METH_VARARGS,(char *)strummer_data_cookie_docstring_ },
    
        { "ctl_cookie",(PyCFunction)strummer_wrapper_::ctl_cookie_method_,METH_VARARGS,(char *)strummer_ctl_cookie_docstring_ },
    
        { "enable",(PyCFunction)strummer_wrapper_::enable_method_,METH_VARARGS,(char *)strummer_enable_docstring_ },
    
        { "set_key_mix",(PyCFunction)strummer_wrapper_::set_key_mix_method_,METH_VARARGS,(char *)strummer_set_key_mix_docstring_ },
    
        { "set_on_threshold",(PyCFunction)strummer_wrapper_::set_on_threshold_method_,METH_VARARGS,(char *)strummer_set_on_threshold_docstring_ },
    
        { "set_off_threshold",(PyCFunction)strummer_wrapper_::set_off_threshold_method_,METH_VARARGS,(char *)strummer_set_off_threshold_docstring_ },
    
        { "set_trigger_window",(PyCFunction)strummer_wrapper_::set_trigger_window_method_,METH_VARARGS,(char *)strummer_set_trigger_window_docstring_ },
    
    
        { "py_lock",(PyCFunction)strummer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)strummer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *strummer_docstring_ = "";

PyTypeObject strummer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.strummer",/*tp_name*/
    sizeof(strummer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)strummer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)strummer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    strummer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)strummer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    strummer_new_,             /* tp_new */
};

PyObject *strummer_wrapcopy_(strummer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *strummer_wrap_(strummer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = strummer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    strummer_object_ *self = 
        PyObject_NEW(strummer_object_, &strummer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        strummer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

strummer_type_ *strummer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &strummer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected strummer object");
        return 0;
    }

    strummer_type_ *object = static_cast<strummer_type_ *>(((strummer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null strummer object in unwrap");
        return 0;
    }

    return object;
}

void strummer_insert_(const std::pair< strummer_type_ *, PyObject *> &p)
{




    strummer_wrapped_.insert(p);


}

void strummer_erase_(strummer_type_ *p)
{




    strummer_wrapped_.erase(p);


}

PyObject * strummer_find_(strummer_type_ *p)
{




    PyObject *ret = 0;
    std::map< strummer_type_ *, PyObject *>::iterator where = strummer_wrapped_.find(p);
    if(where != strummer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct strummer_dispatch_ strummer_realdispatcher_ =
{
    &strummer_typeobject_,
    strummer_wrap_,
    strummer_wrapcopy_,
    strummer_unwrap_,
    strummer_insert_,
    strummer_erase_,
    strummer_find_
};

  

struct stringer_object_ {
    PyObject_HEAD
    stringer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct stringer_wrapper_: public stringer_type_ 
{

    static void __deleter__(stringer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    stringer_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ): stringer_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~stringer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ data_cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return stringer_type_::data_cookie();
                
            
            
        }

        return
            
                data_cookie
            
            
                ();
    }

    static PyObject *data_cookie_method_(stringer_object_ *self, PyObject *args)
    {
        

        stringer_wrapper_ *o = (stringer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null stringer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->data_cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in stringer::data_cookie");
        }

        return 0;
    }



    cookie_type_ ctl_cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return stringer_type_::ctl_cookie();
                
            
            
        }

        return
            
                ctl_cookie
            
            
                ();
    }

    static PyObject *ctl_cookie_method_(stringer_object_ *self, PyObject *args)
    {
        

        stringer_wrapper_ *o = (stringer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null stringer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->ctl_cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in stringer::ctl_cookie");
        }

        return 0;
    }



    void enable_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return stringer_type_::enable( a0 );
                
            
            
        }

        return
            
                enable
            
            
                ( a0 );
    }

    static PyObject *enable_method_(stringer_object_ *self, PyObject *args)
    {
         bool a0; 

        stringer_wrapper_ *o = (stringer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null stringer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enable_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in stringer::enable");
        }

        return 0;
    }





    static PyObject *_method_lock(stringer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(stringer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void stringer_delete_(stringer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            stringer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (stringer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        stringer_wrapper_::__deleter__((stringer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void stringer_dealloc_(stringer_object_* self)
{
    
    stringer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * stringer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    stringer_object_ *self;

    if(!(self = (stringer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int stringer_init_(stringer_object_ *self, PyObject *args, PyObject *kwds)
{
    stringer_delete_(self);

    

    {
         cookie_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_cookie_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new stringer_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing stringer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            stringer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"stringer invalid arguments");
    return -1;

}


static const char *stringer_data_cookie_docstring_ = "";

static const char *stringer_ctl_cookie_docstring_ = "";

static const char *stringer_enable_docstring_ = "";




static PyMethodDef stringer_methods_[] = {
    
        { "data_cookie",(PyCFunction)stringer_wrapper_::data_cookie_method_,METH_VARARGS,(char *)stringer_data_cookie_docstring_ },
    
        { "ctl_cookie",(PyCFunction)stringer_wrapper_::ctl_cookie_method_,METH_VARARGS,(char *)stringer_ctl_cookie_docstring_ },
    
        { "enable",(PyCFunction)stringer_wrapper_::enable_method_,METH_VARARGS,(char *)stringer_enable_docstring_ },
    
    
        { "py_lock",(PyCFunction)stringer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)stringer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *stringer_docstring_ = "";

PyTypeObject stringer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.stringer",/*tp_name*/
    sizeof(stringer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)stringer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)stringer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    stringer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)stringer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    stringer_new_,             /* tp_new */
};

PyObject *stringer_wrapcopy_(stringer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *stringer_wrap_(stringer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = stringer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    stringer_object_ *self = 
        PyObject_NEW(stringer_object_, &stringer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        stringer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

stringer_type_ *stringer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &stringer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected stringer object");
        return 0;
    }

    stringer_type_ *object = static_cast<stringer_type_ *>(((stringer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null stringer object in unwrap");
        return 0;
    }

    return object;
}

void stringer_insert_(const std::pair< stringer_type_ *, PyObject *> &p)
{




    stringer_wrapped_.insert(p);


}

void stringer_erase_(stringer_type_ *p)
{




    stringer_wrapped_.erase(p);


}

PyObject * stringer_find_(stringer_type_ *p)
{




    PyObject *ret = 0;
    std::map< stringer_type_ *, PyObject *>::iterator where = stringer_wrapped_.find(p);
    if(where != stringer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct stringer_dispatch_ stringer_realdispatcher_ =
{
    &stringer_typeobject_,
    stringer_wrap_,
    stringer_wrapcopy_,
    stringer_unwrap_,
    stringer_insert_,
    stringer_erase_,
    stringer_find_
};

  

struct cycler_object_ {
    PyObject_HEAD
    cycler_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct cycler_wrapper_: public cycler_type_ 
{

    static void __deleter__(cycler_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    cycler_wrapper_( PyObject *o, PyInterpreterState *i  ,clockdomain_ctl_type_ * a0  ,int a1  ,const cookie_type_ & a2  ,const cookie_type_ & a3  ,bool a4  ): cycler_type_(  a0 ,a1 ,a2 ,a3 ,a4 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~cycler_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void set_poly_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return cycler_type_::set_poly( a0 );
                
            
            
        }

        return
            
                set_poly
            
            
                ( a0 );
    }

    static PyObject *set_poly_method_(cycler_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        cycler_wrapper_ *o = (cycler_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null cycler c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_poly_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in cycler::set_poly");
        }

        return 0;
    }



    void set_cycle_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return cycler_type_::set_cycle( a0 );
                
            
            
        }

        return
            
                set_cycle
            
            
                ( a0 );
    }

    static PyObject *set_cycle_method_(cycler_object_ *self, PyObject *args)
    {
         bool a0; 

        cycler_wrapper_ *o = (cycler_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null cycler c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_cycle_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in cycler::set_cycle");
        }

        return 0;
    }



    void set_maxdamp_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return cycler_type_::set_maxdamp( a0 );
                
            
            
        }

        return
            
                set_maxdamp
            
            
                ( a0 );
    }

    static PyObject *set_maxdamp_method_(cycler_object_ *self, PyObject *args)
    {
         float a0; 

        cycler_wrapper_ *o = (cycler_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null cycler c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_maxdamp_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in cycler::set_maxdamp");
        }

        return 0;
    }



    void set_invert_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return cycler_type_::set_invert( a0 );
                
            
            
        }

        return
            
                set_invert
            
            
                ( a0 );
    }

    static PyObject *set_invert_method_(cycler_object_ *self, PyObject *args)
    {
         bool a0; 

        cycler_wrapper_ *o = (cycler_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null cycler c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_invert_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in cycler::set_invert");
        }

        return 0;
    }



    void set_curve_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return cycler_type_::set_curve( a0 );
                
            
            
        }

        return
            
                set_curve
            
            
                ( a0 );
    }

    static PyObject *set_curve_method_(cycler_object_ *self, PyObject *args)
    {
         float a0; 

        cycler_wrapper_ *o = (cycler_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null cycler c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_curve_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in cycler::set_curve");
        }

        return 0;
    }



    cookie_type_ main_cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return cycler_type_::main_cookie();
                
            
            
        }

        return
            
                main_cookie
            
            
                ();
    }

    static PyObject *main_cookie_method_(cycler_object_ *self, PyObject *args)
    {
        

        cycler_wrapper_ *o = (cycler_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null cycler c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->main_cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in cycler::main_cookie");
        }

        return 0;
    }



    cookie_type_ feedback_cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return cycler_type_::feedback_cookie();
                
            
            
        }

        return
            
                feedback_cookie
            
            
                ();
    }

    static PyObject *feedback_cookie_method_(cycler_object_ *self, PyObject *args)
    {
        

        cycler_wrapper_ *o = (cycler_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null cycler c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->feedback_cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in cycler::feedback_cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(cycler_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(cycler_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void cycler_delete_(cycler_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            cycler_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (cycler_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        cycler_wrapper_::__deleter__((cycler_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void cycler_dealloc_(cycler_object_* self)
{
    
    cycler_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * cycler_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    cycler_object_ *self;

    if(!(self = (cycler_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int cycler_init_(cycler_object_ *self, PyObject *args, PyObject *kwds)
{
    cycler_delete_(self);

    

    {
         clockdomain_ctl_type_ * a0;  int a1;  cookie_type_ * a2;  cookie_type_ * a3;  bool a4; 

        if(PyArg_ParseTuple(args,"O&O&O&O&O&" ,fpcvt_clockdomain_ctl,&a0,fpcvt_i,&a1,fpcvt_cookie_copy,&a2,fpcvt_cookie_copy,&a3,fpcvt_b,&a4))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new cycler_wrapper_((PyObject *)self,interp ,a0,a1,*a2,*a3,a4);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing cycler");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            cycler_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"cycler invalid arguments");
    return -1;

}


static const char *cycler_set_poly_docstring_ = "";

static const char *cycler_set_cycle_docstring_ = "";

static const char *cycler_set_maxdamp_docstring_ = "";

static const char *cycler_set_invert_docstring_ = "";

static const char *cycler_set_curve_docstring_ = "";

static const char *cycler_main_cookie_docstring_ = "";

static const char *cycler_feedback_cookie_docstring_ = "";




static PyMethodDef cycler_methods_[] = {
    
        { "set_poly",(PyCFunction)cycler_wrapper_::set_poly_method_,METH_VARARGS,(char *)cycler_set_poly_docstring_ },
    
        { "set_cycle",(PyCFunction)cycler_wrapper_::set_cycle_method_,METH_VARARGS,(char *)cycler_set_cycle_docstring_ },
    
        { "set_maxdamp",(PyCFunction)cycler_wrapper_::set_maxdamp_method_,METH_VARARGS,(char *)cycler_set_maxdamp_docstring_ },
    
        { "set_invert",(PyCFunction)cycler_wrapper_::set_invert_method_,METH_VARARGS,(char *)cycler_set_invert_docstring_ },
    
        { "set_curve",(PyCFunction)cycler_wrapper_::set_curve_method_,METH_VARARGS,(char *)cycler_set_curve_docstring_ },
    
        { "main_cookie",(PyCFunction)cycler_wrapper_::main_cookie_method_,METH_VARARGS,(char *)cycler_main_cookie_docstring_ },
    
        { "feedback_cookie",(PyCFunction)cycler_wrapper_::feedback_cookie_method_,METH_VARARGS,(char *)cycler_feedback_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)cycler_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)cycler_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *cycler_docstring_ = "";

PyTypeObject cycler_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.cycler",/*tp_name*/
    sizeof(cycler_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)cycler_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)cycler_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    cycler_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)cycler_init_,  /* tp_init */
    0,                         /* tp_alloc */
    cycler_new_,             /* tp_new */
};

PyObject *cycler_wrapcopy_(cycler_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *cycler_wrap_(cycler_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = cycler_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    cycler_object_ *self = 
        PyObject_NEW(cycler_object_, &cycler_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        cycler_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

cycler_type_ *cycler_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &cycler_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected cycler object");
        return 0;
    }

    cycler_type_ *object = static_cast<cycler_type_ *>(((cycler_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null cycler object in unwrap");
        return 0;
    }

    return object;
}

void cycler_insert_(const std::pair< cycler_type_ *, PyObject *> &p)
{




    cycler_wrapped_.insert(p);


}

void cycler_erase_(cycler_type_ *p)
{




    cycler_wrapped_.erase(p);


}

PyObject * cycler_find_(cycler_type_ *p)
{




    PyObject *ret = 0;
    std::map< cycler_type_ *, PyObject *>::iterator where = cycler_wrapped_.find(p);
    if(where != cycler_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct cycler_dispatch_ cycler_realdispatcher_ =
{
    &cycler_typeobject_,
    cycler_wrap_,
    cycler_wrapcopy_,
    cycler_unwrap_,
    cycler_insert_,
    cycler_erase_,
    cycler_find_
};

  

struct polyctl_object_ {
    PyObject_HEAD
    polyctl_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct polyctl_wrapper_: public polyctl_type_ 
{

    static void __deleter__(polyctl_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    polyctl_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned int a0  ,const cookie_type_ & a1  ,bool a2  ,unsigned int a3  ): polyctl_type_(  a0 ,a1 ,a2 ,a3 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~polyctl_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void set_poly_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return polyctl_type_::set_poly( a0 );
                
            
            
        }

        return
            
                set_poly
            
            
                ( a0 );
    }

    static PyObject *set_poly_method_(polyctl_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        polyctl_wrapper_ *o = (polyctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null polyctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_poly_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in polyctl::set_poly");
        }

        return 0;
    }



    void set_cycle_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return polyctl_type_::set_cycle( a0 );
                
            
            
        }

        return
            
                set_cycle
            
            
                ( a0 );
    }

    static PyObject *set_cycle_method_(polyctl_object_ *self, PyObject *args)
    {
         bool a0; 

        polyctl_wrapper_ *o = (polyctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null polyctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_cycle_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in polyctl::set_cycle");
        }

        return 0;
    }



    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return polyctl_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(polyctl_object_ *self, PyObject *args)
    {
        

        polyctl_wrapper_ *o = (polyctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null polyctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in polyctl::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(polyctl_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(polyctl_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void polyctl_delete_(polyctl_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            polyctl_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (polyctl_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        polyctl_wrapper_::__deleter__((polyctl_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void polyctl_dealloc_(polyctl_object_* self)
{
    
    polyctl_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * polyctl_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    polyctl_object_ *self;

    if(!(self = (polyctl_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int polyctl_init_(polyctl_object_ *self, PyObject *args, PyObject *kwds)
{
    polyctl_delete_(self);

    

    {
         unsigned int a0;  cookie_type_ * a1;  bool a2;  unsigned int a3; 

        if(PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_ui,&a0,fpcvt_cookie_copy,&a1,fpcvt_b,&a2,fpcvt_ui,&a3))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new polyctl_wrapper_((PyObject *)self,interp ,a0,*a1,a2,a3);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing polyctl");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            polyctl_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"polyctl invalid arguments");
    return -1;

}


static const char *polyctl_set_poly_docstring_ = "";

static const char *polyctl_set_cycle_docstring_ = "";

static const char *polyctl_cookie_docstring_ = "";




static PyMethodDef polyctl_methods_[] = {
    
        { "set_poly",(PyCFunction)polyctl_wrapper_::set_poly_method_,METH_VARARGS,(char *)polyctl_set_poly_docstring_ },
    
        { "set_cycle",(PyCFunction)polyctl_wrapper_::set_cycle_method_,METH_VARARGS,(char *)polyctl_set_cycle_docstring_ },
    
        { "cookie",(PyCFunction)polyctl_wrapper_::cookie_method_,METH_VARARGS,(char *)polyctl_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)polyctl_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)polyctl_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *polyctl_docstring_ = "";

PyTypeObject polyctl_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.polyctl",/*tp_name*/
    sizeof(polyctl_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)polyctl_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)polyctl_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    polyctl_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)polyctl_init_,  /* tp_init */
    0,                         /* tp_alloc */
    polyctl_new_,             /* tp_new */
};

PyObject *polyctl_wrapcopy_(polyctl_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *polyctl_wrap_(polyctl_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = polyctl_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    polyctl_object_ *self = 
        PyObject_NEW(polyctl_object_, &polyctl_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        polyctl_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

polyctl_type_ *polyctl_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &polyctl_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected polyctl object");
        return 0;
    }

    polyctl_type_ *object = static_cast<polyctl_type_ *>(((polyctl_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null polyctl object in unwrap");
        return 0;
    }

    return object;
}

void polyctl_insert_(const std::pair< polyctl_type_ *, PyObject *> &p)
{




    polyctl_wrapped_.insert(p);


}

void polyctl_erase_(polyctl_type_ *p)
{




    polyctl_wrapped_.erase(p);


}

PyObject * polyctl_find_(polyctl_type_ *p)
{




    PyObject *ret = 0;
    std::map< polyctl_type_ *, PyObject *>::iterator where = polyctl_wrapped_.find(p);
    if(where != polyctl_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct polyctl_dispatch_ polyctl_realdispatcher_ =
{
    &polyctl_typeobject_,
    polyctl_wrap_,
    polyctl_wrapcopy_,
    polyctl_unwrap_,
    polyctl_insert_,
    polyctl_erase_,
    polyctl_find_
};

  

struct stereomixer_object_ {
    PyObject_HEAD
    stereomixer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct stereomixer_wrapper_: public stereomixer_type_ 
{

    static void __deleter__(stereomixer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    stereomixer_wrapper_( PyObject *o, PyInterpreterState *i  ,const f2f_type_ & a0  ,const f2f_type_ & a1  ,clockdomain_ctl_type_ * a2  ,const cookie_type_ & a3  ): stereomixer_type_(  a0 ,a1 ,a2 ,a3 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~stereomixer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(stereomixer_object_ *self, visitproc visit, void *arg)
    {
        stereomixer_wrapper_ *o = (stereomixer_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(stereomixer_object_ *self)
    {
        stereomixer_wrapper_ *o = (stereomixer_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return stereomixer_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(stereomixer_object_ *self, PyObject *args)
    {
        

        stereomixer_wrapper_ *o = (stereomixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null stereomixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in stereomixer::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(stereomixer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(stereomixer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void stereomixer_delete_(stereomixer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            stereomixer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (stereomixer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        stereomixer_wrapper_::__deleter__((stereomixer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void stereomixer_dealloc_(stereomixer_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    stereomixer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * stereomixer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    stereomixer_object_ *self;

    if(!(self = (stereomixer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int stereomixer_init_(stereomixer_object_ *self, PyObject *args, PyObject *kwds)
{
    stereomixer_delete_(self);

    

    {
         f2f_type_ * a0;  f2f_type_ * a1;  clockdomain_ctl_type_ * a2;  cookie_type_ * a3; 

        if(PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_f2f_copy,&a0,fpcvt_f2f_copy,&a1,fpcvt_clockdomain_ctl,&a2,fpcvt_cookie_copy,&a3))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new stereomixer_wrapper_((PyObject *)self,interp ,*a0,*a1,a2,*a3);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing stereomixer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            stereomixer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"stereomixer invalid arguments");
    return -1;

}


static const char *stereomixer_cookie_docstring_ = "";




static PyMethodDef stereomixer_methods_[] = {
    
        { "cookie",(PyCFunction)stereomixer_wrapper_::cookie_method_,METH_VARARGS,(char *)stereomixer_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)stereomixer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)stereomixer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *stereomixer_docstring_ = "";

PyTypeObject stereomixer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.stereomixer",/*tp_name*/
    sizeof(stereomixer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)stereomixer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)stereomixer_docstring_,       /* tp_doc */
    (traverseproc)stereomixer_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)stereomixer_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    stereomixer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)stereomixer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    stereomixer_new_,             /* tp_new */
};

PyObject *stereomixer_wrapcopy_(stereomixer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *stereomixer_wrap_(stereomixer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = stereomixer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    stereomixer_object_ *self = 
        
        PyObject_GC_New(stereomixer_object_, &stereomixer_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        stereomixer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

stereomixer_type_ *stereomixer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &stereomixer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected stereomixer object");
        return 0;
    }

    stereomixer_type_ *object = static_cast<stereomixer_type_ *>(((stereomixer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null stereomixer object in unwrap");
        return 0;
    }

    return object;
}

void stereomixer_insert_(const std::pair< stereomixer_type_ *, PyObject *> &p)
{




    stereomixer_wrapped_.insert(p);


}

void stereomixer_erase_(stereomixer_type_ *p)
{




    stereomixer_wrapped_.erase(p);


}

PyObject * stereomixer_find_(stereomixer_type_ *p)
{




    PyObject *ret = 0;
    std::map< stereomixer_type_ *, PyObject *>::iterator where = stereomixer_wrapped_.find(p);
    if(where != stereomixer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct stereomixer_dispatch_ stereomixer_realdispatcher_ =
{
    &stereomixer_typeobject_,
    stereomixer_wrap_,
    stereomixer_wrapcopy_,
    stereomixer_unwrap_,
    stereomixer_insert_,
    stereomixer_erase_,
    stereomixer_find_
};

  

struct stereosummer_object_ {
    PyObject_HEAD
    stereosummer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct stereosummer_wrapper_: public stereosummer_type_ 
{

    static void __deleter__(stereosummer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    stereosummer_wrapper_( PyObject *o, PyInterpreterState *i  ,clockdomain_ctl_type_ * a0  ,const cookie_type_ & a1  ,unsigned int a2  ): stereosummer_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~stereosummer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return stereosummer_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(stereosummer_object_ *self, PyObject *args)
    {
        

        stereosummer_wrapper_ *o = (stereosummer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null stereosummer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in stereosummer::cookie");
        }

        return 0;
    }



    void set_channels_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return stereosummer_type_::set_channels( a0 );
                
            
            
        }

        return
            
                set_channels
            
            
                ( a0 );
    }

    static PyObject *set_channels_method_(stereosummer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        stereosummer_wrapper_ *o = (stereosummer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null stereosummer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_channels_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in stereosummer::set_channels");
        }

        return 0;
    }





    static PyObject *_method_lock(stereosummer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(stereosummer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void stereosummer_delete_(stereosummer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            stereosummer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (stereosummer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        stereosummer_wrapper_::__deleter__((stereosummer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void stereosummer_dealloc_(stereosummer_object_* self)
{
    
    stereosummer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * stereosummer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    stereosummer_object_ *self;

    if(!(self = (stereosummer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int stereosummer_init_(stereosummer_object_ *self, PyObject *args, PyObject *kwds)
{
    stereosummer_delete_(self);

    

    {
         clockdomain_ctl_type_ * a0;  cookie_type_ * a1;  unsigned int a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_clockdomain_ctl,&a0,fpcvt_cookie_copy,&a1,fpcvt_ui,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new stereosummer_wrapper_((PyObject *)self,interp ,a0,*a1,a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing stereosummer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            stereosummer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"stereosummer invalid arguments");
    return -1;

}


static const char *stereosummer_cookie_docstring_ = "";

static const char *stereosummer_set_channels_docstring_ = "";




static PyMethodDef stereosummer_methods_[] = {
    
        { "cookie",(PyCFunction)stereosummer_wrapper_::cookie_method_,METH_VARARGS,(char *)stereosummer_cookie_docstring_ },
    
        { "set_channels",(PyCFunction)stereosummer_wrapper_::set_channels_method_,METH_VARARGS,(char *)stereosummer_set_channels_docstring_ },
    
    
        { "py_lock",(PyCFunction)stereosummer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)stereosummer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *stereosummer_docstring_ = "";

PyTypeObject stereosummer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.stereosummer",/*tp_name*/
    sizeof(stereosummer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)stereosummer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)stereosummer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    stereosummer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)stereosummer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    stereosummer_new_,             /* tp_new */
};

PyObject *stereosummer_wrapcopy_(stereosummer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *stereosummer_wrap_(stereosummer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = stereosummer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    stereosummer_object_ *self = 
        PyObject_NEW(stereosummer_object_, &stereosummer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        stereosummer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

stereosummer_type_ *stereosummer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &stereosummer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected stereosummer object");
        return 0;
    }

    stereosummer_type_ *object = static_cast<stereosummer_type_ *>(((stereosummer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null stereosummer object in unwrap");
        return 0;
    }

    return object;
}

void stereosummer_insert_(const std::pair< stereosummer_type_ *, PyObject *> &p)
{




    stereosummer_wrapped_.insert(p);


}

void stereosummer_erase_(stereosummer_type_ *p)
{




    stereosummer_wrapped_.erase(p);


}

PyObject * stereosummer_find_(stereosummer_type_ *p)
{




    PyObject *ret = 0;
    std::map< stereosummer_type_ *, PyObject *>::iterator where = stereosummer_wrapped_.find(p);
    if(where != stereosummer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct stereosummer_dispatch_ stereosummer_realdispatcher_ =
{
    &stereosummer_typeobject_,
    stereosummer_wrap_,
    stereosummer_wrapcopy_,
    stereosummer_unwrap_,
    stereosummer_insert_,
    stereosummer_erase_,
    stereosummer_find_
};

  

struct monomixer_object_ {
    PyObject_HEAD
    monomixer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct monomixer_wrapper_: public monomixer_type_ 
{

    static void __deleter__(monomixer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    monomixer_wrapper_( PyObject *o, PyInterpreterState *i  ,clockdomain_ctl_type_ * a0  ,const cookie_type_ & a1  ): monomixer_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~monomixer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return monomixer_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(monomixer_object_ *self, PyObject *args)
    {
        

        monomixer_wrapper_ *o = (monomixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null monomixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in monomixer::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(monomixer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(monomixer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void monomixer_delete_(monomixer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            monomixer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (monomixer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        monomixer_wrapper_::__deleter__((monomixer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void monomixer_dealloc_(monomixer_object_* self)
{
    
    monomixer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * monomixer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    monomixer_object_ *self;

    if(!(self = (monomixer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int monomixer_init_(monomixer_object_ *self, PyObject *args, PyObject *kwds)
{
    monomixer_delete_(self);

    

    {
         clockdomain_ctl_type_ * a0;  cookie_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_clockdomain_ctl,&a0,fpcvt_cookie_copy,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new monomixer_wrapper_((PyObject *)self,interp ,a0,*a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing monomixer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            monomixer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"monomixer invalid arguments");
    return -1;

}


static const char *monomixer_cookie_docstring_ = "";




static PyMethodDef monomixer_methods_[] = {
    
        { "cookie",(PyCFunction)monomixer_wrapper_::cookie_method_,METH_VARARGS,(char *)monomixer_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)monomixer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)monomixer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *monomixer_docstring_ = "";

PyTypeObject monomixer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.monomixer",/*tp_name*/
    sizeof(monomixer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)monomixer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)monomixer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    monomixer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)monomixer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    monomixer_new_,             /* tp_new */
};

PyObject *monomixer_wrapcopy_(monomixer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *monomixer_wrap_(monomixer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = monomixer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    monomixer_object_ *self = 
        PyObject_NEW(monomixer_object_, &monomixer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        monomixer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

monomixer_type_ *monomixer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &monomixer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected monomixer object");
        return 0;
    }

    monomixer_type_ *object = static_cast<monomixer_type_ *>(((monomixer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null monomixer object in unwrap");
        return 0;
    }

    return object;
}

void monomixer_insert_(const std::pair< monomixer_type_ *, PyObject *> &p)
{




    monomixer_wrapped_.insert(p);


}

void monomixer_erase_(monomixer_type_ *p)
{




    monomixer_wrapped_.erase(p);


}

PyObject * monomixer_find_(monomixer_type_ *p)
{




    PyObject *ret = 0;
    std::map< monomixer_type_ *, PyObject *>::iterator where = monomixer_wrapped_.find(p);
    if(where != monomixer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct monomixer_dispatch_ monomixer_realdispatcher_ =
{
    &monomixer_typeobject_,
    monomixer_wrap_,
    monomixer_wrapcopy_,
    monomixer_unwrap_,
    monomixer_insert_,
    monomixer_erase_,
    monomixer_find_
};

  

struct consolemixer_object_ {
    PyObject_HEAD
    consolemixer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct consolemixer_wrapper_: public consolemixer_type_ 
{

    static void __deleter__(consolemixer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    consolemixer_wrapper_( PyObject *o, PyInterpreterState *i  ,const f2f_type_ & a0  ,const f2f_type_ & a1  ,clockdomain_ctl_type_ * a2  ,const cookie_type_ & a3  ): consolemixer_type_(  a0 ,a1 ,a2 ,a3 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~consolemixer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ create_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return consolemixer_type_::create_channel( a0 );
                
            
            
        }

        return
            
                create_channel
            
            
                ( a0 );
    }

    static PyObject *create_channel_method_(consolemixer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        consolemixer_wrapper_ *o = (consolemixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null consolemixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->create_channel_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in consolemixer::create_channel");
        }

        return 0;
    }



    void remove_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return consolemixer_type_::remove_channel( a0 );
                
            
            
        }

        return
            
                remove_channel
            
            
                ( a0 );
    }

    static PyObject *remove_channel_method_(consolemixer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        consolemixer_wrapper_ *o = (consolemixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null consolemixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->remove_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in consolemixer::remove_channel");
        }

        return 0;
    }



    cookie_type_ create_fx_channel_call_(bool owned ,unsigned int a0 ,const cookie_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return consolemixer_type_::create_fx_channel( a0  ,a1 );
                
            
            
        }

        return
            
                create_fx_channel
            
            
                ( a0  ,a1 );
    }

    static PyObject *create_fx_channel_method_(consolemixer_object_ *self, PyObject *args)
    {
         unsigned int a0;  cookie_type_ * a1; 

        consolemixer_wrapper_ *o = (consolemixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null consolemixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_cookie_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->create_fx_channel_call_ (self->owned ,a0 ,*a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in consolemixer::create_fx_channel");
        }

        return 0;
    }



    void remove_fx_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return consolemixer_type_::remove_fx_channel( a0 );
                
            
            
        }

        return
            
                remove_fx_channel
            
            
                ( a0 );
    }

    static PyObject *remove_fx_channel_method_(consolemixer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        consolemixer_wrapper_ *o = (consolemixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null consolemixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->remove_fx_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in consolemixer::remove_fx_channel");
        }

        return 0;
    }



    void set_fx_send_enable_call_(bool owned ,bool a0 ,unsigned int a1 ,unsigned int a2 ,bool a3 )
    {
        if(owned)
        {
            
                
                
                    return consolemixer_type_::set_fx_send_enable( a0  ,a1  ,a2  ,a3 );
                
            
            
        }

        return
            
                set_fx_send_enable
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *set_fx_send_enable_method_(consolemixer_object_ *self, PyObject *args)
    {
         bool a0;  unsigned int a1;  unsigned int a2;  bool a3; 

        consolemixer_wrapper_ *o = (consolemixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null consolemixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_b,&a0  ,fpcvt_ui,&a1  ,fpcvt_ui,&a2  ,fpcvt_b,&a3 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_fx_send_enable_call_ (self->owned ,a0,a1,a2,a3 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in consolemixer::set_fx_send_enable");
        }

        return 0;
    }



    void set_fx_send_prefader_call_(bool owned ,bool a0 ,unsigned int a1 ,unsigned int a2 ,bool a3 )
    {
        if(owned)
        {
            
                
                
                    return consolemixer_type_::set_fx_send_prefader( a0  ,a1  ,a2  ,a3 );
                
            
            
        }

        return
            
                set_fx_send_prefader
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *set_fx_send_prefader_method_(consolemixer_object_ *self, PyObject *args)
    {
         bool a0;  unsigned int a1;  unsigned int a2;  bool a3; 

        consolemixer_wrapper_ *o = (consolemixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null consolemixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_b,&a0  ,fpcvt_ui,&a1  ,fpcvt_ui,&a2  ,fpcvt_b,&a3 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_fx_send_prefader_call_ (self->owned ,a0,a1,a2,a3 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in consolemixer::set_fx_send_prefader");
        }

        return 0;
    }



    cookie_type_ master_controls_cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return consolemixer_type_::master_controls_cookie();
                
            
            
        }

        return
            
                master_controls_cookie
            
            
                ();
    }

    static PyObject *master_controls_cookie_method_(consolemixer_object_ *self, PyObject *args)
    {
        

        consolemixer_wrapper_ *o = (consolemixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null consolemixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->master_controls_cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in consolemixer::master_controls_cookie");
        }

        return 0;
    }



    void set_curves_call_(bool owned ,const f2f_type_ & a0 ,const f2f_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return consolemixer_type_::set_curves( a0  ,a1 );
                
            
            
        }

        return
            
                set_curves
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_curves_method_(consolemixer_object_ *self, PyObject *args)
    {
         f2f_type_ * a0;  f2f_type_ * a1; 

        consolemixer_wrapper_ *o = (consolemixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null consolemixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_f2f_copy,&a0  ,fpcvt_f2f_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_curves_call_ (self->owned ,*a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in consolemixer::set_curves");
        }

        return 0;
    }





    static PyObject *_method_lock(consolemixer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(consolemixer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void consolemixer_delete_(consolemixer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            consolemixer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (consolemixer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        consolemixer_wrapper_::__deleter__((consolemixer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void consolemixer_dealloc_(consolemixer_object_* self)
{
    
    consolemixer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * consolemixer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    consolemixer_object_ *self;

    if(!(self = (consolemixer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int consolemixer_init_(consolemixer_object_ *self, PyObject *args, PyObject *kwds)
{
    consolemixer_delete_(self);

    

    {
         f2f_type_ * a0;  f2f_type_ * a1;  clockdomain_ctl_type_ * a2;  cookie_type_ * a3; 

        if(PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_f2f_copy,&a0,fpcvt_f2f_copy,&a1,fpcvt_clockdomain_ctl,&a2,fpcvt_cookie_copy,&a3))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new consolemixer_wrapper_((PyObject *)self,interp ,*a0,*a1,a2,*a3);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing consolemixer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            consolemixer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"consolemixer invalid arguments");
    return -1;

}


static const char *consolemixer_create_channel_docstring_ = "";

static const char *consolemixer_remove_channel_docstring_ = "";

static const char *consolemixer_create_fx_channel_docstring_ = "";

static const char *consolemixer_remove_fx_channel_docstring_ = "";

static const char *consolemixer_set_fx_send_enable_docstring_ = "";

static const char *consolemixer_set_fx_send_prefader_docstring_ = "";

static const char *consolemixer_master_controls_cookie_docstring_ = "";

static const char *consolemixer_set_curves_docstring_ = "";




static PyMethodDef consolemixer_methods_[] = {
    
        { "create_channel",(PyCFunction)consolemixer_wrapper_::create_channel_method_,METH_VARARGS,(char *)consolemixer_create_channel_docstring_ },
    
        { "remove_channel",(PyCFunction)consolemixer_wrapper_::remove_channel_method_,METH_VARARGS,(char *)consolemixer_remove_channel_docstring_ },
    
        { "create_fx_channel",(PyCFunction)consolemixer_wrapper_::create_fx_channel_method_,METH_VARARGS,(char *)consolemixer_create_fx_channel_docstring_ },
    
        { "remove_fx_channel",(PyCFunction)consolemixer_wrapper_::remove_fx_channel_method_,METH_VARARGS,(char *)consolemixer_remove_fx_channel_docstring_ },
    
        { "set_fx_send_enable",(PyCFunction)consolemixer_wrapper_::set_fx_send_enable_method_,METH_VARARGS,(char *)consolemixer_set_fx_send_enable_docstring_ },
    
        { "set_fx_send_prefader",(PyCFunction)consolemixer_wrapper_::set_fx_send_prefader_method_,METH_VARARGS,(char *)consolemixer_set_fx_send_prefader_docstring_ },
    
        { "master_controls_cookie",(PyCFunction)consolemixer_wrapper_::master_controls_cookie_method_,METH_VARARGS,(char *)consolemixer_master_controls_cookie_docstring_ },
    
        { "set_curves",(PyCFunction)consolemixer_wrapper_::set_curves_method_,METH_VARARGS,(char *)consolemixer_set_curves_docstring_ },
    
    
        { "py_lock",(PyCFunction)consolemixer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)consolemixer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *consolemixer_docstring_ = "";

PyTypeObject consolemixer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.consolemixer",/*tp_name*/
    sizeof(consolemixer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)consolemixer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)consolemixer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    consolemixer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)consolemixer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    consolemixer_new_,             /* tp_new */
};

PyObject *consolemixer_wrapcopy_(consolemixer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *consolemixer_wrap_(consolemixer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = consolemixer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    consolemixer_object_ *self = 
        PyObject_NEW(consolemixer_object_, &consolemixer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        consolemixer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

consolemixer_type_ *consolemixer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &consolemixer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected consolemixer object");
        return 0;
    }

    consolemixer_type_ *object = static_cast<consolemixer_type_ *>(((consolemixer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null consolemixer object in unwrap");
        return 0;
    }

    return object;
}

void consolemixer_insert_(const std::pair< consolemixer_type_ *, PyObject *> &p)
{




    consolemixer_wrapped_.insert(p);


}

void consolemixer_erase_(consolemixer_type_ *p)
{




    consolemixer_wrapped_.erase(p);


}

PyObject * consolemixer_find_(consolemixer_type_ *p)
{




    PyObject *ret = 0;
    std::map< consolemixer_type_ *, PyObject *>::iterator where = consolemixer_wrapped_.find(p);
    if(where != consolemixer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct consolemixer_dispatch_ consolemixer_realdispatcher_ =
{
    &consolemixer_typeobject_,
    consolemixer_wrap_,
    consolemixer_wrapcopy_,
    consolemixer_unwrap_,
    consolemixer_insert_,
    consolemixer_erase_,
    consolemixer_find_
};

  

struct gate_object_ {
    PyObject_HEAD
    gate_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct gate_wrapper_: public gate_type_ 
{

    static void __deleter__(gate_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    gate_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,bool a1  ): gate_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~gate_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return gate_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(gate_object_ *self, PyObject *args)
    {
        

        gate_wrapper_ *o = (gate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null gate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in gate::cookie");
        }

        return 0;
    }



    change_nb_type_ gate_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return gate_type_::gate();
                
            
            
        }

        return
            
                gate
            
            
                ();
    }

    static PyObject *gate_method_(gate_object_ *self, PyObject *args)
    {
        

        gate_wrapper_ *o = (gate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null gate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->gate_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in gate::gate");
        }

        return 0;
    }



    void enable_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return gate_type_::enable( a0 );
                
            
            
        }

        return
            
                enable
            
            
                ( a0 );
    }

    static PyObject *enable_method_(gate_object_ *self, PyObject *args)
    {
         bool a0; 

        gate_wrapper_ *o = (gate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null gate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enable_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in gate::enable");
        }

        return 0;
    }





    static PyObject *_method_lock(gate_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(gate_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void gate_delete_(gate_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            gate_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (gate_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        gate_wrapper_::__deleter__((gate_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void gate_dealloc_(gate_object_* self)
{
    
    gate_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * gate_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    gate_object_ *self;

    if(!(self = (gate_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int gate_init_(gate_object_ *self, PyObject *args, PyObject *kwds)
{
    gate_delete_(self);

    

    {
         cookie_type_ * a0;  bool a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_b,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new gate_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing gate");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            gate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"gate invalid arguments");
    return -1;

}


static const char *gate_cookie_docstring_ = "";

static const char *gate_gate_docstring_ = "";

static const char *gate_enable_docstring_ = "";




static PyMethodDef gate_methods_[] = {
    
        { "cookie",(PyCFunction)gate_wrapper_::cookie_method_,METH_VARARGS,(char *)gate_cookie_docstring_ },
    
        { "gate",(PyCFunction)gate_wrapper_::gate_method_,METH_VARARGS,(char *)gate_gate_docstring_ },
    
        { "enable",(PyCFunction)gate_wrapper_::enable_method_,METH_VARARGS,(char *)gate_enable_docstring_ },
    
    
        { "py_lock",(PyCFunction)gate_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)gate_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *gate_docstring_ = "";

PyTypeObject gate_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.gate",/*tp_name*/
    sizeof(gate_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)gate_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)gate_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    gate_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)gate_init_,  /* tp_init */
    0,                         /* tp_alloc */
    gate_new_,             /* tp_new */
};

PyObject *gate_wrapcopy_(gate_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *gate_wrap_(gate_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = gate_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    gate_object_ *self = 
        PyObject_NEW(gate_object_, &gate_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        gate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

gate_type_ *gate_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &gate_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected gate object");
        return 0;
    }

    gate_type_ *object = static_cast<gate_type_ *>(((gate_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null gate object in unwrap");
        return 0;
    }

    return object;
}

void gate_insert_(const std::pair< gate_type_ *, PyObject *> &p)
{




    gate_wrapped_.insert(p);


}

void gate_erase_(gate_type_ *p)
{




    gate_wrapped_.erase(p);


}

PyObject * gate_find_(gate_type_ *p)
{




    PyObject *ret = 0;
    std::map< gate_type_ *, PyObject *>::iterator where = gate_wrapped_.find(p);
    if(where != gate_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct gate_dispatch_ gate_realdispatcher_ =
{
    &gate_typeobject_,
    gate_wrap_,
    gate_wrapcopy_,
    gate_unwrap_,
    gate_insert_,
    gate_erase_,
    gate_find_
};

  

struct correlator_object_ {
    PyObject_HEAD
    correlator_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct correlator_wrapper_: public correlator_type_ 
{

    static void __deleter__(correlator_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    correlator_wrapper_( PyObject *o, PyInterpreterState *i  ,clockdomain_ctl_type_ * a0  ,const std::string & a1  ,const d2d_nb_type_ & a2  ,const cookie_type_ & a3  ,unsigned int a4  ,unsigned int a5  ): correlator_type_(  a0 ,a1 ,a2 ,a3 ,a4 ,a5 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~correlator_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(correlator_object_ *self, visitproc visit, void *arg)
    {
        correlator_wrapper_ *o = (correlator_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(correlator_object_ *self)
    {
        correlator_wrapper_ *o = (correlator_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    clocksink_type_ * clocksink_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return correlator_type_::clocksink();
                
            
            
        }

        return
            
                clocksink
            
            
                ();
    }

    static PyObject *clocksink_method_(correlator_object_ *self, PyObject *args)
    {
        

        correlator_wrapper_ *o = (correlator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null correlator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    clocksink_type_ * r = o->clocksink_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_clocksink((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in correlator::clocksink");
        }

        return 0;
    }



    void clock_plumbed_call_(bool owned ,unsigned int a0 ,bool a1 )
    {
        if(owned)
        {
            
                
                
                    return correlator_type_::clock_plumbed( a0  ,a1 );
                
            
            
        }

        return
            
                clock_plumbed
            
            
                ( a0  ,a1 );
    }

    static PyObject *clock_plumbed_method_(correlator_object_ *self, PyObject *args)
    {
         unsigned int a0;  bool a1; 

        correlator_wrapper_ *o = (correlator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null correlator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_b,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clock_plumbed_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in correlator::clock_plumbed");
        }

        return 0;
    }



    void plumb_input_call_(bool owned ,unsigned int a0 ,unsigned int a1 ,const data_type_ & a2 ,int a3 ,unsigned int a4 ,fastdata_type_ * a5 ,const converter_ref_type_ & a6 ,const d2d_nb_type_ & a7 )
    {
        if(owned)
        {
            
                
                
                    return correlator_type_::plumb_input( a0  ,a1  ,a2  ,a3  ,a4  ,a5  ,a6  ,a7 );
                
            
            
        }

        return
            
                plumb_input
            
            
                ( a0  ,a1  ,a2  ,a3  ,a4  ,a5  ,a6  ,a7 );
    }

    static PyObject *plumb_input_method_(correlator_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1;  data_type_ * a2;  int a3;  unsigned int a4;  fastdata_type_ * a5;  converter_ref_type_ * a6;  d2d_nb_type_ * a7; 

        correlator_wrapper_ *o = (correlator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null correlator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&O&O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1  ,fpcvt_data_copy,&a2  ,fpcvt_i,&a3  ,fpcvt_ui,&a4  ,fpcvt_fastdata,&a5  ,fpcvt_converter_ref_copy,&a6  ,fpcvt_d2d_nb_copy,&a7 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->plumb_input_call_ (self->owned ,a0,a1,*a2,a3,a4,a5,*a6,*a7 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in correlator::plumb_input");
        }

        return 0;
    }



    void unplumb_input_call_(bool owned ,unsigned int a0 ,unsigned int a1 ,const data_type_ & a2 ,int a3 )
    {
        if(owned)
        {
            
                
                
                    return correlator_type_::unplumb_input( a0  ,a1  ,a2  ,a3 );
                
            
            
        }

        return
            
                unplumb_input
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *unplumb_input_method_(correlator_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1;  data_type_ * a2;  int a3; 

        correlator_wrapper_ *o = (correlator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null correlator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1  ,fpcvt_data_copy,&a2  ,fpcvt_i,&a3 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->unplumb_input_call_ (self->owned ,a0,a1,*a2,a3 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in correlator::unplumb_input");
        }

        return 0;
    }



    void set_latency_call_(bool owned ,unsigned int a0 ,unsigned int a1 ,unsigned int a2 )
    {
        if(owned)
        {
            
                
                
                    return correlator_type_::set_latency( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                set_latency
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *set_latency_method_(correlator_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1;  unsigned int a2; 

        correlator_wrapper_ *o = (correlator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null correlator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1  ,fpcvt_ui,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_latency_call_ (self->owned ,a0,a1,a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in correlator::set_latency");
        }

        return 0;
    }



    void remove_latency_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return correlator_type_::remove_latency( a0  ,a1 );
                
            
            
        }

        return
            
                remove_latency
            
            
                ( a0  ,a1 );
    }

    static PyObject *remove_latency_method_(correlator_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        correlator_wrapper_ *o = (correlator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null correlator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->remove_latency_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in correlator::remove_latency");
        }

        return 0;
    }



    void kill_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return correlator_type_::kill();
                
            
            
        }

        return
            
                kill
            
            
                ();
    }

    static PyObject *kill_method_(correlator_object_ *self, PyObject *args)
    {
        

        correlator_wrapper_ *o = (correlator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null correlator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->kill_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in correlator::kill");
        }

        return 0;
    }





    static PyObject *_method_lock(correlator_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(correlator_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void correlator_delete_(correlator_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            correlator_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (correlator_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        correlator_wrapper_::__deleter__((correlator_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void correlator_dealloc_(correlator_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    correlator_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * correlator_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    correlator_object_ *self;

    if(!(self = (correlator_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int correlator_init_(correlator_object_ *self, PyObject *args, PyObject *kwds)
{
    correlator_delete_(self);

    

    {
         clockdomain_ctl_type_ * a0;  std::string a1;  d2d_nb_type_ * a2;  cookie_type_ * a3;  unsigned int a4;  unsigned int a5; 

        if(PyArg_ParseTuple(args,"O&O&O&O&O&O&" ,fpcvt_clockdomain_ctl,&a0,fpcvt_stdstr,&a1,fpcvt_d2d_nb_copy,&a2,fpcvt_cookie_copy,&a3,fpcvt_ui,&a4,fpcvt_ui,&a5))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new correlator_wrapper_((PyObject *)self,interp ,a0,a1,*a2,*a3,a4,a5);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing correlator");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            correlator_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"correlator invalid arguments");
    return -1;

}


static const char *correlator_clocksink_docstring_ = "";

static const char *correlator_clock_plumbed_docstring_ = "";

static const char *correlator_plumb_input_docstring_ = "";

static const char *correlator_unplumb_input_docstring_ = "";

static const char *correlator_set_latency_docstring_ = "";

static const char *correlator_remove_latency_docstring_ = "";

static const char *correlator_kill_docstring_ = "";




static PyMethodDef correlator_methods_[] = {
    
        { "clocksink",(PyCFunction)correlator_wrapper_::clocksink_method_,METH_VARARGS,(char *)correlator_clocksink_docstring_ },
    
        { "clock_plumbed",(PyCFunction)correlator_wrapper_::clock_plumbed_method_,METH_VARARGS,(char *)correlator_clock_plumbed_docstring_ },
    
        { "plumb_input",(PyCFunction)correlator_wrapper_::plumb_input_method_,METH_VARARGS,(char *)correlator_plumb_input_docstring_ },
    
        { "unplumb_input",(PyCFunction)correlator_wrapper_::unplumb_input_method_,METH_VARARGS,(char *)correlator_unplumb_input_docstring_ },
    
        { "set_latency",(PyCFunction)correlator_wrapper_::set_latency_method_,METH_VARARGS,(char *)correlator_set_latency_docstring_ },
    
        { "remove_latency",(PyCFunction)correlator_wrapper_::remove_latency_method_,METH_VARARGS,(char *)correlator_remove_latency_docstring_ },
    
        { "kill",(PyCFunction)correlator_wrapper_::kill_method_,METH_VARARGS,(char *)correlator_kill_docstring_ },
    
    
        { "py_lock",(PyCFunction)correlator_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)correlator_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *correlator_docstring_ = "";

PyTypeObject correlator_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.correlator",/*tp_name*/
    sizeof(correlator_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)correlator_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)correlator_docstring_,       /* tp_doc */
    (traverseproc)correlator_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)correlator_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    correlator_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)correlator_init_,  /* tp_init */
    0,                         /* tp_alloc */
    correlator_new_,             /* tp_new */
};

PyObject *correlator_wrapcopy_(correlator_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *correlator_wrap_(correlator_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = correlator_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    correlator_object_ *self = 
        
        PyObject_GC_New(correlator_object_, &correlator_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        correlator_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

correlator_type_ *correlator_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &correlator_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected correlator object");
        return 0;
    }

    correlator_type_ *object = static_cast<correlator_type_ *>(((correlator_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null correlator object in unwrap");
        return 0;
    }

    return object;
}

void correlator_insert_(const std::pair< correlator_type_ *, PyObject *> &p)
{




    correlator_wrapped_.insert(p);


}

void correlator_erase_(correlator_type_ *p)
{




    correlator_wrapped_.erase(p);


}

PyObject * correlator_find_(correlator_type_ *p)
{




    PyObject *ret = 0;
    std::map< correlator_type_ *, PyObject *>::iterator where = correlator_wrapped_.find(p);
    if(where != correlator_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct correlator_dispatch_ correlator_realdispatcher_ =
{
    &correlator_typeobject_,
    correlator_wrap_,
    correlator_wrapcopy_,
    correlator_unwrap_,
    correlator_insert_,
    correlator_erase_,
    correlator_find_
};

  

struct connector_object_ {
    PyObject_HEAD
    client_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct connector_wrapper_: public connector_type_ 
{

    static void __deleter__(connector_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
        cache_4_ = 0;
    
        cache_5_ = 0;
    
        cache_6_ = 0;
    
        cache_7_ = 0;
    
        cache_8_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
        if(cache_4_)
        {
            Py_DECREF(cache_4_);
            cache_4_ = 0;
        }
    
        if(cache_5_)
        {
            Py_DECREF(cache_5_);
            cache_5_ = 0;
        }
    
        if(cache_6_)
        {
            Py_DECREF(cache_6_);
            cache_6_ = 0;
        }
    
        if(cache_7_)
        {
            Py_DECREF(cache_7_);
            cache_7_ = 0;
        }
    
        if(cache_8_)
        {
            Py_DECREF(cache_8_);
            cache_8_ = 0;
        }
    
    }
*/



    connector_wrapper_( PyObject *o, PyInterpreterState *i  ,correlator_type_ * a0  ,unsigned int a1  ,unsigned int a2  ,int a3  ,unsigned int a4  ,const d2d_nb_type_ & a5  ,bool a6  ): connector_type_(  a0 ,a1 ,a2 ,a3 ,a4 ,a5 ,a6 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~connector_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void client_sync(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_sync");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_sync before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_sync after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void client_tree(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_tree");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_tree before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_tree after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void client_opened(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_opened");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_opened before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_opened after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    converter_ref_type_ create_converter(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"create_converter");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("create_converter before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("create_converter after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            converter_ref_type_ * rptr;

            if(!fpcvt_converter_ref_copy(result,&rptr))
            {
                throw pip_err_t("wrong type result converting converter_ref_copy");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        converter_ref_type_ rcopy(*rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_4_;
 
    void close_client(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_4_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"close_client");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_4_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("close_client before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("close_client after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_5_;
 
    void client_data(  const data_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_5_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_data");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_5_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_data_copy,&a0);
        //printf("client_data before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_data after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_6_;
 
    void client_clock(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_6_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_clock");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_6_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_clock before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_clock after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_7_;
 
    void client_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_7_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_7_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_8_;
 
    void client_child(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_8_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_child");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_8_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_child before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_child after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }













    static int gc_traverse_method_(connector_object_ *self, visitproc visit, void *arg)
    {
        connector_wrapper_ *o = (connector_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(connector_object_ *self)
    {
        connector_wrapper_ *o = (connector_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void client_sync_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return connector_type_::client_sync();
                
            
            
        }

        return
            
                client_sync
            
            
                ();
    }

    static PyObject *client_sync_method_(connector_object_ *self, PyObject *args)
    {
        

        connector_wrapper_ *o = (connector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null connector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_sync_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in connector::client_sync");
        }

        return 0;
    }



    void client_tree_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return connector_type_::client_tree();
                
            
            
        }

        return
            
                client_tree
            
            
                ();
    }

    static PyObject *client_tree_method_(connector_object_ *self, PyObject *args)
    {
        

        connector_wrapper_ *o = (connector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null connector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_tree_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in connector::client_tree");
        }

        return 0;
    }



    void client_clock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return connector_type_::client_clock();
                
            
            
        }

        return
            
                client_clock
            
            
                ();
    }

    static PyObject *client_clock_method_(connector_object_ *self, PyObject *args)
    {
        

        connector_wrapper_ *o = (connector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null connector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_clock_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in connector::client_clock");
        }

        return 0;
    }



    converter_ref_type_ create_converter_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method connector::create_converter");
                
                
            
            
        }

        return
            
                create_converter
            
            
                ();
    }

    static PyObject *create_converter_method_(connector_object_ *self, PyObject *args)
    {
        

        connector_wrapper_ *o = (connector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null connector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    converter_ref_type_ r = o->create_converter_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_converter_ref_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in connector::create_converter");
        }

        return 0;
    }



    void close_client_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return connector_type_::close_client();
                
            
            
        }

        return
            
                close_client
            
            
                ();
    }

    static PyObject *close_client_method_(connector_object_ *self, PyObject *args)
    {
        

        connector_wrapper_ *o = (connector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null connector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_client_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in connector::close_client");
        }

        return 0;
    }



    void client_data_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return connector_type_::client_data( a0 );
                
            
            
        }

        return
            
                client_data
            
            
                ( a0 );
    }

    static PyObject *client_data_method_(connector_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        connector_wrapper_ *o = (connector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null connector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_data_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in connector::client_data");
        }

        return 0;
    }



    void client_opened_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return connector_type_::client_opened();
                
            
            
        }

        return
            
                client_opened
            
            
                ();
    }

    static PyObject *client_opened_method_(connector_object_ *self, PyObject *args)
    {
        

        connector_wrapper_ *o = (connector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null connector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_opened_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in connector::client_opened");
        }

        return 0;
    }



    void client_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return connector_type_::client_closed();
                
            
            
        }

        return
            
                client_closed
            
            
                ();
    }

    static PyObject *client_closed_method_(connector_object_ *self, PyObject *args)
    {
        

        connector_wrapper_ *o = (connector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null connector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in connector::client_closed");
        }

        return 0;
    }



    void client_child_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return connector_type_::client_child();
                
            
            
        }

        return
            
                client_child
            
            
                ();
    }

    static PyObject *client_child_method_(connector_object_ *self, PyObject *args)
    {
        

        connector_wrapper_ *o = (connector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null connector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_child_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in connector::client_child");
        }

        return 0;
    }



    void set_clocked_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return connector_type_::set_clocked( a0 );
                
            
            
        }

        return
            
                set_clocked
            
            
                ( a0 );
    }

    static PyObject *set_clocked_method_(connector_object_ *self, PyObject *args)
    {
         bool a0; 

        connector_wrapper_ *o = (connector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null connector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_clocked_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in connector::set_clocked");
        }

        return 0;
    }





    static PyObject *_method_lock(connector_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(connector_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void connector_delete_(connector_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            client_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (connector_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        connector_wrapper_::__deleter__((connector_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void connector_dealloc_(connector_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    connector_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * connector_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    connector_object_ *self;

    if(!(self = (connector_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int connector_init_(connector_object_ *self, PyObject *args, PyObject *kwds)
{
    connector_delete_(self);

    

    {
         correlator_type_ * a0;  unsigned int a1;  unsigned int a2;  int a3;  unsigned int a4;  d2d_nb_type_ * a5;  bool a6; 

        if(PyArg_ParseTuple(args,"O&O&O&O&O&O&O&" ,fpcvt_correlator,&a0,fpcvt_ui,&a1,fpcvt_ui,&a2,fpcvt_i,&a3,fpcvt_ui,&a4,fpcvt_d2d_nb_copy,&a5,fpcvt_b,&a6))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new connector_wrapper_((PyObject *)self,interp ,a0,a1,a2,a3,a4,*a5,a6);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing connector");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            client_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"connector invalid arguments");
    return -1;

}


static const char *connector_client_sync_docstring_ = "";

static const char *connector_client_tree_docstring_ = "";

static const char *connector_client_clock_docstring_ = "";

static const char *connector_create_converter_docstring_ = "";

static const char *connector_close_client_docstring_ = "";

static const char *connector_client_data_docstring_ = "";

static const char *connector_client_opened_docstring_ = "";

static const char *connector_client_closed_docstring_ = "";

static const char *connector_client_child_docstring_ = "";

static const char *connector_set_clocked_docstring_ = "";




static PyMethodDef connector_methods_[] = {
    
        { "client_sync",(PyCFunction)connector_wrapper_::client_sync_method_,METH_VARARGS,(char *)connector_client_sync_docstring_ },
    
        { "client_tree",(PyCFunction)connector_wrapper_::client_tree_method_,METH_VARARGS,(char *)connector_client_tree_docstring_ },
    
        { "client_clock",(PyCFunction)connector_wrapper_::client_clock_method_,METH_VARARGS,(char *)connector_client_clock_docstring_ },
    
        { "create_converter",(PyCFunction)connector_wrapper_::create_converter_method_,METH_VARARGS,(char *)connector_create_converter_docstring_ },
    
        { "close_client",(PyCFunction)connector_wrapper_::close_client_method_,METH_VARARGS,(char *)connector_close_client_docstring_ },
    
        { "client_data",(PyCFunction)connector_wrapper_::client_data_method_,METH_VARARGS,(char *)connector_client_data_docstring_ },
    
        { "client_opened",(PyCFunction)connector_wrapper_::client_opened_method_,METH_VARARGS,(char *)connector_client_opened_docstring_ },
    
        { "client_closed",(PyCFunction)connector_wrapper_::client_closed_method_,METH_VARARGS,(char *)connector_client_closed_docstring_ },
    
        { "client_child",(PyCFunction)connector_wrapper_::client_child_method_,METH_VARARGS,(char *)connector_client_child_docstring_ },
    
        { "set_clocked",(PyCFunction)connector_wrapper_::set_clocked_method_,METH_VARARGS,(char *)connector_set_clocked_docstring_ },
    
    
        { "py_lock",(PyCFunction)connector_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)connector_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *connector_docstring_ = "";

PyTypeObject connector_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.connector",/*tp_name*/
    sizeof(connector_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)connector_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)connector_docstring_,       /* tp_doc */
    (traverseproc)connector_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)connector_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    connector_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)connector_init_,  /* tp_init */
    0,                         /* tp_alloc */
    connector_new_,             /* tp_new */
};

PyObject *connector_wrapcopy_(connector_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *connector_wrap_(connector_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = client_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    connector_object_ *self = 
        
        PyObject_GC_New(connector_object_, &connector_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        client_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

connector_type_ *connector_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &connector_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected connector object");
        return 0;
    }

    connector_type_ *object = static_cast<connector_type_ *>(((connector_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null connector object in unwrap");
        return 0;
    }

    return object;
}

void connector_insert_(const std::pair< connector_type_ *, PyObject *> &p)
{


    client_dispatcher_->insert(p);




}

void connector_erase_(connector_type_ *p)
{


    client_dispatcher_->erase(p);




}

PyObject * connector_find_(connector_type_ *p)
{


    return client_dispatcher_->find(p);




}

static struct connector_dispatch_ connector_realdispatcher_ =
{
    &connector_typeobject_,
    connector_wrap_,
    connector_wrapcopy_,
    connector_unwrap_,
    connector_insert_,
    connector_erase_,
    connector_find_
};

  

struct converter_ref_object_ {
    PyObject_HEAD
    converter_ref_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct converter_ref_wrapper_: public converter_ref_type_ 
{

    static void __deleter__(converter_ref_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    converter_ref_wrapper_( PyObject *o, PyInterpreterState *i  ,const converter_ref_type_ & a0  ): converter_ref_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~converter_ref_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(converter_ref_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(converter_ref_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void converter_ref_delete_(converter_ref_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            converter_ref_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (converter_ref_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        converter_ref_wrapper_::__deleter__((converter_ref_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void converter_ref_dealloc_(converter_ref_object_* self)
{
    
    converter_ref_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * converter_ref_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    converter_ref_object_ *self;

    if(!(self = (converter_ref_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int converter_ref_init_(converter_ref_object_ *self, PyObject *args, PyObject *kwds)
{
    converter_ref_delete_(self);

    

    {
         converter_ref_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_converter_ref_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new converter_ref_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing converter_ref");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            converter_ref_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"converter_ref invalid arguments");
    return -1;

}





static PyMethodDef converter_ref_methods_[] = {
    
    
        { "py_lock",(PyCFunction)converter_ref_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)converter_ref_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *converter_ref_docstring_ = "";

PyTypeObject converter_ref_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.converter_ref",/*tp_name*/
    sizeof(converter_ref_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)converter_ref_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)converter_ref_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    converter_ref_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)converter_ref_init_,  /* tp_init */
    0,                         /* tp_alloc */
    converter_ref_new_,             /* tp_new */
};

PyObject *converter_ref_wrapcopy_(converter_ref_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            converter_ref_object_ *obj = 
                PyObject_NEW(converter_ref_object_, converter_ref_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new converter_ref_type_(*o);
                }

                try
                {
                    converter_ref_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying converter_ref");
                return 0;
            }
        

    
}

PyObject *converter_ref_wrap_(converter_ref_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = converter_ref_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    converter_ref_object_ *self = 
        PyObject_NEW(converter_ref_object_, &converter_ref_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        converter_ref_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

converter_ref_type_ *converter_ref_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &converter_ref_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected converter_ref object");
        return 0;
    }

    converter_ref_type_ *object = static_cast<converter_ref_type_ *>(((converter_ref_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null converter_ref object in unwrap");
        return 0;
    }

    return object;
}

void converter_ref_insert_(const std::pair< converter_ref_type_ *, PyObject *> &p)
{




    converter_ref_wrapped_.insert(p);


}

void converter_ref_erase_(converter_ref_type_ *p)
{




    converter_ref_wrapped_.erase(p);


}

PyObject * converter_ref_find_(converter_ref_type_ *p)
{




    PyObject *ret = 0;
    std::map< converter_ref_type_ *, PyObject *>::iterator where = converter_ref_wrapped_.find(p);
    if(where != converter_ref_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct converter_ref_dispatch_ converter_ref_realdispatcher_ =
{
    &converter_ref_typeobject_,
    converter_ref_wrap_,
    converter_ref_wrapcopy_,
    converter_ref_unwrap_,
    converter_ref_insert_,
    converter_ref_erase_,
    converter_ref_find_
};

  

struct capture_backend_object_ {
    PyObject_HEAD
    capture_backend_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct capture_backend_wrapper_: public capture_backend_type_ 
{

    static void __deleter__(capture_backend_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    capture_backend_wrapper_( PyObject *o, PyInterpreterState *i  ,const std::string & a0  ,unsigned int a1  ,bool a2  ,bool a3  ): capture_backend_type_(  a0 ,a1 ,a2 ,a3 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~capture_backend_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return capture_backend_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(capture_backend_object_ *self, PyObject *args)
    {
        

        capture_backend_wrapper_ *o = (capture_backend_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null capture_backend c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in capture_backend::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(capture_backend_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(capture_backend_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void capture_backend_delete_(capture_backend_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            capture_backend_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (capture_backend_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        capture_backend_wrapper_::__deleter__((capture_backend_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void capture_backend_dealloc_(capture_backend_object_* self)
{
    
    capture_backend_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * capture_backend_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    capture_backend_object_ *self;

    if(!(self = (capture_backend_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int capture_backend_init_(capture_backend_object_ *self, PyObject *args, PyObject *kwds)
{
    capture_backend_delete_(self);

    

    {
         std::string a0;  unsigned int a1;  bool a2;  bool a3; 

        if(PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_stdstr,&a0,fpcvt_ui,&a1,fpcvt_b,&a2,fpcvt_b,&a3))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new capture_backend_wrapper_((PyObject *)self,interp ,a0,a1,a2,a3);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing capture_backend");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            capture_backend_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"capture_backend invalid arguments");
    return -1;

}


static const char *capture_backend_cookie_docstring_ = "";




static PyMethodDef capture_backend_methods_[] = {
    
        { "cookie",(PyCFunction)capture_backend_wrapper_::cookie_method_,METH_VARARGS,(char *)capture_backend_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)capture_backend_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)capture_backend_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *capture_backend_docstring_ = "";

PyTypeObject capture_backend_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.capture_backend",/*tp_name*/
    sizeof(capture_backend_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)capture_backend_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)capture_backend_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    capture_backend_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)capture_backend_init_,  /* tp_init */
    0,                         /* tp_alloc */
    capture_backend_new_,             /* tp_new */
};

PyObject *capture_backend_wrapcopy_(capture_backend_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *capture_backend_wrap_(capture_backend_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = capture_backend_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    capture_backend_object_ *self = 
        PyObject_NEW(capture_backend_object_, &capture_backend_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        capture_backend_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

capture_backend_type_ *capture_backend_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &capture_backend_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected capture_backend object");
        return 0;
    }

    capture_backend_type_ *object = static_cast<capture_backend_type_ *>(((capture_backend_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null capture_backend object in unwrap");
        return 0;
    }

    return object;
}

void capture_backend_insert_(const std::pair< capture_backend_type_ *, PyObject *> &p)
{




    capture_backend_wrapped_.insert(p);


}

void capture_backend_erase_(capture_backend_type_ *p)
{




    capture_backend_wrapped_.erase(p);


}

PyObject * capture_backend_find_(capture_backend_type_ *p)
{




    PyObject *ret = 0;
    std::map< capture_backend_type_ *, PyObject *>::iterator where = capture_backend_wrapped_.find(p);
    if(where != capture_backend_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct capture_backend_dispatch_ capture_backend_realdispatcher_ =
{
    &capture_backend_typeobject_,
    capture_backend_wrap_,
    capture_backend_wrapcopy_,
    capture_backend_unwrap_,
    capture_backend_insert_,
    capture_backend_erase_,
    capture_backend_find_
};

  

struct functor_backend_object_ {
    PyObject_HEAD
    functor_backend_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct functor_backend_wrapper_: public functor_backend_type_ 
{

    static void __deleter__(functor_backend_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    functor_backend_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned int a0  ,bool a1  ): functor_backend_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~functor_backend_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(functor_backend_object_ *self, visitproc visit, void *arg)
    {
        functor_backend_wrapper_ *o = (functor_backend_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(functor_backend_object_ *self)
    {
        functor_backend_wrapper_ *o = (functor_backend_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void set_functor_call_(bool owned ,const data_type_ & a0 ,const change_nb_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return functor_backend_type_::set_functor( a0  ,a1 );
                
            
            
        }

        return
            
                set_functor
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_functor_method_(functor_backend_object_ *self, PyObject *args)
    {
         data_type_ * a0;  change_nb_type_ * a1; 

        functor_backend_wrapper_ *o = (functor_backend_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null functor_backend c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_data_copy,&a0  ,fpcvt_change_nb_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_functor_call_ (self->owned ,*a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in functor_backend::set_functor");
        }

        return 0;
    }



    void clear_functor_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return functor_backend_type_::clear_functor( a0 );
                
            
            
        }

        return
            
                clear_functor
            
            
                ( a0 );
    }

    static PyObject *clear_functor_method_(functor_backend_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        functor_backend_wrapper_ *o = (functor_backend_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null functor_backend c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_functor_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in functor_backend::clear_functor");
        }

        return 0;
    }



    void set_efunctor_call_(bool owned ,const change_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return functor_backend_type_::set_efunctor( a0 );
                
            
            
        }

        return
            
                set_efunctor
            
            
                ( a0 );
    }

    static PyObject *set_efunctor_method_(functor_backend_object_ *self, PyObject *args)
    {
         change_nb_type_ * a0; 

        functor_backend_wrapper_ *o = (functor_backend_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null functor_backend c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_efunctor_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in functor_backend::set_efunctor");
        }

        return 0;
    }



    void clear_efunctor_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return functor_backend_type_::clear_efunctor();
                
            
            
        }

        return
            
                clear_efunctor
            
            
                ();
    }

    static PyObject *clear_efunctor_method_(functor_backend_object_ *self, PyObject *args)
    {
        

        functor_backend_wrapper_ *o = (functor_backend_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null functor_backend c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_efunctor_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in functor_backend::clear_efunctor");
        }

        return 0;
    }



    void set_gfunctor_call_(bool owned ,const change_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return functor_backend_type_::set_gfunctor( a0 );
                
            
            
        }

        return
            
                set_gfunctor
            
            
                ( a0 );
    }

    static PyObject *set_gfunctor_method_(functor_backend_object_ *self, PyObject *args)
    {
         change_nb_type_ * a0; 

        functor_backend_wrapper_ *o = (functor_backend_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null functor_backend c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_gfunctor_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in functor_backend::set_gfunctor");
        }

        return 0;
    }



    void clear_gfunctor_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return functor_backend_type_::clear_gfunctor();
                
            
            
        }

        return
            
                clear_gfunctor
            
            
                ();
    }

    static PyObject *clear_gfunctor_method_(functor_backend_object_ *self, PyObject *args)
    {
        

        functor_backend_wrapper_ *o = (functor_backend_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null functor_backend c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_gfunctor_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in functor_backend::clear_gfunctor");
        }

        return 0;
    }



    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return functor_backend_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(functor_backend_object_ *self, PyObject *args)
    {
        

        functor_backend_wrapper_ *o = (functor_backend_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null functor_backend c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in functor_backend::cookie");
        }

        return 0;
    }



    clockref_type_ * get_clock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return functor_backend_type_::get_clock();
                
            
            
        }

        return
            
                get_clock
            
            
                ();
    }

    static PyObject *get_clock_method_(functor_backend_object_ *self, PyObject *args)
    {
        

        functor_backend_wrapper_ *o = (functor_backend_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null functor_backend c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    clockref_type_ * r = o->get_clock_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_clockref((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in functor_backend::get_clock");
        }

        return 0;
    }



    void send_duplicates_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return functor_backend_type_::send_duplicates( a0 );
                
            
            
        }

        return
            
                send_duplicates
            
            
                ( a0 );
    }

    static PyObject *send_duplicates_method_(functor_backend_object_ *self, PyObject *args)
    {
         bool a0; 

        functor_backend_wrapper_ *o = (functor_backend_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null functor_backend c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->send_duplicates_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in functor_backend::send_duplicates");
        }

        return 0;
    }





    static PyObject *_method_lock(functor_backend_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(functor_backend_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void functor_backend_delete_(functor_backend_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            functor_backend_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (functor_backend_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        functor_backend_wrapper_::__deleter__((functor_backend_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void functor_backend_dealloc_(functor_backend_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    functor_backend_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * functor_backend_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    functor_backend_object_ *self;

    if(!(self = (functor_backend_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int functor_backend_init_(functor_backend_object_ *self, PyObject *args, PyObject *kwds)
{
    functor_backend_delete_(self);

    

    {
         unsigned int a0;  bool a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_ui,&a0,fpcvt_b,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new functor_backend_wrapper_((PyObject *)self,interp ,a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing functor_backend");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            functor_backend_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"functor_backend invalid arguments");
    return -1;

}


static const char *functor_backend_set_functor_docstring_ = "";

static const char *functor_backend_clear_functor_docstring_ = "";

static const char *functor_backend_set_efunctor_docstring_ = "";

static const char *functor_backend_clear_efunctor_docstring_ = "";

static const char *functor_backend_set_gfunctor_docstring_ = "";

static const char *functor_backend_clear_gfunctor_docstring_ = "";

static const char *functor_backend_cookie_docstring_ = "";

static const char *functor_backend_get_clock_docstring_ = "";

static const char *functor_backend_send_duplicates_docstring_ = "";




static PyMethodDef functor_backend_methods_[] = {
    
        { "set_functor",(PyCFunction)functor_backend_wrapper_::set_functor_method_,METH_VARARGS,(char *)functor_backend_set_functor_docstring_ },
    
        { "clear_functor",(PyCFunction)functor_backend_wrapper_::clear_functor_method_,METH_VARARGS,(char *)functor_backend_clear_functor_docstring_ },
    
        { "set_efunctor",(PyCFunction)functor_backend_wrapper_::set_efunctor_method_,METH_VARARGS,(char *)functor_backend_set_efunctor_docstring_ },
    
        { "clear_efunctor",(PyCFunction)functor_backend_wrapper_::clear_efunctor_method_,METH_VARARGS,(char *)functor_backend_clear_efunctor_docstring_ },
    
        { "set_gfunctor",(PyCFunction)functor_backend_wrapper_::set_gfunctor_method_,METH_VARARGS,(char *)functor_backend_set_gfunctor_docstring_ },
    
        { "clear_gfunctor",(PyCFunction)functor_backend_wrapper_::clear_gfunctor_method_,METH_VARARGS,(char *)functor_backend_clear_gfunctor_docstring_ },
    
        { "cookie",(PyCFunction)functor_backend_wrapper_::cookie_method_,METH_VARARGS,(char *)functor_backend_cookie_docstring_ },
    
        { "get_clock",(PyCFunction)functor_backend_wrapper_::get_clock_method_,METH_VARARGS,(char *)functor_backend_get_clock_docstring_ },
    
        { "send_duplicates",(PyCFunction)functor_backend_wrapper_::send_duplicates_method_,METH_VARARGS,(char *)functor_backend_send_duplicates_docstring_ },
    
    
        { "py_lock",(PyCFunction)functor_backend_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)functor_backend_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *functor_backend_docstring_ = "";

PyTypeObject functor_backend_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.functor_backend",/*tp_name*/
    sizeof(functor_backend_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)functor_backend_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)functor_backend_docstring_,       /* tp_doc */
    (traverseproc)functor_backend_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)functor_backend_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    functor_backend_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)functor_backend_init_,  /* tp_init */
    0,                         /* tp_alloc */
    functor_backend_new_,             /* tp_new */
};

PyObject *functor_backend_wrapcopy_(functor_backend_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *functor_backend_wrap_(functor_backend_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = functor_backend_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    functor_backend_object_ *self = 
        
        PyObject_GC_New(functor_backend_object_, &functor_backend_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        functor_backend_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

functor_backend_type_ *functor_backend_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &functor_backend_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected functor_backend object");
        return 0;
    }

    functor_backend_type_ *object = static_cast<functor_backend_type_ *>(((functor_backend_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null functor_backend object in unwrap");
        return 0;
    }

    return object;
}

void functor_backend_insert_(const std::pair< functor_backend_type_ *, PyObject *> &p)
{




    functor_backend_wrapped_.insert(p);


}

void functor_backend_erase_(functor_backend_type_ *p)
{




    functor_backend_wrapped_.erase(p);


}

PyObject * functor_backend_find_(functor_backend_type_ *p)
{




    PyObject *ret = 0;
    std::map< functor_backend_type_ *, PyObject *>::iterator where = functor_backend_wrapped_.find(p);
    if(where != functor_backend_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct functor_backend_dispatch_ functor_backend_realdispatcher_ =
{
    &functor_backend_typeobject_,
    functor_backend_wrap_,
    functor_backend_wrapcopy_,
    functor_backend_unwrap_,
    functor_backend_insert_,
    functor_backend_erase_,
    functor_backend_find_
};

  

struct controller_object_ {
    PyObject_HEAD
    controller_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct controller_wrapper_: public controller_type_ 
{

    static void __deleter__(controller_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
    }
*/



    controller_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,const std::string & a1  ): controller_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~controller_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void controller_clock(  clockref_type_ * a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"controller_clock");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_clockref,&a0);
        //printf("controller_clock before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("controller_clock after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void controller_latency(  unsigned int a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"controller_latency");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_ui,&a0);
        //printf("controller_latency before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("controller_latency after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return controller_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(controller_object_ *self, PyObject *args)
    {
        

        controller_wrapper_ *o = (controller_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controller c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controller::cookie");
        }

        return 0;
    }



    void controller_clock_call_(bool owned ,clockref_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return controller_type_::controller_clock( a0 );
                
            
            
        }

        return
            
                controller_clock
            
            
                ( a0 );
    }

    static PyObject *controller_clock_method_(controller_object_ *self, PyObject *args)
    {
         clockref_type_ * a0; 

        controller_wrapper_ *o = (controller_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controller c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_clockref,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->controller_clock_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controller::controller_clock");
        }

        return 0;
    }



    void controller_latency_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return controller_type_::controller_latency( a0 );
                
            
            
        }

        return
            
                controller_latency
            
            
                ( a0 );
    }

    static PyObject *controller_latency_method_(controller_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        controller_wrapper_ *o = (controller_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controller c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->controller_latency_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controller::controller_latency");
        }

        return 0;
    }





    static PyObject *_method_lock(controller_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(controller_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void controller_delete_(controller_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            controller_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (controller_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        controller_wrapper_::__deleter__((controller_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void controller_dealloc_(controller_object_* self)
{
    
    controller_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * controller_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    controller_object_ *self;

    if(!(self = (controller_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int controller_init_(controller_object_ *self, PyObject *args, PyObject *kwds)
{
    controller_delete_(self);

    

    {
         cookie_type_ * a0;  std::string a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_stdstr,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new controller_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing controller");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            controller_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"controller invalid arguments");
    return -1;

}


static const char *controller_cookie_docstring_ = "";

static const char *controller_controller_clock_docstring_ = "";

static const char *controller_controller_latency_docstring_ = "";




static PyMethodDef controller_methods_[] = {
    
        { "cookie",(PyCFunction)controller_wrapper_::cookie_method_,METH_VARARGS,(char *)controller_cookie_docstring_ },
    
        { "controller_clock",(PyCFunction)controller_wrapper_::controller_clock_method_,METH_VARARGS,(char *)controller_controller_clock_docstring_ },
    
        { "controller_latency",(PyCFunction)controller_wrapper_::controller_latency_method_,METH_VARARGS,(char *)controller_controller_latency_docstring_ },
    
    
        { "py_lock",(PyCFunction)controller_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)controller_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *controller_docstring_ = "";

PyTypeObject controller_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.controller",/*tp_name*/
    sizeof(controller_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)controller_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)controller_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    controller_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)controller_init_,  /* tp_init */
    0,                         /* tp_alloc */
    controller_new_,             /* tp_new */
};

PyObject *controller_wrapcopy_(controller_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *controller_wrap_(controller_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = controller_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    controller_object_ *self = 
        PyObject_NEW(controller_object_, &controller_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        controller_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

controller_type_ *controller_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &controller_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected controller object");
        return 0;
    }

    controller_type_ *object = static_cast<controller_type_ *>(((controller_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null controller object in unwrap");
        return 0;
    }

    return object;
}

void controller_insert_(const std::pair< controller_type_ *, PyObject *> &p)
{




    controller_wrapped_.insert(p);


}

void controller_erase_(controller_type_ *p)
{




    controller_wrapped_.erase(p);


}

PyObject * controller_find_(controller_type_ *p)
{




    PyObject *ret = 0;
    std::map< controller_type_ *, PyObject *>::iterator where = controller_wrapped_.find(p);
    if(where != controller_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct controller_dispatch_ controller_realdispatcher_ =
{
    &controller_typeobject_,
    controller_wrap_,
    controller_wrapcopy_,
    controller_unwrap_,
    controller_insert_,
    controller_erase_,
    controller_find_
};

  

struct xxcontrolled_object_ {
    PyObject_HEAD
    xxcontrolled_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct xxcontrolled_wrapper_: public xxcontrolled_type_ 
{

    static void __deleter__(xxcontrolled_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    xxcontrolled_wrapper_( PyObject *o, PyInterpreterState *i  ): xxcontrolled_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~xxcontrolled_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void attach_call_(bool owned ,controller_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return xxcontrolled_type_::attach( a0 );
                
            
            
        }

        return
            
                attach
            
            
                ( a0 );
    }

    static PyObject *attach_method_(xxcontrolled_object_ *self, PyObject *args)
    {
         controller_type_ * a0; 

        xxcontrolled_wrapper_ *o = (xxcontrolled_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null xxcontrolled c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_controller,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->attach_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in xxcontrolled::attach");
        }

        return 0;
    }



    bool attach_to_call_(bool owned ,controller_type_ * a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return xxcontrolled_type_::attach_to( a0  ,a1 );
                
            
            
        }

        return
            
                attach_to
            
            
                ( a0  ,a1 );
    }

    static PyObject *attach_to_method_(xxcontrolled_object_ *self, PyObject *args)
    {
         controller_type_ * a0;  unsigned int a1; 

        xxcontrolled_wrapper_ *o = (xxcontrolled_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null xxcontrolled c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_controller,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->attach_to_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in xxcontrolled::attach_to");
        }

        return 0;
    }



    void detach_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return xxcontrolled_type_::detach();
                
            
            
        }

        return
            
                detach
            
            
                ();
    }

    static PyObject *detach_method_(xxcontrolled_object_ *self, PyObject *args)
    {
        

        xxcontrolled_wrapper_ *o = (xxcontrolled_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null xxcontrolled c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->detach_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in xxcontrolled::detach");
        }

        return 0;
    }



    int ordinal_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return xxcontrolled_type_::ordinal();
                
            
            
        }

        return
            
                ordinal
            
            
                ();
    }

    static PyObject *ordinal_method_(xxcontrolled_object_ *self, PyObject *args)
    {
        

        xxcontrolled_wrapper_ *o = (xxcontrolled_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null xxcontrolled c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    int r = o->ordinal_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_i((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in xxcontrolled::ordinal");
        }

        return 0;
    }



    fastdata_type_ * fastdata_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return xxcontrolled_type_::fastdata();
                
            
            
        }

        return
            
                fastdata
            
            
                ();
    }

    static PyObject *fastdata_method_(xxcontrolled_object_ *self, PyObject *args)
    {
        

        xxcontrolled_wrapper_ *o = (xxcontrolled_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null xxcontrolled c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    fastdata_type_ * r = o->fastdata_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_fastdata((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in xxcontrolled::fastdata");
        }

        return 0;
    }





    static PyObject *_method_lock(xxcontrolled_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(xxcontrolled_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void xxcontrolled_delete_(xxcontrolled_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            xxcontrolled_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (xxcontrolled_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        xxcontrolled_wrapper_::__deleter__((xxcontrolled_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void xxcontrolled_dealloc_(xxcontrolled_object_* self)
{
    
    xxcontrolled_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * xxcontrolled_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    xxcontrolled_object_ *self;

    if(!(self = (xxcontrolled_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int xxcontrolled_init_(xxcontrolled_object_ *self, PyObject *args, PyObject *kwds)
{
    xxcontrolled_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new xxcontrolled_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing xxcontrolled");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"xxcontrolled invalid arguments");
    return -1;

}


static const char *xxcontrolled_attach_docstring_ = "";

static const char *xxcontrolled_attach_to_docstring_ = "";

static const char *xxcontrolled_detach_docstring_ = "";

static const char *xxcontrolled_ordinal_docstring_ = "";

static const char *xxcontrolled_fastdata_docstring_ = "";




static PyMethodDef xxcontrolled_methods_[] = {
    
        { "attach",(PyCFunction)xxcontrolled_wrapper_::attach_method_,METH_VARARGS,(char *)xxcontrolled_attach_docstring_ },
    
        { "attach_to",(PyCFunction)xxcontrolled_wrapper_::attach_to_method_,METH_VARARGS,(char *)xxcontrolled_attach_to_docstring_ },
    
        { "detach",(PyCFunction)xxcontrolled_wrapper_::detach_method_,METH_VARARGS,(char *)xxcontrolled_detach_docstring_ },
    
        { "ordinal",(PyCFunction)xxcontrolled_wrapper_::ordinal_method_,METH_VARARGS,(char *)xxcontrolled_ordinal_docstring_ },
    
        { "fastdata",(PyCFunction)xxcontrolled_wrapper_::fastdata_method_,METH_VARARGS,(char *)xxcontrolled_fastdata_docstring_ },
    
    
        { "py_lock",(PyCFunction)xxcontrolled_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)xxcontrolled_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *xxcontrolled_docstring_ = "";

PyTypeObject xxcontrolled_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.xxcontrolled",/*tp_name*/
    sizeof(xxcontrolled_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)xxcontrolled_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)xxcontrolled_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    xxcontrolled_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)xxcontrolled_init_,  /* tp_init */
    0,                         /* tp_alloc */
    xxcontrolled_new_,             /* tp_new */
};

PyObject *xxcontrolled_wrapcopy_(xxcontrolled_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *xxcontrolled_wrap_(xxcontrolled_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = xxcontrolled_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    xxcontrolled_object_ *self = 
        PyObject_NEW(xxcontrolled_object_, &xxcontrolled_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

xxcontrolled_type_ *xxcontrolled_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &xxcontrolled_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected xxcontrolled object");
        return 0;
    }

    xxcontrolled_type_ *object = static_cast<xxcontrolled_type_ *>(((xxcontrolled_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null xxcontrolled object in unwrap");
        return 0;
    }

    return object;
}

void xxcontrolled_insert_(const std::pair< xxcontrolled_type_ *, PyObject *> &p)
{




    xxcontrolled_wrapped_.insert(p);


}

void xxcontrolled_erase_(xxcontrolled_type_ *p)
{




    xxcontrolled_wrapped_.erase(p);


}

PyObject * xxcontrolled_find_(xxcontrolled_type_ *p)
{




    PyObject *ret = 0;
    std::map< xxcontrolled_type_ *, PyObject *>::iterator where = xxcontrolled_wrapped_.find(p);
    if(where != xxcontrolled_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct xxcontrolled_dispatch_ xxcontrolled_realdispatcher_ =
{
    &xxcontrolled_typeobject_,
    xxcontrolled_wrap_,
    xxcontrolled_wrapcopy_,
    xxcontrolled_unwrap_,
    xxcontrolled_insert_,
    xxcontrolled_erase_,
    xxcontrolled_find_
};

  

struct controlled_object_ {
    PyObject_HEAD
    xxcontrolled_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct controlled_wrapper_: public controlled_type_ 
{

    static void __deleter__(controlled_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    controlled_wrapper_( PyObject *o, PyInterpreterState *i  ,bool a0  ): controlled_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~controlled_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    int ordinal_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return controlled_type_::ordinal();
                
            
            
        }

        return
            
                ordinal
            
            
                ();
    }

    static PyObject *ordinal_method_(controlled_object_ *self, PyObject *args)
    {
        

        controlled_wrapper_ *o = (controlled_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controlled c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    int r = o->ordinal_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_i((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controlled::ordinal");
        }

        return 0;
    }



    void enable_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return controlled_type_::enable();
                
            
            
        }

        return
            
                enable
            
            
                ();
    }

    static PyObject *enable_method_(controlled_object_ *self, PyObject *args)
    {
        

        controlled_wrapper_ *o = (controlled_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controlled c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enable_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controlled::enable");
        }

        return 0;
    }



    bool attach_to_call_(bool owned ,controller_type_ * a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return controlled_type_::attach_to( a0  ,a1 );
                
            
            
        }

        return
            
                attach_to
            
            
                ( a0  ,a1 );
    }

    static PyObject *attach_to_method_(controlled_object_ *self, PyObject *args)
    {
         controller_type_ * a0;  unsigned int a1; 

        controlled_wrapper_ *o = (controlled_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controlled c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_controller,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->attach_to_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controlled::attach_to");
        }

        return 0;
    }



    void attach_call_(bool owned ,controller_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return controlled_type_::attach( a0 );
                
            
            
        }

        return
            
                attach
            
            
                ( a0 );
    }

    static PyObject *attach_method_(controlled_object_ *self, PyObject *args)
    {
         controller_type_ * a0; 

        controlled_wrapper_ *o = (controlled_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controlled c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_controller,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->attach_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controlled::attach");
        }

        return 0;
    }



    void disable_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return controlled_type_::disable();
                
            
            
        }

        return
            
                disable
            
            
                ();
    }

    static PyObject *disable_method_(controlled_object_ *self, PyObject *args)
    {
        

        controlled_wrapper_ *o = (controlled_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controlled c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->disable_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controlled::disable");
        }

        return 0;
    }



    void detach_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return controlled_type_::detach();
                
            
            
        }

        return
            
                detach
            
            
                ();
    }

    static PyObject *detach_method_(controlled_object_ *self, PyObject *args)
    {
        

        controlled_wrapper_ *o = (controlled_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controlled c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->detach_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controlled::detach");
        }

        return 0;
    }





    static PyObject *_method_lock(controlled_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(controlled_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void controlled_delete_(controlled_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            xxcontrolled_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (controlled_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        controlled_wrapper_::__deleter__((controlled_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void controlled_dealloc_(controlled_object_* self)
{
    
    controlled_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * controlled_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    controlled_object_ *self;

    if(!(self = (controlled_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int controlled_init_(controlled_object_ *self, PyObject *args, PyObject *kwds)
{
    controlled_delete_(self);

    

    {
         bool a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_b,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new controlled_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing controlled");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"controlled invalid arguments");
    return -1;

}


static const char *controlled_ordinal_docstring_ = "";

static const char *controlled_enable_docstring_ = "";

static const char *controlled_attach_to_docstring_ = "";

static const char *controlled_attach_docstring_ = "";

static const char *controlled_disable_docstring_ = "";

static const char *controlled_detach_docstring_ = "";




static PyMethodDef controlled_methods_[] = {
    
        { "ordinal",(PyCFunction)controlled_wrapper_::ordinal_method_,METH_VARARGS,(char *)controlled_ordinal_docstring_ },
    
        { "enable",(PyCFunction)controlled_wrapper_::enable_method_,METH_VARARGS,(char *)controlled_enable_docstring_ },
    
        { "attach_to",(PyCFunction)controlled_wrapper_::attach_to_method_,METH_VARARGS,(char *)controlled_attach_to_docstring_ },
    
        { "attach",(PyCFunction)controlled_wrapper_::attach_method_,METH_VARARGS,(char *)controlled_attach_docstring_ },
    
        { "disable",(PyCFunction)controlled_wrapper_::disable_method_,METH_VARARGS,(char *)controlled_disable_docstring_ },
    
        { "detach",(PyCFunction)controlled_wrapper_::detach_method_,METH_VARARGS,(char *)controlled_detach_docstring_ },
    
    
        { "py_lock",(PyCFunction)controlled_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)controlled_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *controlled_docstring_ = "";

PyTypeObject controlled_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.controlled",/*tp_name*/
    sizeof(controlled_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)controlled_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)controlled_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    controlled_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)controlled_init_,  /* tp_init */
    0,                         /* tp_alloc */
    controlled_new_,             /* tp_new */
};

PyObject *controlled_wrapcopy_(controlled_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *controlled_wrap_(controlled_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = xxcontrolled_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    controlled_object_ *self = 
        PyObject_NEW(controlled_object_, &controlled_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

controlled_type_ *controlled_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &controlled_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected controlled object");
        return 0;
    }

    controlled_type_ *object = static_cast<controlled_type_ *>(((controlled_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null controlled object in unwrap");
        return 0;
    }

    return object;
}

void controlled_insert_(const std::pair< controlled_type_ *, PyObject *> &p)
{


    xxcontrolled_dispatcher_->insert(p);




}

void controlled_erase_(controlled_type_ *p)
{


    xxcontrolled_dispatcher_->erase(p);




}

PyObject * controlled_find_(controlled_type_ *p)
{


    return xxcontrolled_dispatcher_->find(p);




}

static struct controlled_dispatch_ controlled_realdispatcher_ =
{
    &controlled_typeobject_,
    controlled_wrap_,
    controlled_wrapcopy_,
    controlled_unwrap_,
    controlled_insert_,
    controlled_erase_,
    controlled_find_
};

  

struct fasttrigger_object_ {
    PyObject_HEAD
    xxcontrolled_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct fasttrigger_wrapper_: public fasttrigger_type_ 
{

    static void __deleter__(fasttrigger_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    fasttrigger_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned int a0  ): fasttrigger_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~fasttrigger_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    int ordinal_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fasttrigger_type_::ordinal();
                
            
            
        }

        return
            
                ordinal
            
            
                ();
    }

    static PyObject *ordinal_method_(fasttrigger_object_ *self, PyObject *args)
    {
        

        fasttrigger_wrapper_ *o = (fasttrigger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fasttrigger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    int r = o->ordinal_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_i((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fasttrigger::ordinal");
        }

        return 0;
    }



    bool attach_to_call_(bool owned ,controller_type_ * a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return fasttrigger_type_::attach_to( a0  ,a1 );
                
            
            
        }

        return
            
                attach_to
            
            
                ( a0  ,a1 );
    }

    static PyObject *attach_to_method_(fasttrigger_object_ *self, PyObject *args)
    {
         controller_type_ * a0;  unsigned int a1; 

        fasttrigger_wrapper_ *o = (fasttrigger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fasttrigger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_controller,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->attach_to_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fasttrigger::attach_to");
        }

        return 0;
    }



    void ping_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fasttrigger_type_::ping();
                
            
            
        }

        return
            
                ping
            
            
                ();
    }

    static PyObject *ping_method_(fasttrigger_object_ *self, PyObject *args)
    {
        

        fasttrigger_wrapper_ *o = (fasttrigger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fasttrigger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->ping_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fasttrigger::ping");
        }

        return 0;
    }



    void attach_call_(bool owned ,controller_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return fasttrigger_type_::attach( a0 );
                
            
            
        }

        return
            
                attach
            
            
                ( a0 );
    }

    static PyObject *attach_method_(fasttrigger_object_ *self, PyObject *args)
    {
         controller_type_ * a0; 

        fasttrigger_wrapper_ *o = (fasttrigger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fasttrigger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_controller,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->attach_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fasttrigger::attach");
        }

        return 0;
    }



    change_type_ trigger_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fasttrigger_type_::trigger();
                
            
            
        }

        return
            
                trigger
            
            
                ();
    }

    static PyObject *trigger_method_(fasttrigger_object_ *self, PyObject *args)
    {
        

        fasttrigger_wrapper_ *o = (fasttrigger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fasttrigger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_type_ r = o->trigger_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fasttrigger::trigger");
        }

        return 0;
    }



    void detach_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fasttrigger_type_::detach();
                
            
            
        }

        return
            
                detach
            
            
                ();
    }

    static PyObject *detach_method_(fasttrigger_object_ *self, PyObject *args)
    {
        

        fasttrigger_wrapper_ *o = (fasttrigger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fasttrigger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->detach_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fasttrigger::detach");
        }

        return 0;
    }





    static PyObject *_method_lock(fasttrigger_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(fasttrigger_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void fasttrigger_delete_(fasttrigger_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            xxcontrolled_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (fasttrigger_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        fasttrigger_wrapper_::__deleter__((fasttrigger_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void fasttrigger_dealloc_(fasttrigger_object_* self)
{
    
    fasttrigger_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * fasttrigger_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    fasttrigger_object_ *self;

    if(!(self = (fasttrigger_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int fasttrigger_init_(fasttrigger_object_ *self, PyObject *args, PyObject *kwds)
{
    fasttrigger_delete_(self);

    

    {
         unsigned int a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new fasttrigger_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing fasttrigger");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"fasttrigger invalid arguments");
    return -1;

}


static const char *fasttrigger_ordinal_docstring_ = "";

static const char *fasttrigger_attach_to_docstring_ = "";

static const char *fasttrigger_ping_docstring_ = "";

static const char *fasttrigger_attach_docstring_ = "";

static const char *fasttrigger_trigger_docstring_ = "";

static const char *fasttrigger_detach_docstring_ = "";




static PyMethodDef fasttrigger_methods_[] = {
    
        { "ordinal",(PyCFunction)fasttrigger_wrapper_::ordinal_method_,METH_VARARGS,(char *)fasttrigger_ordinal_docstring_ },
    
        { "attach_to",(PyCFunction)fasttrigger_wrapper_::attach_to_method_,METH_VARARGS,(char *)fasttrigger_attach_to_docstring_ },
    
        { "ping",(PyCFunction)fasttrigger_wrapper_::ping_method_,METH_VARARGS,(char *)fasttrigger_ping_docstring_ },
    
        { "attach",(PyCFunction)fasttrigger_wrapper_::attach_method_,METH_VARARGS,(char *)fasttrigger_attach_docstring_ },
    
        { "trigger",(PyCFunction)fasttrigger_wrapper_::trigger_method_,METH_VARARGS,(char *)fasttrigger_trigger_docstring_ },
    
        { "detach",(PyCFunction)fasttrigger_wrapper_::detach_method_,METH_VARARGS,(char *)fasttrigger_detach_docstring_ },
    
    
        { "py_lock",(PyCFunction)fasttrigger_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)fasttrigger_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *fasttrigger_docstring_ = "";

PyTypeObject fasttrigger_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.fasttrigger",/*tp_name*/
    sizeof(fasttrigger_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)fasttrigger_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)fasttrigger_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    fasttrigger_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)fasttrigger_init_,  /* tp_init */
    0,                         /* tp_alloc */
    fasttrigger_new_,             /* tp_new */
};

PyObject *fasttrigger_wrapcopy_(fasttrigger_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *fasttrigger_wrap_(fasttrigger_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = xxcontrolled_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    fasttrigger_object_ *self = 
        PyObject_NEW(fasttrigger_object_, &fasttrigger_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

fasttrigger_type_ *fasttrigger_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &fasttrigger_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected fasttrigger object");
        return 0;
    }

    fasttrigger_type_ *object = static_cast<fasttrigger_type_ *>(((fasttrigger_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null fasttrigger object in unwrap");
        return 0;
    }

    return object;
}

void fasttrigger_insert_(const std::pair< fasttrigger_type_ *, PyObject *> &p)
{


    xxcontrolled_dispatcher_->insert(p);




}

void fasttrigger_erase_(fasttrigger_type_ *p)
{


    xxcontrolled_dispatcher_->erase(p);




}

PyObject * fasttrigger_find_(fasttrigger_type_ *p)
{


    return xxcontrolled_dispatcher_->find(p);




}

static struct fasttrigger_dispatch_ fasttrigger_realdispatcher_ =
{
    &fasttrigger_typeobject_,
    fasttrigger_wrap_,
    fasttrigger_wrapcopy_,
    fasttrigger_unwrap_,
    fasttrigger_insert_,
    fasttrigger_erase_,
    fasttrigger_find_
};

  

struct cookie_object_ {
    PyObject_HEAD
    cookie_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct cookie_wrapper_: public cookie_type_ 
{

    static void __deleter__(cookie_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    cookie_wrapper_( PyObject *o, PyInterpreterState *i  ): cookie_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    cookie_wrapper_( PyObject *o, PyInterpreterState *i  ,root_type_ * a0  ): cookie_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    cookie_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ): cookie_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~cookie_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(cookie_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(cookie_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void cookie_delete_(cookie_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            cookie_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (cookie_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        cookie_wrapper_::__deleter__((cookie_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void cookie_dealloc_(cookie_object_* self)
{
    
    cookie_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * cookie_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    cookie_object_ *self;

    if(!(self = (cookie_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int cookie_init_(cookie_object_ *self, PyObject *args, PyObject *kwds)
{
    cookie_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new cookie_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing cookie");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            cookie_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         root_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_root,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new cookie_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing cookie");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            cookie_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         cookie_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_cookie_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new cookie_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing cookie");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            cookie_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"cookie invalid arguments");
    return -1;

}





static PyMethodDef cookie_methods_[] = {
    
    
        { "py_lock",(PyCFunction)cookie_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)cookie_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *cookie_docstring_ = ""
"    Opaque handle for a stream consumer."
"    "
"    Producers use the cookie to create a bundle structure through which the"
"    consumer receives data.  A cookie can be null, using a null cookie for"
"    connection will throw away any data sent.  Ony null cookies can be created"
"    from python, normally cookies are obtained from libpiw stream classes."
"    "
"    Note that the existance of a cookie doesn't count as a reference to the "
"    source object.  If the object goes away, the cookie becomes a null cookie."
"    You should arrange to keep the stream object providing the cookie alive"
"    by other means."
"    "
"    Initialisers"
"    ============"
"    "
"    cookie(root *)"
"    "
""
;

PyTypeObject cookie_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.cookie",/*tp_name*/
    sizeof(cookie_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)cookie_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)cookie_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    cookie_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)cookie_init_,  /* tp_init */
    0,                         /* tp_alloc */
    cookie_new_,             /* tp_new */
};

PyObject *cookie_wrapcopy_(cookie_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            cookie_object_ *obj = 
                PyObject_NEW(cookie_object_, cookie_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new cookie_type_(*o);
                }

                try
                {
                    cookie_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying cookie");
                return 0;
            }
        

    
}

PyObject *cookie_wrap_(cookie_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = cookie_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    cookie_object_ *self = 
        PyObject_NEW(cookie_object_, &cookie_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        cookie_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

cookie_type_ *cookie_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &cookie_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected cookie object");
        return 0;
    }

    cookie_type_ *object = static_cast<cookie_type_ *>(((cookie_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null cookie object in unwrap");
        return 0;
    }

    return object;
}

void cookie_insert_(const std::pair< cookie_type_ *, PyObject *> &p)
{




    cookie_wrapped_.insert(p);


}

void cookie_erase_(cookie_type_ *p)
{




    cookie_wrapped_.erase(p);


}

PyObject * cookie_find_(cookie_type_ *p)
{




    PyObject *ret = 0;
    std::map< cookie_type_ *, PyObject *>::iterator where = cookie_wrapped_.find(p);
    if(where != cookie_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct cookie_dispatch_ cookie_realdispatcher_ =
{
    &cookie_typeobject_,
    cookie_wrap_,
    cookie_wrapcopy_,
    cookie_unwrap_,
    cookie_insert_,
    cookie_erase_,
    cookie_find_
};

  

struct decode_ctl_object_ {
    PyObject_HEAD
    decode_ctl_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct decode_ctl_wrapper_: public decode_ctl_type_ 
{

    static void __deleter__(decode_ctl_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
    }
*/



    decode_ctl_wrapper_( PyObject *o, PyInterpreterState *i  ): decode_ctl_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~decode_ctl_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    wire_type_ * wire_create(  const event_data_source_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"wire_create");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_event_data_source_copy,&a0);
        //printf("wire_create before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("wire_create after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            wire_type_ * rptr;

            if(!fpcvt_wire(result,&rptr))
            {
                throw pip_err_t("wrong type result converting wire");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        wire_type_ * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_1_;
 
    void set_clock(  clockref_type_ * a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"set_clock");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_clockref,&a0);
        //printf("set_clock before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("set_clock after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void set_latency(  unsigned int a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"set_latency");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_ui,&a0);
        //printf("set_latency before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("set_latency after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    wire_type_ * wire_create_call_(bool owned ,const event_data_source_type_ & a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method decode_ctl::wire_create");
                
                
            
            
        }

        return
            
                wire_create
            
            
                ( a0 );
    }

    static PyObject *wire_create_method_(decode_ctl_object_ *self, PyObject *args)
    {
         event_data_source_type_ * a0; 

        decode_ctl_wrapper_ *o = (decode_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null decode_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_event_data_source_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    wire_type_ * r = o->wire_create_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_wire((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in decode_ctl::wire_create");
        }

        return 0;
    }



    void set_clock_call_(bool owned ,clockref_type_ * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method decode_ctl::set_clock");
                
                
            
            
        }

        return
            
                set_clock
            
            
                ( a0 );
    }

    static PyObject *set_clock_method_(decode_ctl_object_ *self, PyObject *args)
    {
         clockref_type_ * a0; 

        decode_ctl_wrapper_ *o = (decode_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null decode_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_clockref,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_clock_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in decode_ctl::set_clock");
        }

        return 0;
    }



    void set_latency_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method decode_ctl::set_latency");
                
                
            
            
        }

        return
            
                set_latency
            
            
                ( a0 );
    }

    static PyObject *set_latency_method_(decode_ctl_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        decode_ctl_wrapper_ *o = (decode_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null decode_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_latency_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in decode_ctl::set_latency");
        }

        return 0;
    }





    static PyObject *_method_lock(decode_ctl_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(decode_ctl_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void decode_ctl_delete_(decode_ctl_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            decode_ctl_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (decode_ctl_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        decode_ctl_wrapper_::__deleter__((decode_ctl_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void decode_ctl_dealloc_(decode_ctl_object_* self)
{
    
    decode_ctl_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * decode_ctl_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    decode_ctl_object_ *self;

    if(!(self = (decode_ctl_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int decode_ctl_init_(decode_ctl_object_ *self, PyObject *args, PyObject *kwds)
{
    decode_ctl_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new decode_ctl_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing decode_ctl");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            decode_ctl_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"decode_ctl invalid arguments");
    return -1;

}


static const char *decode_ctl_wire_create_docstring_ = "";

static const char *decode_ctl_set_clock_docstring_ = "";

static const char *decode_ctl_set_latency_docstring_ = "";




static PyMethodDef decode_ctl_methods_[] = {
    
        { "wire_create",(PyCFunction)decode_ctl_wrapper_::wire_create_method_,METH_VARARGS,(char *)decode_ctl_wire_create_docstring_ },
    
        { "set_clock",(PyCFunction)decode_ctl_wrapper_::set_clock_method_,METH_VARARGS,(char *)decode_ctl_set_clock_docstring_ },
    
        { "set_latency",(PyCFunction)decode_ctl_wrapper_::set_latency_method_,METH_VARARGS,(char *)decode_ctl_set_latency_docstring_ },
    
    
        { "py_lock",(PyCFunction)decode_ctl_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)decode_ctl_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *decode_ctl_docstring_ = "";

PyTypeObject decode_ctl_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.decode_ctl",/*tp_name*/
    sizeof(decode_ctl_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)decode_ctl_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)decode_ctl_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    decode_ctl_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)decode_ctl_init_,  /* tp_init */
    0,                         /* tp_alloc */
    decode_ctl_new_,             /* tp_new */
};

PyObject *decode_ctl_wrapcopy_(decode_ctl_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *decode_ctl_wrap_(decode_ctl_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = decode_ctl_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    decode_ctl_object_ *self = 
        PyObject_NEW(decode_ctl_object_, &decode_ctl_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        decode_ctl_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

decode_ctl_type_ *decode_ctl_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &decode_ctl_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected decode_ctl object");
        return 0;
    }

    decode_ctl_type_ *object = static_cast<decode_ctl_type_ *>(((decode_ctl_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null decode_ctl object in unwrap");
        return 0;
    }

    return object;
}

void decode_ctl_insert_(const std::pair< decode_ctl_type_ *, PyObject *> &p)
{




    decode_ctl_wrapped_.insert(p);


}

void decode_ctl_erase_(decode_ctl_type_ *p)
{




    decode_ctl_wrapped_.erase(p);


}

PyObject * decode_ctl_find_(decode_ctl_type_ *p)
{




    PyObject *ret = 0;
    std::map< decode_ctl_type_ *, PyObject *>::iterator where = decode_ctl_wrapped_.find(p);
    if(where != decode_ctl_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct decode_ctl_dispatch_ decode_ctl_realdispatcher_ =
{
    &decode_ctl_typeobject_,
    decode_ctl_wrap_,
    decode_ctl_wrapcopy_,
    decode_ctl_unwrap_,
    decode_ctl_insert_,
    decode_ctl_erase_,
    decode_ctl_find_
};

  

struct decoder_object_ {
    PyObject_HEAD
    decoder_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct decoder_wrapper_: public decoder_type_ 
{

    static void __deleter__(decoder_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    decoder_wrapper_( PyObject *o, PyInterpreterState *i  ,decode_ctl_type_ * a0  ): decoder_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~decoder_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return decoder_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(decoder_object_ *self, PyObject *args)
    {
        

        decoder_wrapper_ *o = (decoder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null decoder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in decoder::cookie");
        }

        return 0;
    }



    clockref_type_ * get_clock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return decoder_type_::get_clock();
                
            
            
        }

        return
            
                get_clock
            
            
                ();
    }

    static PyObject *get_clock_method_(decoder_object_ *self, PyObject *args)
    {
        

        decoder_wrapper_ *o = (decoder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null decoder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    clockref_type_ * r = o->get_clock_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_clockref((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in decoder::get_clock");
        }

        return 0;
    }



    void disconnect_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return decoder_type_::disconnect();
                
            
            
        }

        return
            
                disconnect
            
            
                ();
    }

    static PyObject *disconnect_method_(decoder_object_ *self, PyObject *args)
    {
        

        decoder_wrapper_ *o = (decoder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null decoder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->disconnect_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in decoder::disconnect");
        }

        return 0;
    }



    unsigned int get_latency_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return decoder_type_::get_latency();
                
            
            
        }

        return
            
                get_latency
            
            
                ();
    }

    static PyObject *get_latency_method_(decoder_object_ *self, PyObject *args)
    {
        

        decoder_wrapper_ *o = (decoder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null decoder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_latency_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in decoder::get_latency");
        }

        return 0;
    }



    void shutdown_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return decoder_type_::shutdown();
                
            
            
        }

        return
            
                shutdown
            
            
                ();
    }

    static PyObject *shutdown_method_(decoder_object_ *self, PyObject *args)
    {
        

        decoder_wrapper_ *o = (decoder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null decoder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->shutdown_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in decoder::shutdown");
        }

        return 0;
    }





    static PyObject *_method_lock(decoder_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(decoder_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void decoder_delete_(decoder_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            decoder_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (decoder_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        decoder_wrapper_::__deleter__((decoder_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void decoder_dealloc_(decoder_object_* self)
{
    
    decoder_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * decoder_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    decoder_object_ *self;

    if(!(self = (decoder_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int decoder_init_(decoder_object_ *self, PyObject *args, PyObject *kwds)
{
    decoder_delete_(self);

    

    {
         decode_ctl_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_decode_ctl,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new decoder_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing decoder");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            decoder_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"decoder invalid arguments");
    return -1;

}


static const char *decoder_cookie_docstring_ = "";

static const char *decoder_get_clock_docstring_ = "";

static const char *decoder_disconnect_docstring_ = "";

static const char *decoder_get_latency_docstring_ = "";

static const char *decoder_shutdown_docstring_ = "";




static PyMethodDef decoder_methods_[] = {
    
        { "cookie",(PyCFunction)decoder_wrapper_::cookie_method_,METH_VARARGS,(char *)decoder_cookie_docstring_ },
    
        { "get_clock",(PyCFunction)decoder_wrapper_::get_clock_method_,METH_VARARGS,(char *)decoder_get_clock_docstring_ },
    
        { "disconnect",(PyCFunction)decoder_wrapper_::disconnect_method_,METH_VARARGS,(char *)decoder_disconnect_docstring_ },
    
        { "get_latency",(PyCFunction)decoder_wrapper_::get_latency_method_,METH_VARARGS,(char *)decoder_get_latency_docstring_ },
    
        { "shutdown",(PyCFunction)decoder_wrapper_::shutdown_method_,METH_VARARGS,(char *)decoder_shutdown_docstring_ },
    
    
        { "py_lock",(PyCFunction)decoder_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)decoder_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *decoder_docstring_ = "";

PyTypeObject decoder_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.decoder",/*tp_name*/
    sizeof(decoder_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)decoder_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)decoder_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    decoder_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)decoder_init_,  /* tp_init */
    0,                         /* tp_alloc */
    decoder_new_,             /* tp_new */
};

PyObject *decoder_wrapcopy_(decoder_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *decoder_wrap_(decoder_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = decoder_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    decoder_object_ *self = 
        PyObject_NEW(decoder_object_, &decoder_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        decoder_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

decoder_type_ *decoder_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &decoder_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected decoder object");
        return 0;
    }

    decoder_type_ *object = static_cast<decoder_type_ *>(((decoder_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null decoder object in unwrap");
        return 0;
    }

    return object;
}

void decoder_insert_(const std::pair< decoder_type_ *, PyObject *> &p)
{




    decoder_wrapped_.insert(p);


}

void decoder_erase_(decoder_type_ *p)
{




    decoder_wrapped_.erase(p);


}

PyObject * decoder_find_(decoder_type_ *p)
{




    PyObject *ret = 0;
    std::map< decoder_type_ *, PyObject *>::iterator where = decoder_wrapped_.find(p);
    if(where != decoder_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct decoder_dispatch_ decoder_realdispatcher_ =
{
    &decoder_typeobject_,
    decoder_wrap_,
    decoder_wrapcopy_,
    decoder_unwrap_,
    decoder_insert_,
    decoder_erase_,
    decoder_find_
};

  

struct event_data_buffer_object_ {
    PyObject_HEAD
    event_data_buffer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct event_data_buffer_wrapper_: public event_data_buffer_type_ 
{

    static void __deleter__(event_data_buffer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    event_data_buffer_wrapper_( PyObject *o, PyInterpreterState *i  ): event_data_buffer_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~event_data_buffer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void set_signal_call_(bool owned ,unsigned int a0 ,const dataqueue_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return event_data_buffer_type_::set_signal( a0  ,a1 );
                
            
            
        }

        return
            
                set_signal
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_signal_method_(event_data_buffer_object_ *self, PyObject *args)
    {
         unsigned int a0;  dataqueue_type_ * a1; 

        event_data_buffer_wrapper_ *o = (event_data_buffer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null event_data_buffer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_dataqueue_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_signal_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in event_data_buffer::set_signal");
        }

        return 0;
    }





    static PyObject *_method_lock(event_data_buffer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(event_data_buffer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void event_data_buffer_delete_(event_data_buffer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            event_data_buffer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (event_data_buffer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        event_data_buffer_wrapper_::__deleter__((event_data_buffer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void event_data_buffer_dealloc_(event_data_buffer_object_* self)
{
    
    event_data_buffer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * event_data_buffer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    event_data_buffer_object_ *self;

    if(!(self = (event_data_buffer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int event_data_buffer_init_(event_data_buffer_object_ *self, PyObject *args, PyObject *kwds)
{
    event_data_buffer_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new event_data_buffer_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing event_data_buffer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            event_data_buffer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"event_data_buffer invalid arguments");
    return -1;

}


static const char *event_data_buffer_set_signal_docstring_ = "";




static PyMethodDef event_data_buffer_methods_[] = {
    
        { "set_signal",(PyCFunction)event_data_buffer_wrapper_::set_signal_method_,METH_VARARGS,(char *)event_data_buffer_set_signal_docstring_ },
    
    
        { "py_lock",(PyCFunction)event_data_buffer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)event_data_buffer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *event_data_buffer_docstring_ = "";

PyTypeObject event_data_buffer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.event_data_buffer",/*tp_name*/
    sizeof(event_data_buffer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)event_data_buffer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)event_data_buffer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    event_data_buffer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)event_data_buffer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    event_data_buffer_new_,             /* tp_new */
};

PyObject *event_data_buffer_wrapcopy_(event_data_buffer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *event_data_buffer_wrap_(event_data_buffer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = event_data_buffer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    event_data_buffer_object_ *self = 
        PyObject_NEW(event_data_buffer_object_, &event_data_buffer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        event_data_buffer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

event_data_buffer_type_ *event_data_buffer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &event_data_buffer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected event_data_buffer object");
        return 0;
    }

    event_data_buffer_type_ *object = static_cast<event_data_buffer_type_ *>(((event_data_buffer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null event_data_buffer object in unwrap");
        return 0;
    }

    return object;
}

void event_data_buffer_insert_(const std::pair< event_data_buffer_type_ *, PyObject *> &p)
{




    event_data_buffer_wrapped_.insert(p);


}

void event_data_buffer_erase_(event_data_buffer_type_ *p)
{




    event_data_buffer_wrapped_.erase(p);


}

PyObject * event_data_buffer_find_(event_data_buffer_type_ *p)
{




    PyObject *ret = 0;
    std::map< event_data_buffer_type_ *, PyObject *>::iterator where = event_data_buffer_wrapped_.find(p);
    if(where != event_data_buffer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct event_data_buffer_dispatch_ event_data_buffer_realdispatcher_ =
{
    &event_data_buffer_typeobject_,
    event_data_buffer_wrap_,
    event_data_buffer_wrapcopy_,
    event_data_buffer_unwrap_,
    event_data_buffer_insert_,
    event_data_buffer_erase_,
    event_data_buffer_find_
};

  

struct event_data_source_object_ {
    PyObject_HEAD
    event_data_source_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct event_data_source_wrapper_: public event_data_source_type_ 
{

    static void __deleter__(event_data_source_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    event_data_source_wrapper_( PyObject *o, PyInterpreterState *i  ,const event_data_source_type_ & a0  ): event_data_source_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~event_data_source_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(event_data_source_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(event_data_source_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void event_data_source_delete_(event_data_source_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            event_data_source_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (event_data_source_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        event_data_source_wrapper_::__deleter__((event_data_source_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void event_data_source_dealloc_(event_data_source_object_* self)
{
    
    event_data_source_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * event_data_source_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    event_data_source_object_ *self;

    if(!(self = (event_data_source_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int event_data_source_init_(event_data_source_object_ *self, PyObject *args, PyObject *kwds)
{
    event_data_source_delete_(self);

    

    {
         event_data_source_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_event_data_source_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new event_data_source_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing event_data_source");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            event_data_source_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"event_data_source invalid arguments");
    return -1;

}





static PyMethodDef event_data_source_methods_[] = {
    
    
        { "py_lock",(PyCFunction)event_data_source_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)event_data_source_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *event_data_source_docstring_ = "";

PyTypeObject event_data_source_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.event_data_source",/*tp_name*/
    sizeof(event_data_source_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)event_data_source_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)event_data_source_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    event_data_source_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)event_data_source_init_,  /* tp_init */
    0,                         /* tp_alloc */
    event_data_source_new_,             /* tp_new */
};

PyObject *event_data_source_wrapcopy_(event_data_source_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            event_data_source_object_ *obj = 
                PyObject_NEW(event_data_source_object_, event_data_source_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new event_data_source_type_(*o);
                }

                try
                {
                    event_data_source_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying event_data_source");
                return 0;
            }
        

    
}

PyObject *event_data_source_wrap_(event_data_source_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = event_data_source_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    event_data_source_object_ *self = 
        PyObject_NEW(event_data_source_object_, &event_data_source_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        event_data_source_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

event_data_source_type_ *event_data_source_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &event_data_source_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected event_data_source object");
        return 0;
    }

    event_data_source_type_ *object = static_cast<event_data_source_type_ *>(((event_data_source_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null event_data_source object in unwrap");
        return 0;
    }

    return object;
}

void event_data_source_insert_(const std::pair< event_data_source_type_ *, PyObject *> &p)
{




    event_data_source_wrapped_.insert(p);


}

void event_data_source_erase_(event_data_source_type_ *p)
{




    event_data_source_wrapped_.erase(p);


}

PyObject * event_data_source_find_(event_data_source_type_ *p)
{




    PyObject *ret = 0;
    std::map< event_data_source_type_ *, PyObject *>::iterator where = event_data_source_wrapped_.find(p);
    if(where != event_data_source_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct event_data_source_dispatch_ event_data_source_realdispatcher_ =
{
    &event_data_source_typeobject_,
    event_data_source_wrap_,
    event_data_source_wrapcopy_,
    event_data_source_unwrap_,
    event_data_source_insert_,
    event_data_source_erase_,
    event_data_source_find_
};

  

struct event_data_source_real_object_ {
    PyObject_HEAD
    event_data_source_real_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct event_data_source_real_wrapper_: public event_data_source_real_type_ 
{

    static void __deleter__(event_data_source_real_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    event_data_source_real_wrapper_( PyObject *o, PyInterpreterState *i  ,const data_type_ & a0  ): event_data_source_real_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~event_data_source_real_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    event_data_source_type_ source_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return event_data_source_real_type_::source();
                
            
            
        }

        return
            
                source
            
            
                ();
    }

    static PyObject *source_method_(event_data_source_real_object_ *self, PyObject *args)
    {
        

        event_data_source_real_wrapper_ *o = (event_data_source_real_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null event_data_source_real c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    event_data_source_type_ r = o->source_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_event_data_source_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in event_data_source_real::source");
        }

        return 0;
    }



    void start_slow_call_(bool owned ,const data_type_ & a0 ,const event_data_buffer_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return event_data_source_real_type_::start_slow( a0  ,a1 );
                
            
            
        }

        return
            
                start_slow
            
            
                ( a0  ,a1 );
    }

    static PyObject *start_slow_method_(event_data_source_real_object_ *self, PyObject *args)
    {
         data_type_ * a0;  event_data_buffer_type_ * a1; 

        event_data_source_real_wrapper_ *o = (event_data_source_real_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null event_data_source_real c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_data_copy,&a0  ,fpcvt_event_data_buffer_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->start_slow_call_ (self->owned ,*a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in event_data_source_real::start_slow");
        }

        return 0;
    }



    void end_slow_call_(bool owned ,unsigned long long a0 )
    {
        if(owned)
        {
            
                
                
                    return event_data_source_real_type_::end_slow( a0 );
                
            
            
        }

        return
            
                end_slow
            
            
                ( a0 );
    }

    static PyObject *end_slow_method_(event_data_source_real_object_ *self, PyObject *args)
    {
         unsigned long long a0; 

        event_data_source_real_wrapper_ *o = (event_data_source_real_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null event_data_source_real c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ull,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->end_slow_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in event_data_source_real::end_slow");
        }

        return 0;
    }





    static PyObject *_method_lock(event_data_source_real_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(event_data_source_real_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void event_data_source_real_delete_(event_data_source_real_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            event_data_source_real_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (event_data_source_real_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        event_data_source_real_wrapper_::__deleter__((event_data_source_real_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void event_data_source_real_dealloc_(event_data_source_real_object_* self)
{
    
    event_data_source_real_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * event_data_source_real_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    event_data_source_real_object_ *self;

    if(!(self = (event_data_source_real_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int event_data_source_real_init_(event_data_source_real_object_ *self, PyObject *args, PyObject *kwds)
{
    event_data_source_real_delete_(self);

    

    {
         data_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_data_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new event_data_source_real_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing event_data_source_real");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            event_data_source_real_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"event_data_source_real invalid arguments");
    return -1;

}


static const char *event_data_source_real_source_docstring_ = "";

static const char *event_data_source_real_start_slow_docstring_ = "";

static const char *event_data_source_real_end_slow_docstring_ = "";




static PyMethodDef event_data_source_real_methods_[] = {
    
        { "source",(PyCFunction)event_data_source_real_wrapper_::source_method_,METH_VARARGS,(char *)event_data_source_real_source_docstring_ },
    
        { "start_slow",(PyCFunction)event_data_source_real_wrapper_::start_slow_method_,METH_VARARGS,(char *)event_data_source_real_start_slow_docstring_ },
    
        { "end_slow",(PyCFunction)event_data_source_real_wrapper_::end_slow_method_,METH_VARARGS,(char *)event_data_source_real_end_slow_docstring_ },
    
    
        { "py_lock",(PyCFunction)event_data_source_real_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)event_data_source_real_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *event_data_source_real_docstring_ = "";

PyTypeObject event_data_source_real_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.event_data_source_real",/*tp_name*/
    sizeof(event_data_source_real_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)event_data_source_real_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)event_data_source_real_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    event_data_source_real_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)event_data_source_real_init_,  /* tp_init */
    0,                         /* tp_alloc */
    event_data_source_real_new_,             /* tp_new */
};

PyObject *event_data_source_real_wrapcopy_(event_data_source_real_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *event_data_source_real_wrap_(event_data_source_real_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = event_data_source_real_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    event_data_source_real_object_ *self = 
        PyObject_NEW(event_data_source_real_object_, &event_data_source_real_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        event_data_source_real_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

event_data_source_real_type_ *event_data_source_real_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &event_data_source_real_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected event_data_source_real object");
        return 0;
    }

    event_data_source_real_type_ *object = static_cast<event_data_source_real_type_ *>(((event_data_source_real_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null event_data_source_real object in unwrap");
        return 0;
    }

    return object;
}

void event_data_source_real_insert_(const std::pair< event_data_source_real_type_ *, PyObject *> &p)
{




    event_data_source_real_wrapped_.insert(p);


}

void event_data_source_real_erase_(event_data_source_real_type_ *p)
{




    event_data_source_real_wrapped_.erase(p);


}

PyObject * event_data_source_real_find_(event_data_source_real_type_ *p)
{




    PyObject *ret = 0;
    std::map< event_data_source_real_type_ *, PyObject *>::iterator where = event_data_source_real_wrapped_.find(p);
    if(where != event_data_source_real_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct event_data_source_real_dispatch_ event_data_source_real_realdispatcher_ =
{
    &event_data_source_real_typeobject_,
    event_data_source_real_wrap_,
    event_data_source_real_wrapcopy_,
    event_data_source_real_unwrap_,
    event_data_source_real_insert_,
    event_data_source_real_erase_,
    event_data_source_real_find_
};

  

struct wire_ctl_object_ {
    PyObject_HEAD
    wire_ctl_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct wire_ctl_wrapper_: public wire_ctl_type_ 
{

    static void __deleter__(wire_ctl_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    wire_ctl_wrapper_( PyObject *o, PyInterpreterState *i  ): wire_ctl_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~wire_ctl_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void disconnect_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return wire_ctl_type_::disconnect();
                
            
            
        }

        return
            
                disconnect
            
            
                ();
    }

    static PyObject *disconnect_method_(wire_ctl_object_ *self, PyObject *args)
    {
        

        wire_ctl_wrapper_ *o = (wire_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null wire_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->disconnect_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in wire_ctl::disconnect");
        }

        return 0;
    }



    wire_type_ * get_wire_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return wire_ctl_type_::get_wire();
                
            
            
        }

        return
            
                get_wire
            
            
                ();
    }

    static PyObject *get_wire_method_(wire_ctl_object_ *self, PyObject *args)
    {
        

        wire_ctl_wrapper_ *o = (wire_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null wire_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    wire_type_ * r = o->get_wire_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_wire((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in wire_ctl::get_wire");
        }

        return 0;
    }



    void set_wire_call_(bool owned ,wire_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return wire_ctl_type_::set_wire( a0 );
                
            
            
        }

        return
            
                set_wire
            
            
                ( a0 );
    }

    static PyObject *set_wire_method_(wire_ctl_object_ *self, PyObject *args)
    {
         wire_type_ * a0; 

        wire_ctl_wrapper_ *o = (wire_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null wire_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_wire,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_wire_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in wire_ctl::set_wire");
        }

        return 0;
    }





    static PyObject *_method_lock(wire_ctl_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(wire_ctl_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void wire_ctl_delete_(wire_ctl_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            wire_ctl_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (wire_ctl_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        wire_ctl_wrapper_::__deleter__((wire_ctl_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void wire_ctl_dealloc_(wire_ctl_object_* self)
{
    
    wire_ctl_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * wire_ctl_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    wire_ctl_object_ *self;

    if(!(self = (wire_ctl_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int wire_ctl_init_(wire_ctl_object_ *self, PyObject *args, PyObject *kwds)
{
    wire_ctl_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new wire_ctl_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing wire_ctl");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            wire_ctl_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"wire_ctl invalid arguments");
    return -1;

}


static const char *wire_ctl_disconnect_docstring_ = "";

static const char *wire_ctl_get_wire_docstring_ = "";

static const char *wire_ctl_set_wire_docstring_ = "";




static PyMethodDef wire_ctl_methods_[] = {
    
        { "disconnect",(PyCFunction)wire_ctl_wrapper_::disconnect_method_,METH_VARARGS,(char *)wire_ctl_disconnect_docstring_ },
    
        { "get_wire",(PyCFunction)wire_ctl_wrapper_::get_wire_method_,METH_VARARGS,(char *)wire_ctl_get_wire_docstring_ },
    
        { "set_wire",(PyCFunction)wire_ctl_wrapper_::set_wire_method_,METH_VARARGS,(char *)wire_ctl_set_wire_docstring_ },
    
    
        { "py_lock",(PyCFunction)wire_ctl_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)wire_ctl_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *wire_ctl_docstring_ = "";

PyTypeObject wire_ctl_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.wire_ctl",/*tp_name*/
    sizeof(wire_ctl_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)wire_ctl_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)wire_ctl_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    wire_ctl_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)wire_ctl_init_,  /* tp_init */
    0,                         /* tp_alloc */
    wire_ctl_new_,             /* tp_new */
};

PyObject *wire_ctl_wrapcopy_(wire_ctl_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *wire_ctl_wrap_(wire_ctl_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = wire_ctl_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    wire_ctl_object_ *self = 
        PyObject_NEW(wire_ctl_object_, &wire_ctl_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        wire_ctl_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

wire_ctl_type_ *wire_ctl_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &wire_ctl_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected wire_ctl object");
        return 0;
    }

    wire_ctl_type_ *object = static_cast<wire_ctl_type_ *>(((wire_ctl_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null wire_ctl object in unwrap");
        return 0;
    }

    return object;
}

void wire_ctl_insert_(const std::pair< wire_ctl_type_ *, PyObject *> &p)
{




    wire_ctl_wrapped_.insert(p);


}

void wire_ctl_erase_(wire_ctl_type_ *p)
{




    wire_ctl_wrapped_.erase(p);


}

PyObject * wire_ctl_find_(wire_ctl_type_ *p)
{




    PyObject *ret = 0;
    std::map< wire_ctl_type_ *, PyObject *>::iterator where = wire_ctl_wrapped_.find(p);
    if(where != wire_ctl_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct wire_ctl_dispatch_ wire_ctl_realdispatcher_ =
{
    &wire_ctl_typeobject_,
    wire_ctl_wrap_,
    wire_ctl_wrapcopy_,
    wire_ctl_unwrap_,
    wire_ctl_insert_,
    wire_ctl_erase_,
    wire_ctl_find_
};

  

struct root_ctl_object_ {
    PyObject_HEAD
    root_ctl_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct root_ctl_wrapper_: public root_ctl_type_ 
{

    static void __deleter__(root_ctl_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    root_ctl_wrapper_( PyObject *o, PyInterpreterState *i  ): root_ctl_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~root_ctl_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void connect_call_(bool owned ,const cookie_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return root_ctl_type_::connect( a0 );
                
            
            
        }

        return
            
                connect
            
            
                ( a0 );
    }

    static PyObject *connect_method_(root_ctl_object_ *self, PyObject *args)
    {
         cookie_type_ * a0; 

        root_ctl_wrapper_ *o = (root_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_cookie_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->connect_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root_ctl::connect");
        }

        return 0;
    }



    void connect_wire_call_(bool owned ,wire_ctl_type_ * a0 ,const event_data_source_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return root_ctl_type_::connect_wire( a0  ,a1 );
                
            
            
        }

        return
            
                connect_wire
            
            
                ( a0  ,a1 );
    }

    static PyObject *connect_wire_method_(root_ctl_object_ *self, PyObject *args)
    {
         wire_ctl_type_ * a0;  event_data_source_type_ * a1; 

        root_ctl_wrapper_ *o = (root_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_wire_ctl,&a0  ,fpcvt_event_data_source_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->connect_wire_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root_ctl::connect_wire");
        }

        return 0;
    }



    void disconnect_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return root_ctl_type_::disconnect();
                
            
            
        }

        return
            
                disconnect
            
            
                ();
    }

    static PyObject *disconnect_method_(root_ctl_object_ *self, PyObject *args)
    {
        

        root_ctl_wrapper_ *o = (root_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->disconnect_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root_ctl::disconnect");
        }

        return 0;
    }



    void set_clock_call_(bool owned ,clockref_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return root_ctl_type_::set_clock( a0 );
                
            
            
        }

        return
            
                set_clock
            
            
                ( a0 );
    }

    static PyObject *set_clock_method_(root_ctl_object_ *self, PyObject *args)
    {
         clockref_type_ * a0; 

        root_ctl_wrapper_ *o = (root_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_clockref,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_clock_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root_ctl::set_clock");
        }

        return 0;
    }



    void set_latency_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return root_ctl_type_::set_latency( a0 );
                
            
            
        }

        return
            
                set_latency
            
            
                ( a0 );
    }

    static PyObject *set_latency_method_(root_ctl_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        root_ctl_wrapper_ *o = (root_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_latency_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root_ctl::set_latency");
        }

        return 0;
    }





    static PyObject *_method_lock(root_ctl_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(root_ctl_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void root_ctl_delete_(root_ctl_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            root_ctl_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (root_ctl_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        root_ctl_wrapper_::__deleter__((root_ctl_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void root_ctl_dealloc_(root_ctl_object_* self)
{
    
    root_ctl_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * root_ctl_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    root_ctl_object_ *self;

    if(!(self = (root_ctl_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int root_ctl_init_(root_ctl_object_ *self, PyObject *args, PyObject *kwds)
{
    root_ctl_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new root_ctl_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing root_ctl");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            root_ctl_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"root_ctl invalid arguments");
    return -1;

}


static const char *root_ctl_connect_docstring_ = "";

static const char *root_ctl_connect_wire_docstring_ = "";

static const char *root_ctl_disconnect_docstring_ = "";

static const char *root_ctl_set_clock_docstring_ = "";

static const char *root_ctl_set_latency_docstring_ = "";




static PyMethodDef root_ctl_methods_[] = {
    
        { "connect",(PyCFunction)root_ctl_wrapper_::connect_method_,METH_VARARGS,(char *)root_ctl_connect_docstring_ },
    
        { "connect_wire",(PyCFunction)root_ctl_wrapper_::connect_wire_method_,METH_VARARGS,(char *)root_ctl_connect_wire_docstring_ },
    
        { "disconnect",(PyCFunction)root_ctl_wrapper_::disconnect_method_,METH_VARARGS,(char *)root_ctl_disconnect_docstring_ },
    
        { "set_clock",(PyCFunction)root_ctl_wrapper_::set_clock_method_,METH_VARARGS,(char *)root_ctl_set_clock_docstring_ },
    
        { "set_latency",(PyCFunction)root_ctl_wrapper_::set_latency_method_,METH_VARARGS,(char *)root_ctl_set_latency_docstring_ },
    
    
        { "py_lock",(PyCFunction)root_ctl_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)root_ctl_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *root_ctl_docstring_ = "";

PyTypeObject root_ctl_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.root_ctl",/*tp_name*/
    sizeof(root_ctl_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)root_ctl_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)root_ctl_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    root_ctl_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)root_ctl_init_,  /* tp_init */
    0,                         /* tp_alloc */
    root_ctl_new_,             /* tp_new */
};

PyObject *root_ctl_wrapcopy_(root_ctl_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *root_ctl_wrap_(root_ctl_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = root_ctl_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    root_ctl_object_ *self = 
        PyObject_NEW(root_ctl_object_, &root_ctl_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        root_ctl_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

root_ctl_type_ *root_ctl_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &root_ctl_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected root_ctl object");
        return 0;
    }

    root_ctl_type_ *object = static_cast<root_ctl_type_ *>(((root_ctl_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null root_ctl object in unwrap");
        return 0;
    }

    return object;
}

void root_ctl_insert_(const std::pair< root_ctl_type_ *, PyObject *> &p)
{




    root_ctl_wrapped_.insert(p);


}

void root_ctl_erase_(root_ctl_type_ *p)
{




    root_ctl_wrapped_.erase(p);


}

PyObject * root_ctl_find_(root_ctl_type_ *p)
{




    PyObject *ret = 0;
    std::map< root_ctl_type_ *, PyObject *>::iterator where = root_ctl_wrapped_.find(p);
    if(where != root_ctl_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct root_ctl_dispatch_ root_ctl_realdispatcher_ =
{
    &root_ctl_typeobject_,
    root_ctl_wrap_,
    root_ctl_wrapcopy_,
    root_ctl_unwrap_,
    root_ctl_insert_,
    root_ctl_erase_,
    root_ctl_find_
};

  

struct wire_object_ {
    PyObject_HEAD
    wire_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct wire_wrapper_: public wire_type_ 
{

    static void __deleter__(wire_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
    }
*/



    wire_wrapper_( PyObject *o, PyInterpreterState *i  ): wire_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~wire_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void wire_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"wire_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("wire_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("wire_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    void disconnect_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return wire_type_::disconnect();
                
            
            
        }

        return
            
                disconnect
            
            
                ();
    }

    static PyObject *disconnect_method_(wire_object_ *self, PyObject *args)
    {
        

        wire_wrapper_ *o = (wire_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null wire c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->disconnect_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in wire::disconnect");
        }

        return 0;
    }



    void wire_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method wire::wire_closed");
                
                
            
            
        }

        return
            
                wire_closed
            
            
                ();
    }

    static PyObject *wire_closed_method_(wire_object_ *self, PyObject *args)
    {
        

        wire_wrapper_ *o = (wire_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null wire c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->wire_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in wire::wire_closed");
        }

        return 0;
    }





    static PyObject *_method_lock(wire_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(wire_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void wire_delete_(wire_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            wire_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (wire_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        wire_wrapper_::__deleter__((wire_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void wire_dealloc_(wire_object_* self)
{
    
    wire_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * wire_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    wire_object_ *self;

    if(!(self = (wire_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int wire_init_(wire_object_ *self, PyObject *args, PyObject *kwds)
{
    wire_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new wire_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing wire");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            wire_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"wire invalid arguments");
    return -1;

}


static const char *wire_disconnect_docstring_ = "";

static const char *wire_wire_closed_docstring_ = "";




static PyMethodDef wire_methods_[] = {
    
        { "disconnect",(PyCFunction)wire_wrapper_::disconnect_method_,METH_VARARGS,(char *)wire_disconnect_docstring_ },
    
        { "wire_closed",(PyCFunction)wire_wrapper_::wire_closed_method_,METH_VARARGS,(char *)wire_wire_closed_docstring_ },
    
    
        { "py_lock",(PyCFunction)wire_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)wire_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *wire_docstring_ = "";

PyTypeObject wire_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.wire",/*tp_name*/
    sizeof(wire_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)wire_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)wire_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    wire_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)wire_init_,  /* tp_init */
    0,                         /* tp_alloc */
    wire_new_,             /* tp_new */
};

PyObject *wire_wrapcopy_(wire_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *wire_wrap_(wire_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = wire_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    wire_object_ *self = 
        PyObject_NEW(wire_object_, &wire_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        wire_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

wire_type_ *wire_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &wire_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected wire object");
        return 0;
    }

    wire_type_ *object = static_cast<wire_type_ *>(((wire_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null wire object in unwrap");
        return 0;
    }

    return object;
}

void wire_insert_(const std::pair< wire_type_ *, PyObject *> &p)
{




    wire_wrapped_.insert(p);


}

void wire_erase_(wire_type_ *p)
{




    wire_wrapped_.erase(p);


}

PyObject * wire_find_(wire_type_ *p)
{




    PyObject *ret = 0;
    std::map< wire_type_ *, PyObject *>::iterator where = wire_wrapped_.find(p);
    if(where != wire_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct wire_dispatch_ wire_realdispatcher_ =
{
    &wire_typeobject_,
    wire_wrap_,
    wire_wrapcopy_,
    wire_unwrap_,
    wire_insert_,
    wire_erase_,
    wire_find_
};

  

struct root_object_ {
    PyObject_HEAD
    root_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct root_wrapper_: public root_type_ 
{

    static void __deleter__(root_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
        cache_4_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
        if(cache_4_)
        {
            Py_DECREF(cache_4_);
            cache_4_ = 0;
        }
    
    }
*/



    root_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned int a0  ): root_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~root_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    wire_type_ * root_wire(  const event_data_source_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"root_wire");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_event_data_source_copy,&a0);
        //printf("root_wire before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("root_wire after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            wire_type_ * rptr;

            if(!fpcvt_wire(result,&rptr))
            {
                throw pip_err_t("wrong type result converting wire");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        wire_type_ * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_1_;
 
    void root_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"root_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("root_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("root_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void root_clock(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"root_clock");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("root_clock before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("root_clock after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    void root_opened(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"root_opened");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("root_opened before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("root_opened after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_4_;
 
    void root_latency(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_4_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"root_latency");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_4_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("root_latency before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("root_latency after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    void disconnect_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return root_type_::disconnect();
                
            
            
        }

        return
            
                disconnect
            
            
                ();
    }

    static PyObject *disconnect_method_(root_object_ *self, PyObject *args)
    {
        

        root_wrapper_ *o = (root_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->disconnect_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root::disconnect");
        }

        return 0;
    }



    clockref_type_ * get_clock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return root_type_::get_clock();
                
            
            
        }

        return
            
                get_clock
            
            
                ();
    }

    static PyObject *get_clock_method_(root_object_ *self, PyObject *args)
    {
        

        root_wrapper_ *o = (root_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    clockref_type_ * r = o->get_clock_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_clockref((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root::get_clock");
        }

        return 0;
    }



    unsigned int get_latency_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return root_type_::get_latency();
                
            
            
        }

        return
            
                get_latency
            
            
                ();
    }

    static PyObject *get_latency_method_(root_object_ *self, PyObject *args)
    {
        

        root_wrapper_ *o = (root_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_latency_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root::get_latency");
        }

        return 0;
    }



    wire_type_ * root_wire_call_(bool owned ,const event_data_source_type_ & a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method root::root_wire");
                
                
            
            
        }

        return
            
                root_wire
            
            
                ( a0 );
    }

    static PyObject *root_wire_method_(root_object_ *self, PyObject *args)
    {
         event_data_source_type_ * a0; 

        root_wrapper_ *o = (root_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_event_data_source_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    wire_type_ * r = o->root_wire_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_wire((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root::root_wire");
        }

        return 0;
    }



    void root_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method root::root_closed");
                
                
            
            
        }

        return
            
                root_closed
            
            
                ();
    }

    static PyObject *root_closed_method_(root_object_ *self, PyObject *args)
    {
        

        root_wrapper_ *o = (root_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->root_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root::root_closed");
        }

        return 0;
    }



    void root_clock_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method root::root_clock");
                
                
            
            
        }

        return
            
                root_clock
            
            
                ();
    }

    static PyObject *root_clock_method_(root_object_ *self, PyObject *args)
    {
        

        root_wrapper_ *o = (root_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->root_clock_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root::root_clock");
        }

        return 0;
    }



    void root_opened_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method root::root_opened");
                
                
            
            
        }

        return
            
                root_opened
            
            
                ();
    }

    static PyObject *root_opened_method_(root_object_ *self, PyObject *args)
    {
        

        root_wrapper_ *o = (root_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->root_opened_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root::root_opened");
        }

        return 0;
    }



    void root_latency_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method root::root_latency");
                
                
            
            
        }

        return
            
                root_latency
            
            
                ();
    }

    static PyObject *root_latency_method_(root_object_ *self, PyObject *args)
    {
        

        root_wrapper_ *o = (root_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null root c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->root_latency_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in root::root_latency");
        }

        return 0;
    }





    static PyObject *_method_lock(root_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(root_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void root_delete_(root_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            root_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (root_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        root_wrapper_::__deleter__((root_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void root_dealloc_(root_object_* self)
{
    
    root_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * root_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    root_object_ *self;

    if(!(self = (root_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int root_init_(root_object_ *self, PyObject *args, PyObject *kwds)
{
    root_delete_(self);

    

    {
         unsigned int a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new root_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing root");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            root_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"root invalid arguments");
    return -1;

}


static const char *root_disconnect_docstring_ = "";

static const char *root_get_clock_docstring_ = "";

static const char *root_get_latency_docstring_ = "";

static const char *root_root_wire_docstring_ = "";

static const char *root_root_closed_docstring_ = "";

static const char *root_root_clock_docstring_ = "";

static const char *root_root_opened_docstring_ = "";

static const char *root_root_latency_docstring_ = "";




static PyMethodDef root_methods_[] = {
    
        { "disconnect",(PyCFunction)root_wrapper_::disconnect_method_,METH_VARARGS,(char *)root_disconnect_docstring_ },
    
        { "get_clock",(PyCFunction)root_wrapper_::get_clock_method_,METH_VARARGS,(char *)root_get_clock_docstring_ },
    
        { "get_latency",(PyCFunction)root_wrapper_::get_latency_method_,METH_VARARGS,(char *)root_get_latency_docstring_ },
    
        { "root_wire",(PyCFunction)root_wrapper_::root_wire_method_,METH_VARARGS,(char *)root_root_wire_docstring_ },
    
        { "root_closed",(PyCFunction)root_wrapper_::root_closed_method_,METH_VARARGS,(char *)root_root_closed_docstring_ },
    
        { "root_clock",(PyCFunction)root_wrapper_::root_clock_method_,METH_VARARGS,(char *)root_root_clock_docstring_ },
    
        { "root_opened",(PyCFunction)root_wrapper_::root_opened_method_,METH_VARARGS,(char *)root_root_opened_docstring_ },
    
        { "root_latency",(PyCFunction)root_wrapper_::root_latency_method_,METH_VARARGS,(char *)root_root_latency_docstring_ },
    
    
        { "py_lock",(PyCFunction)root_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)root_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *root_docstring_ = "";

PyTypeObject root_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.root",/*tp_name*/
    sizeof(root_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)root_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)root_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    root_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)root_init_,  /* tp_init */
    0,                         /* tp_alloc */
    root_new_,             /* tp_new */
};

PyObject *root_wrapcopy_(root_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *root_wrap_(root_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = root_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    root_object_ *self = 
        PyObject_NEW(root_object_, &root_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        root_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

root_type_ *root_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &root_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected root object");
        return 0;
    }

    root_type_ *object = static_cast<root_type_ *>(((root_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null root object in unwrap");
        return 0;
    }

    return object;
}

void root_insert_(const std::pair< root_type_ *, PyObject *> &p)
{




    root_wrapped_.insert(p);


}

void root_erase_(root_type_ *p)
{




    root_wrapped_.erase(p);


}

PyObject * root_find_(root_type_ *p)
{




    PyObject *ret = 0;
    std::map< root_type_ *, PyObject *>::iterator where = root_wrapped_.find(p);
    if(where != root_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct root_dispatch_ root_realdispatcher_ =
{
    &root_typeobject_,
    root_wrap_,
    root_wrapcopy_,
    root_unwrap_,
    root_insert_,
    root_erase_,
    root_find_
};

  

struct multiplexer_object_ {
    PyObject_HEAD
    multiplexer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct multiplexer_wrapper_: public multiplexer_type_ 
{

    static void __deleter__(multiplexer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    multiplexer_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): multiplexer_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~multiplexer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ get_input_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return multiplexer_type_::get_input( a0 );
                
            
            
        }

        return
            
                get_input
            
            
                ( a0 );
    }

    static PyObject *get_input_method_(multiplexer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        multiplexer_wrapper_ *o = (multiplexer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null multiplexer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->get_input_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in multiplexer::get_input");
        }

        return 0;
    }



    void clear_input_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return multiplexer_type_::clear_input( a0 );
                
            
            
        }

        return
            
                clear_input
            
            
                ( a0 );
    }

    static PyObject *clear_input_method_(multiplexer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        multiplexer_wrapper_ *o = (multiplexer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null multiplexer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_input_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in multiplexer::clear_input");
        }

        return 0;
    }



    change_nb_type_ gate_input_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return multiplexer_type_::gate_input();
                
            
            
        }

        return
            
                gate_input
            
            
                ();
    }

    static PyObject *gate_input_method_(multiplexer_object_ *self, PyObject *args)
    {
        

        multiplexer_wrapper_ *o = (multiplexer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null multiplexer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->gate_input_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in multiplexer::gate_input");
        }

        return 0;
    }





    static PyObject *_method_lock(multiplexer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(multiplexer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void multiplexer_delete_(multiplexer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            multiplexer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (multiplexer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        multiplexer_wrapper_::__deleter__((multiplexer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void multiplexer_dealloc_(multiplexer_object_* self)
{
    
    multiplexer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * multiplexer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    multiplexer_object_ *self;

    if(!(self = (multiplexer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int multiplexer_init_(multiplexer_object_ *self, PyObject *args, PyObject *kwds)
{
    multiplexer_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new multiplexer_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing multiplexer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            multiplexer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"multiplexer invalid arguments");
    return -1;

}


static const char *multiplexer_get_input_docstring_ = "";

static const char *multiplexer_clear_input_docstring_ = "";

static const char *multiplexer_gate_input_docstring_ = "";




static PyMethodDef multiplexer_methods_[] = {
    
        { "get_input",(PyCFunction)multiplexer_wrapper_::get_input_method_,METH_VARARGS,(char *)multiplexer_get_input_docstring_ },
    
        { "clear_input",(PyCFunction)multiplexer_wrapper_::clear_input_method_,METH_VARARGS,(char *)multiplexer_clear_input_docstring_ },
    
        { "gate_input",(PyCFunction)multiplexer_wrapper_::gate_input_method_,METH_VARARGS,(char *)multiplexer_gate_input_docstring_ },
    
    
        { "py_lock",(PyCFunction)multiplexer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)multiplexer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *multiplexer_docstring_ = "";

PyTypeObject multiplexer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.multiplexer",/*tp_name*/
    sizeof(multiplexer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)multiplexer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)multiplexer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    multiplexer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)multiplexer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    multiplexer_new_,             /* tp_new */
};

PyObject *multiplexer_wrapcopy_(multiplexer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *multiplexer_wrap_(multiplexer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = multiplexer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    multiplexer_object_ *self = 
        PyObject_NEW(multiplexer_object_, &multiplexer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        multiplexer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

multiplexer_type_ *multiplexer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &multiplexer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected multiplexer object");
        return 0;
    }

    multiplexer_type_ *object = static_cast<multiplexer_type_ *>(((multiplexer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null multiplexer object in unwrap");
        return 0;
    }

    return object;
}

void multiplexer_insert_(const std::pair< multiplexer_type_ *, PyObject *> &p)
{




    multiplexer_wrapped_.insert(p);


}

void multiplexer_erase_(multiplexer_type_ *p)
{




    multiplexer_wrapped_.erase(p);


}

PyObject * multiplexer_find_(multiplexer_type_ *p)
{




    PyObject *ret = 0;
    std::map< multiplexer_type_ *, PyObject *>::iterator where = multiplexer_wrapped_.find(p);
    if(where != multiplexer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct multiplexer_dispatch_ multiplexer_realdispatcher_ =
{
    &multiplexer_typeobject_,
    multiplexer_wrap_,
    multiplexer_wrapcopy_,
    multiplexer_unwrap_,
    multiplexer_insert_,
    multiplexer_erase_,
    multiplexer_find_
};

  

struct ranger_object_ {
    PyObject_HEAD
    ranger_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct ranger_wrapper_: public ranger_type_ 
{

    static void __deleter__(ranger_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    ranger_wrapper_( PyObject *o, PyInterpreterState *i  ,clockdomain_ctl_type_ * a0  ,const cookie_type_ & a1  ): ranger_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~ranger_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ ctl_cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return ranger_type_::ctl_cookie();
                
            
            
        }

        return
            
                ctl_cookie
            
            
                ();
    }

    static PyObject *ctl_cookie_method_(ranger_object_ *self, PyObject *args)
    {
        

        ranger_wrapper_ *o = (ranger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null ranger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->ctl_cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in ranger::ctl_cookie");
        }

        return 0;
    }



    cookie_type_ data_cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return ranger_type_::data_cookie();
                
            
            
        }

        return
            
                data_cookie
            
            
                ();
    }

    static PyObject *data_cookie_method_(ranger_object_ *self, PyObject *args)
    {
        

        ranger_wrapper_ *o = (ranger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null ranger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->data_cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in ranger::data_cookie");
        }

        return 0;
    }



    void set_sticky_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return ranger_type_::set_sticky( a0 );
                
            
            
        }

        return
            
                set_sticky
            
            
                ( a0 );
    }

    static PyObject *set_sticky_method_(ranger_object_ *self, PyObject *args)
    {
         bool a0; 

        ranger_wrapper_ *o = (ranger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null ranger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_sticky_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in ranger::set_sticky");
        }

        return 0;
    }



    void set_mono_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return ranger_type_::set_mono( a0 );
                
            
            
        }

        return
            
                set_mono
            
            
                ( a0 );
    }

    static PyObject *set_mono_method_(ranger_object_ *self, PyObject *args)
    {
         bool a0; 

        ranger_wrapper_ *o = (ranger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null ranger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_mono_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in ranger::set_mono");
        }

        return 0;
    }



    void set_absolute_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return ranger_type_::set_absolute( a0 );
                
            
            
        }

        return
            
                set_absolute
            
            
                ( a0 );
    }

    static PyObject *set_absolute_method_(ranger_object_ *self, PyObject *args)
    {
         bool a0; 

        ranger_wrapper_ *o = (ranger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null ranger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_absolute_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in ranger::set_absolute");
        }

        return 0;
    }



    void set_curve_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return ranger_type_::set_curve( a0 );
                
            
            
        }

        return
            
                set_curve
            
            
                ( a0 );
    }

    static PyObject *set_curve_method_(ranger_object_ *self, PyObject *args)
    {
         float a0; 

        ranger_wrapper_ *o = (ranger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null ranger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_curve_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in ranger::set_curve");
        }

        return 0;
    }



    void reset_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return ranger_type_::reset();
                
            
            
        }

        return
            
                reset
            
            
                ();
    }

    static PyObject *reset_method_(ranger_object_ *self, PyObject *args)
    {
        

        ranger_wrapper_ *o = (ranger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null ranger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->reset_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in ranger::reset");
        }

        return 0;
    }





    static PyObject *_method_lock(ranger_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(ranger_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void ranger_delete_(ranger_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            ranger_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (ranger_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        ranger_wrapper_::__deleter__((ranger_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void ranger_dealloc_(ranger_object_* self)
{
    
    ranger_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * ranger_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    ranger_object_ *self;

    if(!(self = (ranger_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int ranger_init_(ranger_object_ *self, PyObject *args, PyObject *kwds)
{
    ranger_delete_(self);

    

    {
         clockdomain_ctl_type_ * a0;  cookie_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_clockdomain_ctl,&a0,fpcvt_cookie_copy,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new ranger_wrapper_((PyObject *)self,interp ,a0,*a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing ranger");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            ranger_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"ranger invalid arguments");
    return -1;

}


static const char *ranger_ctl_cookie_docstring_ = "";

static const char *ranger_data_cookie_docstring_ = "";

static const char *ranger_set_sticky_docstring_ = "";

static const char *ranger_set_mono_docstring_ = "";

static const char *ranger_set_absolute_docstring_ = "";

static const char *ranger_set_curve_docstring_ = "";

static const char *ranger_reset_docstring_ = "";




static PyMethodDef ranger_methods_[] = {
    
        { "ctl_cookie",(PyCFunction)ranger_wrapper_::ctl_cookie_method_,METH_VARARGS,(char *)ranger_ctl_cookie_docstring_ },
    
        { "data_cookie",(PyCFunction)ranger_wrapper_::data_cookie_method_,METH_VARARGS,(char *)ranger_data_cookie_docstring_ },
    
        { "set_sticky",(PyCFunction)ranger_wrapper_::set_sticky_method_,METH_VARARGS,(char *)ranger_set_sticky_docstring_ },
    
        { "set_mono",(PyCFunction)ranger_wrapper_::set_mono_method_,METH_VARARGS,(char *)ranger_set_mono_docstring_ },
    
        { "set_absolute",(PyCFunction)ranger_wrapper_::set_absolute_method_,METH_VARARGS,(char *)ranger_set_absolute_docstring_ },
    
        { "set_curve",(PyCFunction)ranger_wrapper_::set_curve_method_,METH_VARARGS,(char *)ranger_set_curve_docstring_ },
    
        { "reset",(PyCFunction)ranger_wrapper_::reset_method_,METH_VARARGS,(char *)ranger_reset_docstring_ },
    
    
        { "py_lock",(PyCFunction)ranger_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)ranger_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *ranger_docstring_ = "";

PyTypeObject ranger_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.ranger",/*tp_name*/
    sizeof(ranger_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)ranger_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)ranger_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    ranger_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)ranger_init_,  /* tp_init */
    0,                         /* tp_alloc */
    ranger_new_,             /* tp_new */
};

PyObject *ranger_wrapcopy_(ranger_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *ranger_wrap_(ranger_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = ranger_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    ranger_object_ *self = 
        PyObject_NEW(ranger_object_, &ranger_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        ranger_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

ranger_type_ *ranger_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &ranger_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected ranger object");
        return 0;
    }

    ranger_type_ *object = static_cast<ranger_type_ *>(((ranger_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null ranger object in unwrap");
        return 0;
    }

    return object;
}

void ranger_insert_(const std::pair< ranger_type_ *, PyObject *> &p)
{




    ranger_wrapped_.insert(p);


}

void ranger_erase_(ranger_type_ *p)
{




    ranger_wrapped_.erase(p);


}

PyObject * ranger_find_(ranger_type_ *p)
{




    PyObject *ret = 0;
    std::map< ranger_type_ *, PyObject *>::iterator where = ranger_wrapped_.find(p);
    if(where != ranger_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct ranger_dispatch_ ranger_realdispatcher_ =
{
    &ranger_typeobject_,
    ranger_wrap_,
    ranger_wrapcopy_,
    ranger_unwrap_,
    ranger_insert_,
    ranger_erase_,
    ranger_find_
};

  

struct lightconvertor_object_ {
    PyObject_HEAD
    lightconvertor_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct lightconvertor_wrapper_: public lightconvertor_type_ 
{

    static void __deleter__(lightconvertor_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    lightconvertor_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ): lightconvertor_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~lightconvertor_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return lightconvertor_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(lightconvertor_object_ *self, PyObject *args)
    {
        

        lightconvertor_wrapper_ *o = (lightconvertor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightconvertor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightconvertor::cookie");
        }

        return 0;
    }



    void set_status_handler_call_(bool owned ,unsigned int a0 ,change_type_ a1 )
    {
        if(owned)
        {
            
                
                
                    return lightconvertor_type_::set_status_handler( a0  ,a1 );
                
            
            
        }

        return
            
                set_status_handler
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_status_handler_method_(lightconvertor_object_ *self, PyObject *args)
    {
         unsigned int a0;  change_type_ * a1; 

        lightconvertor_wrapper_ *o = (lightconvertor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightconvertor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_change_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_status_handler_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightconvertor::set_status_handler");
        }

        return 0;
    }



    void remove_status_handler_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return lightconvertor_type_::remove_status_handler( a0 );
                
            
            
        }

        return
            
                remove_status_handler
            
            
                ( a0 );
    }

    static PyObject *remove_status_handler_method_(lightconvertor_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        lightconvertor_wrapper_ *o = (lightconvertor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightconvertor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->remove_status_handler_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightconvertor::remove_status_handler");
        }

        return 0;
    }



    unsigned char get_status_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return lightconvertor_type_::get_status( a0 );
                
            
            
        }

        return
            
                get_status
            
            
                ( a0 );
    }

    static PyObject *get_status_method_(lightconvertor_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        lightconvertor_wrapper_ *o = (lightconvertor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightconvertor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned char r = o->get_status_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_uc((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightconvertor::get_status");
        }

        return 0;
    }



    void set_default_color_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return lightconvertor_type_::set_default_color( a0  ,a1 );
                
            
            
        }

        return
            
                set_default_color
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_default_color_method_(lightconvertor_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        lightconvertor_wrapper_ *o = (lightconvertor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightconvertor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_default_color_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightconvertor::set_default_color");
        }

        return 0;
    }





    static PyObject *_method_lock(lightconvertor_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(lightconvertor_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void lightconvertor_delete_(lightconvertor_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            lightconvertor_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (lightconvertor_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        lightconvertor_wrapper_::__deleter__((lightconvertor_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void lightconvertor_dealloc_(lightconvertor_object_* self)
{
    
    lightconvertor_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * lightconvertor_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    lightconvertor_object_ *self;

    if(!(self = (lightconvertor_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int lightconvertor_init_(lightconvertor_object_ *self, PyObject *args, PyObject *kwds)
{
    lightconvertor_delete_(self);

    

    {
         cookie_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_cookie_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new lightconvertor_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing lightconvertor");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            lightconvertor_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"lightconvertor invalid arguments");
    return -1;

}


static const char *lightconvertor_cookie_docstring_ = "";

static const char *lightconvertor_set_status_handler_docstring_ = "";

static const char *lightconvertor_remove_status_handler_docstring_ = "";

static const char *lightconvertor_get_status_docstring_ = "";

static const char *lightconvertor_set_default_color_docstring_ = "";




static PyMethodDef lightconvertor_methods_[] = {
    
        { "cookie",(PyCFunction)lightconvertor_wrapper_::cookie_method_,METH_VARARGS,(char *)lightconvertor_cookie_docstring_ },
    
        { "set_status_handler",(PyCFunction)lightconvertor_wrapper_::set_status_handler_method_,METH_VARARGS,(char *)lightconvertor_set_status_handler_docstring_ },
    
        { "remove_status_handler",(PyCFunction)lightconvertor_wrapper_::remove_status_handler_method_,METH_VARARGS,(char *)lightconvertor_remove_status_handler_docstring_ },
    
        { "get_status",(PyCFunction)lightconvertor_wrapper_::get_status_method_,METH_VARARGS,(char *)lightconvertor_get_status_docstring_ },
    
        { "set_default_color",(PyCFunction)lightconvertor_wrapper_::set_default_color_method_,METH_VARARGS,(char *)lightconvertor_set_default_color_docstring_ },
    
    
        { "py_lock",(PyCFunction)lightconvertor_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)lightconvertor_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *lightconvertor_docstring_ = "";

PyTypeObject lightconvertor_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.lightconvertor",/*tp_name*/
    sizeof(lightconvertor_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)lightconvertor_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)lightconvertor_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    lightconvertor_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)lightconvertor_init_,  /* tp_init */
    0,                         /* tp_alloc */
    lightconvertor_new_,             /* tp_new */
};

PyObject *lightconvertor_wrapcopy_(lightconvertor_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *lightconvertor_wrap_(lightconvertor_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = lightconvertor_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    lightconvertor_object_ *self = 
        PyObject_NEW(lightconvertor_object_, &lightconvertor_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        lightconvertor_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

lightconvertor_type_ *lightconvertor_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &lightconvertor_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected lightconvertor object");
        return 0;
    }

    lightconvertor_type_ *object = static_cast<lightconvertor_type_ *>(((lightconvertor_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null lightconvertor object in unwrap");
        return 0;
    }

    return object;
}

void lightconvertor_insert_(const std::pair< lightconvertor_type_ *, PyObject *> &p)
{




    lightconvertor_wrapped_.insert(p);


}

void lightconvertor_erase_(lightconvertor_type_ *p)
{




    lightconvertor_wrapped_.erase(p);


}

PyObject * lightconvertor_find_(lightconvertor_type_ *p)
{




    PyObject *ret = 0;
    std::map< lightconvertor_type_ *, PyObject *>::iterator where = lightconvertor_wrapped_.find(p);
    if(where != lightconvertor_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct lightconvertor_dispatch_ lightconvertor_realdispatcher_ =
{
    &lightconvertor_typeobject_,
    lightconvertor_wrap_,
    lightconvertor_wrapcopy_,
    lightconvertor_unwrap_,
    lightconvertor_insert_,
    lightconvertor_erase_,
    lightconvertor_find_
};

  

struct statusbuffer_object_ {
    PyObject_HEAD
    statusbuffer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct statusbuffer_wrapper_: public statusbuffer_type_ 
{

    static void __deleter__(statusbuffer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    statusbuffer_wrapper_( PyObject *o, PyInterpreterState *i  ,const change_nb_type_ & a0  ,unsigned int a1  ,const cookie_type_ & a2  ): statusbuffer_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~statusbuffer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(statusbuffer_object_ *self, visitproc visit, void *arg)
    {
        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(statusbuffer_object_ *self)
    {
        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void send_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return statusbuffer_type_::send();
                
            
            
        }

        return
            
                send
            
            
                ();
    }

    static PyObject *send_method_(statusbuffer_object_ *self, PyObject *args)
    {
        

        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusbuffer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->send_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusbuffer::send");
        }

        return 0;
    }



    void override_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return statusbuffer_type_::override( a0 );
                
            
            
        }

        return
            
                override
            
            
                ( a0 );
    }

    static PyObject *override_method_(statusbuffer_object_ *self, PyObject *args)
    {
         bool a0; 

        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusbuffer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->override_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusbuffer::override");
        }

        return 0;
    }



    void autosend_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return statusbuffer_type_::autosend( a0 );
                
            
            
        }

        return
            
                autosend
            
            
                ( a0 );
    }

    static PyObject *autosend_method_(statusbuffer_object_ *self, PyObject *args)
    {
         bool a0; 

        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusbuffer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->autosend_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusbuffer::autosend");
        }

        return 0;
    }



    void clear_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return statusbuffer_type_::clear();
                
            
            
        }

        return
            
                clear
            
            
                ();
    }

    static PyObject *clear_method_(statusbuffer_object_ *self, PyObject *args)
    {
        

        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusbuffer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusbuffer::clear");
        }

        return 0;
    }



    void set_status_call_(bool owned ,unsigned int a0 ,unsigned char a1 )
    {
        if(owned)
        {
            
                
                
                    return statusbuffer_type_::set_status( a0  ,a1 );
                
            
            
        }

        return
            
                set_status
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_status_method_(statusbuffer_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned char a1; 

        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusbuffer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_uc,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_status_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusbuffer::set_status");
        }

        return 0;
    }



    unsigned char get_status_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return statusbuffer_type_::get_status( a0 );
                
            
            
        }

        return
            
                get_status
            
            
                ( a0 );
    }

    static PyObject *get_status_method_(statusbuffer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusbuffer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned char r = o->get_status_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_uc((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusbuffer::get_status");
        }

        return 0;
    }



    void set_blink_time_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return statusbuffer_type_::set_blink_time( a0 );
                
            
            
        }

        return
            
                set_blink_time
            
            
                ( a0 );
    }

    static PyObject *set_blink_time_method_(statusbuffer_object_ *self, PyObject *args)
    {
         float a0; 

        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusbuffer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_blink_time_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusbuffer::set_blink_time");
        }

        return 0;
    }



    void set_blink_size_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return statusbuffer_type_::set_blink_size( a0 );
                
            
            
        }

        return
            
                set_blink_size
            
            
                ( a0 );
    }

    static PyObject *set_blink_size_method_(statusbuffer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusbuffer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_blink_size_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusbuffer::set_blink_size");
        }

        return 0;
    }



    change_nb_type_ enabler_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return statusbuffer_type_::enabler();
                
            
            
        }

        return
            
                enabler
            
            
                ();
    }

    static PyObject *enabler_method_(statusbuffer_object_ *self, PyObject *args)
    {
        

        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusbuffer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->enabler_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusbuffer::enabler");
        }

        return 0;
    }



    void enable_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return statusbuffer_type_::enable( a0 );
                
            
            
        }

        return
            
                enable
            
            
                ( a0 );
    }

    static PyObject *enable_method_(statusbuffer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusbuffer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enable_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusbuffer::enable");
        }

        return 0;
    }



    change_nb_type_ blinker_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return statusbuffer_type_::blinker();
                
            
            
        }

        return
            
                blinker
            
            
                ();
    }

    static PyObject *blinker_method_(statusbuffer_object_ *self, PyObject *args)
    {
        

        statusbuffer_wrapper_ *o = (statusbuffer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusbuffer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->blinker_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusbuffer::blinker");
        }

        return 0;
    }





    static PyObject *_method_lock(statusbuffer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(statusbuffer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void statusbuffer_delete_(statusbuffer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            statusbuffer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (statusbuffer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        statusbuffer_wrapper_::__deleter__((statusbuffer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void statusbuffer_dealloc_(statusbuffer_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    statusbuffer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * statusbuffer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    statusbuffer_object_ *self;

    if(!(self = (statusbuffer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int statusbuffer_init_(statusbuffer_object_ *self, PyObject *args, PyObject *kwds)
{
    statusbuffer_delete_(self);

    

    {
         change_nb_type_ * a0;  unsigned int a1;  cookie_type_ * a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_change_nb_copy,&a0,fpcvt_ui,&a1,fpcvt_cookie_copy,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new statusbuffer_wrapper_((PyObject *)self,interp ,*a0,a1,*a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing statusbuffer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            statusbuffer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"statusbuffer invalid arguments");
    return -1;

}


static const char *statusbuffer_send_docstring_ = "";

static const char *statusbuffer_override_docstring_ = "";

static const char *statusbuffer_autosend_docstring_ = "";

static const char *statusbuffer_clear_docstring_ = "";

static const char *statusbuffer_set_status_docstring_ = "";

static const char *statusbuffer_get_status_docstring_ = "";

static const char *statusbuffer_set_blink_time_docstring_ = "";

static const char *statusbuffer_set_blink_size_docstring_ = "";

static const char *statusbuffer_enabler_docstring_ = "";

static const char *statusbuffer_enable_docstring_ = "";

static const char *statusbuffer_blinker_docstring_ = "";




static PyMethodDef statusbuffer_methods_[] = {
    
        { "send",(PyCFunction)statusbuffer_wrapper_::send_method_,METH_VARARGS,(char *)statusbuffer_send_docstring_ },
    
        { "override",(PyCFunction)statusbuffer_wrapper_::override_method_,METH_VARARGS,(char *)statusbuffer_override_docstring_ },
    
        { "autosend",(PyCFunction)statusbuffer_wrapper_::autosend_method_,METH_VARARGS,(char *)statusbuffer_autosend_docstring_ },
    
        { "clear",(PyCFunction)statusbuffer_wrapper_::clear_method_,METH_VARARGS,(char *)statusbuffer_clear_docstring_ },
    
        { "set_status",(PyCFunction)statusbuffer_wrapper_::set_status_method_,METH_VARARGS,(char *)statusbuffer_set_status_docstring_ },
    
        { "get_status",(PyCFunction)statusbuffer_wrapper_::get_status_method_,METH_VARARGS,(char *)statusbuffer_get_status_docstring_ },
    
        { "set_blink_time",(PyCFunction)statusbuffer_wrapper_::set_blink_time_method_,METH_VARARGS,(char *)statusbuffer_set_blink_time_docstring_ },
    
        { "set_blink_size",(PyCFunction)statusbuffer_wrapper_::set_blink_size_method_,METH_VARARGS,(char *)statusbuffer_set_blink_size_docstring_ },
    
        { "enabler",(PyCFunction)statusbuffer_wrapper_::enabler_method_,METH_VARARGS,(char *)statusbuffer_enabler_docstring_ },
    
        { "enable",(PyCFunction)statusbuffer_wrapper_::enable_method_,METH_VARARGS,(char *)statusbuffer_enable_docstring_ },
    
        { "blinker",(PyCFunction)statusbuffer_wrapper_::blinker_method_,METH_VARARGS,(char *)statusbuffer_blinker_docstring_ },
    
    
        { "py_lock",(PyCFunction)statusbuffer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)statusbuffer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *statusbuffer_docstring_ = "";

PyTypeObject statusbuffer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.statusbuffer",/*tp_name*/
    sizeof(statusbuffer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)statusbuffer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)statusbuffer_docstring_,       /* tp_doc */
    (traverseproc)statusbuffer_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)statusbuffer_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    statusbuffer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)statusbuffer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    statusbuffer_new_,             /* tp_new */
};

PyObject *statusbuffer_wrapcopy_(statusbuffer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *statusbuffer_wrap_(statusbuffer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = statusbuffer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    statusbuffer_object_ *self = 
        
        PyObject_GC_New(statusbuffer_object_, &statusbuffer_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        statusbuffer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

statusbuffer_type_ *statusbuffer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &statusbuffer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected statusbuffer object");
        return 0;
    }

    statusbuffer_type_ *object = static_cast<statusbuffer_type_ *>(((statusbuffer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null statusbuffer object in unwrap");
        return 0;
    }

    return object;
}

void statusbuffer_insert_(const std::pair< statusbuffer_type_ *, PyObject *> &p)
{




    statusbuffer_wrapped_.insert(p);


}

void statusbuffer_erase_(statusbuffer_type_ *p)
{




    statusbuffer_wrapped_.erase(p);


}

PyObject * statusbuffer_find_(statusbuffer_type_ *p)
{




    PyObject *ret = 0;
    std::map< statusbuffer_type_ *, PyObject *>::iterator where = statusbuffer_wrapped_.find(p);
    if(where != statusbuffer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct statusbuffer_dispatch_ statusbuffer_realdispatcher_ =
{
    &statusbuffer_typeobject_,
    statusbuffer_wrap_,
    statusbuffer_wrapcopy_,
    statusbuffer_unwrap_,
    statusbuffer_insert_,
    statusbuffer_erase_,
    statusbuffer_find_
};

  

struct lightsource_object_ {
    PyObject_HEAD
    lightsource_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct lightsource_wrapper_: public lightsource_type_ 
{

    static void __deleter__(lightsource_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    lightsource_wrapper_( PyObject *o, PyInterpreterState *i  ,const change_nb_type_ & a0  ,unsigned int a1  ,const cookie_type_ & a2  ): lightsource_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~lightsource_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(lightsource_object_ *self, visitproc visit, void *arg)
    {
        lightsource_wrapper_ *o = (lightsource_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(lightsource_object_ *self)
    {
        lightsource_wrapper_ *o = (lightsource_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void set_size_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return lightsource_type_::set_size( a0 );
                
            
            
        }

        return
            
                set_size
            
            
                ( a0 );
    }

    static PyObject *set_size_method_(lightsource_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        lightsource_wrapper_ *o = (lightsource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightsource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_size_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightsource::set_size");
        }

        return 0;
    }



    unsigned int get_size_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return lightsource_type_::get_size();
                
            
            
        }

        return
            
                get_size
            
            
                ();
    }

    static PyObject *get_size_method_(lightsource_object_ *self, PyObject *args)
    {
        

        lightsource_wrapper_ *o = (lightsource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightsource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_size_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightsource::get_size");
        }

        return 0;
    }



    change_nb_type_ lightswitch_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return lightsource_type_::lightswitch();
                
            
            
        }

        return
            
                lightswitch
            
            
                ();
    }

    static PyObject *lightswitch_method_(lightsource_object_ *self, PyObject *args)
    {
        

        lightsource_wrapper_ *o = (lightsource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightsource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->lightswitch_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightsource::lightswitch");
        }

        return 0;
    }



    change_nb_type_ blinker_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return lightsource_type_::blinker();
                
            
            
        }

        return
            
                blinker
            
            
                ();
    }

    static PyObject *blinker_method_(lightsource_object_ *self, PyObject *args)
    {
        

        lightsource_wrapper_ *o = (lightsource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightsource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->blinker_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightsource::blinker");
        }

        return 0;
    }



    void enable_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return lightsource_type_::enable( a0 );
                
            
            
        }

        return
            
                enable
            
            
                ( a0 );
    }

    static PyObject *enable_method_(lightsource_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        lightsource_wrapper_ *o = (lightsource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightsource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enable_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightsource::enable");
        }

        return 0;
    }



    void set_color_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return lightsource_type_::set_color( a0  ,a1 );
                
            
            
        }

        return
            
                set_color
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_color_method_(lightsource_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        lightsource_wrapper_ *o = (lightsource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightsource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_color_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightsource::set_color");
        }

        return 0;
    }



    void set_status_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return lightsource_type_::set_status( a0  ,a1 );
                
            
            
        }

        return
            
                set_status
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_status_method_(lightsource_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        lightsource_wrapper_ *o = (lightsource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightsource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_status_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightsource::set_status");
        }

        return 0;
    }



    void override_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return lightsource_type_::override( a0 );
                
            
            
        }

        return
            
                override
            
            
                ( a0 );
    }

    static PyObject *override_method_(lightsource_object_ *self, PyObject *args)
    {
         bool a0; 

        lightsource_wrapper_ *o = (lightsource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null lightsource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->override_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in lightsource::override");
        }

        return 0;
    }





    static PyObject *_method_lock(lightsource_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(lightsource_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void lightsource_delete_(lightsource_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            lightsource_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (lightsource_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        lightsource_wrapper_::__deleter__((lightsource_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void lightsource_dealloc_(lightsource_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    lightsource_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * lightsource_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    lightsource_object_ *self;

    if(!(self = (lightsource_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int lightsource_init_(lightsource_object_ *self, PyObject *args, PyObject *kwds)
{
    lightsource_delete_(self);

    

    {
         change_nb_type_ * a0;  unsigned int a1;  cookie_type_ * a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_change_nb_copy,&a0,fpcvt_ui,&a1,fpcvt_cookie_copy,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new lightsource_wrapper_((PyObject *)self,interp ,*a0,a1,*a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing lightsource");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            lightsource_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"lightsource invalid arguments");
    return -1;

}


static const char *lightsource_set_size_docstring_ = "";

static const char *lightsource_get_size_docstring_ = "";

static const char *lightsource_lightswitch_docstring_ = "";

static const char *lightsource_blinker_docstring_ = "";

static const char *lightsource_enable_docstring_ = "";

static const char *lightsource_set_color_docstring_ = "";

static const char *lightsource_set_status_docstring_ = "";

static const char *lightsource_override_docstring_ = "";




static PyMethodDef lightsource_methods_[] = {
    
        { "set_size",(PyCFunction)lightsource_wrapper_::set_size_method_,METH_VARARGS,(char *)lightsource_set_size_docstring_ },
    
        { "get_size",(PyCFunction)lightsource_wrapper_::get_size_method_,METH_VARARGS,(char *)lightsource_get_size_docstring_ },
    
        { "lightswitch",(PyCFunction)lightsource_wrapper_::lightswitch_method_,METH_VARARGS,(char *)lightsource_lightswitch_docstring_ },
    
        { "blinker",(PyCFunction)lightsource_wrapper_::blinker_method_,METH_VARARGS,(char *)lightsource_blinker_docstring_ },
    
        { "enable",(PyCFunction)lightsource_wrapper_::enable_method_,METH_VARARGS,(char *)lightsource_enable_docstring_ },
    
        { "set_color",(PyCFunction)lightsource_wrapper_::set_color_method_,METH_VARARGS,(char *)lightsource_set_color_docstring_ },
    
        { "set_status",(PyCFunction)lightsource_wrapper_::set_status_method_,METH_VARARGS,(char *)lightsource_set_status_docstring_ },
    
        { "override",(PyCFunction)lightsource_wrapper_::override_method_,METH_VARARGS,(char *)lightsource_override_docstring_ },
    
    
        { "py_lock",(PyCFunction)lightsource_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)lightsource_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *lightsource_docstring_ = "";

PyTypeObject lightsource_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.lightsource",/*tp_name*/
    sizeof(lightsource_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)lightsource_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)lightsource_docstring_,       /* tp_doc */
    (traverseproc)lightsource_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)lightsource_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    lightsource_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)lightsource_init_,  /* tp_init */
    0,                         /* tp_alloc */
    lightsource_new_,             /* tp_new */
};

PyObject *lightsource_wrapcopy_(lightsource_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *lightsource_wrap_(lightsource_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = lightsource_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    lightsource_object_ *self = 
        
        PyObject_GC_New(lightsource_object_, &lightsource_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        lightsource_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

lightsource_type_ *lightsource_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &lightsource_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected lightsource object");
        return 0;
    }

    lightsource_type_ *object = static_cast<lightsource_type_ *>(((lightsource_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null lightsource object in unwrap");
        return 0;
    }

    return object;
}

void lightsource_insert_(const std::pair< lightsource_type_ *, PyObject *> &p)
{




    lightsource_wrapped_.insert(p);


}

void lightsource_erase_(lightsource_type_ *p)
{




    lightsource_wrapped_.erase(p);


}

PyObject * lightsource_find_(lightsource_type_ *p)
{




    PyObject *ret = 0;
    std::map< lightsource_type_ *, PyObject *>::iterator where = lightsource_wrapped_.find(p);
    if(where != lightsource_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct lightsource_dispatch_ lightsource_realdispatcher_ =
{
    &lightsource_typeobject_,
    lightsource_wrap_,
    lightsource_wrapcopy_,
    lightsource_unwrap_,
    lightsource_insert_,
    lightsource_erase_,
    lightsource_find_
};

  

struct statusmixer_object_ {
    PyObject_HEAD
    statusmixer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct statusmixer_wrapper_: public statusmixer_type_ 
{

    static void __deleter__(statusmixer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    statusmixer_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ): statusmixer_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~statusmixer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void autoupdate_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return statusmixer_type_::autoupdate( a0 );
                
            
            
        }

        return
            
                autoupdate
            
            
                ( a0 );
    }

    static PyObject *autoupdate_method_(statusmixer_object_ *self, PyObject *args)
    {
         bool a0; 

        statusmixer_wrapper_ *o = (statusmixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusmixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->autoupdate_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusmixer::autoupdate");
        }

        return 0;
    }



    void update_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return statusmixer_type_::update();
                
            
            
        }

        return
            
                update
            
            
                ();
    }

    static PyObject *update_method_(statusmixer_object_ *self, PyObject *args)
    {
        

        statusmixer_wrapper_ *o = (statusmixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusmixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->update_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusmixer::update");
        }

        return 0;
    }



    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return statusmixer_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(statusmixer_object_ *self, PyObject *args)
    {
        

        statusmixer_wrapper_ *o = (statusmixer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null statusmixer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in statusmixer::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(statusmixer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(statusmixer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void statusmixer_delete_(statusmixer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            statusmixer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (statusmixer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        statusmixer_wrapper_::__deleter__((statusmixer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void statusmixer_dealloc_(statusmixer_object_* self)
{
    
    statusmixer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * statusmixer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    statusmixer_object_ *self;

    if(!(self = (statusmixer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int statusmixer_init_(statusmixer_object_ *self, PyObject *args, PyObject *kwds)
{
    statusmixer_delete_(self);

    

    {
         cookie_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_cookie_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new statusmixer_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing statusmixer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            statusmixer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"statusmixer invalid arguments");
    return -1;

}


static const char *statusmixer_autoupdate_docstring_ = "";

static const char *statusmixer_update_docstring_ = "";

static const char *statusmixer_cookie_docstring_ = "";




static PyMethodDef statusmixer_methods_[] = {
    
        { "autoupdate",(PyCFunction)statusmixer_wrapper_::autoupdate_method_,METH_VARARGS,(char *)statusmixer_autoupdate_docstring_ },
    
        { "update",(PyCFunction)statusmixer_wrapper_::update_method_,METH_VARARGS,(char *)statusmixer_update_docstring_ },
    
        { "cookie",(PyCFunction)statusmixer_wrapper_::cookie_method_,METH_VARARGS,(char *)statusmixer_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)statusmixer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)statusmixer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *statusmixer_docstring_ = "";

PyTypeObject statusmixer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.statusmixer",/*tp_name*/
    sizeof(statusmixer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)statusmixer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)statusmixer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    statusmixer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)statusmixer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    statusmixer_new_,             /* tp_new */
};

PyObject *statusmixer_wrapcopy_(statusmixer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *statusmixer_wrap_(statusmixer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = statusmixer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    statusmixer_object_ *self = 
        PyObject_NEW(statusmixer_object_, &statusmixer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        statusmixer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

statusmixer_type_ *statusmixer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &statusmixer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected statusmixer object");
        return 0;
    }

    statusmixer_type_ *object = static_cast<statusmixer_type_ *>(((statusmixer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null statusmixer object in unwrap");
        return 0;
    }

    return object;
}

void statusmixer_insert_(const std::pair< statusmixer_type_ *, PyObject *> &p)
{




    statusmixer_wrapped_.insert(p);


}

void statusmixer_erase_(statusmixer_type_ *p)
{




    statusmixer_wrapped_.erase(p);


}

PyObject * statusmixer_find_(statusmixer_type_ *p)
{




    PyObject *ret = 0;
    std::map< statusmixer_type_ *, PyObject *>::iterator where = statusmixer_wrapped_.find(p);
    if(where != statusmixer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct statusmixer_dispatch_ statusmixer_realdispatcher_ =
{
    &statusmixer_typeobject_,
    statusmixer_wrap_,
    statusmixer_wrapcopy_,
    statusmixer_unwrap_,
    statusmixer_insert_,
    statusmixer_erase_,
    statusmixer_find_
};

  

struct controllerdict_object_ {
    PyObject_HEAD
    controllerdict_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct controllerdict_wrapper_: public controllerdict_type_ 
{

    static void __deleter__(controllerdict_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    controllerdict_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ): controllerdict_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~controllerdict_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    data_type_ get_ctl_value_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return controllerdict_type_::get_ctl_value( a0 );
                
            
            
        }

        return
            
                get_ctl_value
            
            
                ( a0 );
    }

    static PyObject *get_ctl_value_method_(controllerdict_object_ *self, PyObject *args)
    {
         std::string a0; 

        controllerdict_wrapper_ *o = (controllerdict_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controllerdict c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->get_ctl_value_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controllerdict::get_ctl_value");
        }

        return 0;
    }



    data_type_ get_ctl_dict_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return controllerdict_type_::get_ctl_dict();
                
            
            
        }

        return
            
                get_ctl_dict
            
            
                ();
    }

    static PyObject *get_ctl_dict_method_(controllerdict_object_ *self, PyObject *args)
    {
        

        controllerdict_wrapper_ *o = (controllerdict_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controllerdict c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->get_ctl_dict_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controllerdict::get_ctl_dict");
        }

        return 0;
    }



    void put_ctl_call_(bool owned ,const std::string & a0 ,const data_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return controllerdict_type_::put_ctl( a0  ,a1 );
                
            
            
        }

        return
            
                put_ctl
            
            
                ( a0  ,a1 );
    }

    static PyObject *put_ctl_method_(controllerdict_object_ *self, PyObject *args)
    {
         std::string a0;  data_type_ * a1; 

        controllerdict_wrapper_ *o = (controllerdict_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controllerdict c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_stdstr,&a0  ,fpcvt_data_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->put_ctl_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controllerdict::put_ctl");
        }

        return 0;
    }



    void put_call_(bool owned ,const std::string & a0 ,const data_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return controllerdict_type_::put( a0  ,a1 );
                
            
            
        }

        return
            
                put
            
            
                ( a0  ,a1 );
    }

    static PyObject *put_method_(controllerdict_object_ *self, PyObject *args)
    {
         std::string a0;  data_type_ * a1; 

        controllerdict_wrapper_ *o = (controllerdict_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controllerdict c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_stdstr,&a0  ,fpcvt_data_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->put_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controllerdict::put");
        }

        return 0;
    }



    change_type_ changetonic_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return controllerdict_type_::changetonic( a0 );
                
            
            
        }

        return
            
                changetonic
            
            
                ( a0 );
    }

    static PyObject *changetonic_method_(controllerdict_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        controllerdict_wrapper_ *o = (controllerdict_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controllerdict c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_type_ r = o->changetonic_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controllerdict::changetonic");
        }

        return 0;
    }



    change_type_ changescale_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return controllerdict_type_::changescale( a0 );
                
            
            
        }

        return
            
                changescale
            
            
                ( a0 );
    }

    static PyObject *changescale_method_(controllerdict_object_ *self, PyObject *args)
    {
         std::string a0; 

        controllerdict_wrapper_ *o = (controllerdict_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null controllerdict c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_type_ r = o->changescale_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in controllerdict::changescale");
        }

        return 0;
    }





    static PyObject *_method_lock(controllerdict_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(controllerdict_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void controllerdict_delete_(controllerdict_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            controllerdict_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (controllerdict_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        controllerdict_wrapper_::__deleter__((controllerdict_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void controllerdict_dealloc_(controllerdict_object_* self)
{
    
    controllerdict_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * controllerdict_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    controllerdict_object_ *self;

    if(!(self = (controllerdict_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int controllerdict_init_(controllerdict_object_ *self, PyObject *args, PyObject *kwds)
{
    controllerdict_delete_(self);

    

    {
         cookie_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_cookie_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new controllerdict_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing controllerdict");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            controllerdict_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"controllerdict invalid arguments");
    return -1;

}


static const char *controllerdict_get_ctl_value_docstring_ = "";

static const char *controllerdict_get_ctl_dict_docstring_ = "";

static const char *controllerdict_put_ctl_docstring_ = "";

static const char *controllerdict_put_docstring_ = "";

static const char *controllerdict_changetonic_docstring_ = "";

static const char *controllerdict_changescale_docstring_ = "";




static PyMethodDef controllerdict_methods_[] = {
    
        { "get_ctl_value",(PyCFunction)controllerdict_wrapper_::get_ctl_value_method_,METH_VARARGS,(char *)controllerdict_get_ctl_value_docstring_ },
    
        { "get_ctl_dict",(PyCFunction)controllerdict_wrapper_::get_ctl_dict_method_,METH_VARARGS,(char *)controllerdict_get_ctl_dict_docstring_ },
    
        { "put_ctl",(PyCFunction)controllerdict_wrapper_::put_ctl_method_,METH_VARARGS,(char *)controllerdict_put_ctl_docstring_ },
    
        { "put",(PyCFunction)controllerdict_wrapper_::put_method_,METH_VARARGS,(char *)controllerdict_put_docstring_ },
    
        { "changetonic",(PyCFunction)controllerdict_wrapper_::changetonic_method_,METH_VARARGS,(char *)controllerdict_changetonic_docstring_ },
    
        { "changescale",(PyCFunction)controllerdict_wrapper_::changescale_method_,METH_VARARGS,(char *)controllerdict_changescale_docstring_ },
    
    
        { "py_lock",(PyCFunction)controllerdict_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)controllerdict_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *controllerdict_docstring_ = "";

PyTypeObject controllerdict_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.controllerdict",/*tp_name*/
    sizeof(controllerdict_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)controllerdict_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)controllerdict_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    controllerdict_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)controllerdict_init_,  /* tp_init */
    0,                         /* tp_alloc */
    controllerdict_new_,             /* tp_new */
};

PyObject *controllerdict_wrapcopy_(controllerdict_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *controllerdict_wrap_(controllerdict_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = controllerdict_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    controllerdict_object_ *self = 
        PyObject_NEW(controllerdict_object_, &controllerdict_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        controllerdict_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

controllerdict_type_ *controllerdict_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &controllerdict_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected controllerdict object");
        return 0;
    }

    controllerdict_type_ *object = static_cast<controllerdict_type_ *>(((controllerdict_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null controllerdict object in unwrap");
        return 0;
    }

    return object;
}

void controllerdict_insert_(const std::pair< controllerdict_type_ *, PyObject *> &p)
{




    controllerdict_wrapped_.insert(p);


}

void controllerdict_erase_(controllerdict_type_ *p)
{




    controllerdict_wrapped_.erase(p);


}

PyObject * controllerdict_find_(controllerdict_type_ *p)
{




    PyObject *ret = 0;
    std::map< controllerdict_type_ *, PyObject *>::iterator where = controllerdict_wrapped_.find(p);
    if(where != controllerdict_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct controllerdict_dispatch_ controllerdict_realdispatcher_ =
{
    &controllerdict_typeobject_,
    controllerdict_wrap_,
    controllerdict_wrapcopy_,
    controllerdict_unwrap_,
    controllerdict_insert_,
    controllerdict_erase_,
    controllerdict_find_
};

  

struct mapping_info_object_ {
    PyObject_HEAD
    mapping_info_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct mapping_info_wrapper_: public mapping_info_type_ 
{

    static void __deleter__(mapping_info_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    mapping_info_wrapper_( PyObject *o, PyInterpreterState *i  ,const mapping_info_type_ & a0  ): mapping_info_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    mapping_info_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned short a0  ): mapping_info_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    mapping_info_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned short a0  ,bool a1  ,float a2  ,float a3  ,float a4  ,float a5  ,bool a6  ,float a7  ,unsigned int a8  ,unsigned int a9  ,unsigned int a10  ,int a11  ,unsigned int a12  ): mapping_info_type_(  a0 ,a1 ,a2 ,a3 ,a4 ,a5 ,a6 ,a7 ,a8 ,a9 ,a10 ,a11 ,a12 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~mapping_info_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    mapping_info_type_ clone_with_enabled_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return mapping_info_type_::clone_with_enabled( a0 );
                
            
            
        }

        return
            
                clone_with_enabled
            
            
                ( a0 );
    }

    static PyObject *clone_with_enabled_method_(mapping_info_object_ *self, PyObject *args)
    {
         bool a0; 

        mapping_info_wrapper_ *o = (mapping_info_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_info c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->clone_with_enabled_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_info::clone_with_enabled");
        }

        return 0;
    }



    mapping_info_type_ clone_with_scale_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return mapping_info_type_::clone_with_scale( a0 );
                
            
            
        }

        return
            
                clone_with_scale
            
            
                ( a0 );
    }

    static PyObject *clone_with_scale_method_(mapping_info_object_ *self, PyObject *args)
    {
         float a0; 

        mapping_info_wrapper_ *o = (mapping_info_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_info c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->clone_with_scale_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_info::clone_with_scale");
        }

        return 0;
    }



    mapping_info_type_ clone_with_lo_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return mapping_info_type_::clone_with_lo( a0 );
                
            
            
        }

        return
            
                clone_with_lo
            
            
                ( a0 );
    }

    static PyObject *clone_with_lo_method_(mapping_info_object_ *self, PyObject *args)
    {
         float a0; 

        mapping_info_wrapper_ *o = (mapping_info_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_info c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->clone_with_lo_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_info::clone_with_lo");
        }

        return 0;
    }



    mapping_info_type_ clone_with_base_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return mapping_info_type_::clone_with_base( a0 );
                
            
            
        }

        return
            
                clone_with_base
            
            
                ( a0 );
    }

    static PyObject *clone_with_base_method_(mapping_info_object_ *self, PyObject *args)
    {
         float a0; 

        mapping_info_wrapper_ *o = (mapping_info_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_info c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->clone_with_base_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_info::clone_with_base");
        }

        return 0;
    }



    mapping_info_type_ clone_with_hi_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return mapping_info_type_::clone_with_hi( a0 );
                
            
            
        }

        return
            
                clone_with_hi
            
            
                ( a0 );
    }

    static PyObject *clone_with_hi_method_(mapping_info_object_ *self, PyObject *args)
    {
         float a0; 

        mapping_info_wrapper_ *o = (mapping_info_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_info c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->clone_with_hi_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_info::clone_with_hi");
        }

        return 0;
    }



    mapping_info_type_ clone_with_origin_return_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return mapping_info_type_::clone_with_origin_return( a0 );
                
            
            
        }

        return
            
                clone_with_origin_return
            
            
                ( a0 );
    }

    static PyObject *clone_with_origin_return_method_(mapping_info_object_ *self, PyObject *args)
    {
         bool a0; 

        mapping_info_wrapper_ *o = (mapping_info_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_info c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->clone_with_origin_return_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_info::clone_with_origin_return");
        }

        return 0;
    }



    mapping_info_type_ clone_with_decimation_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return mapping_info_type_::clone_with_decimation( a0 );
                
            
            
        }

        return
            
                clone_with_decimation
            
            
                ( a0 );
    }

    static PyObject *clone_with_decimation_method_(mapping_info_object_ *self, PyObject *args)
    {
         float a0; 

        mapping_info_wrapper_ *o = (mapping_info_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_info c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->clone_with_decimation_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_info::clone_with_decimation");
        }

        return 0;
    }



    mapping_info_type_ clone_with_scope_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return mapping_info_type_::clone_with_scope( a0 );
                
            
            
        }

        return
            
                clone_with_scope
            
            
                ( a0 );
    }

    static PyObject *clone_with_scope_method_(mapping_info_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        mapping_info_wrapper_ *o = (mapping_info_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_info c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->clone_with_scope_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_info::clone_with_scope");
        }

        return 0;
    }



    mapping_info_type_ clone_with_channelscope_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return mapping_info_type_::clone_with_channelscope( a0 );
                
            
            
        }

        return
            
                clone_with_channelscope
            
            
                ( a0 );
    }

    static PyObject *clone_with_channelscope_method_(mapping_info_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        mapping_info_wrapper_ *o = (mapping_info_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_info c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->clone_with_channelscope_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_info::clone_with_channelscope");
        }

        return 0;
    }



    mapping_info_type_ clone_with_resolution_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return mapping_info_type_::clone_with_resolution( a0 );
                
            
            
        }

        return
            
                clone_with_resolution
            
            
                ( a0 );
    }

    static PyObject *clone_with_resolution_method_(mapping_info_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        mapping_info_wrapper_ *o = (mapping_info_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_info c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->clone_with_resolution_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_info::clone_with_resolution");
        }

        return 0;
    }



    mapping_info_type_ clone_with_secondarycc_call_(bool owned ,int a0 )
    {
        if(owned)
        {
            
                
                
                    return mapping_info_type_::clone_with_secondarycc( a0 );
                
            
            
        }

        return
            
                clone_with_secondarycc
            
            
                ( a0 );
    }

    static PyObject *clone_with_secondarycc_method_(mapping_info_object_ *self, PyObject *args)
    {
         int a0; 

        mapping_info_wrapper_ *o = (mapping_info_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_info c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_i,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->clone_with_secondarycc_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_info::clone_with_secondarycc");
        }

        return 0;
    }



    mapping_info_type_ clone_with_curve_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return mapping_info_type_::clone_with_curve( a0 );
                
            
            
        }

        return
            
                clone_with_curve
            
            
                ( a0 );
    }

    static PyObject *clone_with_curve_method_(mapping_info_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        mapping_info_wrapper_ *o = (mapping_info_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_info c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->clone_with_curve_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_info::clone_with_curve");
        }

        return 0;
    }





    static PyObject *_method_lock(mapping_info_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(mapping_info_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void mapping_info_delete_(mapping_info_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            mapping_info_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (mapping_info_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        mapping_info_wrapper_::__deleter__((mapping_info_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void mapping_info_dealloc_(mapping_info_object_* self)
{
    
    mapping_info_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * mapping_info_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    mapping_info_object_ *self;

    if(!(self = (mapping_info_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int mapping_info_init_(mapping_info_object_ *self, PyObject *args, PyObject *kwds)
{
    mapping_info_delete_(self);

    

    {
         mapping_info_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_mapping_info_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new mapping_info_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing mapping_info");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            mapping_info_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         unsigned short a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_us,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new mapping_info_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing mapping_info");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            mapping_info_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         unsigned short a0;  bool a1;  float a2;  float a3;  float a4;  float a5;  bool a6;  float a7;  unsigned int a8;  unsigned int a9;  unsigned int a10;  int a11;  unsigned int a12; 

        if(PyArg_ParseTuple(args,"O&O&O&O&O&O&O&O&O&O&O&O&O&" ,fpcvt_us,&a0,fpcvt_b,&a1,fpcvt_f,&a2,fpcvt_f,&a3,fpcvt_f,&a4,fpcvt_f,&a5,fpcvt_b,&a6,fpcvt_f,&a7,fpcvt_ui,&a8,fpcvt_ui,&a9,fpcvt_ui,&a10,fpcvt_i,&a11,fpcvt_ui,&a12))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new mapping_info_wrapper_((PyObject *)self,interp ,a0,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing mapping_info");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            mapping_info_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"mapping_info invalid arguments");
    return -1;

}


static const char *mapping_info_clone_with_enabled_docstring_ = "";

static const char *mapping_info_clone_with_scale_docstring_ = "";

static const char *mapping_info_clone_with_lo_docstring_ = "";

static const char *mapping_info_clone_with_base_docstring_ = "";

static const char *mapping_info_clone_with_hi_docstring_ = "";

static const char *mapping_info_clone_with_origin_return_docstring_ = "";

static const char *mapping_info_clone_with_decimation_docstring_ = "";

static const char *mapping_info_clone_with_scope_docstring_ = "";

static const char *mapping_info_clone_with_channelscope_docstring_ = "";

static const char *mapping_info_clone_with_resolution_docstring_ = "";

static const char *mapping_info_clone_with_secondarycc_docstring_ = "";

static const char *mapping_info_clone_with_curve_docstring_ = "";




static PyMethodDef mapping_info_methods_[] = {
    
        { "clone_with_enabled",(PyCFunction)mapping_info_wrapper_::clone_with_enabled_method_,METH_VARARGS,(char *)mapping_info_clone_with_enabled_docstring_ },
    
        { "clone_with_scale",(PyCFunction)mapping_info_wrapper_::clone_with_scale_method_,METH_VARARGS,(char *)mapping_info_clone_with_scale_docstring_ },
    
        { "clone_with_lo",(PyCFunction)mapping_info_wrapper_::clone_with_lo_method_,METH_VARARGS,(char *)mapping_info_clone_with_lo_docstring_ },
    
        { "clone_with_base",(PyCFunction)mapping_info_wrapper_::clone_with_base_method_,METH_VARARGS,(char *)mapping_info_clone_with_base_docstring_ },
    
        { "clone_with_hi",(PyCFunction)mapping_info_wrapper_::clone_with_hi_method_,METH_VARARGS,(char *)mapping_info_clone_with_hi_docstring_ },
    
        { "clone_with_origin_return",(PyCFunction)mapping_info_wrapper_::clone_with_origin_return_method_,METH_VARARGS,(char *)mapping_info_clone_with_origin_return_docstring_ },
    
        { "clone_with_decimation",(PyCFunction)mapping_info_wrapper_::clone_with_decimation_method_,METH_VARARGS,(char *)mapping_info_clone_with_decimation_docstring_ },
    
        { "clone_with_scope",(PyCFunction)mapping_info_wrapper_::clone_with_scope_method_,METH_VARARGS,(char *)mapping_info_clone_with_scope_docstring_ },
    
        { "clone_with_channelscope",(PyCFunction)mapping_info_wrapper_::clone_with_channelscope_method_,METH_VARARGS,(char *)mapping_info_clone_with_channelscope_docstring_ },
    
        { "clone_with_resolution",(PyCFunction)mapping_info_wrapper_::clone_with_resolution_method_,METH_VARARGS,(char *)mapping_info_clone_with_resolution_docstring_ },
    
        { "clone_with_secondarycc",(PyCFunction)mapping_info_wrapper_::clone_with_secondarycc_method_,METH_VARARGS,(char *)mapping_info_clone_with_secondarycc_docstring_ },
    
        { "clone_with_curve",(PyCFunction)mapping_info_wrapper_::clone_with_curve_method_,METH_VARARGS,(char *)mapping_info_clone_with_curve_docstring_ },
    
    
        { "py_lock",(PyCFunction)mapping_info_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)mapping_info_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *mapping_info_docstring_ = "";

PyTypeObject mapping_info_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.mapping_info",/*tp_name*/
    sizeof(mapping_info_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)mapping_info_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)mapping_info_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    mapping_info_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)mapping_info_init_,  /* tp_init */
    0,                         /* tp_alloc */
    mapping_info_new_,             /* tp_new */
};

PyObject *mapping_info_wrapcopy_(mapping_info_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            mapping_info_object_ *obj = 
                PyObject_NEW(mapping_info_object_, mapping_info_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new mapping_info_type_(*o);
                }

                try
                {
                    mapping_info_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying mapping_info");
                return 0;
            }
        

    
}

PyObject *mapping_info_wrap_(mapping_info_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = mapping_info_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    mapping_info_object_ *self = 
        PyObject_NEW(mapping_info_object_, &mapping_info_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        mapping_info_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

mapping_info_type_ *mapping_info_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &mapping_info_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected mapping_info object");
        return 0;
    }

    mapping_info_type_ *object = static_cast<mapping_info_type_ *>(((mapping_info_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null mapping_info object in unwrap");
        return 0;
    }

    return object;
}

void mapping_info_insert_(const std::pair< mapping_info_type_ *, PyObject *> &p)
{




    mapping_info_wrapped_.insert(p);


}

void mapping_info_erase_(mapping_info_type_ *p)
{




    mapping_info_wrapped_.erase(p);


}

PyObject * mapping_info_find_(mapping_info_type_ *p)
{




    PyObject *ret = 0;
    std::map< mapping_info_type_ *, PyObject *>::iterator where = mapping_info_wrapped_.find(p);
    if(where != mapping_info_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct mapping_info_dispatch_ mapping_info_realdispatcher_ =
{
    &mapping_info_typeobject_,
    mapping_info_wrap_,
    mapping_info_wrapcopy_,
    mapping_info_unwrap_,
    mapping_info_insert_,
    mapping_info_erase_,
    mapping_info_find_
};

  

struct midi_converter_object_ {
    PyObject_HEAD
    midi_converter_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct midi_converter_wrapper_: public midi_converter_type_ 
{

    static void __deleter__(midi_converter_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    midi_converter_wrapper_( PyObject *o, PyInterpreterState *i  ,mapping_observer_type_ & a0  ,midi_channel_delegate_type_ & a1  ,clockdomain_ctl_type_ * a2  ,midi_from_belcanto_type_ & a3  ,const std::string & a4  ): midi_converter_type_(  a0 ,a1 ,a2 ,a3 ,a4 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~midi_converter_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    clockdomain_ctl_type_ * clock_domain_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::clock_domain();
                
            
            
        }

        return
            
                clock_domain
            
            
                ();
    }

    static PyObject *clock_domain_method_(midi_converter_object_ *self, PyObject *args)
    {
        

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    clockdomain_ctl_type_ * r = o->clock_domain_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_clockdomain_ctl((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::clock_domain");
        }

        return 0;
    }



    cookie_type_ parameter_input_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::parameter_input( a0 );
                
            
            
        }

        return
            
                parameter_input
            
            
                ( a0 );
    }

    static PyObject *parameter_input_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->parameter_input_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::parameter_input");
        }

        return 0;
    }



    void set_mapping_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::set_mapping( a0 );
                
            
            
        }

        return
            
                set_mapping
            
            
                ( a0 );
    }

    static PyObject *set_mapping_method_(midi_converter_object_ *self, PyObject *args)
    {
         std::string a0; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_mapping_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::set_mapping");
        }

        return 0;
    }



    std::string get_mapping_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::get_mapping();
                
            
            
        }

        return
            
                get_mapping
            
            
                ();
    }

    static PyObject *get_mapping_method_(midi_converter_object_ *self, PyObject *args)
    {
        

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_mapping_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::get_mapping");
        }

        return 0;
    }



    void parameter_name_changed_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::parameter_name_changed( a0 );
                
            
            
        }

        return
            
                parameter_name_changed
            
            
                ( a0 );
    }

    static PyObject *parameter_name_changed_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->parameter_name_changed_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::parameter_name_changed");
        }

        return 0;
    }



    void map_param_call_(bool owned ,unsigned int a0 ,mapping_info_type_ a1 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::map_param( a0  ,a1 );
                
            
            
        }

        return
            
                map_param
            
            
                ( a0  ,a1 );
    }

    static PyObject *map_param_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0;  mapping_info_type_ * a1; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_mapping_info_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->map_param_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::map_param");
        }

        return 0;
    }



    void map_midi_call_(bool owned ,unsigned int a0 ,mapping_info_type_ a1 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::map_midi( a0  ,a1 );
                
            
            
        }

        return
            
                map_midi
            
            
                ( a0  ,a1 );
    }

    static PyObject *map_midi_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0;  mapping_info_type_ * a1; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_mapping_info_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->map_midi_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::map_midi");
        }

        return 0;
    }



    void unmap_param_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::unmap_param( a0  ,a1 );
                
            
            
        }

        return
            
                unmap_param
            
            
                ( a0  ,a1 );
    }

    static PyObject *unmap_param_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->unmap_param_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::unmap_param");
        }

        return 0;
    }



    void unmap_midi_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::unmap_midi( a0  ,a1 );
                
            
            
        }

        return
            
                unmap_midi
            
            
                ( a0  ,a1 );
    }

    static PyObject *unmap_midi_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->unmap_midi_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::unmap_midi");
        }

        return 0;
    }



    bool is_mapped_param_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::is_mapped_param( a0  ,a1 );
                
            
            
        }

        return
            
                is_mapped_param
            
            
                ( a0  ,a1 );
    }

    static PyObject *is_mapped_param_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_mapped_param_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::is_mapped_param");
        }

        return 0;
    }



    bool is_mapped_midi_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::is_mapped_midi( a0  ,a1 );
                
            
            
        }

        return
            
                is_mapped_midi
            
            
                ( a0  ,a1 );
    }

    static PyObject *is_mapped_midi_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_mapped_midi_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::is_mapped_midi");
        }

        return 0;
    }



    mapping_info_type_ get_info_param_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::get_info_param( a0  ,a1 );
                
            
            
        }

        return
            
                get_info_param
            
            
                ( a0  ,a1 );
    }

    static PyObject *get_info_param_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->get_info_param_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::get_info_param");
        }

        return 0;
    }



    mapping_info_type_ get_info_midi_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::get_info_midi( a0  ,a1 );
                
            
            
        }

        return
            
                get_info_midi
            
            
                ( a0  ,a1 );
    }

    static PyObject *get_info_midi_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->get_info_midi_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::get_info_midi");
        }

        return 0;
    }



    void set_minimum_decimation_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::set_minimum_decimation( a0 );
                
            
            
        }

        return
            
                set_minimum_decimation
            
            
                ( a0 );
    }

    static PyObject *set_minimum_decimation_method_(midi_converter_object_ *self, PyObject *args)
    {
         float a0; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_minimum_decimation_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::set_minimum_decimation");
        }

        return 0;
    }



    void set_midi_notes_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::set_midi_notes( a0 );
                
            
            
        }

        return
            
                set_midi_notes
            
            
                ( a0 );
    }

    static PyObject *set_midi_notes_method_(midi_converter_object_ *self, PyObject *args)
    {
         bool a0; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_midi_notes_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::set_midi_notes");
        }

        return 0;
    }



    void set_midi_pitchbend_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::set_midi_pitchbend( a0 );
                
            
            
        }

        return
            
                set_midi_pitchbend
            
            
                ( a0 );
    }

    static PyObject *set_midi_pitchbend_method_(midi_converter_object_ *self, PyObject *args)
    {
         bool a0; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_midi_pitchbend_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::set_midi_pitchbend");
        }

        return 0;
    }



    void set_midi_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::set_midi_channel( a0 );
                
            
            
        }

        return
            
                set_midi_channel
            
            
                ( a0 );
    }

    static PyObject *set_midi_channel_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_midi_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::set_midi_channel");
        }

        return 0;
    }



    void set_min_midi_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::set_min_midi_channel( a0 );
                
            
            
        }

        return
            
                set_min_midi_channel
            
            
                ( a0 );
    }

    static PyObject *set_min_midi_channel_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_min_midi_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::set_min_midi_channel");
        }

        return 0;
    }



    void set_max_midi_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::set_max_midi_channel( a0 );
                
            
            
        }

        return
            
                set_max_midi_channel
            
            
                ( a0 );
    }

    static PyObject *set_max_midi_channel_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_max_midi_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::set_max_midi_channel");
        }

        return 0;
    }



    void set_program_change_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::set_program_change( a0 );
                
            
            
        }

        return
            
                set_program_change
            
            
                ( a0 );
    }

    static PyObject *set_program_change_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_program_change_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::set_program_change");
        }

        return 0;
    }



    change_nb_type_ change_program_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::change_program();
                
            
            
        }

        return
            
                change_program
            
            
                ();
    }

    static PyObject *change_program_method_(midi_converter_object_ *self, PyObject *args)
    {
        

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->change_program_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::change_program");
        }

        return 0;
    }



    void set_bank_change_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::set_bank_change( a0 );
                
            
            
        }

        return
            
                set_bank_change
            
            
                ( a0 );
    }

    static PyObject *set_bank_change_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_bank_change_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::set_bank_change");
        }

        return 0;
    }



    change_nb_type_ change_bank_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::change_bank();
                
            
            
        }

        return
            
                change_bank
            
            
                ();
    }

    static PyObject *change_bank_method_(midi_converter_object_ *self, PyObject *args)
    {
        

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->change_bank_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::change_bank");
        }

        return 0;
    }



    void set_cc_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::set_cc( a0  ,a1 );
                
            
            
        }

        return
            
                set_cc
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_cc_method_(midi_converter_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_cc_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::set_cc");
        }

        return 0;
    }



    change_nb_type_ change_cc_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::change_cc();
                
            
            
        }

        return
            
                change_cc
            
            
                ();
    }

    static PyObject *change_cc_method_(midi_converter_object_ *self, PyObject *args)
    {
        

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->change_cc_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::change_cc");
        }

        return 0;
    }



    void close_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_converter_type_::close();
                
            
            
        }

        return
            
                close
            
            
                ();
    }

    static PyObject *close_method_(midi_converter_object_ *self, PyObject *args)
    {
        

        midi_converter_wrapper_ *o = (midi_converter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_converter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_converter::close");
        }

        return 0;
    }





    static PyObject *_method_lock(midi_converter_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(midi_converter_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void midi_converter_delete_(midi_converter_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            midi_converter_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (midi_converter_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        midi_converter_wrapper_::__deleter__((midi_converter_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void midi_converter_dealloc_(midi_converter_object_* self)
{
    
    midi_converter_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * midi_converter_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    midi_converter_object_ *self;

    if(!(self = (midi_converter_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int midi_converter_init_(midi_converter_object_ *self, PyObject *args, PyObject *kwds)
{
    midi_converter_delete_(self);

    

    {
         mapping_observer_type_ * a0;  midi_channel_delegate_type_ * a1;  clockdomain_ctl_type_ * a2;  midi_from_belcanto_type_ * a3;  std::string a4; 

        if(PyArg_ParseTuple(args,"O&O&O&O&O&" ,fpcvt_mapping_observer,&a0,fpcvt_midi_channel_delegate,&a1,fpcvt_clockdomain_ctl,&a2,fpcvt_midi_from_belcanto,&a3,fpcvt_stdstr,&a4))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new midi_converter_wrapper_((PyObject *)self,interp ,*a0,*a1,a2,*a3,a4);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing midi_converter");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            midi_converter_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"midi_converter invalid arguments");
    return -1;

}


static const char *midi_converter_clock_domain_docstring_ = "";

static const char *midi_converter_parameter_input_docstring_ = "";

static const char *midi_converter_set_mapping_docstring_ = "";

static const char *midi_converter_get_mapping_docstring_ = "";

static const char *midi_converter_parameter_name_changed_docstring_ = "";

static const char *midi_converter_map_param_docstring_ = "";

static const char *midi_converter_map_midi_docstring_ = "";

static const char *midi_converter_unmap_param_docstring_ = "";

static const char *midi_converter_unmap_midi_docstring_ = "";

static const char *midi_converter_is_mapped_param_docstring_ = "";

static const char *midi_converter_is_mapped_midi_docstring_ = "";

static const char *midi_converter_get_info_param_docstring_ = "";

static const char *midi_converter_get_info_midi_docstring_ = "";

static const char *midi_converter_set_minimum_decimation_docstring_ = "";

static const char *midi_converter_set_midi_notes_docstring_ = "";

static const char *midi_converter_set_midi_pitchbend_docstring_ = "";

static const char *midi_converter_set_midi_channel_docstring_ = "";

static const char *midi_converter_set_min_midi_channel_docstring_ = "";

static const char *midi_converter_set_max_midi_channel_docstring_ = "";

static const char *midi_converter_set_program_change_docstring_ = "";

static const char *midi_converter_change_program_docstring_ = "";

static const char *midi_converter_set_bank_change_docstring_ = "";

static const char *midi_converter_change_bank_docstring_ = "";

static const char *midi_converter_set_cc_docstring_ = "";

static const char *midi_converter_change_cc_docstring_ = "";

static const char *midi_converter_close_docstring_ = "";




static PyMethodDef midi_converter_methods_[] = {
    
        { "clock_domain",(PyCFunction)midi_converter_wrapper_::clock_domain_method_,METH_VARARGS,(char *)midi_converter_clock_domain_docstring_ },
    
        { "parameter_input",(PyCFunction)midi_converter_wrapper_::parameter_input_method_,METH_VARARGS,(char *)midi_converter_parameter_input_docstring_ },
    
        { "set_mapping",(PyCFunction)midi_converter_wrapper_::set_mapping_method_,METH_VARARGS,(char *)midi_converter_set_mapping_docstring_ },
    
        { "get_mapping",(PyCFunction)midi_converter_wrapper_::get_mapping_method_,METH_VARARGS,(char *)midi_converter_get_mapping_docstring_ },
    
        { "parameter_name_changed",(PyCFunction)midi_converter_wrapper_::parameter_name_changed_method_,METH_VARARGS,(char *)midi_converter_parameter_name_changed_docstring_ },
    
        { "map_param",(PyCFunction)midi_converter_wrapper_::map_param_method_,METH_VARARGS,(char *)midi_converter_map_param_docstring_ },
    
        { "map_midi",(PyCFunction)midi_converter_wrapper_::map_midi_method_,METH_VARARGS,(char *)midi_converter_map_midi_docstring_ },
    
        { "unmap_param",(PyCFunction)midi_converter_wrapper_::unmap_param_method_,METH_VARARGS,(char *)midi_converter_unmap_param_docstring_ },
    
        { "unmap_midi",(PyCFunction)midi_converter_wrapper_::unmap_midi_method_,METH_VARARGS,(char *)midi_converter_unmap_midi_docstring_ },
    
        { "is_mapped_param",(PyCFunction)midi_converter_wrapper_::is_mapped_param_method_,METH_VARARGS,(char *)midi_converter_is_mapped_param_docstring_ },
    
        { "is_mapped_midi",(PyCFunction)midi_converter_wrapper_::is_mapped_midi_method_,METH_VARARGS,(char *)midi_converter_is_mapped_midi_docstring_ },
    
        { "get_info_param",(PyCFunction)midi_converter_wrapper_::get_info_param_method_,METH_VARARGS,(char *)midi_converter_get_info_param_docstring_ },
    
        { "get_info_midi",(PyCFunction)midi_converter_wrapper_::get_info_midi_method_,METH_VARARGS,(char *)midi_converter_get_info_midi_docstring_ },
    
        { "set_minimum_decimation",(PyCFunction)midi_converter_wrapper_::set_minimum_decimation_method_,METH_VARARGS,(char *)midi_converter_set_minimum_decimation_docstring_ },
    
        { "set_midi_notes",(PyCFunction)midi_converter_wrapper_::set_midi_notes_method_,METH_VARARGS,(char *)midi_converter_set_midi_notes_docstring_ },
    
        { "set_midi_pitchbend",(PyCFunction)midi_converter_wrapper_::set_midi_pitchbend_method_,METH_VARARGS,(char *)midi_converter_set_midi_pitchbend_docstring_ },
    
        { "set_midi_channel",(PyCFunction)midi_converter_wrapper_::set_midi_channel_method_,METH_VARARGS,(char *)midi_converter_set_midi_channel_docstring_ },
    
        { "set_min_midi_channel",(PyCFunction)midi_converter_wrapper_::set_min_midi_channel_method_,METH_VARARGS,(char *)midi_converter_set_min_midi_channel_docstring_ },
    
        { "set_max_midi_channel",(PyCFunction)midi_converter_wrapper_::set_max_midi_channel_method_,METH_VARARGS,(char *)midi_converter_set_max_midi_channel_docstring_ },
    
        { "set_program_change",(PyCFunction)midi_converter_wrapper_::set_program_change_method_,METH_VARARGS,(char *)midi_converter_set_program_change_docstring_ },
    
        { "change_program",(PyCFunction)midi_converter_wrapper_::change_program_method_,METH_VARARGS,(char *)midi_converter_change_program_docstring_ },
    
        { "set_bank_change",(PyCFunction)midi_converter_wrapper_::set_bank_change_method_,METH_VARARGS,(char *)midi_converter_set_bank_change_docstring_ },
    
        { "change_bank",(PyCFunction)midi_converter_wrapper_::change_bank_method_,METH_VARARGS,(char *)midi_converter_change_bank_docstring_ },
    
        { "set_cc",(PyCFunction)midi_converter_wrapper_::set_cc_method_,METH_VARARGS,(char *)midi_converter_set_cc_docstring_ },
    
        { "change_cc",(PyCFunction)midi_converter_wrapper_::change_cc_method_,METH_VARARGS,(char *)midi_converter_change_cc_docstring_ },
    
        { "close",(PyCFunction)midi_converter_wrapper_::close_method_,METH_VARARGS,(char *)midi_converter_close_docstring_ },
    
    
        { "py_lock",(PyCFunction)midi_converter_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)midi_converter_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *midi_converter_docstring_ = "";

PyTypeObject midi_converter_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.midi_converter",/*tp_name*/
    sizeof(midi_converter_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)midi_converter_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)midi_converter_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    midi_converter_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)midi_converter_init_,  /* tp_init */
    0,                         /* tp_alloc */
    midi_converter_new_,             /* tp_new */
};

PyObject *midi_converter_wrapcopy_(midi_converter_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *midi_converter_wrap_(midi_converter_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = midi_converter_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    midi_converter_object_ *self = 
        PyObject_NEW(midi_converter_object_, &midi_converter_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        midi_converter_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

midi_converter_type_ *midi_converter_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &midi_converter_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected midi_converter object");
        return 0;
    }

    midi_converter_type_ *object = static_cast<midi_converter_type_ *>(((midi_converter_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null midi_converter object in unwrap");
        return 0;
    }

    return object;
}

void midi_converter_insert_(const std::pair< midi_converter_type_ *, PyObject *> &p)
{




    midi_converter_wrapped_.insert(p);


}

void midi_converter_erase_(midi_converter_type_ *p)
{




    midi_converter_wrapped_.erase(p);


}

PyObject * midi_converter_find_(midi_converter_type_ *p)
{




    PyObject *ret = 0;
    std::map< midi_converter_type_ *, PyObject *>::iterator where = midi_converter_wrapped_.find(p);
    if(where != midi_converter_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct midi_converter_dispatch_ midi_converter_realdispatcher_ =
{
    &midi_converter_typeobject_,
    midi_converter_wrap_,
    midi_converter_wrapcopy_,
    midi_converter_unwrap_,
    midi_converter_insert_,
    midi_converter_erase_,
    midi_converter_find_
};

  

struct mapping_observer_object_ {
    PyObject_HEAD
    mapping_observer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct mapping_observer_wrapper_: public mapping_observer_type_ 
{

    static void __deleter__(mapping_observer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
    }
*/



    mapping_observer_wrapper_( PyObject *o, PyInterpreterState *i  ): mapping_observer_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~mapping_observer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void mapping_changed(  const std::string & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"mapping_changed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_stdstr,&a0);
        //printf("mapping_changed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("mapping_changed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void settings_changed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"settings_changed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("settings_changed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("settings_changed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void parameter_changed(  unsigned int a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"parameter_changed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_ui,&a0);
        //printf("parameter_changed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("parameter_changed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    std::string get_parameter_name(  unsigned int a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_parameter_name");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_ui,&a0);
        //printf("get_parameter_name before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_parameter_name after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }

















    void mapping_changed_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method mapping_observer::mapping_changed");
                
                
            
            
        }

        return
            
                mapping_changed
            
            
                ( a0 );
    }

    static PyObject *mapping_changed_method_(mapping_observer_object_ *self, PyObject *args)
    {
         std::string a0; 

        mapping_observer_wrapper_ *o = (mapping_observer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_observer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->mapping_changed_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_observer::mapping_changed");
        }

        return 0;
    }



    void settings_changed_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method mapping_observer::settings_changed");
                
                
            
            
        }

        return
            
                settings_changed
            
            
                ();
    }

    static PyObject *settings_changed_method_(mapping_observer_object_ *self, PyObject *args)
    {
        

        mapping_observer_wrapper_ *o = (mapping_observer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_observer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->settings_changed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_observer::settings_changed");
        }

        return 0;
    }



    void parameter_changed_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method mapping_observer::parameter_changed");
                
                
            
            
        }

        return
            
                parameter_changed
            
            
                ( a0 );
    }

    static PyObject *parameter_changed_method_(mapping_observer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        mapping_observer_wrapper_ *o = (mapping_observer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_observer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->parameter_changed_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_observer::parameter_changed");
        }

        return 0;
    }



    std::string get_parameter_name_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method mapping_observer::get_parameter_name");
                
                
            
            
        }

        return
            
                get_parameter_name
            
            
                ( a0 );
    }

    static PyObject *get_parameter_name_method_(mapping_observer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        mapping_observer_wrapper_ *o = (mapping_observer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mapping_observer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_parameter_name_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mapping_observer::get_parameter_name");
        }

        return 0;
    }





    static PyObject *_method_lock(mapping_observer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(mapping_observer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void mapping_observer_delete_(mapping_observer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            mapping_observer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (mapping_observer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        mapping_observer_wrapper_::__deleter__((mapping_observer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void mapping_observer_dealloc_(mapping_observer_object_* self)
{
    
    mapping_observer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * mapping_observer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    mapping_observer_object_ *self;

    if(!(self = (mapping_observer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int mapping_observer_init_(mapping_observer_object_ *self, PyObject *args, PyObject *kwds)
{
    mapping_observer_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new mapping_observer_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing mapping_observer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            mapping_observer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"mapping_observer invalid arguments");
    return -1;

}


static const char *mapping_observer_mapping_changed_docstring_ = "";

static const char *mapping_observer_settings_changed_docstring_ = "";

static const char *mapping_observer_parameter_changed_docstring_ = "";

static const char *mapping_observer_get_parameter_name_docstring_ = "";




static PyMethodDef mapping_observer_methods_[] = {
    
        { "mapping_changed",(PyCFunction)mapping_observer_wrapper_::mapping_changed_method_,METH_VARARGS,(char *)mapping_observer_mapping_changed_docstring_ },
    
        { "settings_changed",(PyCFunction)mapping_observer_wrapper_::settings_changed_method_,METH_VARARGS,(char *)mapping_observer_settings_changed_docstring_ },
    
        { "parameter_changed",(PyCFunction)mapping_observer_wrapper_::parameter_changed_method_,METH_VARARGS,(char *)mapping_observer_parameter_changed_docstring_ },
    
        { "get_parameter_name",(PyCFunction)mapping_observer_wrapper_::get_parameter_name_method_,METH_VARARGS,(char *)mapping_observer_get_parameter_name_docstring_ },
    
    
        { "py_lock",(PyCFunction)mapping_observer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)mapping_observer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *mapping_observer_docstring_ = "";

PyTypeObject mapping_observer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.mapping_observer",/*tp_name*/
    sizeof(mapping_observer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)mapping_observer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)mapping_observer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    mapping_observer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)mapping_observer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    mapping_observer_new_,             /* tp_new */
};

PyObject *mapping_observer_wrapcopy_(mapping_observer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *mapping_observer_wrap_(mapping_observer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = mapping_observer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    mapping_observer_object_ *self = 
        PyObject_NEW(mapping_observer_object_, &mapping_observer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        mapping_observer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

mapping_observer_type_ *mapping_observer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &mapping_observer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected mapping_observer object");
        return 0;
    }

    mapping_observer_type_ *object = static_cast<mapping_observer_type_ *>(((mapping_observer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null mapping_observer object in unwrap");
        return 0;
    }

    return object;
}

void mapping_observer_insert_(const std::pair< mapping_observer_type_ *, PyObject *> &p)
{




    mapping_observer_wrapped_.insert(p);


}

void mapping_observer_erase_(mapping_observer_type_ *p)
{




    mapping_observer_wrapped_.erase(p);


}

PyObject * mapping_observer_find_(mapping_observer_type_ *p)
{




    PyObject *ret = 0;
    std::map< mapping_observer_type_ *, PyObject *>::iterator where = mapping_observer_wrapped_.find(p);
    if(where != mapping_observer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct mapping_observer_dispatch_ mapping_observer_realdispatcher_ =
{
    &mapping_observer_typeobject_,
    mapping_observer_wrap_,
    mapping_observer_wrapcopy_,
    mapping_observer_unwrap_,
    mapping_observer_insert_,
    mapping_observer_erase_,
    mapping_observer_find_
};

  

struct kgroup_mapper_object_ {
    PyObject_HEAD
    kgroup_mapper_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct kgroup_mapper_wrapper_: public kgroup_mapper_type_ 
{

    static void __deleter__(kgroup_mapper_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    kgroup_mapper_wrapper_( PyObject *o, PyInterpreterState *i  ): kgroup_mapper_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~kgroup_mapper_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    d2d_nb_type_ key_filter_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return kgroup_mapper_type_::key_filter();
                
            
            
        }

        return
            
                key_filter
            
            
                ();
    }

    static PyObject *key_filter_method_(kgroup_mapper_object_ *self, PyObject *args)
    {
        

        kgroup_mapper_wrapper_ *o = (kgroup_mapper_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kgroup_mapper c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    d2d_nb_type_ r = o->key_filter_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kgroup_mapper::key_filter");
        }

        return 0;
    }



    d2d_nb_type_ light_filter_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return kgroup_mapper_type_::light_filter();
                
            
            
        }

        return
            
                light_filter
            
            
                ();
    }

    static PyObject *light_filter_method_(kgroup_mapper_object_ *self, PyObject *args)
    {
        

        kgroup_mapper_wrapper_ *o = (kgroup_mapper_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kgroup_mapper c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    d2d_nb_type_ r = o->light_filter_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kgroup_mapper::light_filter");
        }

        return 0;
    }



    void clear_mapping_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return kgroup_mapper_type_::clear_mapping();
                
            
            
        }

        return
            
                clear_mapping
            
            
                ();
    }

    static PyObject *clear_mapping_method_(kgroup_mapper_object_ *self, PyObject *args)
    {
        

        kgroup_mapper_wrapper_ *o = (kgroup_mapper_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kgroup_mapper c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_mapping_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kgroup_mapper::clear_mapping");
        }

        return 0;
    }



    void activate_mapping_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return kgroup_mapper_type_::activate_mapping();
                
            
            
        }

        return
            
                activate_mapping
            
            
                ();
    }

    static PyObject *activate_mapping_method_(kgroup_mapper_object_ *self, PyObject *args)
    {
        

        kgroup_mapper_wrapper_ *o = (kgroup_mapper_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kgroup_mapper c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->activate_mapping_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kgroup_mapper::activate_mapping");
        }

        return 0;
    }



    void set_mapping_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return kgroup_mapper_type_::set_mapping( a0  ,a1 );
                
            
            
        }

        return
            
                set_mapping
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_mapping_method_(kgroup_mapper_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        kgroup_mapper_wrapper_ *o = (kgroup_mapper_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kgroup_mapper c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_mapping_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kgroup_mapper::set_mapping");
        }

        return 0;
    }





    static PyObject *_method_lock(kgroup_mapper_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(kgroup_mapper_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void kgroup_mapper_delete_(kgroup_mapper_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            kgroup_mapper_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (kgroup_mapper_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        kgroup_mapper_wrapper_::__deleter__((kgroup_mapper_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void kgroup_mapper_dealloc_(kgroup_mapper_object_* self)
{
    
    kgroup_mapper_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * kgroup_mapper_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    kgroup_mapper_object_ *self;

    if(!(self = (kgroup_mapper_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int kgroup_mapper_init_(kgroup_mapper_object_ *self, PyObject *args, PyObject *kwds)
{
    kgroup_mapper_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new kgroup_mapper_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing kgroup_mapper");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            kgroup_mapper_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"kgroup_mapper invalid arguments");
    return -1;

}


static const char *kgroup_mapper_key_filter_docstring_ = "";

static const char *kgroup_mapper_light_filter_docstring_ = "";

static const char *kgroup_mapper_clear_mapping_docstring_ = "";

static const char *kgroup_mapper_activate_mapping_docstring_ = "";

static const char *kgroup_mapper_set_mapping_docstring_ = "";




static PyMethodDef kgroup_mapper_methods_[] = {
    
        { "key_filter",(PyCFunction)kgroup_mapper_wrapper_::key_filter_method_,METH_VARARGS,(char *)kgroup_mapper_key_filter_docstring_ },
    
        { "light_filter",(PyCFunction)kgroup_mapper_wrapper_::light_filter_method_,METH_VARARGS,(char *)kgroup_mapper_light_filter_docstring_ },
    
        { "clear_mapping",(PyCFunction)kgroup_mapper_wrapper_::clear_mapping_method_,METH_VARARGS,(char *)kgroup_mapper_clear_mapping_docstring_ },
    
        { "activate_mapping",(PyCFunction)kgroup_mapper_wrapper_::activate_mapping_method_,METH_VARARGS,(char *)kgroup_mapper_activate_mapping_docstring_ },
    
        { "set_mapping",(PyCFunction)kgroup_mapper_wrapper_::set_mapping_method_,METH_VARARGS,(char *)kgroup_mapper_set_mapping_docstring_ },
    
    
        { "py_lock",(PyCFunction)kgroup_mapper_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)kgroup_mapper_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *kgroup_mapper_docstring_ = "";

PyTypeObject kgroup_mapper_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.kgroup_mapper",/*tp_name*/
    sizeof(kgroup_mapper_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)kgroup_mapper_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)kgroup_mapper_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    kgroup_mapper_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)kgroup_mapper_init_,  /* tp_init */
    0,                         /* tp_alloc */
    kgroup_mapper_new_,             /* tp_new */
};

PyObject *kgroup_mapper_wrapcopy_(kgroup_mapper_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *kgroup_mapper_wrap_(kgroup_mapper_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = kgroup_mapper_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    kgroup_mapper_object_ *self = 
        PyObject_NEW(kgroup_mapper_object_, &kgroup_mapper_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        kgroup_mapper_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

kgroup_mapper_type_ *kgroup_mapper_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &kgroup_mapper_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected kgroup_mapper object");
        return 0;
    }

    kgroup_mapper_type_ *object = static_cast<kgroup_mapper_type_ *>(((kgroup_mapper_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null kgroup_mapper object in unwrap");
        return 0;
    }

    return object;
}

void kgroup_mapper_insert_(const std::pair< kgroup_mapper_type_ *, PyObject *> &p)
{




    kgroup_mapper_wrapped_.insert(p);


}

void kgroup_mapper_erase_(kgroup_mapper_type_ *p)
{




    kgroup_mapper_wrapped_.erase(p);


}

PyObject * kgroup_mapper_find_(kgroup_mapper_type_ *p)
{




    PyObject *ret = 0;
    std::map< kgroup_mapper_type_ *, PyObject *>::iterator where = kgroup_mapper_wrapped_.find(p);
    if(where != kgroup_mapper_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct kgroup_mapper_dispatch_ kgroup_mapper_realdispatcher_ =
{
    &kgroup_mapper_typeobject_,
    kgroup_mapper_wrap_,
    kgroup_mapper_wrapcopy_,
    kgroup_mapper_unwrap_,
    kgroup_mapper_insert_,
    kgroup_mapper_erase_,
    kgroup_mapper_find_
};

  

struct sclone_object_ {
    PyObject_HEAD
    sclone_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct sclone_wrapper_: public sclone_type_ 
{

    static void __deleter__(sclone_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    sclone_wrapper_( PyObject *o, PyInterpreterState *i  ): sclone_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~sclone_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(sclone_object_ *self, visitproc visit, void *arg)
    {
        sclone_wrapper_ *o = (sclone_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(sclone_object_ *self)
    {
        sclone_wrapper_ *o = (sclone_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void set_output_call_(bool owned ,unsigned int a0 ,const cookie_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return sclone_type_::set_output( a0  ,a1 );
                
            
            
        }

        return
            
                set_output
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_output_method_(sclone_object_ *self, PyObject *args)
    {
         unsigned int a0;  cookie_type_ * a1; 

        sclone_wrapper_ *o = (sclone_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null sclone c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_cookie_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_output_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in sclone::set_output");
        }

        return 0;
    }



    void set_filtered_output_call_(bool owned ,unsigned int a0 ,const cookie_type_ & a1 ,const d2b_type_ & a2 )
    {
        if(owned)
        {
            
                
                
                    return sclone_type_::set_filtered_output( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                set_filtered_output
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *set_filtered_output_method_(sclone_object_ *self, PyObject *args)
    {
         unsigned int a0;  cookie_type_ * a1;  d2b_type_ * a2; 

        sclone_wrapper_ *o = (sclone_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null sclone c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_cookie_copy,&a1  ,fpcvt_d2b_copy,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_filtered_output_call_ (self->owned ,a0,*a1,*a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in sclone::set_filtered_output");
        }

        return 0;
    }



    void clear_output_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return sclone_type_::clear_output( a0 );
                
            
            
        }

        return
            
                clear_output
            
            
                ( a0 );
    }

    static PyObject *clear_output_method_(sclone_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        sclone_wrapper_ *o = (sclone_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null sclone c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_output_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in sclone::clear_output");
        }

        return 0;
    }



    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return sclone_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(sclone_object_ *self, PyObject *args)
    {
        

        sclone_wrapper_ *o = (sclone_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null sclone c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in sclone::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(sclone_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(sclone_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void sclone_delete_(sclone_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            sclone_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (sclone_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        sclone_wrapper_::__deleter__((sclone_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void sclone_dealloc_(sclone_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    sclone_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * sclone_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    sclone_object_ *self;

    if(!(self = (sclone_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int sclone_init_(sclone_object_ *self, PyObject *args, PyObject *kwds)
{
    sclone_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new sclone_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing sclone");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            sclone_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"sclone invalid arguments");
    return -1;

}


static const char *sclone_set_output_docstring_ = "";

static const char *sclone_set_filtered_output_docstring_ = "";

static const char *sclone_clear_output_docstring_ = "";

static const char *sclone_cookie_docstring_ = "";




static PyMethodDef sclone_methods_[] = {
    
        { "set_output",(PyCFunction)sclone_wrapper_::set_output_method_,METH_VARARGS,(char *)sclone_set_output_docstring_ },
    
        { "set_filtered_output",(PyCFunction)sclone_wrapper_::set_filtered_output_method_,METH_VARARGS,(char *)sclone_set_filtered_output_docstring_ },
    
        { "clear_output",(PyCFunction)sclone_wrapper_::clear_output_method_,METH_VARARGS,(char *)sclone_clear_output_docstring_ },
    
        { "cookie",(PyCFunction)sclone_wrapper_::cookie_method_,METH_VARARGS,(char *)sclone_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)sclone_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)sclone_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *sclone_docstring_ = "";

PyTypeObject sclone_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.sclone",/*tp_name*/
    sizeof(sclone_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)sclone_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)sclone_docstring_,       /* tp_doc */
    (traverseproc)sclone_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)sclone_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    sclone_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)sclone_init_,  /* tp_init */
    0,                         /* tp_alloc */
    sclone_new_,             /* tp_new */
};

PyObject *sclone_wrapcopy_(sclone_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *sclone_wrap_(sclone_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = sclone_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    sclone_object_ *self = 
        
        PyObject_GC_New(sclone_object_, &sclone_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        sclone_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

sclone_type_ *sclone_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &sclone_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected sclone object");
        return 0;
    }

    sclone_type_ *object = static_cast<sclone_type_ *>(((sclone_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null sclone object in unwrap");
        return 0;
    }

    return object;
}

void sclone_insert_(const std::pair< sclone_type_ *, PyObject *> &p)
{




    sclone_wrapped_.insert(p);


}

void sclone_erase_(sclone_type_ *p)
{




    sclone_wrapped_.erase(p);


}

PyObject * sclone_find_(sclone_type_ *p)
{




    PyObject *ret = 0;
    std::map< sclone_type_ *, PyObject *>::iterator where = sclone_wrapped_.find(p);
    if(where != sclone_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct sclone_dispatch_ sclone_realdispatcher_ =
{
    &sclone_typeobject_,
    sclone_wrap_,
    sclone_wrapcopy_,
    sclone_unwrap_,
    sclone_insert_,
    sclone_erase_,
    sclone_find_
};

  

struct clone_object_ {
    PyObject_HEAD
    clone_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct clone_wrapper_: public clone_type_ 
{

    static void __deleter__(clone_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    clone_wrapper_( PyObject *o, PyInterpreterState *i  ,bool a0  ): clone_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~clone_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(clone_object_ *self, visitproc visit, void *arg)
    {
        clone_wrapper_ *o = (clone_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(clone_object_ *self)
    {
        clone_wrapper_ *o = (clone_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void set_policy_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return clone_type_::set_policy( a0 );
                
            
            
        }

        return
            
                set_policy
            
            
                ( a0 );
    }

    static PyObject *set_policy_method_(clone_object_ *self, PyObject *args)
    {
         bool a0; 

        clone_wrapper_ *o = (clone_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clone c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_policy_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clone::set_policy");
        }

        return 0;
    }



    void set_output_call_(bool owned ,unsigned int a0 ,const cookie_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return clone_type_::set_output( a0  ,a1 );
                
            
            
        }

        return
            
                set_output
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_output_method_(clone_object_ *self, PyObject *args)
    {
         unsigned int a0;  cookie_type_ * a1; 

        clone_wrapper_ *o = (clone_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clone c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_cookie_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_output_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clone::set_output");
        }

        return 0;
    }



    void set_filtered_output_call_(bool owned ,unsigned int a0 ,const cookie_type_ & a1 ,const d2d_nb_type_ & a2 )
    {
        if(owned)
        {
            
                
                
                    return clone_type_::set_filtered_output( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                set_filtered_output
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *set_filtered_output_method_(clone_object_ *self, PyObject *args)
    {
         unsigned int a0;  cookie_type_ * a1;  d2d_nb_type_ * a2; 

        clone_wrapper_ *o = (clone_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clone c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_cookie_copy,&a1  ,fpcvt_d2d_nb_copy,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_filtered_output_call_ (self->owned ,a0,*a1,*a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clone::set_filtered_output");
        }

        return 0;
    }



    void clear_output_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return clone_type_::clear_output( a0 );
                
            
            
        }

        return
            
                clear_output
            
            
                ( a0 );
    }

    static PyObject *clear_output_method_(clone_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        clone_wrapper_ *o = (clone_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clone c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_output_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clone::clear_output");
        }

        return 0;
    }



    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clone_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(clone_object_ *self, PyObject *args)
    {
        

        clone_wrapper_ *o = (clone_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clone c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clone::cookie");
        }

        return 0;
    }



    change_nb_type_ gate_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return clone_type_::gate( a0 );
                
            
            
        }

        return
            
                gate
            
            
                ( a0 );
    }

    static PyObject *gate_method_(clone_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        clone_wrapper_ *o = (clone_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clone c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->gate_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clone::gate");
        }

        return 0;
    }



    void enable_call_(bool owned ,unsigned int a0 ,bool a1 )
    {
        if(owned)
        {
            
                
                
                    return clone_type_::enable( a0  ,a1 );
                
            
            
        }

        return
            
                enable
            
            
                ( a0  ,a1 );
    }

    static PyObject *enable_method_(clone_object_ *self, PyObject *args)
    {
         unsigned int a0;  bool a1; 

        clone_wrapper_ *o = (clone_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clone c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_b,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enable_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clone::enable");
        }

        return 0;
    }





    static PyObject *_method_lock(clone_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(clone_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void clone_delete_(clone_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            clone_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (clone_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        clone_wrapper_::__deleter__((clone_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void clone_dealloc_(clone_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    clone_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * clone_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    clone_object_ *self;

    if(!(self = (clone_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int clone_init_(clone_object_ *self, PyObject *args, PyObject *kwds)
{
    clone_delete_(self);

    

    {
         bool a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_b,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new clone_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing clone");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            clone_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"clone invalid arguments");
    return -1;

}


static const char *clone_set_policy_docstring_ = "";

static const char *clone_set_output_docstring_ = "";

static const char *clone_set_filtered_output_docstring_ = "";

static const char *clone_clear_output_docstring_ = "";

static const char *clone_cookie_docstring_ = "";

static const char *clone_gate_docstring_ = "";

static const char *clone_enable_docstring_ = "";




static PyMethodDef clone_methods_[] = {
    
        { "set_policy",(PyCFunction)clone_wrapper_::set_policy_method_,METH_VARARGS,(char *)clone_set_policy_docstring_ },
    
        { "set_output",(PyCFunction)clone_wrapper_::set_output_method_,METH_VARARGS,(char *)clone_set_output_docstring_ },
    
        { "set_filtered_output",(PyCFunction)clone_wrapper_::set_filtered_output_method_,METH_VARARGS,(char *)clone_set_filtered_output_docstring_ },
    
        { "clear_output",(PyCFunction)clone_wrapper_::clear_output_method_,METH_VARARGS,(char *)clone_clear_output_docstring_ },
    
        { "cookie",(PyCFunction)clone_wrapper_::cookie_method_,METH_VARARGS,(char *)clone_cookie_docstring_ },
    
        { "gate",(PyCFunction)clone_wrapper_::gate_method_,METH_VARARGS,(char *)clone_gate_docstring_ },
    
        { "enable",(PyCFunction)clone_wrapper_::enable_method_,METH_VARARGS,(char *)clone_enable_docstring_ },
    
    
        { "py_lock",(PyCFunction)clone_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)clone_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *clone_docstring_ = "";

PyTypeObject clone_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.clone",/*tp_name*/
    sizeof(clone_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)clone_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)clone_docstring_,       /* tp_doc */
    (traverseproc)clone_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)clone_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    clone_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)clone_init_,  /* tp_init */
    0,                         /* tp_alloc */
    clone_new_,             /* tp_new */
};

PyObject *clone_wrapcopy_(clone_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *clone_wrap_(clone_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = clone_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    clone_object_ *self = 
        
        PyObject_GC_New(clone_object_, &clone_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        clone_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

clone_type_ *clone_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &clone_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected clone object");
        return 0;
    }

    clone_type_ *object = static_cast<clone_type_ *>(((clone_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null clone object in unwrap");
        return 0;
    }

    return object;
}

void clone_insert_(const std::pair< clone_type_ *, PyObject *> &p)
{




    clone_wrapped_.insert(p);


}

void clone_erase_(clone_type_ *p)
{




    clone_wrapped_.erase(p);


}

PyObject * clone_find_(clone_type_ *p)
{




    PyObject *ret = 0;
    std::map< clone_type_ *, PyObject *>::iterator where = clone_wrapped_.find(p);
    if(where != clone_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct clone_dispatch_ clone_realdispatcher_ =
{
    &clone_typeobject_,
    clone_wrap_,
    clone_wrapcopy_,
    clone_unwrap_,
    clone_insert_,
    clone_erase_,
    clone_find_
};

  

struct aggregator_object_ {
    PyObject_HEAD
    aggregator_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct aggregator_wrapper_: public aggregator_type_ 
{

    static void __deleter__(aggregator_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    aggregator_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): aggregator_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~aggregator_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(aggregator_object_ *self, visitproc visit, void *arg)
    {
        aggregator_wrapper_ *o = (aggregator_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(aggregator_object_ *self)
    {
        aggregator_wrapper_ *o = (aggregator_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    cookie_type_ get_filtered_output_call_(bool owned ,unsigned int a0 ,const d2d_nb_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return aggregator_type_::get_filtered_output( a0  ,a1 );
                
            
            
        }

        return
            
                get_filtered_output
            
            
                ( a0  ,a1 );
    }

    static PyObject *get_filtered_output_method_(aggregator_object_ *self, PyObject *args)
    {
         unsigned int a0;  d2d_nb_type_ * a1; 

        aggregator_wrapper_ *o = (aggregator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null aggregator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_d2d_nb_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->get_filtered_output_call_ (self->owned ,a0 ,*a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in aggregator::get_filtered_output");
        }

        return 0;
    }



    cookie_type_ get_output_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return aggregator_type_::get_output( a0 );
                
            
            
        }

        return
            
                get_output
            
            
                ( a0 );
    }

    static PyObject *get_output_method_(aggregator_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        aggregator_wrapper_ *o = (aggregator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null aggregator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->get_output_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in aggregator::get_output");
        }

        return 0;
    }



    void clear_output_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return aggregator_type_::clear_output( a0 );
                
            
            
        }

        return
            
                clear_output
            
            
                ( a0 );
    }

    static PyObject *clear_output_method_(aggregator_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        aggregator_wrapper_ *o = (aggregator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null aggregator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_output_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in aggregator::clear_output");
        }

        return 0;
    }



    unsigned int size_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return aggregator_type_::size();
                
            
            
        }

        return
            
                size
            
            
                ();
    }

    static PyObject *size_method_(aggregator_object_ *self, PyObject *args)
    {
        

        aggregator_wrapper_ *o = (aggregator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null aggregator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->size_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in aggregator::size");
        }

        return 0;
    }





    static PyObject *_method_lock(aggregator_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(aggregator_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void aggregator_delete_(aggregator_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            aggregator_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (aggregator_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        aggregator_wrapper_::__deleter__((aggregator_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void aggregator_dealloc_(aggregator_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    aggregator_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * aggregator_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    aggregator_object_ *self;

    if(!(self = (aggregator_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int aggregator_init_(aggregator_object_ *self, PyObject *args, PyObject *kwds)
{
    aggregator_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new aggregator_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing aggregator");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            aggregator_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"aggregator invalid arguments");
    return -1;

}


static const char *aggregator_get_filtered_output_docstring_ = "";

static const char *aggregator_get_output_docstring_ = ""
"        self.get_output(output_name) -> cookie for new output"
"        "
"        Create a new output bundle named output_name (integer)"
"        the output will appear as a bundle named output_name in"
"        the top level of the aggregators output.  The output"
"        must currently not exist."
"        "
""
;

static const char *aggregator_clear_output_docstring_ = ""
"        self.clear_output(output_name) -> ."
"        "
"        Clear the given output from the aggregated output.  Any"
"        user of the output will continue but output data will be"
"        thrown away."
"        "
""
;

static const char *aggregator_size_docstring_ = "";




static PyMethodDef aggregator_methods_[] = {
    
        { "get_filtered_output",(PyCFunction)aggregator_wrapper_::get_filtered_output_method_,METH_VARARGS,(char *)aggregator_get_filtered_output_docstring_ },
    
        { "get_output",(PyCFunction)aggregator_wrapper_::get_output_method_,METH_VARARGS,(char *)aggregator_get_output_docstring_ },
    
        { "clear_output",(PyCFunction)aggregator_wrapper_::clear_output_method_,METH_VARARGS,(char *)aggregator_clear_output_docstring_ },
    
        { "size",(PyCFunction)aggregator_wrapper_::size_method_,METH_VARARGS,(char *)aggregator_size_docstring_ },
    
    
        { "py_lock",(PyCFunction)aggregator_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)aggregator_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *aggregator_docstring_ = ""
"    Aggregator stream class."
"    "
"    Aggregate a number of streams into one, which each stream appearing in"
"    the output at a specified name.  The streams to aggregate may be dynamically"
"    changed."
"    "
"    Initialisers"
"    ============"
"    "
"    aggregator(cookie, clockdomain)"
"        cookie: cookie from downstream"
"    "
""
;

PyTypeObject aggregator_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.aggregator",/*tp_name*/
    sizeof(aggregator_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)aggregator_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)aggregator_docstring_,       /* tp_doc */
    (traverseproc)aggregator_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)aggregator_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    aggregator_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)aggregator_init_,  /* tp_init */
    0,                         /* tp_alloc */
    aggregator_new_,             /* tp_new */
};

PyObject *aggregator_wrapcopy_(aggregator_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *aggregator_wrap_(aggregator_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = aggregator_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    aggregator_object_ *self = 
        
        PyObject_GC_New(aggregator_object_, &aggregator_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        aggregator_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

aggregator_type_ *aggregator_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &aggregator_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected aggregator object");
        return 0;
    }

    aggregator_type_ *object = static_cast<aggregator_type_ *>(((aggregator_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null aggregator object in unwrap");
        return 0;
    }

    return object;
}

void aggregator_insert_(const std::pair< aggregator_type_ *, PyObject *> &p)
{




    aggregator_wrapped_.insert(p);


}

void aggregator_erase_(aggregator_type_ *p)
{




    aggregator_wrapped_.erase(p);


}

PyObject * aggregator_find_(aggregator_type_ *p)
{




    PyObject *ret = 0;
    std::map< aggregator_type_ *, PyObject *>::iterator where = aggregator_wrapped_.find(p);
    if(where != aggregator_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct aggregator_dispatch_ aggregator_realdispatcher_ =
{
    &aggregator_typeobject_,
    aggregator_wrap_,
    aggregator_wrapcopy_,
    aggregator_unwrap_,
    aggregator_insert_,
    aggregator_erase_,
    aggregator_find_
};

  

struct scrolldelegate_object_ {
    PyObject_HEAD
    scrolldelegate_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct scrolldelegate_wrapper_: public scrolldelegate_type_ 
{

    static void __deleter__(scrolldelegate_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
    }
*/



    scrolldelegate_wrapper_( PyObject *o, PyInterpreterState *i  ): scrolldelegate_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~scrolldelegate_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void scroll(  float a0  ,float a1  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"scroll");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&)" ,tpcvt_f,&a0,tpcvt_f,&a1);
        //printf("scroll before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("scroll after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void tap(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"tap");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("tap before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("tap after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    void scroll_call_(bool owned ,float a0 ,float a1 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method scrolldelegate::scroll");
                
                
            
            
        }

        return
            
                scroll
            
            
                ( a0  ,a1 );
    }

    static PyObject *scroll_method_(scrolldelegate_object_ *self, PyObject *args)
    {
         float a0;  float a1; 

        scrolldelegate_wrapper_ *o = (scrolldelegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scrolldelegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_f,&a0  ,fpcvt_f,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->scroll_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scrolldelegate::scroll");
        }

        return 0;
    }



    void tap_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method scrolldelegate::tap");
                
                
            
            
        }

        return
            
                tap
            
            
                ();
    }

    static PyObject *tap_method_(scrolldelegate_object_ *self, PyObject *args)
    {
        

        scrolldelegate_wrapper_ *o = (scrolldelegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scrolldelegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->tap_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scrolldelegate::tap");
        }

        return 0;
    }





    static PyObject *_method_lock(scrolldelegate_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(scrolldelegate_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void scrolldelegate_delete_(scrolldelegate_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            scrolldelegate_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (scrolldelegate_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        scrolldelegate_wrapper_::__deleter__((scrolldelegate_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void scrolldelegate_dealloc_(scrolldelegate_object_* self)
{
    
    scrolldelegate_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * scrolldelegate_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    scrolldelegate_object_ *self;

    if(!(self = (scrolldelegate_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int scrolldelegate_init_(scrolldelegate_object_ *self, PyObject *args, PyObject *kwds)
{
    scrolldelegate_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new scrolldelegate_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing scrolldelegate");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            scrolldelegate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"scrolldelegate invalid arguments");
    return -1;

}


static const char *scrolldelegate_scroll_docstring_ = "";

static const char *scrolldelegate_tap_docstring_ = "";




static PyMethodDef scrolldelegate_methods_[] = {
    
        { "scroll",(PyCFunction)scrolldelegate_wrapper_::scroll_method_,METH_VARARGS,(char *)scrolldelegate_scroll_docstring_ },
    
        { "tap",(PyCFunction)scrolldelegate_wrapper_::tap_method_,METH_VARARGS,(char *)scrolldelegate_tap_docstring_ },
    
    
        { "py_lock",(PyCFunction)scrolldelegate_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)scrolldelegate_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *scrolldelegate_docstring_ = "";

PyTypeObject scrolldelegate_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.scrolldelegate",/*tp_name*/
    sizeof(scrolldelegate_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)scrolldelegate_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)scrolldelegate_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    scrolldelegate_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)scrolldelegate_init_,  /* tp_init */
    0,                         /* tp_alloc */
    scrolldelegate_new_,             /* tp_new */
};

PyObject *scrolldelegate_wrapcopy_(scrolldelegate_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *scrolldelegate_wrap_(scrolldelegate_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = scrolldelegate_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    scrolldelegate_object_ *self = 
        PyObject_NEW(scrolldelegate_object_, &scrolldelegate_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        scrolldelegate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

scrolldelegate_type_ *scrolldelegate_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &scrolldelegate_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected scrolldelegate object");
        return 0;
    }

    scrolldelegate_type_ *object = static_cast<scrolldelegate_type_ *>(((scrolldelegate_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null scrolldelegate object in unwrap");
        return 0;
    }

    return object;
}

void scrolldelegate_insert_(const std::pair< scrolldelegate_type_ *, PyObject *> &p)
{




    scrolldelegate_wrapped_.insert(p);


}

void scrolldelegate_erase_(scrolldelegate_type_ *p)
{




    scrolldelegate_wrapped_.erase(p);


}

PyObject * scrolldelegate_find_(scrolldelegate_type_ *p)
{




    PyObject *ret = 0;
    std::map< scrolldelegate_type_ *, PyObject *>::iterator where = scrolldelegate_wrapped_.find(p);
    if(where != scrolldelegate_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct scrolldelegate_dispatch_ scrolldelegate_realdispatcher_ =
{
    &scrolldelegate_typeobject_,
    scrolldelegate_wrap_,
    scrolldelegate_wrapcopy_,
    scrolldelegate_unwrap_,
    scrolldelegate_insert_,
    scrolldelegate_erase_,
    scrolldelegate_find_
};

  

struct scroller_object_ {
    PyObject_HEAD
    scroller_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct scroller_wrapper_: public scroller_type_ 
{

    static void __deleter__(scroller_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    scroller_wrapper_( PyObject *o, PyInterpreterState *i  ,scrolldelegate_type_ * a0  ,float a1  ,float a2  ,unsigned int a3  ): scroller_type_(  a0 ,a1 ,a2 ,a3 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~scroller_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scroller_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(scroller_object_ *self, PyObject *args)
    {
        

        scroller_wrapper_ *o = (scroller_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scroller c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scroller::cookie");
        }

        return 0;
    }



    void reset_call_(bool owned ,float a0 ,float a1 )
    {
        if(owned)
        {
            
                
                
                    return scroller_type_::reset( a0  ,a1 );
                
            
            
        }

        return
            
                reset
            
            
                ( a0  ,a1 );
    }

    static PyObject *reset_method_(scroller_object_ *self, PyObject *args)
    {
         float a0;  float a1; 

        scroller_wrapper_ *o = (scroller_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scroller c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_f,&a0  ,fpcvt_f,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->reset_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scroller::reset");
        }

        return 0;
    }



    void reset_v_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return scroller_type_::reset_v( a0 );
                
            
            
        }

        return
            
                reset_v
            
            
                ( a0 );
    }

    static PyObject *reset_v_method_(scroller_object_ *self, PyObject *args)
    {
         float a0; 

        scroller_wrapper_ *o = (scroller_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scroller c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->reset_v_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scroller::reset_v");
        }

        return 0;
    }



    void reset_h_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return scroller_type_::reset_h( a0 );
                
            
            
        }

        return
            
                reset_h
            
            
                ( a0 );
    }

    static PyObject *reset_h_method_(scroller_object_ *self, PyObject *args)
    {
         float a0; 

        scroller_wrapper_ *o = (scroller_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scroller c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->reset_h_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scroller::reset_h");
        }

        return 0;
    }



    void enable_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scroller_type_::enable();
                
            
            
        }

        return
            
                enable
            
            
                ();
    }

    static PyObject *enable_method_(scroller_object_ *self, PyObject *args)
    {
        

        scroller_wrapper_ *o = (scroller_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scroller c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enable_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scroller::enable");
        }

        return 0;
    }



    void disable_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scroller_type_::disable();
                
            
            
        }

        return
            
                disable
            
            
                ();
    }

    static PyObject *disable_method_(scroller_object_ *self, PyObject *args)
    {
        

        scroller_wrapper_ *o = (scroller_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scroller c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->disable_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scroller::disable");
        }

        return 0;
    }



    void set_scheme_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return scroller_type_::set_scheme( a0 );
                
            
            
        }

        return
            
                set_scheme
            
            
                ( a0 );
    }

    static PyObject *set_scheme_method_(scroller_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        scroller_wrapper_ *o = (scroller_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scroller c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_scheme_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scroller::set_scheme");
        }

        return 0;
    }





    static PyObject *_method_lock(scroller_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(scroller_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void scroller_delete_(scroller_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            scroller_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (scroller_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        scroller_wrapper_::__deleter__((scroller_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void scroller_dealloc_(scroller_object_* self)
{
    
    scroller_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * scroller_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    scroller_object_ *self;

    if(!(self = (scroller_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int scroller_init_(scroller_object_ *self, PyObject *args, PyObject *kwds)
{
    scroller_delete_(self);

    

    {
         scrolldelegate_type_ * a0;  float a1;  float a2;  unsigned int a3; 

        if(PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_scrolldelegate,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_ui,&a3))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new scroller_wrapper_((PyObject *)self,interp ,a0,a1,a2,a3);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing scroller");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            scroller_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"scroller invalid arguments");
    return -1;

}


static const char *scroller_cookie_docstring_ = "";

static const char *scroller_reset_docstring_ = "";

static const char *scroller_reset_v_docstring_ = "";

static const char *scroller_reset_h_docstring_ = "";

static const char *scroller_enable_docstring_ = "";

static const char *scroller_disable_docstring_ = "";

static const char *scroller_set_scheme_docstring_ = "";




static PyMethodDef scroller_methods_[] = {
    
        { "cookie",(PyCFunction)scroller_wrapper_::cookie_method_,METH_VARARGS,(char *)scroller_cookie_docstring_ },
    
        { "reset",(PyCFunction)scroller_wrapper_::reset_method_,METH_VARARGS,(char *)scroller_reset_docstring_ },
    
        { "reset_v",(PyCFunction)scroller_wrapper_::reset_v_method_,METH_VARARGS,(char *)scroller_reset_v_docstring_ },
    
        { "reset_h",(PyCFunction)scroller_wrapper_::reset_h_method_,METH_VARARGS,(char *)scroller_reset_h_docstring_ },
    
        { "enable",(PyCFunction)scroller_wrapper_::enable_method_,METH_VARARGS,(char *)scroller_enable_docstring_ },
    
        { "disable",(PyCFunction)scroller_wrapper_::disable_method_,METH_VARARGS,(char *)scroller_disable_docstring_ },
    
        { "set_scheme",(PyCFunction)scroller_wrapper_::set_scheme_method_,METH_VARARGS,(char *)scroller_set_scheme_docstring_ },
    
    
        { "py_lock",(PyCFunction)scroller_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)scroller_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *scroller_docstring_ = "";

PyTypeObject scroller_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.scroller",/*tp_name*/
    sizeof(scroller_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)scroller_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)scroller_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    scroller_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)scroller_init_,  /* tp_init */
    0,                         /* tp_alloc */
    scroller_new_,             /* tp_new */
};

PyObject *scroller_wrapcopy_(scroller_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *scroller_wrap_(scroller_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = scroller_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    scroller_object_ *self = 
        PyObject_NEW(scroller_object_, &scroller_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        scroller_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

scroller_type_ *scroller_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &scroller_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected scroller object");
        return 0;
    }

    scroller_type_ *object = static_cast<scroller_type_ *>(((scroller_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null scroller object in unwrap");
        return 0;
    }

    return object;
}

void scroller_insert_(const std::pair< scroller_type_ *, PyObject *> &p)
{




    scroller_wrapped_.insert(p);


}

void scroller_erase_(scroller_type_ *p)
{




    scroller_wrapped_.erase(p);


}

PyObject * scroller_find_(scroller_type_ *p)
{




    PyObject *ret = 0;
    std::map< scroller_type_ *, PyObject *>::iterator where = scroller_wrapped_.find(p);
    if(where != scroller_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct scroller_dispatch_ scroller_realdispatcher_ =
{
    &scroller_typeobject_,
    scroller_wrap_,
    scroller_wrapcopy_,
    scroller_unwrap_,
    scroller_insert_,
    scroller_erase_,
    scroller_find_
};

  

struct scroller2_object_ {
    PyObject_HEAD
    scroller2_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct scroller2_wrapper_: public scroller2_type_ 
{

    static void __deleter__(scroller2_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    scroller2_wrapper_( PyObject *o, PyInterpreterState *i  ,const change_type_ & a0  ): scroller2_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~scroller2_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scroller2_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(scroller2_object_ *self, PyObject *args)
    {
        

        scroller2_wrapper_ *o = (scroller2_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scroller2 c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scroller2::cookie");
        }

        return 0;
    }



    void enable_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scroller2_type_::enable();
                
            
            
        }

        return
            
                enable
            
            
                ();
    }

    static PyObject *enable_method_(scroller2_object_ *self, PyObject *args)
    {
        

        scroller2_wrapper_ *o = (scroller2_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scroller2 c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enable_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scroller2::enable");
        }

        return 0;
    }



    void disable_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scroller2_type_::disable();
                
            
            
        }

        return
            
                disable
            
            
                ();
    }

    static PyObject *disable_method_(scroller2_object_ *self, PyObject *args)
    {
        

        scroller2_wrapper_ *o = (scroller2_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scroller2 c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->disable_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scroller2::disable");
        }

        return 0;
    }





    static PyObject *_method_lock(scroller2_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(scroller2_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void scroller2_delete_(scroller2_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            scroller2_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (scroller2_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        scroller2_wrapper_::__deleter__((scroller2_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void scroller2_dealloc_(scroller2_object_* self)
{
    
    scroller2_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * scroller2_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    scroller2_object_ *self;

    if(!(self = (scroller2_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int scroller2_init_(scroller2_object_ *self, PyObject *args, PyObject *kwds)
{
    scroller2_delete_(self);

    

    {
         change_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_change_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new scroller2_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing scroller2");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            scroller2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"scroller2 invalid arguments");
    return -1;

}


static const char *scroller2_cookie_docstring_ = "";

static const char *scroller2_enable_docstring_ = "";

static const char *scroller2_disable_docstring_ = "";




static PyMethodDef scroller2_methods_[] = {
    
        { "cookie",(PyCFunction)scroller2_wrapper_::cookie_method_,METH_VARARGS,(char *)scroller2_cookie_docstring_ },
    
        { "enable",(PyCFunction)scroller2_wrapper_::enable_method_,METH_VARARGS,(char *)scroller2_enable_docstring_ },
    
        { "disable",(PyCFunction)scroller2_wrapper_::disable_method_,METH_VARARGS,(char *)scroller2_disable_docstring_ },
    
    
        { "py_lock",(PyCFunction)scroller2_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)scroller2_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *scroller2_docstring_ = "";

PyTypeObject scroller2_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.scroller2",/*tp_name*/
    sizeof(scroller2_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)scroller2_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)scroller2_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    scroller2_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)scroller2_init_,  /* tp_init */
    0,                         /* tp_alloc */
    scroller2_new_,             /* tp_new */
};

PyObject *scroller2_wrapcopy_(scroller2_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *scroller2_wrap_(scroller2_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = scroller2_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    scroller2_object_ *self = 
        PyObject_NEW(scroller2_object_, &scroller2_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        scroller2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

scroller2_type_ *scroller2_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &scroller2_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected scroller2 object");
        return 0;
    }

    scroller2_type_ *object = static_cast<scroller2_type_ *>(((scroller2_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null scroller2 object in unwrap");
        return 0;
    }

    return object;
}

void scroller2_insert_(const std::pair< scroller2_type_ *, PyObject *> &p)
{




    scroller2_wrapped_.insert(p);


}

void scroller2_erase_(scroller2_type_ *p)
{




    scroller2_wrapped_.erase(p);


}

PyObject * scroller2_find_(scroller2_type_ *p)
{




    PyObject *ret = 0;
    std::map< scroller2_type_ *, PyObject *>::iterator where = scroller2_wrapped_.find(p);
    if(where != scroller2_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct scroller2_dispatch_ scroller2_realdispatcher_ =
{
    &scroller2_typeobject_,
    scroller2_wrap_,
    scroller2_wrapcopy_,
    scroller2_unwrap_,
    scroller2_insert_,
    scroller2_erase_,
    scroller2_find_
};

  

struct splitter_node_object_ {
    PyObject_HEAD
    server_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct splitter_node_wrapper_: public splitter_node_type_ 
{

    static void __deleter__(splitter_node_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
        cache_4_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
        if(cache_4_)
        {
            Py_DECREF(cache_4_);
            cache_4_ = 0;
        }
    
    }
*/



    splitter_node_wrapper_( PyObject *o, PyInterpreterState *i  ): splitter_node_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~splitter_node_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void server_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"server_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("server_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("server_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void server_opened(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"server_opened");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("server_opened before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("server_opened after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void server_tree(  const data_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"server_tree");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_data_copy,&a0);
        //printf("server_tree before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("server_tree after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    void close_server(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"close_server");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("close_server before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("close_server after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_4_;
 
    void server_change(  const data_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_4_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"server_change");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_4_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_data_copy,&a0);
        //printf("server_change before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("server_change after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }













    static int gc_traverse_method_(splitter_node_object_ *self, visitproc visit, void *arg)
    {
        splitter_node_wrapper_ *o = (splitter_node_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(splitter_node_object_ *self)
    {
        splitter_node_wrapper_ *o = (splitter_node_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void server_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return splitter_node_type_::server_closed();
                
            
            
        }

        return
            
                server_closed
            
            
                ();
    }

    static PyObject *server_closed_method_(splitter_node_object_ *self, PyObject *args)
    {
        

        splitter_node_wrapper_ *o = (splitter_node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null splitter_node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->server_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in splitter_node::server_closed");
        }

        return 0;
    }



    void server_opened_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return splitter_node_type_::server_opened();
                
            
            
        }

        return
            
                server_opened
            
            
                ();
    }

    static PyObject *server_opened_method_(splitter_node_object_ *self, PyObject *args)
    {
        

        splitter_node_wrapper_ *o = (splitter_node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null splitter_node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->server_opened_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in splitter_node::server_opened");
        }

        return 0;
    }



    void server_tree_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return splitter_node_type_::server_tree( a0 );
                
            
            
        }

        return
            
                server_tree
            
            
                ( a0 );
    }

    static PyObject *server_tree_method_(splitter_node_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        splitter_node_wrapper_ *o = (splitter_node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null splitter_node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->server_tree_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in splitter_node::server_tree");
        }

        return 0;
    }



    void close_server_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return splitter_node_type_::close_server();
                
            
            
        }

        return
            
                close_server
            
            
                ();
    }

    static PyObject *close_server_method_(splitter_node_object_ *self, PyObject *args)
    {
        

        splitter_node_wrapper_ *o = (splitter_node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null splitter_node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_server_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in splitter_node::close_server");
        }

        return 0;
    }



    void server_change_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return splitter_node_type_::server_change( a0 );
                
            
            
        }

        return
            
                server_change
            
            
                ( a0 );
    }

    static PyObject *server_change_method_(splitter_node_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        splitter_node_wrapper_ *o = (splitter_node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null splitter_node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->server_change_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in splitter_node::server_change");
        }

        return 0;
    }





    static PyObject *_method_lock(splitter_node_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(splitter_node_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void splitter_node_delete_(splitter_node_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            server_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (splitter_node_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        splitter_node_wrapper_::__deleter__((splitter_node_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void splitter_node_dealloc_(splitter_node_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    splitter_node_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * splitter_node_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    splitter_node_object_ *self;

    if(!(self = (splitter_node_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int splitter_node_init_(splitter_node_object_ *self, PyObject *args, PyObject *kwds)
{
    splitter_node_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new splitter_node_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing splitter_node");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            server_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"splitter_node invalid arguments");
    return -1;

}


static const char *splitter_node_server_closed_docstring_ = ""
"        self.server_closed() -> ."
"        "
"        abstract method invoked when the server is closed.  When this "
"        method is called, the server is still partially open, in that the"
"        servername() and path() methods are still valid."
"        "
"        When implementing this action, you should call server.server_closed"
"        to perform the default actions."
"        "
"        Note that this method is invoked only when the server is closed by "
"        outside agency."
"        "
""
;

static const char *splitter_node_server_opened_docstring_ = ""
"        self.server_opened() -> ."
"        "
"        abstract method invoked when the server is opened.  Can be used"
"        for initialisation actions which depend on the server being open."
"        "
"        When implementing this action, you should call server.server_opened"
"        to perform the default actions"
"        "
""
;

static const char *splitter_node_server_tree_docstring_ = "";

static const char *splitter_node_close_server_docstring_ = ""
"        self.close_server() -> ."
"        "
"        Call to close a server.  This method can be overidden to perform "
"        extra cleanup actions, but the default implementation should always"
"        be called with server.close_server."
"        "
"        After the default implementation is called, many server methods are"
"        no longer available, such as servername() and path()"
"        "
""
;

static const char *splitter_node_server_change_docstring_ = ""
"        self.server_change(requested_data) -> ."
"        "
"        abstract method invoked when a belcanto client requests a data change."
"        When implementing this action, you should call server.server_closed"
"        to perform the default actions."
"        "
""
;




static PyMethodDef splitter_node_methods_[] = {
    
        { "server_closed",(PyCFunction)splitter_node_wrapper_::server_closed_method_,METH_VARARGS,(char *)splitter_node_server_closed_docstring_ },
    
        { "server_opened",(PyCFunction)splitter_node_wrapper_::server_opened_method_,METH_VARARGS,(char *)splitter_node_server_opened_docstring_ },
    
        { "server_tree",(PyCFunction)splitter_node_wrapper_::server_tree_method_,METH_VARARGS,(char *)splitter_node_server_tree_docstring_ },
    
        { "close_server",(PyCFunction)splitter_node_wrapper_::close_server_method_,METH_VARARGS,(char *)splitter_node_close_server_docstring_ },
    
        { "server_change",(PyCFunction)splitter_node_wrapper_::server_change_method_,METH_VARARGS,(char *)splitter_node_server_change_docstring_ },
    
    
        { "py_lock",(PyCFunction)splitter_node_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)splitter_node_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *splitter_node_docstring_ = "";

PyTypeObject splitter_node_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.splitter_node",/*tp_name*/
    sizeof(splitter_node_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)splitter_node_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)splitter_node_docstring_,       /* tp_doc */
    (traverseproc)splitter_node_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)splitter_node_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    splitter_node_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)splitter_node_init_,  /* tp_init */
    0,                         /* tp_alloc */
    splitter_node_new_,             /* tp_new */
};

PyObject *splitter_node_wrapcopy_(splitter_node_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *splitter_node_wrap_(splitter_node_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = server_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    splitter_node_object_ *self = 
        
        PyObject_GC_New(splitter_node_object_, &splitter_node_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        server_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

splitter_node_type_ *splitter_node_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &splitter_node_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected splitter_node object");
        return 0;
    }

    splitter_node_type_ *object = static_cast<splitter_node_type_ *>(((splitter_node_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null splitter_node object in unwrap");
        return 0;
    }

    return object;
}

void splitter_node_insert_(const std::pair< splitter_node_type_ *, PyObject *> &p)
{


    server_dispatcher_->insert(p);




}

void splitter_node_erase_(splitter_node_type_ *p)
{


    server_dispatcher_->erase(p);




}

PyObject * splitter_node_find_(splitter_node_type_ *p)
{


    return server_dispatcher_->find(p);




}

static struct splitter_node_dispatch_ splitter_node_realdispatcher_ =
{
    &splitter_node_typeobject_,
    splitter_node_wrap_,
    splitter_node_wrapcopy_,
    splitter_node_unwrap_,
    splitter_node_insert_,
    splitter_node_erase_,
    splitter_node_find_
};

  

struct splitter_object_ {
    PyObject_HEAD
    splitter_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct splitter_wrapper_: public splitter_type_ 
{

    static void __deleter__(splitter_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
    }
*/



    splitter_wrapper_( PyObject *o, PyInterpreterState *i  ): splitter_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~splitter_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void set_latency(  unsigned int a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"set_latency");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_ui,&a0);
        //printf("set_latency before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("set_latency after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void set_clock(  clockref_type_ * a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"set_clock");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_clockref,&a0);
        //printf("set_clock before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("set_clock after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return splitter_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(splitter_object_ *self, PyObject *args)
    {
        

        splitter_wrapper_ *o = (splitter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null splitter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in splitter::cookie");
        }

        return 0;
    }



    void add_signal_call_(bool owned ,unsigned int a0 ,splitter_node_type_ * a1 )
    {
        if(owned)
        {
            
                
                
                    return splitter_type_::add_signal( a0  ,a1 );
                
            
            
        }

        return
            
                add_signal
            
            
                ( a0  ,a1 );
    }

    static PyObject *add_signal_method_(splitter_object_ *self, PyObject *args)
    {
         unsigned int a0;  splitter_node_type_ * a1; 

        splitter_wrapper_ *o = (splitter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null splitter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_splitter_node,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->add_signal_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in splitter::add_signal");
        }

        return 0;
    }



    void remove_signal_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return splitter_type_::remove_signal( a0 );
                
            
            
        }

        return
            
                remove_signal
            
            
                ( a0 );
    }

    static PyObject *remove_signal_method_(splitter_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        splitter_wrapper_ *o = (splitter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null splitter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->remove_signal_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in splitter::remove_signal");
        }

        return 0;
    }



    void set_latency_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return splitter_type_::set_latency( a0 );
                
            
            
        }

        return
            
                set_latency
            
            
                ( a0 );
    }

    static PyObject *set_latency_method_(splitter_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        splitter_wrapper_ *o = (splitter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null splitter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_latency_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in splitter::set_latency");
        }

        return 0;
    }



    void set_clock_call_(bool owned ,clockref_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return splitter_type_::set_clock( a0 );
                
            
            
        }

        return
            
                set_clock
            
            
                ( a0 );
    }

    static PyObject *set_clock_method_(splitter_object_ *self, PyObject *args)
    {
         clockref_type_ * a0; 

        splitter_wrapper_ *o = (splitter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null splitter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_clockref,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_clock_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in splitter::set_clock");
        }

        return 0;
    }



    clockref_type_ * get_clock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return splitter_type_::get_clock();
                
            
            
        }

        return
            
                get_clock
            
            
                ();
    }

    static PyObject *get_clock_method_(splitter_object_ *self, PyObject *args)
    {
        

        splitter_wrapper_ *o = (splitter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null splitter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    clockref_type_ * r = o->get_clock_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_clockref((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in splitter::get_clock");
        }

        return 0;
    }



    unsigned int get_latency_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return splitter_type_::get_latency();
                
            
            
        }

        return
            
                get_latency
            
            
                ();
    }

    static PyObject *get_latency_method_(splitter_object_ *self, PyObject *args)
    {
        

        splitter_wrapper_ *o = (splitter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null splitter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_latency_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in splitter::get_latency");
        }

        return 0;
    }





    static PyObject *_method_lock(splitter_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(splitter_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void splitter_delete_(splitter_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            splitter_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (splitter_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        splitter_wrapper_::__deleter__((splitter_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void splitter_dealloc_(splitter_object_* self)
{
    
    splitter_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * splitter_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    splitter_object_ *self;

    if(!(self = (splitter_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int splitter_init_(splitter_object_ *self, PyObject *args, PyObject *kwds)
{
    splitter_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new splitter_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing splitter");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            splitter_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"splitter invalid arguments");
    return -1;

}


static const char *splitter_cookie_docstring_ = "";

static const char *splitter_add_signal_docstring_ = "";

static const char *splitter_remove_signal_docstring_ = "";

static const char *splitter_set_latency_docstring_ = "";

static const char *splitter_set_clock_docstring_ = "";

static const char *splitter_get_clock_docstring_ = "";

static const char *splitter_get_latency_docstring_ = "";




static PyMethodDef splitter_methods_[] = {
    
        { "cookie",(PyCFunction)splitter_wrapper_::cookie_method_,METH_VARARGS,(char *)splitter_cookie_docstring_ },
    
        { "add_signal",(PyCFunction)splitter_wrapper_::add_signal_method_,METH_VARARGS,(char *)splitter_add_signal_docstring_ },
    
        { "remove_signal",(PyCFunction)splitter_wrapper_::remove_signal_method_,METH_VARARGS,(char *)splitter_remove_signal_docstring_ },
    
        { "set_latency",(PyCFunction)splitter_wrapper_::set_latency_method_,METH_VARARGS,(char *)splitter_set_latency_docstring_ },
    
        { "set_clock",(PyCFunction)splitter_wrapper_::set_clock_method_,METH_VARARGS,(char *)splitter_set_clock_docstring_ },
    
        { "get_clock",(PyCFunction)splitter_wrapper_::get_clock_method_,METH_VARARGS,(char *)splitter_get_clock_docstring_ },
    
        { "get_latency",(PyCFunction)splitter_wrapper_::get_latency_method_,METH_VARARGS,(char *)splitter_get_latency_docstring_ },
    
    
        { "py_lock",(PyCFunction)splitter_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)splitter_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *splitter_docstring_ = "";

PyTypeObject splitter_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.splitter",/*tp_name*/
    sizeof(splitter_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)splitter_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)splitter_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    splitter_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)splitter_init_,  /* tp_init */
    0,                         /* tp_alloc */
    splitter_new_,             /* tp_new */
};

PyObject *splitter_wrapcopy_(splitter_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *splitter_wrap_(splitter_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = splitter_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    splitter_object_ *self = 
        PyObject_NEW(splitter_object_, &splitter_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        splitter_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

splitter_type_ *splitter_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &splitter_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected splitter object");
        return 0;
    }

    splitter_type_ *object = static_cast<splitter_type_ *>(((splitter_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null splitter object in unwrap");
        return 0;
    }

    return object;
}

void splitter_insert_(const std::pair< splitter_type_ *, PyObject *> &p)
{




    splitter_wrapped_.insert(p);


}

void splitter_erase_(splitter_type_ *p)
{




    splitter_wrapped_.erase(p);


}

PyObject * splitter_find_(splitter_type_ *p)
{




    PyObject *ret = 0;
    std::map< splitter_type_ *, PyObject *>::iterator where = splitter_wrapped_.find(p);
    if(where != splitter_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct splitter_dispatch_ splitter_realdispatcher_ =
{
    &splitter_typeobject_,
    splitter_wrap_,
    splitter_wrapcopy_,
    splitter_unwrap_,
    splitter_insert_,
    splitter_erase_,
    splitter_find_
};

  

struct d2b_object_ {
    PyObject_HEAD
    d2b_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct d2b_wrapper_: public d2b_type_ 
{

    static void __deleter__(d2b_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    d2b_wrapper_( PyObject *o, PyInterpreterState *i  ): d2b_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    d2b_wrapper_( PyObject *o, PyInterpreterState *i  ,const d2b_type_ & a0  ): d2b_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~d2b_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(d2b_object_ *self, visitproc visit, void *arg)
    {
        d2b_wrapper_ *o = (d2b_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(d2b_object_ *self)
    {
        d2b_wrapper_ *o = (d2b_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning bool doc ""

    bool special_call_call_( const data_type_ & a0 )
    {
        return
             invoke 
            
                ( a0 );
    }

    static PyObject *special_call_method_(d2b_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"d2b::__call__ doesn't support keywords");
            return 0;
        }

         data_type_ * a0; 

        d2b_wrapper_ *o = (d2b_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null d2b c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        

            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->special_call_call_ (*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2b::__call__");
        }

        return 0;
    }




    void set_call_(bool owned ,const d2b_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return d2b_type_::set( a0 );
                
            
            
        }

        return
            
                set
            
            
                ( a0 );
    }

    static PyObject *set_method_(d2b_object_ *self, PyObject *args)
    {
         d2b_type_ * a0; 

        d2b_wrapper_ *o = (d2b_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null d2b c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_d2b_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2b::set");
        }

        return 0;
    }



    bool invoke_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return d2b_type_::invoke( a0 );
                
            
            
        }

        return
            
                invoke
            
            
                ( a0 );
    }

    static PyObject *invoke_method_(d2b_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        d2b_wrapper_ *o = (d2b_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null d2b c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->invoke_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2b::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(d2b_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(d2b_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void d2b_delete_(d2b_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            d2b_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (d2b_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        d2b_wrapper_::__deleter__((d2b_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void d2b_dealloc_(d2b_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    d2b_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * d2b_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    d2b_object_ *self;

    if(!(self = (d2b_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int d2b_init_(d2b_object_ *self, PyObject *args, PyObject *kwds)
{
    d2b_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new d2b_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing d2b");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            d2b_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         d2b_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_d2b_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new d2b_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing d2b");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            d2b_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"d2b invalid arguments");
    return -1;

}


static const char *d2b_set_docstring_ = "";

static const char *d2b_invoke_docstring_ = ""
"        self.invoke(data) -> "
"        "
"        invoke the functor with a data argument"
"        "
""
;




static PyMethodDef d2b_methods_[] = {
    
        { "set",(PyCFunction)d2b_wrapper_::set_method_,METH_VARARGS,(char *)d2b_set_docstring_ },
    
        { "invoke",(PyCFunction)d2b_wrapper_::invoke_method_,METH_VARARGS,(char *)d2b_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)d2b_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)d2b_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *d2b_docstring_ = ""
"    Functor with 1 data argument and bool return value."
"    "
"    Initialisers"
"    ============"
"      "
"    d2b() - creates a null functor which does nothing"
"    "
""
;

PyTypeObject d2b_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.d2b",/*tp_name*/
    sizeof(d2b_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)d2b_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)d2b_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)d2b_docstring_,       /* tp_doc */
    (traverseproc)d2b_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)d2b_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    d2b_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)d2b_init_,  /* tp_init */
    0,                         /* tp_alloc */
    d2b_new_,             /* tp_new */
};

PyObject *d2b_wrapcopy_(d2b_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            d2b_object_ *obj = 
                
                PyObject_GC_New(d2b_object_, d2b_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new d2b_type_(*o);
                }

                try
                {
                    d2b_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying d2b");
                return 0;
            }
        

    
}

PyObject *d2b_wrap_(d2b_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = d2b_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    d2b_object_ *self = 
        
        PyObject_GC_New(d2b_object_, &d2b_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        d2b_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

d2b_type_ *d2b_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &d2b_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected d2b object");
        return 0;
    }

    d2b_type_ *object = static_cast<d2b_type_ *>(((d2b_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null d2b object in unwrap");
        return 0;
    }

    return object;
}

void d2b_insert_(const std::pair< d2b_type_ *, PyObject *> &p)
{




    d2b_wrapped_.insert(p);


}

void d2b_erase_(d2b_type_ *p)
{




    d2b_wrapped_.erase(p);


}

PyObject * d2b_find_(d2b_type_ *p)
{




    PyObject *ret = 0;
    std::map< d2b_type_ *, PyObject *>::iterator where = d2b_wrapped_.find(p);
    if(where != d2b_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct d2b_dispatch_ d2b_realdispatcher_ =
{
    &d2b_typeobject_,
    d2b_wrap_,
    d2b_wrapcopy_,
    d2b_unwrap_,
    d2b_insert_,
    d2b_erase_,
    d2b_find_
};

  

struct d2b_nb_object_ {
    PyObject_HEAD
    d2b_nb_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct d2b_nb_wrapper_: public d2b_nb_type_ 
{

    static void __deleter__(d2b_nb_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    d2b_nb_wrapper_( PyObject *o, PyInterpreterState *i  ): d2b_nb_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    d2b_nb_wrapper_( PyObject *o, PyInterpreterState *i  ,const d2b_nb_type_ & a0  ): d2b_nb_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~d2b_nb_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(d2b_nb_object_ *self, visitproc visit, void *arg)
    {
        d2b_nb_wrapper_ *o = (d2b_nb_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(d2b_nb_object_ *self)
    {
        d2b_nb_wrapper_ *o = (d2b_nb_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning bool doc ""

    bool special_call_call_( const data_nb_type_ & a0 )
    {
        return
             invoke 
            
                ( a0 );
    }

    static PyObject *special_call_method_(d2b_nb_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"d2b_nb::__call__ doesn't support keywords");
            return 0;
        }

         data_nb_type_ * a0; 

        d2b_nb_wrapper_ *o = (d2b_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null d2b_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        

            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->special_call_call_ (*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2b_nb::__call__");
        }

        return 0;
    }




    void set_call_(bool owned ,const d2b_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return d2b_nb_type_::set( a0 );
                
            
            
        }

        return
            
                set
            
            
                ( a0 );
    }

    static PyObject *set_method_(d2b_nb_object_ *self, PyObject *args)
    {
         d2b_nb_type_ * a0; 

        d2b_nb_wrapper_ *o = (d2b_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null d2b_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_d2b_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2b_nb::set");
        }

        return 0;
    }



    bool invoke_call_(bool owned ,const data_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return d2b_nb_type_::invoke( a0 );
                
            
            
        }

        return
            
                invoke
            
            
                ( a0 );
    }

    static PyObject *invoke_method_(d2b_nb_object_ *self, PyObject *args)
    {
         data_nb_type_ * a0; 

        d2b_nb_wrapper_ *o = (d2b_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null d2b_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->invoke_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2b_nb::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(d2b_nb_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(d2b_nb_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void d2b_nb_delete_(d2b_nb_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            d2b_nb_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (d2b_nb_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        d2b_nb_wrapper_::__deleter__((d2b_nb_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void d2b_nb_dealloc_(d2b_nb_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    d2b_nb_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * d2b_nb_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    d2b_nb_object_ *self;

    if(!(self = (d2b_nb_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int d2b_nb_init_(d2b_nb_object_ *self, PyObject *args, PyObject *kwds)
{
    d2b_nb_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new d2b_nb_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing d2b_nb");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            d2b_nb_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         d2b_nb_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_d2b_nb_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new d2b_nb_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing d2b_nb");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            d2b_nb_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"d2b_nb invalid arguments");
    return -1;

}


static const char *d2b_nb_set_docstring_ = "";

static const char *d2b_nb_invoke_docstring_ = ""
"        self.invoke(data_nb) -> "
"        "
"        invoke the functor with a data_nb argument"
"        "
""
;




static PyMethodDef d2b_nb_methods_[] = {
    
        { "set",(PyCFunction)d2b_nb_wrapper_::set_method_,METH_VARARGS,(char *)d2b_nb_set_docstring_ },
    
        { "invoke",(PyCFunction)d2b_nb_wrapper_::invoke_method_,METH_VARARGS,(char *)d2b_nb_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)d2b_nb_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)d2b_nb_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *d2b_nb_docstring_ = ""
"    Functor with 1 data_nb argument and bool return value."
"    "
"    Initialisers"
"    ============"
"      "
"    d2b_nb() - creates a null functor which does nothing"
"    "
""
;

PyTypeObject d2b_nb_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.d2b_nb",/*tp_name*/
    sizeof(d2b_nb_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)d2b_nb_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)d2b_nb_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)d2b_nb_docstring_,       /* tp_doc */
    (traverseproc)d2b_nb_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)d2b_nb_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    d2b_nb_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)d2b_nb_init_,  /* tp_init */
    0,                         /* tp_alloc */
    d2b_nb_new_,             /* tp_new */
};

PyObject *d2b_nb_wrapcopy_(d2b_nb_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            d2b_nb_object_ *obj = 
                
                PyObject_GC_New(d2b_nb_object_, d2b_nb_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new d2b_nb_type_(*o);
                }

                try
                {
                    d2b_nb_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying d2b_nb");
                return 0;
            }
        

    
}

PyObject *d2b_nb_wrap_(d2b_nb_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = d2b_nb_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    d2b_nb_object_ *self = 
        
        PyObject_GC_New(d2b_nb_object_, &d2b_nb_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        d2b_nb_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

d2b_nb_type_ *d2b_nb_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &d2b_nb_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected d2b_nb object");
        return 0;
    }

    d2b_nb_type_ *object = static_cast<d2b_nb_type_ *>(((d2b_nb_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null d2b_nb object in unwrap");
        return 0;
    }

    return object;
}

void d2b_nb_insert_(const std::pair< d2b_nb_type_ *, PyObject *> &p)
{




    d2b_nb_wrapped_.insert(p);


}

void d2b_nb_erase_(d2b_nb_type_ *p)
{




    d2b_nb_wrapped_.erase(p);


}

PyObject * d2b_nb_find_(d2b_nb_type_ *p)
{




    PyObject *ret = 0;
    std::map< d2b_nb_type_ *, PyObject *>::iterator where = d2b_nb_wrapped_.find(p);
    if(where != d2b_nb_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct d2b_nb_dispatch_ d2b_nb_realdispatcher_ =
{
    &d2b_nb_typeobject_,
    d2b_nb_wrap_,
    d2b_nb_wrapcopy_,
    d2b_nb_unwrap_,
    d2b_nb_insert_,
    d2b_nb_erase_,
    d2b_nb_find_
};

  

struct d2d_object_ {
    PyObject_HEAD
    d2d_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct d2d_wrapper_: public d2d_type_ 
{

    static void __deleter__(d2d_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    d2d_wrapper_( PyObject *o, PyInterpreterState *i  ): d2d_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    d2d_wrapper_( PyObject *o, PyInterpreterState *i  ,const d2d_type_ & a0  ): d2d_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~d2d_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(d2d_object_ *self, visitproc visit, void *arg)
    {
        d2d_wrapper_ *o = (d2d_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(d2d_object_ *self)
    {
        d2d_wrapper_ *o = (d2d_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning data_type_ * doc ""

    data_type_ special_call_call_( const data_type_ & a0 )
    {
        return
             invoke 
            
                ( a0 );
    }

    static PyObject *special_call_method_(d2d_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"d2d::__call__ doesn't support keywords");
            return 0;
        }

         data_type_ * a0; 

        d2d_wrapper_ *o = (d2d_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null d2d c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        

            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->special_call_call_ (*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2d::__call__");
        }

        return 0;
    }




    void set_call_(bool owned ,const d2d_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return d2d_type_::set( a0 );
                
            
            
        }

        return
            
                set
            
            
                ( a0 );
    }

    static PyObject *set_method_(d2d_object_ *self, PyObject *args)
    {
         d2d_type_ * a0; 

        d2d_wrapper_ *o = (d2d_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null d2d c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_d2d_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2d::set");
        }

        return 0;
    }



    data_type_ invoke_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return d2d_type_::invoke( a0 );
                
            
            
        }

        return
            
                invoke
            
            
                ( a0 );
    }

    static PyObject *invoke_method_(d2d_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        d2d_wrapper_ *o = (d2d_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null d2d c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->invoke_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2d::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(d2d_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(d2d_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void d2d_delete_(d2d_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            d2d_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (d2d_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        d2d_wrapper_::__deleter__((d2d_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void d2d_dealloc_(d2d_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    d2d_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * d2d_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    d2d_object_ *self;

    if(!(self = (d2d_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int d2d_init_(d2d_object_ *self, PyObject *args, PyObject *kwds)
{
    d2d_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new d2d_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing d2d");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            d2d_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         d2d_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_d2d_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new d2d_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing d2d");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            d2d_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"d2d invalid arguments");
    return -1;

}


static const char *d2d_set_docstring_ = "";

static const char *d2d_invoke_docstring_ = ""
"        self.invoke(data) -> "
"        "
"        invoke the functor with a data argument"
"        "
""
;




static PyMethodDef d2d_methods_[] = {
    
        { "set",(PyCFunction)d2d_wrapper_::set_method_,METH_VARARGS,(char *)d2d_set_docstring_ },
    
        { "invoke",(PyCFunction)d2d_wrapper_::invoke_method_,METH_VARARGS,(char *)d2d_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)d2d_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)d2d_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *d2d_docstring_ = ""
"    Functor with 1 data argument and data return value."
"    "
"    Initialisers"
"    ============"
"      "
"    d2d() - creates a null functor which does nothing"
"    "
""
;

PyTypeObject d2d_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.d2d",/*tp_name*/
    sizeof(d2d_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)d2d_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)d2d_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)d2d_docstring_,       /* tp_doc */
    (traverseproc)d2d_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)d2d_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    d2d_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)d2d_init_,  /* tp_init */
    0,                         /* tp_alloc */
    d2d_new_,             /* tp_new */
};

PyObject *d2d_wrapcopy_(d2d_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            d2d_object_ *obj = 
                
                PyObject_GC_New(d2d_object_, d2d_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new d2d_type_(*o);
                }

                try
                {
                    d2d_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying d2d");
                return 0;
            }
        

    
}

PyObject *d2d_wrap_(d2d_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = d2d_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    d2d_object_ *self = 
        
        PyObject_GC_New(d2d_object_, &d2d_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        d2d_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

d2d_type_ *d2d_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &d2d_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected d2d object");
        return 0;
    }

    d2d_type_ *object = static_cast<d2d_type_ *>(((d2d_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null d2d object in unwrap");
        return 0;
    }

    return object;
}

void d2d_insert_(const std::pair< d2d_type_ *, PyObject *> &p)
{




    d2d_wrapped_.insert(p);


}

void d2d_erase_(d2d_type_ *p)
{




    d2d_wrapped_.erase(p);


}

PyObject * d2d_find_(d2d_type_ *p)
{




    PyObject *ret = 0;
    std::map< d2d_type_ *, PyObject *>::iterator where = d2d_wrapped_.find(p);
    if(where != d2d_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct d2d_dispatch_ d2d_realdispatcher_ =
{
    &d2d_typeobject_,
    d2d_wrap_,
    d2d_wrapcopy_,
    d2d_unwrap_,
    d2d_insert_,
    d2d_erase_,
    d2d_find_
};

  

struct d2d_nb_object_ {
    PyObject_HEAD
    d2d_nb_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct d2d_nb_wrapper_: public d2d_nb_type_ 
{

    static void __deleter__(d2d_nb_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    d2d_nb_wrapper_( PyObject *o, PyInterpreterState *i  ): d2d_nb_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    d2d_nb_wrapper_( PyObject *o, PyInterpreterState *i  ,const d2d_nb_type_ & a0  ): d2d_nb_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~d2d_nb_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(d2d_nb_object_ *self, visitproc visit, void *arg)
    {
        d2d_nb_wrapper_ *o = (d2d_nb_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(d2d_nb_object_ *self)
    {
        d2d_nb_wrapper_ *o = (d2d_nb_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning data_nb_type_ * doc ""

    data_nb_type_ special_call_call_( const data_nb_type_ & a0 )
    {
        return
             invoke 
            
                ( a0 );
    }

    static PyObject *special_call_method_(d2d_nb_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"d2d_nb::__call__ doesn't support keywords");
            return 0;
        }

         data_nb_type_ * a0; 

        d2d_nb_wrapper_ *o = (d2d_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null d2d_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        

            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_nb_type_ r = o->special_call_call_ (*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2d_nb::__call__");
        }

        return 0;
    }




    void set_call_(bool owned ,const d2d_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return d2d_nb_type_::set( a0 );
                
            
            
        }

        return
            
                set
            
            
                ( a0 );
    }

    static PyObject *set_method_(d2d_nb_object_ *self, PyObject *args)
    {
         d2d_nb_type_ * a0; 

        d2d_nb_wrapper_ *o = (d2d_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null d2d_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_d2d_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2d_nb::set");
        }

        return 0;
    }



    data_nb_type_ invoke_call_(bool owned ,const data_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return d2d_nb_type_::invoke( a0 );
                
            
            
        }

        return
            
                invoke
            
            
                ( a0 );
    }

    static PyObject *invoke_method_(d2d_nb_object_ *self, PyObject *args)
    {
         data_nb_type_ * a0; 

        d2d_nb_wrapper_ *o = (d2d_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null d2d_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_nb_type_ r = o->invoke_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2d_nb::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(d2d_nb_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(d2d_nb_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void d2d_nb_delete_(d2d_nb_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            d2d_nb_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (d2d_nb_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        d2d_nb_wrapper_::__deleter__((d2d_nb_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void d2d_nb_dealloc_(d2d_nb_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    d2d_nb_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * d2d_nb_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    d2d_nb_object_ *self;

    if(!(self = (d2d_nb_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int d2d_nb_init_(d2d_nb_object_ *self, PyObject *args, PyObject *kwds)
{
    d2d_nb_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new d2d_nb_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing d2d_nb");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            d2d_nb_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         d2d_nb_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_d2d_nb_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new d2d_nb_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing d2d_nb");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            d2d_nb_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"d2d_nb invalid arguments");
    return -1;

}


static const char *d2d_nb_set_docstring_ = "";

static const char *d2d_nb_invoke_docstring_ = ""
"        self.invoke(data_nb) -> "
"        "
"        invoke the functor with a data argument"
"        "
""
;




static PyMethodDef d2d_nb_methods_[] = {
    
        { "set",(PyCFunction)d2d_nb_wrapper_::set_method_,METH_VARARGS,(char *)d2d_nb_set_docstring_ },
    
        { "invoke",(PyCFunction)d2d_nb_wrapper_::invoke_method_,METH_VARARGS,(char *)d2d_nb_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)d2d_nb_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)d2d_nb_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *d2d_nb_docstring_ = ""
"    Functor with 1 data argument and data return value."
"    "
"    Initialisers"
"    ============"
"      "
"    d2d_nb() - creates a null functor which does nothing"
"    "
""
;

PyTypeObject d2d_nb_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.d2d_nb",/*tp_name*/
    sizeof(d2d_nb_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)d2d_nb_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)d2d_nb_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)d2d_nb_docstring_,       /* tp_doc */
    (traverseproc)d2d_nb_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)d2d_nb_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    d2d_nb_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)d2d_nb_init_,  /* tp_init */
    0,                         /* tp_alloc */
    d2d_nb_new_,             /* tp_new */
};

PyObject *d2d_nb_wrapcopy_(d2d_nb_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            d2d_nb_object_ *obj = 
                
                PyObject_GC_New(d2d_nb_object_, d2d_nb_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new d2d_nb_type_(*o);
                }

                try
                {
                    d2d_nb_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying d2d_nb");
                return 0;
            }
        

    
}

PyObject *d2d_nb_wrap_(d2d_nb_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = d2d_nb_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    d2d_nb_object_ *self = 
        
        PyObject_GC_New(d2d_nb_object_, &d2d_nb_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        d2d_nb_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

d2d_nb_type_ *d2d_nb_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &d2d_nb_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected d2d_nb object");
        return 0;
    }

    d2d_nb_type_ *object = static_cast<d2d_nb_type_ *>(((d2d_nb_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null d2d_nb object in unwrap");
        return 0;
    }

    return object;
}

void d2d_nb_insert_(const std::pair< d2d_nb_type_ *, PyObject *> &p)
{




    d2d_nb_wrapped_.insert(p);


}

void d2d_nb_erase_(d2d_nb_type_ *p)
{




    d2d_nb_wrapped_.erase(p);


}

PyObject * d2d_nb_find_(d2d_nb_type_ *p)
{




    PyObject *ret = 0;
    std::map< d2d_nb_type_ *, PyObject *>::iterator where = d2d_nb_wrapped_.find(p);
    if(where != d2d_nb_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct d2d_nb_dispatch_ d2d_nb_realdispatcher_ =
{
    &d2d_nb_typeobject_,
    d2d_nb_wrap_,
    d2d_nb_wrapcopy_,
    d2d_nb_unwrap_,
    d2d_nb_insert_,
    d2d_nb_erase_,
    d2d_nb_find_
};

  

struct dd2d_object_ {
    PyObject_HEAD
    dd2d_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct dd2d_wrapper_: public dd2d_type_ 
{

    static void __deleter__(dd2d_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    dd2d_wrapper_( PyObject *o, PyInterpreterState *i  ): dd2d_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    dd2d_wrapper_( PyObject *o, PyInterpreterState *i  ,const dd2d_type_ & a0  ): dd2d_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~dd2d_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(dd2d_object_ *self, visitproc visit, void *arg)
    {
        dd2d_wrapper_ *o = (dd2d_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(dd2d_object_ *self)
    {
        dd2d_wrapper_ *o = (dd2d_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning data_type_ * doc ""

    data_type_ special_call_call_( data_type_ a0  ,data_type_ a1 )
    {
        return
             invoke 
            
                ( a0  ,a1 );
    }

    static PyObject *special_call_method_(dd2d_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"dd2d::__call__ doesn't support keywords");
            return 0;
        }

         data_type_ * a0;  data_type_ * a1; 

        dd2d_wrapper_ *o = (dd2d_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null dd2d c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_data_copy,&a0  ,fpcvt_data_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        

        

            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->special_call_call_ (*a0 ,*a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in dd2d::__call__");
        }

        return 0;
    }




    data_type_ invoke_call_(bool owned ,data_type_ a0 ,data_type_ a1 )
    {
        if(owned)
        {
            
                
                
                    return dd2d_type_::invoke( a0  ,a1 );
                
            
            
        }

        return
            
                invoke
            
            
                ( a0  ,a1 );
    }

    static PyObject *invoke_method_(dd2d_object_ *self, PyObject *args)
    {
         data_type_ * a0;  data_type_ * a1; 

        dd2d_wrapper_ *o = (dd2d_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null dd2d c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_data_copy,&a0  ,fpcvt_data_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->invoke_call_ (self->owned ,*a0 ,*a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in dd2d::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(dd2d_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(dd2d_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void dd2d_delete_(dd2d_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            dd2d_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (dd2d_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        dd2d_wrapper_::__deleter__((dd2d_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void dd2d_dealloc_(dd2d_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    dd2d_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * dd2d_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    dd2d_object_ *self;

    if(!(self = (dd2d_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int dd2d_init_(dd2d_object_ *self, PyObject *args, PyObject *kwds)
{
    dd2d_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new dd2d_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing dd2d");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            dd2d_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         dd2d_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_dd2d_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new dd2d_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing dd2d");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            dd2d_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"dd2d invalid arguments");
    return -1;

}


static const char *dd2d_invoke_docstring_ = ""
"        self.invoke(data,data) -> data"
"        invoke the functor"
"        "
""
;




static PyMethodDef dd2d_methods_[] = {
    
        { "invoke",(PyCFunction)dd2d_wrapper_::invoke_method_,METH_VARARGS,(char *)dd2d_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)dd2d_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)dd2d_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *dd2d_docstring_ = ""
"    Functor that takes 2 datas and returns a datas"
"    "
"    Initialisers"
"    ============"
"      "
"    dd2d() - creates a null functor which returns 0.0"
"    "
""
;

PyTypeObject dd2d_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.dd2d",/*tp_name*/
    sizeof(dd2d_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)dd2d_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)dd2d_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)dd2d_docstring_,       /* tp_doc */
    (traverseproc)dd2d_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)dd2d_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    dd2d_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)dd2d_init_,  /* tp_init */
    0,                         /* tp_alloc */
    dd2d_new_,             /* tp_new */
};

PyObject *dd2d_wrapcopy_(dd2d_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            dd2d_object_ *obj = 
                
                PyObject_GC_New(dd2d_object_, dd2d_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new dd2d_type_(*o);
                }

                try
                {
                    dd2d_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying dd2d");
                return 0;
            }
        

    
}

PyObject *dd2d_wrap_(dd2d_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = dd2d_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    dd2d_object_ *self = 
        
        PyObject_GC_New(dd2d_object_, &dd2d_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        dd2d_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

dd2d_type_ *dd2d_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &dd2d_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected dd2d object");
        return 0;
    }

    dd2d_type_ *object = static_cast<dd2d_type_ *>(((dd2d_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null dd2d object in unwrap");
        return 0;
    }

    return object;
}

void dd2d_insert_(const std::pair< dd2d_type_ *, PyObject *> &p)
{




    dd2d_wrapped_.insert(p);


}

void dd2d_erase_(dd2d_type_ *p)
{




    dd2d_wrapped_.erase(p);


}

PyObject * dd2d_find_(dd2d_type_ *p)
{




    PyObject *ret = 0;
    std::map< dd2d_type_ *, PyObject *>::iterator where = dd2d_wrapped_.find(p);
    if(where != dd2d_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct dd2d_dispatch_ dd2d_realdispatcher_ =
{
    &dd2d_typeobject_,
    dd2d_wrap_,
    dd2d_wrapcopy_,
    dd2d_unwrap_,
    dd2d_insert_,
    dd2d_erase_,
    dd2d_find_
};

  

struct dd2d_nb_object_ {
    PyObject_HEAD
    dd2d_nb_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct dd2d_nb_wrapper_: public dd2d_nb_type_ 
{

    static void __deleter__(dd2d_nb_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    dd2d_nb_wrapper_( PyObject *o, PyInterpreterState *i  ): dd2d_nb_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    dd2d_nb_wrapper_( PyObject *o, PyInterpreterState *i  ,const dd2d_nb_type_ & a0  ): dd2d_nb_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~dd2d_nb_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(dd2d_nb_object_ *self, visitproc visit, void *arg)
    {
        dd2d_nb_wrapper_ *o = (dd2d_nb_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(dd2d_nb_object_ *self)
    {
        dd2d_nb_wrapper_ *o = (dd2d_nb_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning data_nb_type_ * doc ""

    data_nb_type_ special_call_call_( data_nb_type_ a0  ,data_nb_type_ a1 )
    {
        return
             invoke 
            
                ( a0  ,a1 );
    }

    static PyObject *special_call_method_(dd2d_nb_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"dd2d_nb::__call__ doesn't support keywords");
            return 0;
        }

         data_nb_type_ * a0;  data_nb_type_ * a1; 

        dd2d_nb_wrapper_ *o = (dd2d_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null dd2d_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_data_nb_copy,&a0  ,fpcvt_data_nb_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        

        

            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_nb_type_ r = o->special_call_call_ (*a0 ,*a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in dd2d_nb::__call__");
        }

        return 0;
    }




    data_nb_type_ invoke_call_(bool owned ,data_nb_type_ a0 ,data_nb_type_ a1 )
    {
        if(owned)
        {
            
                
                
                    return dd2d_nb_type_::invoke( a0  ,a1 );
                
            
            
        }

        return
            
                invoke
            
            
                ( a0  ,a1 );
    }

    static PyObject *invoke_method_(dd2d_nb_object_ *self, PyObject *args)
    {
         data_nb_type_ * a0;  data_nb_type_ * a1; 

        dd2d_nb_wrapper_ *o = (dd2d_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null dd2d_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_data_nb_copy,&a0  ,fpcvt_data_nb_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_nb_type_ r = o->invoke_call_ (self->owned ,*a0 ,*a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in dd2d_nb::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(dd2d_nb_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(dd2d_nb_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void dd2d_nb_delete_(dd2d_nb_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            dd2d_nb_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (dd2d_nb_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        dd2d_nb_wrapper_::__deleter__((dd2d_nb_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void dd2d_nb_dealloc_(dd2d_nb_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    dd2d_nb_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * dd2d_nb_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    dd2d_nb_object_ *self;

    if(!(self = (dd2d_nb_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int dd2d_nb_init_(dd2d_nb_object_ *self, PyObject *args, PyObject *kwds)
{
    dd2d_nb_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new dd2d_nb_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing dd2d_nb");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            dd2d_nb_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         dd2d_nb_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_dd2d_nb_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new dd2d_nb_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing dd2d_nb");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            dd2d_nb_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"dd2d_nb invalid arguments");
    return -1;

}


static const char *dd2d_nb_invoke_docstring_ = ""
"        self.invoke(data_nb,data_nb) -> data_nb"
"        invoke the functor"
"        "
""
;




static PyMethodDef dd2d_nb_methods_[] = {
    
        { "invoke",(PyCFunction)dd2d_nb_wrapper_::invoke_method_,METH_VARARGS,(char *)dd2d_nb_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)dd2d_nb_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)dd2d_nb_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *dd2d_nb_docstring_ = ""
"    Functor that takes 2 datas and returns a datas"
"    "
"    Initialisers"
"    ============"
"      "
"    dd2d_nb() - creates a null functor which returns 0.0"
"    "
""
;

PyTypeObject dd2d_nb_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.dd2d_nb",/*tp_name*/
    sizeof(dd2d_nb_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)dd2d_nb_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)dd2d_nb_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)dd2d_nb_docstring_,       /* tp_doc */
    (traverseproc)dd2d_nb_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)dd2d_nb_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    dd2d_nb_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)dd2d_nb_init_,  /* tp_init */
    0,                         /* tp_alloc */
    dd2d_nb_new_,             /* tp_new */
};

PyObject *dd2d_nb_wrapcopy_(dd2d_nb_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            dd2d_nb_object_ *obj = 
                
                PyObject_GC_New(dd2d_nb_object_, dd2d_nb_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new dd2d_nb_type_(*o);
                }

                try
                {
                    dd2d_nb_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying dd2d_nb");
                return 0;
            }
        

    
}

PyObject *dd2d_nb_wrap_(dd2d_nb_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = dd2d_nb_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    dd2d_nb_object_ *self = 
        
        PyObject_GC_New(dd2d_nb_object_, &dd2d_nb_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        dd2d_nb_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

dd2d_nb_type_ *dd2d_nb_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &dd2d_nb_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected dd2d_nb object");
        return 0;
    }

    dd2d_nb_type_ *object = static_cast<dd2d_nb_type_ *>(((dd2d_nb_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null dd2d_nb object in unwrap");
        return 0;
    }

    return object;
}

void dd2d_nb_insert_(const std::pair< dd2d_nb_type_ *, PyObject *> &p)
{




    dd2d_nb_wrapped_.insert(p);


}

void dd2d_nb_erase_(dd2d_nb_type_ *p)
{




    dd2d_nb_wrapped_.erase(p);


}

PyObject * dd2d_nb_find_(dd2d_nb_type_ *p)
{




    PyObject *ret = 0;
    std::map< dd2d_nb_type_ *, PyObject *>::iterator where = dd2d_nb_wrapped_.find(p);
    if(where != dd2d_nb_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct dd2d_nb_dispatch_ dd2d_nb_realdispatcher_ =
{
    &dd2d_nb_typeobject_,
    dd2d_nb_wrap_,
    dd2d_nb_wrapcopy_,
    dd2d_nb_unwrap_,
    dd2d_nb_insert_,
    dd2d_nb_erase_,
    dd2d_nb_find_
};

  

struct change_object_ {
    PyObject_HEAD
    change_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct change_wrapper_: public change_type_ 
{

    static void __deleter__(change_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    change_wrapper_( PyObject *o, PyInterpreterState *i  ): change_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    change_wrapper_( PyObject *o, PyInterpreterState *i  ,const change_type_ & a0  ): change_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~change_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(change_object_ *self, visitproc visit, void *arg)
    {
        change_wrapper_ *o = (change_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(change_object_ *self)
    {
        change_wrapper_ *o = (change_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning  doc ""

    void special_call_call_( const data_type_ & a0 )
    {
        return
             invoke 
            
                ( a0 );
    }

    static PyObject *special_call_method_(change_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"change::__call__ doesn't support keywords");
            return 0;
        }

         data_type_ * a0; 

        change_wrapper_ *o = (change_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null change c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock  ;
                o->special_call_call_ (*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }

        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in change::__call__");
        }

        return 0;
    }




    void invoke_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return change_type_::invoke( a0 );
                
            
            
        }

        return
            
                invoke
            
            
                ( a0 );
    }

    static PyObject *invoke_method_(change_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        change_wrapper_ *o = (change_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null change c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->invoke_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in change::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(change_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(change_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void change_delete_(change_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            change_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (change_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        change_wrapper_::__deleter__((change_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void change_dealloc_(change_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    change_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * change_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    change_object_ *self;

    if(!(self = (change_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int change_init_(change_object_ *self, PyObject *args, PyObject *kwds)
{
    change_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new change_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing change");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            change_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         change_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_change_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new change_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing change");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            change_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"change invalid arguments");
    return -1;

}


static const char *change_invoke_docstring_ = ""
"        self.invoke(data) -> "
"        "
"        invoke the functor with a data argument"
"        "
""
;




static PyMethodDef change_methods_[] = {
    
        { "invoke",(PyCFunction)change_wrapper_::invoke_method_,METH_VARARGS,(char *)change_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)change_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)change_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *change_docstring_ = ""
"    Functor with 1 data argument and no return value."
"    "
"    Change functors can be obtained from libpiw c++ objects or"
"    created from python callables using make_change_functor"
"    "
"    Initialisers"
"    ============"
"      "
"    change() - creates a null functor which does nothing"
"    "
""
;

PyTypeObject change_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.change",/*tp_name*/
    sizeof(change_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)change_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)change_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)change_docstring_,       /* tp_doc */
    (traverseproc)change_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)change_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    change_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)change_init_,  /* tp_init */
    0,                         /* tp_alloc */
    change_new_,             /* tp_new */
};

PyObject *change_wrapcopy_(change_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            change_object_ *obj = 
                
                PyObject_GC_New(change_object_, change_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new change_type_(*o);
                }

                try
                {
                    change_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying change");
                return 0;
            }
        

    
}

PyObject *change_wrap_(change_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = change_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    change_object_ *self = 
        
        PyObject_GC_New(change_object_, &change_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        change_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

change_type_ *change_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &change_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected change object");
        return 0;
    }

    change_type_ *object = static_cast<change_type_ *>(((change_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null change object in unwrap");
        return 0;
    }

    return object;
}

void change_insert_(const std::pair< change_type_ *, PyObject *> &p)
{




    change_wrapped_.insert(p);


}

void change_erase_(change_type_ *p)
{




    change_wrapped_.erase(p);


}

PyObject * change_find_(change_type_ *p)
{




    PyObject *ret = 0;
    std::map< change_type_ *, PyObject *>::iterator where = change_wrapped_.find(p);
    if(where != change_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct change_dispatch_ change_realdispatcher_ =
{
    &change_typeobject_,
    change_wrap_,
    change_wrapcopy_,
    change_unwrap_,
    change_insert_,
    change_erase_,
    change_find_
};

  

struct change_nb_object_ {
    PyObject_HEAD
    change_nb_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct change_nb_wrapper_: public change_nb_type_ 
{

    static void __deleter__(change_nb_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    change_nb_wrapper_( PyObject *o, PyInterpreterState *i  ): change_nb_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    change_nb_wrapper_( PyObject *o, PyInterpreterState *i  ,const change_nb_type_ & a0  ): change_nb_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~change_nb_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(change_nb_object_ *self, visitproc visit, void *arg)
    {
        change_nb_wrapper_ *o = (change_nb_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(change_nb_object_ *self)
    {
        change_nb_wrapper_ *o = (change_nb_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning  doc ""

    void special_call_call_( const data_nb_type_ & a0 )
    {
        return
             invoke 
            
                ( a0 );
    }

    static PyObject *special_call_method_(change_nb_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"change_nb::__call__ doesn't support keywords");
            return 0;
        }

         data_nb_type_ * a0; 

        change_nb_wrapper_ *o = (change_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null change_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock  ;
                o->special_call_call_ (*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }

        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in change_nb::__call__");
        }

        return 0;
    }




    void invoke_call_(bool owned ,const data_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return change_nb_type_::invoke( a0 );
                
            
            
        }

        return
            
                invoke
            
            
                ( a0 );
    }

    static PyObject *invoke_method_(change_nb_object_ *self, PyObject *args)
    {
         data_nb_type_ * a0; 

        change_nb_wrapper_ *o = (change_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null change_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->invoke_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in change_nb::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(change_nb_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(change_nb_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void change_nb_delete_(change_nb_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            change_nb_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (change_nb_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        change_nb_wrapper_::__deleter__((change_nb_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void change_nb_dealloc_(change_nb_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    change_nb_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * change_nb_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    change_nb_object_ *self;

    if(!(self = (change_nb_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int change_nb_init_(change_nb_object_ *self, PyObject *args, PyObject *kwds)
{
    change_nb_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new change_nb_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing change_nb");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            change_nb_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         change_nb_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_change_nb_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new change_nb_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing change_nb");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            change_nb_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"change_nb invalid arguments");
    return -1;

}


static const char *change_nb_invoke_docstring_ = ""
"        self.invoke(data_nb) -> "
"        "
"        invoke the functor with a data_nb argument"
"        "
""
;




static PyMethodDef change_nb_methods_[] = {
    
        { "invoke",(PyCFunction)change_nb_wrapper_::invoke_method_,METH_VARARGS,(char *)change_nb_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)change_nb_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)change_nb_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *change_nb_docstring_ = ""
"    Functor with 1 data_nb argument and no return value."
"    "
"    Change functors can be obtained from libpiw c++ objects or"
"    created from python callables using make_change_functor"
"    "
"    Initialisers"
"    ============"
"      "
"    change_nb() - creates a null functor which does nothing"
"    "
""
;

PyTypeObject change_nb_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.change_nb",/*tp_name*/
    sizeof(change_nb_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)change_nb_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)change_nb_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)change_nb_docstring_,       /* tp_doc */
    (traverseproc)change_nb_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)change_nb_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    change_nb_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)change_nb_init_,  /* tp_init */
    0,                         /* tp_alloc */
    change_nb_new_,             /* tp_new */
};

PyObject *change_nb_wrapcopy_(change_nb_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            change_nb_object_ *obj = 
                
                PyObject_GC_New(change_nb_object_, change_nb_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new change_nb_type_(*o);
                }

                try
                {
                    change_nb_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying change_nb");
                return 0;
            }
        

    
}

PyObject *change_nb_wrap_(change_nb_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = change_nb_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    change_nb_object_ *self = 
        
        PyObject_GC_New(change_nb_object_, &change_nb_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        change_nb_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

change_nb_type_ *change_nb_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &change_nb_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected change_nb object");
        return 0;
    }

    change_nb_type_ *object = static_cast<change_nb_type_ *>(((change_nb_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null change_nb object in unwrap");
        return 0;
    }

    return object;
}

void change_nb_insert_(const std::pair< change_nb_type_ *, PyObject *> &p)
{




    change_nb_wrapped_.insert(p);


}

void change_nb_erase_(change_nb_type_ *p)
{




    change_nb_wrapped_.erase(p);


}

PyObject * change_nb_find_(change_nb_type_ *p)
{




    PyObject *ret = 0;
    std::map< change_nb_type_ *, PyObject *>::iterator where = change_nb_wrapped_.find(p);
    if(where != change_nb_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct change_nb_dispatch_ change_nb_realdispatcher_ =
{
    &change_nb_typeobject_,
    change_nb_wrap_,
    change_nb_wrapcopy_,
    change_nb_unwrap_,
    change_nb_insert_,
    change_nb_erase_,
    change_nb_find_
};

  

struct function1_object_ {
    PyObject_HEAD
    function1_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct function1_wrapper_: public function1_type_ 
{

    static void __deleter__(function1_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    function1_wrapper_( PyObject *o, PyInterpreterState *i  ,bool a0  ,unsigned int a1  ,unsigned int a2  ,const data_type_ & a3  ,const cookie_type_ & a4  ): function1_type_(  a0 ,a1 ,a2 ,a3 ,a4 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~function1_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(function1_object_ *self, visitproc visit, void *arg)
    {
        function1_wrapper_ *o = (function1_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(function1_object_ *self)
    {
        function1_wrapper_ *o = (function1_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void set_functor_call_(bool owned ,const d2d_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return function1_type_::set_functor( a0 );
                
            
            
        }

        return
            
                set_functor
            
            
                ( a0 );
    }

    static PyObject *set_functor_method_(function1_object_ *self, PyObject *args)
    {
         d2d_nb_type_ * a0; 

        function1_wrapper_ *o = (function1_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null function1 c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_d2d_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_functor_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in function1::set_functor");
        }

        return 0;
    }



    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return function1_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(function1_object_ *self, PyObject *args)
    {
        

        function1_wrapper_ *o = (function1_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null function1 c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in function1::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(function1_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(function1_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void function1_delete_(function1_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            function1_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (function1_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        function1_wrapper_::__deleter__((function1_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void function1_dealloc_(function1_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    function1_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * function1_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    function1_object_ *self;

    if(!(self = (function1_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int function1_init_(function1_object_ *self, PyObject *args, PyObject *kwds)
{
    function1_delete_(self);

    

    {
         bool a0;  unsigned int a1;  unsigned int a2;  data_type_ * a3;  cookie_type_ * a4; 

        if(PyArg_ParseTuple(args,"O&O&O&O&O&" ,fpcvt_b,&a0,fpcvt_ui,&a1,fpcvt_ui,&a2,fpcvt_data_copy,&a3,fpcvt_cookie_copy,&a4))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new function1_wrapper_((PyObject *)self,interp ,a0,a1,a2,*a3,*a4);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing function1");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            function1_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"function1 invalid arguments");
    return -1;

}


static const char *function1_set_functor_docstring_ = ""
"        self.set_functor(d2d) -> ."
"        "
"        change the functor being used.  All outputs will be udated to"
"        the value of this functor as applied to current inputs."
"        "
""
;

static const char *function1_cookie_docstring_ = ""
"        self.cookie() -> cookie to pass to upstream"
"        "
""
;




static PyMethodDef function1_methods_[] = {
    
        { "set_functor",(PyCFunction)function1_wrapper_::set_functor_method_,METH_VARARGS,(char *)function1_set_functor_docstring_ },
    
        { "cookie",(PyCFunction)function1_wrapper_::cookie_method_,METH_VARARGS,(char *)function1_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)function1_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)function1_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *function1_docstring_ = ""
"    Filter stream class."
"    "
"    function1 applies a filter to every wire in its input bundle.  The filter"
"    is represented by a d2d functor.  The input signals may be passed thru"
"    intact, with the output signal added, or may be blocked, so that the output"
"    of the filter contains just one signal.  The filter can be driver from "
"    any input signal, or by the name of the wire within the enclosing bundle."
"    "
"    Note that the functor should be implemented in C.  If you want to use a"
"    python callable, you can use make_d2d_table to sample the function into"
"    a table for linear interpolation."
"    "
"    Initialisers"
"    ============"
"    "
"    function(thru,input,output,downstream)"
"       - thru: bool thru mode"
"       - input: signal to use as filter input (0 to use wire name)"
"       - output: signal name for output"
"       - downstream: cookie from downstream object"
"    "
""
;

PyTypeObject function1_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.function1",/*tp_name*/
    sizeof(function1_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)function1_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)function1_docstring_,       /* tp_doc */
    (traverseproc)function1_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)function1_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    function1_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)function1_init_,  /* tp_init */
    0,                         /* tp_alloc */
    function1_new_,             /* tp_new */
};

PyObject *function1_wrapcopy_(function1_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *function1_wrap_(function1_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = function1_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    function1_object_ *self = 
        
        PyObject_GC_New(function1_object_, &function1_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        function1_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

function1_type_ *function1_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &function1_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected function1 object");
        return 0;
    }

    function1_type_ *object = static_cast<function1_type_ *>(((function1_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null function1 object in unwrap");
        return 0;
    }

    return object;
}

void function1_insert_(const std::pair< function1_type_ *, PyObject *> &p)
{




    function1_wrapped_.insert(p);


}

void function1_erase_(function1_type_ *p)
{




    function1_wrapped_.erase(p);


}

PyObject * function1_find_(function1_type_ *p)
{




    PyObject *ret = 0;
    std::map< function1_type_ *, PyObject *>::iterator where = function1_wrapped_.find(p);
    if(where != function1_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct function1_dispatch_ function1_realdispatcher_ =
{
    &function1_typeobject_,
    function1_wrap_,
    function1_wrapcopy_,
    function1_unwrap_,
    function1_insert_,
    function1_erase_,
    function1_find_
};

  

struct function2_object_ {
    PyObject_HEAD
    function2_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct function2_wrapper_: public function2_type_ 
{

    static void __deleter__(function2_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    function2_wrapper_( PyObject *o, PyInterpreterState *i  ,bool a0  ,unsigned int a1  ,unsigned int a2  ,unsigned int a3  ,const data_type_ & a4  ,const data_type_ & a5  ,const cookie_type_ & a6  ): function2_type_(  a0 ,a1 ,a2 ,a3 ,a4 ,a5 ,a6 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~function2_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(function2_object_ *self, visitproc visit, void *arg)
    {
        function2_wrapper_ *o = (function2_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(function2_object_ *self)
    {
        function2_wrapper_ *o = (function2_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void set_functor_call_(bool owned ,const dd2d_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return function2_type_::set_functor( a0 );
                
            
            
        }

        return
            
                set_functor
            
            
                ( a0 );
    }

    static PyObject *set_functor_method_(function2_object_ *self, PyObject *args)
    {
         dd2d_nb_type_ * a0; 

        function2_wrapper_ *o = (function2_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null function2 c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_dd2d_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_functor_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in function2::set_functor");
        }

        return 0;
    }



    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return function2_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(function2_object_ *self, PyObject *args)
    {
        

        function2_wrapper_ *o = (function2_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null function2 c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in function2::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(function2_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(function2_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void function2_delete_(function2_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            function2_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (function2_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        function2_wrapper_::__deleter__((function2_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void function2_dealloc_(function2_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    function2_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * function2_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    function2_object_ *self;

    if(!(self = (function2_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int function2_init_(function2_object_ *self, PyObject *args, PyObject *kwds)
{
    function2_delete_(self);

    

    {
         bool a0;  unsigned int a1;  unsigned int a2;  unsigned int a3;  data_type_ * a4;  data_type_ * a5;  cookie_type_ * a6; 

        if(PyArg_ParseTuple(args,"O&O&O&O&O&O&O&" ,fpcvt_b,&a0,fpcvt_ui,&a1,fpcvt_ui,&a2,fpcvt_ui,&a3,fpcvt_data_copy,&a4,fpcvt_data_copy,&a5,fpcvt_cookie_copy,&a6))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new function2_wrapper_((PyObject *)self,interp ,a0,a1,a2,a3,*a4,*a5,*a6);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing function2");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            function2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"function2 invalid arguments");
    return -1;

}


static const char *function2_set_functor_docstring_ = "";

static const char *function2_cookie_docstring_ = "";




static PyMethodDef function2_methods_[] = {
    
        { "set_functor",(PyCFunction)function2_wrapper_::set_functor_method_,METH_VARARGS,(char *)function2_set_functor_docstring_ },
    
        { "cookie",(PyCFunction)function2_wrapper_::cookie_method_,METH_VARARGS,(char *)function2_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)function2_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)function2_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *function2_docstring_ = "";

PyTypeObject function2_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.function2",/*tp_name*/
    sizeof(function2_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)function2_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)function2_docstring_,       /* tp_doc */
    (traverseproc)function2_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)function2_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    function2_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)function2_init_,  /* tp_init */
    0,                         /* tp_alloc */
    function2_new_,             /* tp_new */
};

PyObject *function2_wrapcopy_(function2_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *function2_wrap_(function2_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = function2_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    function2_object_ *self = 
        
        PyObject_GC_New(function2_object_, &function2_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        function2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

function2_type_ *function2_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &function2_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected function2 object");
        return 0;
    }

    function2_type_ *object = static_cast<function2_type_ *>(((function2_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null function2 object in unwrap");
        return 0;
    }

    return object;
}

void function2_insert_(const std::pair< function2_type_ *, PyObject *> &p)
{




    function2_wrapped_.insert(p);


}

void function2_erase_(function2_type_ *p)
{




    function2_wrapped_.erase(p);


}

PyObject * function2_find_(function2_type_ *p)
{




    PyObject *ret = 0;
    std::map< function2_type_ *, PyObject *>::iterator where = function2_wrapped_.find(p);
    if(where != function2_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct function2_dispatch_ function2_realdispatcher_ =
{
    &function2_typeobject_,
    function2_wrap_,
    function2_wrapcopy_,
    function2_unwrap_,
    function2_insert_,
    function2_erase_,
    function2_find_
};

  

struct window_object_ {
    PyObject_HEAD
    window_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct window_wrapper_: public window_type_ 
{

    static void __deleter__(window_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
    }
*/



    window_wrapper_( PyObject *o, PyInterpreterState *i  ): window_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~window_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void window_state(  bool a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"window_state");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_b,&a0);
        //printf("window_state before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("window_state after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void window_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"window_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("window_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("window_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }













    static int gc_traverse_method_(window_object_ *self, visitproc visit, void *arg)
    {
        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(window_object_ *self)
    {
        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void set_window_state_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return window_type_::set_window_state( a0 );
                
            
            
        }

        return
            
                set_window_state
            
            
                ( a0 );
    }

    static PyObject *set_window_state_method_(window_object_ *self, PyObject *args)
    {
         bool a0; 

        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null window c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_window_state_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in window::set_window_state");
        }

        return 0;
    }



    void set_window_title_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                
                    return window_type_::set_window_title( a0 );
                
            
            
        }

        return
            
                set_window_title
            
            
                ( a0 );
    }

    static PyObject *set_window_title_method_(window_object_ *self, PyObject *args)
    {
         const char * a0; 

        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null window c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_window_title_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in window::set_window_title");
        }

        return 0;
    }



    void set_state_handler_call_(bool owned ,status_type_ a0 )
    {
        if(owned)
        {
            
                
                
                    return window_type_::set_state_handler( a0 );
                
            
            
        }

        return
            
                set_state_handler
            
            
                ( a0 );
    }

    static PyObject *set_state_handler_method_(window_object_ *self, PyObject *args)
    {
         status_type_ * a0; 

        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null window c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_status_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_state_handler_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in window::set_state_handler");
        }

        return 0;
    }



    void clear_state_handler_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return window_type_::clear_state_handler();
                
            
            
        }

        return
            
                clear_state_handler
            
            
                ();
    }

    static PyObject *clear_state_handler_method_(window_object_ *self, PyObject *args)
    {
        

        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null window c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_state_handler_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in window::clear_state_handler");
        }

        return 0;
    }



    void window_state_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return window_type_::window_state( a0 );
                
            
            
        }

        return
            
                window_state
            
            
                ( a0 );
    }

    static PyObject *window_state_method_(window_object_ *self, PyObject *args)
    {
         bool a0; 

        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null window c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->window_state_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in window::window_state");
        }

        return 0;
    }



    void window_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return window_type_::window_closed();
                
            
            
        }

        return
            
                window_closed
            
            
                ();
    }

    static PyObject *window_closed_method_(window_object_ *self, PyObject *args)
    {
        

        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null window c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->window_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in window::window_closed");
        }

        return 0;
    }



    bool open_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return window_type_::open();
                
            
            
        }

        return
            
                open
            
            
                ();
    }

    static PyObject *open_method_(window_object_ *self, PyObject *args)
    {
        

        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null window c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in window::open");
        }

        return 0;
    }



    bool opening_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return window_type_::opening();
                
            
            
        }

        return
            
                opening
            
            
                ();
    }

    static PyObject *opening_method_(window_object_ *self, PyObject *args)
    {
        

        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null window c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->opening_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in window::opening");
        }

        return 0;
    }



    bool closing_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return window_type_::closing();
                
            
            
        }

        return
            
                closing
            
            
                ();
    }

    static PyObject *closing_method_(window_object_ *self, PyObject *args)
    {
        

        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null window c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->closing_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in window::closing");
        }

        return 0;
    }



    bool running_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return window_type_::running();
                
            
            
        }

        return
            
                running
            
            
                ();
    }

    static PyObject *running_method_(window_object_ *self, PyObject *args)
    {
        

        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null window c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->running_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in window::running");
        }

        return 0;
    }



    void close_window_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return window_type_::close_window();
                
            
            
        }

        return
            
                close_window
            
            
                ();
    }

    static PyObject *close_window_method_(window_object_ *self, PyObject *args)
    {
        

        window_wrapper_ *o = (window_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null window c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_window_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in window::close_window");
        }

        return 0;
    }





    static PyObject *_method_lock(window_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(window_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void window_delete_(window_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            window_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (window_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        window_wrapper_::__deleter__((window_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void window_dealloc_(window_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    window_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * window_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    window_object_ *self;

    if(!(self = (window_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int window_init_(window_object_ *self, PyObject *args, PyObject *kwds)
{
    window_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new window_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing window");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            window_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"window invalid arguments");
    return -1;

}


static const char *window_set_window_state_docstring_ = "";

static const char *window_set_window_title_docstring_ = "";

static const char *window_set_state_handler_docstring_ = "";

static const char *window_clear_state_handler_docstring_ = "";

static const char *window_window_state_docstring_ = "";

static const char *window_window_closed_docstring_ = "";

static const char *window_open_docstring_ = "";

static const char *window_opening_docstring_ = "";

static const char *window_closing_docstring_ = "";

static const char *window_running_docstring_ = "";

static const char *window_close_window_docstring_ = "";




static PyMethodDef window_methods_[] = {
    
        { "set_window_state",(PyCFunction)window_wrapper_::set_window_state_method_,METH_VARARGS,(char *)window_set_window_state_docstring_ },
    
        { "set_window_title",(PyCFunction)window_wrapper_::set_window_title_method_,METH_VARARGS,(char *)window_set_window_title_docstring_ },
    
        { "set_state_handler",(PyCFunction)window_wrapper_::set_state_handler_method_,METH_VARARGS,(char *)window_set_state_handler_docstring_ },
    
        { "clear_state_handler",(PyCFunction)window_wrapper_::clear_state_handler_method_,METH_VARARGS,(char *)window_clear_state_handler_docstring_ },
    
        { "window_state",(PyCFunction)window_wrapper_::window_state_method_,METH_VARARGS,(char *)window_window_state_docstring_ },
    
        { "window_closed",(PyCFunction)window_wrapper_::window_closed_method_,METH_VARARGS,(char *)window_window_closed_docstring_ },
    
        { "open",(PyCFunction)window_wrapper_::open_method_,METH_VARARGS,(char *)window_open_docstring_ },
    
        { "opening",(PyCFunction)window_wrapper_::opening_method_,METH_VARARGS,(char *)window_opening_docstring_ },
    
        { "closing",(PyCFunction)window_wrapper_::closing_method_,METH_VARARGS,(char *)window_closing_docstring_ },
    
        { "running",(PyCFunction)window_wrapper_::running_method_,METH_VARARGS,(char *)window_running_docstring_ },
    
        { "close_window",(PyCFunction)window_wrapper_::close_window_method_,METH_VARARGS,(char *)window_close_window_docstring_ },
    
    
        { "py_lock",(PyCFunction)window_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)window_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *window_docstring_ = "";

PyTypeObject window_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.window",/*tp_name*/
    sizeof(window_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)window_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)window_docstring_,       /* tp_doc */
    (traverseproc)window_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)window_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    window_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)window_init_,  /* tp_init */
    0,                         /* tp_alloc */
    window_new_,             /* tp_new */
};

PyObject *window_wrapcopy_(window_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *window_wrap_(window_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = window_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    window_object_ *self = 
        
        PyObject_GC_New(window_object_, &window_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        window_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

window_type_ *window_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &window_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected window object");
        return 0;
    }

    window_type_ *object = static_cast<window_type_ *>(((window_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null window object in unwrap");
        return 0;
    }

    return object;
}

void window_insert_(const std::pair< window_type_ *, PyObject *> &p)
{




    window_wrapped_.insert(p);


}

void window_erase_(window_type_ *p)
{




    window_wrapped_.erase(p);


}

PyObject * window_find_(window_type_ *p)
{




    PyObject *ret = 0;
    std::map< window_type_ *, PyObject *>::iterator where = window_wrapped_.find(p);
    if(where != window_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct window_dispatch_ window_realdispatcher_ =
{
    &window_typeobject_,
    window_wrap_,
    window_wrapcopy_,
    window_unwrap_,
    window_insert_,
    window_erase_,
    window_find_
};

  

struct thing_object_ {
    PyObject_HEAD
    thing_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct thing_wrapper_: public thing_type_ 
{

    static void __deleter__(thing_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
    }
*/



    thing_wrapper_( PyObject *o, PyInterpreterState *i  ): thing_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~thing_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void thing_timer_slow(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"thing_timer_slow");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("thing_timer_slow before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("thing_timer_slow after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void thing_trigger_slow(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"thing_trigger_slow");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("thing_trigger_slow before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("thing_trigger_slow after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void thing_dequeue_slow(  const data_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"thing_dequeue_slow");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_data_copy,&a0);
        //printf("thing_dequeue_slow before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("thing_dequeue_slow after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    void thing_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"thing_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("thing_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("thing_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }













    static int gc_traverse_method_(thing_object_ *self, visitproc visit, void *arg)
    {
        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(thing_object_ *self)
    {
        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    notify_type_ trigger_fast_functor_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::trigger_fast_functor();
                
            
            
        }

        return
            
                trigger_fast_functor
            
            
                ();
    }

    static PyObject *trigger_fast_functor_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    notify_type_ r = o->trigger_fast_functor_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_notify_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::trigger_fast_functor");
        }

        return 0;
    }



    notify_type_ trigger_slow_functor_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::trigger_slow_functor();
                
            
            
        }

        return
            
                trigger_slow_functor
            
            
                ();
    }

    static PyObject *trigger_slow_functor_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    notify_type_ r = o->trigger_slow_functor_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_notify_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::trigger_slow_functor");
        }

        return 0;
    }



    change_nb_type_ enqueue_fast_functor_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::enqueue_fast_functor();
                
            
            
        }

        return
            
                enqueue_fast_functor
            
            
                ();
    }

    static PyObject *enqueue_fast_functor_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->enqueue_fast_functor_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::enqueue_fast_functor");
        }

        return 0;
    }



    change_type_ enqueue_slow_functor_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::enqueue_slow_functor();
                
            
            
        }

        return
            
                enqueue_slow_functor
            
            
                ();
    }

    static PyObject *enqueue_slow_functor_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_type_ r = o->enqueue_slow_functor_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::enqueue_slow_functor");
        }

        return 0;
    }



    change_nb_type_ enqueue_slow_nb_functor_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::enqueue_slow_nb_functor();
                
            
            
        }

        return
            
                enqueue_slow_nb_functor
            
            
                ();
    }

    static PyObject *enqueue_slow_nb_functor_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->enqueue_slow_nb_functor_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::enqueue_slow_nb_functor");
        }

        return 0;
    }



    void set_slow_timer_handler_call_(bool owned ,notify_type_ a0 )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::set_slow_timer_handler( a0 );
                
            
            
        }

        return
            
                set_slow_timer_handler
            
            
                ( a0 );
    }

    static PyObject *set_slow_timer_handler_method_(thing_object_ *self, PyObject *args)
    {
         notify_type_ * a0; 

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_notify_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_slow_timer_handler_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::set_slow_timer_handler");
        }

        return 0;
    }



    void set_fast_timer_handler_call_(bool owned ,notify_type_ a0 )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::set_fast_timer_handler( a0 );
                
            
            
        }

        return
            
                set_fast_timer_handler
            
            
                ( a0 );
    }

    static PyObject *set_fast_timer_handler_method_(thing_object_ *self, PyObject *args)
    {
         notify_type_ * a0; 

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_notify_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_fast_timer_handler_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::set_fast_timer_handler");
        }

        return 0;
    }



    void set_slow_trigger_handler_call_(bool owned ,notify_type_ a0 )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::set_slow_trigger_handler( a0 );
                
            
            
        }

        return
            
                set_slow_trigger_handler
            
            
                ( a0 );
    }

    static PyObject *set_slow_trigger_handler_method_(thing_object_ *self, PyObject *args)
    {
         notify_type_ * a0; 

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_notify_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_slow_trigger_handler_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::set_slow_trigger_handler");
        }

        return 0;
    }



    void set_fast_trigger_handler_call_(bool owned ,notify_type_ a0 )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::set_fast_trigger_handler( a0 );
                
            
            
        }

        return
            
                set_fast_trigger_handler
            
            
                ( a0 );
    }

    static PyObject *set_fast_trigger_handler_method_(thing_object_ *self, PyObject *args)
    {
         notify_type_ * a0; 

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_notify_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_fast_trigger_handler_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::set_fast_trigger_handler");
        }

        return 0;
    }



    void set_slow_dequeue_handler_call_(bool owned ,change_type_ a0 )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::set_slow_dequeue_handler( a0 );
                
            
            
        }

        return
            
                set_slow_dequeue_handler
            
            
                ( a0 );
    }

    static PyObject *set_slow_dequeue_handler_method_(thing_object_ *self, PyObject *args)
    {
         change_type_ * a0; 

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_slow_dequeue_handler_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::set_slow_dequeue_handler");
        }

        return 0;
    }



    void set_fast_dequeue_handler_call_(bool owned ,change_nb_type_ a0 )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::set_fast_dequeue_handler( a0 );
                
            
            
        }

        return
            
                set_fast_dequeue_handler
            
            
                ( a0 );
    }

    static PyObject *set_fast_dequeue_handler_method_(thing_object_ *self, PyObject *args)
    {
         change_nb_type_ * a0; 

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_fast_dequeue_handler_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::set_fast_dequeue_handler");
        }

        return 0;
    }



    void clear_slow_timer_handler_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::clear_slow_timer_handler();
                
            
            
        }

        return
            
                clear_slow_timer_handler
            
            
                ();
    }

    static PyObject *clear_slow_timer_handler_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_slow_timer_handler_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::clear_slow_timer_handler");
        }

        return 0;
    }



    void clear_slow_trigger_handler_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::clear_slow_trigger_handler();
                
            
            
        }

        return
            
                clear_slow_trigger_handler
            
            
                ();
    }

    static PyObject *clear_slow_trigger_handler_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_slow_trigger_handler_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::clear_slow_trigger_handler");
        }

        return 0;
    }



    void timer_slow_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::timer_slow( a0 );
                
            
            
        }

        return
            
                timer_slow
            
            
                ( a0 );
    }

    static PyObject *timer_slow_method_(thing_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->timer_slow_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::timer_slow");
        }

        return 0;
    }



    void cancel_timer_slow_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::cancel_timer_slow();
                
            
            
        }

        return
            
                cancel_timer_slow
            
            
                ();
    }

    static PyObject *cancel_timer_slow_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->cancel_timer_slow_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::cancel_timer_slow");
        }

        return 0;
    }



    void trigger_slow_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::trigger_slow();
                
            
            
        }

        return
            
                trigger_slow
            
            
                ();
    }

    static PyObject *trigger_slow_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->trigger_slow_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::trigger_slow");
        }

        return 0;
    }



    void enqueue_slow_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::enqueue_slow( a0 );
                
            
            
        }

        return
            
                enqueue_slow
            
            
                ( a0 );
    }

    static PyObject *enqueue_slow_method_(thing_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enqueue_slow_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::enqueue_slow");
        }

        return 0;
    }



    void enqueue_fast_call_(bool owned ,const data_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::enqueue_fast( a0 );
                
            
            
        }

        return
            
                enqueue_fast
            
            
                ( a0 );
    }

    static PyObject *enqueue_fast_method_(thing_object_ *self, PyObject *args)
    {
         data_nb_type_ * a0; 

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enqueue_fast_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::enqueue_fast");
        }

        return 0;
    }



    void flush_slow_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::flush_slow();
                
            
            
        }

        return
            
                flush_slow
            
            
                ();
    }

    static PyObject *flush_slow_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->flush_slow_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::flush_slow");
        }

        return 0;
    }



    void thing_timer_slow_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::thing_timer_slow();
                
            
            
        }

        return
            
                thing_timer_slow
            
            
                ();
    }

    static PyObject *thing_timer_slow_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->thing_timer_slow_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::thing_timer_slow");
        }

        return 0;
    }



    void thing_trigger_slow_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::thing_trigger_slow();
                
            
            
        }

        return
            
                thing_trigger_slow
            
            
                ();
    }

    static PyObject *thing_trigger_slow_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->thing_trigger_slow_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::thing_trigger_slow");
        }

        return 0;
    }



    void thing_dequeue_slow_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::thing_dequeue_slow( a0 );
                
            
            
        }

        return
            
                thing_dequeue_slow
            
            
                ( a0 );
    }

    static PyObject *thing_dequeue_slow_method_(thing_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->thing_dequeue_slow_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::thing_dequeue_slow");
        }

        return 0;
    }



    void thing_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::thing_closed();
                
            
            
        }

        return
            
                thing_closed
            
            
                ();
    }

    static PyObject *thing_closed_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->thing_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::thing_closed");
        }

        return 0;
    }



    bool open_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::open();
                
            
            
        }

        return
            
                open
            
            
                ();
    }

    static PyObject *open_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::open");
        }

        return 0;
    }



    bool opening_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::opening();
                
            
            
        }

        return
            
                opening
            
            
                ();
    }

    static PyObject *opening_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->opening_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::opening");
        }

        return 0;
    }



    bool closing_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::closing();
                
            
            
        }

        return
            
                closing
            
            
                ();
    }

    static PyObject *closing_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->closing_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::closing");
        }

        return 0;
    }



    bool running_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::running();
                
            
            
        }

        return
            
                running
            
            
                ();
    }

    static PyObject *running_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->running_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::running");
        }

        return 0;
    }



    void close_thing_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thing_type_::close_thing();
                
            
            
        }

        return
            
                close_thing
            
            
                ();
    }

    static PyObject *close_thing_method_(thing_object_ *self, PyObject *args)
    {
        

        thing_wrapper_ *o = (thing_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thing c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_thing_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thing::close_thing");
        }

        return 0;
    }





    static PyObject *_method_lock(thing_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(thing_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void thing_delete_(thing_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            thing_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (thing_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        thing_wrapper_::__deleter__((thing_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void thing_dealloc_(thing_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    thing_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * thing_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    thing_object_ *self;

    if(!(self = (thing_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int thing_init_(thing_object_ *self, PyObject *args, PyObject *kwds)
{
    thing_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new thing_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing thing");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            thing_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"thing invalid arguments");
    return -1;

}


static const char *thing_trigger_fast_functor_docstring_ = "";

static const char *thing_trigger_slow_functor_docstring_ = "";

static const char *thing_enqueue_fast_functor_docstring_ = "";

static const char *thing_enqueue_slow_functor_docstring_ = "";

static const char *thing_enqueue_slow_nb_functor_docstring_ = "";

static const char *thing_set_slow_timer_handler_docstring_ = "";

static const char *thing_set_fast_timer_handler_docstring_ = "";

static const char *thing_set_slow_trigger_handler_docstring_ = "";

static const char *thing_set_fast_trigger_handler_docstring_ = "";

static const char *thing_set_slow_dequeue_handler_docstring_ = "";

static const char *thing_set_fast_dequeue_handler_docstring_ = "";

static const char *thing_clear_slow_timer_handler_docstring_ = "";

static const char *thing_clear_slow_trigger_handler_docstring_ = "";

static const char *thing_timer_slow_docstring_ = "";

static const char *thing_cancel_timer_slow_docstring_ = "";

static const char *thing_trigger_slow_docstring_ = "";

static const char *thing_enqueue_slow_docstring_ = "";

static const char *thing_enqueue_fast_docstring_ = "";

static const char *thing_flush_slow_docstring_ = "";

static const char *thing_thing_timer_slow_docstring_ = "";

static const char *thing_thing_trigger_slow_docstring_ = "";

static const char *thing_thing_dequeue_slow_docstring_ = "";

static const char *thing_thing_closed_docstring_ = "";

static const char *thing_open_docstring_ = "";

static const char *thing_opening_docstring_ = "";

static const char *thing_closing_docstring_ = "";

static const char *thing_running_docstring_ = "";

static const char *thing_close_thing_docstring_ = "";




static PyMethodDef thing_methods_[] = {
    
        { "trigger_fast_functor",(PyCFunction)thing_wrapper_::trigger_fast_functor_method_,METH_VARARGS,(char *)thing_trigger_fast_functor_docstring_ },
    
        { "trigger_slow_functor",(PyCFunction)thing_wrapper_::trigger_slow_functor_method_,METH_VARARGS,(char *)thing_trigger_slow_functor_docstring_ },
    
        { "enqueue_fast_functor",(PyCFunction)thing_wrapper_::enqueue_fast_functor_method_,METH_VARARGS,(char *)thing_enqueue_fast_functor_docstring_ },
    
        { "enqueue_slow_functor",(PyCFunction)thing_wrapper_::enqueue_slow_functor_method_,METH_VARARGS,(char *)thing_enqueue_slow_functor_docstring_ },
    
        { "enqueue_slow_nb_functor",(PyCFunction)thing_wrapper_::enqueue_slow_nb_functor_method_,METH_VARARGS,(char *)thing_enqueue_slow_nb_functor_docstring_ },
    
        { "set_slow_timer_handler",(PyCFunction)thing_wrapper_::set_slow_timer_handler_method_,METH_VARARGS,(char *)thing_set_slow_timer_handler_docstring_ },
    
        { "set_fast_timer_handler",(PyCFunction)thing_wrapper_::set_fast_timer_handler_method_,METH_VARARGS,(char *)thing_set_fast_timer_handler_docstring_ },
    
        { "set_slow_trigger_handler",(PyCFunction)thing_wrapper_::set_slow_trigger_handler_method_,METH_VARARGS,(char *)thing_set_slow_trigger_handler_docstring_ },
    
        { "set_fast_trigger_handler",(PyCFunction)thing_wrapper_::set_fast_trigger_handler_method_,METH_VARARGS,(char *)thing_set_fast_trigger_handler_docstring_ },
    
        { "set_slow_dequeue_handler",(PyCFunction)thing_wrapper_::set_slow_dequeue_handler_method_,METH_VARARGS,(char *)thing_set_slow_dequeue_handler_docstring_ },
    
        { "set_fast_dequeue_handler",(PyCFunction)thing_wrapper_::set_fast_dequeue_handler_method_,METH_VARARGS,(char *)thing_set_fast_dequeue_handler_docstring_ },
    
        { "clear_slow_timer_handler",(PyCFunction)thing_wrapper_::clear_slow_timer_handler_method_,METH_VARARGS,(char *)thing_clear_slow_timer_handler_docstring_ },
    
        { "clear_slow_trigger_handler",(PyCFunction)thing_wrapper_::clear_slow_trigger_handler_method_,METH_VARARGS,(char *)thing_clear_slow_trigger_handler_docstring_ },
    
        { "timer_slow",(PyCFunction)thing_wrapper_::timer_slow_method_,METH_VARARGS,(char *)thing_timer_slow_docstring_ },
    
        { "cancel_timer_slow",(PyCFunction)thing_wrapper_::cancel_timer_slow_method_,METH_VARARGS,(char *)thing_cancel_timer_slow_docstring_ },
    
        { "trigger_slow",(PyCFunction)thing_wrapper_::trigger_slow_method_,METH_VARARGS,(char *)thing_trigger_slow_docstring_ },
    
        { "enqueue_slow",(PyCFunction)thing_wrapper_::enqueue_slow_method_,METH_VARARGS,(char *)thing_enqueue_slow_docstring_ },
    
        { "enqueue_fast",(PyCFunction)thing_wrapper_::enqueue_fast_method_,METH_VARARGS,(char *)thing_enqueue_fast_docstring_ },
    
        { "flush_slow",(PyCFunction)thing_wrapper_::flush_slow_method_,METH_VARARGS,(char *)thing_flush_slow_docstring_ },
    
        { "thing_timer_slow",(PyCFunction)thing_wrapper_::thing_timer_slow_method_,METH_VARARGS,(char *)thing_thing_timer_slow_docstring_ },
    
        { "thing_trigger_slow",(PyCFunction)thing_wrapper_::thing_trigger_slow_method_,METH_VARARGS,(char *)thing_thing_trigger_slow_docstring_ },
    
        { "thing_dequeue_slow",(PyCFunction)thing_wrapper_::thing_dequeue_slow_method_,METH_VARARGS,(char *)thing_thing_dequeue_slow_docstring_ },
    
        { "thing_closed",(PyCFunction)thing_wrapper_::thing_closed_method_,METH_VARARGS,(char *)thing_thing_closed_docstring_ },
    
        { "open",(PyCFunction)thing_wrapper_::open_method_,METH_VARARGS,(char *)thing_open_docstring_ },
    
        { "opening",(PyCFunction)thing_wrapper_::opening_method_,METH_VARARGS,(char *)thing_opening_docstring_ },
    
        { "closing",(PyCFunction)thing_wrapper_::closing_method_,METH_VARARGS,(char *)thing_closing_docstring_ },
    
        { "running",(PyCFunction)thing_wrapper_::running_method_,METH_VARARGS,(char *)thing_running_docstring_ },
    
        { "close_thing",(PyCFunction)thing_wrapper_::close_thing_method_,METH_VARARGS,(char *)thing_close_thing_docstring_ },
    
    
        { "py_lock",(PyCFunction)thing_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)thing_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *thing_docstring_ = "";

PyTypeObject thing_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.thing",/*tp_name*/
    sizeof(thing_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)thing_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)thing_docstring_,       /* tp_doc */
    (traverseproc)thing_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)thing_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    thing_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)thing_init_,  /* tp_init */
    0,                         /* tp_alloc */
    thing_new_,             /* tp_new */
};

PyObject *thing_wrapcopy_(thing_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *thing_wrap_(thing_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = thing_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    thing_object_ *self = 
        
        PyObject_GC_New(thing_object_, &thing_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        thing_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

thing_type_ *thing_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &thing_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected thing object");
        return 0;
    }

    thing_type_ *object = static_cast<thing_type_ *>(((thing_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null thing object in unwrap");
        return 0;
    }

    return object;
}

void thing_insert_(const std::pair< thing_type_ *, PyObject *> &p)
{




    thing_wrapped_.insert(p);


}

void thing_erase_(thing_type_ *p)
{




    thing_wrapped_.erase(p);


}

PyObject * thing_find_(thing_type_ *p)
{




    PyObject *ret = 0;
    std::map< thing_type_ *, PyObject *>::iterator where = thing_wrapped_.find(p);
    if(where != thing_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct thing_dispatch_ thing_realdispatcher_ =
{
    &thing_typeobject_,
    thing_wrap_,
    thing_wrapcopy_,
    thing_unwrap_,
    thing_insert_,
    thing_erase_,
    thing_find_
};

  

struct index_object_ {
    PyObject_HEAD
    index_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct index_wrapper_: public index_type_ 
{

    static void __deleter__(index_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
    }
*/



    index_wrapper_( PyObject *o, PyInterpreterState *i  ): index_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~index_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void close_index(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"close_index");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("close_index before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("close_index after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void index_opened(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"index_opened");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("index_opened before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("index_opened after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void index_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"index_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("index_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("index_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    void index_changed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"index_changed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("index_changed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("index_changed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }













    static int gc_traverse_method_(index_object_ *self, visitproc visit, void *arg)
    {
        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(index_object_ *self)
    {
        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    bool open_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return index_type_::open();
                
            
            
        }

        return
            
                open
            
            
                ();
    }

    static PyObject *open_method_(index_object_ *self, PyObject *args)
    {
        

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::open");
        }

        return 0;
    }



    bool opening_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return index_type_::opening();
                
            
            
        }

        return
            
                opening
            
            
                ();
    }

    static PyObject *opening_method_(index_object_ *self, PyObject *args)
    {
        

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->opening_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::opening");
        }

        return 0;
    }



    bool closing_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return index_type_::closing();
                
            
            
        }

        return
            
                closing
            
            
                ();
    }

    static PyObject *closing_method_(index_object_ *self, PyObject *args)
    {
        

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->closing_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::closing");
        }

        return 0;
    }



    bool running_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return index_type_::running();
                
            
            
        }

        return
            
                running
            
            
                ();
    }

    static PyObject *running_method_(index_object_ *self, PyObject *args)
    {
        

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->running_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::running");
        }

        return 0;
    }



    void set_change_handler_call_(bool owned ,const notify_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return index_type_::set_change_handler( a0 );
                
            
            
        }

        return
            
                set_change_handler
            
            
                ( a0 );
    }

    static PyObject *set_change_handler_method_(index_object_ *self, PyObject *args)
    {
         notify_type_ * a0; 

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_notify_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_change_handler_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::set_change_handler");
        }

        return 0;
    }



    void clear_change_handler_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return index_type_::clear_change_handler();
                
            
            
        }

        return
            
                clear_change_handler
            
            
                ();
    }

    static PyObject *clear_change_handler_method_(index_object_ *self, PyObject *args)
    {
        

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_change_handler_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::clear_change_handler");
        }

        return 0;
    }



    data_type_ index_name_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return index_type_::index_name();
                
            
            
        }

        return
            
                index_name
            
            
                ();
    }

    static PyObject *index_name_method_(index_object_ *self, PyObject *args)
    {
        

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->index_name_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::index_name");
        }

        return 0;
    }



    data_type_ member_name_call_(bool owned ,int a0 )
    {
        if(owned)
        {
            
                
                
                    return index_type_::member_name( a0 );
                
            
            
        }

        return
            
                member_name
            
            
                ( a0 );
    }

    static PyObject *member_name_method_(index_object_ *self, PyObject *args)
    {
         int a0; 

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_i,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->member_name_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::member_name");
        }

        return 0;
    }



    int member_count_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return index_type_::member_count();
                
            
            
        }

        return
            
                member_count
            
            
                ();
    }

    static PyObject *member_count_method_(index_object_ *self, PyObject *args)
    {
        

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    int r = o->member_count_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_i((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::member_count");
        }

        return 0;
    }



    unsigned short member_cookie_call_(bool owned ,int a0 )
    {
        if(owned)
        {
            
                
                
                    return index_type_::member_cookie( a0 );
                
            
            
        }

        return
            
                member_cookie
            
            
                ( a0 );
    }

    static PyObject *member_cookie_method_(index_object_ *self, PyObject *args)
    {
         int a0; 

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_i,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned short r = o->member_cookie_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::member_cookie");
        }

        return 0;
    }



    void member_died_call_(bool owned ,data_type_ a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return index_type_::member_died( a0  ,a1 );
                
            
            
        }

        return
            
                member_died
            
            
                ( a0  ,a1 );
    }

    static PyObject *member_died_method_(index_object_ *self, PyObject *args)
    {
         data_type_ * a0;  unsigned short a1; 

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_data_copy,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->member_died_call_ (self->owned ,*a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::member_died");
        }

        return 0;
    }



    void close_index_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return index_type_::close_index();
                
            
            
        }

        return
            
                close_index
            
            
                ();
    }

    static PyObject *close_index_method_(index_object_ *self, PyObject *args)
    {
        

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_index_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::close_index");
        }

        return 0;
    }



    void index_opened_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return index_type_::index_opened();
                
            
            
        }

        return
            
                index_opened
            
            
                ();
    }

    static PyObject *index_opened_method_(index_object_ *self, PyObject *args)
    {
        

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->index_opened_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::index_opened");
        }

        return 0;
    }



    void index_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return index_type_::index_closed();
                
            
            
        }

        return
            
                index_closed
            
            
                ();
    }

    static PyObject *index_closed_method_(index_object_ *self, PyObject *args)
    {
        

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->index_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::index_closed");
        }

        return 0;
    }



    void index_changed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return index_type_::index_changed();
                
            
            
        }

        return
            
                index_changed
            
            
                ();
    }

    static PyObject *index_changed_method_(index_object_ *self, PyObject *args)
    {
        

        index_wrapper_ *o = (index_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null index c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->index_changed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in index::index_changed");
        }

        return 0;
    }





    static PyObject *_method_lock(index_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(index_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void index_delete_(index_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            index_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (index_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        index_wrapper_::__deleter__((index_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void index_dealloc_(index_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    index_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * index_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    index_object_ *self;

    if(!(self = (index_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int index_init_(index_object_ *self, PyObject *args, PyObject *kwds)
{
    index_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new index_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing index");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            index_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"index invalid arguments");
    return -1;

}


static const char *index_open_docstring_ = "";

static const char *index_opening_docstring_ = "";

static const char *index_closing_docstring_ = "";

static const char *index_running_docstring_ = "";

static const char *index_set_change_handler_docstring_ = "";

static const char *index_clear_change_handler_docstring_ = "";

static const char *index_index_name_docstring_ = "";

static const char *index_member_name_docstring_ = "";

static const char *index_member_count_docstring_ = "";

static const char *index_member_cookie_docstring_ = "";

static const char *index_member_died_docstring_ = "";

static const char *index_close_index_docstring_ = "";

static const char *index_index_opened_docstring_ = "";

static const char *index_index_closed_docstring_ = "";

static const char *index_index_changed_docstring_ = "";




static PyMethodDef index_methods_[] = {
    
        { "open",(PyCFunction)index_wrapper_::open_method_,METH_VARARGS,(char *)index_open_docstring_ },
    
        { "opening",(PyCFunction)index_wrapper_::opening_method_,METH_VARARGS,(char *)index_opening_docstring_ },
    
        { "closing",(PyCFunction)index_wrapper_::closing_method_,METH_VARARGS,(char *)index_closing_docstring_ },
    
        { "running",(PyCFunction)index_wrapper_::running_method_,METH_VARARGS,(char *)index_running_docstring_ },
    
        { "set_change_handler",(PyCFunction)index_wrapper_::set_change_handler_method_,METH_VARARGS,(char *)index_set_change_handler_docstring_ },
    
        { "clear_change_handler",(PyCFunction)index_wrapper_::clear_change_handler_method_,METH_VARARGS,(char *)index_clear_change_handler_docstring_ },
    
        { "index_name",(PyCFunction)index_wrapper_::index_name_method_,METH_VARARGS,(char *)index_index_name_docstring_ },
    
        { "member_name",(PyCFunction)index_wrapper_::member_name_method_,METH_VARARGS,(char *)index_member_name_docstring_ },
    
        { "member_count",(PyCFunction)index_wrapper_::member_count_method_,METH_VARARGS,(char *)index_member_count_docstring_ },
    
        { "member_cookie",(PyCFunction)index_wrapper_::member_cookie_method_,METH_VARARGS,(char *)index_member_cookie_docstring_ },
    
        { "member_died",(PyCFunction)index_wrapper_::member_died_method_,METH_VARARGS,(char *)index_member_died_docstring_ },
    
        { "close_index",(PyCFunction)index_wrapper_::close_index_method_,METH_VARARGS,(char *)index_close_index_docstring_ },
    
        { "index_opened",(PyCFunction)index_wrapper_::index_opened_method_,METH_VARARGS,(char *)index_index_opened_docstring_ },
    
        { "index_closed",(PyCFunction)index_wrapper_::index_closed_method_,METH_VARARGS,(char *)index_index_closed_docstring_ },
    
        { "index_changed",(PyCFunction)index_wrapper_::index_changed_method_,METH_VARARGS,(char *)index_index_changed_docstring_ },
    
    
        { "py_lock",(PyCFunction)index_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)index_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *index_docstring_ = ""
"    Indexes"
""
"    Servers can advertise themselves on an index.  This class allows an index"
"    to be monitored.  It's activated using tsd_index and the servers can be"
"    enumerated using member_count() and member_name()"
""
"    Whenever the index changes, the index_changed() method gets called"
"    "
""
;

PyTypeObject index_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.index",/*tp_name*/
    sizeof(index_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)index_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)index_docstring_,       /* tp_doc */
    (traverseproc)index_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)index_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    index_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)index_init_,  /* tp_init */
    0,                         /* tp_alloc */
    index_new_,             /* tp_new */
};

PyObject *index_wrapcopy_(index_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *index_wrap_(index_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = index_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    index_object_ *self = 
        
        PyObject_GC_New(index_object_, &index_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        index_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

index_type_ *index_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &index_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected index object");
        return 0;
    }

    index_type_ *object = static_cast<index_type_ *>(((index_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null index object in unwrap");
        return 0;
    }

    return object;
}

void index_insert_(const std::pair< index_type_ *, PyObject *> &p)
{




    index_wrapped_.insert(p);


}

void index_erase_(index_type_ *p)
{




    index_wrapped_.erase(p);


}

PyObject * index_find_(index_type_ *p)
{




    PyObject *ret = 0;
    std::map< index_type_ *, PyObject *>::iterator where = index_wrapped_.find(p);
    if(where != index_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct index_dispatch_ index_realdispatcher_ =
{
    &index_typeobject_,
    index_wrap_,
    index_wrapcopy_,
    index_unwrap_,
    index_insert_,
    index_erase_,
    index_find_
};

  

struct server_object_ {
    PyObject_HEAD
    server_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct server_wrapper_: public server_type_ 
{

    static void __deleter__(server_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
        cache_4_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
        if(cache_4_)
        {
            Py_DECREF(cache_4_);
            cache_4_ = 0;
        }
    
    }
*/



    server_wrapper_( PyObject *o, PyInterpreterState *i  ): server_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    server_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned int a0  ): server_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~server_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void server_opened(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"server_opened");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("server_opened before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("server_opened after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void server_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"server_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("server_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("server_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void server_change(  const data_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"server_change");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_data_copy,&a0);
        //printf("server_change before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("server_change after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    void close_server(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"close_server");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("close_server before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("close_server after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_4_;
 
    void server_tree(  const data_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_4_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"server_tree");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_4_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_data_copy,&a0);
        //printf("server_tree before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("server_tree after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }













    static int gc_traverse_method_(server_object_ *self, visitproc visit, void *arg)
    {
        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(server_object_ *self)
    {
        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void server_opened_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::server_opened();
                
            
            
        }

        return
            
                server_opened
            
            
                ();
    }

    static PyObject *server_opened_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->server_opened_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::server_opened");
        }

        return 0;
    }



    void server_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::server_closed();
                
            
            
        }

        return
            
                server_closed
            
            
                ();
    }

    static PyObject *server_closed_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->server_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::server_closed");
        }

        return 0;
    }



    void server_change_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return server_type_::server_change( a0 );
                
            
            
        }

        return
            
                server_change
            
            
                ( a0 );
    }

    static PyObject *server_change_method_(server_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->server_change_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::server_change");
        }

        return 0;
    }



    void close_server_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::close_server();
                
            
            
        }

        return
            
                close_server
            
            
                ();
    }

    static PyObject *close_server_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_server_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::close_server");
        }

        return 0;
    }



    void server_tree_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return server_type_::server_tree( a0 );
                
            
            
        }

        return
            
                server_tree
            
            
                ( a0 );
    }

    static PyObject *server_tree_method_(server_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->server_tree_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::server_tree");
        }

        return 0;
    }



    void set_data_call_(bool owned ,data_type_ a0 )
    {
        if(owned)
        {
            
                
                
                    return server_type_::set_data( a0 );
                
            
            
        }

        return
            
                set_data
            
            
                ( a0 );
    }

    static PyObject *set_data_method_(server_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_data_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::set_data");
        }

        return 0;
    }



    data_type_ get_data_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::get_data();
                
            
            
        }

        return
            
                get_data
            
            
                ();
    }

    static PyObject *get_data_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->get_data_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::get_data");
        }

        return 0;
    }



    bool has_value_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::has_value();
                
            
            
        }

        return
            
                has_value
            
            
                ();
    }

    static PyObject *has_value_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->has_value_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::has_value");
        }

        return 0;
    }



    void advertise_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                
                    return server_type_::advertise( a0 );
                
            
            
        }

        return
            
                advertise
            
            
                ( a0 );
    }

    static PyObject *advertise_method_(server_object_ *self, PyObject *args)
    {
         const char * a0; 

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->advertise_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::advertise");
        }

        return 0;
    }



    void unadvertise_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                
                    return server_type_::unadvertise( a0 );
                
            
            
        }

        return
            
                unadvertise
            
            
                ( a0 );
    }

    static PyObject *unadvertise_method_(server_object_ *self, PyObject *args)
    {
         const char * a0; 

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->unadvertise_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::unadvertise");
        }

        return 0;
    }



    data_type_ servername_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::servername();
                
            
            
        }

        return
            
                servername
            
            
                ();
    }

    static PyObject *servername_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->servername_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::servername");
        }

        return 0;
    }



    data_type_ path_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::path();
                
            
            
        }

        return
            
                path
            
            
                ();
    }

    static PyObject *path_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->path_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::path");
        }

        return 0;
    }



    void set_change_handler_call_(bool owned ,const change_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return server_type_::set_change_handler( a0 );
                
            
            
        }

        return
            
                set_change_handler
            
            
                ( a0 );
    }

    static PyObject *set_change_handler_method_(server_object_ *self, PyObject *args)
    {
         change_type_ * a0; 

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_change_handler_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::set_change_handler");
        }

        return 0;
    }



    void clear_change_handler_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::clear_change_handler();
                
            
            
        }

        return
            
                clear_change_handler
            
            
                ();
    }

    static PyObject *clear_change_handler_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_change_handler_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::clear_change_handler");
        }

        return 0;
    }



    bool open_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::open();
                
            
            
        }

        return
            
                open
            
            
                ();
    }

    static PyObject *open_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::open");
        }

        return 0;
    }



    void child_add_call_(bool owned ,unsigned char a0 ,server_type_ * a1 )
    {
        if(owned)
        {
            
                
                
                    return server_type_::child_add( a0  ,a1 );
                
            
            
        }

        return
            
                child_add
            
            
                ( a0  ,a1 );
    }

    static PyObject *child_add_method_(server_object_ *self, PyObject *args)
    {
         unsigned char a0;  server_type_ * a1; 

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_uc,&a0  ,fpcvt_server,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->child_add_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::child_add");
        }

        return 0;
    }



    bool opening_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::opening();
                
            
            
        }

        return
            
                opening
            
            
                ();
    }

    static PyObject *opening_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->opening_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::opening");
        }

        return 0;
    }



    bool closing_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::closing();
                
            
            
        }

        return
            
                closing
            
            
                ();
    }

    static PyObject *closing_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->closing_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::closing");
        }

        return 0;
    }



    bool running_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::running();
                
            
            
        }

        return
            
                running
            
            
                ();
    }

    static PyObject *running_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->running_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::running");
        }

        return 0;
    }



    void set_source_call_(bool owned ,fastdata_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return server_type_::set_source( a0 );
                
            
            
        }

        return
            
                set_source
            
            
                ( a0 );
    }

    static PyObject *set_source_method_(server_object_ *self, PyObject *args)
    {
         fastdata_type_ * a0; 

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_fastdata,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_source_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::set_source");
        }

        return 0;
    }



    void clear_source_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::clear_source();
                
            
            
        }

        return
            
                clear_source
            
            
                ();
    }

    static PyObject *clear_source_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_source_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::clear_source");
        }

        return 0;
    }



    void set_flags_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return server_type_::set_flags( a0 );
                
            
            
        }

        return
            
                set_flags
            
            
                ( a0 );
    }

    static PyObject *set_flags_method_(server_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_flags_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::set_flags");
        }

        return 0;
    }



    unsigned int get_flags_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::get_flags();
                
            
            
        }

        return
            
                get_flags
            
            
                ();
    }

    static PyObject *get_flags_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_flags_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::get_flags");
        }

        return 0;
    }



    void set_readonly_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::set_readonly();
                
            
            
        }

        return
            
                set_readonly
            
            
                ();
    }

    static PyObject *set_readonly_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_readonly_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::set_readonly");
        }

        return 0;
    }



    void set_readwrite_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::set_readwrite();
                
            
            
        }

        return
            
                set_readwrite
            
            
                ();
    }

    static PyObject *set_readwrite_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_readwrite_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::set_readwrite");
        }

        return 0;
    }



    void set_clock_call_(bool owned ,clockref_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return server_type_::set_clock( a0 );
                
            
            
        }

        return
            
                set_clock
            
            
                ( a0 );
    }

    static PyObject *set_clock_method_(server_object_ *self, PyObject *args)
    {
         clockref_type_ * a0; 

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_clockref,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_clock_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::set_clock");
        }

        return 0;
    }



    void clear_clock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::clear_clock();
                
            
            
        }

        return
            
                clear_clock
            
            
                ();
    }

    static PyObject *clear_clock_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_clock_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::clear_clock");
        }

        return 0;
    }



    std::string id_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return server_type_::id();
                
            
            
        }

        return
            
                id
            
            
                ();
    }

    static PyObject *id_method_(server_object_ *self, PyObject *args)
    {
        

        server_wrapper_ *o = (server_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null server c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->id_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in server::id");
        }

        return 0;
    }





    static PyObject *_method_lock(server_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(server_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void server_delete_(server_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            server_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (server_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        server_wrapper_::__deleter__((server_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void server_dealloc_(server_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    server_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * server_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    server_object_ *self;

    if(!(self = (server_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int server_init_(server_object_ *self, PyObject *args, PyObject *kwds)
{
    server_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new server_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing server");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            server_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         unsigned int a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new server_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing server");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            server_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"server invalid arguments");
    return -1;

}


static const char *server_server_opened_docstring_ = ""
"        self.server_opened() -> ."
"        "
"        abstract method invoked when the server is opened.  Can be used"
"        for initialisation actions which depend on the server being open."
"        "
"        When implementing this action, you should call server.server_opened"
"        to perform the default actions"
"        "
""
;

static const char *server_server_closed_docstring_ = ""
"        self.server_closed() -> ."
"        "
"        abstract method invoked when the server is closed.  When this "
"        method is called, the server is still partially open, in that the"
"        servername() and path() methods are still valid."
"        "
"        When implementing this action, you should call server.server_closed"
"        to perform the default actions."
"        "
"        Note that this method is invoked only when the server is closed by "
"        outside agency."
"        "
""
;

static const char *server_server_change_docstring_ = ""
"        self.server_change(requested_data) -> ."
"        "
"        abstract method invoked when a belcanto client requests a data change."
"        When implementing this action, you should call server.server_closed"
"        to perform the default actions."
"        "
""
;

static const char *server_close_server_docstring_ = ""
"        self.close_server() -> ."
"        "
"        Call to close a server.  This method can be overidden to perform "
"        extra cleanup actions, but the default implementation should always"
"        be called with server.close_server."
"        "
"        After the default implementation is called, many server methods are"
"        no longer available, such as servername() and path()"
"        "
""
;

static const char *server_server_tree_docstring_ = "";

static const char *server_set_data_docstring_ = ""
"        self.set(data) -> ."
"        "
"        Change the value of this server.  The server need not be open.  The"
"        current value will be applied when the server is opened."
"        "
""
;

static const char *server_get_data_docstring_ = ""
"        self.get() -> current value"
"        "
"        returns the current value of the server as changed with set()"
"        "
""
;

static const char *server_has_value_docstring_ = ""
"        self.has_value() -> ."
"        "
"        Does the server currently have a non-null value?"
"        "
""
;

static const char *server_advertise_docstring_ = ""
"        self.advertise(name) -> ."
""
"        Advertise this server on index name"
"        "
""
;

static const char *server_unadvertise_docstring_ = ""
"        self.unadvertise(name) -> ."
""
"        Stop advertising this server on index name"
"        "
""
;

static const char *server_servername_docstring_ = ""
"        self.servername() -> belcanto name of state tree"
"        returns the belcanto name of the state variable tree of which this"
"        server is part.  The server must be open"
"        "
""
;

static const char *server_path_docstring_ = ""
"        self.path() -> state variable name within server tree"
"        "
"        returns the name of this server within the tree of which it is"
"        a part.  the server must be open."
"        "
""
;

static const char *server_set_change_handler_docstring_ = ""
"        self.set_change_handler(change_functor) -> ."
"        "
"        Install a functor which will be called when a client requests "
"        a change of value.  Only one functor at a time can be installed"
"        "
""
;

static const char *server_clear_change_handler_docstring_ = ""
"        self.clear_change_handler() -> ."
"        "
"        clear the change handler"
"        "
""
;

static const char *server_open_docstring_ = ""
"        open() -> current state"
"        "
"        returns whether the server is open"
"        "
""
;

static const char *server_child_add_docstring_ = ""
"        child_add(name,server) -> .  "
""
"        add server as child of his one"
"        "
""
;

static const char *server_opening_docstring_ = "";

static const char *server_closing_docstring_ = "";

static const char *server_running_docstring_ = "";

static const char *server_set_source_docstring_ = "";

static const char *server_clear_source_docstring_ = "";

static const char *server_set_flags_docstring_ = "";

static const char *server_get_flags_docstring_ = "";

static const char *server_set_readonly_docstring_ = "";

static const char *server_set_readwrite_docstring_ = "";

static const char *server_set_clock_docstring_ = "";

static const char *server_clear_clock_docstring_ = "";

static const char *server_id_docstring_ = "";




static PyMethodDef server_methods_[] = {
    
        { "server_opened",(PyCFunction)server_wrapper_::server_opened_method_,METH_VARARGS,(char *)server_server_opened_docstring_ },
    
        { "server_closed",(PyCFunction)server_wrapper_::server_closed_method_,METH_VARARGS,(char *)server_server_closed_docstring_ },
    
        { "server_change",(PyCFunction)server_wrapper_::server_change_method_,METH_VARARGS,(char *)server_server_change_docstring_ },
    
        { "close_server",(PyCFunction)server_wrapper_::close_server_method_,METH_VARARGS,(char *)server_close_server_docstring_ },
    
        { "server_tree",(PyCFunction)server_wrapper_::server_tree_method_,METH_VARARGS,(char *)server_server_tree_docstring_ },
    
        { "set_data",(PyCFunction)server_wrapper_::set_data_method_,METH_VARARGS,(char *)server_set_data_docstring_ },
    
        { "get_data",(PyCFunction)server_wrapper_::get_data_method_,METH_VARARGS,(char *)server_get_data_docstring_ },
    
        { "has_value",(PyCFunction)server_wrapper_::has_value_method_,METH_VARARGS,(char *)server_has_value_docstring_ },
    
        { "advertise",(PyCFunction)server_wrapper_::advertise_method_,METH_VARARGS,(char *)server_advertise_docstring_ },
    
        { "unadvertise",(PyCFunction)server_wrapper_::unadvertise_method_,METH_VARARGS,(char *)server_unadvertise_docstring_ },
    
        { "servername",(PyCFunction)server_wrapper_::servername_method_,METH_VARARGS,(char *)server_servername_docstring_ },
    
        { "path",(PyCFunction)server_wrapper_::path_method_,METH_VARARGS,(char *)server_path_docstring_ },
    
        { "set_change_handler",(PyCFunction)server_wrapper_::set_change_handler_method_,METH_VARARGS,(char *)server_set_change_handler_docstring_ },
    
        { "clear_change_handler",(PyCFunction)server_wrapper_::clear_change_handler_method_,METH_VARARGS,(char *)server_clear_change_handler_docstring_ },
    
        { "open",(PyCFunction)server_wrapper_::open_method_,METH_VARARGS,(char *)server_open_docstring_ },
    
        { "child_add",(PyCFunction)server_wrapper_::child_add_method_,METH_VARARGS,(char *)server_child_add_docstring_ },
    
        { "opening",(PyCFunction)server_wrapper_::opening_method_,METH_VARARGS,(char *)server_opening_docstring_ },
    
        { "closing",(PyCFunction)server_wrapper_::closing_method_,METH_VARARGS,(char *)server_closing_docstring_ },
    
        { "running",(PyCFunction)server_wrapper_::running_method_,METH_VARARGS,(char *)server_running_docstring_ },
    
        { "set_source",(PyCFunction)server_wrapper_::set_source_method_,METH_VARARGS,(char *)server_set_source_docstring_ },
    
        { "clear_source",(PyCFunction)server_wrapper_::clear_source_method_,METH_VARARGS,(char *)server_clear_source_docstring_ },
    
        { "set_flags",(PyCFunction)server_wrapper_::set_flags_method_,METH_VARARGS,(char *)server_set_flags_docstring_ },
    
        { "get_flags",(PyCFunction)server_wrapper_::get_flags_method_,METH_VARARGS,(char *)server_get_flags_docstring_ },
    
        { "set_readonly",(PyCFunction)server_wrapper_::set_readonly_method_,METH_VARARGS,(char *)server_set_readonly_docstring_ },
    
        { "set_readwrite",(PyCFunction)server_wrapper_::set_readwrite_method_,METH_VARARGS,(char *)server_set_readwrite_docstring_ },
    
        { "set_clock",(PyCFunction)server_wrapper_::set_clock_method_,METH_VARARGS,(char *)server_set_clock_docstring_ },
    
        { "clear_clock",(PyCFunction)server_wrapper_::clear_clock_method_,METH_VARARGS,(char *)server_clear_clock_docstring_ },
    
        { "id",(PyCFunction)server_wrapper_::id_method_,METH_VARARGS,(char *)server_id_docstring_ },
    
    
        { "py_lock",(PyCFunction)server_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)server_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *server_docstring_ = ""
"    State variable tree base class"
"    "
"    server is the base class of all state variables and idioms."
"    It represents a single state variable, which is the root of it's"
"    subtree.  When a server is created,"
"    it is closed.  It can be opened either by being made a"
"    child of another server, or by being made a root of a state variable"
"    tree.  When a server is opened, the server_opened method is called"
"    which can be overridden if desired."
"    "
"    Servers have a type which cannot be changed.  The value of the server"
"    is set only by this class.  When a client requests a change, server_change"
"    is called, which can be overridden.  Unless set() is called to update"
"    the value, the change will not be visible to clients."
"    "
"    The default value of a server is an appropraite null of the correct type,"
"    such as 0, 0.0, empty string, false, etc."
"    "
"    When a server is opened, it may add child servers to itself to form"
"    a tree.  Various classes derived from server allow tree's to be formed"
"    in different ways."
"    "
"    Changes can be responded to in one of two ways.  If one is deriving from"
"    server, the most convenient way is to implement server_changed.  Otherwise,"
"    (typically when server is part of a larger aggregate) one may register a"
"    change functor which will be invoked upon a change request."
"    "
"    Initialisers"
"    ============"
"    "
"    server(flags)"
"       - flags: compination of server_ro,server_fast etc"
"    "
""
;

PyTypeObject server_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.server",/*tp_name*/
    sizeof(server_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)server_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)server_docstring_,       /* tp_doc */
    (traverseproc)server_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)server_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    server_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)server_init_,  /* tp_init */
    0,                         /* tp_alloc */
    server_new_,             /* tp_new */
};

PyObject *server_wrapcopy_(server_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *server_wrap_(server_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = server_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    server_object_ *self = 
        
        PyObject_GC_New(server_object_, &server_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        server_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

server_type_ *server_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &server_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected server object");
        return 0;
    }

    server_type_ *object = static_cast<server_type_ *>(((server_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null server object in unwrap");
        return 0;
    }

    return object;
}

void server_insert_(const std::pair< server_type_ *, PyObject *> &p)
{




    server_wrapped_.insert(p);


}

void server_erase_(server_type_ *p)
{




    server_wrapped_.erase(p);


}

PyObject * server_find_(server_type_ *p)
{




    PyObject *ret = 0;
    std::map< server_type_ *, PyObject *>::iterator where = server_wrapped_.find(p);
    if(where != server_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct server_dispatch_ server_realdispatcher_ =
{
    &server_typeobject_,
    server_wrap_,
    server_wrapcopy_,
    server_unwrap_,
    server_insert_,
    server_erase_,
    server_find_
};

  

struct client_object_ {
    PyObject_HEAD
    client_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct client_wrapper_: public client_type_ 
{

    static void __deleter__(client_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
        cache_4_ = 0;
    
        cache_5_ = 0;
    
        cache_6_ = 0;
    
        cache_7_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
        if(cache_4_)
        {
            Py_DECREF(cache_4_);
            cache_4_ = 0;
        }
    
        if(cache_5_)
        {
            Py_DECREF(cache_5_);
            cache_5_ = 0;
        }
    
        if(cache_6_)
        {
            Py_DECREF(cache_6_);
            cache_6_ = 0;
        }
    
        if(cache_7_)
        {
            Py_DECREF(cache_7_);
            cache_7_ = 0;
        }
    
    }
*/



    client_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned int a0  ): client_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~client_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void close_client(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"close_client");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("close_client before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("close_client after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void client_tree(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_tree");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_tree before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_tree after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void client_child(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_child");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_child before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_child after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    void client_data(  const data_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_data");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_data_copy,&a0);
        //printf("client_data before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_data after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_4_;
 
    void client_opened(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_4_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_opened");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_4_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_opened before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_opened after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_5_;
 
    void client_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_5_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_5_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_6_;
 
    void client_sync(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_6_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_sync");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_6_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_sync before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_sync after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_7_;
 
    void client_clock(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_7_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_clock");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_7_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_clock before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_clock after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }













    static int gc_traverse_method_(client_object_ *self, visitproc visit, void *arg)
    {
        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(client_object_ *self)
    {
        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    const data_type_ & get_data_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::get_data();
                
            
            
        }

        return
            
                get_data
            
            
                ();
    }

    static PyObject *get_data_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    const data_type_ & r = o->get_data_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::get_data");
        }

        return 0;
    }



    void set_data_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::set_data( a0 );
                
            
            
        }

        return
            
                set_data
            
            
                ( a0 );
    }

    static PyObject *set_data_method_(client_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_data_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::set_data");
        }

        return 0;
    }



    bool open_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::open();
                
            
            
        }

        return
            
                open
            
            
                ();
    }

    static PyObject *open_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::open");
        }

        return 0;
    }



    bool opening_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::opening();
                
            
            
        }

        return
            
                opening
            
            
                ();
    }

    static PyObject *opening_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->opening_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::opening");
        }

        return 0;
    }



    bool closing_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::closing();
                
            
            
        }

        return
            
                closing
            
            
                ();
    }

    static PyObject *closing_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->closing_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::closing");
        }

        return 0;
    }



    bool running_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::running();
                
            
            
        }

        return
            
                running
            
            
                ();
    }

    static PyObject *running_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->running_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::running");
        }

        return 0;
    }



    unsigned short cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned short r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::cookie");
        }

        return 0;
    }



    unsigned int get_host_flags_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::get_host_flags();
                
            
            
        }

        return
            
                get_host_flags
            
            
                ();
    }

    static PyObject *get_host_flags_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_host_flags_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::get_host_flags");
        }

        return 0;
    }



    unsigned int get_plug_flags_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::get_plug_flags();
                
            
            
        }

        return
            
                get_plug_flags
            
            
                ();
    }

    static PyObject *get_plug_flags_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_plug_flags_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::get_plug_flags");
        }

        return 0;
    }



    data_type_ servername_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::servername();
                
            
            
        }

        return
            
                servername
            
            
                ();
    }

    static PyObject *servername_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->servername_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::servername");
        }

        return 0;
    }



    data_type_ path_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::path();
                
            
            
        }

        return
            
                path
            
            
                ();
    }

    static PyObject *path_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->path_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::path");
        }

        return 0;
    }



    void sync_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::sync();
                
            
            
        }

        return
            
                sync
            
            
                ();
    }

    static PyObject *sync_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->sync_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::sync");
        }

        return 0;
    }



    unsigned char enum_child_call_(bool owned ,unsigned char a0 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::enum_child( a0 );
                
            
            
        }

        return
            
                enum_child
            
            
                ( a0 );
    }

    static PyObject *enum_child_method_(client_object_ *self, PyObject *args)
    {
         unsigned char a0; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_uc,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned char r = o->enum_child_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_uc((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::enum_child");
        }

        return 0;
    }



    void clone_call_(bool owned ,client_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::clone( a0 );
                
            
            
        }

        return
            
                clone
            
            
                ( a0 );
    }

    static PyObject *clone_method_(client_object_ *self, PyObject *args)
    {
         client_type_ * a0; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_client,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clone_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::clone");
        }

        return 0;
    }



    void child_add_call_(bool owned ,unsigned char a0 ,client_type_ * a1 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::child_add( a0  ,a1 );
                
            
            
        }

        return
            
                child_add
            
            
                ( a0  ,a1 );
    }

    static PyObject *child_add_method_(client_object_ *self, PyObject *args)
    {
         unsigned char a0;  client_type_ * a1; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_uc,&a0  ,fpcvt_client,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->child_add_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::child_add");
        }

        return 0;
    }



    void child_add_str_call_(bool owned ,const unsigned char * a0 ,unsigned int a1 ,client_type_ * a2 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::child_add_str( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                child_add_str
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *child_add_str_method_(client_object_ *self, PyObject *args)
    {
         const unsigned char * a0;  unsigned int a1;  client_type_ * a2; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_ustr,&a0  ,fpcvt_ui,&a1  ,fpcvt_client,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->child_add_str_call_ (self->owned ,a0,a1,a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::child_add_str");
        }

        return 0;
    }



    unsigned int child_flags_str_call_(bool owned ,const unsigned char * a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::child_flags_str( a0  ,a1 );
                
            
            
        }

        return
            
                child_flags_str
            
            
                ( a0  ,a1 );
    }

    static PyObject *child_flags_str_method_(client_object_ *self, PyObject *args)
    {
         const unsigned char * a0;  unsigned int a1; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ustr,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->child_flags_str_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::child_flags_str");
        }

        return 0;
    }



    data_type_ child_data_str_call_(bool owned ,const unsigned char * a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::child_data_str( a0  ,a1 );
                
            
            
        }

        return
            
                child_data_str
            
            
                ( a0  ,a1 );
    }

    static PyObject *child_data_str_method_(client_object_ *self, PyObject *args)
    {
         const unsigned char * a0;  unsigned int a1; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ustr,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->child_data_str_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::child_data_str");
        }

        return 0;
    }



    bool child_exists_str_call_(bool owned ,const unsigned char * a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::child_exists_str( a0  ,a1 );
                
            
            
        }

        return
            
                child_exists_str
            
            
                ( a0  ,a1 );
    }

    static PyObject *child_exists_str_method_(client_object_ *self, PyObject *args)
    {
         const unsigned char * a0;  unsigned int a1; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ustr,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->child_exists_str_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::child_exists_str");
        }

        return 0;
    }



    void child_change_str_call_(bool owned ,const unsigned char * a0 ,unsigned int a1 ,const data_type_ & a2 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::child_change_str( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                child_change_str
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *child_change_str_method_(client_object_ *self, PyObject *args)
    {
         const unsigned char * a0;  unsigned int a1;  data_type_ * a2; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_ustr,&a0  ,fpcvt_ui,&a1  ,fpcvt_data_copy,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->child_change_str_call_ (self->owned ,a0,a1,*a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::child_change_str");
        }

        return 0;
    }



    unsigned char child_enum_child_str_call_(bool owned ,const unsigned char * a0 ,unsigned int a1 ,unsigned char a2 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::child_enum_child_str( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                child_enum_child_str
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *child_enum_child_str_method_(client_object_ *self, PyObject *args)
    {
         const unsigned char * a0;  unsigned int a1;  unsigned char a2; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_ustr,&a0  ,fpcvt_ui,&a1  ,fpcvt_uc,&a2 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned char r = o->child_enum_child_str_call_ (self->owned ,a0 ,a1 ,a2  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_uc((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::child_enum_child_str");
        }

        return 0;
    }



    unsigned long child_dcrc_call_(bool owned ,const unsigned char * a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::child_dcrc( a0  ,a1 );
                
            
            
        }

        return
            
                child_dcrc
            
            
                ( a0  ,a1 );
    }

    static PyObject *child_dcrc_method_(client_object_ *self, PyObject *args)
    {
         const unsigned char * a0;  unsigned int a1; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ustr,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->child_dcrc_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::child_dcrc");
        }

        return 0;
    }



    unsigned long child_tcrc_call_(bool owned ,const unsigned char * a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::child_tcrc( a0  ,a1 );
                
            
            
        }

        return
            
                child_tcrc
            
            
                ( a0  ,a1 );
    }

    static PyObject *child_tcrc_method_(client_object_ *self, PyObject *args)
    {
         const unsigned char * a0;  unsigned int a1; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ustr,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->child_tcrc_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::child_tcrc");
        }

        return 0;
    }



    unsigned long child_ncrc_call_(bool owned ,const unsigned char * a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::child_ncrc( a0  ,a1 );
                
            
            
        }

        return
            
                child_ncrc
            
            
                ( a0  ,a1 );
    }

    static PyObject *child_ncrc_method_(client_object_ *self, PyObject *args)
    {
         const unsigned char * a0;  unsigned int a1; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ustr,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->child_ncrc_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::child_ncrc");
        }

        return 0;
    }



    void set_tree_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::set_tree( a0 );
                
            
            
        }

        return
            
                set_tree
            
            
                ( a0 );
    }

    static PyObject *set_tree_method_(client_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_tree_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::set_tree");
        }

        return 0;
    }



    void set_change_handler_call_(bool owned ,const change_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::set_change_handler( a0 );
                
            
            
        }

        return
            
                set_change_handler
            
            
                ( a0 );
    }

    static PyObject *set_change_handler_method_(client_object_ *self, PyObject *args)
    {
         change_type_ * a0; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_change_handler_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::set_change_handler");
        }

        return 0;
    }



    void clear_change_handler_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::clear_change_handler();
                
            
            
        }

        return
            
                clear_change_handler
            
            
                ();
    }

    static PyObject *clear_change_handler_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_change_handler_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::clear_change_handler");
        }

        return 0;
    }



    void set_sink_call_(bool owned ,fastdata_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::set_sink( a0 );
                
            
            
        }

        return
            
                set_sink
            
            
                ( a0 );
    }

    static PyObject *set_sink_method_(client_object_ *self, PyObject *args)
    {
         fastdata_type_ * a0; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_fastdata,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_sink_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::set_sink");
        }

        return 0;
    }



    void clear_sink_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::clear_sink();
                
            
            
        }

        return
            
                clear_sink
            
            
                ();
    }

    static PyObject *clear_sink_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_sink_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::clear_sink");
        }

        return 0;
    }



    bool set_downstream_call_(bool owned ,clockref_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::set_downstream( a0 );
                
            
            
        }

        return
            
                set_downstream
            
            
                ( a0 );
    }

    static PyObject *set_downstream_method_(client_object_ *self, PyObject *args)
    {
         clockref_type_ * a0; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_clockref,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->set_downstream_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::set_downstream");
        }

        return 0;
    }



    void clear_downstream_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::clear_downstream();
                
            
            
        }

        return
            
                clear_downstream
            
            
                ();
    }

    static PyObject *clear_downstream_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_downstream_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::clear_downstream");
        }

        return 0;
    }



    unsigned long dcrc_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::dcrc();
                
            
            
        }

        return
            
                dcrc
            
            
                ();
    }

    static PyObject *dcrc_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->dcrc_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::dcrc");
        }

        return 0;
    }



    unsigned long tcrc_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::tcrc();
                
            
            
        }

        return
            
                tcrc
            
            
                ();
    }

    static PyObject *tcrc_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->tcrc_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::tcrc");
        }

        return 0;
    }



    unsigned long ncrc_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::ncrc();
                
            
            
        }

        return
            
                ncrc
            
            
                ();
    }

    static PyObject *ncrc_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->ncrc_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::ncrc");
        }

        return 0;
    }



    std::string id_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::id();
                
            
            
        }

        return
            
                id
            
            
                ();
    }

    static PyObject *id_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->id_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::id");
        }

        return 0;
    }



    void close_client_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::close_client();
                
            
            
        }

        return
            
                close_client
            
            
                ();
    }

    static PyObject *close_client_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_client_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::close_client");
        }

        return 0;
    }



    void client_tree_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::client_tree();
                
            
            
        }

        return
            
                client_tree
            
            
                ();
    }

    static PyObject *client_tree_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_tree_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::client_tree");
        }

        return 0;
    }



    void client_child_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::client_child();
                
            
            
        }

        return
            
                client_child
            
            
                ();
    }

    static PyObject *client_child_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_child_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::client_child");
        }

        return 0;
    }



    void client_data_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return client_type_::client_data( a0 );
                
            
            
        }

        return
            
                client_data
            
            
                ( a0 );
    }

    static PyObject *client_data_method_(client_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_data_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::client_data");
        }

        return 0;
    }



    void client_opened_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::client_opened();
                
            
            
        }

        return
            
                client_opened
            
            
                ();
    }

    static PyObject *client_opened_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_opened_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::client_opened");
        }

        return 0;
    }



    void client_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::client_closed();
                
            
            
        }

        return
            
                client_closed
            
            
                ();
    }

    static PyObject *client_closed_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::client_closed");
        }

        return 0;
    }



    void client_sync_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::client_sync();
                
            
            
        }

        return
            
                client_sync
            
            
                ();
    }

    static PyObject *client_sync_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_sync_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::client_sync");
        }

        return 0;
    }



    void client_clock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return client_type_::client_clock();
                
            
            
        }

        return
            
                client_clock
            
            
                ();
    }

    static PyObject *client_clock_method_(client_object_ *self, PyObject *args)
    {
        

        client_wrapper_ *o = (client_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null client c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_clock_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in client::client_clock");
        }

        return 0;
    }





    static PyObject *_method_lock(client_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(client_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void client_delete_(client_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            client_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (client_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        client_wrapper_::__deleter__((client_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void client_dealloc_(client_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    client_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * client_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    client_object_ *self;

    if(!(self = (client_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int client_init_(client_object_ *self, PyObject *args, PyObject *kwds)
{
    client_delete_(self);

    

    {
         unsigned int a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new client_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing client");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            client_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"client invalid arguments");
    return -1;

}


static const char *client_get_data_docstring_ = "";

static const char *client_set_data_docstring_ = "";

static const char *client_open_docstring_ = "";

static const char *client_opening_docstring_ = "";

static const char *client_closing_docstring_ = "";

static const char *client_running_docstring_ = "";

static const char *client_cookie_docstring_ = "";

static const char *client_get_host_flags_docstring_ = "";

static const char *client_get_plug_flags_docstring_ = "";

static const char *client_servername_docstring_ = "";

static const char *client_path_docstring_ = "";

static const char *client_sync_docstring_ = "";

static const char *client_enum_child_docstring_ = "";

static const char *client_clone_docstring_ = "";

static const char *client_child_add_docstring_ = "";

static const char *client_child_add_str_docstring_ = "";

static const char *client_child_flags_str_docstring_ = "";

static const char *client_child_data_str_docstring_ = "";

static const char *client_child_exists_str_docstring_ = "";

static const char *client_child_change_str_docstring_ = "";

static const char *client_child_enum_child_str_docstring_ = "";

static const char *client_child_dcrc_docstring_ = "";

static const char *client_child_tcrc_docstring_ = "";

static const char *client_child_ncrc_docstring_ = "";

static const char *client_set_tree_docstring_ = "";

static const char *client_set_change_handler_docstring_ = "";

static const char *client_clear_change_handler_docstring_ = "";

static const char *client_set_sink_docstring_ = "";

static const char *client_clear_sink_docstring_ = "";

static const char *client_set_downstream_docstring_ = "";

static const char *client_clear_downstream_docstring_ = "";

static const char *client_dcrc_docstring_ = "";

static const char *client_tcrc_docstring_ = "";

static const char *client_ncrc_docstring_ = "";

static const char *client_id_docstring_ = "";

static const char *client_close_client_docstring_ = "";

static const char *client_client_tree_docstring_ = "";

static const char *client_client_child_docstring_ = "";

static const char *client_client_data_docstring_ = "";

static const char *client_client_opened_docstring_ = "";

static const char *client_client_closed_docstring_ = "";

static const char *client_client_sync_docstring_ = "";

static const char *client_client_clock_docstring_ = "";




static PyMethodDef client_methods_[] = {
    
        { "get_data",(PyCFunction)client_wrapper_::get_data_method_,METH_VARARGS,(char *)client_get_data_docstring_ },
    
        { "set_data",(PyCFunction)client_wrapper_::set_data_method_,METH_VARARGS,(char *)client_set_data_docstring_ },
    
        { "open",(PyCFunction)client_wrapper_::open_method_,METH_VARARGS,(char *)client_open_docstring_ },
    
        { "opening",(PyCFunction)client_wrapper_::opening_method_,METH_VARARGS,(char *)client_opening_docstring_ },
    
        { "closing",(PyCFunction)client_wrapper_::closing_method_,METH_VARARGS,(char *)client_closing_docstring_ },
    
        { "running",(PyCFunction)client_wrapper_::running_method_,METH_VARARGS,(char *)client_running_docstring_ },
    
        { "cookie",(PyCFunction)client_wrapper_::cookie_method_,METH_VARARGS,(char *)client_cookie_docstring_ },
    
        { "get_host_flags",(PyCFunction)client_wrapper_::get_host_flags_method_,METH_VARARGS,(char *)client_get_host_flags_docstring_ },
    
        { "get_plug_flags",(PyCFunction)client_wrapper_::get_plug_flags_method_,METH_VARARGS,(char *)client_get_plug_flags_docstring_ },
    
        { "servername",(PyCFunction)client_wrapper_::servername_method_,METH_VARARGS,(char *)client_servername_docstring_ },
    
        { "path",(PyCFunction)client_wrapper_::path_method_,METH_VARARGS,(char *)client_path_docstring_ },
    
        { "sync",(PyCFunction)client_wrapper_::sync_method_,METH_VARARGS,(char *)client_sync_docstring_ },
    
        { "enum_child",(PyCFunction)client_wrapper_::enum_child_method_,METH_VARARGS,(char *)client_enum_child_docstring_ },
    
        { "clone",(PyCFunction)client_wrapper_::clone_method_,METH_VARARGS,(char *)client_clone_docstring_ },
    
        { "child_add",(PyCFunction)client_wrapper_::child_add_method_,METH_VARARGS,(char *)client_child_add_docstring_ },
    
        { "child_add_str",(PyCFunction)client_wrapper_::child_add_str_method_,METH_VARARGS,(char *)client_child_add_str_docstring_ },
    
        { "child_flags_str",(PyCFunction)client_wrapper_::child_flags_str_method_,METH_VARARGS,(char *)client_child_flags_str_docstring_ },
    
        { "child_data_str",(PyCFunction)client_wrapper_::child_data_str_method_,METH_VARARGS,(char *)client_child_data_str_docstring_ },
    
        { "child_exists_str",(PyCFunction)client_wrapper_::child_exists_str_method_,METH_VARARGS,(char *)client_child_exists_str_docstring_ },
    
        { "child_change_str",(PyCFunction)client_wrapper_::child_change_str_method_,METH_VARARGS,(char *)client_child_change_str_docstring_ },
    
        { "child_enum_child_str",(PyCFunction)client_wrapper_::child_enum_child_str_method_,METH_VARARGS,(char *)client_child_enum_child_str_docstring_ },
    
        { "child_dcrc",(PyCFunction)client_wrapper_::child_dcrc_method_,METH_VARARGS,(char *)client_child_dcrc_docstring_ },
    
        { "child_tcrc",(PyCFunction)client_wrapper_::child_tcrc_method_,METH_VARARGS,(char *)client_child_tcrc_docstring_ },
    
        { "child_ncrc",(PyCFunction)client_wrapper_::child_ncrc_method_,METH_VARARGS,(char *)client_child_ncrc_docstring_ },
    
        { "set_tree",(PyCFunction)client_wrapper_::set_tree_method_,METH_VARARGS,(char *)client_set_tree_docstring_ },
    
        { "set_change_handler",(PyCFunction)client_wrapper_::set_change_handler_method_,METH_VARARGS,(char *)client_set_change_handler_docstring_ },
    
        { "clear_change_handler",(PyCFunction)client_wrapper_::clear_change_handler_method_,METH_VARARGS,(char *)client_clear_change_handler_docstring_ },
    
        { "set_sink",(PyCFunction)client_wrapper_::set_sink_method_,METH_VARARGS,(char *)client_set_sink_docstring_ },
    
        { "clear_sink",(PyCFunction)client_wrapper_::clear_sink_method_,METH_VARARGS,(char *)client_clear_sink_docstring_ },
    
        { "set_downstream",(PyCFunction)client_wrapper_::set_downstream_method_,METH_VARARGS,(char *)client_set_downstream_docstring_ },
    
        { "clear_downstream",(PyCFunction)client_wrapper_::clear_downstream_method_,METH_VARARGS,(char *)client_clear_downstream_docstring_ },
    
        { "dcrc",(PyCFunction)client_wrapper_::dcrc_method_,METH_VARARGS,(char *)client_dcrc_docstring_ },
    
        { "tcrc",(PyCFunction)client_wrapper_::tcrc_method_,METH_VARARGS,(char *)client_tcrc_docstring_ },
    
        { "ncrc",(PyCFunction)client_wrapper_::ncrc_method_,METH_VARARGS,(char *)client_ncrc_docstring_ },
    
        { "id",(PyCFunction)client_wrapper_::id_method_,METH_VARARGS,(char *)client_id_docstring_ },
    
        { "close_client",(PyCFunction)client_wrapper_::close_client_method_,METH_VARARGS,(char *)client_close_client_docstring_ },
    
        { "client_tree",(PyCFunction)client_wrapper_::client_tree_method_,METH_VARARGS,(char *)client_client_tree_docstring_ },
    
        { "client_child",(PyCFunction)client_wrapper_::client_child_method_,METH_VARARGS,(char *)client_client_child_docstring_ },
    
        { "client_data",(PyCFunction)client_wrapper_::client_data_method_,METH_VARARGS,(char *)client_client_data_docstring_ },
    
        { "client_opened",(PyCFunction)client_wrapper_::client_opened_method_,METH_VARARGS,(char *)client_client_opened_docstring_ },
    
        { "client_closed",(PyCFunction)client_wrapper_::client_closed_method_,METH_VARARGS,(char *)client_client_closed_docstring_ },
    
        { "client_sync",(PyCFunction)client_wrapper_::client_sync_method_,METH_VARARGS,(char *)client_client_sync_docstring_ },
    
        { "client_clock",(PyCFunction)client_wrapper_::client_clock_method_,METH_VARARGS,(char *)client_client_clock_docstring_ },
    
    
        { "py_lock",(PyCFunction)client_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)client_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *client_docstring_ = "";

PyTypeObject client_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.client",/*tp_name*/
    sizeof(client_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)client_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)client_docstring_,       /* tp_doc */
    (traverseproc)client_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)client_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    client_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)client_init_,  /* tp_init */
    0,                         /* tp_alloc */
    client_new_,             /* tp_new */
};

PyObject *client_wrapcopy_(client_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *client_wrap_(client_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = client_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    client_object_ *self = 
        
        PyObject_GC_New(client_object_, &client_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        client_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

client_type_ *client_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &client_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected client object");
        return 0;
    }

    client_type_ *object = static_cast<client_type_ *>(((client_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null client object in unwrap");
        return 0;
    }

    return object;
}

void client_insert_(const std::pair< client_type_ *, PyObject *> &p)
{




    client_wrapped_.insert(p);


}

void client_erase_(client_type_ *p)
{




    client_wrapped_.erase(p);


}

PyObject * client_find_(client_type_ *p)
{




    PyObject *ret = 0;
    std::map< client_type_ *, PyObject *>::iterator where = client_wrapped_.find(p);
    if(where != client_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct client_dispatch_ client_realdispatcher_ =
{
    &client_typeobject_,
    client_wrap_,
    client_wrapcopy_,
    client_unwrap_,
    client_insert_,
    client_erase_,
    client_find_
};

  

struct subcanchor_object_ {
    PyObject_HEAD
    client_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct subcanchor_wrapper_: public subcanchor_type_ 
{

    static void __deleter__(subcanchor_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
        cache_4_ = 0;
    
        cache_5_ = 0;
    
        cache_6_ = 0;
    
        cache_7_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
        if(cache_4_)
        {
            Py_DECREF(cache_4_);
            cache_4_ = 0;
        }
    
        if(cache_5_)
        {
            Py_DECREF(cache_5_);
            cache_5_ = 0;
        }
    
        if(cache_6_)
        {
            Py_DECREF(cache_6_);
            cache_6_ = 0;
        }
    
        if(cache_7_)
        {
            Py_DECREF(cache_7_);
            cache_7_ = 0;
        }
    
    }
*/



    subcanchor_wrapper_( PyObject *o, PyInterpreterState *i  ): subcanchor_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~subcanchor_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void client_sync(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_sync");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_sync before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_sync after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void client_tree(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_tree");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_tree before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_tree after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void client_opened(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_opened");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_opened before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_opened after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    void close_client(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"close_client");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("close_client before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("close_client after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_4_;
 
    void client_data(  const data_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_4_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_data");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_4_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_data_copy,&a0);
        //printf("client_data before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_data after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_5_;
 
    void client_clock(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_5_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_clock");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_5_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_clock before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_clock after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_6_;
 
    void client_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_6_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_6_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_7_;
 
    void client_child(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_7_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_child");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_7_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_child before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_child after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }













    static int gc_traverse_method_(subcanchor_object_ *self, visitproc visit, void *arg)
    {
        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(subcanchor_object_ *self)
    {
        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void client_sync_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::client_sync();
                
            
            
        }

        return
            
                client_sync
            
            
                ();
    }

    static PyObject *client_sync_method_(subcanchor_object_ *self, PyObject *args)
    {
        

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_sync_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::client_sync");
        }

        return 0;
    }



    void client_tree_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::client_tree();
                
            
            
        }

        return
            
                client_tree
            
            
                ();
    }

    static PyObject *client_tree_method_(subcanchor_object_ *self, PyObject *args)
    {
        

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_tree_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::client_tree");
        }

        return 0;
    }



    bool clear_path_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::clear_path();
                
            
            
        }

        return
            
                clear_path
            
            
                ();
    }

    static PyObject *clear_path_method_(subcanchor_object_ *self, PyObject *args)
    {
        

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->clear_path_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::clear_path");
        }

        return 0;
    }



    void client_clock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::client_clock();
                
            
            
        }

        return
            
                client_clock
            
            
                ();
    }

    static PyObject *client_clock_method_(subcanchor_object_ *self, PyObject *args)
    {
        

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_clock_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::client_clock");
        }

        return 0;
    }



    void clear_client_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::clear_client();
                
            
            
        }

        return
            
                clear_client
            
            
                ();
    }

    static PyObject *clear_client_method_(subcanchor_object_ *self, PyObject *args)
    {
        

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_client_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::clear_client");
        }

        return 0;
    }



    void close_client_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::close_client();
                
            
            
        }

        return
            
                close_client
            
            
                ();
    }

    static PyObject *close_client_method_(subcanchor_object_ *self, PyObject *args)
    {
        

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_client_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::close_client");
        }

        return 0;
    }



    std::string get_path_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::get_path();
                
            
            
        }

        return
            
                get_path
            
            
                ();
    }

    static PyObject *get_path_method_(subcanchor_object_ *self, PyObject *args)
    {
        

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_path_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::get_path");
        }

        return 0;
    }



    bool set_path_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::set_path( a0 );
                
            
            
        }

        return
            
                set_path
            
            
                ( a0 );
    }

    static PyObject *set_path_method_(subcanchor_object_ *self, PyObject *args)
    {
         std::string a0; 

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->set_path_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::set_path");
        }

        return 0;
    }



    void client_data_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::client_data( a0 );
                
            
            
        }

        return
            
                client_data
            
            
                ( a0 );
    }

    static PyObject *client_data_method_(subcanchor_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_data_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::client_data");
        }

        return 0;
    }



    void client_opened_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::client_opened();
                
            
            
        }

        return
            
                client_opened
            
            
                ();
    }

    static PyObject *client_opened_method_(subcanchor_object_ *self, PyObject *args)
    {
        

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_opened_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::client_opened");
        }

        return 0;
    }



    void set_client_call_(bool owned ,client_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::set_client( a0 );
                
            
            
        }

        return
            
                set_client
            
            
                ( a0 );
    }

    static PyObject *set_client_method_(subcanchor_object_ *self, PyObject *args)
    {
         client_type_ * a0; 

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_client,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_client_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::set_client");
        }

        return 0;
    }



    void client_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::client_closed();
                
            
            
        }

        return
            
                client_closed
            
            
                ();
    }

    static PyObject *client_closed_method_(subcanchor_object_ *self, PyObject *args)
    {
        

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::client_closed");
        }

        return 0;
    }



    void client_child_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return subcanchor_type_::client_child();
                
            
            
        }

        return
            
                client_child
            
            
                ();
    }

    static PyObject *client_child_method_(subcanchor_object_ *self, PyObject *args)
    {
        

        subcanchor_wrapper_ *o = (subcanchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null subcanchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_child_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in subcanchor::client_child");
        }

        return 0;
    }





    static PyObject *_method_lock(subcanchor_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(subcanchor_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void subcanchor_delete_(subcanchor_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            client_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (subcanchor_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        subcanchor_wrapper_::__deleter__((subcanchor_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void subcanchor_dealloc_(subcanchor_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    subcanchor_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * subcanchor_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    subcanchor_object_ *self;

    if(!(self = (subcanchor_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int subcanchor_init_(subcanchor_object_ *self, PyObject *args, PyObject *kwds)
{
    subcanchor_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new subcanchor_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing subcanchor");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            client_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"subcanchor invalid arguments");
    return -1;

}


static const char *subcanchor_client_sync_docstring_ = "";

static const char *subcanchor_client_tree_docstring_ = "";

static const char *subcanchor_clear_path_docstring_ = "";

static const char *subcanchor_client_clock_docstring_ = "";

static const char *subcanchor_clear_client_docstring_ = "";

static const char *subcanchor_close_client_docstring_ = "";

static const char *subcanchor_get_path_docstring_ = "";

static const char *subcanchor_set_path_docstring_ = "";

static const char *subcanchor_client_data_docstring_ = "";

static const char *subcanchor_client_opened_docstring_ = "";

static const char *subcanchor_set_client_docstring_ = "";

static const char *subcanchor_client_closed_docstring_ = "";

static const char *subcanchor_client_child_docstring_ = "";




static PyMethodDef subcanchor_methods_[] = {
    
        { "client_sync",(PyCFunction)subcanchor_wrapper_::client_sync_method_,METH_VARARGS,(char *)subcanchor_client_sync_docstring_ },
    
        { "client_tree",(PyCFunction)subcanchor_wrapper_::client_tree_method_,METH_VARARGS,(char *)subcanchor_client_tree_docstring_ },
    
        { "clear_path",(PyCFunction)subcanchor_wrapper_::clear_path_method_,METH_VARARGS,(char *)subcanchor_clear_path_docstring_ },
    
        { "client_clock",(PyCFunction)subcanchor_wrapper_::client_clock_method_,METH_VARARGS,(char *)subcanchor_client_clock_docstring_ },
    
        { "clear_client",(PyCFunction)subcanchor_wrapper_::clear_client_method_,METH_VARARGS,(char *)subcanchor_clear_client_docstring_ },
    
        { "close_client",(PyCFunction)subcanchor_wrapper_::close_client_method_,METH_VARARGS,(char *)subcanchor_close_client_docstring_ },
    
        { "get_path",(PyCFunction)subcanchor_wrapper_::get_path_method_,METH_VARARGS,(char *)subcanchor_get_path_docstring_ },
    
        { "set_path",(PyCFunction)subcanchor_wrapper_::set_path_method_,METH_VARARGS,(char *)subcanchor_set_path_docstring_ },
    
        { "client_data",(PyCFunction)subcanchor_wrapper_::client_data_method_,METH_VARARGS,(char *)subcanchor_client_data_docstring_ },
    
        { "client_opened",(PyCFunction)subcanchor_wrapper_::client_opened_method_,METH_VARARGS,(char *)subcanchor_client_opened_docstring_ },
    
        { "set_client",(PyCFunction)subcanchor_wrapper_::set_client_method_,METH_VARARGS,(char *)subcanchor_set_client_docstring_ },
    
        { "client_closed",(PyCFunction)subcanchor_wrapper_::client_closed_method_,METH_VARARGS,(char *)subcanchor_client_closed_docstring_ },
    
        { "client_child",(PyCFunction)subcanchor_wrapper_::client_child_method_,METH_VARARGS,(char *)subcanchor_client_child_docstring_ },
    
    
        { "py_lock",(PyCFunction)subcanchor_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)subcanchor_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *subcanchor_docstring_ = "";

PyTypeObject subcanchor_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.subcanchor",/*tp_name*/
    sizeof(subcanchor_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)subcanchor_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)subcanchor_docstring_,       /* tp_doc */
    (traverseproc)subcanchor_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)subcanchor_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    subcanchor_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)subcanchor_init_,  /* tp_init */
    0,                         /* tp_alloc */
    subcanchor_new_,             /* tp_new */
};

PyObject *subcanchor_wrapcopy_(subcanchor_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *subcanchor_wrap_(subcanchor_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = client_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    subcanchor_object_ *self = 
        
        PyObject_GC_New(subcanchor_object_, &subcanchor_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        client_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

subcanchor_type_ *subcanchor_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &subcanchor_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected subcanchor object");
        return 0;
    }

    subcanchor_type_ *object = static_cast<subcanchor_type_ *>(((subcanchor_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null subcanchor object in unwrap");
        return 0;
    }

    return object;
}

void subcanchor_insert_(const std::pair< subcanchor_type_ *, PyObject *> &p)
{


    client_dispatcher_->insert(p);




}

void subcanchor_erase_(subcanchor_type_ *p)
{


    client_dispatcher_->erase(p);




}

PyObject * subcanchor_find_(subcanchor_type_ *p)
{


    return client_dispatcher_->find(p);




}

static struct subcanchor_dispatch_ subcanchor_realdispatcher_ =
{
    &subcanchor_typeobject_,
    subcanchor_wrap_,
    subcanchor_wrapcopy_,
    subcanchor_unwrap_,
    subcanchor_insert_,
    subcanchor_erase_,
    subcanchor_find_
};

  

struct canchor_object_ {
    PyObject_HEAD
    canchor_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct canchor_wrapper_: public canchor_type_ 
{

    static void __deleter__(canchor_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    canchor_wrapper_( PyObject *o, PyInterpreterState *i  ): canchor_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~canchor_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    bool set_address_call_(bool owned ,const data_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return canchor_type_::set_address( a0 );
                
            
            
        }

        return
            
                set_address
            
            
                ( a0 );
    }

    static PyObject *set_address_method_(canchor_object_ *self, PyObject *args)
    {
         data_nb_type_ * a0; 

        canchor_wrapper_ *o = (canchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null canchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->set_address_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in canchor::set_address");
        }

        return 0;
    }



    bool set_address_str_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return canchor_type_::set_address_str( a0 );
                
            
            
        }

        return
            
                set_address_str
            
            
                ( a0 );
    }

    static PyObject *set_address_str_method_(canchor_object_ *self, PyObject *args)
    {
         std::string a0; 

        canchor_wrapper_ *o = (canchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null canchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->set_address_str_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in canchor::set_address_str");
        }

        return 0;
    }



    std::string get_address_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return canchor_type_::get_address();
                
            
            
        }

        return
            
                get_address
            
            
                ();
    }

    static PyObject *get_address_method_(canchor_object_ *self, PyObject *args)
    {
        

        canchor_wrapper_ *o = (canchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null canchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_address_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in canchor::get_address");
        }

        return 0;
    }



    void set_client_call_(bool owned ,client_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return canchor_type_::set_client( a0 );
                
            
            
        }

        return
            
                set_client
            
            
                ( a0 );
    }

    static PyObject *set_client_method_(canchor_object_ *self, PyObject *args)
    {
         client_type_ * a0; 

        canchor_wrapper_ *o = (canchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null canchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_client,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_client_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in canchor::set_client");
        }

        return 0;
    }



    void set_slow_client_call_(bool owned ,client_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return canchor_type_::set_slow_client( a0 );
                
            
            
        }

        return
            
                set_slow_client
            
            
                ( a0 );
    }

    static PyObject *set_slow_client_method_(canchor_object_ *self, PyObject *args)
    {
         client_type_ * a0; 

        canchor_wrapper_ *o = (canchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null canchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_client,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_slow_client_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in canchor::set_slow_client");
        }

        return 0;
    }





    static PyObject *_method_lock(canchor_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(canchor_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void canchor_delete_(canchor_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            canchor_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (canchor_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        canchor_wrapper_::__deleter__((canchor_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void canchor_dealloc_(canchor_object_* self)
{
    
    canchor_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * canchor_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    canchor_object_ *self;

    if(!(self = (canchor_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int canchor_init_(canchor_object_ *self, PyObject *args, PyObject *kwds)
{
    canchor_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new canchor_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing canchor");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            canchor_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"canchor invalid arguments");
    return -1;

}


static const char *canchor_set_address_docstring_ = "";

static const char *canchor_set_address_str_docstring_ = "";

static const char *canchor_get_address_docstring_ = "";

static const char *canchor_set_client_docstring_ = "";

static const char *canchor_set_slow_client_docstring_ = "";




static PyMethodDef canchor_methods_[] = {
    
        { "set_address",(PyCFunction)canchor_wrapper_::set_address_method_,METH_VARARGS,(char *)canchor_set_address_docstring_ },
    
        { "set_address_str",(PyCFunction)canchor_wrapper_::set_address_str_method_,METH_VARARGS,(char *)canchor_set_address_str_docstring_ },
    
        { "get_address",(PyCFunction)canchor_wrapper_::get_address_method_,METH_VARARGS,(char *)canchor_get_address_docstring_ },
    
        { "set_client",(PyCFunction)canchor_wrapper_::set_client_method_,METH_VARARGS,(char *)canchor_set_client_docstring_ },
    
        { "set_slow_client",(PyCFunction)canchor_wrapper_::set_slow_client_method_,METH_VARARGS,(char *)canchor_set_slow_client_docstring_ },
    
    
        { "py_lock",(PyCFunction)canchor_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)canchor_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *canchor_docstring_ = "";

PyTypeObject canchor_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.canchor",/*tp_name*/
    sizeof(canchor_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)canchor_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)canchor_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    canchor_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)canchor_init_,  /* tp_init */
    0,                         /* tp_alloc */
    canchor_new_,             /* tp_new */
};

PyObject *canchor_wrapcopy_(canchor_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *canchor_wrap_(canchor_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = canchor_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    canchor_object_ *self = 
        PyObject_NEW(canchor_object_, &canchor_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        canchor_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

canchor_type_ *canchor_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &canchor_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected canchor object");
        return 0;
    }

    canchor_type_ *object = static_cast<canchor_type_ *>(((canchor_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null canchor object in unwrap");
        return 0;
    }

    return object;
}

void canchor_insert_(const std::pair< canchor_type_ *, PyObject *> &p)
{




    canchor_wrapped_.insert(p);


}

void canchor_erase_(canchor_type_ *p)
{




    canchor_wrapped_.erase(p);


}

PyObject * canchor_find_(canchor_type_ *p)
{




    PyObject *ret = 0;
    std::map< canchor_type_ *, PyObject *>::iterator where = canchor_wrapped_.find(p);
    if(where != canchor_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct canchor_dispatch_ canchor_realdispatcher_ =
{
    &canchor_typeobject_,
    canchor_wrap_,
    canchor_wrapcopy_,
    canchor_unwrap_,
    canchor_insert_,
    canchor_erase_,
    canchor_find_
};

  

struct anchor_object_ {
    PyObject_HEAD
    anchor_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct anchor_wrapper_: public anchor_type_ 
{

    static void __deleter__(anchor_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    anchor_wrapper_( PyObject *o, PyInterpreterState *i  ): anchor_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~anchor_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void set_address_str_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return anchor_type_::set_address_str( a0 );
                
            
            
        }

        return
            
                set_address_str
            
            
                ( a0 );
    }

    static PyObject *set_address_str_method_(anchor_object_ *self, PyObject *args)
    {
         std::string a0; 

        anchor_wrapper_ *o = (anchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null anchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_address_str_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in anchor::set_address_str");
        }

        return 0;
    }



    void set_server_call_(bool owned ,server_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return anchor_type_::set_server( a0 );
                
            
            
        }

        return
            
                set_server
            
            
                ( a0 );
    }

    static PyObject *set_server_method_(anchor_object_ *self, PyObject *args)
    {
         server_type_ * a0; 

        anchor_wrapper_ *o = (anchor_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null anchor c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_server,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_server_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in anchor::set_server");
        }

        return 0;
    }





    static PyObject *_method_lock(anchor_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(anchor_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void anchor_delete_(anchor_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            anchor_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (anchor_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        anchor_wrapper_::__deleter__((anchor_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void anchor_dealloc_(anchor_object_* self)
{
    
    anchor_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * anchor_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    anchor_object_ *self;

    if(!(self = (anchor_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int anchor_init_(anchor_object_ *self, PyObject *args, PyObject *kwds)
{
    anchor_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new anchor_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing anchor");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            anchor_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"anchor invalid arguments");
    return -1;

}


static const char *anchor_set_address_str_docstring_ = "";

static const char *anchor_set_server_docstring_ = "";




static PyMethodDef anchor_methods_[] = {
    
        { "set_address_str",(PyCFunction)anchor_wrapper_::set_address_str_method_,METH_VARARGS,(char *)anchor_set_address_str_docstring_ },
    
        { "set_server",(PyCFunction)anchor_wrapper_::set_server_method_,METH_VARARGS,(char *)anchor_set_server_docstring_ },
    
    
        { "py_lock",(PyCFunction)anchor_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)anchor_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *anchor_docstring_ = "";

PyTypeObject anchor_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.anchor",/*tp_name*/
    sizeof(anchor_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)anchor_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)anchor_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    anchor_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)anchor_init_,  /* tp_init */
    0,                         /* tp_alloc */
    anchor_new_,             /* tp_new */
};

PyObject *anchor_wrapcopy_(anchor_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *anchor_wrap_(anchor_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = anchor_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    anchor_object_ *self = 
        PyObject_NEW(anchor_object_, &anchor_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        anchor_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

anchor_type_ *anchor_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &anchor_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected anchor object");
        return 0;
    }

    anchor_type_ *object = static_cast<anchor_type_ *>(((anchor_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null anchor object in unwrap");
        return 0;
    }

    return object;
}

void anchor_insert_(const std::pair< anchor_type_ *, PyObject *> &p)
{




    anchor_wrapped_.insert(p);


}

void anchor_erase_(anchor_type_ *p)
{




    anchor_wrapped_.erase(p);


}

PyObject * anchor_find_(anchor_type_ *p)
{




    PyObject *ret = 0;
    std::map< anchor_type_ *, PyObject *>::iterator where = anchor_wrapped_.find(p);
    if(where != anchor_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct anchor_dispatch_ anchor_realdispatcher_ =
{
    &anchor_typeobject_,
    anchor_wrap_,
    anchor_wrapcopy_,
    anchor_unwrap_,
    anchor_insert_,
    anchor_erase_,
    anchor_find_
};

  

struct f_event_sender_object_ {
    PyObject_HEAD
    f_event_sender_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct f_event_sender_wrapper_: public f_event_sender_type_ 
{

    static void __deleter__(f_event_sender_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    f_event_sender_wrapper_( PyObject *o, PyInterpreterState *i  ): f_event_sender_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    f_event_sender_wrapper_( PyObject *o, PyInterpreterState *i  ,const f_event_sender_type_ & a0  ): f_event_sender_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~f_event_sender_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(f_event_sender_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(f_event_sender_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void f_event_sender_delete_(f_event_sender_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            f_event_sender_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (f_event_sender_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        f_event_sender_wrapper_::__deleter__((f_event_sender_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void f_event_sender_dealloc_(f_event_sender_object_* self)
{
    
    f_event_sender_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * f_event_sender_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    f_event_sender_object_ *self;

    if(!(self = (f_event_sender_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int f_event_sender_init_(f_event_sender_object_ *self, PyObject *args, PyObject *kwds)
{
    f_event_sender_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new f_event_sender_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing f_event_sender");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            f_event_sender_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         f_event_sender_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_f_event_sender_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new f_event_sender_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing f_event_sender");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            f_event_sender_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"f_event_sender invalid arguments");
    return -1;

}





static PyMethodDef f_event_sender_methods_[] = {
    
    
        { "py_lock",(PyCFunction)f_event_sender_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)f_event_sender_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *f_event_sender_docstring_ = "";

PyTypeObject f_event_sender_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.f_event_sender",/*tp_name*/
    sizeof(f_event_sender_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)f_event_sender_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)f_event_sender_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    f_event_sender_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)f_event_sender_init_,  /* tp_init */
    0,                         /* tp_alloc */
    f_event_sender_new_,             /* tp_new */
};

PyObject *f_event_sender_wrapcopy_(f_event_sender_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            f_event_sender_object_ *obj = 
                PyObject_NEW(f_event_sender_object_, f_event_sender_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new f_event_sender_type_(*o);
                }

                try
                {
                    f_event_sender_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying f_event_sender");
                return 0;
            }
        

    
}

PyObject *f_event_sender_wrap_(f_event_sender_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = f_event_sender_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    f_event_sender_object_ *self = 
        PyObject_NEW(f_event_sender_object_, &f_event_sender_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        f_event_sender_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

f_event_sender_type_ *f_event_sender_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &f_event_sender_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected f_event_sender object");
        return 0;
    }

    f_event_sender_type_ *object = static_cast<f_event_sender_type_ *>(((f_event_sender_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null f_event_sender object in unwrap");
        return 0;
    }

    return object;
}

void f_event_sender_insert_(const std::pair< f_event_sender_type_ *, PyObject *> &p)
{




    f_event_sender_wrapped_.insert(p);


}

void f_event_sender_erase_(f_event_sender_type_ *p)
{




    f_event_sender_wrapped_.erase(p);


}

PyObject * f_event_sender_find_(f_event_sender_type_ *p)
{




    PyObject *ret = 0;
    std::map< f_event_sender_type_ *, PyObject *>::iterator where = f_event_sender_wrapped_.find(p);
    if(where != f_event_sender_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct f_event_sender_dispatch_ f_event_sender_realdispatcher_ =
{
    &f_event_sender_typeobject_,
    f_event_sender_wrap_,
    f_event_sender_wrapcopy_,
    f_event_sender_unwrap_,
    f_event_sender_insert_,
    f_event_sender_erase_,
    f_event_sender_find_
};

  

struct f_event_receiver_object_ {
    PyObject_HEAD
    f_event_receiver_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct f_event_receiver_wrapper_: public f_event_receiver_type_ 
{

    static void __deleter__(f_event_receiver_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    f_event_receiver_wrapper_( PyObject *o, PyInterpreterState *i  ): f_event_receiver_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    f_event_receiver_wrapper_( PyObject *o, PyInterpreterState *i  ,const f_event_receiver_type_ & a0  ): f_event_receiver_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~f_event_receiver_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(f_event_receiver_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(f_event_receiver_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void f_event_receiver_delete_(f_event_receiver_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            f_event_receiver_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (f_event_receiver_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        f_event_receiver_wrapper_::__deleter__((f_event_receiver_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void f_event_receiver_dealloc_(f_event_receiver_object_* self)
{
    
    f_event_receiver_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * f_event_receiver_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    f_event_receiver_object_ *self;

    if(!(self = (f_event_receiver_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int f_event_receiver_init_(f_event_receiver_object_ *self, PyObject *args, PyObject *kwds)
{
    f_event_receiver_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new f_event_receiver_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing f_event_receiver");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            f_event_receiver_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         f_event_receiver_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_f_event_receiver_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new f_event_receiver_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing f_event_receiver");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            f_event_receiver_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"f_event_receiver invalid arguments");
    return -1;

}





static PyMethodDef f_event_receiver_methods_[] = {
    
    
        { "py_lock",(PyCFunction)f_event_receiver_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)f_event_receiver_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *f_event_receiver_docstring_ = "";

PyTypeObject f_event_receiver_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.f_event_receiver",/*tp_name*/
    sizeof(f_event_receiver_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)f_event_receiver_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)f_event_receiver_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    f_event_receiver_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)f_event_receiver_init_,  /* tp_init */
    0,                         /* tp_alloc */
    f_event_receiver_new_,             /* tp_new */
};

PyObject *f_event_receiver_wrapcopy_(f_event_receiver_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            f_event_receiver_object_ *obj = 
                PyObject_NEW(f_event_receiver_object_, f_event_receiver_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new f_event_receiver_type_(*o);
                }

                try
                {
                    f_event_receiver_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying f_event_receiver");
                return 0;
            }
        

    
}

PyObject *f_event_receiver_wrap_(f_event_receiver_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = f_event_receiver_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    f_event_receiver_object_ *self = 
        PyObject_NEW(f_event_receiver_object_, &f_event_receiver_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        f_event_receiver_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

f_event_receiver_type_ *f_event_receiver_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &f_event_receiver_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected f_event_receiver object");
        return 0;
    }

    f_event_receiver_type_ *object = static_cast<f_event_receiver_type_ *>(((f_event_receiver_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null f_event_receiver object in unwrap");
        return 0;
    }

    return object;
}

void f_event_receiver_insert_(const std::pair< f_event_receiver_type_ *, PyObject *> &p)
{




    f_event_receiver_wrapped_.insert(p);


}

void f_event_receiver_erase_(f_event_receiver_type_ *p)
{




    f_event_receiver_wrapped_.erase(p);


}

PyObject * f_event_receiver_find_(f_event_receiver_type_ *p)
{




    PyObject *ret = 0;
    std::map< f_event_receiver_type_ *, PyObject *>::iterator where = f_event_receiver_wrapped_.find(p);
    if(where != f_event_receiver_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct f_event_receiver_dispatch_ f_event_receiver_realdispatcher_ =
{
    &f_event_receiver_typeobject_,
    f_event_receiver_wrap_,
    f_event_receiver_wrapcopy_,
    f_event_receiver_unwrap_,
    f_event_receiver_insert_,
    f_event_receiver_erase_,
    f_event_receiver_find_
};

  

struct dataqueue_object_ {
    PyObject_HEAD
    dataqueue_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct dataqueue_wrapper_: public dataqueue_type_ 
{

    static void __deleter__(dataqueue_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    dataqueue_wrapper_( PyObject *o, PyInterpreterState *i  ,const dataqueue_type_ & a0  ): dataqueue_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~dataqueue_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void write_slow_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return dataqueue_type_::write_slow( a0 );
                
            
            
        }

        return
            
                write_slow
            
            
                ( a0 );
    }

    static PyObject *write_slow_method_(dataqueue_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        dataqueue_wrapper_ *o = (dataqueue_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null dataqueue c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->write_slow_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in dataqueue::write_slow");
        }

        return 0;
    }



    void trigger_slow_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return dataqueue_type_::trigger_slow();
                
            
            
        }

        return
            
                trigger_slow
            
            
                ();
    }

    static PyObject *trigger_slow_method_(dataqueue_object_ *self, PyObject *args)
    {
        

        dataqueue_wrapper_ *o = (dataqueue_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null dataqueue c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->trigger_slow_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in dataqueue::trigger_slow");
        }

        return 0;
    }



    change_nb_type_ sender_fast_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return dataqueue_type_::sender_fast();
                
            
            
        }

        return
            
                sender_fast
            
            
                ();
    }

    static PyObject *sender_fast_method_(dataqueue_object_ *self, PyObject *args)
    {
        

        dataqueue_wrapper_ *o = (dataqueue_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null dataqueue c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->sender_fast_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in dataqueue::sender_fast");
        }

        return 0;
    }



    change_type_ sender_slow_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return dataqueue_type_::sender_slow();
                
            
            
        }

        return
            
                sender_slow
            
            
                ();
    }

    static PyObject *sender_slow_method_(dataqueue_object_ *self, PyObject *args)
    {
        

        dataqueue_wrapper_ *o = (dataqueue_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null dataqueue c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_type_ r = o->sender_slow_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in dataqueue::sender_slow");
        }

        return 0;
    }





    static PyObject *_method_lock(dataqueue_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(dataqueue_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void dataqueue_delete_(dataqueue_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            dataqueue_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (dataqueue_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        dataqueue_wrapper_::__deleter__((dataqueue_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void dataqueue_dealloc_(dataqueue_object_* self)
{
    
    dataqueue_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * dataqueue_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    dataqueue_object_ *self;

    if(!(self = (dataqueue_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int dataqueue_init_(dataqueue_object_ *self, PyObject *args, PyObject *kwds)
{
    dataqueue_delete_(self);

    

    {
         dataqueue_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_dataqueue_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new dataqueue_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing dataqueue");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            dataqueue_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"dataqueue invalid arguments");
    return -1;

}


static const char *dataqueue_write_slow_docstring_ = "";

static const char *dataqueue_trigger_slow_docstring_ = "";

static const char *dataqueue_sender_fast_docstring_ = "";

static const char *dataqueue_sender_slow_docstring_ = "";




static PyMethodDef dataqueue_methods_[] = {
    
        { "write_slow",(PyCFunction)dataqueue_wrapper_::write_slow_method_,METH_VARARGS,(char *)dataqueue_write_slow_docstring_ },
    
        { "trigger_slow",(PyCFunction)dataqueue_wrapper_::trigger_slow_method_,METH_VARARGS,(char *)dataqueue_trigger_slow_docstring_ },
    
        { "sender_fast",(PyCFunction)dataqueue_wrapper_::sender_fast_method_,METH_VARARGS,(char *)dataqueue_sender_fast_docstring_ },
    
        { "sender_slow",(PyCFunction)dataqueue_wrapper_::sender_slow_method_,METH_VARARGS,(char *)dataqueue_sender_slow_docstring_ },
    
    
        { "py_lock",(PyCFunction)dataqueue_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)dataqueue_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *dataqueue_docstring_ = "";

PyTypeObject dataqueue_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.dataqueue",/*tp_name*/
    sizeof(dataqueue_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)dataqueue_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)dataqueue_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    dataqueue_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)dataqueue_init_,  /* tp_init */
    0,                         /* tp_alloc */
    dataqueue_new_,             /* tp_new */
};

PyObject *dataqueue_wrapcopy_(dataqueue_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            dataqueue_object_ *obj = 
                PyObject_NEW(dataqueue_object_, dataqueue_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new dataqueue_type_(*o);
                }

                try
                {
                    dataqueue_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying dataqueue");
                return 0;
            }
        

    
}

PyObject *dataqueue_wrap_(dataqueue_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = dataqueue_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    dataqueue_object_ *self = 
        PyObject_NEW(dataqueue_object_, &dataqueue_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        dataqueue_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

dataqueue_type_ *dataqueue_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &dataqueue_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected dataqueue object");
        return 0;
    }

    dataqueue_type_ *object = static_cast<dataqueue_type_ *>(((dataqueue_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null dataqueue object in unwrap");
        return 0;
    }

    return object;
}

void dataqueue_insert_(const std::pair< dataqueue_type_ *, PyObject *> &p)
{




    dataqueue_wrapped_.insert(p);


}

void dataqueue_erase_(dataqueue_type_ *p)
{




    dataqueue_wrapped_.erase(p);


}

PyObject * dataqueue_find_(dataqueue_type_ *p)
{




    PyObject *ret = 0;
    std::map< dataqueue_type_ *, PyObject *>::iterator where = dataqueue_wrapped_.find(p);
    if(where != dataqueue_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct dataqueue_dispatch_ dataqueue_realdispatcher_ =
{
    &dataqueue_typeobject_,
    dataqueue_wrap_,
    dataqueue_wrapcopy_,
    dataqueue_unwrap_,
    dataqueue_insert_,
    dataqueue_erase_,
    dataqueue_find_
};

  

struct fastdata_object_ {
    PyObject_HEAD
    fastdata_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct fastdata_wrapper_: public fastdata_type_ 
{

    static void __deleter__(fastdata_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
    }
*/



    fastdata_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned short a0  ): fastdata_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~fastdata_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void close_fastdata(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"close_fastdata");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("close_fastdata before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("close_fastdata after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void fastdata_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"fastdata_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("fastdata_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("fastdata_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }













    static int gc_traverse_method_(fastdata_object_ *self, visitproc visit, void *arg)
    {
        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(fastdata_object_ *self)
    {
        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void set_upstream_call_(bool owned ,fastdata_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::set_upstream( a0 );
                
            
            
        }

        return
            
                set_upstream
            
            
                ( a0 );
    }

    static PyObject *set_upstream_method_(fastdata_object_ *self, PyObject *args)
    {
         fastdata_type_ * a0; 

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_fastdata,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_upstream_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::set_upstream");
        }

        return 0;
    }



    void clear_upstream_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::clear_upstream();
                
            
            
        }

        return
            
                clear_upstream
            
            
                ();
    }

    static PyObject *clear_upstream_method_(fastdata_object_ *self, PyObject *args)
    {
        

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_upstream_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::clear_upstream");
        }

        return 0;
    }



    void enable_call_(bool owned ,bool a0 ,bool a1 ,bool a2 )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::enable( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                enable
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *enable_method_(fastdata_object_ *self, PyObject *args)
    {
         bool a0;  bool a1;  bool a2; 

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_b,&a0  ,fpcvt_b,&a1  ,fpcvt_b,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enable_call_ (self->owned ,a0,a1,a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::enable");
        }

        return 0;
    }



    data_nb_type_ current_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::current( a0 );
                
            
            
        }

        return
            
                current
            
            
                ( a0 );
    }

    static PyObject *current_method_(fastdata_object_ *self, PyObject *args)
    {
         bool a0; 

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_nb_type_ r = o->current_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::current");
        }

        return 0;
    }



    bool open_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::open();
                
            
            
        }

        return
            
                open
            
            
                ();
    }

    static PyObject *open_method_(fastdata_object_ *self, PyObject *args)
    {
        

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::open");
        }

        return 0;
    }



    f_event_sender_type_ id_sender_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::id_sender();
                
            
            
        }

        return
            
                id_sender
            
            
                ();
    }

    static PyObject *id_sender_method_(fastdata_object_ *self, PyObject *args)
    {
        

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    f_event_sender_type_ r = o->id_sender_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f_event_sender_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::id_sender");
        }

        return 0;
    }



    void send_slow_call_(bool owned ,const data_type_ & a0 ,const dataqueue_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::send_slow( a0  ,a1 );
                
            
            
        }

        return
            
                send_slow
            
            
                ( a0  ,a1 );
    }

    static PyObject *send_slow_method_(fastdata_object_ *self, PyObject *args)
    {
         data_type_ * a0;  dataqueue_type_ * a1; 

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_data_copy,&a0  ,fpcvt_dataqueue_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->send_slow_call_ (self->owned ,*a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::send_slow");
        }

        return 0;
    }



    void set_change_handler_call_(bool owned ,const change_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::set_change_handler( a0 );
                
            
            
        }

        return
            
                set_change_handler
            
            
                ( a0 );
    }

    static PyObject *set_change_handler_method_(fastdata_object_ *self, PyObject *args)
    {
         change_nb_type_ * a0; 

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_change_handler_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::set_change_handler");
        }

        return 0;
    }



    void clear_change_handler_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::clear_change_handler();
                
            
            
        }

        return
            
                clear_change_handler
            
            
                ();
    }

    static PyObject *clear_change_handler_method_(fastdata_object_ *self, PyObject *args)
    {
        

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_change_handler_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::clear_change_handler");
        }

        return 0;
    }



    void set_id_change_handler_call_(bool owned ,const f_event_receiver_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::set_id_change_handler( a0 );
                
            
            
        }

        return
            
                set_id_change_handler
            
            
                ( a0 );
    }

    static PyObject *set_id_change_handler_method_(fastdata_object_ *self, PyObject *args)
    {
         f_event_receiver_type_ * a0; 

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f_event_receiver_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_id_change_handler_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::set_id_change_handler");
        }

        return 0;
    }



    void clear_id_change_handler_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::clear_id_change_handler();
                
            
            
        }

        return
            
                clear_id_change_handler
            
            
                ();
    }

    static PyObject *clear_id_change_handler_method_(fastdata_object_ *self, PyObject *args)
    {
        

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_id_change_handler_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::clear_id_change_handler");
        }

        return 0;
    }



    void close_fastdata_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::close_fastdata();
                
            
            
        }

        return
            
                close_fastdata
            
            
                ();
    }

    static PyObject *close_fastdata_method_(fastdata_object_ *self, PyObject *args)
    {
        

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_fastdata_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::close_fastdata");
        }

        return 0;
    }



    void fastdata_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fastdata_type_::fastdata_closed();
                
            
            
        }

        return
            
                fastdata_closed
            
            
                ();
    }

    static PyObject *fastdata_closed_method_(fastdata_object_ *self, PyObject *args)
    {
        

        fastdata_wrapper_ *o = (fastdata_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->fastdata_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata::fastdata_closed");
        }

        return 0;
    }





    static PyObject *_method_lock(fastdata_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(fastdata_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void fastdata_delete_(fastdata_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            fastdata_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (fastdata_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        fastdata_wrapper_::__deleter__((fastdata_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void fastdata_dealloc_(fastdata_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    fastdata_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * fastdata_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    fastdata_object_ *self;

    if(!(self = (fastdata_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int fastdata_init_(fastdata_object_ *self, PyObject *args, PyObject *kwds)
{
    fastdata_delete_(self);

    

    {
         unsigned short a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_us,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new fastdata_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing fastdata");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            fastdata_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"fastdata invalid arguments");
    return -1;

}


static const char *fastdata_set_upstream_docstring_ = "";

static const char *fastdata_clear_upstream_docstring_ = "";

static const char *fastdata_enable_docstring_ = "";

static const char *fastdata_current_docstring_ = "";

static const char *fastdata_open_docstring_ = "";

static const char *fastdata_id_sender_docstring_ = "";

static const char *fastdata_send_slow_docstring_ = "";

static const char *fastdata_set_change_handler_docstring_ = "";

static const char *fastdata_clear_change_handler_docstring_ = "";

static const char *fastdata_set_id_change_handler_docstring_ = "";

static const char *fastdata_clear_id_change_handler_docstring_ = "";

static const char *fastdata_close_fastdata_docstring_ = "";

static const char *fastdata_fastdata_closed_docstring_ = "";




static PyMethodDef fastdata_methods_[] = {
    
        { "set_upstream",(PyCFunction)fastdata_wrapper_::set_upstream_method_,METH_VARARGS,(char *)fastdata_set_upstream_docstring_ },
    
        { "clear_upstream",(PyCFunction)fastdata_wrapper_::clear_upstream_method_,METH_VARARGS,(char *)fastdata_clear_upstream_docstring_ },
    
        { "enable",(PyCFunction)fastdata_wrapper_::enable_method_,METH_VARARGS,(char *)fastdata_enable_docstring_ },
    
        { "current",(PyCFunction)fastdata_wrapper_::current_method_,METH_VARARGS,(char *)fastdata_current_docstring_ },
    
        { "open",(PyCFunction)fastdata_wrapper_::open_method_,METH_VARARGS,(char *)fastdata_open_docstring_ },
    
        { "id_sender",(PyCFunction)fastdata_wrapper_::id_sender_method_,METH_VARARGS,(char *)fastdata_id_sender_docstring_ },
    
        { "send_slow",(PyCFunction)fastdata_wrapper_::send_slow_method_,METH_VARARGS,(char *)fastdata_send_slow_docstring_ },
    
        { "set_change_handler",(PyCFunction)fastdata_wrapper_::set_change_handler_method_,METH_VARARGS,(char *)fastdata_set_change_handler_docstring_ },
    
        { "clear_change_handler",(PyCFunction)fastdata_wrapper_::clear_change_handler_method_,METH_VARARGS,(char *)fastdata_clear_change_handler_docstring_ },
    
        { "set_id_change_handler",(PyCFunction)fastdata_wrapper_::set_id_change_handler_method_,METH_VARARGS,(char *)fastdata_set_id_change_handler_docstring_ },
    
        { "clear_id_change_handler",(PyCFunction)fastdata_wrapper_::clear_id_change_handler_method_,METH_VARARGS,(char *)fastdata_clear_id_change_handler_docstring_ },
    
        { "close_fastdata",(PyCFunction)fastdata_wrapper_::close_fastdata_method_,METH_VARARGS,(char *)fastdata_close_fastdata_docstring_ },
    
        { "fastdata_closed",(PyCFunction)fastdata_wrapper_::fastdata_closed_method_,METH_VARARGS,(char *)fastdata_fastdata_closed_docstring_ },
    
    
        { "py_lock",(PyCFunction)fastdata_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)fastdata_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *fastdata_docstring_ = "";

PyTypeObject fastdata_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.fastdata",/*tp_name*/
    sizeof(fastdata_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)fastdata_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)fastdata_docstring_,       /* tp_doc */
    (traverseproc)fastdata_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)fastdata_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    fastdata_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)fastdata_init_,  /* tp_init */
    0,                         /* tp_alloc */
    fastdata_new_,             /* tp_new */
};

PyObject *fastdata_wrapcopy_(fastdata_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *fastdata_wrap_(fastdata_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = fastdata_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    fastdata_object_ *self = 
        
        PyObject_GC_New(fastdata_object_, &fastdata_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        fastdata_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

fastdata_type_ *fastdata_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &fastdata_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected fastdata object");
        return 0;
    }

    fastdata_type_ *object = static_cast<fastdata_type_ *>(((fastdata_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null fastdata object in unwrap");
        return 0;
    }

    return object;
}

void fastdata_insert_(const std::pair< fastdata_type_ *, PyObject *> &p)
{




    fastdata_wrapped_.insert(p);


}

void fastdata_erase_(fastdata_type_ *p)
{




    fastdata_wrapped_.erase(p);


}

PyObject * fastdata_find_(fastdata_type_ *p)
{




    PyObject *ret = 0;
    std::map< fastdata_type_ *, PyObject *>::iterator where = fastdata_wrapped_.find(p);
    if(where != fastdata_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct fastdata_dispatch_ fastdata_realdispatcher_ =
{
    &fastdata_typeobject_,
    fastdata_wrap_,
    fastdata_wrapcopy_,
    fastdata_unwrap_,
    fastdata_insert_,
    fastdata_erase_,
    fastdata_find_
};

  

struct fastdata_receiver_object_ {
    PyObject_HEAD
    fastdata_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct fastdata_receiver_wrapper_: public fastdata_receiver_type_ 
{

    static void __deleter__(fastdata_receiver_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
    }
*/



    fastdata_receiver_wrapper_( PyObject *o, PyInterpreterState *i  ,const change_nb_type_ & a0  ): fastdata_receiver_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~fastdata_receiver_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void fastdata_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"fastdata_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("fastdata_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("fastdata_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void close_fastdata(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"close_fastdata");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("close_fastdata before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("close_fastdata after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }













    static int gc_traverse_method_(fastdata_receiver_object_ *self, visitproc visit, void *arg)
    {
        fastdata_receiver_wrapper_ *o = (fastdata_receiver_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(fastdata_receiver_object_ *self)
    {
        fastdata_receiver_wrapper_ *o = (fastdata_receiver_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void fastdata_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fastdata_receiver_type_::fastdata_closed();
                
            
            
        }

        return
            
                fastdata_closed
            
            
                ();
    }

    static PyObject *fastdata_closed_method_(fastdata_receiver_object_ *self, PyObject *args)
    {
        

        fastdata_receiver_wrapper_ *o = (fastdata_receiver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata_receiver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->fastdata_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata_receiver::fastdata_closed");
        }

        return 0;
    }



    void close_fastdata_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fastdata_receiver_type_::close_fastdata();
                
            
            
        }

        return
            
                close_fastdata
            
            
                ();
    }

    static PyObject *close_fastdata_method_(fastdata_receiver_object_ *self, PyObject *args)
    {
        

        fastdata_receiver_wrapper_ *o = (fastdata_receiver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fastdata_receiver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_fastdata_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastdata_receiver::close_fastdata");
        }

        return 0;
    }





    static PyObject *_method_lock(fastdata_receiver_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(fastdata_receiver_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void fastdata_receiver_delete_(fastdata_receiver_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            fastdata_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (fastdata_receiver_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        fastdata_receiver_wrapper_::__deleter__((fastdata_receiver_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void fastdata_receiver_dealloc_(fastdata_receiver_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    fastdata_receiver_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * fastdata_receiver_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    fastdata_receiver_object_ *self;

    if(!(self = (fastdata_receiver_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int fastdata_receiver_init_(fastdata_receiver_object_ *self, PyObject *args, PyObject *kwds)
{
    fastdata_receiver_delete_(self);

    

    {
         change_nb_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_change_nb_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new fastdata_receiver_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing fastdata_receiver");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            fastdata_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"fastdata_receiver invalid arguments");
    return -1;

}


static const char *fastdata_receiver_fastdata_closed_docstring_ = "";

static const char *fastdata_receiver_close_fastdata_docstring_ = "";




static PyMethodDef fastdata_receiver_methods_[] = {
    
        { "fastdata_closed",(PyCFunction)fastdata_receiver_wrapper_::fastdata_closed_method_,METH_VARARGS,(char *)fastdata_receiver_fastdata_closed_docstring_ },
    
        { "close_fastdata",(PyCFunction)fastdata_receiver_wrapper_::close_fastdata_method_,METH_VARARGS,(char *)fastdata_receiver_close_fastdata_docstring_ },
    
    
        { "py_lock",(PyCFunction)fastdata_receiver_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)fastdata_receiver_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *fastdata_receiver_docstring_ = "";

PyTypeObject fastdata_receiver_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.fastdata_receiver",/*tp_name*/
    sizeof(fastdata_receiver_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)fastdata_receiver_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)fastdata_receiver_docstring_,       /* tp_doc */
    (traverseproc)fastdata_receiver_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)fastdata_receiver_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    fastdata_receiver_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)fastdata_receiver_init_,  /* tp_init */
    0,                         /* tp_alloc */
    fastdata_receiver_new_,             /* tp_new */
};

PyObject *fastdata_receiver_wrapcopy_(fastdata_receiver_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *fastdata_receiver_wrap_(fastdata_receiver_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = fastdata_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    fastdata_receiver_object_ *self = 
        
        PyObject_GC_New(fastdata_receiver_object_, &fastdata_receiver_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        fastdata_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

fastdata_receiver_type_ *fastdata_receiver_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &fastdata_receiver_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected fastdata_receiver object");
        return 0;
    }

    fastdata_receiver_type_ *object = static_cast<fastdata_receiver_type_ *>(((fastdata_receiver_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null fastdata_receiver object in unwrap");
        return 0;
    }

    return object;
}

void fastdata_receiver_insert_(const std::pair< fastdata_receiver_type_ *, PyObject *> &p)
{


    fastdata_dispatcher_->insert(p);




}

void fastdata_receiver_erase_(fastdata_receiver_type_ *p)
{


    fastdata_dispatcher_->erase(p);




}

PyObject * fastdata_receiver_find_(fastdata_receiver_type_ *p)
{


    return fastdata_dispatcher_->find(p);




}

static struct fastdata_receiver_dispatch_ fastdata_receiver_realdispatcher_ =
{
    &fastdata_receiver_typeobject_,
    fastdata_receiver_wrap_,
    fastdata_receiver_wrapcopy_,
    fastdata_receiver_unwrap_,
    fastdata_receiver_insert_,
    fastdata_receiver_erase_,
    fastdata_receiver_find_
};

  

struct term_object_ {
    PyObject_HEAD
    term_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct term_wrapper_: public term_type_ 
{

    static void __deleter__(term_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    term_wrapper_( PyObject *o, PyInterpreterState *i  ): term_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    term_wrapper_( PyObject *o, PyInterpreterState *i  ,const data_type_ & a0  ): term_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    term_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned int a0  ): term_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    term_wrapper_( PyObject *o, PyInterpreterState *i  ,const char * a0  ,unsigned int a1  ): term_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    term_wrapper_( PyObject *o, PyInterpreterState *i  ,const term_type_ & a0  ): term_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~term_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    unsigned int type_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return term_type_::type();
                
            
            
        }

        return
            
                type
            
            
                ();
    }

    static PyObject *type_method_(term_object_ *self, PyObject *args)
    {
        

        term_wrapper_ *o = (term_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null term c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->type_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in term::type");
        }

        return 0;
    }



    unsigned int arity_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return term_type_::arity();
                
            
            
        }

        return
            
                arity
            
            
                ();
    }

    static PyObject *arity_method_(term_object_ *self, PyObject *args)
    {
        

        term_wrapper_ *o = (term_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null term c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->arity_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in term::arity");
        }

        return 0;
    }



    const char * pred_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return term_type_::pred();
                
            
            
        }

        return
            
                pred
            
            
                ();
    }

    static PyObject *pred_method_(term_object_ *self, PyObject *args)
    {
        

        term_wrapper_ *o = (term_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null term c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    const char * r = o->pred_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_str((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in term::pred");
        }

        return 0;
    }



    data_type_ value_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return term_type_::value();
                
            
            
        }

        return
            
                value
            
            
                ();
    }

    static PyObject *value_method_(term_object_ *self, PyObject *args)
    {
        

        term_wrapper_ *o = (term_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null term c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->value_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in term::value");
        }

        return 0;
    }



    term_type_ arg_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return term_type_::arg( a0 );
                
            
            
        }

        return
            
                arg
            
            
                ( a0 );
    }

    static PyObject *arg_method_(term_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        term_wrapper_ *o = (term_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null term c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    term_type_ r = o->arg_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_term_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in term::arg");
        }

        return 0;
    }



    void set_arg_call_(bool owned ,unsigned int a0 ,const term_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return term_type_::set_arg( a0  ,a1 );
                
            
            
        }

        return
            
                set_arg
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_arg_method_(term_object_ *self, PyObject *args)
    {
         unsigned int a0;  term_type_ * a1; 

        term_wrapper_ *o = (term_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null term c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_term_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_arg_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in term::set_arg");
        }

        return 0;
    }



    void append_arg_call_(bool owned ,const term_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return term_type_::append_arg( a0 );
                
            
            
        }

        return
            
                append_arg
            
            
                ( a0 );
    }

    static PyObject *append_arg_method_(term_object_ *self, PyObject *args)
    {
         term_type_ * a0; 

        term_wrapper_ *o = (term_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null term c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_term_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->append_arg_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in term::append_arg");
        }

        return 0;
    }



    void add_arg_call_(bool owned ,int a0 ,const term_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return term_type_::add_arg( a0  ,a1 );
                
            
            
        }

        return
            
                add_arg
            
            
                ( a0  ,a1 );
    }

    static PyObject *add_arg_method_(term_object_ *self, PyObject *args)
    {
         int a0;  term_type_ * a1; 

        term_wrapper_ *o = (term_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null term c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_i,&a0  ,fpcvt_term_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->add_arg_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in term::add_arg");
        }

        return 0;
    }



    std::string render_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return term_type_::render();
                
            
            
        }

        return
            
                render
            
            
                ();
    }

    static PyObject *render_method_(term_object_ *self, PyObject *args)
    {
        

        term_wrapper_ *o = (term_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null term c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->render_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in term::render");
        }

        return 0;
    }





    static PyObject *_method_lock(term_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(term_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void term_delete_(term_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            term_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (term_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        term_wrapper_::__deleter__((term_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void term_dealloc_(term_object_* self)
{
    
    term_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * term_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    term_object_ *self;

    if(!(self = (term_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int term_init_(term_object_ *self, PyObject *args, PyObject *kwds)
{
    term_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new term_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing term");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            term_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         data_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_data_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new term_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing term");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            term_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         unsigned int a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new term_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing term");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            term_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         const char * a0;  unsigned int a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_ui,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new term_wrapper_((PyObject *)self,interp ,a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing term");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            term_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         term_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_term_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new term_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing term");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            term_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"term invalid arguments");
    return -1;

}


static const char *term_type_docstring_ = "";

static const char *term_arity_docstring_ = "";

static const char *term_pred_docstring_ = "";

static const char *term_value_docstring_ = "";

static const char *term_arg_docstring_ = "";

static const char *term_set_arg_docstring_ = "";

static const char *term_append_arg_docstring_ = "";

static const char *term_add_arg_docstring_ = "";

static const char *term_render_docstring_ = "";




static PyMethodDef term_methods_[] = {
    
        { "type",(PyCFunction)term_wrapper_::type_method_,METH_VARARGS,(char *)term_type_docstring_ },
    
        { "arity",(PyCFunction)term_wrapper_::arity_method_,METH_VARARGS,(char *)term_arity_docstring_ },
    
        { "pred",(PyCFunction)term_wrapper_::pred_method_,METH_VARARGS,(char *)term_pred_docstring_ },
    
        { "value",(PyCFunction)term_wrapper_::value_method_,METH_VARARGS,(char *)term_value_docstring_ },
    
        { "arg",(PyCFunction)term_wrapper_::arg_method_,METH_VARARGS,(char *)term_arg_docstring_ },
    
        { "set_arg",(PyCFunction)term_wrapper_::set_arg_method_,METH_VARARGS,(char *)term_set_arg_docstring_ },
    
        { "append_arg",(PyCFunction)term_wrapper_::append_arg_method_,METH_VARARGS,(char *)term_append_arg_docstring_ },
    
        { "add_arg",(PyCFunction)term_wrapper_::add_arg_method_,METH_VARARGS,(char *)term_add_arg_docstring_ },
    
        { "render",(PyCFunction)term_wrapper_::render_method_,METH_VARARGS,(char *)term_render_docstring_ },
    
    
        { "py_lock",(PyCFunction)term_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)term_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *term_docstring_ = "";

PyTypeObject term_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.term",/*tp_name*/
    sizeof(term_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)term_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)term_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    term_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)term_init_,  /* tp_init */
    0,                         /* tp_alloc */
    term_new_,             /* tp_new */
};

PyObject *term_wrapcopy_(term_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            term_object_ *obj = 
                PyObject_NEW(term_object_, term_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new term_type_(*o);
                }

                try
                {
                    term_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying term");
                return 0;
            }
        

    
}

PyObject *term_wrap_(term_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = term_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    term_object_ *self = 
        PyObject_NEW(term_object_, &term_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        term_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

term_type_ *term_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &term_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected term object");
        return 0;
    }

    term_type_ *object = static_cast<term_type_ *>(((term_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null term object in unwrap");
        return 0;
    }

    return object;
}

void term_insert_(const std::pair< term_type_ *, PyObject *> &p)
{




    term_wrapped_.insert(p);


}

void term_erase_(term_type_ *p)
{




    term_wrapped_.erase(p);


}

PyObject * term_find_(term_type_ *p)
{




    PyObject *ret = 0;
    std::map< term_type_ *, PyObject *>::iterator where = term_wrapped_.find(p);
    if(where != term_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct term_dispatch_ term_realdispatcher_ =
{
    &term_typeobject_,
    term_wrap_,
    term_wrapcopy_,
    term_unwrap_,
    term_insert_,
    term_erase_,
    term_find_
};

  

struct data_base_object_ {
    PyObject_HEAD
    data_base_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct data_base_wrapper_: public data_base_type_ 
{

    static void __deleter__(data_base_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    void __safe_dtors()
    {
        
    }

    virtual ~data_base_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }




    // special method __str__ returning std::string doc 

    std::string special_str_call_()
    {
        return
             repr 
            
                ();
    }

    static PyObject *special_str_method_(data_base_object_ *self)
    {
        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        try
        {
            PyObject *ro;
            {   unlock_p2c unlock ;
                {   
                    std::string r = o->special_str_call_();
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;
        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::__str__");
        }

        return 0;
    }




    // special method __cmp__ returning int doc ""

    int special_cmp_call_(const data_base_type_ & a)
    {
        int r =
             compare 
            
                (a);

        if(r<0) return -1;
        if(r>0) return 1;
        return 0;
    }

    static int special_cmp_method_(data_base_object_ *self, data_base_object_ *other)
    {
        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);
        PyObject *oo = data_base_dispatcher_->wrap(other->object,false);
        data_base_type_ * ooa;

        if(!o || !oo)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object in __cmp__, probably forgot to __init__");
            Py_DECREF(oo);
            return -1;
        }

        if(!fpcvt_data_base_copy(oo,&ooa))
        {
            Py_DECREF(oo);
            return -1;
        }

        try
        {
            {   unlock_p2c unlock ;
                int r = o->special_cmp_call_(*ooa);
                {   relock_p2c relock(unlock);
                    Py_DECREF(oo);
                    return r;
                }
            }
        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
            Py_DECREF(oo);
            return -1;
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::__cmp__");
            Py_DECREF(oo);
            return -1;
        }
    }




    // special method __hash__ returning long doc 

    long special_hash_call_()
    {
        return
             hash 
            
                ();
    }

    static long special_hash_method_(data_base_object_ *self)
    {
        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        try
        {
            {   unlock_p2c unlock  ;
                long r = o->special_hash_call_();
                return r;
            }
        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::__hash__");
        }

        return 0;
    }




    // special method __repr__ returning std::string doc 

    std::string special_repr_call_()
    {
        return
             repr 
            
                ();
    }

    static PyObject *special_repr_method_(data_base_object_ *self)
    {
        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        try
        {
            PyObject *ro;
            {   unlock_p2c unlock  ;
                {
                    std::string r = o->special_repr_call_();
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            } 

            return ro;
        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::__repr__");
        }

        return 0;
    }









    unsigned long long time_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::time();
                
            
            
        }

        return
            
                time
            
            
                ();
    }

    static PyObject *time_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long long r = o->time_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ull((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::time");
        }

        return 0;
    }



    unsigned int type_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::type();
                
            
            
        }

        return
            
                type
            
            
                ();
    }

    static PyObject *type_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->type_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::type");
        }

        return 0;
    }



    bool is_array_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::is_array();
                
            
            
        }

        return
            
                is_array
            
            
                ();
    }

    static PyObject *is_array_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_array_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::is_array");
        }

        return 0;
    }



    bool is_string_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::is_string();
                
            
            
        }

        return
            
                is_string
            
            
                ();
    }

    static PyObject *is_string_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_string_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::is_string");
        }

        return 0;
    }



    bool is_float_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::is_float();
                
            
            
        }

        return
            
                is_float
            
            
                ();
    }

    static PyObject *is_float_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_float_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::is_float");
        }

        return 0;
    }



    bool is_long_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::is_long();
                
            
            
        }

        return
            
                is_long
            
            
                ();
    }

    static PyObject *is_long_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_long_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::is_long");
        }

        return 0;
    }



    bool is_null_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::is_null();
                
            
            
        }

        return
            
                is_null
            
            
                ();
    }

    static PyObject *is_null_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_null_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::is_null");
        }

        return 0;
    }



    bool is_bool_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::is_bool();
                
            
            
        }

        return
            
                is_bool
            
            
                ();
    }

    static PyObject *is_bool_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_bool_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::is_bool");
        }

        return 0;
    }



    bool is_path_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::is_path();
                
            
            
        }

        return
            
                is_path
            
            
                ();
    }

    static PyObject *is_path_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_path_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::is_path");
        }

        return 0;
    }



    bool is_dict_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::is_dict();
                
            
            
        }

        return
            
                is_dict
            
            
                ();
    }

    static PyObject *is_dict_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_dict_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::is_dict");
        }

        return 0;
    }



    bool is_blob_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::is_blob();
                
            
            
        }

        return
            
                is_blob
            
            
                ();
    }

    static PyObject *is_blob_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_blob_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::is_blob");
        }

        return 0;
    }



    std::string as_stdstr_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_stdstr();
                
            
            
        }

        return
            
                as_stdstr
            
            
                ();
    }

    static PyObject *as_stdstr_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->as_stdstr_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_stdstr");
        }

        return 0;
    }



    std::string as_blob2_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_blob2();
                
            
            
        }

        return
            
                as_blob2
            
            
                ();
    }

    static PyObject *as_blob2_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->as_blob2_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_blob2");
        }

        return 0;
    }



    std::string as_pathstr_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_pathstr();
                
            
            
        }

        return
            
                as_pathstr
            
            
                ();
    }

    static PyObject *as_pathstr_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->as_pathstr_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_pathstr");
        }

        return 0;
    }



    const char * as_string_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_string();
                
            
            
        }

        return
            
                as_string
            
            
                ();
    }

    static PyObject *as_string_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    const char * r = o->as_string_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_str((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_string");
        }

        return 0;
    }



    unsigned int as_stringlen_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_stringlen();
                
            
            
        }

        return
            
                as_stringlen
            
            
                ();
    }

    static PyObject *as_stringlen_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->as_stringlen_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_stringlen");
        }

        return 0;
    }



    float as_float_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_float();
                
            
            
        }

        return
            
                as_float
            
            
                ();
    }

    static PyObject *as_float_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->as_float_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_float");
        }

        return 0;
    }



    long as_long_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_long();
                
            
            
        }

        return
            
                as_long
            
            
                ();
    }

    static PyObject *as_long_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    long r = o->as_long_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_l((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_long");
        }

        return 0;
    }



    bool as_bool_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_bool();
                
            
            
        }

        return
            
                as_bool
            
            
                ();
    }

    static PyObject *as_bool_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->as_bool_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_bool");
        }

        return 0;
    }



    unsigned int as_dict_nkeys_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_dict_nkeys();
                
            
            
        }

        return
            
                as_dict_nkeys
            
            
                ();
    }

    static PyObject *as_dict_nkeys_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->as_dict_nkeys_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_dict_nkeys");
        }

        return 0;
    }



    std::string as_dict_key_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_dict_key( a0 );
                
            
            
        }

        return
            
                as_dict_key
            
            
                ( a0 );
    }

    static PyObject *as_dict_key_method_(data_base_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->as_dict_key_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_dict_key");
        }

        return 0;
    }



    int compare_call_(bool owned ,const data_base_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::compare( a0 );
                
            
            
        }

        return
            
                compare
            
            
                ( a0 );
    }

    static PyObject *compare_method_(data_base_object_ *self, PyObject *args)
    {
         data_base_type_ * a0; 

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_base_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    int r = o->compare_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_i((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::compare");
        }

        return 0;
    }



    std::string repr_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::repr();
                
            
            
        }

        return
            
                repr
            
            
                ();
    }

    static PyObject *repr_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->repr_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::repr");
        }

        return 0;
    }



    float as_norm_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_norm();
                
            
            
        }

        return
            
                as_norm
            
            
                ();
    }

    static PyObject *as_norm_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->as_norm_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_norm");
        }

        return 0;
    }



    float as_denorm_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_denorm();
                
            
            
        }

        return
            
                as_denorm
            
            
                ();
    }

    static PyObject *as_denorm_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->as_denorm_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_denorm");
        }

        return 0;
    }



    float as_renorm_call_(bool owned ,float a0 ,float a1 ,float a2 )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_renorm( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                as_renorm
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *as_renorm_method_(data_base_object_ *self, PyObject *args)
    {
         float a0;  float a1;  float a2; 

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_f,&a0  ,fpcvt_f,&a1  ,fpcvt_f,&a2 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->as_renorm_call_ (self->owned ,a0 ,a1 ,a2  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_renorm");
        }

        return 0;
    }



    float as_array_ubound_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_array_ubound();
                
            
            
        }

        return
            
                as_array_ubound
            
            
                ();
    }

    static PyObject *as_array_ubound_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->as_array_ubound_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_array_ubound");
        }

        return 0;
    }



    float as_array_lbound_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_array_lbound();
                
            
            
        }

        return
            
                as_array_lbound
            
            
                ();
    }

    static PyObject *as_array_lbound_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->as_array_lbound_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_array_lbound");
        }

        return 0;
    }



    float as_array_rest_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_array_rest();
                
            
            
        }

        return
            
                as_array_rest
            
            
                ();
    }

    static PyObject *as_array_rest_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->as_array_rest_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_array_rest");
        }

        return 0;
    }



    unsigned int as_arraylen_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_arraylen();
                
            
            
        }

        return
            
                as_arraylen
            
            
                ();
    }

    static PyObject *as_arraylen_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->as_arraylen_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_arraylen");
        }

        return 0;
    }



    float as_array_member_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::as_array_member( a0 );
                
            
            
        }

        return
            
                as_array_member
            
            
                ( a0 );
    }

    static PyObject *as_array_member_method_(data_base_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->as_array_member_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::as_array_member");
        }

        return 0;
    }



    unsigned int data_type_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_base_type_::data_type();
                
            
            
        }

        return
            
                data_type
            
            
                ();
    }

    static PyObject *data_type_method_(data_base_object_ *self, PyObject *args)
    {
        

        data_base_wrapper_ *o = (data_base_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_base c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->data_type_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_base::data_type");
        }

        return 0;
    }





    static PyObject *_method_lock(data_base_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(data_base_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void data_base_delete_(data_base_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            data_base_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (data_base_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        data_base_wrapper_::__deleter__((data_base_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void data_base_dealloc_(data_base_object_* self)
{
    
    data_base_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * data_base_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    data_base_object_ *self;

    if(!(self = (data_base_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int data_base_init_(data_base_object_ *self, PyObject *args, PyObject *kwds)
{
    data_base_delete_(self);

    

    PyErr_SetString(PyExc_RuntimeError,"data_base invalid arguments");
    return -1;

}


static const char *data_base_time_docstring_ = ""
"        self.time() -> data timestamp"
"        "
""
;

static const char *data_base_type_docstring_ = ""
"        self.type() -> data type"
"        "
""
;

static const char *data_base_is_array_docstring_ = ""
"        self.is_array() -> True if data is a buffer"
"        "
""
;

static const char *data_base_is_string_docstring_ = ""
"        self.is_string() -> True if data is a string, False otherwise"
"        "
""
;

static const char *data_base_is_float_docstring_ = ""
"        self.is_float() -> True if data is a float, False otherwise"
"        "
""
;

static const char *data_base_is_long_docstring_ = ""
"        self.is_long() -> True if data is an integer, False otherwise"
"        "
""
;

static const char *data_base_is_null_docstring_ = ""
"        self.is_null() -> True if data is an null, False otherwise"
"        "
""
;

static const char *data_base_is_bool_docstring_ = ""
"        self.is_bool() -> True if data is a bool, False otherwise"
"        "
""
;

static const char *data_base_is_path_docstring_ = ""
"        self.is_path() -> True if data is a path, False otherwise"
"        "
""
;

static const char *data_base_is_dict_docstring_ = ""
"        self.is_dict() -> True if data is a dict, False otherwise"
"        "
""
;

static const char *data_base_is_blob_docstring_ = ""
"        self.is_blob() -> True if data is a blob, False otherwise"
"        "
""
;

static const char *data_base_as_stdstr_docstring_ = "";

static const char *data_base_as_blob2_docstring_ = "";

static const char *data_base_as_pathstr_docstring_ = "";

static const char *data_base_as_string_docstring_ = ""
"        self.as_string() -> string value"
"        "
"        If data is not a string, an exception is thrown"
"        "
""
;

static const char *data_base_as_stringlen_docstring_ = "";

static const char *data_base_as_float_docstring_ = ""
"        self.as_float() -> float value"
"        "
"        If data is not a float, an exception is thrown"
"        "
""
;

static const char *data_base_as_long_docstring_ = ""
"        self.as_long() -> long value"
"        "
"        If data is not a long, an exception is thrown"
"        "
""
;

static const char *data_base_as_bool_docstring_ = ""
"        self.as_bool() -> bool value"
"        "
"        If data is not a bool, an exception is thrown"
"        "
""
;

static const char *data_base_as_dict_nkeys_docstring_ = "";

static const char *data_base_as_dict_key_docstring_ = "";

static const char *data_base_compare_docstring_ = ""
"        self.compare(other) -> int"
"        "
"        compare self with other"
"        "
""
;

static const char *data_base_repr_docstring_ = ""
"        self.repr() -> string"
"        "
"        convert to string representation"
"        "
""
;

static const char *data_base_as_norm_docstring_ = "";

static const char *data_base_as_denorm_docstring_ = "";

static const char *data_base_as_renorm_docstring_ = "";

static const char *data_base_as_array_ubound_docstring_ = "";

static const char *data_base_as_array_lbound_docstring_ = "";

static const char *data_base_as_array_rest_docstring_ = "";

static const char *data_base_as_arraylen_docstring_ = "";

static const char *data_base_as_array_member_docstring_ = "";

static const char *data_base_data_type_docstring_ = "";




static PyMethodDef data_base_methods_[] = {
    
        { "time",(PyCFunction)data_base_wrapper_::time_method_,METH_VARARGS,(char *)data_base_time_docstring_ },
    
        { "type",(PyCFunction)data_base_wrapper_::type_method_,METH_VARARGS,(char *)data_base_type_docstring_ },
    
        { "is_array",(PyCFunction)data_base_wrapper_::is_array_method_,METH_VARARGS,(char *)data_base_is_array_docstring_ },
    
        { "is_string",(PyCFunction)data_base_wrapper_::is_string_method_,METH_VARARGS,(char *)data_base_is_string_docstring_ },
    
        { "is_float",(PyCFunction)data_base_wrapper_::is_float_method_,METH_VARARGS,(char *)data_base_is_float_docstring_ },
    
        { "is_long",(PyCFunction)data_base_wrapper_::is_long_method_,METH_VARARGS,(char *)data_base_is_long_docstring_ },
    
        { "is_null",(PyCFunction)data_base_wrapper_::is_null_method_,METH_VARARGS,(char *)data_base_is_null_docstring_ },
    
        { "is_bool",(PyCFunction)data_base_wrapper_::is_bool_method_,METH_VARARGS,(char *)data_base_is_bool_docstring_ },
    
        { "is_path",(PyCFunction)data_base_wrapper_::is_path_method_,METH_VARARGS,(char *)data_base_is_path_docstring_ },
    
        { "is_dict",(PyCFunction)data_base_wrapper_::is_dict_method_,METH_VARARGS,(char *)data_base_is_dict_docstring_ },
    
        { "is_blob",(PyCFunction)data_base_wrapper_::is_blob_method_,METH_VARARGS,(char *)data_base_is_blob_docstring_ },
    
        { "as_stdstr",(PyCFunction)data_base_wrapper_::as_stdstr_method_,METH_VARARGS,(char *)data_base_as_stdstr_docstring_ },
    
        { "as_blob2",(PyCFunction)data_base_wrapper_::as_blob2_method_,METH_VARARGS,(char *)data_base_as_blob2_docstring_ },
    
        { "as_pathstr",(PyCFunction)data_base_wrapper_::as_pathstr_method_,METH_VARARGS,(char *)data_base_as_pathstr_docstring_ },
    
        { "as_string",(PyCFunction)data_base_wrapper_::as_string_method_,METH_VARARGS,(char *)data_base_as_string_docstring_ },
    
        { "as_stringlen",(PyCFunction)data_base_wrapper_::as_stringlen_method_,METH_VARARGS,(char *)data_base_as_stringlen_docstring_ },
    
        { "as_float",(PyCFunction)data_base_wrapper_::as_float_method_,METH_VARARGS,(char *)data_base_as_float_docstring_ },
    
        { "as_long",(PyCFunction)data_base_wrapper_::as_long_method_,METH_VARARGS,(char *)data_base_as_long_docstring_ },
    
        { "as_bool",(PyCFunction)data_base_wrapper_::as_bool_method_,METH_VARARGS,(char *)data_base_as_bool_docstring_ },
    
        { "as_dict_nkeys",(PyCFunction)data_base_wrapper_::as_dict_nkeys_method_,METH_VARARGS,(char *)data_base_as_dict_nkeys_docstring_ },
    
        { "as_dict_key",(PyCFunction)data_base_wrapper_::as_dict_key_method_,METH_VARARGS,(char *)data_base_as_dict_key_docstring_ },
    
        { "compare",(PyCFunction)data_base_wrapper_::compare_method_,METH_VARARGS,(char *)data_base_compare_docstring_ },
    
        { "repr",(PyCFunction)data_base_wrapper_::repr_method_,METH_VARARGS,(char *)data_base_repr_docstring_ },
    
        { "as_norm",(PyCFunction)data_base_wrapper_::as_norm_method_,METH_VARARGS,(char *)data_base_as_norm_docstring_ },
    
        { "as_denorm",(PyCFunction)data_base_wrapper_::as_denorm_method_,METH_VARARGS,(char *)data_base_as_denorm_docstring_ },
    
        { "as_renorm",(PyCFunction)data_base_wrapper_::as_renorm_method_,METH_VARARGS,(char *)data_base_as_renorm_docstring_ },
    
        { "as_array_ubound",(PyCFunction)data_base_wrapper_::as_array_ubound_method_,METH_VARARGS,(char *)data_base_as_array_ubound_docstring_ },
    
        { "as_array_lbound",(PyCFunction)data_base_wrapper_::as_array_lbound_method_,METH_VARARGS,(char *)data_base_as_array_lbound_docstring_ },
    
        { "as_array_rest",(PyCFunction)data_base_wrapper_::as_array_rest_method_,METH_VARARGS,(char *)data_base_as_array_rest_docstring_ },
    
        { "as_arraylen",(PyCFunction)data_base_wrapper_::as_arraylen_method_,METH_VARARGS,(char *)data_base_as_arraylen_docstring_ },
    
        { "as_array_member",(PyCFunction)data_base_wrapper_::as_array_member_method_,METH_VARARGS,(char *)data_base_as_array_member_docstring_ },
    
        { "data_type",(PyCFunction)data_base_wrapper_::data_type_method_,METH_VARARGS,(char *)data_base_data_type_docstring_ },
    
    
        { "py_lock",(PyCFunction)data_base_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)data_base_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *data_base_docstring_ = ""
"    Belcanto data"
"    "
"    All belcanto data is stored in a data object.  A data is immutable"
"    once created, and the representation will be shared even between"
"    agents."
"    "
"    Every data contains a timestamp"
"    "
""
;

PyTypeObject data_base_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.data_base",/*tp_name*/
    sizeof(data_base_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)data_base_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
     (cmpfunc)data_base_wrapper_::special_cmp_method_  , /* tp_cmp */
     (reprfunc)data_base_wrapper_::special_repr_method_  , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
     (hashfunc)data_base_wrapper_::special_hash_method_  , /* tp_hash */
      0 , /* tp_call */
     (reprfunc)data_base_wrapper_::special_str_method_  , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)data_base_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    data_base_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)data_base_init_,  /* tp_init */
    0,                         /* tp_alloc */
    data_base_new_,             /* tp_new */
};

PyObject *data_base_wrapcopy_(data_base_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *data_base_wrap_(data_base_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = data_base_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    data_base_object_ *self = 
        PyObject_NEW(data_base_object_, &data_base_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        data_base_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

data_base_type_ *data_base_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &data_base_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected data_base object");
        return 0;
    }

    data_base_type_ *object = static_cast<data_base_type_ *>(((data_base_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null data_base object in unwrap");
        return 0;
    }

    return object;
}

void data_base_insert_(const std::pair< data_base_type_ *, PyObject *> &p)
{




    data_base_wrapped_.insert(p);


}

void data_base_erase_(data_base_type_ *p)
{




    data_base_wrapped_.erase(p);


}

PyObject * data_base_find_(data_base_type_ *p)
{




    PyObject *ret = 0;
    std::map< data_base_type_ *, PyObject *>::iterator where = data_base_wrapped_.find(p);
    if(where != data_base_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct data_base_dispatch_ data_base_realdispatcher_ =
{
    &data_base_typeobject_,
    data_base_wrap_,
    data_base_wrapcopy_,
    data_base_unwrap_,
    data_base_insert_,
    data_base_erase_,
    data_base_find_
};

  

struct data_object_ {
    PyObject_HEAD
    data_base_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct data_wrapper_: public data_type_ 
{

    static void __deleter__(data_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    data_wrapper_( PyObject *o, PyInterpreterState *i  ): data_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    data_wrapper_( PyObject *o, PyInterpreterState *i  ,const data_type_ & a0  ): data_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~data_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    data_type_ as_dict_lookup_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return data_type_::as_dict_lookup( a0 );
                
            
            
        }

        return
            
                as_dict_lookup
            
            
                ( a0 );
    }

    static PyObject *as_dict_lookup_method_(data_object_ *self, PyObject *args)
    {
         std::string a0; 

        data_wrapper_ *o = (data_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->as_dict_lookup_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data::as_dict_lookup");
        }

        return 0;
    }



    data_type_ make_normal_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_type_::make_normal();
                
            
            
        }

        return
            
                make_normal
            
            
                ();
    }

    static PyObject *make_normal_method_(data_object_ *self, PyObject *args)
    {
        

        data_wrapper_ *o = (data_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->make_normal_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data::make_normal");
        }

        return 0;
    }



    data_type_ as_dict_value_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return data_type_::as_dict_value( a0 );
                
            
            
        }

        return
            
                as_dict_value
            
            
                ( a0 );
    }

    static PyObject *as_dict_value_method_(data_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        data_wrapper_ *o = (data_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->as_dict_value_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data::as_dict_value");
        }

        return 0;
    }



    data_nb_type_ make_nb_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_type_::make_nb();
                
            
            
        }

        return
            
                make_nb
            
            
                ();
    }

    static PyObject *make_nb_method_(data_object_ *self, PyObject *args)
    {
        

        data_wrapper_ *o = (data_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_nb_type_ r = o->make_nb_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data::make_nb");
        }

        return 0;
    }



    data_type_ restamp_call_(bool owned ,unsigned long long a0 )
    {
        if(owned)
        {
            
                
                
                    return data_type_::restamp( a0 );
                
            
            
        }

        return
            
                restamp
            
            
                ( a0 );
    }

    static PyObject *restamp_method_(data_object_ *self, PyObject *args)
    {
         unsigned long long a0; 

        data_wrapper_ *o = (data_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ull,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->restamp_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data::restamp");
        }

        return 0;
    }





    static PyObject *_method_lock(data_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(data_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void data_delete_(data_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            data_base_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (data_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        data_wrapper_::__deleter__((data_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void data_dealloc_(data_object_* self)
{
    
    data_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * data_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    data_object_ *self;

    if(!(self = (data_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int data_init_(data_object_ *self, PyObject *args, PyObject *kwds)
{
    data_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new data_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing data");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            data_base_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         data_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_data_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new data_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing data");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            data_base_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"data invalid arguments");
    return -1;

}


static const char *data_as_dict_lookup_docstring_ = "";

static const char *data_make_normal_docstring_ = "";

static const char *data_as_dict_value_docstring_ = "";

static const char *data_make_nb_docstring_ = "";

static const char *data_restamp_docstring_ = "";




static PyMethodDef data_methods_[] = {
    
        { "as_dict_lookup",(PyCFunction)data_wrapper_::as_dict_lookup_method_,METH_VARARGS,(char *)data_as_dict_lookup_docstring_ },
    
        { "make_normal",(PyCFunction)data_wrapper_::make_normal_method_,METH_VARARGS,(char *)data_make_normal_docstring_ },
    
        { "as_dict_value",(PyCFunction)data_wrapper_::as_dict_value_method_,METH_VARARGS,(char *)data_as_dict_value_docstring_ },
    
        { "make_nb",(PyCFunction)data_wrapper_::make_nb_method_,METH_VARARGS,(char *)data_make_nb_docstring_ },
    
        { "restamp",(PyCFunction)data_wrapper_::restamp_method_,METH_VARARGS,(char *)data_restamp_docstring_ },
    
    
        { "py_lock",(PyCFunction)data_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)data_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *data_docstring_ = "";

PyTypeObject data_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.data",/*tp_name*/
    sizeof(data_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)data_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)data_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    data_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)data_init_,  /* tp_init */
    0,                         /* tp_alloc */
    data_new_,             /* tp_new */
};

PyObject *data_wrapcopy_(data_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            data_object_ *obj = 
                PyObject_NEW(data_object_, data_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new data_type_(*o);
                }

                try
                {
                    data_base_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying data");
                return 0;
            }
        

    
}

PyObject *data_wrap_(data_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = data_base_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    data_object_ *self = 
        PyObject_NEW(data_object_, &data_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        data_base_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

data_type_ *data_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &data_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected data object");
        return 0;
    }

    data_type_ *object = static_cast<data_type_ *>(((data_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null data object in unwrap");
        return 0;
    }

    return object;
}

void data_insert_(const std::pair< data_type_ *, PyObject *> &p)
{


    data_base_dispatcher_->insert(p);




}

void data_erase_(data_type_ *p)
{


    data_base_dispatcher_->erase(p);




}

PyObject * data_find_(data_type_ *p)
{


    return data_base_dispatcher_->find(p);




}

static struct data_dispatch_ data_realdispatcher_ =
{
    &data_typeobject_,
    data_wrap_,
    data_wrapcopy_,
    data_unwrap_,
    data_insert_,
    data_erase_,
    data_find_
};

  

struct data_nb_object_ {
    PyObject_HEAD
    data_base_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct data_nb_wrapper_: public data_nb_type_ 
{

    static void __deleter__(data_nb_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    data_nb_wrapper_( PyObject *o, PyInterpreterState *i  ): data_nb_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    data_nb_wrapper_( PyObject *o, PyInterpreterState *i  ,const data_nb_type_ & a0  ): data_nb_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~data_nb_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    data_nb_type_ as_dict_lookup_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return data_nb_type_::as_dict_lookup( a0 );
                
            
            
        }

        return
            
                as_dict_lookup
            
            
                ( a0 );
    }

    static PyObject *as_dict_lookup_method_(data_nb_object_ *self, PyObject *args)
    {
         std::string a0; 

        data_nb_wrapper_ *o = (data_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_nb_type_ r = o->as_dict_lookup_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_nb::as_dict_lookup");
        }

        return 0;
    }



    data_type_ make_normal_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_nb_type_::make_normal();
                
            
            
        }

        return
            
                make_normal
            
            
                ();
    }

    static PyObject *make_normal_method_(data_nb_object_ *self, PyObject *args)
    {
        

        data_nb_wrapper_ *o = (data_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->make_normal_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_nb::make_normal");
        }

        return 0;
    }



    data_nb_type_ as_dict_value_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return data_nb_type_::as_dict_value( a0 );
                
            
            
        }

        return
            
                as_dict_value
            
            
                ( a0 );
    }

    static PyObject *as_dict_value_method_(data_nb_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        data_nb_wrapper_ *o = (data_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_nb_type_ r = o->as_dict_value_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_nb::as_dict_value");
        }

        return 0;
    }



    data_nb_type_ make_nb_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return data_nb_type_::make_nb();
                
            
            
        }

        return
            
                make_nb
            
            
                ();
    }

    static PyObject *make_nb_method_(data_nb_object_ *self, PyObject *args)
    {
        

        data_nb_wrapper_ *o = (data_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_nb_type_ r = o->make_nb_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_nb::make_nb");
        }

        return 0;
    }



    data_nb_type_ restamp_call_(bool owned ,unsigned long long a0 )
    {
        if(owned)
        {
            
                
                
                    return data_nb_type_::restamp( a0 );
                
            
            
        }

        return
            
                restamp
            
            
                ( a0 );
    }

    static PyObject *restamp_method_(data_nb_object_ *self, PyObject *args)
    {
         unsigned long long a0; 

        data_nb_wrapper_ *o = (data_nb_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null data_nb c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ull,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_nb_type_ r = o->restamp_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in data_nb::restamp");
        }

        return 0;
    }





    static PyObject *_method_lock(data_nb_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(data_nb_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void data_nb_delete_(data_nb_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            data_base_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (data_nb_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        data_nb_wrapper_::__deleter__((data_nb_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void data_nb_dealloc_(data_nb_object_* self)
{
    
    data_nb_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * data_nb_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    data_nb_object_ *self;

    if(!(self = (data_nb_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int data_nb_init_(data_nb_object_ *self, PyObject *args, PyObject *kwds)
{
    data_nb_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new data_nb_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing data_nb");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            data_base_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         data_nb_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_data_nb_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new data_nb_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing data_nb");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            data_base_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"data_nb invalid arguments");
    return -1;

}


static const char *data_nb_as_dict_lookup_docstring_ = "";

static const char *data_nb_make_normal_docstring_ = "";

static const char *data_nb_as_dict_value_docstring_ = "";

static const char *data_nb_make_nb_docstring_ = "";

static const char *data_nb_restamp_docstring_ = "";




static PyMethodDef data_nb_methods_[] = {
    
        { "as_dict_lookup",(PyCFunction)data_nb_wrapper_::as_dict_lookup_method_,METH_VARARGS,(char *)data_nb_as_dict_lookup_docstring_ },
    
        { "make_normal",(PyCFunction)data_nb_wrapper_::make_normal_method_,METH_VARARGS,(char *)data_nb_make_normal_docstring_ },
    
        { "as_dict_value",(PyCFunction)data_nb_wrapper_::as_dict_value_method_,METH_VARARGS,(char *)data_nb_as_dict_value_docstring_ },
    
        { "make_nb",(PyCFunction)data_nb_wrapper_::make_nb_method_,METH_VARARGS,(char *)data_nb_make_nb_docstring_ },
    
        { "restamp",(PyCFunction)data_nb_wrapper_::restamp_method_,METH_VARARGS,(char *)data_nb_restamp_docstring_ },
    
    
        { "py_lock",(PyCFunction)data_nb_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)data_nb_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *data_nb_docstring_ = "";

PyTypeObject data_nb_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.data_nb",/*tp_name*/
    sizeof(data_nb_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)data_nb_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)data_nb_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    data_nb_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)data_nb_init_,  /* tp_init */
    0,                         /* tp_alloc */
    data_nb_new_,             /* tp_new */
};

PyObject *data_nb_wrapcopy_(data_nb_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            data_nb_object_ *obj = 
                PyObject_NEW(data_nb_object_, data_nb_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new data_nb_type_(*o);
                }

                try
                {
                    data_base_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying data_nb");
                return 0;
            }
        

    
}

PyObject *data_nb_wrap_(data_nb_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = data_base_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    data_nb_object_ *self = 
        PyObject_NEW(data_nb_object_, &data_nb_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        data_base_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

data_nb_type_ *data_nb_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &data_nb_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected data_nb object");
        return 0;
    }

    data_nb_type_ *object = static_cast<data_nb_type_ *>(((data_nb_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null data_nb object in unwrap");
        return 0;
    }

    return object;
}

void data_nb_insert_(const std::pair< data_nb_type_ *, PyObject *> &p)
{


    data_base_dispatcher_->insert(p);




}

void data_nb_erase_(data_nb_type_ *p)
{


    data_base_dispatcher_->erase(p);




}

PyObject * data_nb_find_(data_nb_type_ *p)
{


    return data_base_dispatcher_->find(p);




}

static struct data_nb_dispatch_ data_nb_realdispatcher_ =
{
    &data_nb_typeobject_,
    data_nb_wrap_,
    data_nb_wrapcopy_,
    data_nb_unwrap_,
    data_nb_insert_,
    data_nb_erase_,
    data_nb_find_
};

  

struct clockdomain_ctl_object_ {
    PyObject_HEAD
    clockdomain_ctl_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct clockdomain_ctl_wrapper_: public clockdomain_ctl_type_ 
{

    static void __deleter__(clockdomain_ctl_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    clockdomain_ctl_wrapper_( PyObject *o, PyInterpreterState *i  ): clockdomain_ctl_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~clockdomain_ctl_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(clockdomain_ctl_object_ *self, visitproc visit, void *arg)
    {
        clockdomain_ctl_wrapper_ *o = (clockdomain_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(clockdomain_ctl_object_ *self)
    {
        clockdomain_ctl_wrapper_ *o = (clockdomain_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void set_source_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return clockdomain_ctl_type_::set_source( a0 );
                
            
            
        }

        return
            
                set_source
            
            
                ( a0 );
    }

    static PyObject *set_source_method_(clockdomain_ctl_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        clockdomain_ctl_wrapper_ *o = (clockdomain_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clockdomain_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_source_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clockdomain_ctl::set_source");
        }

        return 0;
    }



    void sink_call_(bool owned ,clockref_type_ * a0 ,const char * a1 )
    {
        if(owned)
        {
            
                
                
                    return clockdomain_ctl_type_::sink( a0  ,a1 );
                
            
            
        }

        return
            
                sink
            
            
                ( a0  ,a1 );
    }

    static PyObject *sink_method_(clockdomain_ctl_object_ *self, PyObject *args)
    {
         clockref_type_ * a0;  const char * a1; 

        clockdomain_ctl_wrapper_ *o = (clockdomain_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clockdomain_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_clockref,&a0  ,fpcvt_str,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->sink_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clockdomain_ctl::sink");
        }

        return 0;
    }



    unsigned long get_sample_rate_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clockdomain_ctl_type_::get_sample_rate();
                
            
            
        }

        return
            
                get_sample_rate
            
            
                ();
    }

    static PyObject *get_sample_rate_method_(clockdomain_ctl_object_ *self, PyObject *args)
    {
        

        clockdomain_ctl_wrapper_ *o = (clockdomain_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clockdomain_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->get_sample_rate_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clockdomain_ctl::get_sample_rate");
        }

        return 0;
    }



    unsigned int get_buffer_size_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clockdomain_ctl_type_::get_buffer_size();
                
            
            
        }

        return
            
                get_buffer_size
            
            
                ();
    }

    static PyObject *get_buffer_size_method_(clockdomain_ctl_object_ *self, PyObject *args)
    {
        

        clockdomain_ctl_wrapper_ *o = (clockdomain_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clockdomain_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_buffer_size_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clockdomain_ctl::get_buffer_size");
        }

        return 0;
    }



    data_type_ get_source_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clockdomain_ctl_type_::get_source();
                
            
            
        }

        return
            
                get_source
            
            
                ();
    }

    static PyObject *get_source_method_(clockdomain_ctl_object_ *self, PyObject *args)
    {
        

        clockdomain_ctl_wrapper_ *o = (clockdomain_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clockdomain_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->get_source_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clockdomain_ctl::get_source");
        }

        return 0;
    }



    void add_listener_call_(bool owned ,const notify_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return clockdomain_ctl_type_::add_listener( a0 );
                
            
            
        }

        return
            
                add_listener
            
            
                ( a0 );
    }

    static PyObject *add_listener_method_(clockdomain_ctl_object_ *self, PyObject *args)
    {
         notify_type_ * a0; 

        clockdomain_ctl_wrapper_ *o = (clockdomain_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clockdomain_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_notify_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->add_listener_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clockdomain_ctl::add_listener");
        }

        return 0;
    }



    void remove_listener_call_(bool owned ,const notify_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return clockdomain_ctl_type_::remove_listener( a0 );
                
            
            
        }

        return
            
                remove_listener
            
            
                ( a0 );
    }

    static PyObject *remove_listener_method_(clockdomain_ctl_object_ *self, PyObject *args)
    {
         notify_type_ * a0; 

        clockdomain_ctl_wrapper_ *o = (clockdomain_ctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clockdomain_ctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_notify_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->remove_listener_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clockdomain_ctl::remove_listener");
        }

        return 0;
    }





    static PyObject *_method_lock(clockdomain_ctl_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(clockdomain_ctl_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void clockdomain_ctl_delete_(clockdomain_ctl_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            clockdomain_ctl_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (clockdomain_ctl_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        clockdomain_ctl_wrapper_::__deleter__((clockdomain_ctl_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void clockdomain_ctl_dealloc_(clockdomain_ctl_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    clockdomain_ctl_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * clockdomain_ctl_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    clockdomain_ctl_object_ *self;

    if(!(self = (clockdomain_ctl_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int clockdomain_ctl_init_(clockdomain_ctl_object_ *self, PyObject *args, PyObject *kwds)
{
    clockdomain_ctl_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new clockdomain_ctl_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing clockdomain_ctl");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            clockdomain_ctl_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"clockdomain_ctl invalid arguments");
    return -1;

}


static const char *clockdomain_ctl_set_source_docstring_ = "";

static const char *clockdomain_ctl_sink_docstring_ = "";

static const char *clockdomain_ctl_get_sample_rate_docstring_ = "";

static const char *clockdomain_ctl_get_buffer_size_docstring_ = "";

static const char *clockdomain_ctl_get_source_docstring_ = "";

static const char *clockdomain_ctl_add_listener_docstring_ = "";

static const char *clockdomain_ctl_remove_listener_docstring_ = "";




static PyMethodDef clockdomain_ctl_methods_[] = {
    
        { "set_source",(PyCFunction)clockdomain_ctl_wrapper_::set_source_method_,METH_VARARGS,(char *)clockdomain_ctl_set_source_docstring_ },
    
        { "sink",(PyCFunction)clockdomain_ctl_wrapper_::sink_method_,METH_VARARGS,(char *)clockdomain_ctl_sink_docstring_ },
    
        { "get_sample_rate",(PyCFunction)clockdomain_ctl_wrapper_::get_sample_rate_method_,METH_VARARGS,(char *)clockdomain_ctl_get_sample_rate_docstring_ },
    
        { "get_buffer_size",(PyCFunction)clockdomain_ctl_wrapper_::get_buffer_size_method_,METH_VARARGS,(char *)clockdomain_ctl_get_buffer_size_docstring_ },
    
        { "get_source",(PyCFunction)clockdomain_ctl_wrapper_::get_source_method_,METH_VARARGS,(char *)clockdomain_ctl_get_source_docstring_ },
    
        { "add_listener",(PyCFunction)clockdomain_ctl_wrapper_::add_listener_method_,METH_VARARGS,(char *)clockdomain_ctl_add_listener_docstring_ },
    
        { "remove_listener",(PyCFunction)clockdomain_ctl_wrapper_::remove_listener_method_,METH_VARARGS,(char *)clockdomain_ctl_remove_listener_docstring_ },
    
    
        { "py_lock",(PyCFunction)clockdomain_ctl_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)clockdomain_ctl_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *clockdomain_ctl_docstring_ = "";

PyTypeObject clockdomain_ctl_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.clockdomain_ctl",/*tp_name*/
    sizeof(clockdomain_ctl_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)clockdomain_ctl_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)clockdomain_ctl_docstring_,       /* tp_doc */
    (traverseproc)clockdomain_ctl_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)clockdomain_ctl_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    clockdomain_ctl_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)clockdomain_ctl_init_,  /* tp_init */
    0,                         /* tp_alloc */
    clockdomain_ctl_new_,             /* tp_new */
};

PyObject *clockdomain_ctl_wrapcopy_(clockdomain_ctl_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *clockdomain_ctl_wrap_(clockdomain_ctl_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = clockdomain_ctl_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    clockdomain_ctl_object_ *self = 
        
        PyObject_GC_New(clockdomain_ctl_object_, &clockdomain_ctl_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        clockdomain_ctl_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

clockdomain_ctl_type_ *clockdomain_ctl_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &clockdomain_ctl_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected clockdomain_ctl object");
        return 0;
    }

    clockdomain_ctl_type_ *object = static_cast<clockdomain_ctl_type_ *>(((clockdomain_ctl_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null clockdomain_ctl object in unwrap");
        return 0;
    }

    return object;
}

void clockdomain_ctl_insert_(const std::pair< clockdomain_ctl_type_ *, PyObject *> &p)
{




    clockdomain_ctl_wrapped_.insert(p);


}

void clockdomain_ctl_erase_(clockdomain_ctl_type_ *p)
{




    clockdomain_ctl_wrapped_.erase(p);


}

PyObject * clockdomain_ctl_find_(clockdomain_ctl_type_ *p)
{




    PyObject *ret = 0;
    std::map< clockdomain_ctl_type_ *, PyObject *>::iterator where = clockdomain_ctl_wrapped_.find(p);
    if(where != clockdomain_ctl_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct clockdomain_ctl_dispatch_ clockdomain_ctl_realdispatcher_ =
{
    &clockdomain_ctl_typeobject_,
    clockdomain_ctl_wrap_,
    clockdomain_ctl_wrapcopy_,
    clockdomain_ctl_unwrap_,
    clockdomain_ctl_insert_,
    clockdomain_ctl_erase_,
    clockdomain_ctl_find_
};

  

struct clocksink_object_ {
    PyObject_HEAD
    clockref_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct clocksink_wrapper_: public clocksink_type_ 
{

    static void __deleter__(clocksink_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    clocksink_wrapper_( PyObject *o, PyInterpreterState *i  ): clocksink_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~clocksink_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void remove_downstream_call_(bool owned ,clockref_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return clocksink_type_::remove_downstream( a0 );
                
            
            
        }

        return
            
                remove_downstream
            
            
                ( a0 );
    }

    static PyObject *remove_downstream_method_(clocksink_object_ *self, PyObject *args)
    {
         clockref_type_ * a0; 

        clocksink_wrapper_ *o = (clocksink_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksink c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_clockref,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->remove_downstream_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksink::remove_downstream");
        }

        return 0;
    }



    bool opening_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clocksink_type_::opening();
                
            
            
        }

        return
            
                opening
            
            
                ();
    }

    static PyObject *opening_method_(clocksink_object_ *self, PyObject *args)
    {
        

        clocksink_wrapper_ *o = (clocksink_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksink c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->opening_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksink::opening");
        }

        return 0;
    }



    void tick_enable_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return clocksink_type_::tick_enable( a0 );
                
            
            
        }

        return
            
                tick_enable
            
            
                ( a0 );
    }

    static PyObject *tick_enable_method_(clocksink_object_ *self, PyObject *args)
    {
         bool a0; 

        clocksink_wrapper_ *o = (clocksink_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksink c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->tick_enable_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksink::tick_enable");
        }

        return 0;
    }



    void add_downstream_call_(bool owned ,clockref_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return clocksink_type_::add_downstream( a0 );
                
            
            
        }

        return
            
                add_downstream
            
            
                ( a0 );
    }

    static PyObject *add_downstream_method_(clocksink_object_ *self, PyObject *args)
    {
         clockref_type_ * a0; 

        clocksink_wrapper_ *o = (clocksink_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksink c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_clockref,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->add_downstream_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksink::add_downstream");
        }

        return 0;
    }



    bool running_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clocksink_type_::running();
                
            
            
        }

        return
            
                running
            
            
                ();
    }

    static PyObject *running_method_(clocksink_object_ *self, PyObject *args)
    {
        

        clocksink_wrapper_ *o = (clocksink_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksink c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->running_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksink::running");
        }

        return 0;
    }



    void tick_disable_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clocksink_type_::tick_disable();
                
            
            
        }

        return
            
                tick_disable
            
            
                ();
    }

    static PyObject *tick_disable_method_(clocksink_object_ *self, PyObject *args)
    {
        

        clocksink_wrapper_ *o = (clocksink_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksink c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->tick_disable_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksink::tick_disable");
        }

        return 0;
    }



    void add_upstream_call_(bool owned ,clockref_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return clocksink_type_::add_upstream( a0 );
                
            
            
        }

        return
            
                add_upstream
            
            
                ( a0 );
    }

    static PyObject *add_upstream_method_(clocksink_object_ *self, PyObject *args)
    {
         clockref_type_ * a0; 

        clocksink_wrapper_ *o = (clocksink_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksink c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_clockref,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->add_upstream_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksink::add_upstream");
        }

        return 0;
    }



    void close_sink_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clocksink_type_::close_sink();
                
            
            
        }

        return
            
                close_sink
            
            
                ();
    }

    static PyObject *close_sink_method_(clocksink_object_ *self, PyObject *args)
    {
        

        clocksink_wrapper_ *o = (clocksink_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksink c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_sink_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksink::close_sink");
        }

        return 0;
    }



    bool closing_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clocksink_type_::closing();
                
            
            
        }

        return
            
                closing
            
            
                ();
    }

    static PyObject *closing_method_(clocksink_object_ *self, PyObject *args)
    {
        

        clocksink_wrapper_ *o = (clocksink_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksink c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->closing_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksink::closing");
        }

        return 0;
    }



    bool open_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clocksink_type_::open();
                
            
            
        }

        return
            
                open
            
            
                ();
    }

    static PyObject *open_method_(clocksink_object_ *self, PyObject *args)
    {
        

        clocksink_wrapper_ *o = (clocksink_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksink c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksink::open");
        }

        return 0;
    }



    void remove_upstream_call_(bool owned ,clockref_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return clocksink_type_::remove_upstream( a0 );
                
            
            
        }

        return
            
                remove_upstream
            
            
                ( a0 );
    }

    static PyObject *remove_upstream_method_(clocksink_object_ *self, PyObject *args)
    {
         clockref_type_ * a0; 

        clocksink_wrapper_ *o = (clocksink_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksink c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_clockref,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->remove_upstream_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksink::remove_upstream");
        }

        return 0;
    }





    static PyObject *_method_lock(clocksink_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(clocksink_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void clocksink_delete_(clocksink_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            clockref_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (clocksink_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        clocksink_wrapper_::__deleter__((clocksink_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void clocksink_dealloc_(clocksink_object_* self)
{
    
    clocksink_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * clocksink_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    clocksink_object_ *self;

    if(!(self = (clocksink_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int clocksink_init_(clocksink_object_ *self, PyObject *args, PyObject *kwds)
{
    clocksink_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new clocksink_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing clocksink");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            clockref_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"clocksink invalid arguments");
    return -1;

}


static const char *clocksink_remove_downstream_docstring_ = "";

static const char *clocksink_opening_docstring_ = "";

static const char *clocksink_tick_enable_docstring_ = "";

static const char *clocksink_add_downstream_docstring_ = "";

static const char *clocksink_running_docstring_ = "";

static const char *clocksink_tick_disable_docstring_ = "";

static const char *clocksink_add_upstream_docstring_ = "";

static const char *clocksink_close_sink_docstring_ = "";

static const char *clocksink_closing_docstring_ = "";

static const char *clocksink_open_docstring_ = "";

static const char *clocksink_remove_upstream_docstring_ = "";




static PyMethodDef clocksink_methods_[] = {
    
        { "remove_downstream",(PyCFunction)clocksink_wrapper_::remove_downstream_method_,METH_VARARGS,(char *)clocksink_remove_downstream_docstring_ },
    
        { "opening",(PyCFunction)clocksink_wrapper_::opening_method_,METH_VARARGS,(char *)clocksink_opening_docstring_ },
    
        { "tick_enable",(PyCFunction)clocksink_wrapper_::tick_enable_method_,METH_VARARGS,(char *)clocksink_tick_enable_docstring_ },
    
        { "add_downstream",(PyCFunction)clocksink_wrapper_::add_downstream_method_,METH_VARARGS,(char *)clocksink_add_downstream_docstring_ },
    
        { "running",(PyCFunction)clocksink_wrapper_::running_method_,METH_VARARGS,(char *)clocksink_running_docstring_ },
    
        { "tick_disable",(PyCFunction)clocksink_wrapper_::tick_disable_method_,METH_VARARGS,(char *)clocksink_tick_disable_docstring_ },
    
        { "add_upstream",(PyCFunction)clocksink_wrapper_::add_upstream_method_,METH_VARARGS,(char *)clocksink_add_upstream_docstring_ },
    
        { "close_sink",(PyCFunction)clocksink_wrapper_::close_sink_method_,METH_VARARGS,(char *)clocksink_close_sink_docstring_ },
    
        { "closing",(PyCFunction)clocksink_wrapper_::closing_method_,METH_VARARGS,(char *)clocksink_closing_docstring_ },
    
        { "open",(PyCFunction)clocksink_wrapper_::open_method_,METH_VARARGS,(char *)clocksink_open_docstring_ },
    
        { "remove_upstream",(PyCFunction)clocksink_wrapper_::remove_upstream_method_,METH_VARARGS,(char *)clocksink_remove_upstream_docstring_ },
    
    
        { "py_lock",(PyCFunction)clocksink_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)clocksink_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *clocksink_docstring_ = "";

PyTypeObject clocksink_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.clocksink",/*tp_name*/
    sizeof(clocksink_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)clocksink_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)clocksink_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    clocksink_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)clocksink_init_,  /* tp_init */
    0,                         /* tp_alloc */
    clocksink_new_,             /* tp_new */
};

PyObject *clocksink_wrapcopy_(clocksink_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *clocksink_wrap_(clocksink_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = clockref_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    clocksink_object_ *self = 
        PyObject_NEW(clocksink_object_, &clocksink_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        clockref_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

clocksink_type_ *clocksink_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &clocksink_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected clocksink object");
        return 0;
    }

    clocksink_type_ *object = static_cast<clocksink_type_ *>(((clocksink_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null clocksink object in unwrap");
        return 0;
    }

    return object;
}

void clocksink_insert_(const std::pair< clocksink_type_ *, PyObject *> &p)
{


    clockref_dispatcher_->insert(p);




}

void clocksink_erase_(clocksink_type_ *p)
{


    clockref_dispatcher_->erase(p);




}

PyObject * clocksink_find_(clocksink_type_ *p)
{


    return clockref_dispatcher_->find(p);




}

static struct clocksink_dispatch_ clocksink_realdispatcher_ =
{
    &clocksink_typeobject_,
    clocksink_wrap_,
    clocksink_wrapcopy_,
    clocksink_unwrap_,
    clocksink_insert_,
    clocksink_erase_,
    clocksink_find_
};

  

struct clocksource_object_ {
    PyObject_HEAD
    clocksource_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct clocksource_wrapper_: public clocksource_type_ 
{

    static void __deleter__(clocksource_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    clocksource_wrapper_( PyObject *o, PyInterpreterState *i  ): clocksource_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~clocksource_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void tick_slow_call_(bool owned ,unsigned long long a0 )
    {
        if(owned)
        {
            
                
                
                    return clocksource_type_::tick_slow( a0 );
                
            
            
        }

        return
            
                tick_slow
            
            
                ( a0 );
    }

    static PyObject *tick_slow_method_(clocksource_object_ *self, PyObject *args)
    {
         unsigned long long a0; 

        clocksource_wrapper_ *o = (clocksource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ull,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->tick_slow_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksource::tick_slow");
        }

        return 0;
    }



    void set_details_call_(bool owned ,unsigned int a0 ,unsigned long a1 )
    {
        if(owned)
        {
            
                
                
                    return clocksource_type_::set_details( a0  ,a1 );
                
            
            
        }

        return
            
                set_details
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_details_method_(clocksource_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned long a1; 

        clocksource_wrapper_ *o = (clocksource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ul,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_details_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksource::set_details");
        }

        return 0;
    }



    void close_source_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clocksource_type_::close_source();
                
            
            
        }

        return
            
                close_source
            
            
                ();
    }

    static PyObject *close_source_method_(clocksource_object_ *self, PyObject *args)
    {
        

        clocksource_wrapper_ *o = (clocksource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_source_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksource::close_source");
        }

        return 0;
    }



    bool open_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clocksource_type_::open();
                
            
            
        }

        return
            
                open
            
            
                ();
    }

    static PyObject *open_method_(clocksource_object_ *self, PyObject *args)
    {
        

        clocksource_wrapper_ *o = (clocksource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksource::open");
        }

        return 0;
    }



    bool opening_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clocksource_type_::opening();
                
            
            
        }

        return
            
                opening
            
            
                ();
    }

    static PyObject *opening_method_(clocksource_object_ *self, PyObject *args)
    {
        

        clocksource_wrapper_ *o = (clocksource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->opening_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksource::opening");
        }

        return 0;
    }



    bool closing_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clocksource_type_::closing();
                
            
            
        }

        return
            
                closing
            
            
                ();
    }

    static PyObject *closing_method_(clocksource_object_ *self, PyObject *args)
    {
        

        clocksource_wrapper_ *o = (clocksource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->closing_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksource::closing");
        }

        return 0;
    }



    bool running_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clocksource_type_::running();
                
            
            
        }

        return
            
                running
            
            
                ();
    }

    static PyObject *running_method_(clocksource_object_ *self, PyObject *args)
    {
        

        clocksource_wrapper_ *o = (clocksource_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clocksource c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->running_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clocksource::running");
        }

        return 0;
    }





    static PyObject *_method_lock(clocksource_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(clocksource_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void clocksource_delete_(clocksource_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            clocksource_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (clocksource_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        clocksource_wrapper_::__deleter__((clocksource_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void clocksource_dealloc_(clocksource_object_* self)
{
    
    clocksource_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * clocksource_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    clocksource_object_ *self;

    if(!(self = (clocksource_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int clocksource_init_(clocksource_object_ *self, PyObject *args, PyObject *kwds)
{
    clocksource_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new clocksource_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing clocksource");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            clocksource_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"clocksource invalid arguments");
    return -1;

}


static const char *clocksource_tick_slow_docstring_ = "";

static const char *clocksource_set_details_docstring_ = "";

static const char *clocksource_close_source_docstring_ = "";

static const char *clocksource_open_docstring_ = "";

static const char *clocksource_opening_docstring_ = "";

static const char *clocksource_closing_docstring_ = "";

static const char *clocksource_running_docstring_ = "";




static PyMethodDef clocksource_methods_[] = {
    
        { "tick_slow",(PyCFunction)clocksource_wrapper_::tick_slow_method_,METH_VARARGS,(char *)clocksource_tick_slow_docstring_ },
    
        { "set_details",(PyCFunction)clocksource_wrapper_::set_details_method_,METH_VARARGS,(char *)clocksource_set_details_docstring_ },
    
        { "close_source",(PyCFunction)clocksource_wrapper_::close_source_method_,METH_VARARGS,(char *)clocksource_close_source_docstring_ },
    
        { "open",(PyCFunction)clocksource_wrapper_::open_method_,METH_VARARGS,(char *)clocksource_open_docstring_ },
    
        { "opening",(PyCFunction)clocksource_wrapper_::opening_method_,METH_VARARGS,(char *)clocksource_opening_docstring_ },
    
        { "closing",(PyCFunction)clocksource_wrapper_::closing_method_,METH_VARARGS,(char *)clocksource_closing_docstring_ },
    
        { "running",(PyCFunction)clocksource_wrapper_::running_method_,METH_VARARGS,(char *)clocksource_running_docstring_ },
    
    
        { "py_lock",(PyCFunction)clocksource_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)clocksource_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *clocksource_docstring_ = "";

PyTypeObject clocksource_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.clocksource",/*tp_name*/
    sizeof(clocksource_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)clocksource_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)clocksource_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    clocksource_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)clocksource_init_,  /* tp_init */
    0,                         /* tp_alloc */
    clocksource_new_,             /* tp_new */
};

PyObject *clocksource_wrapcopy_(clocksource_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *clocksource_wrap_(clocksource_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = clocksource_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    clocksource_object_ *self = 
        PyObject_NEW(clocksource_object_, &clocksource_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        clocksource_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

clocksource_type_ *clocksource_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &clocksource_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected clocksource object");
        return 0;
    }

    clocksource_type_ *object = static_cast<clocksource_type_ *>(((clocksource_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null clocksource object in unwrap");
        return 0;
    }

    return object;
}

void clocksource_insert_(const std::pair< clocksource_type_ *, PyObject *> &p)
{




    clocksource_wrapped_.insert(p);


}

void clocksource_erase_(clocksource_type_ *p)
{




    clocksource_wrapped_.erase(p);


}

PyObject * clocksource_find_(clocksource_type_ *p)
{




    PyObject *ret = 0;
    std::map< clocksource_type_ *, PyObject *>::iterator where = clocksource_wrapped_.find(p);
    if(where != clocksource_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct clocksource_dispatch_ clocksource_realdispatcher_ =
{
    &clocksource_typeobject_,
    clocksource_wrap_,
    clocksource_wrapcopy_,
    clocksource_unwrap_,
    clocksource_insert_,
    clocksource_erase_,
    clocksource_find_
};

  

struct fast_pitchbender_object_ {
    PyObject_HEAD
    fast_pitchbender_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct fast_pitchbender_wrapper_: public fast_pitchbender_type_ 
{

    static void __deleter__(fast_pitchbender_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    fast_pitchbender_wrapper_( PyObject *o, PyInterpreterState *i  ): fast_pitchbender_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~fast_pitchbender_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    change_nb_type_ set_range_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fast_pitchbender_type_::set_range();
                
            
            
        }

        return
            
                set_range
            
            
                ();
    }

    static PyObject *set_range_method_(fast_pitchbender_object_ *self, PyObject *args)
    {
        

        fast_pitchbender_wrapper_ *o = (fast_pitchbender_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fast_pitchbender c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->set_range_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fast_pitchbender::set_range");
        }

        return 0;
    }



    change_nb_type_ set_offset_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fast_pitchbender_type_::set_offset();
                
            
            
        }

        return
            
                set_offset
            
            
                ();
    }

    static PyObject *set_offset_method_(fast_pitchbender_object_ *self, PyObject *args)
    {
        

        fast_pitchbender_wrapper_ *o = (fast_pitchbender_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fast_pitchbender c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->set_offset_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fast_pitchbender::set_offset");
        }

        return 0;
    }



    dd2d_nb_type_ bend_functor_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return fast_pitchbender_type_::bend_functor();
                
            
            
        }

        return
            
                bend_functor
            
            
                ();
    }

    static PyObject *bend_functor_method_(fast_pitchbender_object_ *self, PyObject *args)
    {
        

        fast_pitchbender_wrapper_ *o = (fast_pitchbender_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null fast_pitchbender c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    dd2d_nb_type_ r = o->bend_functor_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_dd2d_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in fast_pitchbender::bend_functor");
        }

        return 0;
    }





    static PyObject *_method_lock(fast_pitchbender_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(fast_pitchbender_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void fast_pitchbender_delete_(fast_pitchbender_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            fast_pitchbender_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (fast_pitchbender_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        fast_pitchbender_wrapper_::__deleter__((fast_pitchbender_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void fast_pitchbender_dealloc_(fast_pitchbender_object_* self)
{
    
    fast_pitchbender_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * fast_pitchbender_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    fast_pitchbender_object_ *self;

    if(!(self = (fast_pitchbender_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int fast_pitchbender_init_(fast_pitchbender_object_ *self, PyObject *args, PyObject *kwds)
{
    fast_pitchbender_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new fast_pitchbender_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing fast_pitchbender");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            fast_pitchbender_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"fast_pitchbender invalid arguments");
    return -1;

}


static const char *fast_pitchbender_set_range_docstring_ = "";

static const char *fast_pitchbender_set_offset_docstring_ = "";

static const char *fast_pitchbender_bend_functor_docstring_ = "";




static PyMethodDef fast_pitchbender_methods_[] = {
    
        { "set_range",(PyCFunction)fast_pitchbender_wrapper_::set_range_method_,METH_VARARGS,(char *)fast_pitchbender_set_range_docstring_ },
    
        { "set_offset",(PyCFunction)fast_pitchbender_wrapper_::set_offset_method_,METH_VARARGS,(char *)fast_pitchbender_set_offset_docstring_ },
    
        { "bend_functor",(PyCFunction)fast_pitchbender_wrapper_::bend_functor_method_,METH_VARARGS,(char *)fast_pitchbender_bend_functor_docstring_ },
    
    
        { "py_lock",(PyCFunction)fast_pitchbender_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)fast_pitchbender_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *fast_pitchbender_docstring_ = "";

PyTypeObject fast_pitchbender_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.fast_pitchbender",/*tp_name*/
    sizeof(fast_pitchbender_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)fast_pitchbender_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)fast_pitchbender_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    fast_pitchbender_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)fast_pitchbender_init_,  /* tp_init */
    0,                         /* tp_alloc */
    fast_pitchbender_new_,             /* tp_new */
};

PyObject *fast_pitchbender_wrapcopy_(fast_pitchbender_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *fast_pitchbender_wrap_(fast_pitchbender_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = fast_pitchbender_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    fast_pitchbender_object_ *self = 
        PyObject_NEW(fast_pitchbender_object_, &fast_pitchbender_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        fast_pitchbender_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

fast_pitchbender_type_ *fast_pitchbender_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &fast_pitchbender_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected fast_pitchbender object");
        return 0;
    }

    fast_pitchbender_type_ *object = static_cast<fast_pitchbender_type_ *>(((fast_pitchbender_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null fast_pitchbender object in unwrap");
        return 0;
    }

    return object;
}

void fast_pitchbender_insert_(const std::pair< fast_pitchbender_type_ *, PyObject *> &p)
{




    fast_pitchbender_wrapped_.insert(p);


}

void fast_pitchbender_erase_(fast_pitchbender_type_ *p)
{




    fast_pitchbender_wrapped_.erase(p);


}

PyObject * fast_pitchbender_find_(fast_pitchbender_type_ *p)
{




    PyObject *ret = 0;
    std::map< fast_pitchbender_type_ *, PyObject *>::iterator where = fast_pitchbender_wrapped_.find(p);
    if(where != fast_pitchbender_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct fast_pitchbender_dispatch_ fast_pitchbender_realdispatcher_ =
{
    &fast_pitchbender_typeobject_,
    fast_pitchbender_wrap_,
    fast_pitchbender_wrapcopy_,
    fast_pitchbender_unwrap_,
    fast_pitchbender_insert_,
    fast_pitchbender_erase_,
    fast_pitchbender_find_
};

  

struct panner_object_ {
    PyObject_HEAD
    panner_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct panner_wrapper_: public panner_type_ 
{

    static void __deleter__(panner_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    panner_wrapper_( PyObject *o, PyInterpreterState *i  ,const f2f_type_ & a0  ,const cookie_type_ & a1  ,clockdomain_ctl_type_ * a2  ): panner_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~panner_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(panner_object_ *self, visitproc visit, void *arg)
    {
        panner_wrapper_ *o = (panner_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(panner_object_ *self)
    {
        panner_wrapper_ *o = (panner_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return panner_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(panner_object_ *self, PyObject *args)
    {
        

        panner_wrapper_ *o = (panner_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null panner c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in panner::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(panner_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(panner_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void panner_delete_(panner_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            panner_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (panner_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        panner_wrapper_::__deleter__((panner_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void panner_dealloc_(panner_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    panner_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * panner_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    panner_object_ *self;

    if(!(self = (panner_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int panner_init_(panner_object_ *self, PyObject *args, PyObject *kwds)
{
    panner_delete_(self);

    

    {
         f2f_type_ * a0;  cookie_type_ * a1;  clockdomain_ctl_type_ * a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_f2f_copy,&a0,fpcvt_cookie_copy,&a1,fpcvt_clockdomain_ctl,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new panner_wrapper_((PyObject *)self,interp ,*a0,*a1,a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing panner");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            panner_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"panner invalid arguments");
    return -1;

}


static const char *panner_cookie_docstring_ = "";




static PyMethodDef panner_methods_[] = {
    
        { "cookie",(PyCFunction)panner_wrapper_::cookie_method_,METH_VARARGS,(char *)panner_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)panner_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)panner_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *panner_docstring_ = "";

PyTypeObject panner_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.panner",/*tp_name*/
    sizeof(panner_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)panner_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)panner_docstring_,       /* tp_doc */
    (traverseproc)panner_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)panner_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    panner_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)panner_init_,  /* tp_init */
    0,                         /* tp_alloc */
    panner_new_,             /* tp_new */
};

PyObject *panner_wrapcopy_(panner_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *panner_wrap_(panner_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = panner_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    panner_object_ *self = 
        
        PyObject_GC_New(panner_object_, &panner_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        panner_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

panner_type_ *panner_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &panner_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected panner object");
        return 0;
    }

    panner_type_ *object = static_cast<panner_type_ *>(((panner_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null panner object in unwrap");
        return 0;
    }

    return object;
}

void panner_insert_(const std::pair< panner_type_ *, PyObject *> &p)
{




    panner_wrapped_.insert(p);


}

void panner_erase_(panner_type_ *p)
{




    panner_wrapped_.erase(p);


}

PyObject * panner_find_(panner_type_ *p)
{




    PyObject *ret = 0;
    std::map< panner_type_ *, PyObject *>::iterator where = panner_wrapped_.find(p);
    if(where != panner_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct panner_dispatch_ panner_realdispatcher_ =
{
    &panner_typeobject_,
    panner_wrap_,
    panner_wrapcopy_,
    panner_unwrap_,
    panner_insert_,
    panner_erase_,
    panner_find_
};

  

struct gain_object_ {
    PyObject_HEAD
    gain_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct gain_wrapper_: public gain_type_ 
{

    static void __deleter__(gain_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    gain_wrapper_( PyObject *o, PyInterpreterState *i  ,const f2f_type_ & a0  ,const cookie_type_ & a1  ,clockdomain_ctl_type_ * a2  ): gain_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~gain_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(gain_object_ *self, visitproc visit, void *arg)
    {
        gain_wrapper_ *o = (gain_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(gain_object_ *self)
    {
        gain_wrapper_ *o = (gain_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return gain_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(gain_object_ *self, PyObject *args)
    {
        

        gain_wrapper_ *o = (gain_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null gain c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in gain::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(gain_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(gain_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void gain_delete_(gain_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            gain_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (gain_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        gain_wrapper_::__deleter__((gain_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void gain_dealloc_(gain_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    gain_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * gain_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    gain_object_ *self;

    if(!(self = (gain_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int gain_init_(gain_object_ *self, PyObject *args, PyObject *kwds)
{
    gain_delete_(self);

    

    {
         f2f_type_ * a0;  cookie_type_ * a1;  clockdomain_ctl_type_ * a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_f2f_copy,&a0,fpcvt_cookie_copy,&a1,fpcvt_clockdomain_ctl,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new gain_wrapper_((PyObject *)self,interp ,*a0,*a1,a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing gain");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            gain_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"gain invalid arguments");
    return -1;

}


static const char *gain_cookie_docstring_ = "";




static PyMethodDef gain_methods_[] = {
    
        { "cookie",(PyCFunction)gain_wrapper_::cookie_method_,METH_VARARGS,(char *)gain_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)gain_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)gain_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *gain_docstring_ = "";

PyTypeObject gain_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.gain",/*tp_name*/
    sizeof(gain_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)gain_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)gain_docstring_,       /* tp_doc */
    (traverseproc)gain_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)gain_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    gain_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)gain_init_,  /* tp_init */
    0,                         /* tp_alloc */
    gain_new_,             /* tp_new */
};

PyObject *gain_wrapcopy_(gain_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *gain_wrap_(gain_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = gain_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    gain_object_ *self = 
        
        PyObject_GC_New(gain_object_, &gain_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        gain_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

gain_type_ *gain_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &gain_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected gain object");
        return 0;
    }

    gain_type_ *object = static_cast<gain_type_ *>(((gain_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null gain object in unwrap");
        return 0;
    }

    return object;
}

void gain_insert_(const std::pair< gain_type_ *, PyObject *> &p)
{




    gain_wrapped_.insert(p);


}

void gain_erase_(gain_type_ *p)
{




    gain_wrapped_.erase(p);


}

PyObject * gain_find_(gain_type_ *p)
{




    PyObject *ret = 0;
    std::map< gain_type_ *, PyObject *>::iterator where = gain_wrapped_.find(p);
    if(where != gain_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct gain_dispatch_ gain_realdispatcher_ =
{
    &gain_typeobject_,
    gain_wrap_,
    gain_wrapcopy_,
    gain_unwrap_,
    gain_insert_,
    gain_erase_,
    gain_find_
};

  

struct delay_object_ {
    PyObject_HEAD
    delay_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct delay_wrapper_: public delay_type_ 
{

    static void __deleter__(delay_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    delay_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): delay_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~delay_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ audio_cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return delay_type_::audio_cookie();
                
            
            
        }

        return
            
                audio_cookie
            
            
                ();
    }

    static PyObject *audio_cookie_method_(delay_object_ *self, PyObject *args)
    {
        

        delay_wrapper_ *o = (delay_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null delay c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->audio_cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in delay::audio_cookie");
        }

        return 0;
    }



    cookie_type_ tap_cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return delay_type_::tap_cookie();
                
            
            
        }

        return
            
                tap_cookie
            
            
                ();
    }

    static PyObject *tap_cookie_method_(delay_object_ *self, PyObject *args)
    {
        

        delay_wrapper_ *o = (delay_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null delay c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->tap_cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in delay::tap_cookie");
        }

        return 0;
    }



    void reset_delay_lines_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return delay_type_::reset_delay_lines();
                
            
            
        }

        return
            
                reset_delay_lines
            
            
                ();
    }

    static PyObject *reset_delay_lines_method_(delay_object_ *self, PyObject *args)
    {
        

        delay_wrapper_ *o = (delay_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null delay c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->reset_delay_lines_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in delay::reset_delay_lines");
        }

        return 0;
    }



    void set_enable_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return delay_type_::set_enable( a0 );
                
            
            
        }

        return
            
                set_enable
            
            
                ( a0 );
    }

    static PyObject *set_enable_method_(delay_object_ *self, PyObject *args)
    {
         bool a0; 

        delay_wrapper_ *o = (delay_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null delay c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_enable_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in delay::set_enable");
        }

        return 0;
    }



    void set_enable_time_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return delay_type_::set_enable_time( a0 );
                
            
            
        }

        return
            
                set_enable_time
            
            
                ( a0 );
    }

    static PyObject *set_enable_time_method_(delay_object_ *self, PyObject *args)
    {
         float a0; 

        delay_wrapper_ *o = (delay_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null delay c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_enable_time_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in delay::set_enable_time");
        }

        return 0;
    }





    static PyObject *_method_lock(delay_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(delay_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void delay_delete_(delay_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            delay_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (delay_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        delay_wrapper_::__deleter__((delay_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void delay_dealloc_(delay_object_* self)
{
    
    delay_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * delay_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    delay_object_ *self;

    if(!(self = (delay_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int delay_init_(delay_object_ *self, PyObject *args, PyObject *kwds)
{
    delay_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new delay_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing delay");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            delay_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"delay invalid arguments");
    return -1;

}


static const char *delay_audio_cookie_docstring_ = "";

static const char *delay_tap_cookie_docstring_ = "";

static const char *delay_reset_delay_lines_docstring_ = "";

static const char *delay_set_enable_docstring_ = "";

static const char *delay_set_enable_time_docstring_ = "";




static PyMethodDef delay_methods_[] = {
    
        { "audio_cookie",(PyCFunction)delay_wrapper_::audio_cookie_method_,METH_VARARGS,(char *)delay_audio_cookie_docstring_ },
    
        { "tap_cookie",(PyCFunction)delay_wrapper_::tap_cookie_method_,METH_VARARGS,(char *)delay_tap_cookie_docstring_ },
    
        { "reset_delay_lines",(PyCFunction)delay_wrapper_::reset_delay_lines_method_,METH_VARARGS,(char *)delay_reset_delay_lines_docstring_ },
    
        { "set_enable",(PyCFunction)delay_wrapper_::set_enable_method_,METH_VARARGS,(char *)delay_set_enable_docstring_ },
    
        { "set_enable_time",(PyCFunction)delay_wrapper_::set_enable_time_method_,METH_VARARGS,(char *)delay_set_enable_time_docstring_ },
    
    
        { "py_lock",(PyCFunction)delay_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)delay_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *delay_docstring_ = "";

PyTypeObject delay_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.delay",/*tp_name*/
    sizeof(delay_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)delay_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)delay_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    delay_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)delay_init_,  /* tp_init */
    0,                         /* tp_alloc */
    delay_new_,             /* tp_new */
};

PyObject *delay_wrapcopy_(delay_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *delay_wrap_(delay_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = delay_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    delay_object_ *self = 
        PyObject_NEW(delay_object_, &delay_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        delay_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

delay_type_ *delay_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &delay_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected delay object");
        return 0;
    }

    delay_type_ *object = static_cast<delay_type_ *>(((delay_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null delay object in unwrap");
        return 0;
    }

    return object;
}

void delay_insert_(const std::pair< delay_type_ *, PyObject *> &p)
{




    delay_wrapped_.insert(p);


}

void delay_erase_(delay_type_ *p)
{




    delay_wrapped_.erase(p);


}

PyObject * delay_find_(delay_type_ *p)
{




    PyObject *ret = 0;
    std::map< delay_type_ *, PyObject *>::iterator where = delay_wrapped_.find(p);
    if(where != delay_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct delay_dispatch_ delay_realdispatcher_ =
{
    &delay_typeobject_,
    delay_wrap_,
    delay_wrapcopy_,
    delay_unwrap_,
    delay_insert_,
    delay_erase_,
    delay_find_
};

  

struct linear_gain_object_ {
    PyObject_HEAD
    linear_gain_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct linear_gain_wrapper_: public linear_gain_type_ 
{

    static void __deleter__(linear_gain_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    linear_gain_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): linear_gain_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~linear_gain_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return linear_gain_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(linear_gain_object_ *self, PyObject *args)
    {
        

        linear_gain_wrapper_ *o = (linear_gain_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null linear_gain c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in linear_gain::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(linear_gain_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(linear_gain_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void linear_gain_delete_(linear_gain_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            linear_gain_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (linear_gain_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        linear_gain_wrapper_::__deleter__((linear_gain_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void linear_gain_dealloc_(linear_gain_object_* self)
{
    
    linear_gain_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * linear_gain_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    linear_gain_object_ *self;

    if(!(self = (linear_gain_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int linear_gain_init_(linear_gain_object_ *self, PyObject *args, PyObject *kwds)
{
    linear_gain_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new linear_gain_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing linear_gain");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            linear_gain_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"linear_gain invalid arguments");
    return -1;

}


static const char *linear_gain_cookie_docstring_ = "";




static PyMethodDef linear_gain_methods_[] = {
    
        { "cookie",(PyCFunction)linear_gain_wrapper_::cookie_method_,METH_VARARGS,(char *)linear_gain_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)linear_gain_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)linear_gain_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *linear_gain_docstring_ = "";

PyTypeObject linear_gain_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.linear_gain",/*tp_name*/
    sizeof(linear_gain_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)linear_gain_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)linear_gain_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    linear_gain_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)linear_gain_init_,  /* tp_init */
    0,                         /* tp_alloc */
    linear_gain_new_,             /* tp_new */
};

PyObject *linear_gain_wrapcopy_(linear_gain_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *linear_gain_wrap_(linear_gain_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = linear_gain_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    linear_gain_object_ *self = 
        PyObject_NEW(linear_gain_object_, &linear_gain_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        linear_gain_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

linear_gain_type_ *linear_gain_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &linear_gain_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected linear_gain object");
        return 0;
    }

    linear_gain_type_ *object = static_cast<linear_gain_type_ *>(((linear_gain_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null linear_gain object in unwrap");
        return 0;
    }

    return object;
}

void linear_gain_insert_(const std::pair< linear_gain_type_ *, PyObject *> &p)
{




    linear_gain_wrapped_.insert(p);


}

void linear_gain_erase_(linear_gain_type_ *p)
{




    linear_gain_wrapped_.erase(p);


}

PyObject * linear_gain_find_(linear_gain_type_ *p)
{




    PyObject *ret = 0;
    std::map< linear_gain_type_ *, PyObject *>::iterator where = linear_gain_wrapped_.find(p);
    if(where != linear_gain_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct linear_gain_dispatch_ linear_gain_realdispatcher_ =
{
    &linear_gain_typeobject_,
    linear_gain_wrap_,
    linear_gain_wrapcopy_,
    linear_gain_unwrap_,
    linear_gain_insert_,
    linear_gain_erase_,
    linear_gain_find_
};

  

struct selector_object_ {
    PyObject_HEAD
    selector_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct selector_wrapper_: public selector_type_ 
{

    static void __deleter__(selector_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    selector_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,const change_nb_type_ & a1  ,unsigned int a2  ,bool a3  ): selector_type_(  a0 ,a1 ,a2 ,a3 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~selector_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(selector_object_ *self, visitproc visit, void *arg)
    {
        selector_wrapper_ *o = (selector_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(selector_object_ *self)
    {
        selector_wrapper_ *o = (selector_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void gate_output_call_(bool owned ,unsigned int a0 ,const change_nb_type_ & a1 ,const change_nb_type_ & a2 )
    {
        if(owned)
        {
            
                
                
                    return selector_type_::gate_output( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                gate_output
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *gate_output_method_(selector_object_ *self, PyObject *args)
    {
         unsigned int a0;  change_nb_type_ * a1;  change_nb_type_ * a2; 

        selector_wrapper_ *o = (selector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null selector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_change_nb_copy,&a1  ,fpcvt_change_nb_copy,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->gate_output_call_ (self->owned ,a0,*a1,*a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in selector::gate_output");
        }

        return 0;
    }



    change_nb_type_ gate_input_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return selector_type_::gate_input( a0 );
                
            
            
        }

        return
            
                gate_input
            
            
                ( a0 );
    }

    static PyObject *gate_input_method_(selector_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        selector_wrapper_ *o = (selector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null selector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->gate_input_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in selector::gate_input");
        }

        return 0;
    }



    change_nb_type_ mode_input_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return selector_type_::mode_input();
                
            
            
        }

        return
            
                mode_input
            
            
                ();
    }

    static PyObject *mode_input_method_(selector_object_ *self, PyObject *args)
    {
        

        selector_wrapper_ *o = (selector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null selector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->mode_input_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in selector::mode_input");
        }

        return 0;
    }



    void clear_output_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return selector_type_::clear_output( a0 );
                
            
            
        }

        return
            
                clear_output
            
            
                ( a0 );
    }

    static PyObject *clear_output_method_(selector_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        selector_wrapper_ *o = (selector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null selector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_output_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in selector::clear_output");
        }

        return 0;
    }



    void choose_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return selector_type_::choose( a0 );
                
            
            
        }

        return
            
                choose
            
            
                ( a0 );
    }

    static PyObject *choose_method_(selector_object_ *self, PyObject *args)
    {
         bool a0; 

        selector_wrapper_ *o = (selector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null selector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->choose_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in selector::choose");
        }

        return 0;
    }



    void activate_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return selector_type_::activate( a0 );
                
            
            
        }

        return
            
                activate
            
            
                ( a0 );
    }

    static PyObject *activate_method_(selector_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        selector_wrapper_ *o = (selector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null selector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->activate_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in selector::activate");
        }

        return 0;
    }



    void select_call_(bool owned ,unsigned int a0 ,bool a1 )
    {
        if(owned)
        {
            
                
                
                    return selector_type_::select( a0  ,a1 );
                
            
            
        }

        return
            
                select
            
            
                ( a0  ,a1 );
    }

    static PyObject *select_method_(selector_object_ *self, PyObject *args)
    {
         unsigned int a0;  bool a1; 

        selector_wrapper_ *o = (selector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null selector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_b,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->select_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in selector::select");
        }

        return 0;
    }





    static PyObject *_method_lock(selector_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(selector_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void selector_delete_(selector_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            selector_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (selector_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        selector_wrapper_::__deleter__((selector_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void selector_dealloc_(selector_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    selector_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * selector_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    selector_object_ *self;

    if(!(self = (selector_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int selector_init_(selector_object_ *self, PyObject *args, PyObject *kwds)
{
    selector_delete_(self);

    

    {
         cookie_type_ * a0;  change_nb_type_ * a1;  unsigned int a2;  bool a3; 

        if(PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_change_nb_copy,&a1,fpcvt_ui,&a2,fpcvt_b,&a3))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new selector_wrapper_((PyObject *)self,interp ,*a0,*a1,a2,a3);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing selector");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            selector_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"selector invalid arguments");
    return -1;

}


static const char *selector_gate_output_docstring_ = "";

static const char *selector_gate_input_docstring_ = "";

static const char *selector_mode_input_docstring_ = "";

static const char *selector_clear_output_docstring_ = "";

static const char *selector_choose_docstring_ = "";

static const char *selector_activate_docstring_ = "";

static const char *selector_select_docstring_ = "";




static PyMethodDef selector_methods_[] = {
    
        { "gate_output",(PyCFunction)selector_wrapper_::gate_output_method_,METH_VARARGS,(char *)selector_gate_output_docstring_ },
    
        { "gate_input",(PyCFunction)selector_wrapper_::gate_input_method_,METH_VARARGS,(char *)selector_gate_input_docstring_ },
    
        { "mode_input",(PyCFunction)selector_wrapper_::mode_input_method_,METH_VARARGS,(char *)selector_mode_input_docstring_ },
    
        { "clear_output",(PyCFunction)selector_wrapper_::clear_output_method_,METH_VARARGS,(char *)selector_clear_output_docstring_ },
    
        { "choose",(PyCFunction)selector_wrapper_::choose_method_,METH_VARARGS,(char *)selector_choose_docstring_ },
    
        { "activate",(PyCFunction)selector_wrapper_::activate_method_,METH_VARARGS,(char *)selector_activate_docstring_ },
    
        { "select",(PyCFunction)selector_wrapper_::select_method_,METH_VARARGS,(char *)selector_select_docstring_ },
    
    
        { "py_lock",(PyCFunction)selector_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)selector_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *selector_docstring_ = "";

PyTypeObject selector_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.selector",/*tp_name*/
    sizeof(selector_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)selector_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)selector_docstring_,       /* tp_doc */
    (traverseproc)selector_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)selector_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    selector_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)selector_init_,  /* tp_init */
    0,                         /* tp_alloc */
    selector_new_,             /* tp_new */
};

PyObject *selector_wrapcopy_(selector_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *selector_wrap_(selector_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = selector_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    selector_object_ *self = 
        
        PyObject_GC_New(selector_object_, &selector_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        selector_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

selector_type_ *selector_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &selector_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected selector object");
        return 0;
    }

    selector_type_ *object = static_cast<selector_type_ *>(((selector_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null selector object in unwrap");
        return 0;
    }

    return object;
}

void selector_insert_(const std::pair< selector_type_ *, PyObject *> &p)
{




    selector_wrapped_.insert(p);


}

void selector_erase_(selector_type_ *p)
{




    selector_wrapped_.erase(p);


}

PyObject * selector_find_(selector_type_ *p)
{




    PyObject *ret = 0;
    std::map< selector_type_ *, PyObject *>::iterator where = selector_wrapped_.find(p);
    if(where != selector_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct selector_dispatch_ selector_realdispatcher_ =
{
    &selector_typeobject_,
    selector_wrap_,
    selector_wrapcopy_,
    selector_unwrap_,
    selector_insert_,
    selector_erase_,
    selector_find_
};

  

struct velocitydetect_object_ {
    PyObject_HEAD
    velocitydetect_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct velocitydetect_wrapper_: public velocitydetect_type_ 
{

    static void __deleter__(velocitydetect_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    velocitydetect_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,unsigned int a1  ,unsigned int a2  ): velocitydetect_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~velocitydetect_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return velocitydetect_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(velocitydetect_object_ *self, PyObject *args)
    {
        

        velocitydetect_wrapper_ *o = (velocitydetect_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null velocitydetect c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in velocitydetect::cookie");
        }

        return 0;
    }



    void set_samples_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return velocitydetect_type_::set_samples( a0 );
                
            
            
        }

        return
            
                set_samples
            
            
                ( a0 );
    }

    static PyObject *set_samples_method_(velocitydetect_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        velocitydetect_wrapper_ *o = (velocitydetect_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null velocitydetect c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_samples_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in velocitydetect::set_samples");
        }

        return 0;
    }



    void set_curve_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return velocitydetect_type_::set_curve( a0 );
                
            
            
        }

        return
            
                set_curve
            
            
                ( a0 );
    }

    static PyObject *set_curve_method_(velocitydetect_object_ *self, PyObject *args)
    {
         float a0; 

        velocitydetect_wrapper_ *o = (velocitydetect_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null velocitydetect c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_curve_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in velocitydetect::set_curve");
        }

        return 0;
    }



    void set_scale_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return velocitydetect_type_::set_scale( a0 );
                
            
            
        }

        return
            
                set_scale
            
            
                ( a0 );
    }

    static PyObject *set_scale_method_(velocitydetect_object_ *self, PyObject *args)
    {
         float a0; 

        velocitydetect_wrapper_ *o = (velocitydetect_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null velocitydetect c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_scale_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in velocitydetect::set_scale");
        }

        return 0;
    }





    static PyObject *_method_lock(velocitydetect_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(velocitydetect_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void velocitydetect_delete_(velocitydetect_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            velocitydetect_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (velocitydetect_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        velocitydetect_wrapper_::__deleter__((velocitydetect_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void velocitydetect_dealloc_(velocitydetect_object_* self)
{
    
    velocitydetect_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * velocitydetect_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    velocitydetect_object_ *self;

    if(!(self = (velocitydetect_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int velocitydetect_init_(velocitydetect_object_ *self, PyObject *args, PyObject *kwds)
{
    velocitydetect_delete_(self);

    

    {
         cookie_type_ * a0;  unsigned int a1;  unsigned int a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_ui,&a1,fpcvt_ui,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new velocitydetect_wrapper_((PyObject *)self,interp ,*a0,a1,a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing velocitydetect");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            velocitydetect_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"velocitydetect invalid arguments");
    return -1;

}


static const char *velocitydetect_cookie_docstring_ = "";

static const char *velocitydetect_set_samples_docstring_ = "";

static const char *velocitydetect_set_curve_docstring_ = "";

static const char *velocitydetect_set_scale_docstring_ = "";




static PyMethodDef velocitydetect_methods_[] = {
    
        { "cookie",(PyCFunction)velocitydetect_wrapper_::cookie_method_,METH_VARARGS,(char *)velocitydetect_cookie_docstring_ },
    
        { "set_samples",(PyCFunction)velocitydetect_wrapper_::set_samples_method_,METH_VARARGS,(char *)velocitydetect_set_samples_docstring_ },
    
        { "set_curve",(PyCFunction)velocitydetect_wrapper_::set_curve_method_,METH_VARARGS,(char *)velocitydetect_set_curve_docstring_ },
    
        { "set_scale",(PyCFunction)velocitydetect_wrapper_::set_scale_method_,METH_VARARGS,(char *)velocitydetect_set_scale_docstring_ },
    
    
        { "py_lock",(PyCFunction)velocitydetect_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)velocitydetect_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *velocitydetect_docstring_ = "";

PyTypeObject velocitydetect_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.velocitydetect",/*tp_name*/
    sizeof(velocitydetect_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)velocitydetect_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)velocitydetect_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    velocitydetect_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)velocitydetect_init_,  /* tp_init */
    0,                         /* tp_alloc */
    velocitydetect_new_,             /* tp_new */
};

PyObject *velocitydetect_wrapcopy_(velocitydetect_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *velocitydetect_wrap_(velocitydetect_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = velocitydetect_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    velocitydetect_object_ *self = 
        PyObject_NEW(velocitydetect_object_, &velocitydetect_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        velocitydetect_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

velocitydetect_type_ *velocitydetect_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &velocitydetect_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected velocitydetect object");
        return 0;
    }

    velocitydetect_type_ *object = static_cast<velocitydetect_type_ *>(((velocitydetect_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null velocitydetect object in unwrap");
        return 0;
    }

    return object;
}

void velocitydetect_insert_(const std::pair< velocitydetect_type_ *, PyObject *> &p)
{




    velocitydetect_wrapped_.insert(p);


}

void velocitydetect_erase_(velocitydetect_type_ *p)
{




    velocitydetect_wrapped_.erase(p);


}

PyObject * velocitydetect_find_(velocitydetect_type_ *p)
{




    PyObject *ret = 0;
    std::map< velocitydetect_type_ *, PyObject *>::iterator where = velocitydetect_wrapped_.find(p);
    if(where != velocitydetect_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct velocitydetect_dispatch_ velocitydetect_realdispatcher_ =
{
    &velocitydetect_typeobject_,
    velocitydetect_wrap_,
    velocitydetect_wrapcopy_,
    velocitydetect_unwrap_,
    velocitydetect_insert_,
    velocitydetect_erase_,
    velocitydetect_find_
};

  

struct scaler_controller_object_ {
    PyObject_HEAD
    scaler_controller_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct scaler_controller_wrapper_: public scaler_controller_type_ 
{

    static void __deleter__(scaler_controller_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    scaler_controller_wrapper_( PyObject *o, PyInterpreterState *i  ): scaler_controller_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~scaler_controller_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scaler_controller_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(scaler_controller_object_ *self, PyObject *args)
    {
        

        scaler_controller_wrapper_ *o = (scaler_controller_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaler_controller c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaler_controller::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(scaler_controller_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(scaler_controller_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void scaler_controller_delete_(scaler_controller_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            scaler_controller_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (scaler_controller_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        scaler_controller_wrapper_::__deleter__((scaler_controller_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void scaler_controller_dealloc_(scaler_controller_object_* self)
{
    
    scaler_controller_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * scaler_controller_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    scaler_controller_object_ *self;

    if(!(self = (scaler_controller_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int scaler_controller_init_(scaler_controller_object_ *self, PyObject *args, PyObject *kwds)
{
    scaler_controller_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new scaler_controller_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing scaler_controller");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            scaler_controller_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"scaler_controller invalid arguments");
    return -1;

}


static const char *scaler_controller_cookie_docstring_ = "";




static PyMethodDef scaler_controller_methods_[] = {
    
        { "cookie",(PyCFunction)scaler_controller_wrapper_::cookie_method_,METH_VARARGS,(char *)scaler_controller_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)scaler_controller_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)scaler_controller_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *scaler_controller_docstring_ = "";

PyTypeObject scaler_controller_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.scaler_controller",/*tp_name*/
    sizeof(scaler_controller_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)scaler_controller_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)scaler_controller_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    scaler_controller_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)scaler_controller_init_,  /* tp_init */
    0,                         /* tp_alloc */
    scaler_controller_new_,             /* tp_new */
};

PyObject *scaler_controller_wrapcopy_(scaler_controller_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *scaler_controller_wrap_(scaler_controller_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = scaler_controller_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    scaler_controller_object_ *self = 
        PyObject_NEW(scaler_controller_object_, &scaler_controller_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        scaler_controller_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

scaler_controller_type_ *scaler_controller_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &scaler_controller_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected scaler_controller object");
        return 0;
    }

    scaler_controller_type_ *object = static_cast<scaler_controller_type_ *>(((scaler_controller_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null scaler_controller object in unwrap");
        return 0;
    }

    return object;
}

void scaler_controller_insert_(const std::pair< scaler_controller_type_ *, PyObject *> &p)
{




    scaler_controller_wrapped_.insert(p);


}

void scaler_controller_erase_(scaler_controller_type_ *p)
{




    scaler_controller_wrapped_.erase(p);


}

PyObject * scaler_controller_find_(scaler_controller_type_ *p)
{




    PyObject *ret = 0;
    std::map< scaler_controller_type_ *, PyObject *>::iterator where = scaler_controller_wrapped_.find(p);
    if(where != scaler_controller_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct scaler_controller_dispatch_ scaler_controller_realdispatcher_ =
{
    &scaler_controller_typeobject_,
    scaler_controller_wrap_,
    scaler_controller_wrapcopy_,
    scaler_controller_unwrap_,
    scaler_controller_insert_,
    scaler_controller_erase_,
    scaler_controller_find_
};

  

struct scaler_object_ {
    PyObject_HEAD
    scaler_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct scaler_wrapper_: public scaler_type_ 
{

    static void __deleter__(scaler_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    scaler_wrapper_( PyObject *o, PyInterpreterState *i  ,scaler_controller_type_ * a0  ,const cookie_type_ & a1  ,const f2f_type_ & a2  ): scaler_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~scaler_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(scaler_object_ *self, visitproc visit, void *arg)
    {
        scaler_wrapper_ *o = (scaler_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(scaler_object_ *self)
    {
        scaler_wrapper_ *o = (scaler_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scaler_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(scaler_object_ *self, PyObject *args)
    {
        

        scaler_wrapper_ *o = (scaler_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaler c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaler::cookie");
        }

        return 0;
    }



    void set_bend_curve_call_(bool owned ,const f2f_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return scaler_type_::set_bend_curve( a0 );
                
            
            
        }

        return
            
                set_bend_curve
            
            
                ( a0 );
    }

    static PyObject *set_bend_curve_method_(scaler_object_ *self, PyObject *args)
    {
         f2f_type_ * a0; 

        scaler_wrapper_ *o = (scaler_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaler c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f2f_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_bend_curve_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaler::set_bend_curve");
        }

        return 0;
    }





    static PyObject *_method_lock(scaler_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(scaler_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void scaler_delete_(scaler_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            scaler_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (scaler_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        scaler_wrapper_::__deleter__((scaler_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void scaler_dealloc_(scaler_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    scaler_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * scaler_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    scaler_object_ *self;

    if(!(self = (scaler_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int scaler_init_(scaler_object_ *self, PyObject *args, PyObject *kwds)
{
    scaler_delete_(self);

    

    {
         scaler_controller_type_ * a0;  cookie_type_ * a1;  f2f_type_ * a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_scaler_controller,&a0,fpcvt_cookie_copy,&a1,fpcvt_f2f_copy,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new scaler_wrapper_((PyObject *)self,interp ,a0,*a1,*a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing scaler");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            scaler_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"scaler invalid arguments");
    return -1;

}


static const char *scaler_cookie_docstring_ = "";

static const char *scaler_set_bend_curve_docstring_ = "";




static PyMethodDef scaler_methods_[] = {
    
        { "cookie",(PyCFunction)scaler_wrapper_::cookie_method_,METH_VARARGS,(char *)scaler_cookie_docstring_ },
    
        { "set_bend_curve",(PyCFunction)scaler_wrapper_::set_bend_curve_method_,METH_VARARGS,(char *)scaler_set_bend_curve_docstring_ },
    
    
        { "py_lock",(PyCFunction)scaler_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)scaler_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *scaler_docstring_ = "";

PyTypeObject scaler_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.scaler",/*tp_name*/
    sizeof(scaler_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)scaler_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)scaler_docstring_,       /* tp_doc */
    (traverseproc)scaler_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)scaler_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    scaler_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)scaler_init_,  /* tp_init */
    0,                         /* tp_alloc */
    scaler_new_,             /* tp_new */
};

PyObject *scaler_wrapcopy_(scaler_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *scaler_wrap_(scaler_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = scaler_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    scaler_object_ *self = 
        
        PyObject_GC_New(scaler_object_, &scaler_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        scaler_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

scaler_type_ *scaler_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &scaler_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected scaler object");
        return 0;
    }

    scaler_type_ *object = static_cast<scaler_type_ *>(((scaler_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null scaler object in unwrap");
        return 0;
    }

    return object;
}

void scaler_insert_(const std::pair< scaler_type_ *, PyObject *> &p)
{




    scaler_wrapped_.insert(p);


}

void scaler_erase_(scaler_type_ *p)
{




    scaler_wrapped_.erase(p);


}

PyObject * scaler_find_(scaler_type_ *p)
{




    PyObject *ret = 0;
    std::map< scaler_type_ *, PyObject *>::iterator where = scaler_wrapped_.find(p);
    if(where != scaler_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct scaler_dispatch_ scaler_realdispatcher_ =
{
    &scaler_typeobject_,
    scaler_wrap_,
    scaler_wrapcopy_,
    scaler_unwrap_,
    scaler_insert_,
    scaler_erase_,
    scaler_find_
};

  

struct evtdump_object_ {
    PyObject_HEAD
    evtdump_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct evtdump_wrapper_: public evtdump_type_ 
{

    static void __deleter__(evtdump_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    evtdump_wrapper_( PyObject *o, PyInterpreterState *i  ,const std::string & a0  ,const cookie_type_ & a1  ,clockdomain_ctl_type_ * a2  ): evtdump_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~evtdump_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return evtdump_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(evtdump_object_ *self, PyObject *args)
    {
        

        evtdump_wrapper_ *o = (evtdump_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null evtdump c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in evtdump::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(evtdump_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(evtdump_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void evtdump_delete_(evtdump_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            evtdump_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (evtdump_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        evtdump_wrapper_::__deleter__((evtdump_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void evtdump_dealloc_(evtdump_object_* self)
{
    
    evtdump_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * evtdump_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    evtdump_object_ *self;

    if(!(self = (evtdump_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int evtdump_init_(evtdump_object_ *self, PyObject *args, PyObject *kwds)
{
    evtdump_delete_(self);

    

    {
         std::string a0;  cookie_type_ * a1;  clockdomain_ctl_type_ * a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_stdstr,&a0,fpcvt_cookie_copy,&a1,fpcvt_clockdomain_ctl,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new evtdump_wrapper_((PyObject *)self,interp ,a0,*a1,a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing evtdump");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            evtdump_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"evtdump invalid arguments");
    return -1;

}


static const char *evtdump_cookie_docstring_ = "";




static PyMethodDef evtdump_methods_[] = {
    
        { "cookie",(PyCFunction)evtdump_wrapper_::cookie_method_,METH_VARARGS,(char *)evtdump_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)evtdump_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)evtdump_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *evtdump_docstring_ = "";

PyTypeObject evtdump_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.evtdump",/*tp_name*/
    sizeof(evtdump_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)evtdump_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)evtdump_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    evtdump_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)evtdump_init_,  /* tp_init */
    0,                         /* tp_alloc */
    evtdump_new_,             /* tp_new */
};

PyObject *evtdump_wrapcopy_(evtdump_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *evtdump_wrap_(evtdump_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = evtdump_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    evtdump_object_ *self = 
        PyObject_NEW(evtdump_object_, &evtdump_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        evtdump_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

evtdump_type_ *evtdump_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &evtdump_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected evtdump object");
        return 0;
    }

    evtdump_type_ *object = static_cast<evtdump_type_ *>(((evtdump_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null evtdump object in unwrap");
        return 0;
    }

    return object;
}

void evtdump_insert_(const std::pair< evtdump_type_ *, PyObject *> &p)
{




    evtdump_wrapped_.insert(p);


}

void evtdump_erase_(evtdump_type_ *p)
{




    evtdump_wrapped_.erase(p);


}

PyObject * evtdump_find_(evtdump_type_ *p)
{




    PyObject *ret = 0;
    std::map< evtdump_type_ *, PyObject *>::iterator where = evtdump_wrapped_.find(p);
    if(where != evtdump_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct evtdump_dispatch_ evtdump_realdispatcher_ =
{
    &evtdump_typeobject_,
    evtdump_wrap_,
    evtdump_wrapcopy_,
    evtdump_unwrap_,
    evtdump_insert_,
    evtdump_erase_,
    evtdump_find_
};

  

struct wavrecorder_object_ {
    PyObject_HEAD
    wavrecorder_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct wavrecorder_wrapper_: public wavrecorder_type_ 
{

    static void __deleter__(wavrecorder_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    wavrecorder_wrapper_( PyObject *o, PyInterpreterState *i  ,clockdomain_ctl_type_ * a0  ): wavrecorder_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~wavrecorder_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return wavrecorder_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(wavrecorder_object_ *self, PyObject *args)
    {
        

        wavrecorder_wrapper_ *o = (wavrecorder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null wavrecorder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in wavrecorder::cookie");
        }

        return 0;
    }



    void setfile_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                
                    return wavrecorder_type_::setfile( a0 );
                
            
            
        }

        return
            
                setfile
            
            
                ( a0 );
    }

    static PyObject *setfile_method_(wavrecorder_object_ *self, PyObject *args)
    {
         const char * a0; 

        wavrecorder_wrapper_ *o = (wavrecorder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null wavrecorder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->setfile_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in wavrecorder::setfile");
        }

        return 0;
    }



    change_nb_type_ record_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return wavrecorder_type_::record();
                
            
            
        }

        return
            
                record
            
            
                ();
    }

    static PyObject *record_method_(wavrecorder_object_ *self, PyObject *args)
    {
        

        wavrecorder_wrapper_ *o = (wavrecorder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null wavrecorder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->record_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in wavrecorder::record");
        }

        return 0;
    }





    static PyObject *_method_lock(wavrecorder_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(wavrecorder_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void wavrecorder_delete_(wavrecorder_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            wavrecorder_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (wavrecorder_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        wavrecorder_wrapper_::__deleter__((wavrecorder_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void wavrecorder_dealloc_(wavrecorder_object_* self)
{
    
    wavrecorder_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * wavrecorder_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    wavrecorder_object_ *self;

    if(!(self = (wavrecorder_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int wavrecorder_init_(wavrecorder_object_ *self, PyObject *args, PyObject *kwds)
{
    wavrecorder_delete_(self);

    

    {
         clockdomain_ctl_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_clockdomain_ctl,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new wavrecorder_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing wavrecorder");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            wavrecorder_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"wavrecorder invalid arguments");
    return -1;

}


static const char *wavrecorder_cookie_docstring_ = "";

static const char *wavrecorder_setfile_docstring_ = "";

static const char *wavrecorder_record_docstring_ = "";




static PyMethodDef wavrecorder_methods_[] = {
    
        { "cookie",(PyCFunction)wavrecorder_wrapper_::cookie_method_,METH_VARARGS,(char *)wavrecorder_cookie_docstring_ },
    
        { "setfile",(PyCFunction)wavrecorder_wrapper_::setfile_method_,METH_VARARGS,(char *)wavrecorder_setfile_docstring_ },
    
        { "record",(PyCFunction)wavrecorder_wrapper_::record_method_,METH_VARARGS,(char *)wavrecorder_record_docstring_ },
    
    
        { "py_lock",(PyCFunction)wavrecorder_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)wavrecorder_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *wavrecorder_docstring_ = "";

PyTypeObject wavrecorder_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.wavrecorder",/*tp_name*/
    sizeof(wavrecorder_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)wavrecorder_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)wavrecorder_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    wavrecorder_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)wavrecorder_init_,  /* tp_init */
    0,                         /* tp_alloc */
    wavrecorder_new_,             /* tp_new */
};

PyObject *wavrecorder_wrapcopy_(wavrecorder_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *wavrecorder_wrap_(wavrecorder_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = wavrecorder_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    wavrecorder_object_ *self = 
        PyObject_NEW(wavrecorder_object_, &wavrecorder_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        wavrecorder_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

wavrecorder_type_ *wavrecorder_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &wavrecorder_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected wavrecorder object");
        return 0;
    }

    wavrecorder_type_ *object = static_cast<wavrecorder_type_ *>(((wavrecorder_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null wavrecorder object in unwrap");
        return 0;
    }

    return object;
}

void wavrecorder_insert_(const std::pair< wavrecorder_type_ *, PyObject *> &p)
{




    wavrecorder_wrapped_.insert(p);


}

void wavrecorder_erase_(wavrecorder_type_ *p)
{




    wavrecorder_wrapped_.erase(p);


}

PyObject * wavrecorder_find_(wavrecorder_type_ *p)
{




    PyObject *ret = 0;
    std::map< wavrecorder_type_ *, PyObject *>::iterator where = wavrecorder_wrapped_.find(p);
    if(where != wavrecorder_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct wavrecorder_dispatch_ wavrecorder_realdispatcher_ =
{
    &wavrecorder_typeobject_,
    wavrecorder_wrap_,
    wavrecorder_wrapcopy_,
    wavrecorder_unwrap_,
    wavrecorder_insert_,
    wavrecorder_erase_,
    wavrecorder_find_
};

  

struct python_delegate_object_ {
    PyObject_HEAD
    python_delegate_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct python_delegate_wrapper_: public python_delegate_type_ 
{

    static void __deleter__(python_delegate_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
        cache_4_ = 0;
    
        cache_5_ = 0;
    
        cache_6_ = 0;
    
        cache_7_ = 0;
    
        cache_8_ = 0;
    
        cache_9_ = 0;
    
        cache_10_ = 0;
    
        cache_11_ = 0;
    
        cache_12_ = 0;
    
        cache_13_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
        if(cache_4_)
        {
            Py_DECREF(cache_4_);
            cache_4_ = 0;
        }
    
        if(cache_5_)
        {
            Py_DECREF(cache_5_);
            cache_5_ = 0;
        }
    
        if(cache_6_)
        {
            Py_DECREF(cache_6_);
            cache_6_ = 0;
        }
    
        if(cache_7_)
        {
            Py_DECREF(cache_7_);
            cache_7_ = 0;
        }
    
        if(cache_8_)
        {
            Py_DECREF(cache_8_);
            cache_8_ = 0;
        }
    
        if(cache_9_)
        {
            Py_DECREF(cache_9_);
            cache_9_ = 0;
        }
    
        if(cache_10_)
        {
            Py_DECREF(cache_10_);
            cache_10_ = 0;
        }
    
        if(cache_11_)
        {
            Py_DECREF(cache_11_);
            cache_11_ = 0;
        }
    
        if(cache_12_)
        {
            Py_DECREF(cache_12_);
            cache_12_ = 0;
        }
    
        if(cache_13_)
        {
            Py_DECREF(cache_13_);
            cache_13_ = 0;
        }
    
    }
*/



    python_delegate_wrapper_( PyObject *o, PyInterpreterState *i  ): python_delegate_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~python_delegate_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void py_destroy_any(  PyObject * a0  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_destroy_any");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_py,&a0);
        //printf("py_destroy_any before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_destroy_any after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    PyObject * py_make_term(  PyObject * a0  ,PyObject * a1  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_make_term");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&)" ,tpcvt_py,&a0,tpcvt_py,&a1);
        //printf("py_make_term before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_make_term after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            PyObject * rptr;

            if(!fpcvt_py(result,&rptr))
            {
                throw pip_err_t("wrong type result converting py");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        PyObject * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_2_;
 
    PyObject * py_make_list(  PyObject * a0  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_make_list");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_py,&a0);
        //printf("py_make_list before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_make_list after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            PyObject * rptr;

            if(!fpcvt_py(result,&rptr))
            {
                throw pip_err_t("wrong type result converting py");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        PyObject * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_3_;
 
    PyObject * py_make_collection(  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_make_collection");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("py_make_collection before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_make_collection after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            PyObject * rptr;

            if(!fpcvt_py(result,&rptr))
            {
                throw pip_err_t("wrong type result converting py");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        PyObject * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_4_;
 
    PyObject * py_make_split(  PyObject * a0  ,PyObject * a1  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_4_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_make_split");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_4_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&)" ,tpcvt_py,&a0,tpcvt_py,&a1);
        //printf("py_make_split before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_make_split after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            PyObject * rptr;

            if(!fpcvt_py(result,&rptr))
            {
                throw pip_err_t("wrong type result converting py");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        PyObject * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_5_;
 
    void py_add_arg(  PyObject * a0  ,PyObject * a1  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_5_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_add_arg");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_5_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&)" ,tpcvt_py,&a0,tpcvt_py,&a1);
        //printf("py_add_arg before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_add_arg after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_6_;
 
    PyObject * py_make_long(  long a0  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_6_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_make_long");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_6_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_l,&a0);
        //printf("py_make_long before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_make_long after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            PyObject * rptr;

            if(!fpcvt_py(result,&rptr))
            {
                throw pip_err_t("wrong type result converting py");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        PyObject * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_7_;
 
    PyObject * py_make_float(  float a0  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_7_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_make_float");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_7_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_f,&a0);
        //printf("py_make_float before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_make_float after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            PyObject * rptr;

            if(!fpcvt_py(result,&rptr))
            {
                throw pip_err_t("wrong type result converting py");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        PyObject * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_8_;
 
    PyObject * py_make_bool(  bool a0  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_8_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_make_bool");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_8_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_b,&a0);
        //printf("py_make_bool before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_make_bool after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            PyObject * rptr;

            if(!fpcvt_py(result,&rptr))
            {
                throw pip_err_t("wrong type result converting py");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        PyObject * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_9_;
 
    PyObject * py_make_none(  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_9_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_make_none");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_9_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("py_make_none before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_make_none after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            PyObject * rptr;

            if(!fpcvt_py(result,&rptr))
            {
                throw pip_err_t("wrong type result converting py");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        PyObject * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_10_;
 
    PyObject * py_make_string(  PyObject * a0  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_10_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_make_string");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_10_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_py,&a0);
        //printf("py_make_string before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_make_string after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            PyObject * rptr;

            if(!fpcvt_py(result,&rptr))
            {
                throw pip_err_t("wrong type result converting py");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        PyObject * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_11_;
 
    PyObject * py_make_variable(  PyObject * a0  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_11_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_make_variable");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_11_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_py,&a0);
        //printf("py_make_variable before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_make_variable after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            PyObject * rptr;

            if(!fpcvt_py(result,&rptr))
            {
                throw pip_err_t("wrong type result converting py");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        PyObject * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_12_;
 
    PyObject * py_make_subst1(  PyObject * a0  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_12_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_make_subst1");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_12_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_py,&a0);
        //printf("py_make_subst1 before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_make_subst1 after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            PyObject * rptr;

            if(!fpcvt_py(result,&rptr))
            {
                throw pip_err_t("wrong type result converting py");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        PyObject * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_13_;
 
    PyObject * py_make_subst2(  PyObject * a0  ,PyObject * a1  )
    {
        lock_c2p lock(_interp,true);

        //PyObject *method = cache_13_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"py_make_subst2");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_13_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&)" ,tpcvt_py,&a0,tpcvt_py,&a1);
        //printf("py_make_subst2 before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("py_make_subst2 after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            PyObject * rptr;

            if(!fpcvt_py(result,&rptr))
            {
                throw pip_err_t("wrong type result converting py");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        PyObject * rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }

















    void py_destroy_any_call_(bool owned ,PyObject * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_destroy_any");
                
                
            
            
        }

        return
            
                py_destroy_any
            
            
                ( a0 );
    }

    static PyObject *py_destroy_any_method_(python_delegate_object_ *self, PyObject *args)
    {
         PyObject * a0; 

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_py,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock (true);
                o->py_destroy_any_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_destroy_any");
        }

        return 0;
    }



    PyObject * py_make_term_call_(bool owned ,PyObject * a0 ,PyObject * a1 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_make_term");
                
                
            
            
        }

        return
            
                py_make_term
            
            
                ( a0  ,a1 );
    }

    static PyObject *py_make_term_method_(python_delegate_object_ *self, PyObject *args)
    {
         PyObject * a0;  PyObject * a1; 

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_py,&a0  ,fpcvt_py,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    PyObject * r = o->py_make_term_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_make_term");
        }

        return 0;
    }



    PyObject * py_make_list_call_(bool owned ,PyObject * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_make_list");
                
                
            
            
        }

        return
            
                py_make_list
            
            
                ( a0 );
    }

    static PyObject *py_make_list_method_(python_delegate_object_ *self, PyObject *args)
    {
         PyObject * a0; 

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_py,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    PyObject * r = o->py_make_list_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_make_list");
        }

        return 0;
    }



    PyObject * py_make_collection_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_make_collection");
                
                
            
            
        }

        return
            
                py_make_collection
            
            
                ();
    }

    static PyObject *py_make_collection_method_(python_delegate_object_ *self, PyObject *args)
    {
        

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    PyObject * r = o->py_make_collection_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_make_collection");
        }

        return 0;
    }



    PyObject * py_make_split_call_(bool owned ,PyObject * a0 ,PyObject * a1 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_make_split");
                
                
            
            
        }

        return
            
                py_make_split
            
            
                ( a0  ,a1 );
    }

    static PyObject *py_make_split_method_(python_delegate_object_ *self, PyObject *args)
    {
         PyObject * a0;  PyObject * a1; 

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_py,&a0  ,fpcvt_py,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    PyObject * r = o->py_make_split_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_make_split");
        }

        return 0;
    }



    void py_add_arg_call_(bool owned ,PyObject * a0 ,PyObject * a1 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_add_arg");
                
                
            
            
        }

        return
            
                py_add_arg
            
            
                ( a0  ,a1 );
    }

    static PyObject *py_add_arg_method_(python_delegate_object_ *self, PyObject *args)
    {
         PyObject * a0;  PyObject * a1; 

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_py,&a0  ,fpcvt_py,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock (true);
                o->py_add_arg_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_add_arg");
        }

        return 0;
    }



    PyObject * py_make_long_call_(bool owned ,long a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_make_long");
                
                
            
            
        }

        return
            
                py_make_long
            
            
                ( a0 );
    }

    static PyObject *py_make_long_method_(python_delegate_object_ *self, PyObject *args)
    {
         long a0; 

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_l,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    PyObject * r = o->py_make_long_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_make_long");
        }

        return 0;
    }



    PyObject * py_make_float_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_make_float");
                
                
            
            
        }

        return
            
                py_make_float
            
            
                ( a0 );
    }

    static PyObject *py_make_float_method_(python_delegate_object_ *self, PyObject *args)
    {
         float a0; 

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    PyObject * r = o->py_make_float_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_make_float");
        }

        return 0;
    }



    PyObject * py_make_bool_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_make_bool");
                
                
            
            
        }

        return
            
                py_make_bool
            
            
                ( a0 );
    }

    static PyObject *py_make_bool_method_(python_delegate_object_ *self, PyObject *args)
    {
         bool a0; 

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    PyObject * r = o->py_make_bool_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_make_bool");
        }

        return 0;
    }



    PyObject * py_make_none_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_make_none");
                
                
            
            
        }

        return
            
                py_make_none
            
            
                ();
    }

    static PyObject *py_make_none_method_(python_delegate_object_ *self, PyObject *args)
    {
        

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    PyObject * r = o->py_make_none_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_make_none");
        }

        return 0;
    }



    PyObject * py_make_string_call_(bool owned ,PyObject * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_make_string");
                
                
            
            
        }

        return
            
                py_make_string
            
            
                ( a0 );
    }

    static PyObject *py_make_string_method_(python_delegate_object_ *self, PyObject *args)
    {
         PyObject * a0; 

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_py,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    PyObject * r = o->py_make_string_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_make_string");
        }

        return 0;
    }



    PyObject * py_make_variable_call_(bool owned ,PyObject * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_make_variable");
                
                
            
            
        }

        return
            
                py_make_variable
            
            
                ( a0 );
    }

    static PyObject *py_make_variable_method_(python_delegate_object_ *self, PyObject *args)
    {
         PyObject * a0; 

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_py,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    PyObject * r = o->py_make_variable_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_make_variable");
        }

        return 0;
    }



    PyObject * py_make_subst1_call_(bool owned ,PyObject * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_make_subst1");
                
                
            
            
        }

        return
            
                py_make_subst1
            
            
                ( a0 );
    }

    static PyObject *py_make_subst1_method_(python_delegate_object_ *self, PyObject *args)
    {
         PyObject * a0; 

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_py,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    PyObject * r = o->py_make_subst1_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_make_subst1");
        }

        return 0;
    }



    PyObject * py_make_subst2_call_(bool owned ,PyObject * a0 ,PyObject * a1 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method python_delegate::py_make_subst2");
                
                
            
            
        }

        return
            
                py_make_subst2
            
            
                ( a0  ,a1 );
    }

    static PyObject *py_make_subst2_method_(python_delegate_object_ *self, PyObject *args)
    {
         PyObject * a0;  PyObject * a1; 

        python_delegate_wrapper_ *o = (python_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null python_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_py,&a0  ,fpcvt_py,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    PyObject * r = o->py_make_subst2_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_delegate::py_make_subst2");
        }

        return 0;
    }





    static PyObject *_method_lock(python_delegate_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(python_delegate_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void python_delegate_delete_(python_delegate_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            python_delegate_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (python_delegate_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        python_delegate_wrapper_::__deleter__((python_delegate_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void python_delegate_dealloc_(python_delegate_object_* self)
{
    
    python_delegate_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * python_delegate_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    python_delegate_object_ *self;

    if(!(self = (python_delegate_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int python_delegate_init_(python_delegate_object_ *self, PyObject *args, PyObject *kwds)
{
    python_delegate_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new python_delegate_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing python_delegate");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            python_delegate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"python_delegate invalid arguments");
    return -1;

}


static const char *python_delegate_py_destroy_any_docstring_ = "";

static const char *python_delegate_py_make_term_docstring_ = "";

static const char *python_delegate_py_make_list_docstring_ = "";

static const char *python_delegate_py_make_collection_docstring_ = "";

static const char *python_delegate_py_make_split_docstring_ = "";

static const char *python_delegate_py_add_arg_docstring_ = "";

static const char *python_delegate_py_make_long_docstring_ = "";

static const char *python_delegate_py_make_float_docstring_ = "";

static const char *python_delegate_py_make_bool_docstring_ = "";

static const char *python_delegate_py_make_none_docstring_ = "";

static const char *python_delegate_py_make_string_docstring_ = "";

static const char *python_delegate_py_make_variable_docstring_ = "";

static const char *python_delegate_py_make_subst1_docstring_ = "";

static const char *python_delegate_py_make_subst2_docstring_ = "";




static PyMethodDef python_delegate_methods_[] = {
    
        { "py_destroy_any",(PyCFunction)python_delegate_wrapper_::py_destroy_any_method_,METH_VARARGS,(char *)python_delegate_py_destroy_any_docstring_ },
    
        { "py_make_term",(PyCFunction)python_delegate_wrapper_::py_make_term_method_,METH_VARARGS,(char *)python_delegate_py_make_term_docstring_ },
    
        { "py_make_list",(PyCFunction)python_delegate_wrapper_::py_make_list_method_,METH_VARARGS,(char *)python_delegate_py_make_list_docstring_ },
    
        { "py_make_collection",(PyCFunction)python_delegate_wrapper_::py_make_collection_method_,METH_VARARGS,(char *)python_delegate_py_make_collection_docstring_ },
    
        { "py_make_split",(PyCFunction)python_delegate_wrapper_::py_make_split_method_,METH_VARARGS,(char *)python_delegate_py_make_split_docstring_ },
    
        { "py_add_arg",(PyCFunction)python_delegate_wrapper_::py_add_arg_method_,METH_VARARGS,(char *)python_delegate_py_add_arg_docstring_ },
    
        { "py_make_long",(PyCFunction)python_delegate_wrapper_::py_make_long_method_,METH_VARARGS,(char *)python_delegate_py_make_long_docstring_ },
    
        { "py_make_float",(PyCFunction)python_delegate_wrapper_::py_make_float_method_,METH_VARARGS,(char *)python_delegate_py_make_float_docstring_ },
    
        { "py_make_bool",(PyCFunction)python_delegate_wrapper_::py_make_bool_method_,METH_VARARGS,(char *)python_delegate_py_make_bool_docstring_ },
    
        { "py_make_none",(PyCFunction)python_delegate_wrapper_::py_make_none_method_,METH_VARARGS,(char *)python_delegate_py_make_none_docstring_ },
    
        { "py_make_string",(PyCFunction)python_delegate_wrapper_::py_make_string_method_,METH_VARARGS,(char *)python_delegate_py_make_string_docstring_ },
    
        { "py_make_variable",(PyCFunction)python_delegate_wrapper_::py_make_variable_method_,METH_VARARGS,(char *)python_delegate_py_make_variable_docstring_ },
    
        { "py_make_subst1",(PyCFunction)python_delegate_wrapper_::py_make_subst1_method_,METH_VARARGS,(char *)python_delegate_py_make_subst1_docstring_ },
    
        { "py_make_subst2",(PyCFunction)python_delegate_wrapper_::py_make_subst2_method_,METH_VARARGS,(char *)python_delegate_py_make_subst2_docstring_ },
    
    
        { "py_lock",(PyCFunction)python_delegate_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)python_delegate_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *python_delegate_docstring_ = "";

PyTypeObject python_delegate_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.python_delegate",/*tp_name*/
    sizeof(python_delegate_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)python_delegate_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)python_delegate_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    python_delegate_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)python_delegate_init_,  /* tp_init */
    0,                         /* tp_alloc */
    python_delegate_new_,             /* tp_new */
};

PyObject *python_delegate_wrapcopy_(python_delegate_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *python_delegate_wrap_(python_delegate_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = python_delegate_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    python_delegate_object_ *self = 
        PyObject_NEW(python_delegate_object_, &python_delegate_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        python_delegate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

python_delegate_type_ *python_delegate_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &python_delegate_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected python_delegate object");
        return 0;
    }

    python_delegate_type_ *object = static_cast<python_delegate_type_ *>(((python_delegate_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null python_delegate object in unwrap");
        return 0;
    }

    return object;
}

void python_delegate_insert_(const std::pair< python_delegate_type_ *, PyObject *> &p)
{




    python_delegate_wrapped_.insert(p);


}

void python_delegate_erase_(python_delegate_type_ *p)
{




    python_delegate_wrapped_.erase(p);


}

PyObject * python_delegate_find_(python_delegate_type_ *p)
{




    PyObject *ret = 0;
    std::map< python_delegate_type_ *, PyObject *>::iterator where = python_delegate_wrapped_.find(p);
    if(where != python_delegate_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct python_delegate_dispatch_ python_delegate_realdispatcher_ =
{
    &python_delegate_typeobject_,
    python_delegate_wrap_,
    python_delegate_wrapcopy_,
    python_delegate_unwrap_,
    python_delegate_insert_,
    python_delegate_erase_,
    python_delegate_find_
};

  

struct midi_channel_delegate_object_ {
    PyObject_HEAD
    midi_channel_delegate_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct midi_channel_delegate_wrapper_: public midi_channel_delegate_type_ 
{

    static void __deleter__(midi_channel_delegate_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
        cache_4_ = 0;
    
        cache_5_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
        if(cache_4_)
        {
            Py_DECREF(cache_4_);
            cache_4_ = 0;
        }
    
        if(cache_5_)
        {
            Py_DECREF(cache_5_);
            cache_5_ = 0;
        }
    
    }
*/



    midi_channel_delegate_wrapper_( PyObject *o, PyInterpreterState *i  ): midi_channel_delegate_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~midi_channel_delegate_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void set_midi_channel(  unsigned int a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"set_midi_channel");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_ui,&a0);
        //printf("set_midi_channel before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("set_midi_channel after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void set_min_channel(  unsigned int a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"set_min_channel");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_ui,&a0);
        //printf("set_min_channel before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("set_min_channel after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void set_max_channel(  unsigned int a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"set_max_channel");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_ui,&a0);
        //printf("set_max_channel before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("set_max_channel after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    unsigned int get_midi_channel(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_midi_channel");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("get_midi_channel before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_midi_channel after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            unsigned int rptr;

            if(!fpcvt_ui(result,&rptr))
            {
                throw pip_err_t("wrong type result converting ui");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        unsigned int rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_4_;
 
    unsigned int get_min_channel(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_4_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_min_channel");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_4_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("get_min_channel before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_min_channel after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            unsigned int rptr;

            if(!fpcvt_ui(result,&rptr))
            {
                throw pip_err_t("wrong type result converting ui");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        unsigned int rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_5_;
 
    unsigned int get_max_channel(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_5_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_max_channel");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_5_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("get_max_channel before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_max_channel after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            unsigned int rptr;

            if(!fpcvt_ui(result,&rptr))
            {
                throw pip_err_t("wrong type result converting ui");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        unsigned int rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }

















    void set_midi_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method midi_channel_delegate::set_midi_channel");
                
                
            
            
        }

        return
            
                set_midi_channel
            
            
                ( a0 );
    }

    static PyObject *set_midi_channel_method_(midi_channel_delegate_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_channel_delegate_wrapper_ *o = (midi_channel_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_channel_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_midi_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_channel_delegate::set_midi_channel");
        }

        return 0;
    }



    void set_min_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method midi_channel_delegate::set_min_channel");
                
                
            
            
        }

        return
            
                set_min_channel
            
            
                ( a0 );
    }

    static PyObject *set_min_channel_method_(midi_channel_delegate_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_channel_delegate_wrapper_ *o = (midi_channel_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_channel_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_min_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_channel_delegate::set_min_channel");
        }

        return 0;
    }



    void set_max_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method midi_channel_delegate::set_max_channel");
                
                
            
            
        }

        return
            
                set_max_channel
            
            
                ( a0 );
    }

    static PyObject *set_max_channel_method_(midi_channel_delegate_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_channel_delegate_wrapper_ *o = (midi_channel_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_channel_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_max_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_channel_delegate::set_max_channel");
        }

        return 0;
    }



    unsigned int get_midi_channel_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method midi_channel_delegate::get_midi_channel");
                
                
            
            
        }

        return
            
                get_midi_channel
            
            
                ();
    }

    static PyObject *get_midi_channel_method_(midi_channel_delegate_object_ *self, PyObject *args)
    {
        

        midi_channel_delegate_wrapper_ *o = (midi_channel_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_channel_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_midi_channel_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_channel_delegate::get_midi_channel");
        }

        return 0;
    }



    unsigned int get_min_channel_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method midi_channel_delegate::get_min_channel");
                
                
            
            
        }

        return
            
                get_min_channel
            
            
                ();
    }

    static PyObject *get_min_channel_method_(midi_channel_delegate_object_ *self, PyObject *args)
    {
        

        midi_channel_delegate_wrapper_ *o = (midi_channel_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_channel_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_min_channel_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_channel_delegate::get_min_channel");
        }

        return 0;
    }



    unsigned int get_max_channel_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method midi_channel_delegate::get_max_channel");
                
                
            
            
        }

        return
            
                get_max_channel
            
            
                ();
    }

    static PyObject *get_max_channel_method_(midi_channel_delegate_object_ *self, PyObject *args)
    {
        

        midi_channel_delegate_wrapper_ *o = (midi_channel_delegate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_channel_delegate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_max_channel_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_channel_delegate::get_max_channel");
        }

        return 0;
    }





    static PyObject *_method_lock(midi_channel_delegate_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(midi_channel_delegate_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void midi_channel_delegate_delete_(midi_channel_delegate_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            midi_channel_delegate_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (midi_channel_delegate_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        midi_channel_delegate_wrapper_::__deleter__((midi_channel_delegate_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void midi_channel_delegate_dealloc_(midi_channel_delegate_object_* self)
{
    
    midi_channel_delegate_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * midi_channel_delegate_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    midi_channel_delegate_object_ *self;

    if(!(self = (midi_channel_delegate_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int midi_channel_delegate_init_(midi_channel_delegate_object_ *self, PyObject *args, PyObject *kwds)
{
    midi_channel_delegate_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new midi_channel_delegate_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing midi_channel_delegate");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            midi_channel_delegate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"midi_channel_delegate invalid arguments");
    return -1;

}


static const char *midi_channel_delegate_set_midi_channel_docstring_ = "";

static const char *midi_channel_delegate_set_min_channel_docstring_ = "";

static const char *midi_channel_delegate_set_max_channel_docstring_ = "";

static const char *midi_channel_delegate_get_midi_channel_docstring_ = "";

static const char *midi_channel_delegate_get_min_channel_docstring_ = "";

static const char *midi_channel_delegate_get_max_channel_docstring_ = "";




static PyMethodDef midi_channel_delegate_methods_[] = {
    
        { "set_midi_channel",(PyCFunction)midi_channel_delegate_wrapper_::set_midi_channel_method_,METH_VARARGS,(char *)midi_channel_delegate_set_midi_channel_docstring_ },
    
        { "set_min_channel",(PyCFunction)midi_channel_delegate_wrapper_::set_min_channel_method_,METH_VARARGS,(char *)midi_channel_delegate_set_min_channel_docstring_ },
    
        { "set_max_channel",(PyCFunction)midi_channel_delegate_wrapper_::set_max_channel_method_,METH_VARARGS,(char *)midi_channel_delegate_set_max_channel_docstring_ },
    
        { "get_midi_channel",(PyCFunction)midi_channel_delegate_wrapper_::get_midi_channel_method_,METH_VARARGS,(char *)midi_channel_delegate_get_midi_channel_docstring_ },
    
        { "get_min_channel",(PyCFunction)midi_channel_delegate_wrapper_::get_min_channel_method_,METH_VARARGS,(char *)midi_channel_delegate_get_min_channel_docstring_ },
    
        { "get_max_channel",(PyCFunction)midi_channel_delegate_wrapper_::get_max_channel_method_,METH_VARARGS,(char *)midi_channel_delegate_get_max_channel_docstring_ },
    
    
        { "py_lock",(PyCFunction)midi_channel_delegate_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)midi_channel_delegate_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *midi_channel_delegate_docstring_ = "";

PyTypeObject midi_channel_delegate_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.midi_channel_delegate",/*tp_name*/
    sizeof(midi_channel_delegate_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)midi_channel_delegate_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)midi_channel_delegate_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    midi_channel_delegate_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)midi_channel_delegate_init_,  /* tp_init */
    0,                         /* tp_alloc */
    midi_channel_delegate_new_,             /* tp_new */
};

PyObject *midi_channel_delegate_wrapcopy_(midi_channel_delegate_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *midi_channel_delegate_wrap_(midi_channel_delegate_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = midi_channel_delegate_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    midi_channel_delegate_object_ *self = 
        PyObject_NEW(midi_channel_delegate_object_, &midi_channel_delegate_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        midi_channel_delegate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

midi_channel_delegate_type_ *midi_channel_delegate_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &midi_channel_delegate_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected midi_channel_delegate object");
        return 0;
    }

    midi_channel_delegate_type_ *object = static_cast<midi_channel_delegate_type_ *>(((midi_channel_delegate_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null midi_channel_delegate object in unwrap");
        return 0;
    }

    return object;
}

void midi_channel_delegate_insert_(const std::pair< midi_channel_delegate_type_ *, PyObject *> &p)
{




    midi_channel_delegate_wrapped_.insert(p);


}

void midi_channel_delegate_erase_(midi_channel_delegate_type_ *p)
{




    midi_channel_delegate_wrapped_.erase(p);


}

PyObject * midi_channel_delegate_find_(midi_channel_delegate_type_ *p)
{




    PyObject *ret = 0;
    std::map< midi_channel_delegate_type_ *, PyObject *>::iterator where = midi_channel_delegate_wrapped_.find(p);
    if(where != midi_channel_delegate_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct midi_channel_delegate_dispatch_ midi_channel_delegate_realdispatcher_ =
{
    &midi_channel_delegate_typeobject_,
    midi_channel_delegate_wrap_,
    midi_channel_delegate_wrapcopy_,
    midi_channel_delegate_unwrap_,
    midi_channel_delegate_insert_,
    midi_channel_delegate_erase_,
    midi_channel_delegate_find_
};

  

struct midi_from_belcanto_object_ {
    PyObject_HEAD
    midi_from_belcanto_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct midi_from_belcanto_wrapper_: public midi_from_belcanto_type_ 
{

    static void __deleter__(midi_from_belcanto_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    midi_from_belcanto_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): midi_from_belcanto_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~midi_from_belcanto_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_from_belcanto_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(midi_from_belcanto_object_ *self, PyObject *args)
    {
        

        midi_from_belcanto_wrapper_ *o = (midi_from_belcanto_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_from_belcanto c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_from_belcanto::cookie");
        }

        return 0;
    }



    void set_midi_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_from_belcanto_type_::set_midi_channel( a0 );
                
            
            
        }

        return
            
                set_midi_channel
            
            
                ( a0 );
    }

    static PyObject *set_midi_channel_method_(midi_from_belcanto_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_from_belcanto_wrapper_ *o = (midi_from_belcanto_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_from_belcanto c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_midi_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_from_belcanto::set_midi_channel");
        }

        return 0;
    }



    void set_min_midi_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_from_belcanto_type_::set_min_midi_channel( a0 );
                
            
            
        }

        return
            
                set_min_midi_channel
            
            
                ( a0 );
    }

    static PyObject *set_min_midi_channel_method_(midi_from_belcanto_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_from_belcanto_wrapper_ *o = (midi_from_belcanto_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_from_belcanto c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_min_midi_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_from_belcanto::set_min_midi_channel");
        }

        return 0;
    }



    void set_max_midi_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_from_belcanto_type_::set_max_midi_channel( a0 );
                
            
            
        }

        return
            
                set_max_midi_channel
            
            
                ( a0 );
    }

    static PyObject *set_max_midi_channel_method_(midi_from_belcanto_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_from_belcanto_wrapper_ *o = (midi_from_belcanto_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_from_belcanto c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_max_midi_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_from_belcanto::set_max_midi_channel");
        }

        return 0;
    }



    void set_program_change_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_from_belcanto_type_::set_program_change( a0 );
                
            
            
        }

        return
            
                set_program_change
            
            
                ( a0 );
    }

    static PyObject *set_program_change_method_(midi_from_belcanto_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_from_belcanto_wrapper_ *o = (midi_from_belcanto_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_from_belcanto c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_program_change_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_from_belcanto::set_program_change");
        }

        return 0;
    }



    change_nb_type_ change_program_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_from_belcanto_type_::change_program();
                
            
            
        }

        return
            
                change_program
            
            
                ();
    }

    static PyObject *change_program_method_(midi_from_belcanto_object_ *self, PyObject *args)
    {
        

        midi_from_belcanto_wrapper_ *o = (midi_from_belcanto_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_from_belcanto c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->change_program_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_from_belcanto::change_program");
        }

        return 0;
    }



    void set_bank_change_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_from_belcanto_type_::set_bank_change( a0 );
                
            
            
        }

        return
            
                set_bank_change
            
            
                ( a0 );
    }

    static PyObject *set_bank_change_method_(midi_from_belcanto_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_from_belcanto_wrapper_ *o = (midi_from_belcanto_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_from_belcanto c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_bank_change_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_from_belcanto::set_bank_change");
        }

        return 0;
    }



    change_nb_type_ change_bank_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_from_belcanto_type_::change_bank();
                
            
            
        }

        return
            
                change_bank
            
            
                ();
    }

    static PyObject *change_bank_method_(midi_from_belcanto_object_ *self, PyObject *args)
    {
        

        midi_from_belcanto_wrapper_ *o = (midi_from_belcanto_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_from_belcanto c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->change_bank_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_from_belcanto::change_bank");
        }

        return 0;
    }



    void set_cc_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return midi_from_belcanto_type_::set_cc( a0  ,a1 );
                
            
            
        }

        return
            
                set_cc
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_cc_method_(midi_from_belcanto_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        midi_from_belcanto_wrapper_ *o = (midi_from_belcanto_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_from_belcanto c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_cc_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_from_belcanto::set_cc");
        }

        return 0;
    }



    change_nb_type_ change_cc_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_from_belcanto_type_::change_cc();
                
            
            
        }

        return
            
                change_cc
            
            
                ();
    }

    static PyObject *change_cc_method_(midi_from_belcanto_object_ *self, PyObject *args)
    {
        

        midi_from_belcanto_wrapper_ *o = (midi_from_belcanto_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_from_belcanto c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->change_cc_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_from_belcanto::change_cc");
        }

        return 0;
    }



    void set_omni_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_from_belcanto_type_::set_omni( a0 );
                
            
            
        }

        return
            
                set_omni
            
            
                ( a0 );
    }

    static PyObject *set_omni_method_(midi_from_belcanto_object_ *self, PyObject *args)
    {
         bool a0; 

        midi_from_belcanto_wrapper_ *o = (midi_from_belcanto_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_from_belcanto c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_omni_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_from_belcanto::set_omni");
        }

        return 0;
    }





    static PyObject *_method_lock(midi_from_belcanto_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(midi_from_belcanto_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void midi_from_belcanto_delete_(midi_from_belcanto_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            midi_from_belcanto_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (midi_from_belcanto_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        midi_from_belcanto_wrapper_::__deleter__((midi_from_belcanto_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void midi_from_belcanto_dealloc_(midi_from_belcanto_object_* self)
{
    
    midi_from_belcanto_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * midi_from_belcanto_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    midi_from_belcanto_object_ *self;

    if(!(self = (midi_from_belcanto_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int midi_from_belcanto_init_(midi_from_belcanto_object_ *self, PyObject *args, PyObject *kwds)
{
    midi_from_belcanto_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new midi_from_belcanto_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing midi_from_belcanto");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            midi_from_belcanto_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"midi_from_belcanto invalid arguments");
    return -1;

}


static const char *midi_from_belcanto_cookie_docstring_ = "";

static const char *midi_from_belcanto_set_midi_channel_docstring_ = "";

static const char *midi_from_belcanto_set_min_midi_channel_docstring_ = "";

static const char *midi_from_belcanto_set_max_midi_channel_docstring_ = "";

static const char *midi_from_belcanto_set_program_change_docstring_ = "";

static const char *midi_from_belcanto_change_program_docstring_ = "";

static const char *midi_from_belcanto_set_bank_change_docstring_ = "";

static const char *midi_from_belcanto_change_bank_docstring_ = "";

static const char *midi_from_belcanto_set_cc_docstring_ = "";

static const char *midi_from_belcanto_change_cc_docstring_ = "";

static const char *midi_from_belcanto_set_omni_docstring_ = "";




static PyMethodDef midi_from_belcanto_methods_[] = {
    
        { "cookie",(PyCFunction)midi_from_belcanto_wrapper_::cookie_method_,METH_VARARGS,(char *)midi_from_belcanto_cookie_docstring_ },
    
        { "set_midi_channel",(PyCFunction)midi_from_belcanto_wrapper_::set_midi_channel_method_,METH_VARARGS,(char *)midi_from_belcanto_set_midi_channel_docstring_ },
    
        { "set_min_midi_channel",(PyCFunction)midi_from_belcanto_wrapper_::set_min_midi_channel_method_,METH_VARARGS,(char *)midi_from_belcanto_set_min_midi_channel_docstring_ },
    
        { "set_max_midi_channel",(PyCFunction)midi_from_belcanto_wrapper_::set_max_midi_channel_method_,METH_VARARGS,(char *)midi_from_belcanto_set_max_midi_channel_docstring_ },
    
        { "set_program_change",(PyCFunction)midi_from_belcanto_wrapper_::set_program_change_method_,METH_VARARGS,(char *)midi_from_belcanto_set_program_change_docstring_ },
    
        { "change_program",(PyCFunction)midi_from_belcanto_wrapper_::change_program_method_,METH_VARARGS,(char *)midi_from_belcanto_change_program_docstring_ },
    
        { "set_bank_change",(PyCFunction)midi_from_belcanto_wrapper_::set_bank_change_method_,METH_VARARGS,(char *)midi_from_belcanto_set_bank_change_docstring_ },
    
        { "change_bank",(PyCFunction)midi_from_belcanto_wrapper_::change_bank_method_,METH_VARARGS,(char *)midi_from_belcanto_change_bank_docstring_ },
    
        { "set_cc",(PyCFunction)midi_from_belcanto_wrapper_::set_cc_method_,METH_VARARGS,(char *)midi_from_belcanto_set_cc_docstring_ },
    
        { "change_cc",(PyCFunction)midi_from_belcanto_wrapper_::change_cc_method_,METH_VARARGS,(char *)midi_from_belcanto_change_cc_docstring_ },
    
        { "set_omni",(PyCFunction)midi_from_belcanto_wrapper_::set_omni_method_,METH_VARARGS,(char *)midi_from_belcanto_set_omni_docstring_ },
    
    
        { "py_lock",(PyCFunction)midi_from_belcanto_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)midi_from_belcanto_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *midi_from_belcanto_docstring_ = "";

PyTypeObject midi_from_belcanto_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.midi_from_belcanto",/*tp_name*/
    sizeof(midi_from_belcanto_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)midi_from_belcanto_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)midi_from_belcanto_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    midi_from_belcanto_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)midi_from_belcanto_init_,  /* tp_init */
    0,                         /* tp_alloc */
    midi_from_belcanto_new_,             /* tp_new */
};

PyObject *midi_from_belcanto_wrapcopy_(midi_from_belcanto_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *midi_from_belcanto_wrap_(midi_from_belcanto_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = midi_from_belcanto_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    midi_from_belcanto_object_ *self = 
        PyObject_NEW(midi_from_belcanto_object_, &midi_from_belcanto_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        midi_from_belcanto_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

midi_from_belcanto_type_ *midi_from_belcanto_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &midi_from_belcanto_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected midi_from_belcanto object");
        return 0;
    }

    midi_from_belcanto_type_ *object = static_cast<midi_from_belcanto_type_ *>(((midi_from_belcanto_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null midi_from_belcanto object in unwrap");
        return 0;
    }

    return object;
}

void midi_from_belcanto_insert_(const std::pair< midi_from_belcanto_type_ *, PyObject *> &p)
{




    midi_from_belcanto_wrapped_.insert(p);


}

void midi_from_belcanto_erase_(midi_from_belcanto_type_ *p)
{




    midi_from_belcanto_wrapped_.erase(p);


}

PyObject * midi_from_belcanto_find_(midi_from_belcanto_type_ *p)
{




    PyObject *ret = 0;
    std::map< midi_from_belcanto_type_ *, PyObject *>::iterator where = midi_from_belcanto_wrapped_.find(p);
    if(where != midi_from_belcanto_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct midi_from_belcanto_dispatch_ midi_from_belcanto_realdispatcher_ =
{
    &midi_from_belcanto_typeobject_,
    midi_from_belcanto_wrap_,
    midi_from_belcanto_wrapcopy_,
    midi_from_belcanto_unwrap_,
    midi_from_belcanto_insert_,
    midi_from_belcanto_erase_,
    midi_from_belcanto_find_
};

  

struct tsd_snapshot_object_ {
    PyObject_HEAD
    tsd_snapshot_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct tsd_snapshot_wrapper_: public tsd_snapshot_type_ 
{

    static void __deleter__(tsd_snapshot_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    tsd_snapshot_wrapper_( PyObject *o, PyInterpreterState *i  ): tsd_snapshot_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~tsd_snapshot_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void log_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                
                    return tsd_snapshot_type_::log( a0 );
                
            
            
        }

        return
            
                log
            
            
                ( a0 );
    }

    static PyObject *log_method_(tsd_snapshot_object_ *self, PyObject *args)
    {
         const char * a0; 

        tsd_snapshot_wrapper_ *o = (tsd_snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null tsd_snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->log_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_snapshot::log");
        }

        return 0;
    }



    void save_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return tsd_snapshot_type_::save();
                
            
            
        }

        return
            
                save
            
            
                ();
    }

    static PyObject *save_method_(tsd_snapshot_object_ *self, PyObject *args)
    {
        

        tsd_snapshot_wrapper_ *o = (tsd_snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null tsd_snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->save_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_snapshot::save");
        }

        return 0;
    }



    void install_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return tsd_snapshot_type_::install();
                
            
            
        }

        return
            
                install
            
            
                ();
    }

    static PyObject *install_method_(tsd_snapshot_object_ *self, PyObject *args)
    {
        

        tsd_snapshot_wrapper_ *o = (tsd_snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null tsd_snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->install_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_snapshot::install");
        }

        return 0;
    }





    static PyObject *_method_lock(tsd_snapshot_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(tsd_snapshot_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void tsd_snapshot_delete_(tsd_snapshot_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            tsd_snapshot_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (tsd_snapshot_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        tsd_snapshot_wrapper_::__deleter__((tsd_snapshot_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void tsd_snapshot_dealloc_(tsd_snapshot_object_* self)
{
    
    tsd_snapshot_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * tsd_snapshot_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    tsd_snapshot_object_ *self;

    if(!(self = (tsd_snapshot_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int tsd_snapshot_init_(tsd_snapshot_object_ *self, PyObject *args, PyObject *kwds)
{
    tsd_snapshot_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new tsd_snapshot_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing tsd_snapshot");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            tsd_snapshot_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"tsd_snapshot invalid arguments");
    return -1;

}


static const char *tsd_snapshot_log_docstring_ = "";

static const char *tsd_snapshot_save_docstring_ = "";

static const char *tsd_snapshot_install_docstring_ = "";




static PyMethodDef tsd_snapshot_methods_[] = {
    
        { "log",(PyCFunction)tsd_snapshot_wrapper_::log_method_,METH_VARARGS,(char *)tsd_snapshot_log_docstring_ },
    
        { "save",(PyCFunction)tsd_snapshot_wrapper_::save_method_,METH_VARARGS,(char *)tsd_snapshot_save_docstring_ },
    
        { "install",(PyCFunction)tsd_snapshot_wrapper_::install_method_,METH_VARARGS,(char *)tsd_snapshot_install_docstring_ },
    
    
        { "py_lock",(PyCFunction)tsd_snapshot_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)tsd_snapshot_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *tsd_snapshot_docstring_ = ""
"    Saves the current global environment for later restore, usually in a different"
"    thread."
"    "
""
;

PyTypeObject tsd_snapshot_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piw_native.tsd_snapshot",/*tp_name*/
    sizeof(tsd_snapshot_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)tsd_snapshot_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)tsd_snapshot_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    tsd_snapshot_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)tsd_snapshot_init_,  /* tp_init */
    0,                         /* tp_alloc */
    tsd_snapshot_new_,             /* tp_new */
};

PyObject *tsd_snapshot_wrapcopy_(tsd_snapshot_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *tsd_snapshot_wrap_(tsd_snapshot_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = tsd_snapshot_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    tsd_snapshot_object_ *self = 
        PyObject_NEW(tsd_snapshot_object_, &tsd_snapshot_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        tsd_snapshot_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

tsd_snapshot_type_ *tsd_snapshot_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &tsd_snapshot_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected tsd_snapshot object");
        return 0;
    }

    tsd_snapshot_type_ *object = static_cast<tsd_snapshot_type_ *>(((tsd_snapshot_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null tsd_snapshot object in unwrap");
        return 0;
    }

    return object;
}

void tsd_snapshot_insert_(const std::pair< tsd_snapshot_type_ *, PyObject *> &p)
{




    tsd_snapshot_wrapped_.insert(p);


}

void tsd_snapshot_erase_(tsd_snapshot_type_ *p)
{




    tsd_snapshot_wrapped_.erase(p);


}

PyObject * tsd_snapshot_find_(tsd_snapshot_type_ *p)
{




    PyObject *ret = 0;
    std::map< tsd_snapshot_type_ *, PyObject *>::iterator where = tsd_snapshot_wrapped_.find(p);
    if(where != tsd_snapshot_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct tsd_snapshot_dispatch_ tsd_snapshot_realdispatcher_ =
{
    &tsd_snapshot_typeobject_,
    tsd_snapshot_wrap_,
    tsd_snapshot_wrapcopy_,
    tsd_snapshot_unwrap_,
    tsd_snapshot_insert_,
    tsd_snapshot_erase_,
    tsd_snapshot_find_
};

         


static const char *module_method_create_samplearray_docstring_ = "";

static PyObject *module_method_create_samplearray(PyObject *self, PyObject *args)
{
     const char * a0;  unsigned int a1;  unsigned int a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_str,&a0,fpcvt_ui,&a1,fpcvt_ui,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    samplearray_type_ r = piw::create_samplearray  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_samplearray_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in create_samplearray");
    }

    return 0;
}


static const char *module_method_create_sample_docstring_ = "";

static PyObject *module_method_create_sample(PyObject *self, PyObject *args)
{
     samplearray_type_ * a0;  unsigned int a1;  unsigned int a2;  unsigned int a3;  unsigned int a4;  float a5;  float a6;  float a7; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&O&O&O&" ,fpcvt_samplearray_copy,&a0,fpcvt_ui,&a1,fpcvt_ui,&a2,fpcvt_ui,&a3,fpcvt_ui,&a4,fpcvt_f,&a5,fpcvt_f,&a6,fpcvt_f,&a7))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    sample_type_ r = piw::create_sample  (  *a0  ,a1  ,a2  ,a3  ,a4  ,a5  ,a6  ,a7 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_sample_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in create_sample");
    }

    return 0;
}


static const char *module_method_create_zone_docstring_ = "";

static PyObject *module_method_create_zone(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2;  float a3;  float a4;  float a5;  float a6;  float a7;  float a8;  float a9;  float a10;  sample_type_ * a11; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&O&O&O&O&O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_f,&a3,fpcvt_f,&a4,fpcvt_f,&a5,fpcvt_f,&a6,fpcvt_f,&a7,fpcvt_f,&a8,fpcvt_f,&a9,fpcvt_f,&a10,fpcvt_sample_copy,&a11))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    zone_type_ r = piw::create_zone  (  a0  ,a1  ,a2  ,a3  ,a4  ,a5  ,a6  ,a7  ,a8  ,a9  ,a10  ,*a11 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_zone_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in create_zone");
    }

    return 0;
}


static const char *module_method_create_preset_docstring_ = "";

static PyObject *module_method_create_preset(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    preset_type_ r = piw::create_preset  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_preset_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in create_preset");
    }

    return 0;
}


static const char *module_method_root_filter_docstring_ = "";

static PyObject *module_method_root_filter(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::root_filter  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in root_filter");
    }

    return 0;
}


static const char *module_method_null_filter_docstring_ = "";

static PyObject *module_method_null_filter(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::null_filter  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in null_filter");
    }

    return 0;
}


static const char *module_method_signal_dsc_filter_docstring_ = "";

static PyObject *module_method_signal_dsc_filter(PyObject *self, PyObject *args)
{
     unsigned int a0;  unsigned int a1;  const char * a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_ui,&a0,fpcvt_ui,&a1,fpcvt_str,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::signal_dsc_filter  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in signal_dsc_filter");
    }

    return 0;
}


static const char *module_method_signal_cnc_filter_docstring_ = "";

static PyObject *module_method_signal_cnc_filter(PyObject *self, PyObject *args)
{
     unsigned int a0;  unsigned int a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_ui,&a0,fpcvt_ui,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::signal_cnc_filter  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in signal_cnc_filter");
    }

    return 0;
}


static const char *module_method_aggregation_filter_docstring_ = "";

static PyObject *module_method_aggregation_filter(PyObject *self, PyObject *args)
{
     unsigned int a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::aggregation_filter  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in aggregation_filter");
    }

    return 0;
}


static const char *module_method_aggregation_filter3_docstring_ = "";

static PyObject *module_method_aggregation_filter3(PyObject *self, PyObject *args)
{
     unsigned int a0;  unsigned int a1;  unsigned int a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_ui,&a0,fpcvt_ui,&a1,fpcvt_ui,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::aggregation_filter3  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in aggregation_filter3");
    }

    return 0;
}


static const char *module_method_deaggregation_filter_docstring_ = "";

static PyObject *module_method_deaggregation_filter(PyObject *self, PyObject *args)
{
     unsigned int a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::deaggregation_filter  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in deaggregation_filter");
    }

    return 0;
}


static const char *module_method_grist_aggregation_filter_docstring_ = "";

static PyObject *module_method_grist_aggregation_filter(PyObject *self, PyObject *args)
{
     unsigned int a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::grist_aggregation_filter  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in grist_aggregation_filter");
    }

    return 0;
}


static const char *module_method_gristchaff_aggregation_filter_docstring_ = "";

static PyObject *module_method_gristchaff_aggregation_filter(PyObject *self, PyObject *args)
{
     unsigned int a0;  unsigned int a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_ui,&a0,fpcvt_ui,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::gristchaff_aggregation_filter  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in gristchaff_aggregation_filter");
    }

    return 0;
}


static const char *module_method_grist_deaggregation_filter_docstring_ = "";

static PyObject *module_method_grist_deaggregation_filter(PyObject *self, PyObject *args)
{
     unsigned int a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::grist_deaggregation_filter  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in grist_deaggregation_filter");
    }

    return 0;
}


static const char *module_method_grist_deaggregation_filter2_docstring_ = "";

static PyObject *module_method_grist_deaggregation_filter2(PyObject *self, PyObject *args)
{
     unsigned int a0;  unsigned int a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_ui,&a0,fpcvt_ui,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::grist_deaggregation_filter2  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in grist_deaggregation_filter2");
    }

    return 0;
}


static const char *module_method_last_gt_filter_docstring_ = "";

static PyObject *module_method_last_gt_filter(PyObject *self, PyObject *args)
{
     unsigned int a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::last_gt_filter  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in last_gt_filter");
    }

    return 0;
}


static const char *module_method_last_lt_filter_docstring_ = "";

static PyObject *module_method_last_lt_filter(PyObject *self, PyObject *args)
{
     unsigned int a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::last_lt_filter  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in last_lt_filter");
    }

    return 0;
}


static const char *module_method_last_filter_docstring_ = "";

static PyObject *module_method_last_filter(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::last_filter  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in last_filter");
    }

    return 0;
}


static const char *module_method_grist_filter_docstring_ = "";

static PyObject *module_method_grist_filter(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::grist_filter  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in grist_filter");
    }

    return 0;
}


static const char *module_method_dict_merger_docstring_ = "";

static PyObject *module_method_dict_merger(PyObject *self, PyObject *args)
{
     data_type_ * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_data_copy,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::dict_merger  (  *a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in dict_merger");
    }

    return 0;
}


static const char *module_method_null_converter_docstring_ = "";

static PyObject *module_method_null_converter(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    converter_ref_type_ r = piw::null_converter  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_converter_ref_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in null_converter");
    }

    return 0;
}


static const char *module_method_throttling_converter_docstring_ = "";

static PyObject *module_method_throttling_converter(PyObject *self, PyObject *args)
{
     unsigned long long a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ull,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    converter_ref_type_ r = piw::throttling_converter  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_converter_ref_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in throttling_converter");
    }

    return 0;
}


static const char *module_method_triggering_converter_docstring_ = "";

static PyObject *module_method_triggering_converter(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    converter_ref_type_ r = piw::triggering_converter  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_converter_ref_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in triggering_converter");
    }

    return 0;
}


static const char *module_method_impulse_converter_docstring_ = "";

static PyObject *module_method_impulse_converter(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    converter_ref_type_ r = piw::impulse_converter  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_converter_ref_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in impulse_converter");
    }

    return 0;
}


static const char *module_method_resampling_converter_docstring_ = "";

static PyObject *module_method_resampling_converter(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    converter_ref_type_ r = piw::resampling_converter  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_converter_ref_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in resampling_converter");
    }

    return 0;
}


static const char *module_method_interpolating_converter_docstring_ = "";

static PyObject *module_method_interpolating_converter(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    converter_ref_type_ r = piw::interpolating_converter  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_converter_ref_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in interpolating_converter");
    }

    return 0;
}


static const char *module_method_lopass_converter_docstring_ = "";

static PyObject *module_method_lopass_converter(PyObject *self, PyObject *args)
{
     float a0;  float a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    converter_ref_type_ r = piw::lopass_converter  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_converter_ref_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in lopass_converter");
    }

    return 0;
}


static const char *module_method_deferred_sender_docstring_ = "";

static PyObject *module_method_deferred_sender(PyObject *self, PyObject *args)
{
     change_nb_type_ * a0;  data_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_change_nb_copy,&a0,fpcvt_data_copy,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_nb_type_ r = piw::deferred_sender  (  *a0  ,*a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in deferred_sender");
    }

    return 0;
}


static const char *module_method_slowchange_docstring_ = "";

static PyObject *module_method_slowchange(PyObject *self, PyObject *args)
{
     change_type_ * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_change_copy,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_type_ r = piw::slowchange  (  *a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in slowchange");
    }

    return 0;
}


static const char *module_method_slowchange_polled_docstring_ = "";

static PyObject *module_method_slowchange_polled(PyObject *self, PyObject *args)
{
     change_type_ * a0;  unsigned int a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_change_copy,&a0,fpcvt_ui,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_nb_type_ r = piw::slowchange_polled  (  *a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in slowchange_polled");
    }

    return 0;
}


static const char *module_method_slowtrigger_docstring_ = "";

static PyObject *module_method_slowtrigger(PyObject *self, PyObject *args)
{
     change_type_ * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_change_copy,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_type_ r = piw::slowtrigger  (  *a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in slowtrigger");
    }

    return 0;
}


static const char *module_method_make_change_normal_docstring_ = "";

static PyObject *module_method_make_change_normal(PyObject *self, PyObject *args)
{
     change_nb_type_ * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_change_nb_copy,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_type_ r = piw::make_change_normal  (  *a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in make_change_normal");
    }

    return 0;
}


static const char *module_method_make_change_nb_docstring_ = "";

static PyObject *module_method_make_change_nb(PyObject *self, PyObject *args)
{
     change_type_ * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_change_copy,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_nb_type_ r = piw::make_change_nb  (  *a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in make_change_nb");
    }

    return 0;
}


static const char *module_method_first_filter_docstring_ = "";

static PyObject *module_method_first_filter(PyObject *self, PyObject *args)
{
     unsigned int a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2b_type_ r = piw::first_filter  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2b_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in first_filter");
    }

    return 0;
}


static const char *module_method_indirectchange_docstring_ = "";

static PyObject *module_method_indirectchange(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_type_ r = piw::indirectchange  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in indirectchange");
    }

    return 0;
}


static const char *module_method_indirectchange_set_docstring_ = "";

static PyObject *module_method_indirectchange_set(PyObject *self, PyObject *args)
{
     change_type_ * a0;  change_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_change,&a0,fpcvt_change_copy,&a1))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             indirectchange_set (  *a0  ,*a1  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in indirectchange_set");
    }

    return 0;
}


static const char *module_method_indirectchange_clear_docstring_ = "";

static PyObject *module_method_indirectchange_clear(PyObject *self, PyObject *args)
{
     change_type_ * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_change,&a0))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             indirectchange_clear (  *a0  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in indirectchange_clear");
    }

    return 0;
}


static const char *module_method_indirectchange_nb_docstring_ = "";

static PyObject *module_method_indirectchange_nb(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_nb_type_ r = piw::indirectchange_nb  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in indirectchange_nb");
    }

    return 0;
}


static const char *module_method_indirectchange_set_nb_docstring_ = "";

static PyObject *module_method_indirectchange_set_nb(PyObject *self, PyObject *args)
{
     change_nb_type_ * a0;  change_nb_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_change_nb,&a0,fpcvt_change_nb_copy,&a1))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             indirectchange_set_nb (  *a0  ,*a1  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in indirectchange_set_nb");
    }

    return 0;
}


static const char *module_method_indirectchange_clear_nb_docstring_ = "";

static PyObject *module_method_indirectchange_clear_nb(PyObject *self, PyObject *args)
{
     change_nb_type_ * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_change_nb,&a0))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             indirectchange_clear_nb (  *a0  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in indirectchange_clear_nb");
    }

    return 0;
}


static const char *module_method_changelist_docstring_ = "";

static PyObject *module_method_changelist(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_type_ r = piw::changelist  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in changelist");
    }

    return 0;
}


static const char *module_method_changelist_connect_docstring_ = "";

static PyObject *module_method_changelist_connect(PyObject *self, PyObject *args)
{
     change_type_ * a0;  change_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_change,&a0,fpcvt_change_copy,&a1))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             changelist_connect (  *a0  ,*a1  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in changelist_connect");
    }

    return 0;
}


static const char *module_method_changelist_disconnect_docstring_ = "";

static PyObject *module_method_changelist_disconnect(PyObject *self, PyObject *args)
{
     change_type_ * a0;  change_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_change,&a0,fpcvt_change_copy,&a1))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             changelist_disconnect (  *a0  ,*a1  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in changelist_disconnect");
    }

    return 0;
}


static const char *module_method_changelist_nb_docstring_ = "";

static PyObject *module_method_changelist_nb(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_nb_type_ r = piw::changelist_nb  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in changelist_nb");
    }

    return 0;
}


static const char *module_method_changelist_connect_nb_docstring_ = "";

static PyObject *module_method_changelist_connect_nb(PyObject *self, PyObject *args)
{
     change_nb_type_ * a0;  change_nb_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_change_nb,&a0,fpcvt_change_nb_copy,&a1))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             changelist_connect_nb (  *a0  ,*a1  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in changelist_connect_nb");
    }

    return 0;
}


static const char *module_method_changelist_disconnect_nb_docstring_ = "";

static PyObject *module_method_changelist_disconnect_nb(PyObject *self, PyObject *args)
{
     change_nb_type_ * a0;  change_nb_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_change_nb,&a0,fpcvt_change_nb_copy,&a1))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             changelist_disconnect_nb (  *a0  ,*a1  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in changelist_disconnect_nb");
    }

    return 0;
}


static const char *module_method_change2_docstring_ = "";

static PyObject *module_method_change2(PyObject *self, PyObject *args)
{
     change_type_ * a0;  change_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_change_copy,&a0,fpcvt_change_copy,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_type_ r = piw::change2  (  *a0  ,*a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in change2");
    }

    return 0;
}


static const char *module_method_change2_nb_docstring_ = "";

static PyObject *module_method_change2_nb(PyObject *self, PyObject *args)
{
     change_type_ * a0;  change_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_change_copy,&a0,fpcvt_change_copy,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_nb_type_ r = piw::change2_nb  (  *a0  ,*a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in change2_nb");
    }

    return 0;
}


static const char *module_method_change_nb2_docstring_ = "";

static PyObject *module_method_change_nb2(PyObject *self, PyObject *args)
{
     change_nb_type_ * a0;  change_nb_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_change_nb_copy,&a0,fpcvt_change_nb_copy,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_nb_type_ r = piw::change_nb2  (  *a0  ,*a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in change_nb2");
    }

    return 0;
}


static const char *module_method_trigger_docstring_ = "";

static PyObject *module_method_trigger(PyObject *self, PyObject *args)
{
     change_nb_type_ * a0;  data_nb_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_change_nb_copy,&a0,fpcvt_data_nb_copy,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_type_ r = piw::trigger  (  *a0  ,*a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in trigger");
    }

    return 0;
}


static const char *module_method_parse_state_term_docstring_ = "";

static PyObject *module_method_parse_state_term(PyObject *self, PyObject *args)
{
     std::string a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_stdstr,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    term_type_ r = piw::parse_state_term  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_term_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in parse_state_term");
    }

    return 0;
}


static const char *module_method_makefloat_nb_docstring_ = ""
"    makefloat(value,timestamp) -> data"
"    "
"    make a float data value"
"    "
""
;

static PyObject *module_method_makefloat_nb(PyObject *self, PyObject *args)
{
     float a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_f,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makefloat_nb  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makefloat_nb");
    }

    return 0;
}


static const char *module_method_makenull_nb_docstring_ = "";

static PyObject *module_method_makenull_nb(PyObject *self, PyObject *args)
{
     unsigned long long a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ull,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makenull_nb  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makenull_nb");
    }

    return 0;
}


static const char *module_method_makestring_len_nb_docstring_ = ""
"    makestring(value,timestamp) -> data"
"    "
"    make a string data value"
"    "
""
;

static PyObject *module_method_makestring_len_nb(PyObject *self, PyObject *args)
{
     const char * a0;  unsigned int a1;  unsigned long long a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_str,&a0,fpcvt_ui,&a1,fpcvt_ull,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makestring_len_nb  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makestring_len_nb");
    }

    return 0;
}


static const char *module_method_makestring_nb_docstring_ = ""
"    makestring(value,timestamp) -> data"
"    "
"    make a string data value"
"    "
""
;

static PyObject *module_method_makestring_nb(PyObject *self, PyObject *args)
{
     const char * a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makestring_nb  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makestring_nb");
    }

    return 0;
}


static const char *module_method_makebool_nb_docstring_ = ""
"    makebool(value,timestamp) -> data"
"    "
"    make a bool data value"
"    "
""
;

static PyObject *module_method_makebool_nb(PyObject *self, PyObject *args)
{
     bool a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_b,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makebool_nb  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makebool_nb");
    }

    return 0;
}


static const char *module_method_makelong_nb_docstring_ = ""
"    makelong(value,timestamp) -> data"
"    "
"    make a long data value"
"    "
""
;

static PyObject *module_method_makelong_nb(PyObject *self, PyObject *args)
{
     long a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_l,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makelong_nb  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makelong_nb");
    }

    return 0;
}


static const char *module_method_pathone_nb_docstring_ = ""
"    pathnull(value,timestamp) -> data"
" "
"    make a path of length 1"
"    "
""
;

static PyObject *module_method_pathone_nb(PyObject *self, PyObject *args)
{
     unsigned int a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_ui,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::pathone_nb  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in pathone_nb");
    }

    return 0;
}


static const char *module_method_pathnull_nb_docstring_ = ""
"    pathnull(timestamp) -> data"
" "
"    make a null path"
"    "
""
;

static PyObject *module_method_pathnull_nb(PyObject *self, PyObject *args)
{
     unsigned long long a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ull,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::pathnull_nb  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in pathnull_nb");
    }

    return 0;
}


static const char *module_method_dictnull_docstring_ = "";

static PyObject *module_method_dictnull(PyObject *self, PyObject *args)
{
     unsigned long long a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ull,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::dictnull  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in dictnull");
    }

    return 0;
}


static const char *module_method_dictset_docstring_ = "";

static PyObject *module_method_dictset(PyObject *self, PyObject *args)
{
     data_type_ * a0;  std::string a1;  data_type_ * a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_data_copy,&a0,fpcvt_stdstr,&a1,fpcvt_data_copy,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::dictset  (  *a0  ,a1  ,*a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in dictset");
    }

    return 0;
}


static const char *module_method_dictupdate_docstring_ = "";

static PyObject *module_method_dictupdate(PyObject *self, PyObject *args)
{
     data_type_ * a0;  data_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_data_copy,&a0,fpcvt_data_copy,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r =  dictupdate (  *a0  ,*a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in dictupdate");
    }

    return 0;
}


static const char *module_method_dictdel_docstring_ = "";

static PyObject *module_method_dictdel(PyObject *self, PyObject *args)
{
     data_type_ * a0;  std::string a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_data_copy,&a0,fpcvt_stdstr,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r =  dictdel (  *a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in dictdel");
    }

    return 0;
}


static const char *module_method_dictnull_nb_docstring_ = "";

static PyObject *module_method_dictnull_nb(PyObject *self, PyObject *args)
{
     unsigned long long a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ull,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::dictnull_nb  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in dictnull_nb");
    }

    return 0;
}


static const char *module_method_dictset_nb_docstring_ = "";

static PyObject *module_method_dictset_nb(PyObject *self, PyObject *args)
{
     data_nb_type_ * a0;  std::string a1;  data_nb_type_ * a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_data_nb_copy,&a0,fpcvt_stdstr,&a1,fpcvt_data_nb_copy,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::dictset_nb  (  *a0  ,a1  ,*a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in dictset_nb");
    }

    return 0;
}


static const char *module_method_parsepath_nb_docstring_ = ""
"    parsepath(string,timestamp) ->data"
""
"    parse a dotted notation path "
"    "
""
;

static PyObject *module_method_parsepath_nb(PyObject *self, PyObject *args)
{
     const char * a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::parsepath_nb  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in parsepath_nb");
    }

    return 0;
}


static const char *module_method_makepath_nb_docstring_ = ""
"    makepath(value,len,timestamp) -> data"
""
"    make a path from a string"
"    "
""
;

static PyObject *module_method_makepath_nb(PyObject *self, PyObject *args)
{
     const unsigned char * a0;  unsigned int a1;  unsigned long long a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_ustr,&a0,fpcvt_ui,&a1,fpcvt_ull,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makepath_nb  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makepath_nb");
    }

    return 0;
}


static const char *module_method_makedouble_bounded_units_nb_docstring_ = "";

static PyObject *module_method_makedouble_bounded_units_nb(PyObject *self, PyObject *args)
{
     unsigned int a0;  float a1;  float a2;  float a3;  double a4;  unsigned long long a5; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&O&" ,fpcvt_ui,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_f,&a3,fpcvt_d,&a4,fpcvt_ull,&a5))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makedouble_bounded_units_nb  (  a0  ,a1  ,a2  ,a3  ,a4  ,a5 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makedouble_bounded_units_nb");
    }

    return 0;
}


static const char *module_method_makefloat_bounded_units_nb_docstring_ = "";

static PyObject *module_method_makefloat_bounded_units_nb(PyObject *self, PyObject *args)
{
     unsigned int a0;  float a1;  float a2;  float a3;  float a4;  unsigned long long a5; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&O&" ,fpcvt_ui,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_f,&a3,fpcvt_f,&a4,fpcvt_ull,&a5))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makefloat_bounded_units_nb  (  a0  ,a1  ,a2  ,a3  ,a4  ,a5 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makefloat_bounded_units_nb");
    }

    return 0;
}


static const char *module_method_makelong_bounded_units_nb_docstring_ = "";

static PyObject *module_method_makelong_bounded_units_nb(PyObject *self, PyObject *args)
{
     unsigned int a0;  float a1;  float a2;  float a3;  long a4;  unsigned long long a5; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&O&" ,fpcvt_ui,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_f,&a3,fpcvt_l,&a4,fpcvt_ull,&a5))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makelong_bounded_units_nb  (  a0  ,a1  ,a2  ,a3  ,a4  ,a5 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makelong_bounded_units_nb");
    }

    return 0;
}


static const char *module_method_makedouble_bounded_nb_docstring_ = "";

static PyObject *module_method_makedouble_bounded_nb(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2;  double a3;  unsigned long long a4; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_d,&a3,fpcvt_ull,&a4))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makedouble_bounded_nb  (  a0  ,a1  ,a2  ,a3  ,a4 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makedouble_bounded_nb");
    }

    return 0;
}


static const char *module_method_makefloat_bounded_nb_docstring_ = "";

static PyObject *module_method_makefloat_bounded_nb(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2;  float a3;  unsigned long long a4; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_f,&a3,fpcvt_ull,&a4))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makefloat_bounded_nb  (  a0  ,a1  ,a2  ,a3  ,a4 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makefloat_bounded_nb");
    }

    return 0;
}


static const char *module_method_makelong_bounded_nb_docstring_ = "";

static PyObject *module_method_makelong_bounded_nb(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2;  long a3;  unsigned long long a4; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_l,&a3,fpcvt_ull,&a4))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_nb_type_ r = piw::makelong_bounded_nb  (  a0  ,a1  ,a2  ,a3  ,a4 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makelong_bounded_nb");
    }

    return 0;
}


static const char *module_method_makefloat_docstring_ = ""
"    makefloat(value,timestamp) -> data"
"    "
"    make a float data value"
"    "
""
;

static PyObject *module_method_makefloat(PyObject *self, PyObject *args)
{
     float a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_f,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makefloat  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makefloat");
    }

    return 0;
}


static const char *module_method_makenull_docstring_ = "";

static PyObject *module_method_makenull(PyObject *self, PyObject *args)
{
     unsigned long long a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ull,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makenull  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makenull");
    }

    return 0;
}


static const char *module_method_makestring_len_docstring_ = ""
"    makestring(value,timestamp) -> data"
"    "
"    make a string data value"
"    "
""
;

static PyObject *module_method_makestring_len(PyObject *self, PyObject *args)
{
     const char * a0;  unsigned int a1;  unsigned long long a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_str,&a0,fpcvt_ui,&a1,fpcvt_ull,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makestring_len  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makestring_len");
    }

    return 0;
}


static const char *module_method_makestring_docstring_ = ""
"    makestring(value,timestamp) -> data"
"    "
"    make a string data value"
"    "
""
;

static PyObject *module_method_makestring(PyObject *self, PyObject *args)
{
     const char * a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makestring  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makestring");
    }

    return 0;
}


static const char *module_method_makeblob2_docstring_ = ""
"    makeblob(value,timestamp) -> data"
"    "
"    make a blob data value"
"    "
""
;

static PyObject *module_method_makeblob2(PyObject *self, PyObject *args)
{
     std::string a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_stdstr,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makeblob2  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makeblob2");
    }

    return 0;
}


static const char *module_method_makebool_docstring_ = ""
"    makebool(value,timestamp) -> data"
"    "
"    make a bool data value"
"    "
""
;

static PyObject *module_method_makebool(PyObject *self, PyObject *args)
{
     bool a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_b,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makebool  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makebool");
    }

    return 0;
}


static const char *module_method_makelong_docstring_ = ""
"    makelong(value,timestamp) -> data"
"    "
"    make a long data value"
"    "
""
;

static PyObject *module_method_makelong(PyObject *self, PyObject *args)
{
     long a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_l,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makelong  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makelong");
    }

    return 0;
}


static const char *module_method_pathone_docstring_ = ""
"    pathone(value,timestamp) -> data"
""
"    make a path of length 1"
"    "
""
;

static PyObject *module_method_pathone(PyObject *self, PyObject *args)
{
     unsigned int a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_ui,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::pathone  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in pathone");
    }

    return 0;
}


static const char *module_method_pathnull_docstring_ = ""
"    pathnull(timestamp) -> data"
""
"    make a null path"
"    "
""
;

static PyObject *module_method_pathnull(PyObject *self, PyObject *args)
{
     unsigned long long a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ull,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::pathnull  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in pathnull");
    }

    return 0;
}


static const char *module_method_parsepath_docstring_ = ""
"    parsepath(string,timestamp) ->data"
""
"    parse a dotted notation path "
"    "
""
;

static PyObject *module_method_parsepath(PyObject *self, PyObject *args)
{
     const char * a0;  unsigned long long a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_ull,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::parsepath  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in parsepath");
    }

    return 0;
}


static const char *module_method_makepath_docstring_ = ""
"    makepath(value,len,timestamp) -> data"
""
"    make a path from a string"
"    "
""
;

static PyObject *module_method_makepath(PyObject *self, PyObject *args)
{
     const unsigned char * a0;  unsigned int a1;  unsigned long long a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_ustr,&a0,fpcvt_ui,&a1,fpcvt_ull,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makepath  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makepath");
    }

    return 0;
}


static const char *module_method_makedouble_bounded_docstring_ = "";

static PyObject *module_method_makedouble_bounded(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2;  double a3;  unsigned long long a4; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_d,&a3,fpcvt_ull,&a4))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makedouble_bounded  (  a0  ,a1  ,a2  ,a3  ,a4 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makedouble_bounded");
    }

    return 0;
}


static const char *module_method_makefloat_bounded_docstring_ = "";

static PyObject *module_method_makefloat_bounded(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2;  float a3;  unsigned long long a4; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_f,&a3,fpcvt_ull,&a4))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makefloat_bounded  (  a0  ,a1  ,a2  ,a3  ,a4 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makefloat_bounded");
    }

    return 0;
}


static const char *module_method_makelong_bounded_docstring_ = "";

static PyObject *module_method_makelong_bounded(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2;  long a3;  unsigned long long a4; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_l,&a3,fpcvt_ull,&a4))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makelong_bounded  (  a0  ,a1  ,a2  ,a3  ,a4 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makelong_bounded");
    }

    return 0;
}


static const char *module_method_makedouble_bounded_units_docstring_ = "";

static PyObject *module_method_makedouble_bounded_units(PyObject *self, PyObject *args)
{
     unsigned int a0;  float a1;  float a2;  float a3;  double a4;  unsigned long long a5; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&O&" ,fpcvt_ui,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_f,&a3,fpcvt_d,&a4,fpcvt_ull,&a5))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makedouble_bounded_units  (  a0  ,a1  ,a2  ,a3  ,a4  ,a5 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makedouble_bounded_units");
    }

    return 0;
}


static const char *module_method_makefloat_bounded_units_docstring_ = "";

static PyObject *module_method_makefloat_bounded_units(PyObject *self, PyObject *args)
{
     unsigned int a0;  float a1;  float a2;  float a3;  float a4;  unsigned long long a5; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&O&" ,fpcvt_ui,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_f,&a3,fpcvt_f,&a4,fpcvt_ull,&a5))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makefloat_bounded_units  (  a0  ,a1  ,a2  ,a3  ,a4  ,a5 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makefloat_bounded_units");
    }

    return 0;
}


static const char *module_method_makelong_bounded_units_docstring_ = "";

static PyObject *module_method_makelong_bounded_units(PyObject *self, PyObject *args)
{
     unsigned int a0;  float a1;  float a2;  float a3;  long a4;  unsigned long long a5; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&O&" ,fpcvt_ui,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_f,&a3,fpcvt_l,&a4,fpcvt_ull,&a5))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    data_type_ r = piw::makelong_bounded_units  (  a0  ,a1  ,a2  ,a3  ,a4  ,a5 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in makelong_bounded_units");
    }

    return 0;
}


static const char *module_method_normalise_docstring_ = ""
"    normalise(upper-bound, lower-bound, zero, denorm) -> norm"
"    convert float to normalised form (-1,1)"
"    "
""
;

static PyObject *module_method_normalise(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2;  float a3; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_f,&a3))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    float r = piw::normalise  (  a0  ,a1  ,a2  ,a3 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in normalise");
    }

    return 0;
}


static const char *module_method_denormalise_docstring_ = ""
"    denormalise(upper-bound, lower-bound, zero, norm) -> denorm"
"    convert float from normalised form to given range"
"    "
""
;

static PyObject *module_method_denormalise(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2;  float a3; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_f,&a3))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    float r = piw::denormalise  (  a0  ,a1  ,a2  ,a3 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in denormalise");
    }

    return 0;
}


static const char *module_method_address2server_docstring_ = ""
"    parse an address and return the server part as a string"
"    "
""
;

static PyObject *module_method_address2server(PyObject *self, PyObject *args)
{
     std::string a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_stdstr,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = piw::address2server  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in address2server");
    }

    return 0;
}


static const char *module_method_bint_normalizer_docstring_ = "";

static PyObject *module_method_bint_normalizer(PyObject *self, PyObject *args)
{
     int a0;  int a1;  int a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_i,&a0,fpcvt_i,&a1,fpcvt_i,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::bint_normalizer  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in bint_normalizer");
    }

    return 0;
}


static const char *module_method_bint_denormalizer_docstring_ = "";

static PyObject *module_method_bint_denormalizer(PyObject *self, PyObject *args)
{
     int a0;  int a1;  int a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_i,&a0,fpcvt_i,&a1,fpcvt_i,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::bint_denormalizer  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in bint_denormalizer");
    }

    return 0;
}


static const char *module_method_bfloat_normalizer_docstring_ = "";

static PyObject *module_method_bfloat_normalizer(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::bfloat_normalizer  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in bfloat_normalizer");
    }

    return 0;
}


static const char *module_method_bfloat_denormalizer_docstring_ = "";

static PyObject *module_method_bfloat_denormalizer(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::bfloat_denormalizer  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in bfloat_denormalizer");
    }

    return 0;
}


static const char *module_method_bool_normalizer_docstring_ = "";

static PyObject *module_method_bool_normalizer(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::bool_normalizer  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in bool_normalizer");
    }

    return 0;
}


static const char *module_method_bool_denormalizer_docstring_ = "";

static PyObject *module_method_bool_denormalizer(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::bool_denormalizer  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in bool_denormalizer");
    }

    return 0;
}


static const char *module_method_null_normalizer_docstring_ = "";

static PyObject *module_method_null_normalizer(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::null_normalizer  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in null_normalizer");
    }

    return 0;
}


static const char *module_method_null_denormalizer_docstring_ = "";

static PyObject *module_method_null_denormalizer(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::null_denormalizer  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in null_denormalizer");
    }

    return 0;
}


static const char *module_method_string_normalizer_docstring_ = "";

static PyObject *module_method_string_normalizer(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::string_normalizer  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in string_normalizer");
    }

    return 0;
}


static const char *module_method_string_denormalizer_docstring_ = "";

static PyObject *module_method_string_denormalizer(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::string_denormalizer  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in string_denormalizer");
    }

    return 0;
}


static const char *module_method_d2d_chain_docstring_ = "";

static PyObject *module_method_d2d_chain(PyObject *self, PyObject *args)
{
     d2d_nb_type_ * a0;  d2d_nb_type_ * a1;  change_nb_type_ * a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_d2d_nb_copy,&a0,fpcvt_d2d_nb_copy,&a1,fpcvt_change_nb_copy,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_nb_type_ r = piw::d2d_chain  (  *a0  ,*a1  ,*a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2d_chain");
    }

    return 0;
}


static const char *module_method_d2d_convert_docstring_ = "";

static PyObject *module_method_d2d_convert(PyObject *self, PyObject *args)
{
     d2d_nb_type_ * a0;  change_nb_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_d2d_nb_copy,&a0,fpcvt_change_nb_copy,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_nb_type_ r = piw::d2d_convert  (  *a0  ,*a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in d2d_convert");
    }

    return 0;
}


static const char *module_method_fastchange_docstring_ = "";

static PyObject *module_method_fastchange(PyObject *self, PyObject *args)
{
     change_nb_type_ * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_change_nb_copy,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    change_type_ r = piw::fastchange  (  *a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in fastchange");
    }

    return 0;
}


static const char *module_method_add_docstring_ = "";

static PyObject *module_method_add(PyObject *self, PyObject *args)
{
     float a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_f,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    dd2d_nb_type_ r = piw::add  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_dd2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in add");
    }

    return 0;
}


static const char *module_method_make_d2d_const_docstring_ = "";

static PyObject *module_method_make_d2d_const(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2;  float a3; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_f,&a3))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::make_d2d_const  (  a0  ,a1  ,a2  ,a3 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in make_d2d_const");
    }

    return 0;
}


static const char *module_method_make_f2f_identity_docstring_ = "";

static PyObject *module_method_make_f2f_identity(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    f2f_type_ r = piw::make_f2f_identity  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f2f_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in make_f2f_identity");
    }

    return 0;
}


static const char *module_method_make_f2f_table_docstring_ = ""
"    make_f2f_table(low,high,samples,functor) -> functor"
"    "
"    Convert f2f functor into table form.  The function is"
"    sampled at samples intervals between low and high values."
"    The resulting functor linearly interpolates between samples"
"    "
"    This function can be used to convert a python callable into"
"    a functor which is usable from the high speed thread"
"    "
""
;

static PyObject *module_method_make_f2f_table(PyObject *self, PyObject *args)
{
     float a0;  float a1;  unsigned int a2;  f2f_type_ * a3; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_ui,&a2,fpcvt_f2f_copy,&a3))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    f2f_type_ r = piw::make_f2f_table  (  a0  ,a1  ,a2  ,*a3 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f2f_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in make_f2f_table");
    }

    return 0;
}


static const char *module_method_make_d2d_table_docstring_ = ""
"    make_d2d_table(low,high,zero,samples,functor) -> functor"
"    "
"    Convert d2d functor into table form.  The function is"
"    sampled at samples intervals between low and high values."
"    The resulting functor linearly interpolates between samples"
"    "
"    This function can be used to convert a python callable into"
"    a functor which is usable from the high speed thread"
"    "
""
;

static PyObject *module_method_make_d2d_table(PyObject *self, PyObject *args)
{
     float a0;  float a1;  float a2;  unsigned int a3;  d2d_nb_type_ * a4; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1,fpcvt_f,&a2,fpcvt_ui,&a3,fpcvt_d2d_nb_copy,&a4))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = piw::make_d2d_table  (  a0  ,a1  ,a2  ,a3  ,*a4 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in make_d2d_table");
    }

    return 0;
}


static const char *module_method_make_pitchbender_docstring_ = "";

static PyObject *module_method_make_pitchbender(PyObject *self, PyObject *args)
{
     float a0;  float a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_f,&a0,fpcvt_f,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    dd2d_nb_type_ r = piw::make_pitchbender  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_dd2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in make_pitchbender");
    }

    return 0;
}


static const char *module_method_step_bucket_docstring_ = "";

static PyObject *module_method_step_bucket(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    f2f_type_ r = piw::step_bucket  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f2f_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in step_bucket");
    }

    return 0;
}


static const char *module_method_dump_client_docstring_ = "";

static PyObject *module_method_dump_client(PyObject *self, PyObject *args)
{
     client_type_ * a0;  unsigned int a1;  bool a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_client,&a0,fpcvt_ui,&a1,fpcvt_b,&a2))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::dump_client  (  a0  ,a1  ,a2  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in dump_client");
    }

    return 0;
}


static const char *module_method_parse_clause_docstring_ = "";

static PyObject *module_method_parse_clause(PyObject *self, PyObject *args)
{
     python_delegate_type_ * a0;  const char * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_python_delegate,&a0,fpcvt_str,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock(true);
                {
                    PyObject * r = piw::py_parse_clause  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in parse_clause");
    }

    return 0;
}


static const char *module_method_parse_clauselist_docstring_ = "";

static PyObject *module_method_parse_clauselist(PyObject *self, PyObject *args)
{
     python_delegate_type_ * a0;  const char * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_python_delegate,&a0,fpcvt_str,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock(true);
                {
                    PyObject * r = piw::py_parse_clauselist  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in parse_clauselist");
    }

    return 0;
}


static const char *module_method_parse_term_docstring_ = "";

static PyObject *module_method_parse_term(PyObject *self, PyObject *args)
{
     python_delegate_type_ * a0;  const char * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_python_delegate,&a0,fpcvt_str,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock(true);
                {
                    PyObject * r = piw::py_parse_term  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in parse_term");
    }

    return 0;
}


static const char *module_method_parse_termlist_docstring_ = "";

static PyObject *module_method_parse_termlist(PyObject *self, PyObject *args)
{
     python_delegate_type_ * a0;  const char * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_python_delegate,&a0,fpcvt_str,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock(true);
                {
                    PyObject * r = piw::py_parse_termlist  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_py((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in parse_termlist");
    }

    return 0;
}


static const char *module_method_tsd_killed_docstring_ = "";

static PyObject *module_method_tsd_killed(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    bool r = piw::tsd_killed  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_killed");
    }

    return 0;
}


static const char *module_method_tsd_exit_docstring_ = "";

static PyObject *module_method_tsd_exit(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_exit  (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_exit");
    }

    return 0;
}


static const char *module_method_tsd_dump_docstring_ = "";

static PyObject *module_method_tsd_dump(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_dump  (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_dump");
    }

    return 0;
}


static const char *module_method_tsd_lock_docstring_ = "";

static PyObject *module_method_tsd_lock(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_lock  (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_lock");
    }

    return 0;
}


static const char *module_method_tsd_unlock_docstring_ = "";

static PyObject *module_method_tsd_unlock(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_unlock  (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_unlock");
    }

    return 0;
}


static const char *module_method_tsd_client_docstring_ = "";

static PyObject *module_method_tsd_client(PyObject *self, PyObject *args)
{
     const char * a0;  client_type_ * a1;  bool a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_str,&a0,fpcvt_client,&a1,fpcvt_b,&a2))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_client  (  a0  ,a1  ,a2  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_client");
    }

    return 0;
}


static const char *module_method_tsd_server_docstring_ = "";

static PyObject *module_method_tsd_server(PyObject *self, PyObject *args)
{
     const char * a0;  server_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_server,&a1))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_server  (  a0  ,a1  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_server");
    }

    return 0;
}


static const char *module_method_tsd_fastdata_docstring_ = "";

static PyObject *module_method_tsd_fastdata(PyObject *self, PyObject *args)
{
     fastdata_type_ * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_fastdata,&a0))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_fastdata  (  a0  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_fastdata");
    }

    return 0;
}


static const char *module_method_tsd_dataqueue_docstring_ = "";

static PyObject *module_method_tsd_dataqueue(PyObject *self, PyObject *args)
{
     unsigned int a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    dataqueue_type_ r = piw::tsd_dataqueue  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_dataqueue_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_dataqueue");
    }

    return 0;
}


static const char *module_method_tsd_thing_docstring_ = "";

static PyObject *module_method_tsd_thing(PyObject *self, PyObject *args)
{
     thing_type_ * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_thing,&a0))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_thing  (  a0  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_thing");
    }

    return 0;
}


static const char *module_method_tsd_window_docstring_ = "";

static PyObject *module_method_tsd_window(PyObject *self, PyObject *args)
{
     window_type_ * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_window,&a0))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_window  (  a0  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_window");
    }

    return 0;
}


static const char *module_method_tsd_index_docstring_ = "";

static PyObject *module_method_tsd_index(PyObject *self, PyObject *args)
{
     const char * a0;  index_type_ * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_index,&a1))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_index  (  a0  ,a1  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_index");
    }

    return 0;
}


static const char *module_method_tsd_log_docstring_ = "";

static PyObject *module_method_tsd_log(PyObject *self, PyObject *args)
{
     const char * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_str,&a0))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_log  (  a0  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_log");
    }

    return 0;
}


static const char *module_method_tsd_alert_docstring_ = "";

static PyObject *module_method_tsd_alert(PyObject *self, PyObject *args)
{
     const char * a0;  const char * a1;  const char * a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_str,&a0,fpcvt_str,&a1,fpcvt_str,&a2))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_alert  (  a0  ,a1  ,a2  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_alert");
    }

    return 0;
}


static const char *module_method_tsd_rpcserver_docstring_ = "";

static PyObject *module_method_tsd_rpcserver(PyObject *self, PyObject *args)
{
     rpcserver_type_ * a0;  const char * a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_rpcserver,&a0,fpcvt_str,&a1))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_rpcserver  (  a0  ,a1  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_rpcserver");
    }

    return 0;
}


static const char *module_method_tsd_rpcclient_docstring_ = "";

static PyObject *module_method_tsd_rpcclient(PyObject *self, PyObject *args)
{
     rpcclient_type_ * a0;  const char * a1;  data_type_ * a2;  data_type_ * a3;  data_type_ * a4;  unsigned long a5; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&O&O&" ,fpcvt_rpcclient,&a0,fpcvt_str,&a1,fpcvt_data_copy,&a2,fpcvt_data_copy,&a3,fpcvt_data_copy,&a4,fpcvt_ul,&a5))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_rpcclient  (  a0  ,a1  ,*a2  ,*a3  ,*a4  ,a5  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_rpcclient");
    }

    return 0;
}


static const char *module_method_tsd_user_docstring_ = "";

static PyObject *module_method_tsd_user(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = piw::tsd_user  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_user");
    }

    return 0;
}


static const char *module_method_tsd_time_docstring_ = "";

static PyObject *module_method_tsd_time(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    unsigned long long r = piw::tsd_time  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ull((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_time");
    }

    return 0;
}


static const char *module_method_tsd_clocksource_docstring_ = "";

static PyObject *module_method_tsd_clocksource(PyObject *self, PyObject *args)
{
     data_type_ * a0;  unsigned int a1;  unsigned int a2;  clocksource_type_ * a3; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_data_copy,&a0,fpcvt_ui,&a1,fpcvt_ui,&a2,fpcvt_clocksource,&a3))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             tsd_clocksource (  *a0  ,a1  ,a2  ,a3  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_clocksource");
    }

    return 0;
}


static const char *module_method_tsd_clearcontext_docstring_ = "";

static PyObject *module_method_tsd_clearcontext(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            piw::tsd_clearcontext  (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in tsd_clearcontext");
    }

    return 0;
}




static const char *module_method_make_d2b_functor_docstring_ = ""
"    make_d2b_functor(callable) -> new d2b functor"
"    "
"    Make d2b functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_d2b_functor(PyObject *self, PyObject *args)
{

   return piw_make_d2b_functor(args);

}

static PyObject *module_method_make_d2b_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_d2b_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_d2b_functor");
        return 0;
    }
}


static const char *module_method_make_d2b_nb_functor_docstring_ = ""
"    make_d2b_nb_functor(callable) -> new d2b_nb functor"
"    "
"    Make d2b_nb functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_d2b_nb_functor(PyObject *self, PyObject *args)
{

   return piw_make_d2b_nb_functor(args);

}

static PyObject *module_method_make_d2b_nb_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_d2b_nb_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_d2b_nb_functor");
        return 0;
    }
}


static const char *module_method_make_d2d_functor_docstring_ = ""
"    make_d2d_functor(callable) -> new d2d functor"
"    "
"    Make d2d functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_d2d_functor(PyObject *self, PyObject *args)
{

   return piw_make_d2d_functor(args);

}

static PyObject *module_method_make_d2d_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_d2d_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_d2d_functor");
        return 0;
    }
}


static const char *module_method_make_d2d_nb_functor_docstring_ = ""
"    make_d2d_nb_functor(callable) -> new d2d_nb functor"
"    "
"    Make d2d_nb functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_d2d_nb_functor(PyObject *self, PyObject *args)
{

   return piw_make_d2d_nb_functor(args);

}

static PyObject *module_method_make_d2d_nb_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_d2d_nb_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_d2d_nb_functor");
        return 0;
    }
}


static const char *module_method_make_dd2d_functor_docstring_ = ""
"    make_dd2d_functor(callable) -> new dd2d functor"
"    "
"    Make dd2d functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_dd2d_functor(PyObject *self, PyObject *args)
{

   return piw_make_dd2d_functor(args);

}

static PyObject *module_method_make_dd2d_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_dd2d_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_dd2d_functor");
        return 0;
    }
}


static const char *module_method_make_dd2d_nb_functor_docstring_ = ""
"    make_dd2d_nb_functor(callable) -> new dd2d functor"
"    "
"    Make dd2d_nb functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_dd2d_nb_functor(PyObject *self, PyObject *args)
{

   return piw_make_dd2d_nb_functor(args);

}

static PyObject *module_method_make_dd2d_nb_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_dd2d_nb_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_dd2d_nb_functor");
        return 0;
    }
}


static const char *module_method_make_change_functor_docstring_ = ""
"    make_change_functor(callable) -> new change functor"
"    "
"    Make change functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_change_functor(PyObject *self, PyObject *args)
{

   return piw_make_change_functor(args);

}

static PyObject *module_method_make_change_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_change_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_change_functor");
        return 0;
    }
}


static const char *module_method_make_change_nb_functor_docstring_ = ""
"    make_change_nb_functor(callable) -> new change_nb functor"
"    "
"    Make change_nb functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_change_nb_functor(PyObject *self, PyObject *args)
{

   return piw_make_change_nb_functor(args);

}

static PyObject *module_method_make_change_nb_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_change_nb_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_change_nb_functor");
        return 0;
    }
}


static const char *module_method_setenv_docstring_ = ""
"    setenv(environment) -> ."
"    "
"    Initialise piw during instance creation.  libpiw requires certain"
"    thread specific data to have been initialised in order to function."
"    Normally, this is taken care of automatically, but during agent "
"    initialisation this is not the case."
"    "
"    This function should always be called by an agents main function "
"    prior to doing anything else, using the environment supplied to main"
"    "
""
;

static PyObject *module_method_inner_setenv(PyObject *self, PyObject *args)
{

   PyObject *o; bct_entity_t e;
   if(!PyArg_ParseTuple(args,"O",&o)) { return 0; }
   if(!(e=(bct_entity_t)PyCObject_AsVoidPtr(o))) { PyErr_SetString(PyExc_TypeError,"expected CObject"); return 0; }
   piw::tsd_setcontext(e);
   Py_INCREF(Py_None); return Py_None;

}

static PyObject *module_method_setenv(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_setenv(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::setenv");
        return 0;
    }
}



static PyMethodDef module_method_table[] = {
    
{ "create_samplearray",module_method_create_samplearray,METH_VARARGS, (char *)module_method_create_samplearray_docstring_ }, 
{ "create_sample",module_method_create_sample,METH_VARARGS, (char *)module_method_create_sample_docstring_ }, 
{ "create_zone",module_method_create_zone,METH_VARARGS, (char *)module_method_create_zone_docstring_ }, 
{ "create_preset",module_method_create_preset,METH_VARARGS, (char *)module_method_create_preset_docstring_ }, 
{ "root_filter",module_method_root_filter,METH_VARARGS, (char *)module_method_root_filter_docstring_ }, 
{ "null_filter",module_method_null_filter,METH_VARARGS, (char *)module_method_null_filter_docstring_ }, 
{ "signal_dsc_filter",module_method_signal_dsc_filter,METH_VARARGS, (char *)module_method_signal_dsc_filter_docstring_ }, 
{ "signal_cnc_filter",module_method_signal_cnc_filter,METH_VARARGS, (char *)module_method_signal_cnc_filter_docstring_ }, 
{ "aggregation_filter",module_method_aggregation_filter,METH_VARARGS, (char *)module_method_aggregation_filter_docstring_ }, 
{ "aggregation_filter3",module_method_aggregation_filter3,METH_VARARGS, (char *)module_method_aggregation_filter3_docstring_ }, 
{ "deaggregation_filter",module_method_deaggregation_filter,METH_VARARGS, (char *)module_method_deaggregation_filter_docstring_ }, 
{ "grist_aggregation_filter",module_method_grist_aggregation_filter,METH_VARARGS, (char *)module_method_grist_aggregation_filter_docstring_ }, 
{ "gristchaff_aggregation_filter",module_method_gristchaff_aggregation_filter,METH_VARARGS, (char *)module_method_gristchaff_aggregation_filter_docstring_ }, 
{ "grist_deaggregation_filter",module_method_grist_deaggregation_filter,METH_VARARGS, (char *)module_method_grist_deaggregation_filter_docstring_ }, 
{ "grist_deaggregation_filter2",module_method_grist_deaggregation_filter2,METH_VARARGS, (char *)module_method_grist_deaggregation_filter2_docstring_ }, 
{ "last_gt_filter",module_method_last_gt_filter,METH_VARARGS, (char *)module_method_last_gt_filter_docstring_ }, 
{ "last_lt_filter",module_method_last_lt_filter,METH_VARARGS, (char *)module_method_last_lt_filter_docstring_ }, 
{ "last_filter",module_method_last_filter,METH_VARARGS, (char *)module_method_last_filter_docstring_ }, 
{ "grist_filter",module_method_grist_filter,METH_VARARGS, (char *)module_method_grist_filter_docstring_ }, 
{ "dict_merger",module_method_dict_merger,METH_VARARGS, (char *)module_method_dict_merger_docstring_ }, 
{ "null_converter",module_method_null_converter,METH_VARARGS, (char *)module_method_null_converter_docstring_ }, 
{ "throttling_converter",module_method_throttling_converter,METH_VARARGS, (char *)module_method_throttling_converter_docstring_ }, 
{ "triggering_converter",module_method_triggering_converter,METH_VARARGS, (char *)module_method_triggering_converter_docstring_ }, 
{ "impulse_converter",module_method_impulse_converter,METH_VARARGS, (char *)module_method_impulse_converter_docstring_ }, 
{ "resampling_converter",module_method_resampling_converter,METH_VARARGS, (char *)module_method_resampling_converter_docstring_ }, 
{ "interpolating_converter",module_method_interpolating_converter,METH_VARARGS, (char *)module_method_interpolating_converter_docstring_ }, 
{ "lopass_converter",module_method_lopass_converter,METH_VARARGS, (char *)module_method_lopass_converter_docstring_ }, 
{ "deferred_sender",module_method_deferred_sender,METH_VARARGS, (char *)module_method_deferred_sender_docstring_ }, 
{ "slowchange",module_method_slowchange,METH_VARARGS, (char *)module_method_slowchange_docstring_ }, 
{ "slowchange_polled",module_method_slowchange_polled,METH_VARARGS, (char *)module_method_slowchange_polled_docstring_ }, 
{ "slowtrigger",module_method_slowtrigger,METH_VARARGS, (char *)module_method_slowtrigger_docstring_ }, 
{ "make_change_normal",module_method_make_change_normal,METH_VARARGS, (char *)module_method_make_change_normal_docstring_ }, 
{ "make_change_nb",module_method_make_change_nb,METH_VARARGS, (char *)module_method_make_change_nb_docstring_ }, 
{ "first_filter",module_method_first_filter,METH_VARARGS, (char *)module_method_first_filter_docstring_ }, 
{ "indirectchange",module_method_indirectchange,METH_VARARGS, (char *)module_method_indirectchange_docstring_ }, 
{ "indirectchange_set",module_method_indirectchange_set,METH_VARARGS, (char *)module_method_indirectchange_set_docstring_ }, 
{ "indirectchange_clear",module_method_indirectchange_clear,METH_VARARGS, (char *)module_method_indirectchange_clear_docstring_ }, 
{ "indirectchange_nb",module_method_indirectchange_nb,METH_VARARGS, (char *)module_method_indirectchange_nb_docstring_ }, 
{ "indirectchange_set_nb",module_method_indirectchange_set_nb,METH_VARARGS, (char *)module_method_indirectchange_set_nb_docstring_ }, 
{ "indirectchange_clear_nb",module_method_indirectchange_clear_nb,METH_VARARGS, (char *)module_method_indirectchange_clear_nb_docstring_ }, 
{ "changelist",module_method_changelist,METH_VARARGS, (char *)module_method_changelist_docstring_ }, 
{ "changelist_connect",module_method_changelist_connect,METH_VARARGS, (char *)module_method_changelist_connect_docstring_ }, 
{ "changelist_disconnect",module_method_changelist_disconnect,METH_VARARGS, (char *)module_method_changelist_disconnect_docstring_ }, 
{ "changelist_nb",module_method_changelist_nb,METH_VARARGS, (char *)module_method_changelist_nb_docstring_ }, 
{ "changelist_connect_nb",module_method_changelist_connect_nb,METH_VARARGS, (char *)module_method_changelist_connect_nb_docstring_ }, 
{ "changelist_disconnect_nb",module_method_changelist_disconnect_nb,METH_VARARGS, (char *)module_method_changelist_disconnect_nb_docstring_ }, 
{ "change2",module_method_change2,METH_VARARGS, (char *)module_method_change2_docstring_ }, 
{ "change2_nb",module_method_change2_nb,METH_VARARGS, (char *)module_method_change2_nb_docstring_ }, 
{ "change_nb2",module_method_change_nb2,METH_VARARGS, (char *)module_method_change_nb2_docstring_ }, 
{ "trigger",module_method_trigger,METH_VARARGS, (char *)module_method_trigger_docstring_ }, 
{ "parse_state_term",module_method_parse_state_term,METH_VARARGS, (char *)module_method_parse_state_term_docstring_ }, 
{ "makefloat_nb",module_method_makefloat_nb,METH_VARARGS, (char *)module_method_makefloat_nb_docstring_ }, 
{ "makenull_nb",module_method_makenull_nb,METH_VARARGS, (char *)module_method_makenull_nb_docstring_ }, 
{ "makestring_len_nb",module_method_makestring_len_nb,METH_VARARGS, (char *)module_method_makestring_len_nb_docstring_ }, 
{ "makestring_nb",module_method_makestring_nb,METH_VARARGS, (char *)module_method_makestring_nb_docstring_ }, 
{ "makebool_nb",module_method_makebool_nb,METH_VARARGS, (char *)module_method_makebool_nb_docstring_ }, 
{ "makelong_nb",module_method_makelong_nb,METH_VARARGS, (char *)module_method_makelong_nb_docstring_ }, 
{ "pathone_nb",module_method_pathone_nb,METH_VARARGS, (char *)module_method_pathone_nb_docstring_ }, 
{ "pathnull_nb",module_method_pathnull_nb,METH_VARARGS, (char *)module_method_pathnull_nb_docstring_ }, 
{ "dictnull",module_method_dictnull,METH_VARARGS, (char *)module_method_dictnull_docstring_ }, 
{ "dictset",module_method_dictset,METH_VARARGS, (char *)module_method_dictset_docstring_ }, 
{ "dictupdate",module_method_dictupdate,METH_VARARGS, (char *)module_method_dictupdate_docstring_ }, 
{ "dictdel",module_method_dictdel,METH_VARARGS, (char *)module_method_dictdel_docstring_ }, 
{ "dictnull_nb",module_method_dictnull_nb,METH_VARARGS, (char *)module_method_dictnull_nb_docstring_ }, 
{ "dictset_nb",module_method_dictset_nb,METH_VARARGS, (char *)module_method_dictset_nb_docstring_ }, 
{ "parsepath_nb",module_method_parsepath_nb,METH_VARARGS, (char *)module_method_parsepath_nb_docstring_ }, 
{ "makepath_nb",module_method_makepath_nb,METH_VARARGS, (char *)module_method_makepath_nb_docstring_ }, 
{ "makedouble_bounded_units_nb",module_method_makedouble_bounded_units_nb,METH_VARARGS, (char *)module_method_makedouble_bounded_units_nb_docstring_ }, 
{ "makefloat_bounded_units_nb",module_method_makefloat_bounded_units_nb,METH_VARARGS, (char *)module_method_makefloat_bounded_units_nb_docstring_ }, 
{ "makelong_bounded_units_nb",module_method_makelong_bounded_units_nb,METH_VARARGS, (char *)module_method_makelong_bounded_units_nb_docstring_ }, 
{ "makedouble_bounded_nb",module_method_makedouble_bounded_nb,METH_VARARGS, (char *)module_method_makedouble_bounded_nb_docstring_ }, 
{ "makefloat_bounded_nb",module_method_makefloat_bounded_nb,METH_VARARGS, (char *)module_method_makefloat_bounded_nb_docstring_ }, 
{ "makelong_bounded_nb",module_method_makelong_bounded_nb,METH_VARARGS, (char *)module_method_makelong_bounded_nb_docstring_ }, 
{ "makefloat",module_method_makefloat,METH_VARARGS, (char *)module_method_makefloat_docstring_ }, 
{ "makenull",module_method_makenull,METH_VARARGS, (char *)module_method_makenull_docstring_ }, 
{ "makestring_len",module_method_makestring_len,METH_VARARGS, (char *)module_method_makestring_len_docstring_ }, 
{ "makestring",module_method_makestring,METH_VARARGS, (char *)module_method_makestring_docstring_ }, 
{ "makeblob2",module_method_makeblob2,METH_VARARGS, (char *)module_method_makeblob2_docstring_ }, 
{ "makebool",module_method_makebool,METH_VARARGS, (char *)module_method_makebool_docstring_ }, 
{ "makelong",module_method_makelong,METH_VARARGS, (char *)module_method_makelong_docstring_ }, 
{ "pathone",module_method_pathone,METH_VARARGS, (char *)module_method_pathone_docstring_ }, 
{ "pathnull",module_method_pathnull,METH_VARARGS, (char *)module_method_pathnull_docstring_ }, 
{ "parsepath",module_method_parsepath,METH_VARARGS, (char *)module_method_parsepath_docstring_ }, 
{ "makepath",module_method_makepath,METH_VARARGS, (char *)module_method_makepath_docstring_ }, 
{ "makedouble_bounded",module_method_makedouble_bounded,METH_VARARGS, (char *)module_method_makedouble_bounded_docstring_ }, 
{ "makefloat_bounded",module_method_makefloat_bounded,METH_VARARGS, (char *)module_method_makefloat_bounded_docstring_ }, 
{ "makelong_bounded",module_method_makelong_bounded,METH_VARARGS, (char *)module_method_makelong_bounded_docstring_ }, 
{ "makedouble_bounded_units",module_method_makedouble_bounded_units,METH_VARARGS, (char *)module_method_makedouble_bounded_units_docstring_ }, 
{ "makefloat_bounded_units",module_method_makefloat_bounded_units,METH_VARARGS, (char *)module_method_makefloat_bounded_units_docstring_ }, 
{ "makelong_bounded_units",module_method_makelong_bounded_units,METH_VARARGS, (char *)module_method_makelong_bounded_units_docstring_ }, 
{ "normalise",module_method_normalise,METH_VARARGS, (char *)module_method_normalise_docstring_ }, 
{ "denormalise",module_method_denormalise,METH_VARARGS, (char *)module_method_denormalise_docstring_ }, 
{ "address2server",module_method_address2server,METH_VARARGS, (char *)module_method_address2server_docstring_ }, 
{ "bint_normalizer",module_method_bint_normalizer,METH_VARARGS, (char *)module_method_bint_normalizer_docstring_ }, 
{ "bint_denormalizer",module_method_bint_denormalizer,METH_VARARGS, (char *)module_method_bint_denormalizer_docstring_ }, 
{ "bfloat_normalizer",module_method_bfloat_normalizer,METH_VARARGS, (char *)module_method_bfloat_normalizer_docstring_ }, 
{ "bfloat_denormalizer",module_method_bfloat_denormalizer,METH_VARARGS, (char *)module_method_bfloat_denormalizer_docstring_ }, 
{ "bool_normalizer",module_method_bool_normalizer,METH_VARARGS, (char *)module_method_bool_normalizer_docstring_ }, 
{ "bool_denormalizer",module_method_bool_denormalizer,METH_VARARGS, (char *)module_method_bool_denormalizer_docstring_ }, 
{ "null_normalizer",module_method_null_normalizer,METH_VARARGS, (char *)module_method_null_normalizer_docstring_ }, 
{ "null_denormalizer",module_method_null_denormalizer,METH_VARARGS, (char *)module_method_null_denormalizer_docstring_ }, 
{ "string_normalizer",module_method_string_normalizer,METH_VARARGS, (char *)module_method_string_normalizer_docstring_ }, 
{ "string_denormalizer",module_method_string_denormalizer,METH_VARARGS, (char *)module_method_string_denormalizer_docstring_ }, 
{ "d2d_chain",module_method_d2d_chain,METH_VARARGS, (char *)module_method_d2d_chain_docstring_ }, 
{ "d2d_convert",module_method_d2d_convert,METH_VARARGS, (char *)module_method_d2d_convert_docstring_ }, 
{ "fastchange",module_method_fastchange,METH_VARARGS, (char *)module_method_fastchange_docstring_ }, 
{ "add",module_method_add,METH_VARARGS, (char *)module_method_add_docstring_ }, 
{ "make_d2d_const",module_method_make_d2d_const,METH_VARARGS, (char *)module_method_make_d2d_const_docstring_ }, 
{ "make_f2f_identity",module_method_make_f2f_identity,METH_VARARGS, (char *)module_method_make_f2f_identity_docstring_ }, 
{ "make_f2f_table",module_method_make_f2f_table,METH_VARARGS, (char *)module_method_make_f2f_table_docstring_ }, 
{ "make_d2d_table",module_method_make_d2d_table,METH_VARARGS, (char *)module_method_make_d2d_table_docstring_ }, 
{ "make_pitchbender",module_method_make_pitchbender,METH_VARARGS, (char *)module_method_make_pitchbender_docstring_ }, 
{ "step_bucket",module_method_step_bucket,METH_VARARGS, (char *)module_method_step_bucket_docstring_ }, 
{ "dump_client",module_method_dump_client,METH_VARARGS, (char *)module_method_dump_client_docstring_ }, 
{ "parse_clause",module_method_parse_clause,METH_VARARGS, (char *)module_method_parse_clause_docstring_ }, 
{ "parse_clauselist",module_method_parse_clauselist,METH_VARARGS, (char *)module_method_parse_clauselist_docstring_ }, 
{ "parse_term",module_method_parse_term,METH_VARARGS, (char *)module_method_parse_term_docstring_ }, 
{ "parse_termlist",module_method_parse_termlist,METH_VARARGS, (char *)module_method_parse_termlist_docstring_ }, 
{ "tsd_killed",module_method_tsd_killed,METH_VARARGS, (char *)module_method_tsd_killed_docstring_ }, 
{ "tsd_exit",module_method_tsd_exit,METH_VARARGS, (char *)module_method_tsd_exit_docstring_ }, 
{ "tsd_dump",module_method_tsd_dump,METH_VARARGS, (char *)module_method_tsd_dump_docstring_ }, 
{ "tsd_lock",module_method_tsd_lock,METH_VARARGS, (char *)module_method_tsd_lock_docstring_ }, 
{ "tsd_unlock",module_method_tsd_unlock,METH_VARARGS, (char *)module_method_tsd_unlock_docstring_ }, 
{ "tsd_client",module_method_tsd_client,METH_VARARGS, (char *)module_method_tsd_client_docstring_ }, 
{ "tsd_server",module_method_tsd_server,METH_VARARGS, (char *)module_method_tsd_server_docstring_ }, 
{ "tsd_fastdata",module_method_tsd_fastdata,METH_VARARGS, (char *)module_method_tsd_fastdata_docstring_ }, 
{ "tsd_dataqueue",module_method_tsd_dataqueue,METH_VARARGS, (char *)module_method_tsd_dataqueue_docstring_ }, 
{ "tsd_thing",module_method_tsd_thing,METH_VARARGS, (char *)module_method_tsd_thing_docstring_ }, 
{ "tsd_window",module_method_tsd_window,METH_VARARGS, (char *)module_method_tsd_window_docstring_ }, 
{ "tsd_index",module_method_tsd_index,METH_VARARGS, (char *)module_method_tsd_index_docstring_ }, 
{ "tsd_log",module_method_tsd_log,METH_VARARGS, (char *)module_method_tsd_log_docstring_ }, 
{ "tsd_alert",module_method_tsd_alert,METH_VARARGS, (char *)module_method_tsd_alert_docstring_ }, 
{ "tsd_rpcserver",module_method_tsd_rpcserver,METH_VARARGS, (char *)module_method_tsd_rpcserver_docstring_ }, 
{ "tsd_rpcclient",module_method_tsd_rpcclient,METH_VARARGS, (char *)module_method_tsd_rpcclient_docstring_ }, 
{ "tsd_user",module_method_tsd_user,METH_VARARGS, (char *)module_method_tsd_user_docstring_ }, 
{ "tsd_time",module_method_tsd_time,METH_VARARGS, (char *)module_method_tsd_time_docstring_ }, 
{ "tsd_clocksource",module_method_tsd_clocksource,METH_VARARGS, (char *)module_method_tsd_clocksource_docstring_ }, 
{ "tsd_clearcontext",module_method_tsd_clearcontext,METH_VARARGS, (char *)module_method_tsd_clearcontext_docstring_ }, 
    
{ "make_d2b_functor",module_method_make_d2b_functor,METH_VARARGS, (char *)module_method_make_d2b_functor_docstring_ }, 
{ "make_d2b_nb_functor",module_method_make_d2b_nb_functor,METH_VARARGS, (char *)module_method_make_d2b_nb_functor_docstring_ }, 
{ "make_d2d_functor",module_method_make_d2d_functor,METH_VARARGS, (char *)module_method_make_d2d_functor_docstring_ }, 
{ "make_d2d_nb_functor",module_method_make_d2d_nb_functor,METH_VARARGS, (char *)module_method_make_d2d_nb_functor_docstring_ }, 
{ "make_dd2d_functor",module_method_make_dd2d_functor,METH_VARARGS, (char *)module_method_make_dd2d_functor_docstring_ }, 
{ "make_dd2d_nb_functor",module_method_make_dd2d_nb_functor,METH_VARARGS, (char *)module_method_make_dd2d_nb_functor_docstring_ }, 
{ "make_change_functor",module_method_make_change_functor,METH_VARARGS, (char *)module_method_make_change_functor_docstring_ }, 
{ "make_change_nb_functor",module_method_make_change_nb_functor,METH_VARARGS, (char *)module_method_make_change_nb_functor_docstring_ }, 
{ "setenv",module_method_setenv,METH_VARARGS, (char *)module_method_setenv_docstring_ }, 
    {0}
};
};


struct piw_change_sink_t: public piw::change_t::sinktype_t
{
    piw_change_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~piw_change_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return true;
    }

    void invoke(const piw::data_t &d) const
    {
        lock_c2p lock(_interp);

        PyObject *r = PyObject_CallFunction(_callable,(char *)"O&",tpcvt_data_copy,&d);

        if(r) Py_DECREF(r);
        else PyErr_Print();

        if(!r)
        {
            std::cerr << "Functor with exception " ;
            PyObject_Print(_callable,stderr,0);
            std::cerr << std::endl;
            PIC_THROW("python exception inside functor ");
        }
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

PyObject *piw_make_change_functor(PyObject *args)
{
    PyObject *o;

    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make change functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    piw::change_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new piw::change_t(pic::ref(new piw_change_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return change_wrap_(c,true);
}

struct piw_change_nb_sink_t: public piw::change_nb_t::sinktype_t
{
    piw_change_nb_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~piw_change_nb_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return true;
    }

    void invoke(const piw::data_nb_t &d) const
    {
        lock_c2p lock(_interp);

        PyObject *r = PyObject_CallFunction(_callable,(char *)"O&",tpcvt_data_copy,&d);

        if(r) Py_DECREF(r);
        else PyErr_Print();

        if(!r)
        {
            std::cerr << "Functor with exception " ;
            PyObject_Print(_callable,stderr,0);
            std::cerr << std::endl;
            PIC_THROW("python exception inside functor ");
        }
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

PyObject *piw_make_change_nb_functor(PyObject *args)
{
    PyObject *o;

    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make change functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    piw::change_nb_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new piw::change_nb_t(pic::ref(new piw_change_nb_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return change_nb_wrap_(c,true);
}

struct piw_d2d_sink_t: public piw::d2d_t::sinktype_t
{
    piw_d2d_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~piw_d2d_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return true;
    }

    piw::data_t invoke(const piw::data_t &d) const
    {
        lock_c2p lock(_interp);

        PyObject *r = PyObject_CallFunction(_callable,(char *)"O&",tpcvt_data_copy,&d);

        if(!r)
        {
            PyErr_Print();
            PIC_THROW("python exception inside functor");
        }

        piw::data_t *rvp;

        if(!fpcvt_data(r,&rvp))
        {
            Py_DECREF(r);
            PyErr_Print();
            PIC_THROW("incorrect (non data) return type from functor");
        }

        {   unlock_p2c unlock;
            {
                piw::data_t rv(*rvp);
                {   relock_p2c relock(unlock);
                    Py_DECREF(r);
                }
                return rv;
            }
        }
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

PyObject *piw_make_d2d_functor(PyObject *args)
{
    PyObject *o;

    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make d2d functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    piw::d2d_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new piw::d2d_t(pic::ref(new piw_d2d_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return d2d_wrap_(c,true);
}

struct piw_d2d_nb_sink_t: public piw::d2d_nb_t::sinktype_t
{
    piw_d2d_nb_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~piw_d2d_nb_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return true;
    }

    piw::data_nb_t invoke(const piw::data_nb_t &d) const
    {
        lock_c2p lock(_interp);

        PyObject *r = PyObject_CallFunction(_callable,(char *)"O&",tpcvt_data_copy,&d);

        if(!r)
        {
            PyErr_Print();
            PIC_THROW("python exception inside functor");
        }

        piw::data_nb_t *rvp;

        if(!fpcvt_data(r,&rvp))
        {
            Py_DECREF(r);
            PyErr_Print();
            PIC_THROW("incorrect (non data) return type from functor");
        }

        {   unlock_p2c unlock;
            {
                piw::data_nb_t rv(*rvp);
                {   relock_p2c relock(unlock);
                    Py_DECREF(r);
                }
                return rv;
            }
        }
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

PyObject *piw_make_d2d_nb_functor(PyObject *args)
{
    PyObject *o;

    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make d2d functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    piw::d2d_nb_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new piw::d2d_nb_t(pic::ref(new piw_d2d_nb_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return d2d_nb_wrap_(c,true);
}

struct piw_d2b_sink_t: public piw::d2b_t::sinktype_t
{
    piw_d2b_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~piw_d2b_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return true;
    }

    bool invoke(const piw::data_t &d) const
    {
        lock_c2p lock(_interp);

        PyObject *r = PyObject_CallFunction(_callable,(char *)"O&",tpcvt_data_copy,&d);

        if(!r)
        {
            PyErr_Print();
            PIC_THROW("python exception inside functor");
        }

        bool rv;

        if(!fpcvt_b(r,&rv))
        {
            Py_DECREF(r);
            PyErr_Print();
            PIC_THROW("incorrect (non bool) return type from functor");
        }

        Py_DECREF(r);
        return rv;
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

PyObject *piw_make_d2b_functor(PyObject *args)
{
    PyObject *o;

    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make d2b functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    piw::d2b_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new piw::d2b_t(pic::ref(new piw_d2b_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return d2b_wrap_(c,true);
}

struct piw_d2b_nb_sink_t: public piw::d2b_nb_t::sinktype_t
{
    piw_d2b_nb_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~piw_d2b_nb_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return true;
    }

    bool invoke(const piw::data_nb_t &d) const
    {
        lock_c2p lock(_interp);

        PyObject *r = PyObject_CallFunction(_callable,(char *)"O&",tpcvt_data_copy,&d);

        if(!r)
        {
            PyErr_Print();
            PIC_THROW("python exception inside functor");
        }

        bool rv;

        if(!fpcvt_b(r,&rv))
        {
            Py_DECREF(r);
            PyErr_Print();
            PIC_THROW("incorrect (non bool) return type from functor");
        }

        Py_DECREF(r);
        return rv;
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

PyObject *piw_make_d2b_nb_functor(PyObject *args)
{
    PyObject *o;

    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make d2b functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    piw::d2b_nb_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new piw::d2b_nb_t(pic::ref(new piw_d2b_nb_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return d2b_nb_wrap_(c,true);
}

struct piw_dd2d_sink_t: public piw::dd2d_t::sinktype_t
{
    piw_dd2d_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~piw_dd2d_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return true;
    }

    piw::data_t invoke(const piw::data_t &d1, const piw::data_t &d2) const
    {
        lock_c2p lock(_interp);

        PyObject *r = PyObject_CallFunction(_callable,(char *)"O&O&",tpcvt_data,&d1,tpcvt_data,&d2);

        if(!r)
        {
            PyErr_Print();
            PIC_THROW("python exception inside functor");
        }

        piw::data_t *rvp;

        if(!fpcvt_data(r,&rvp))
        {
            Py_DECREF(r);
            PyErr_Print();
            PIC_THROW("incorrect (non data) return type from functor");
        }

        {   unlock_p2c unlock;
            {
                piw::data_t rv(*rvp);
                {   relock_p2c relock(unlock);
                    Py_DECREF(r);
                }
                return rv;
            }
        }
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

static PyObject *piw_make_dd2d_functor(PyObject *args)
{
    PyObject *o;

    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make dd2d functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    piw::dd2d_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new piw::dd2d_t(pic::ref(new piw_dd2d_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return dd2d_wrap_(c,true);
}

struct piw_dd2d_nb_sink_t: public piw::dd2d_nb_t::sinktype_t
{
    piw_dd2d_nb_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~piw_dd2d_nb_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return true;
    }

    piw::data_nb_t invoke(const piw::data_nb_t &d1, const piw::data_nb_t &d2) const
    {
        lock_c2p lock(_interp);

        PyObject *r = PyObject_CallFunction(_callable,(char *)"O&O&",tpcvt_data,&d1,tpcvt_data,&d2);

        if(!r)
        {
            PyErr_Print();
            PIC_THROW("python exception inside functor");
        }

        piw::data_nb_t *rvp;

        if(!fpcvt_data(r,&rvp))
        {
            Py_DECREF(r);
            PyErr_Print();
            PIC_THROW("incorrect (non data) return type from functor");
        }

        {   unlock_p2c unlock;
            {
                piw::data_nb_t rv(*rvp);
                {   relock_p2c relock(unlock);
                    Py_DECREF(r);
                }
                return rv;
            }
        }
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

static PyObject *piw_make_dd2d_nb_functor(PyObject *args)
{
    PyObject *o;

    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make dd2d functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    piw::dd2d_nb_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new piw::dd2d_nb_t(pic::ref(new piw_dd2d_nb_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return dd2d_nb_wrap_(c,true);
}



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initpiw_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            event_dispatcher_ = &event_realdispatcher_;
        
    
        
            scheduler_dispatcher_ = &scheduler_realdispatcher_;
        
    
        
            rpctoken_dispatcher_ = &rpctoken_realdispatcher_;
        
    
        
            rpcserver_dispatcher_ = &rpcserver_realdispatcher_;
        
    
        
            rpcclient_dispatcher_ = &rpcclient_realdispatcher_;
        
    
        
            samplearray_dispatcher_ = &samplearray_realdispatcher_;
        
    
        
            sample_dispatcher_ = &sample_realdispatcher_;
        
    
        
            zone_dispatcher_ = &zone_realdispatcher_;
        
    
        
            preset_dispatcher_ = &preset_realdispatcher_;
        
    
        
            clockref_dispatcher_ = &clockref_realdispatcher_;
        
    
        
            strummer_dispatcher_ = &strummer_realdispatcher_;
        
    
        
            stringer_dispatcher_ = &stringer_realdispatcher_;
        
    
        
            cycler_dispatcher_ = &cycler_realdispatcher_;
        
    
        
            polyctl_dispatcher_ = &polyctl_realdispatcher_;
        
    
        
            stereomixer_dispatcher_ = &stereomixer_realdispatcher_;
        
    
        
            stereosummer_dispatcher_ = &stereosummer_realdispatcher_;
        
    
        
            monomixer_dispatcher_ = &monomixer_realdispatcher_;
        
    
        
            consolemixer_dispatcher_ = &consolemixer_realdispatcher_;
        
    
        
            gate_dispatcher_ = &gate_realdispatcher_;
        
    
        
            correlator_dispatcher_ = &correlator_realdispatcher_;
        
    
        
            connector_dispatcher_ = &connector_realdispatcher_;
        
    
        
            converter_ref_dispatcher_ = &converter_ref_realdispatcher_;
        
    
        
            capture_backend_dispatcher_ = &capture_backend_realdispatcher_;
        
    
        
            functor_backend_dispatcher_ = &functor_backend_realdispatcher_;
        
    
        
            controller_dispatcher_ = &controller_realdispatcher_;
        
    
        
            xxcontrolled_dispatcher_ = &xxcontrolled_realdispatcher_;
        
    
        
            controlled_dispatcher_ = &controlled_realdispatcher_;
        
    
        
            fasttrigger_dispatcher_ = &fasttrigger_realdispatcher_;
        
    
        
            cookie_dispatcher_ = &cookie_realdispatcher_;
        
    
        
            decode_ctl_dispatcher_ = &decode_ctl_realdispatcher_;
        
    
        
            decoder_dispatcher_ = &decoder_realdispatcher_;
        
    
        
            event_data_buffer_dispatcher_ = &event_data_buffer_realdispatcher_;
        
    
        
            event_data_source_dispatcher_ = &event_data_source_realdispatcher_;
        
    
        
            event_data_source_real_dispatcher_ = &event_data_source_real_realdispatcher_;
        
    
        
            wire_ctl_dispatcher_ = &wire_ctl_realdispatcher_;
        
    
        
            root_ctl_dispatcher_ = &root_ctl_realdispatcher_;
        
    
        
            wire_dispatcher_ = &wire_realdispatcher_;
        
    
        
            root_dispatcher_ = &root_realdispatcher_;
        
    
        
            multiplexer_dispatcher_ = &multiplexer_realdispatcher_;
        
    
        
            ranger_dispatcher_ = &ranger_realdispatcher_;
        
    
        
            lightconvertor_dispatcher_ = &lightconvertor_realdispatcher_;
        
    
        
            statusbuffer_dispatcher_ = &statusbuffer_realdispatcher_;
        
    
        
            lightsource_dispatcher_ = &lightsource_realdispatcher_;
        
    
        
            statusmixer_dispatcher_ = &statusmixer_realdispatcher_;
        
    
        
            controllerdict_dispatcher_ = &controllerdict_realdispatcher_;
        
    
        
            mapping_info_dispatcher_ = &mapping_info_realdispatcher_;
        
    
        
            midi_converter_dispatcher_ = &midi_converter_realdispatcher_;
        
    
        
            mapping_observer_dispatcher_ = &mapping_observer_realdispatcher_;
        
    
        
            kgroup_mapper_dispatcher_ = &kgroup_mapper_realdispatcher_;
        
    
        
            sclone_dispatcher_ = &sclone_realdispatcher_;
        
    
        
            clone_dispatcher_ = &clone_realdispatcher_;
        
    
        
            aggregator_dispatcher_ = &aggregator_realdispatcher_;
        
    
        
            scrolldelegate_dispatcher_ = &scrolldelegate_realdispatcher_;
        
    
        
            scroller_dispatcher_ = &scroller_realdispatcher_;
        
    
        
            scroller2_dispatcher_ = &scroller2_realdispatcher_;
        
    
        
            splitter_node_dispatcher_ = &splitter_node_realdispatcher_;
        
    
        
            splitter_dispatcher_ = &splitter_realdispatcher_;
        
    
        
            d2b_dispatcher_ = &d2b_realdispatcher_;
        
    
        
            d2b_nb_dispatcher_ = &d2b_nb_realdispatcher_;
        
    
        
            d2d_dispatcher_ = &d2d_realdispatcher_;
        
    
        
            d2d_nb_dispatcher_ = &d2d_nb_realdispatcher_;
        
    
        
            dd2d_dispatcher_ = &dd2d_realdispatcher_;
        
    
        
            dd2d_nb_dispatcher_ = &dd2d_nb_realdispatcher_;
        
    
        
            change_dispatcher_ = &change_realdispatcher_;
        
    
        
            change_nb_dispatcher_ = &change_nb_realdispatcher_;
        
    
        
            function1_dispatcher_ = &function1_realdispatcher_;
        
    
        
            function2_dispatcher_ = &function2_realdispatcher_;
        
    
        
            window_dispatcher_ = &window_realdispatcher_;
        
    
        
            thing_dispatcher_ = &thing_realdispatcher_;
        
    
        
            index_dispatcher_ = &index_realdispatcher_;
        
    
        
            server_dispatcher_ = &server_realdispatcher_;
        
    
        
            client_dispatcher_ = &client_realdispatcher_;
        
    
        
            subcanchor_dispatcher_ = &subcanchor_realdispatcher_;
        
    
        
            canchor_dispatcher_ = &canchor_realdispatcher_;
        
    
        
            anchor_dispatcher_ = &anchor_realdispatcher_;
        
    
        
            f_event_sender_dispatcher_ = &f_event_sender_realdispatcher_;
        
    
        
            f_event_receiver_dispatcher_ = &f_event_receiver_realdispatcher_;
        
    
        
            dataqueue_dispatcher_ = &dataqueue_realdispatcher_;
        
    
        
            fastdata_dispatcher_ = &fastdata_realdispatcher_;
        
    
        
            fastdata_receiver_dispatcher_ = &fastdata_receiver_realdispatcher_;
        
    
        
            term_dispatcher_ = &term_realdispatcher_;
        
    
        
            data_base_dispatcher_ = &data_base_realdispatcher_;
        
    
        
            data_dispatcher_ = &data_realdispatcher_;
        
    
        
            data_nb_dispatcher_ = &data_nb_realdispatcher_;
        
    
        
            clockdomain_ctl_dispatcher_ = &clockdomain_ctl_realdispatcher_;
        
    
        
            clocksink_dispatcher_ = &clocksink_realdispatcher_;
        
    
        
            clocksource_dispatcher_ = &clocksource_realdispatcher_;
        
    
        
            fast_pitchbender_dispatcher_ = &fast_pitchbender_realdispatcher_;
        
    
        
            panner_dispatcher_ = &panner_realdispatcher_;
        
    
        
            gain_dispatcher_ = &gain_realdispatcher_;
        
    
        
            delay_dispatcher_ = &delay_realdispatcher_;
        
    
        
            linear_gain_dispatcher_ = &linear_gain_realdispatcher_;
        
    
        
            selector_dispatcher_ = &selector_realdispatcher_;
        
    
        
            velocitydetect_dispatcher_ = &velocitydetect_realdispatcher_;
        
    
        
            scaler_controller_dispatcher_ = &scaler_controller_realdispatcher_;
        
    
        
            scaler_dispatcher_ = &scaler_realdispatcher_;
        
    
        
            evtdump_dispatcher_ = &evtdump_realdispatcher_;
        
    
        
            wavrecorder_dispatcher_ = &wavrecorder_realdispatcher_;
        
    
        
            python_delegate_dispatcher_ = &python_delegate_realdispatcher_;
        
    
        
            midi_channel_delegate_dispatcher_ = &midi_channel_delegate_realdispatcher_;
        
    
        
            midi_from_belcanto_dispatcher_ = &midi_from_belcanto_realdispatcher_;
        
    
        
            tsd_snapshot_dispatcher_ = &tsd_snapshot_realdispatcher_;
        
    
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("picross_native")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_notify_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_notify_dispatch in module picross_native");
                return;
            }

            notify_dispatcher_ = (notify_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_f_string_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_f_string_dispatch in module picross_native");
                return;
            }

            f_string_dispatcher_ = (f_string_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_f2f_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_f2f_dispatch in module picross_native");
                return;
            }

            f2f_dispatcher_ = (f2f_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_status_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_status_dispatch in module picross_native");
                return;
            }

            status_dispatcher_ = (status_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
                event_typeobject_.tp_base = controlled_dispatcher_->type;
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
                connector_typeobject_.tp_base = client_dispatcher_->type;
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
                controlled_typeobject_.tp_base = xxcontrolled_dispatcher_->type;
            
        
    
        
            
                fasttrigger_typeobject_.tp_base = xxcontrolled_dispatcher_->type;
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
                splitter_node_typeobject_.tp_base = server_dispatcher_->type;
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
                subcanchor_typeobject_.tp_base = client_dispatcher_->type;
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
                fastdata_receiver_typeobject_.tp_base = fastdata_dispatcher_->type;
            
        
    
        
            
        
    
        
            
        
    
        
            
                data_typeobject_.tp_base = data_base_dispatcher_->type;
            
        
    
        
            
                data_nb_typeobject_.tp_base = data_base_dispatcher_->type;
            
        
    
        
            
        
    
        
            
                clocksink_typeobject_.tp_base = clockref_dispatcher_->type;
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&event_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&scheduler_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&rpctoken_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&rpcserver_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&rpcclient_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&samplearray_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&sample_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&zone_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&preset_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&clockref_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&strummer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&stringer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&cycler_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&polyctl_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&stereomixer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&stereosummer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&monomixer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&consolemixer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&gate_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&correlator_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&connector_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&converter_ref_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&capture_backend_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&functor_backend_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&controller_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&xxcontrolled_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&controlled_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&fasttrigger_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&cookie_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&decode_ctl_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&decoder_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&event_data_buffer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&event_data_source_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&event_data_source_real_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&wire_ctl_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&root_ctl_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&wire_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&root_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&multiplexer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&ranger_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&lightconvertor_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&statusbuffer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&lightsource_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&statusmixer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&controllerdict_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&mapping_info_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&midi_converter_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&mapping_observer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&kgroup_mapper_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&sclone_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&clone_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&aggregator_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&scrolldelegate_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&scroller_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&scroller2_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&splitter_node_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&splitter_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&d2b_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&d2b_nb_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&d2d_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&d2d_nb_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&dd2d_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&dd2d_nb_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&change_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&change_nb_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&function1_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&function2_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&window_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&thing_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&index_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&server_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&client_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&subcanchor_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&canchor_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&anchor_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&f_event_sender_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&f_event_receiver_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&dataqueue_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&fastdata_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&fastdata_receiver_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&term_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&data_base_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&data_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&data_nb_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&clockdomain_ctl_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&clocksink_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&clocksource_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&fast_pitchbender_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&panner_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&gain_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&delay_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&linear_gain_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&selector_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&velocitydetect_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&scaler_controller_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&scaler_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&evtdump_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&wavrecorder_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&python_delegate_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&midi_channel_delegate_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&midi_from_belcanto_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&tsd_snapshot_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("piw_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(event_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_event_dispatch",s);
            Py_INCREF(&event_typeobject_);
            PyModule_AddObject(m,"event",(PyObject *)&event_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(scheduler_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_scheduler_dispatch",s);
            Py_INCREF(&scheduler_typeobject_);
            PyModule_AddObject(m,"scheduler",(PyObject *)&scheduler_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(rpctoken_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_rpctoken_dispatch",s);
            Py_INCREF(&rpctoken_typeobject_);
            PyModule_AddObject(m,"rpctoken",(PyObject *)&rpctoken_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(rpcserver_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_rpcserver_dispatch",s);
            Py_INCREF(&rpcserver_typeobject_);
            PyModule_AddObject(m,"rpcserver",(PyObject *)&rpcserver_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(rpcclient_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_rpcclient_dispatch",s);
            Py_INCREF(&rpcclient_typeobject_);
            PyModule_AddObject(m,"rpcclient",(PyObject *)&rpcclient_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(samplearray_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_samplearray_dispatch",s);
            Py_INCREF(&samplearray_typeobject_);
            PyModule_AddObject(m,"samplearray",(PyObject *)&samplearray_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(sample_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_sample_dispatch",s);
            Py_INCREF(&sample_typeobject_);
            PyModule_AddObject(m,"sample",(PyObject *)&sample_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(zone_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_zone_dispatch",s);
            Py_INCREF(&zone_typeobject_);
            PyModule_AddObject(m,"zone",(PyObject *)&zone_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(preset_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_preset_dispatch",s);
            Py_INCREF(&preset_typeobject_);
            PyModule_AddObject(m,"preset",(PyObject *)&preset_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(clockref_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_clockref_dispatch",s);
            Py_INCREF(&clockref_typeobject_);
            PyModule_AddObject(m,"clockref",(PyObject *)&clockref_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(strummer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_strummer_dispatch",s);
            Py_INCREF(&strummer_typeobject_);
            PyModule_AddObject(m,"strummer",(PyObject *)&strummer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(stringer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_stringer_dispatch",s);
            Py_INCREF(&stringer_typeobject_);
            PyModule_AddObject(m,"stringer",(PyObject *)&stringer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(cycler_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_cycler_dispatch",s);
            Py_INCREF(&cycler_typeobject_);
            PyModule_AddObject(m,"cycler",(PyObject *)&cycler_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(polyctl_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_polyctl_dispatch",s);
            Py_INCREF(&polyctl_typeobject_);
            PyModule_AddObject(m,"polyctl",(PyObject *)&polyctl_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(stereomixer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_stereomixer_dispatch",s);
            Py_INCREF(&stereomixer_typeobject_);
            PyModule_AddObject(m,"stereomixer",(PyObject *)&stereomixer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(stereosummer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_stereosummer_dispatch",s);
            Py_INCREF(&stereosummer_typeobject_);
            PyModule_AddObject(m,"stereosummer",(PyObject *)&stereosummer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(monomixer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_monomixer_dispatch",s);
            Py_INCREF(&monomixer_typeobject_);
            PyModule_AddObject(m,"monomixer",(PyObject *)&monomixer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(consolemixer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_consolemixer_dispatch",s);
            Py_INCREF(&consolemixer_typeobject_);
            PyModule_AddObject(m,"consolemixer",(PyObject *)&consolemixer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(gate_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_gate_dispatch",s);
            Py_INCREF(&gate_typeobject_);
            PyModule_AddObject(m,"gate",(PyObject *)&gate_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(correlator_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_correlator_dispatch",s);
            Py_INCREF(&correlator_typeobject_);
            PyModule_AddObject(m,"correlator",(PyObject *)&correlator_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(connector_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_connector_dispatch",s);
            Py_INCREF(&connector_typeobject_);
            PyModule_AddObject(m,"connector",(PyObject *)&connector_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(converter_ref_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_converter_ref_dispatch",s);
            Py_INCREF(&converter_ref_typeobject_);
            PyModule_AddObject(m,"converter_ref",(PyObject *)&converter_ref_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(capture_backend_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_capture_backend_dispatch",s);
            Py_INCREF(&capture_backend_typeobject_);
            PyModule_AddObject(m,"capture_backend",(PyObject *)&capture_backend_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(functor_backend_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_functor_backend_dispatch",s);
            Py_INCREF(&functor_backend_typeobject_);
            PyModule_AddObject(m,"functor_backend",(PyObject *)&functor_backend_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(controller_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_controller_dispatch",s);
            Py_INCREF(&controller_typeobject_);
            PyModule_AddObject(m,"controller",(PyObject *)&controller_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(xxcontrolled_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_xxcontrolled_dispatch",s);
            Py_INCREF(&xxcontrolled_typeobject_);
            PyModule_AddObject(m,"xxcontrolled",(PyObject *)&xxcontrolled_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(controlled_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_controlled_dispatch",s);
            Py_INCREF(&controlled_typeobject_);
            PyModule_AddObject(m,"controlled",(PyObject *)&controlled_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(fasttrigger_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_fasttrigger_dispatch",s);
            Py_INCREF(&fasttrigger_typeobject_);
            PyModule_AddObject(m,"fasttrigger",(PyObject *)&fasttrigger_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(cookie_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_cookie_dispatch",s);
            Py_INCREF(&cookie_typeobject_);
            PyModule_AddObject(m,"cookie",(PyObject *)&cookie_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(decode_ctl_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_decode_ctl_dispatch",s);
            Py_INCREF(&decode_ctl_typeobject_);
            PyModule_AddObject(m,"decode_ctl",(PyObject *)&decode_ctl_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(decoder_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_decoder_dispatch",s);
            Py_INCREF(&decoder_typeobject_);
            PyModule_AddObject(m,"decoder",(PyObject *)&decoder_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(event_data_buffer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_event_data_buffer_dispatch",s);
            Py_INCREF(&event_data_buffer_typeobject_);
            PyModule_AddObject(m,"event_data_buffer",(PyObject *)&event_data_buffer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(event_data_source_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_event_data_source_dispatch",s);
            Py_INCREF(&event_data_source_typeobject_);
            PyModule_AddObject(m,"event_data_source",(PyObject *)&event_data_source_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(event_data_source_real_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_event_data_source_real_dispatch",s);
            Py_INCREF(&event_data_source_real_typeobject_);
            PyModule_AddObject(m,"event_data_source_real",(PyObject *)&event_data_source_real_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(wire_ctl_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_wire_ctl_dispatch",s);
            Py_INCREF(&wire_ctl_typeobject_);
            PyModule_AddObject(m,"wire_ctl",(PyObject *)&wire_ctl_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(root_ctl_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_root_ctl_dispatch",s);
            Py_INCREF(&root_ctl_typeobject_);
            PyModule_AddObject(m,"root_ctl",(PyObject *)&root_ctl_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(wire_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_wire_dispatch",s);
            Py_INCREF(&wire_typeobject_);
            PyModule_AddObject(m,"wire",(PyObject *)&wire_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(root_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_root_dispatch",s);
            Py_INCREF(&root_typeobject_);
            PyModule_AddObject(m,"root",(PyObject *)&root_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(multiplexer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_multiplexer_dispatch",s);
            Py_INCREF(&multiplexer_typeobject_);
            PyModule_AddObject(m,"multiplexer",(PyObject *)&multiplexer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(ranger_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_ranger_dispatch",s);
            Py_INCREF(&ranger_typeobject_);
            PyModule_AddObject(m,"ranger",(PyObject *)&ranger_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(lightconvertor_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_lightconvertor_dispatch",s);
            Py_INCREF(&lightconvertor_typeobject_);
            PyModule_AddObject(m,"lightconvertor",(PyObject *)&lightconvertor_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(statusbuffer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_statusbuffer_dispatch",s);
            Py_INCREF(&statusbuffer_typeobject_);
            PyModule_AddObject(m,"statusbuffer",(PyObject *)&statusbuffer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(lightsource_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_lightsource_dispatch",s);
            Py_INCREF(&lightsource_typeobject_);
            PyModule_AddObject(m,"lightsource",(PyObject *)&lightsource_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(statusmixer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_statusmixer_dispatch",s);
            Py_INCREF(&statusmixer_typeobject_);
            PyModule_AddObject(m,"statusmixer",(PyObject *)&statusmixer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(controllerdict_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_controllerdict_dispatch",s);
            Py_INCREF(&controllerdict_typeobject_);
            PyModule_AddObject(m,"controllerdict",(PyObject *)&controllerdict_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(mapping_info_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_mapping_info_dispatch",s);
            Py_INCREF(&mapping_info_typeobject_);
            PyModule_AddObject(m,"mapping_info",(PyObject *)&mapping_info_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(midi_converter_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_midi_converter_dispatch",s);
            Py_INCREF(&midi_converter_typeobject_);
            PyModule_AddObject(m,"midi_converter",(PyObject *)&midi_converter_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(mapping_observer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_mapping_observer_dispatch",s);
            Py_INCREF(&mapping_observer_typeobject_);
            PyModule_AddObject(m,"mapping_observer",(PyObject *)&mapping_observer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(kgroup_mapper_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_kgroup_mapper_dispatch",s);
            Py_INCREF(&kgroup_mapper_typeobject_);
            PyModule_AddObject(m,"kgroup_mapper",(PyObject *)&kgroup_mapper_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(sclone_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_sclone_dispatch",s);
            Py_INCREF(&sclone_typeobject_);
            PyModule_AddObject(m,"sclone",(PyObject *)&sclone_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(clone_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_clone_dispatch",s);
            Py_INCREF(&clone_typeobject_);
            PyModule_AddObject(m,"clone",(PyObject *)&clone_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(aggregator_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_aggregator_dispatch",s);
            Py_INCREF(&aggregator_typeobject_);
            PyModule_AddObject(m,"aggregator",(PyObject *)&aggregator_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(scrolldelegate_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_scrolldelegate_dispatch",s);
            Py_INCREF(&scrolldelegate_typeobject_);
            PyModule_AddObject(m,"scrolldelegate",(PyObject *)&scrolldelegate_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(scroller_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_scroller_dispatch",s);
            Py_INCREF(&scroller_typeobject_);
            PyModule_AddObject(m,"scroller",(PyObject *)&scroller_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(scroller2_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_scroller2_dispatch",s);
            Py_INCREF(&scroller2_typeobject_);
            PyModule_AddObject(m,"scroller2",(PyObject *)&scroller2_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(splitter_node_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_splitter_node_dispatch",s);
            Py_INCREF(&splitter_node_typeobject_);
            PyModule_AddObject(m,"splitter_node",(PyObject *)&splitter_node_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(splitter_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_splitter_dispatch",s);
            Py_INCREF(&splitter_typeobject_);
            PyModule_AddObject(m,"splitter",(PyObject *)&splitter_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(d2b_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_d2b_dispatch",s);
            Py_INCREF(&d2b_typeobject_);
            PyModule_AddObject(m,"d2b",(PyObject *)&d2b_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(d2b_nb_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_d2b_nb_dispatch",s);
            Py_INCREF(&d2b_nb_typeobject_);
            PyModule_AddObject(m,"d2b_nb",(PyObject *)&d2b_nb_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(d2d_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_d2d_dispatch",s);
            Py_INCREF(&d2d_typeobject_);
            PyModule_AddObject(m,"d2d",(PyObject *)&d2d_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(d2d_nb_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_d2d_nb_dispatch",s);
            Py_INCREF(&d2d_nb_typeobject_);
            PyModule_AddObject(m,"d2d_nb",(PyObject *)&d2d_nb_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(dd2d_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_dd2d_dispatch",s);
            Py_INCREF(&dd2d_typeobject_);
            PyModule_AddObject(m,"dd2d",(PyObject *)&dd2d_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(dd2d_nb_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_dd2d_nb_dispatch",s);
            Py_INCREF(&dd2d_nb_typeobject_);
            PyModule_AddObject(m,"dd2d_nb",(PyObject *)&dd2d_nb_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(change_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_change_dispatch",s);
            Py_INCREF(&change_typeobject_);
            PyModule_AddObject(m,"change",(PyObject *)&change_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(change_nb_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_change_nb_dispatch",s);
            Py_INCREF(&change_nb_typeobject_);
            PyModule_AddObject(m,"change_nb",(PyObject *)&change_nb_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(function1_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_function1_dispatch",s);
            Py_INCREF(&function1_typeobject_);
            PyModule_AddObject(m,"function1",(PyObject *)&function1_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(function2_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_function2_dispatch",s);
            Py_INCREF(&function2_typeobject_);
            PyModule_AddObject(m,"function2",(PyObject *)&function2_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(window_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_window_dispatch",s);
            Py_INCREF(&window_typeobject_);
            PyModule_AddObject(m,"window",(PyObject *)&window_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(thing_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_thing_dispatch",s);
            Py_INCREF(&thing_typeobject_);
            PyModule_AddObject(m,"thing",(PyObject *)&thing_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(index_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_index_dispatch",s);
            Py_INCREF(&index_typeobject_);
            PyModule_AddObject(m,"index",(PyObject *)&index_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(server_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_server_dispatch",s);
            Py_INCREF(&server_typeobject_);
            PyModule_AddObject(m,"server",(PyObject *)&server_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(client_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_client_dispatch",s);
            Py_INCREF(&client_typeobject_);
            PyModule_AddObject(m,"client",(PyObject *)&client_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(subcanchor_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_subcanchor_dispatch",s);
            Py_INCREF(&subcanchor_typeobject_);
            PyModule_AddObject(m,"subcanchor",(PyObject *)&subcanchor_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(canchor_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_canchor_dispatch",s);
            Py_INCREF(&canchor_typeobject_);
            PyModule_AddObject(m,"canchor",(PyObject *)&canchor_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(anchor_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_anchor_dispatch",s);
            Py_INCREF(&anchor_typeobject_);
            PyModule_AddObject(m,"anchor",(PyObject *)&anchor_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(f_event_sender_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_f_event_sender_dispatch",s);
            Py_INCREF(&f_event_sender_typeobject_);
            PyModule_AddObject(m,"f_event_sender",(PyObject *)&f_event_sender_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(f_event_receiver_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_f_event_receiver_dispatch",s);
            Py_INCREF(&f_event_receiver_typeobject_);
            PyModule_AddObject(m,"f_event_receiver",(PyObject *)&f_event_receiver_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(dataqueue_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_dataqueue_dispatch",s);
            Py_INCREF(&dataqueue_typeobject_);
            PyModule_AddObject(m,"dataqueue",(PyObject *)&dataqueue_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(fastdata_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_fastdata_dispatch",s);
            Py_INCREF(&fastdata_typeobject_);
            PyModule_AddObject(m,"fastdata",(PyObject *)&fastdata_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(fastdata_receiver_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_fastdata_receiver_dispatch",s);
            Py_INCREF(&fastdata_receiver_typeobject_);
            PyModule_AddObject(m,"fastdata_receiver",(PyObject *)&fastdata_receiver_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(term_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_term_dispatch",s);
            Py_INCREF(&term_typeobject_);
            PyModule_AddObject(m,"term",(PyObject *)&term_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(data_base_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_data_base_dispatch",s);
            Py_INCREF(&data_base_typeobject_);
            PyModule_AddObject(m,"data_base",(PyObject *)&data_base_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(data_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_data_dispatch",s);
            Py_INCREF(&data_typeobject_);
            PyModule_AddObject(m,"data",(PyObject *)&data_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(data_nb_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_data_nb_dispatch",s);
            Py_INCREF(&data_nb_typeobject_);
            PyModule_AddObject(m,"data_nb",(PyObject *)&data_nb_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(clockdomain_ctl_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_clockdomain_ctl_dispatch",s);
            Py_INCREF(&clockdomain_ctl_typeobject_);
            PyModule_AddObject(m,"clockdomain_ctl",(PyObject *)&clockdomain_ctl_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(clocksink_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_clocksink_dispatch",s);
            Py_INCREF(&clocksink_typeobject_);
            PyModule_AddObject(m,"clocksink",(PyObject *)&clocksink_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(clocksource_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_clocksource_dispatch",s);
            Py_INCREF(&clocksource_typeobject_);
            PyModule_AddObject(m,"clocksource",(PyObject *)&clocksource_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(fast_pitchbender_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_fast_pitchbender_dispatch",s);
            Py_INCREF(&fast_pitchbender_typeobject_);
            PyModule_AddObject(m,"fast_pitchbender",(PyObject *)&fast_pitchbender_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(panner_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_panner_dispatch",s);
            Py_INCREF(&panner_typeobject_);
            PyModule_AddObject(m,"panner",(PyObject *)&panner_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(gain_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_gain_dispatch",s);
            Py_INCREF(&gain_typeobject_);
            PyModule_AddObject(m,"gain",(PyObject *)&gain_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(delay_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_delay_dispatch",s);
            Py_INCREF(&delay_typeobject_);
            PyModule_AddObject(m,"delay",(PyObject *)&delay_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(linear_gain_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_linear_gain_dispatch",s);
            Py_INCREF(&linear_gain_typeobject_);
            PyModule_AddObject(m,"linear_gain",(PyObject *)&linear_gain_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(selector_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_selector_dispatch",s);
            Py_INCREF(&selector_typeobject_);
            PyModule_AddObject(m,"selector",(PyObject *)&selector_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(velocitydetect_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_velocitydetect_dispatch",s);
            Py_INCREF(&velocitydetect_typeobject_);
            PyModule_AddObject(m,"velocitydetect",(PyObject *)&velocitydetect_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(scaler_controller_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_scaler_controller_dispatch",s);
            Py_INCREF(&scaler_controller_typeobject_);
            PyModule_AddObject(m,"scaler_controller",(PyObject *)&scaler_controller_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(scaler_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_scaler_dispatch",s);
            Py_INCREF(&scaler_typeobject_);
            PyModule_AddObject(m,"scaler",(PyObject *)&scaler_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(evtdump_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_evtdump_dispatch",s);
            Py_INCREF(&evtdump_typeobject_);
            PyModule_AddObject(m,"evtdump",(PyObject *)&evtdump_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(wavrecorder_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_wavrecorder_dispatch",s);
            Py_INCREF(&wavrecorder_typeobject_);
            PyModule_AddObject(m,"wavrecorder",(PyObject *)&wavrecorder_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(python_delegate_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_python_delegate_dispatch",s);
            Py_INCREF(&python_delegate_typeobject_);
            PyModule_AddObject(m,"python_delegate",(PyObject *)&python_delegate_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(midi_channel_delegate_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_midi_channel_delegate_dispatch",s);
            Py_INCREF(&midi_channel_delegate_typeobject_);
            PyModule_AddObject(m,"midi_channel_delegate",(PyObject *)&midi_channel_delegate_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(midi_from_belcanto_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_midi_from_belcanto_dispatch",s);
            Py_INCREF(&midi_from_belcanto_typeobject_);
            PyModule_AddObject(m,"midi_from_belcanto",(PyObject *)&midi_from_belcanto_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(tsd_snapshot_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_tsd_snapshot_dispatch",s);
            Py_INCREF(&tsd_snapshot_typeobject_);
            PyModule_AddObject(m,"tsd_snapshot",(PyObject *)&tsd_snapshot_typeobject_);
        
    
        
    
        
    
        
    
        
    

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

